package com.intuit.payroll.samples;

import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.j.EffectiveDatedDatumCollectionIf;
import com.intuit.payroll.types.enumtype.PayrollState;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.payroll.types.enumtype.AccountingType;
import com.intuit.payroll.types.enumtype.DatumType;
import com.intuit.dataTypes.QBDate;

import java.util.Collection;
import java.util.TreeSet;
import java.util.Map;
import java.util.HashMap;
import java.util.LinkedList;

/**
 * Crude suggestion of how to use the QBO Payroll Engine to display company tax info (names, rates, limits etc) 
 */
public class GeminiTaxSetup {

    private static final String LINEBREAK = "\r\n";

    static private class AgencyView
    {
        private String defaultAgencyName;
        private PayrollDatumIf accountNumber;
        private Map<String, TableRowView> rows = new HashMap<String, TableRowView>();
        private Collection<TableRowView> orderedRows = new LinkedList<TableRowView>();

        public AgencyView (String defaultAgencyName, PayrollDatumIf accountNumber)
        {
            this.defaultAgencyName = defaultAgencyName;
            this.accountNumber = accountNumber;
        }

        public void addTax (PayrollItemIf tax)
        {
            String baseName = tax.getDetailType().getBaseName(tax.getName());
            TableRowView row = rows.get(baseName);
            if (row == null) {
                row = new TableRowView(baseName);
                rows.put (baseName, row);
                orderedRows.add(row);
            }
            row.addTax (tax);
        }

        // For example "99-9999999"
        private String buildAccountNumberFormats()
        {
            StringBuilder result = new StringBuilder();
            Collection<String> formats = accountNumber.getValidationInfo().getStringFormats();
            if (formats != null) {
                for (String format : formats) {
                    if (result.length() > 0) result.append(" or ");
                    result.append(format);
                }
            }
            return result.toString();
        }

        // Agency and account number info
        private String buildHeader ()
        {
            StringBuilder result = new StringBuilder();
            result.append("Payee: [");
            result.append(defaultAgencyName);
            result.append("]");
            result.append(LINEBREAK);
            result.append(accountNumber.getLabel());
            if (accountNumber.isEditable()) {
                result.append("[");
            }
            result.append(accountNumber.getValue());
            if (accountNumber.isEditable()) {
                result.append("]");
            }
            result.append(" ");
            result.append(buildAccountNumberFormats());
            result.append(LINEBREAK);
            result.append(accountNumber.getNote());
            result.append(LINEBREAK);
            return result.toString();

        }

        /**
         * @return sample representation of the company tax setup UI, best viewed
         * as tab-delimited MS Excel file so the columns line up. 
         */
        public String buildTable ()
        {
            StringBuilder result = new StringBuilder();
            // agency and account number
            result.append(buildHeader());

            // Table header
            result.append("Tax");
            result.append("\t");
            result.append("Company Rate");
            result.append("\t");
            result.append("Employee Rate");
            result.append("\t");
            result.append("Wage Limit");
            result.append(LINEBREAK);


            // Tax rows
            for (TableRowView row : orderedRows) {
                result.append(row.getTaxName());
                result.append("\t");
                result.append(row.getCompanyRate());
                result.append("\t");
                result.append(row.getEmployeeRate());
                result.append("\t");
                result.append(row.getWageLimit());
                result.append(LINEBREAK);
            }
            return result.toString();
        }
    }

    // Handles one tax (both ee and er info)
    static private class TableRowView {
        private String baseName;
        private PayrollItemIf eePaid;
        private PayrollItemIf erPaid;

        public TableRowView (String baseName)
        {
            this.baseName = baseName;
        }

        public void addTax (PayrollItemIf tax)
        {
            if (tax.getDetailType().getAccountingType() == AccountingType.LIABILITY_AND_EXPENSE) {
                erPaid = tax;
            } else {
                eePaid = tax;
            }
        }

        public String getTaxName ()
        {
            return baseName;
        }

        public String getEmployeeRate ()
        {
            return getData(eePaid, DatumID.RATE);
        }

        public String getCompanyRate ()
        {
            return getData(erPaid, DatumID.RATE);
        }

        public String getWageLimit ()
        {
            // assumes er and ee-paid taxes have the same wage limit
            return getData(erPaid, DatumID.WAGE_LIMIT);
        }

        private static String getEffectiveDatedData (EffectiveDatedDatumCollectionIf datedData)
        {
            StringBuilder result = new StringBuilder();
            for (PayrollDatumIf datedValue : datedData.getVisibleDatumList()) {
                if (result.length() > 0) result.append (", ");
                result.append (datedValue.getValue());
                QBDate asOf = datedValue.getEffectiveBeginDate();
                if (asOf != null) {
                    result.append (" As of: ");
                    result.append (asOf);
                }
            }
            return result.toString();
        }

        /**
         * @param tax eg CASUI, FICE, FICR etc
         * @param field eg RATE or WAGE_LIMIT
         * @return string description of rate or limit info for the passed tax
         */
        private static String getData(PayrollItemIf tax, DatumID field)
        {
            StringBuilder result = new StringBuilder();
            if (tax != null) {
                PayrollDatumIf rate = tax.getDatum(field);
                if (rate.isEditable()) {
                    // need editable field
                    result.append("[");
                }
                Object val = rate.getValue();
                if (rate.getValueType() == DatumType.EFFECTIVE_DATED_DATUM_LIST) {
                    result.append(getEffectiveDatedData((EffectiveDatedDatumCollectionIf) val));
                } else {
                    result.append(val);
                }
                if (rate.isEditable()) {
                    // need editable field
                    result.append("]");
                }
                String desc = rate.getDescription();
                if (desc != null) {
                    // Could be message about inexperience rate
                    result.append (" ");
                    result.append (desc);
                }
            }
            return result.toString();
        }
    }

    /**
     * Creates a unique identifier which is a combination of default (suggested)
     * vendor name and account number label.  
     * @param tax eg FIT, CASUI
     * @return agency key
     */
    private static String getAgencyKey (PayrollItemIf tax)
    {
        String result = tax.getDetailType().getVendorDefaultName ();
        PayrollDatumIf accountNumber = tax.getDatum(DatumID.ACCOUNT_NUMBER);
        if (accountNumber != null) {
            result += accountNumber.getLabel();
        }
        return result;
    }

    public static void main (String[] args) {

        // First day paychecks are allowed in the system (for this company)
        QBDate payrollStartDate = new QBDate (5,1,2008);

        // Create app's context for this company
        Gemini.Context context = new Gemini.Context(payrollStartDate);

        GeminiTest.loadCompanyInfo(context);

        CompanyIf co = context.getCompany();

        // Specify that this company is new to payroll (makes the SUI rate notes show up
        // and defaults to inexperience rate)
        co.setDatumValue(DatumID.NEW_COMPANY, Boolean.TRUE);

        // Get federal and CA state required taxes.
        // Use the natural sort order of pitems which puts Federal taxes
        // before state and then, income taxes, unemployment, surcharge, disability etc.
        Collection<PayrollItemIf> taxes = new TreeSet<PayrollItemIf>();
        taxes.addAll(co.getRequiredTaxes(null, co.getSystemDate()));
        taxes.addAll(co.getRequiredTaxes(PayrollState.CA, co.getSystemDate()));

        // Use the natural sort order of pitems, but chunk them according to agency
        Map<String, AgencyView> agencyMap = new HashMap<String, AgencyView>();
        Collection<AgencyView> orderedAgencies = new LinkedList<AgencyView>();
        for (PayrollItemIf tax : taxes) {
            // Key is combination of account number label and default agency name
            String agencyKey = getAgencyKey (tax);
            AgencyView agencyView = agencyMap.get(agencyKey);
            if (agencyView == null) {
                agencyView = new AgencyView(tax.getDetailType().getVendorDefaultName(), tax.getDatum(DatumID.ACCOUNT_NUMBER));
                orderedAgencies.add(agencyView);
                agencyMap.put(agencyKey, agencyView);
            }
            agencyView.addTax(tax);
        }

        for (AgencyView agency : orderedAgencies) {
            System.out.println(agency.buildTable());
            System.out.println();
        }
    }
}
