package com.intuit.payroll.samples;

import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.j.PayrollItemListInfoIf;
import com.intuit.payroll.j.EmployeePayrollItemIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.j.EffectiveDatedDatumCollectionIf;
import com.intuit.payroll.j.EmployeeIf;
import com.intuit.payroll.j.PaycheckIf;
import com.intuit.payroll.j.PaycheckLineIf;
import com.intuit.payroll.j.EffectiveDatedValueIf;
import com.intuit.payroll.j.PayrollItemTotalsCollectionIf;
import com.intuit.payroll.j.PayrollItemTotalsIf;
import com.intuit.payroll.j.TaxJurisdictionIf;
import com.intuit.payroll.j.PayrollFactory;
import com.intuit.payroll.j.PayEmployeesIf;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.payroll.types.enumtype.PayFrequencyType;
import com.intuit.payroll.types.enumtype.PayrollState;
import com.intuit.payroll.types.enumtype.TotalsTransactionType;
import com.intuit.payroll.types.enumtype.EmployeeSetupType;
import com.intuit.payroll.types.enumtype.CalcBasedOn;
import com.intuit.dataTypes.QBDate;
import com.intuit.dataTypes.Rate;
import com.intuit.dataTypes.Money;
import com.intuit.dataTypes.TimeDuration;
import com.intuit.dataTypes.Quantity;

import java.util.Iterator;
import java.util.Map;
import java.util.List;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.EnumMap;
import java.util.Arrays;

/**
 * Ugly sample code to illustrate basic requirements for integration with Payroll Engine.
 * Top level class represents some biz logic for the Payroll Engine client application.
 * Illustrates hourly paycheck. Salary is slightly more complicated.
 * Missing details on employee-specific tax setup info (assumes default values for now).
 */
public class GeminiTest {

    // Sample payroll item table in Gemini database. Columns:
    // id: primary key, Payroll Item unique ID (your ID for the payroll item, unique for the company).
    // type: Gemini item type
    // name: DatumID.NAME user's name for this item (null means accept the default)
    // amount: DatumID.AMOUNT default amount (may be applicable for some deductions, hourly wage etc)
    // etc...
    private static final Object[][] DB_PAYROLL_ITEMS = {
            // pitem_id,    type,                               name,           amount
            {   1,          Gemini.PItemID.SALARY,              "My Salary",    null, },
            {   2,          Gemini.PItemID.HOURLY,              "My Hourly",    new Money (50), },
            {   20,         Gemini.PItemID.FOUR01K,             "401k",         null, },    // user may specify default rate/limit.  Default limit is provided (15500.00 starting 1/1/2007) 
            {   25,         Gemini.PItemID.SEC125HEALTH,        "Health Ins",   null, },       
            {   26,         Gemini.PItemID.OTHER_REIMBURSEMENT, "Reimburse",    null, },       
            {   3,          Gemini.PItemID.FIT,                 null,           null, },
            {   4,          Gemini.PItemID.FICE,                null,           null, },
            {   19,         Gemini.PItemID.FUTA,                null,           null, },
            {   5,          Gemini.PItemID.FICR,                null,           null, },
            {   6,          Gemini.PItemID.MEDE,                null,           null, },
            {   7,          Gemini.PItemID.MEDR,                null,           null, },
            {   8,          Gemini.PItemID.CASIT,               null,           null, },
            {   9,          Gemini.PItemID.CASUI,               null,           null, },
            {   10,         Gemini.PItemID.CASDI,               null,           null, },
            {   11,         Gemini.PItemID.CAETT,               null,           null, },
            {   12,         Gemini.PItemID.VACATION_ACCRUE,     null,           null, },
            {   13,         Gemini.PItemID.VACATION_DISBURSE,   null,           null, },
            {   14,         Gemini.PItemID.VACATION_PAYOUT,     null,           null, },
            //...
    };

    // TaxJurisdiction is a proxy for true GeoCode.  Since we don't have compliance support for GeoCode we
    // create our own user-defined GeoCode using state + local taxes.  It works backwards.
    // In a real GeoCode-based system, we would tell the user what taxes are applicable.  In our system,
    // the user tells us what local taxes the employee pays, and we turn that into a sort of GeoCode.
    // The tax jurisdiction is sometimes needed for tax forms and historical adjustments.
    // Since Gemini v1 is for states with no local taxes, you can probably ignore for now, or
    // just use a stub such as this...
    private static final TaxJurisdictionIf CA_JUR = PayrollFactory.createTaxJurisdiction(PayrollState.CA, null);


    // Sample employee totals table in Gemini database.  These totals are derivable from
    // paychecks in the database, but the cached, quarterly figures are useful for many
    // payroll calculations (including paychecks).
    //
    // Columns:  (primary key is: emp_id, pitem_id, begin, end)
    // emp_id:      Employee unique ID (your ID for this employee, unique for the company)
    // pitem_id:    key to DB_PAYROLL_ITEMS
    // begin:       first date of period, inclusive (eg 1/1/2007)
    // end:         first date of next period (eg 4/1/2007)
    // live_jur:    DatumID.LIVE_TAX_JURISDICTION, geocode for residence location where wages earned
    // work_jur:    DatumID.WORK_TAX_JURISDICTION, geocode for work location where wages earned
    // amount:      DatumID.TOTAL_AMOUNT (amount earned or accrued)
    // wage_base:   DatumID.TOTAL_WAGE_BASE (wages used as basis for calculating this tax)
    // istt:        DatumID.TOTAL_INCOME_SUBJECT_TO_TAX (unclipped wages)
    // hours:       DatumID.TOTAL_HOURS
    private static final Object[][] DB_EMPLOYEE_TOTALS = {
            //  emp_id,     pitem_id,   begin,                  end,                    live_jur,   work_jur,   amount,                 wage_base,          istt,               hours,
            // salary
            {   1,          1,          new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money(10000),       null,               null,               null, },
            // vacation accrued (note continuous, not quarterly totals)
            {   1,          12,         null,                   null,                   CA_JUR,     CA_JUR,     null,                   null,               null,               new TimeDuration("10:00"), },
            // FIT
            {   1,          3,          new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (2433),       new Money(10000),   new Money(10000),   null, },
            // FUTA
            {   1,          19,         new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (56),         new Money(7000),    new Money(10000),   null, },
            // FICE/FICR
            {   1,          4,          new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (620),        new Money(10000),   new Money(10000),   null, },
            {   1,          5,          new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (620),        new Money(10000),   new Money(10000),   null, },
            // MEDE/MEDR
            {   1,          6,          new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (145),        new Money(10000),   new Money(10000),   null, },
            {   1,          7,          new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (145),        new Money(10000),   new Money(10000),   null, },
            // CASIT
            {   1,          8,          new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money ("750.78"),   new Money(10000),   new Money(10000),   null, },
            // CASUI
            {   1,          9,          new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (70),         new Money(7000),    new Money(10000),   null, },
            {   1,          11,         new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (7),          new Money(7000),    new Money(10000),   null, },
            // CASDI
            {   1,          10,         new QBDate(1,1,2007),   new QBDate(4,1,2007),   CA_JUR,     CA_JUR,     new Money (60),         new Money(10000),    new Money(10000),   null, },
    };

    // Convenience mapping from type to client unique ID (per company)
    // This shouldn't be "static", but since the company data is also static...
    static final Map<Gemini.PItemID, Integer> ID_MAP;

    static {
        ID_MAP = new EnumMap<Gemini.PItemID, Integer>(Gemini.PItemID.class);
        for (Object[] item : DB_PAYROLL_ITEMS) {
            Integer pitemUniqueID = (Integer) item[0];
            Gemini.PItemID type = (Gemini.PItemID) item[1];
            ID_MAP.put (type, pitemUniqueID);
        }
    }

    /**
     * For now only SUI rate.
     * @param context Gemini cached payroll info
     */
    private static void loadTaxRates (Gemini.Context context)
    {
        // set CASUI rate
        PayrollItemIf casui = context.getPayrollItem(ID_MAP.get(Gemini.PItemID.CASUI));
        PayrollDatumIf rateContainer = casui.getDatum(DatumID.RATE);

        // Set single rate for all time (easiest)
        rateContainer.setValue(new Rate("2%"));


        // But rates are effective dated, so the value is acutally a list of dated values
        // This works too...
        EffectiveDatedDatumCollectionIf datedValues = (EffectiveDatedDatumCollectionIf) rateContainer.getValue();
        List<EffectiveDatedValueIf> list = new LinkedList<EffectiveDatedValueIf>();

        // Rate was 1% until 7/1 when it changed to 2%
        // The paycheck in this example is dated 6/1/2007, so it gets the 1% rate
        list.add(new Gemini.EffectiveDatedValue(new Rate("1%"), null));
        list.add(new Gemini.EffectiveDatedValue(new Rate("2%"), new QBDate (7,1,2007)));
        datedValues.setValues(list);
    }

    /**
     * Load lightweight pitem objects into the context.
     * @param context Gemini cached payroll info
     */
    private static void loadPayrollItems (Gemini.Context context)
    {
        for (Object[] item : DB_PAYROLL_ITEMS) {
            Integer pitemUniqueID = (Integer) item[0];
            Gemini.PItemID type = (Gemini.PItemID) item[1];
            String name = (String) item[2];

            // Create the fast, lightweight pitem
            context.createPayrollItem(type, name, pitemUniqueID);
        }

        // Currently the samples do not show how to set up taxability,
        // so for now, initialize the default values
        context.getUserTaxability().testRestoreDefaults();
    }

    /**
     * Load company info.  For now only SUI rate.  Ignores taxability
     * @param context Gemini cached payroll info
     */
    protected static void loadCompanyInfo (Gemini.Context context)
    {
        loadPayrollItems(context);

        loadTaxRates(context);
    }

    /**
     * Load employee setup info from DB.  Missing lots of stuff but here's an idea
     * @param epi Payroll Engine object for employee-specific payroll item info
     * @param salariedEmp true if the employee is salaried
     */
    private static void loadEmployeePayrollItemInfo(EmployeePayrollItemIf epi, boolean salariedEmp)
    {
        Integer piUniqueID = epi.getPayrollItem().getUniqueID();
        if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.SALARY))) {
            epi.setDatumValue(DatumID.RATE, new Money (50000));

        } else if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.CASIT))) {

            // These vary quite a bit by state
            epi.setDatumValue(DatumID.DEDUCTIONS, Integer.valueOf(1));
            epi.setDatumValue(DatumID.ALLOWANCES, Integer.valueOf(2));
            epi.setDatumValue(DatumID.EXTRA_WITHHOLDING, new Money (100));

            // Assume default filing status for now.
            // Gemini will either need to use the Payroll Engine's filing status list in a data-driven way
            // or figure out how to map its own employee tax setup data to the Payroll Engine values
//            epi.setDatumValue(DatumID.FILING_STATUS, ?)
            
        } else if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.FOUR01K))) {

            // Notice that default accrual limit (based on system date after 1/1/2007) is 15500.00
            // In rare cases the employee may need to override
            Object defaultValue = epi.getDatumValue(DatumID.ACCRUAL_LIMIT);

            // Accrue as 10% of compensation (rate or amount may be overridden on the paycheck)
            epi.setDatumValue (DatumID.RATE, new Rate("10%"));

        } else if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.SEC125HEALTH))) {

            // employee-specific limit?
//            epi.setDatumValue(DatumID.ACCRUAL_LIMIT, new Money(5000));

            // employee-specific rate
            epi.setDatumValue (DatumID.RATE, new Money(100));   // flat amount
//            epi.setDatumValue (DatumID.RATE, new Rate("10%"));  // rate
        }
        
        GeminiPaidTimeOff.loadEmployeePayrollItemInfo (epi, salariedEmp);
    }

    private static void loadPaycheckInfo (PaycheckLineIf line) {
        Integer piUniqueID = line.getPayrollItem().getUniqueID();
        if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.HOURLY))) {
            // This is the hourly item, set the hourly info
            // Normally you would get this default from the employee record but
            // user would be allowed to edit on the paycheck itself as well
            line.setDatumValue(DatumID.RATE, new Money(50));
            line.setDatumValue(DatumID.HOURS, new TimeDuration("40:00"));

        } else if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.FOUR01K))) {

            // Could override rate or amount here, but don't for now 
        } else if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.SEC125HEALTH))) {

            // Could override rate or amount here, but don't for now
        } else if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.VACATION_DISBURSE))) {
            line.setDatumValue(DatumID.HOURS, new TimeDuration("3:00"));

        } else if (piUniqueID.equals(ID_MAP.get(Gemini.PItemID.VACATION_PAYOUT))) {
            // Not common, pay out vacation hours without counting against salary paycheck total hours
            line.setDatumValue(DatumID.HOURS, new TimeDuration("5:00"));
        }
    }

    private static void addSalaryPaycheckLines (Gemini.Context context, PaycheckIf paycheck)
    {
        EmployeeIf e = paycheck.getEmployee();
        Iterator<PayrollItemListInfoIf> i = context.getPayrollItemIterator();
        while (i.hasNext()) {
            // Simple cached pitem info
            PayrollItemListInfoIf piInfo = i.next();

            if (ID_MAP.get(Gemini.PItemID.HOURLY).equals(piInfo.getUniqueID())) {
                // skip hourly item because this is an salary paycheck
                continue;
            }

            // Get the full Payroll Engine object
            PayrollItemIf pi = piInfo.getPayrollItem();

            // Create the employee payroll item object
            EmployeePayrollItemIf epi = e.createEmployeePayrollItem(pi);
            loadEmployeePayrollItemInfo(epi, true);

            // Paycheck line
            PaycheckLineIf line = paycheck.createLine(epi);
            loadPaycheckInfo (line);
        }
    }

    /**
     * Pretend we are loading this info from the employee table in the database
     * @param e employee to update
     * @param context pitem list, payroll engine context 
     */
    protected static void loadEmployeeInfo (EmployeeIf e, Gemini.Context context) {
        // Paycheck frequency
        e.setDatumValue(DatumID.PAY_FREQUENCY, PayFrequencyType.MONTHLY);

        // Where employee lives for tax purposes
        e.setDatumValue(DatumID.LIVE_LOCATION, PayrollState.CA);

        // Where employee works for tax purposes
        e.setDatumValue(DatumID.WORK_LOCATION, PayrollState.CA);

        // Hire date (optional, but needed for "on hire date" PTO accrual)
        e.setDatumValue(DatumID.HIRE_DATE, new QBDate (5,2,2001));

        // Date employee was set up in payroll system (different from hire date)
        // Used to determine if annual PTO should be added in the start year.
        // If the annual PTO anniversary is before the start date, we assume
        // user has entered PTO in starting balance so we should not automatically
        // accrue it until the following year
        e.setDatumValue(DatumID.PAYROLL_START_DATE, new QBDate (1,1,2003));

        // Load quarterly totals from DB
        loadEmployeeTotals(e, context);

        // Gemini company-specific employee ID
        Integer empID = Integer.valueOf(101);
        e.finishInit(empID, false);
    }

    /**
     * Pretend we are loading this info from the employee payroll totals table in the database.
     * These are quarterly (or continuous) cached totals used for paycheck and other payroll
     * calculations
     * @param e employee to update
     * @param context company context (including access to payroll items)
     */
    private static void loadEmployeeTotals (EmployeeIf e, Gemini.Context context) {

        PayrollItemTotalsCollectionIf totals = e.getTotalsCollection();

        final DatumID[] columns = {
                DatumID.TOTAL_AMOUNT,
                DatumID.TOTAL_WAGE_BASE,
                DatumID.TOTAL_INCOME_SUBJECT_TO_TAX,
                DatumID.TOTAL_HOURS,
        };

        // Read totals from database
        for (Object[] dbTotals : DB_EMPLOYEE_TOTALS) {
//            Integer empUniqueID = (Integer) dbTotals[0];
            Integer pitemUniqueID = (Integer) dbTotals[1];
            QBDate periodBegin = (QBDate) dbTotals[2];
            QBDate periodEnd = (QBDate) dbTotals[3];
            TaxJurisdictionIf live = (TaxJurisdictionIf) dbTotals[4];
            TaxJurisdictionIf work = (TaxJurisdictionIf) dbTotals[5];
            Map<DatumID, Object> values = new TreeMap<DatumID, Object>();
            values.put (DatumID.TOTAL_AMOUNT, dbTotals[6]);
            values.put (DatumID.TOTAL_WAGE_BASE, dbTotals[7]);
            values.put (DatumID.TOTAL_INCOME_SUBJECT_TO_TAX, dbTotals[8]);
            values.put (DatumID.TOTAL_HOURS, dbTotals[9]);

            PayrollItemTotalsIf itemTotals = totals.getTotals(pitemUniqueID);
            if (itemTotals == null) {
                // not created yet... need payroll item object for creation
                PayrollItemIf pi = context.getPayrollItem(pitemUniqueID);
                itemTotals = totals.createTotals(pi);
            }
            for (DatumID column : columns) {
                Object val = values.get(column);
                if (val != null) {
                    itemTotals.setTotalFromDB(column, TotalsTransactionType.ALL, val, periodBegin, periodEnd, live, work);
                    itemTotals.setTotalFromDB(column, TotalsTransactionType.PAYCHECKS_YTDS, val, periodBegin, periodEnd, live, work);
                }
            }
        }
    }

    /**
     * Creates salary paychecks.  Prints to stdout.
     * @param context company info
     */
    private static void createSalaryPaycheck (Gemini.Context context)
    {
        // Create Payroll Engine employee object
        CompanyIf co = context.getCompany();
        EmployeeIf e = co.createEmployee();
        loadEmployeeInfo(e, context);

        // Create Payroll Engine paycheck object
        PaycheckIf paycheck = e.createPaycheck(new QBDate (2,1,2007));
        paycheck.setDatumValue(DatumID.PAY_PERIOD_BEGIN, new QBDate (1,1,2007));
        paycheck.setDatumValue(DatumID.PAY_PERIOD_END, new QBDate (1,31,2007));

        // Note:
        // Set PARTIAL_PAY_PERIOD to have Payroll Engine calculate paycheck based on PAY_PERIOD_BEGIN/END
        // dates instead of PAY_FREQUENCY (eg partial pay period due to termination)
        //paycheck.setDatumValue(DatumID.PARTIAL_PAY_PERIOD, Boolean.TRUE);

        addSalaryPaycheckLines(context, paycheck);

        // Text view of the entire paycheck
        System.out.println (paycheck);

        // Update the employee totals
        e.getTotalsCollection().update(paycheck);

        // Text view of the totals
        System.out.println(e.getTotalsCollection());

        // Now create another paycheck
        paycheck = e.createPaycheck(new QBDate (3,1,2007));
        paycheck.setDatumValue(DatumID.PAY_PERIOD_BEGIN, new QBDate (2,1,2007));
        paycheck.setDatumValue(DatumID.PAY_PERIOD_END, new QBDate (2,28,2007));

        addSalaryPaycheckLines(context, paycheck);

        // Text view of the entire paycheck, notice YTD amounts are set
        System.out.println (paycheck);


        // Imagine user adding odd paycheck out of order.
        paycheck = e.createPaycheck(new QBDate (2,15,2007));
        paycheck.setDatumValue(DatumID.PAY_PERIOD_BEGIN, new QBDate (2,1,2007));
        paycheck.setDatumValue(DatumID.PAY_PERIOD_END, new QBDate (2,15,2007));
        addSalaryPaycheckLines(context, paycheck);

        // Note:
        // Set PARTIAL_PAY_PERIOD to have Payroll Engine calculate paycheck based on PAY_PERIOD_BEGIN/END
        // dates instead of PAY_FREQUENCY (eg partial pay period due to termination)
        // See 'Saraly Total Hours' in output for difference
        paycheck.setDatumValue(DatumID.PARTIAL_PAY_PERIOD, Boolean.TRUE);

        // Client needs to find the previous paycheck and grab ytd amounts from there.
        for (PaycheckLineIf line : paycheck.getPaycheckLines()) {
            // Not realistic to have all lines previous ytd the same value.  Also doesn't match
            // emloyee totals, but OK for demonstration
            Money ytd = new Money (10);

            // Note use of setFromDB() instead of setValue().  That is because PREV_YEAR_TO_DATE_AMOUNT
            // is a calculated value.  setFromDB() [means value loaded from database] has the effect
            // of inhibiting the normal calc
            line.getDBDatum(DatumID.PREV_YEAR_TO_DATE_AMOUNT).setFromDB(ytd);

            // Also in real case, need to handle the other PREV_ datum fields
        }

        // Text view of the entire paycheck, notice YTD amounts are set
        System.out.println (paycheck);
    }

    /**
     * Creates simple hourly paycheck, no ytd.  Prints to stdout.
     * @param context company info
     */
    private static void createHoulyPaycheck (Gemini.Context context)
    {
        // Create Payroll Engine employee object
        CompanyIf co = context.getCompany();
        EmployeeIf e = co.createEmployee();
        loadEmployeeInfo(e, context);

        // Create Payroll Engine paycheck object
        QBDate checkDate = new QBDate (6,1,2007);
        PaycheckIf paycheck = e.createPaycheck(checkDate);
        Iterator<PayrollItemListInfoIf> i = context.getPayrollItemIterator();
        while (i.hasNext()) {
            // Simple cached pitem info
            PayrollItemListInfoIf piInfo = i.next();

            // Get the full Payroll Engine object
            PayrollItemIf pi = piInfo.getPayrollItem();


            if (ID_MAP.get(Gemini.PItemID.SALARY).equals(piInfo.getUniqueID())) {
                // This is an hourly line, but there are some salary totals,
                // so add an 'historical' line

                // This creates an epi but doesn't add it to the employee so PE
                // won't think the employee is 'salaried' anymore
                EmployeePayrollItemIf epi = e.createOnlyEmployeePayrollItem(pi.getPayrollItem());
                paycheck.createHistoryLine(epi);

            } else if (ID_MAP.get(Gemini.PItemID.VACATION_PAYOUT).equals(piInfo.getUniqueID())) {
                // nothing, payout not applicable to hourly paychecks
                
            } else {
                // Normal paycheck line

                // Create the employee payroll item object (normally this would already be part of the employee)
                EmployeePayrollItemIf epi = e.createEmployeePayrollItem(pi.getPayrollItem());
                loadEmployeePayrollItemInfo(epi, false);

                PaycheckLineIf line = paycheck.createLine(epi);
                loadPaycheckInfo (line);
            }
        }

        // Convenience method for net amount
        System.out.println (paycheck.getNetAmount());

        // Same result as above
        System.out.println (paycheck.getDatumValue(DatumID.NET_AMOUNT));

        // Text view of the entire paycheck
        System.out.println (paycheck);
    }

    /**
     * Illustrates some of the possibilities for reimbursable expense calculations.
     * This is not intended as an example of how to write client code.  Ideally that
     * would be data driven from the pitem and epitem setup info.  This example
     * merely shows how some of the possible calcs and data-entry options work.
     * @param context company context including access to client's pitem data
     */
    private static void createReimbursementPaycheck (Gemini.Context context)
    {
        // Create Payroll Engine employee object
        CompanyIf co = context.getCompany();
        EmployeeIf e = co.createEmployee();
        loadEmployeeInfo(e, context);

        // Let Payroll tell us what taxes employee should have based on live/work
        e.createRequiredEmployeePayrollItems(EmployeeSetupType.FEDERAL_TAX);
        e.createRequiredEmployeePayrollItems(EmployeeSetupType.STATE_TAX);

        // Add hourly pay too
        PayrollItemIf hourly = context.getPayrollItem(Gemini.PItemID.HOURLY);
        e.createEmployeePayrollItem(hourly);

        // And reimbursement item.  Alternatively could add on the fly to the paycheck only... 
        PayrollItemIf reimburse = context.getPayrollItem(Gemini.PItemID.OTHER_REIMBURSEMENT);
        EmployeePayrollItemIf reimburseEpi = e.createEmployeePayrollItem(reimburse);

        // Create Payroll Engine paycheck object
        QBDate checkDate = new QBDate (6,1,2007);
        PaycheckIf paycheck = e.createPaycheck(checkDate);
        PaycheckLineIf reimburseLine = null;
        for (EmployeePayrollItemIf epi : e.getEmployeePayrollItems()) {

            loadEmployeePayrollItemInfo(epi, true);

            // Paycheck line
            PaycheckLineIf line = paycheck.createLine(epi);
            loadPaycheckInfo (line);

            if (ID_MAP.get(Gemini.PItemID.OTHER_REIMBURSEMENT).equals(epi.getPayrollItem().getUniqueID())) {
                reimburseLine = line;
            }
        }

        // For fun, also create the data for pay employee grid
        PayEmployeesIf payEmployeesGrid = co.createPayEmployees(checkDate);
        payEmployeesGrid.addEmployee(e, false);
        payEmployeesGrid.addEmployeePayrollItem(e, reimburseEpi);
        PayrollDatumIf gridDatum = payEmployeesGrid.getCell(e, reimburse);

        // Default pitem CALC_BASED_ON for other reimbursement is GROSS
        // Other choices are QUANTITY and FLAT_AMOUNT
        PayrollDatumIf calcBasedOn = reimburse.getDatum(DatumID.CALC_BASED_ON);
        Gemini.myAssert(CalcBasedOn.GROSS.equals(calcBasedOn.getValue()));
        System.out.println ("Allowed calc options: " + Arrays.asList(calcBasedOn.getValidValues()));

        // Default pitem CALC_BASED_ON for other reimbursement is GROSS
        // so adding a 1% rate will result 2000 * 1% = 20 amount
        System.out.println(gridDatum.getNote());    // flyover suggestion on the pay employee grid
        reimburseLine.setDatumValue(DatumID.RATE, new Rate("1%"));
        PayrollDatumIf amt = reimburseLine.getDatum(DatumID.AMOUNT);
        PayrollDatumIf qty = reimburseLine.getDatum(DatumID.QUANTITY);
        System.out.println("2000 * 1% = " + amt.getValue());
        System.out.println();

        // RATE field is editable on the paycheck
        PayrollDatumIf rate = reimburseLine.getDatum(DatumID.RATE);
        Gemini.myAssert (rate.isEditable());
        Gemini.myAssert (amt.isEditable());
        Gemini.myAssert (! qty.isEditable());


        // Try flat amount
        calcBasedOn.setValue(CalcBasedOn.FLAT_AMOUNT);
        System.out.println(gridDatum.getNote());    // flyover suggestion on the pay employee grid
        Gemini.myAssert (rate.isEditable());
        Gemini.myAssert (amt.isEditable());
        Gemini.myAssert (! qty.isEditable());
        amt.setValue(new Money(10));
        // Normally flat amount is taken from the employee setup
        // If you want to specify flat amount on the paycheck line, you need
        // to let the payroll engine know
        reimburseLine.setDatumValue(DatumID.USER_OVERRODE_AMOUNT, Boolean.TRUE);
        System.out.println("10.00 = " + amt.getValue());
        System.out.println();
        // Undo override for other examples
        reimburseLine.setDatumValue(DatumID.USER_OVERRODE_AMOUNT, Boolean.FALSE);


        // Try based on quantity
        calcBasedOn.setValue(CalcBasedOn.QUANTITY);
        System.out.println(gridDatum.getNote());    // flyover suggestion on the pay employee grid
        Gemini.myAssert (rate.isEditable());
        Gemini.myAssert (amt.isEditable());
        Gemini.myAssert (qty.isEditable());
        reimburseLine.setDatumValue(DatumID.QUANTITY, new Quantity(100));
        reimburseLine.setDatumValue(DatumID.RATE, new Money(10));
        System.out.println("100(qty) * 10(rate) = " + amt.getValue());
        System.out.println();

        // rate can be % too
        reimburseLine.setDatumValue(DatumID.RATE, new Rate("30%"));
        System.out.println("100(qty) * 30%(rate) = " + amt.getValue());


//        System.out.println(paycheck);
    }

    public static void main (String[] args) {

        // First day paychecks are allowed in the system (for this company)
        QBDate payrollStartDate = new QBDate (5,1,2007);

        // Create app's context for this company
        Gemini.Context context = new Gemini.Context(payrollStartDate);

        // Set company-specific SUI rate etc
        loadCompanyInfo(context);

        List<String> params = Arrays.asList(args);
        if (params.contains("salary")) {
            // salary paychecks
            createSalaryPaycheck(context);
        } else if (params.contains("hourly")) {
            // Create simple hourly paycheck
            createHoulyPaycheck(context);
        } else if (params.contains("reimburse")) {
            createReimbursementPaycheck(context);
        } else {
            System.out.println ("Try 'salary', 'hourly' or 'reimburse'");
        }
    }
}
