package com.intuit.sbfinance.json;

import com.intuit.schema.finance.v3.BatchItemRequest;
import com.intuit.schema.finance.v3.BatchItemResponse;
import com.intuit.schema.finance.v3.RecurringTransaction;
import com.intuit.schema.finance.v3.IntuitResponse;
import com.intuit.schema.finance.v3.IntuitEntity;
import com.intuit.schema.finance.v3.ObjectFactory;
import com.intuit.schema.finance.v3.QueryResponse;

import org.codehaus.jackson.JsonParser;
import org.codehaus.jackson.JsonProcessingException;
import org.codehaus.jackson.map.DeserializationContext;
import org.codehaus.jackson.map.DeserializationProblemHandler;
import org.codehaus.jackson.map.JsonDeserializer;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.JavaType;

import java.io.IOException;
import java.util.List;

/**
 * This deserialization handler takes cares of the special json text for IntuitObject that are JAXBElement
 * or a list of JAXBElement. Since we serialize these elements to be like an entity itself, the default
 * Java bean that is generated from the XSD will not contain these elements. So they will be forwarded to
 * the handler to figure out how to handle the elements.
 * 
 * In this handler, we then read the object back as a entity, and then set it correctly to the appropriate field.
 * @author yujungs700
 *
 */
public class CustomDeserializationProblemHandler extends DeserializationProblemHandler {
    
    private static ObjectFactory objFactory = new ObjectFactory();
    private static ObjectMapper mapper = new ObjectMapper();
    static {
        SbFinanceJacksonJsonProvider.setupObjectMapperDeserializationConfig(mapper);
    }
    
    @Override
    public boolean handleUnknownProperty(DeserializationContext ctxt, JsonDeserializer<?> deserializer, Object bean, String propertyName)
            throws IOException, JsonProcessingException {
        if (bean instanceof BatchItemRequest) {
            Object intuitObject = deserializeJaxbElement(ctxt, deserializer, propertyName);
            if (intuitObject != null) {
                ((BatchItemRequest) bean).setIntuitObject(objFactory.createIntuitObject((IntuitEntity)intuitObject));
                return true;
            }
        } else if (bean instanceof IntuitResponse) {
            Object intuitObject = deserializeJaxbElement(ctxt, deserializer, propertyName);
            if (intuitObject != null) {
                ((IntuitResponse) bean).setIntuitObject(objFactory.createIntuitObject((IntuitEntity)intuitObject));
                return true;
            }
        } else if (bean instanceof BatchItemResponse) {
            Object intuitObject = deserializeJaxbElement(ctxt, deserializer, propertyName);
            if (intuitObject != null) {
                ((BatchItemResponse) bean).setIntuitObject(objFactory.createIntuitObject((IntuitEntity)intuitObject));
                return true;
            }
        } else if (bean instanceof RecurringTransaction) {
            Object intuitObject = deserializeJaxbElement(ctxt, deserializer, propertyName);
            if (intuitObject != null) {
                ((RecurringTransaction) bean).setIntuitObject(objFactory.createIntuitObject((IntuitEntity)intuitObject));
                return true;
            }
        } else if (bean instanceof QueryResponse) {
            Class<?> v3EntityClass = JsonUtils.getServiceV3EntityClass(propertyName);
            if (v3EntityClass != null) {
                JsonParser jParser = ctxt.getParser();
                JavaType jsonType = ctxt.getTypeFactory().constructCollectionType(List.class, v3EntityClass);
                JsonDeserializer<Object> listEntitySerializer = ctxt.getDeserializerProvider().findValueDeserializer(ctxt.getConfig(), jsonType, null);
                @SuppressWarnings("unchecked")
                List<IntuitEntity> valueList = (List<IntuitEntity>)listEntitySerializer.deserialize(jParser, ctxt);
                QueryResponse queryBean = (QueryResponse)bean;
                for (IntuitEntity obj : valueList) {
                    queryBean.getIntuitObject().add(objFactory.createIntuitObject(obj));
                }
                return true;
            }
        }
        return super.handleUnknownProperty(ctxt, deserializer, bean, propertyName);
    }

    private Object deserializeJaxbElement(DeserializationContext ctxt, JsonDeserializer<?> deserializer, String propertyName) throws IOException, JsonProcessingException {
        Class<?> v3EntityClass = JsonUtils.getServiceV3EntityClass(propertyName);
        if (v3EntityClass != null) {
            JsonParser jParser = ctxt.getParser();
            JavaType jsonType = ctxt.getTypeFactory().constructType(v3EntityClass);
            JsonDeserializer<Object> entitySerializer = ctxt.getDeserializerProvider().findValueDeserializer(ctxt.getConfig(), jsonType, null);
            return entitySerializer.deserialize(jParser, ctxt);
        }
        return null;
    }
}
