package com.intuit.sbfinance.json;

import com.intuit.schema.finance.v3.BatchItemRequest;
import com.intuit.schema.finance.v3.BatchItemResponse;
import com.intuit.schema.finance.v3.RecurringTransaction;
import com.intuit.schema.finance.v3.IntuitResponse;
import com.intuit.schema.finance.v3.IntuitEntity;
import com.intuit.schema.finance.v3.QueryResponse;

import org.codehaus.jackson.JsonGenerator;
import org.codehaus.jackson.map.JsonSerializer;
import org.codehaus.jackson.map.SerializerProvider;
import org.codehaus.jackson.map.ser.BeanPropertyFilter;
import org.codehaus.jackson.map.ser.BeanPropertyWriter;

import javax.xml.bind.JAXBElement;

import java.util.ArrayList;
import java.util.List;

/**
 * This class is a JSON property filter that looks at specific elements of v3 services' generated class, and customize its serialization.
 * In particular in QBO service V3, there are several elements that are of a type JAXBElement with value that extends from IntuitEntity.
 * JSON by default serialize extra information about the JAXBElement with the same name from the field name. This makes the serializer
 * writes out the same JSON note name, but with different content from each type.
 * 
 * So here what we want to do is to simulate what JAXB marshaller will do, which is to serialze the class name with the content.
 * @author yujungs700
 *
 */
public class JaxbElementPropertyFilter implements BeanPropertyFilter {

    private final static String INTUIT_OBJECT_FIELD_NAME = "IntuitObject";
    
    @SuppressWarnings("unchecked")
    public void serializeAsField(Object bean, JsonGenerator jgen, SerializerProvider provider, BeanPropertyWriter writer) throws Exception {
        // based on class type, modify the property that we want to change only
        if (bean instanceof IntuitResponse || 
            bean instanceof BatchItemResponse ||
            bean instanceof RecurringTransaction ||
            bean instanceof BatchItemRequest) { // we don't actually need to serialize BatchItemRequest from the server, but this helps with testing
            if (writer.getName().equals(INTUIT_OBJECT_FIELD_NAME)) {
                JAXBElement<? extends IntuitEntity> jaxbElement = (JAXBElement<? extends IntuitEntity>)writer.get(bean);
                if (jaxbElement != null) {
                    IntuitEntity obj  = jaxbElement.getValue();
                    jgen.writeFieldName(JsonUtils.getClassName(obj));
                    JsonSerializer<Object> ser = provider.findValueSerializer(obj.getClass(), null);
                    ser.serialize(obj, jgen, provider);
                    return;
                }
            }
        } else if (bean instanceof QueryResponse) {
            if (writer.getName().equals(INTUIT_OBJECT_FIELD_NAME)) {
                List<JAXBElement<? extends IntuitEntity>> jaxbElementList = (List<JAXBElement<? extends IntuitEntity>>)writer.get(bean);
                if (jaxbElementList != null) {
                    List<IntuitEntity> valueList = new ArrayList<IntuitEntity>();
                    for (JAXBElement<? extends IntuitEntity> jaxbElement : jaxbElementList) {
                        valueList.add(jaxbElement.getValue());
                    }
                    // get field name from first element class name
                    // here we just want to write out the class name not the actual field name because the field name by itself
                    // does not tell JSON serializer enough info about which class to map to
                    jgen.writeFieldName(JsonUtils.getClassName(valueList.get(0)));
                    JsonSerializer<Object> ser = provider.findValueSerializer(valueList.getClass(), null);
                    ser.serialize(valueList, jgen, provider);
                    return;
                }
            }
        } 
        writer.serializeAsField(bean, jgen, provider);
    }
    
}
