package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.Set;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;
import com.intuit.sb.payroll.entity.model.EmployerAdjLineItem;
import com.intuit.sb.payroll.entity.model.EmployerAdjustment;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckLineItem;
import com.intuit.sb.payroll.entity.model.PaycheckWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PayrollItemDtlPtdDatum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.utils.PtdUtils;

/**
 * Re-usable and independantly testable functions used within the WorkerPayments Component live here.
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
public class WorkerPaymentUtilities {

	/**
	 * Sums up the running totals on the {@link PayrollRun} based on the values in the {@link Paycheck}s.
	 * 
	 * This should be called when new {@link PayrollRun}s are committed or when {@link Paycheck}s within
	 * a {@link PayrollRun} are modified (Voided cancelled) as the totals may change as a result.
	 * 
	 * Note: Voided and Cancelled Paychecks will not be a part of the calculation.
	 * 
	 * @param payrollRun
	 */
	static void updatePayrollRunTotals(PayrollRun payrollRun) {
		Money netPay = new Money();
		Money grossPay = new Money();
		Money workerPdTaxes = new Money();
		Money employerPdTaxes = new Money();
		Money workerPdBenefits = new Money();
		Money employerPdBenefits = new Money();
		
		for (Paycheck paycheck : payrollRun.getPaychecks()) {
			if (paycheck.getWorkflowStatus() != PaycheckWorkflowStatusEnum.VOIDED) {
				//Adjust Running totals
	            netPay = netPay.add(paycheck.getNetAmount());
	            grossPay = grossPay.add(paycheck.getGrossAmount());
	            workerPdTaxes = workerPdTaxes.add(paycheck.getWorkerPdTaxes());
	            employerPdTaxes = employerPdTaxes.add(paycheck.getEmployerPdTaxes());
	            workerPdBenefits = workerPdBenefits.add(paycheck.getWorkerPdBenefits());
	            employerPdBenefits = employerPdBenefits.add(paycheck.getEmployerPdBenefits());
			}
		}
		
		Tax tax;
		for(EmployerAdjustment employerAdjustment : payrollRun.getEmployerAdjustments()) {
			for(EmployerAdjLineItem employerAdjLineItem : employerAdjustment.getEmployerAdjLineItems()) {
				tax = (Tax) employerAdjLineItem.getPayrollItem();
				switch(tax.getSourceEntity()) {
					case WORKER:
						workerPdTaxes = workerPdTaxes.add(employerAdjLineItem.getAdjustmentAmount());
						break;
					case EMPLOYER:
						employerPdTaxes = employerPdTaxes.add(employerAdjLineItem.getAdjustmentAmount());
						break;
					default:
						break;					
				}
			}
		}
		
		payrollRun.setTotalNetPay(netPay);
		payrollRun.setTotalGrossPay(grossPay);
		payrollRun.setTotalWorkerPdTaxes(workerPdTaxes);
		payrollRun.setTotalEmployerPdTaxes(employerPdTaxes);
		payrollRun.setTotalWorkerPdBenefits(workerPdBenefits);
		payrollRun.setTotalEmployerPdBenefits(employerPdBenefits);
	}
	
	/**
	 * Returns true if all the {@link Paycheck}s in the {@link PayrollRun}.
	 * @param payrollRun
	 * @return
	 */
	static boolean distributionComplete(PayrollRun payrollRun) {
		Set<Paycheck> paychecks = payrollRun.getPaychecks();
		for (Paycheck paycheck : paychecks) {
			if (paycheck.getWorkflowStatus() != PaycheckWorkflowStatusEnum.DISTRIBUTED &&
				paycheck.getWorkflowStatus() != PaycheckWorkflowStatusEnum.VOIDED) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * A worker is ready for payment if they meet the workflow requirements as well as the date constraints (they have
	 * to be actively employed).
	 * 
	 * @param worker
	 * @param payrollRun
	 * @return true if the Worker is ready for payment.
	 */
	static boolean isWorkerReadyForPayment(Worker worker, PayrollRun payrollRun) {
		//Check last state evaluation
		final boolean workerReadyForPay = worker.isReadyForPay();
		if (!workerReadyForPay) {
			return false;
		}

		//Check to make sure that this worker falls into this period.
		final boolean workerHireDatePriorToPeriodEnd = 
			DateUtil.isBeforeOrOnSameDay(worker.getEmploymentPeriod().getStartDate(), 
														payrollRun.getPayPeriod().getEndDate());
		if (!workerHireDatePriorToPeriodEnd) {
			return false;
		}
		//All clear
		return true;
	}

	/**
	 * Removes the difference of the voided check
	 * @param worker
	 * @param paycheck
	 */
	public static void updatePtdsForWorkerAfterVoid(Worker worker, Paycheck paycheck) {
		//The line items on the paycheck relate to their respective PayrollItemDtls, adjust them accordingly.
		Set<PaycheckLineItem> paycheckLineItems = paycheck.getPaycheckLineItems();
		for (PaycheckLineItem paycheckLineItem : paycheckLineItems) {
			final PayrollItemDtl sourcePayrollItemDtl = paycheckLineItem.getSourcePayrollItemDtl();
			//Type means Q1, Q2 etc.  We only want to change the relevant quarter.
			PtdTypeEnum type = PtdUtils.getPtdTypeForDate(paycheck.getPayDate());
			final PayrollItemDtlPtdDatum payrollItemDatum = sourcePayrollItemDtl.getPtdDatumOfType(type);

			//FIXME handle quantity as well
			final PtdDatumType datum = payrollItemDatum.getDatum();
			if (datum.getDatumType() == DatumType1Enum.MONEY) {
				final PtdDatumType paycheckDatum = paycheckLineItem.getPtdDatumOfType(type).getDatum();
				final Money incomeSubjectToTax = datum.getIncomeSubjectToTaxValue();
				final Money total = datum.getMoneyValue();
				final Money wageBase = datum.getWageBaseValue();
				//Remove by the amount on the line item datum.
				datum.setIncomeSubjectToTaxValue(incomeSubjectToTax.subtract(paycheckDatum.getIncomeSubjectToTaxValue()));
				datum.setMoneyValue(total.subtract(paycheckDatum.getMoneyValue()));
				datum.setWageBaseValue(wageBase.subtract(paycheckDatum.getWageBaseValue()));
			}
		}
	}
}
