package com.intuit.sb.payroll;

import java.util.Collections;
import java.util.List;

import javax.ejb.ApplicationException;

/**
 * A WorkflowException occurs when there is a violation of the business
 * process (as opposed to a data constraint, as manifested in the 
 * ValidationException).
 * 
 * The exception has a list of Tasks that need to be completed in order to
 * execute the relevant business process step.
 */
@ApplicationException(rollback=true)
public class WorkflowException extends Exception {

	/**
	 * Serial Id
	 */
	private static final long serialVersionUID = 3606563469639434381L;

	/**
	 * A List of tasks to complete (if any).
	 */
	private List<TaskDesc> tasksToComplete;
	
	@SuppressWarnings("unchecked")
	private void init() {
		tasksToComplete = Collections.EMPTY_LIST;
	}

	/**
	 * Basic constructor.
	 */
	public WorkflowException() {
		super();
		init();
	}

	/**
	 * Common case constructor.
	 * @param message A text message that provides context on the specific exception case.
	 */
	public WorkflowException(String message) {
		super(message);
		init();
	}

	/**
	 * Exception capturing a message and a list of tasks that have not been completed. These
	 * tasks are prerequisites which must be completed before the operation can succeed.
	 * @param message A text message that provides context on the specific exception case.
	 * @param tasksToComplete the ordered list of incomplete tasks.
	 */
	public WorkflowException(String message, List<TaskDesc> tasksToComplete) {
		super(message);
		setTasksToComplete(tasksToComplete);
	}

	/**
	 * Exception capturing a nested cause indicating why validation of the required workflow
	 * could not be completed. Note that this does not necessarily mean the workflow is
	 * incomplete but more likely indicates a problem with the workflow system itself.
	 * This form of the exception should be used with care as nested exceptions rarely
	 * contain any useful information for a user. The user is unable to take action based
	 * on the exception stack trace and is better served by a focused message.
	 * @param cause the nested cause.
	 */
	public WorkflowException(Throwable cause) {
		super(cause);
		init();
	}

	/**
	 * Exception capturing a message and nested cause indicating why validation of the required workflow
	 * could not be completed. Note that this does not necessarily mean the workflow is
	 * incomplete but more likely indicates a problem with the workflow system itself.
	 * This form of the exception should be used with care as nested exceptions rarely
	 * contain any useful information for a user. The user is unable to take action based
	 * on the exception stack trace and is better served by a simple focused message without
	 * the addition of the nested cause.
	 * @param message A text message that provides context on the specific exception case.
	 * @param cause the nested cause.
	 */
	public WorkflowException(String message, Throwable cause) {
		super(message, cause);
		init();
	}

	/**
	 * Gets the list of incomplete tasks that must be performed before the operation can
	 * succeed.
	 * @return the ordered list of tasks that need to be completed in the returned sequence.
	 */
	public List<TaskDesc> getTasksToComplete() {
		return tasksToComplete;
	}

	/**
	 * Sets a list of tasks that have not been completed. These tasks are prerequisites
	 * which must be completed before the operation can succeed.
	 * @param tasksToComplete the ordered list of incomplete tasks.
	 * @throws IllegalArgumentException if tasksToComplete is null.
	 */
	public void setTasksToComplete(List<TaskDesc> tasksToComplete) {
		if (tasksToComplete == null) {
			throw new IllegalArgumentException("List of Tasks To Complete must not be null");
		}
		this.tasksToComplete = tasksToComplete;
	}
}
