package com.intuit.sb.payroll.common.i18n;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;

/**
 * <p>This class is responsible for creating locale-specific instances of international abstract classes
 * and interfaces.</p>
 * 
 * <p>This class depends on a property file located in the class path at the location specified by the 
 * <code>localeobjectfactory.config</code> system property. The property file defines a mapping between
 * the fully-qualified abstract class (or interface) name and the fully-qualified concrete implementation name.
 * The configuration file is loaded from the <code>com.intuit.sb.payroll.common</code> package.</p>
 * 
 * <p>For example,</p>
 * 
 * <pre><![CDATA[
 * com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.NationalRegionMapper=com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.PayrollStateMapper
 * ]]></pre>
 * 
 * <p>The preceding example will specify that an object of type <code>PayrollStateMapper</code> will be constructed
 * when the abstract type <code>NationalRegionMapper</code> is requested. Note that the concrete class must 
 * implement/extend the abstract class or interface.</p>
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
@SuppressWarnings("nls")
public class LocaleObjectFactory {
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(LocaleObjectFactory.class);

	public static final String CONFIG_PROPERTY = "localeobjectfactory.config";
	public static final String DEFAULT_CONFIG_LOCATION = "/com/intuit/sb/payroll/localeobjectfactory-config.properties";

	private static final String RESOURCE_BUNDLE = "com.intuit.sb.payroll.common.i18n.localeobjectfactory-resources";

	private static LocaleObjectFactory instance;

	private final Map<Class<?>, Class<?>> classMap = new HashMap<Class<?>, Class<?>>();

	public static LocaleObjectFactory getInstance() {
		// In normal operation, instance will be initialized in the static constructor. In testing, however,
		// it's possible for the instance to be cleared, resulting in a null instance. Here, we will initialize
		// the singleton in that case.
		if (null == instance) {
			// Note: not synchronizing - the tests are not multithreaded.
			instance = new LocaleObjectFactory();
		}

		return instance;
	}

	private LocaleObjectFactory() {
		try {
			configure();
		} catch (IOException e) {
			String msg = LocaleGuru.getInstance().getString(RESOURCE_BUNDLE, "config.error");

			sLogger.error(msg, e);

			throw new InfrastructureException(msg, e);
		}
	}

	private synchronized void configure() throws IOException {
		String configLocation = System.getProperty(CONFIG_PROPERTY);

		if (null == configLocation || configLocation.length() == 0) {
			configLocation = DEFAULT_CONFIG_LOCATION;
		}

		InputStream stream = getClass().getResourceAsStream(configLocation);

		if (null == stream) {
			String msg = LocaleGuru.getInstance().getString(RESOURCE_BUNDLE, "config.missing", configLocation);

			sLogger.error(msg);

			throw new InfrastructureException(msg);
		}

		Properties props = new Properties();

		try {
			props.load(stream);
		} finally {
			stream.close();
		}

		loadMappings(props);
	}

	private void loadMappings(Properties props) {
		Enumeration<?> propertyNames = props.propertyNames();

		while (propertyNames.hasMoreElements()) {
			String abstractClassName = (String) propertyNames.nextElement();

			String concreteClassName = props.getProperty(abstractClassName);

			Class<?> abstractClass = loadClass(abstractClassName);
			Class<?> concreteClass = loadClass(concreteClassName);

			// make sure the concrete class extends or implements the abstract class
			if (!abstractClass.isAssignableFrom(concreteClass)) {
				// not assignable
				String msg = LocaleGuru.getInstance().getString(RESOURCE_BUNDLE, "classes.notassignable",
						concreteClassName, abstractClassName);

				sLogger.error(msg);

				throw new InfrastructureException(msg);
			}

			// Note: not checking the class for whether it can be instantiated: we'll check that later when we
			// try to instantiate it - this way we can let the JRE figure that out for us.
			classMap.put(abstractClass, concreteClass);
		}
	}

	private Class<?> loadClass(String className) {
		try {
            return Class.forName(className);
		} catch (ClassNotFoundException e) {
			String msg = LocaleGuru.getInstance().getString(RESOURCE_BUNDLE, "class.notfound", className);

			sLogger.error(msg);

			throw new InfrastructureException(msg);
		}
	}

	public Class<?> getClass(Class<?> abstractClass) {
		return classMap.get(abstractClass);
	}

	@SuppressWarnings("unchecked")
	public <T> T newInstance(Class<T> abstractClass) {
		Class<?> clazz = getClass(abstractClass);

		try {
			Object obj = clazz.newInstance();

			// this is an unchecked cast but we checked for consistency when we loaded the configuration
			// so this should be okay.
			return (T) obj;
		} catch (Exception e) {
			String msg = LocaleGuru.getInstance().getString(RESOURCE_BUNDLE, "class.creationerror", clazz.getName());

			sLogger.error(msg);

			throw new InfrastructureException(msg);
		}
	}

	static void clear() {
		instance = null;
	}
}
