package com.intuit.sb.payroll.common.validation;

import java.math.BigDecimal;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;

/**
 * Utility class for validating passed parameters.
 * 
 */
public class PayrollParamValidator {
	/**
	 * Checks to ensure that the passed value is positive number
	 * 
	 * @param variable Value to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * 
	 * @throws ValidationException if an argument is null
	 * @throws ValidationException if <code>number</code> is not positive or a string argument is "". The exception
	 *             message includes the parameter name for easy identification.
	 */
	public static void checkIsPositive(long variable, String variableName) throws ValidationException {
		checkIsNotNullOrEmptyString(variableName, "variableName");

		if (variable <= 0) {
			throw new ValidationException(String.valueOf(variable));
		}
	}

	/**
	 * Checks to ensure that the passed value is a non-negative number. That is, the number must be greater than or
	 * equal to 0 or an exception will be thrown.
	 * 
	 * @param variable Value to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * 
	 * @throws ValidationException if an argument is null
	 * @throws ValidationException if <code>number</code> is not positive or a string argument is "". The exception
	 *             message includes the parameter name for easy identification.
	 */
	public static void checkIsNonNegative(long variable, String variableName) throws ValidationException {
		checkIsNotNullOrEmptyString(variableName, "variableName");

		if (variable < 0) {
			throw new ValidationException(String.valueOf(variable));
		}
	}

	/**
	 * Checks to ensure that the passed value is a non-negative number. That is, the number must be greater than or
	 * equal to 0 or an exception will be thrown.
	 * 
	 * @param variable Value to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * 
	 * @throws ValidationException if an argument is null
	 * @throws ValidationException if <code>number</code> is not positive or a string argument is "". The exception
	 *             message includes the parameter name for easy identification.
	 */
	public static void checkIsNonNegative(BigDecimal variable, String variableName) throws ValidationException {
		checkIsNotNullOrEmptyString(variableName, "variableName");

		if (variable.compareTo(new BigDecimal("0")) < 0) {
			throw new ValidationException(String.valueOf(variable));
		}
	}

	/**
	 * Checks to ensure that the passed value is not null
	 * 
	 * @param variable Value to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * 
	 * @throws ValidationException if an argument is null
	 * @throws ValidationException if a string argument is "". The exception message includes the parameter name for
	 *             easy identification.
	 */
	public static void checkIsNotNull(Object variable, String variableName) throws ValidationException {
		if (variableName == null) {
			throw new ValidationException("Variable name is null");
		}

		if (variableName.length() == 0) {
			throw new ValidationException(variableName + " is empty");
		}

		if (variable == null) {
			throw new ValidationException(variableName + " is null");
		}
	}

	/**
	 * Checks to ensure that the passed string is not "" or null
	 * 
	 * @param variable Value to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * 
	 * @throws ValidationException if an argument is null
	 * @throws ValidationException if a string argument is "". The exception message includes the parameter name for
	 *             easy identification.
	 */
	public static void checkIsNotNullOrEmptyString(String variable, String variableName) throws ValidationException {
		checkIsNotNull(variable, variableName);

		if (variable.length() == 0) {
			throw new ValidationException(variableName + " is empty");
		}
	}

	/**
	 * Checks to ensure that the passed string is not blank (contains 0 or more white spaces only) or null
	 * 
	 * @param variable Value to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * 
	 * @throws ValidationException if an argument is null
	 * @throws ValidationException if a string argument is blank. The exception message includes the parameter name for
	 *             easy identification.
	 */
	public static void checkIsNotNullOrBlankString(String variable, String variableName) throws ValidationException {
		checkIsNotNull(variable, variableName);

		if (variable.trim().length() == 0) {
			throw new ValidationException(variableName + " is empty");
		}
	}

	/**
	 * Checks to ensure that the passesd array variable is not null or empty.
	 * 
	 * @param variable Array variable to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * @throws ValidationException if variable is null
	 * @throws ValidationException if array variable is empty.
	 */
	public static void checkIsNotNullOrEmptyArray(Object[] variable, String variableName) throws ValidationException {
		checkIsNotNull(variable, variableName);
		if (variable.length == 0) {
			throw new ValidationException(variableName + " is empty");
		}
	}

	/**
	 * Checks to ensure that the passed value is true
	 * 
	 * @param variable Value to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * 
	 * @throws ValidationException if an argument is null
	 * @throws ValidationException if variable is false or a string argument is "". The exception message includes the
	 *             parameter name for easy identification.
	 */
	public static void checkIsTrue(boolean variable, String variableName) throws ValidationException {
		checkIsNotNullOrEmptyString(variableName, "variableName");

		if (variable != true) {
			throw new ValidationException(variableName + " is false");
		}
	}

	/**
	 * Checks to ensure that the passed value is true
	 * 
	 * @param variable Value to be checked.
	 * @param variableName Name used to identify variable in error message.
	 * @param message The message used in the generated exception if the test fails
	 * 
	 * @throws ValidationException if an argument is null
	 * @throws ValidationException if variable is false or a string argument is "". The exception message includes the
	 *             parameter name for easy identification.
	 */
	public static void checkIsTrue(boolean variable, String variableName, String message) throws ValidationException {
		checkIsNotNullOrEmptyString(variableName, "variableName");
		checkIsNotNullOrEmptyString(message, "message");

		if (!variable) {
			throw new ValidationException(message);
		}
	}

	/**
	 * Validates that the buffer, the offset, and the count are all valid. This function should be used when copying
	 * into or out of array buffers. For example, if you are reading a series of bytes into a buffer, the offset would
	 * refer to the position in the buffer into which the copying will begin and count refers to the number of bytes
	 * that will be read into that buffer.
	 * 
	 * @param buffer the array to check
	 * @param offset the position within the buffer in which the operation (such as reading or writing) will begin
	 * @param count the number of elements that will be affected by the operation
	 * @throws ValidationException if buffer is null
	 * @throws ValidationException if offset is less than 0 or count is less than 0
	 * @throws ValidationException if offset + count is greater than the buffer's length
	 */
	public static void checkArrayParams(byte[] buffer, int offset, int count) throws ValidationException{
		// Make sure the array is not null:
		checkIsNotNull(buffer, "Buffer");

		// Offset cannot be negative. That is, it must be >= 0.
		checkIsNonNegative((long) offset, "Offset");

		// Count cannot be negative. That is, it must be >= 0.
		checkIsNonNegative((long) count, "Count");

		// Offset + count must be less than the length of the array:
		if (offset + count > buffer.length) {
			throw new ValidationException("Offset + count cannot be greater than the size of the buffer.");
		}
	}

	/**
	 * Validates that the buffer, the offset, and the count are all valid. This function should be used when copying
	 * into or out of array buffers. For example, if you are reading a series of bytes into a buffer, the offset would
	 * refer to the position in the buffer into which the copying will begin and count refers to the number of bytes
	 * that will be read into that buffer.
	 * 
	 * @param buffer the array to check
	 * @param offset the position within the buffer in which the operation (such as reading or writing) will begin
	 * @param count the number of elements that will be affected by the operation
	 * @throws ValidationException if buffer is null
	 * @throws ValidationException if offset is less than 0, count is less than 0, or offset + count is greater than the
	 *             buffer's length
	 */
	public static void checkArrayParams(char[] buffer, int offset, int count) throws ValidationException {
		// Make sure the array is not null:
		checkIsNotNull(buffer, "Buffer");

		// Offset cannot be negative. That is, it must be >= 0.
		checkIsNonNegative((long) offset, "Offset");

		// Count cannot be negative. That is, it must be >= 0.
		checkIsNonNegative((long) count, "Count");

		// Offset + count must be less than the length of the array:
		if (offset + count > buffer.length) {
			throw new ValidationException("Offset + count cannot be greater than the size of the buffer.");
		}
	}
	
	/**
	 * Ensures that a date is greater than or equal to today. Today starts at time 0 on today's
	 * date so that any hour/minute/second/millisecond value in the supplied date is ignored
	 * for the purposes of the comparison.
	 * @param date the date to validate.
	 * @throws ValidationException if the date is before today.
	 */
	public static void checkDateNotBeforeToday(Date date) throws ValidationException {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		Date startOfToday = DateUtil.createCalendar(now.get(Calendar.YEAR), now.get(Calendar.MONTH),
		                                            now.get(Calendar.DAY_OF_MONTH)).getTime();
		if (date.before(startOfToday)) {
			SimpleDateFormat simpleDateFormat = new SimpleDateFormat("MMMMM d yyyy");
			String formattedDate = simpleDateFormat.format(date);
			//FIXME [I18N] resource translation
			String mesg = MessageFormat.format("Date {0} is before today", formattedDate);
			throw new ValidationException(mesg);
		}
	}

	/**
	 * Ensures that a date is less than or equal to today. Today starts at time 0 on today's
	 * date so that any hour/minute/second/millisecond value in the supplied date are ignored
	 * for the purposes of the comparison.
	 * @param date the date to validate.
	 * @throws ValidationException if the date is after today.
	 */
	public static void checkDateNotAfterToday(Date date) throws ValidationException {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		Calendar startOfTomorrow = DateUtil.createCalendar(now.get(Calendar.YEAR), now.get(Calendar.MONTH),
		                                                   now.get(Calendar.DAY_OF_MONTH));
		startOfTomorrow.add(Calendar.DAY_OF_MONTH, 1); 
		if (date.after(startOfTomorrow.getTime())) {
			SimpleDateFormat simpleDateFormat = new SimpleDateFormat("MMMMM d yyyy");
			String formattedDate = simpleDateFormat.format(date);
			//FIXME [I18N] resource translation
			String mesg = MessageFormat.format("Date {0} is after today", formattedDate);
			throw new ValidationException(mesg);
		}
	}

	/**
	 * Validate the Given {@link List List} of objects so that:
	 * <ul>
	 * 		<li>The List is not null</li>
	 * 		<li>The List has AT LEAST ONE element (i.e. - its size() > 0) and</li>
	 * 		<li>The contents of the List are non-null Objects</li>
	 * </ul>
	 * 
	 * This is added because current available methods in this class use "Array".<br>
	 * A List can be converted to Array only if it is non-null.<br>
	 * @see PayrollParamValidator.checkIsNotNullOrEmptyArray
	 * PayrollParamValidator.checkIsNotNullOrEmptyArray does not check the contents of the array<br>
	 * 
	 * @param list the {@link List List} of objects to be checked
	 * @param variableName Reference name for the list
	 * @throws ValidationException
	 */
	public static void checkListAndContents(List<? extends Object> list, String variableName) 
			throws ValidationException {
//		"list" should not be null
		checkIsNotNull(list, variableName);	
		
//		"list" should have AT LEAST ONE element
		if (list.size() == 0) {				
			throw new ValidationException(variableName + " is empty");
		}
		
//		"list" elements should be non-null
		for (int j = 0; j < list.size(); j++) {
			checkIsNotNull(list.get(j), variableName + "[" + j + "]");
		}
	}
}
