package com.intuit.sb.payroll.common.i18n;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.Calendar;
import java.util.Locale;
import java.util.TimeZone;

import org.testng.Assert;
import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.InfrastructureException;

/**
 * Tests the LocaleGuru.
 * 
 * @author gcoulombe
 *
 */
public class LocaleGuruTest
{
	@Test
	public void testConfigureFromPropertiesFile()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "test_localeprofile.properties");
		
		// should have offering and default profile, no user profile
		LocaleGuru objectFactory = LocaleGuru.getInstance();
		
		AssertJUnit.assertEquals("hi", objectFactory.getLanguage());
		AssertJUnit.assertEquals("GB", objectFactory.getCountry());
	}
	
	@AfterMethod
	public void tearDown()
	{
		// reset the singleton between calls
		LocaleGuru.clear();
		
		System.clearProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY);
	}
	
	@Test
	public void testConfigureWithNoPropertiesFile()
	{
		// should have default profile only. No user profile.
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru objectFactory = LocaleGuru.getInstance();
		
		AssertJUnit.assertEquals(Locale.getDefault().getLanguage(), objectFactory.getLanguage());
		AssertJUnit.assertEquals(Locale.getDefault().getCountry(), objectFactory.getCountry());
	}
	
	@Test(expectedExceptions=InfrastructureException.class)
	public void testBogusTimeZone()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		// Make sure the user profile is set to something so we'll be sure the getOfferingLocale call
		// ignores it.
		LocaleProfile userProfile = new LocaleProfile();
		
		userProfile.setCountry("IN");
		userProfile.setLanguage("hi");
		userProfile.setCurrency("CAD");
		userProfile.setDateFormat("YYYY-MM-DD");
		userProfile.setCurrencyFormat("###,###,###.##");
		userProfile.setTimeZone("America/DoesNotExist");
		
		guru.setUserProfile(userProfile);
		
		guru.getTimeZone();
	}
	
	@Test
	public void testGetCalendarFromUserProfile()
	{
		// TODO set user profile
		
		
		Calendar c = LocaleGuru.getInstance().getCalendar();
		
		AssertJUnit.assertNotNull(c);
		
		Calendar expected = Calendar.getInstance(Locale.getDefault());
		AssertJUnit.assertEquals(expected.getTimeZone(), c.getTimeZone());
		
		// comparing within 100 millis, just to be safe.
		AssertJUnit.assertEquals(expected.getTimeInMillis(), c.getTimeInMillis(), 100);
	}
	
	@Test
	public void testGetCalendarFromOfferingProfile()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "test_localeprofile.properties");
		
		// should have offering and default profile, no user profile
		LocaleGuru objectFactory = LocaleGuru.getInstance();
		
		Calendar c = objectFactory.getCalendar();
		
		AssertJUnit.assertNotNull(c);
		
		Calendar expected = Calendar.getInstance(TimeZone.getTimeZone("Asia/Calcutta"), new Locale("hi", "GB"));
		AssertJUnit.assertEquals(expected.getTimeZone(), c.getTimeZone());
		
		// comparing within 100 millis, just to be safe.
		AssertJUnit.assertEquals(expected.getTimeInMillis(), c.getTimeInMillis(), 100);
	}
	
	@Test
	public void testGetCalendarFromDefaultProfile()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru objectFactory = LocaleGuru.getInstance();
		
		Calendar c = objectFactory.getCalendar();
		
		AssertJUnit.assertNotNull(c);
		
		Calendar expected = Calendar.getInstance(new Locale("hi", "GB"));
		AssertJUnit.assertEquals(expected.getTimeZone(), c.getTimeZone());
		
		// comparing within 100 millis, just to be safe.
		AssertJUnit.assertEquals(expected.getTimeInMillis(), c.getTimeInMillis(), 100);
	}
	
	@Test
	public void testGetTimeZone()
	{			
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "test_localeprofile.properties");
		
		// should have offering and default profile, no user profile
		LocaleGuru objectFactory = LocaleGuru.getInstance();
		
		TimeZone tz = objectFactory.getTimeZone();
		
		AssertJUnit.assertNotNull(tz);
		
		AssertJUnit.assertEquals("Asia/Calcutta", tz.getID());
	}
	
	@Test
	public void testGetUserTimeZone()
	{
		LocaleGuru guru = LocaleGuru.getInstance();
		
		LocaleProfile userProfile = new LocaleProfile();
		
		userProfile.setCountry("CA");
		userProfile.setLanguage("en");
		userProfile.setCurrency("CAD");
		userProfile.setDateFormat("YYYY-MM-DD");
		userProfile.setCurrencyFormat("###,###,###.##");
		userProfile.setTimeZone("America/Edmonton");
		
		guru.setUserProfile(userProfile);
		
		// Sanity check
		AssertJUnit.assertEquals(TimeZone.getTimeZone("America/Edmonton"), guru.getTimeZone());
	}
	
	@Test
	public void testGetTimeZoneWithoutConfig()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		AssertJUnit.assertEquals(TimeZone.getDefault(), LocaleGuru.getInstance().getTimeZone());
	}
	
	@Test
	public void testDateFormatDefaultProfile()
	{
		LocaleGuru guru = LocaleGuru.getInstance();
		DateFormat format = guru.getDateFormat();
		DateFormat defaultFormat = DateFormat.getDateInstance();
		Assert.assertEquals(format, defaultFormat);
	}
	
	@Test
	public void testCurrencyFormatFromDefault()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		NumberFormat currencyFormat = LocaleGuru.getInstance().getCurrencyFormat();
		
		BigDecimal num = new BigDecimal("123456789.9945");
		
		AssertJUnit.assertEquals(NumberFormat.getCurrencyInstance().format(num), currencyFormat.format(num));
	}
	
	@Test
	public void testCurrencyFormatFromOffering()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "test_localeprofile.properties");
		
		NumberFormat currencyFormat = LocaleGuru.getInstance().getCurrencyFormat();
		
		BigDecimal num = new BigDecimal("123456789.9945");
		
		AssertJUnit.assertEquals(NumberFormat.getCurrencyInstance(new Locale("hi", "GB")).format(num), currencyFormat.format(num));
		
//		System.out.println("FYI: " + currencyFormat.format(num));
	}
	
	@Test
	public void testCurrencyFormatFromUserProfile()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "test_localeprofile.properties");
		
		LocaleProfile userProfile = new LocaleProfile();
		
		userProfile.setCountry("CA");
		userProfile.setLanguage("en");
		userProfile.setCurrency("CAD");
		userProfile.setDateFormat("YYYY-MM-DD");
		userProfile.setCurrencyFormat("###,###,###.##");
		userProfile.setTimeZone("America/Edmonton");
		
		LocaleGuru.getInstance().setUserProfile(userProfile);
		
		NumberFormat currencyFormat = LocaleGuru.getInstance().getCurrencyFormat();
		
		BigDecimal num = new BigDecimal("123456789.9945");
		
//		System.out.println("FYI: " + currencyFormat.format(num));
		
		AssertJUnit.assertEquals(NumberFormat.getCurrencyInstance(new Locale("en", "CA")).format(num), currencyFormat.format(num));
	}
	
	@Test
	public void testGetStringFromDefaultProfile()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		String bundle = "com.intuit.sb.payroll.common.i18n.testresources";
		
		AssertJUnit.assertEquals("English Hello World", guru.getString(bundle, "hello", "World"));
		
		// Too many parameters
		AssertJUnit.assertEquals("English Hello World", guru.getString(bundle, "hello", "World", "This", "Is", "A", "Test"));
		
		// Not enough parameters
		AssertJUnit.assertEquals("English Hello {0}", guru.getString(bundle, "hello"));
		
//		// play with the resource name
//		AssertJUnit.assertNull(guru.getString(bundle, null));
//		AssertJUnit.assertNull(guru.getString(bundle, ""));
//		AssertJUnit.assertNull(guru.getString(bundle, "nonexistant"));
	}
	
	@Test(expectedExceptions = IllegalArgumentException.class)
	public void testNullBundleName()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		guru.getString(null, "hello");
	}
	
	@Test(expectedExceptions = IllegalArgumentException.class)
	public void testEmptyBundleName()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		guru.getString("", "hello");
	}
	
	@Test(expectedExceptions = InfrastructureException.class)
	public void testMissingBundle()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		guru.getString("nonexistant", "hello");
	}
	
	@Test(expectedExceptions = IllegalArgumentException.class)
	public void testNullStringKey()
	{
		String bundle = "com.intuit.sb.payroll.common.i18n.testresources";
		
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		guru.getString(bundle, null);
	}
	
	@Test(expectedExceptions = IllegalArgumentException.class)
	public void testEmptyStringKey()
	{
		String bundle = "com.intuit.sb.payroll.common.i18n.testresources";
		
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		guru.getString(bundle, "");
	}
	
	@Test(expectedExceptions = InfrastructureException.class)
	public void testMissingStringKey()
	{
		String bundle = "com.intuit.sb.payroll.common.i18n.testresources";
		
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		guru.getString(bundle, "nonexistant");
	}
	
	@Test
	public void getLocaleFromDefault()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		AssertJUnit.assertEquals(Locale.getDefault(), guru.getLocale());
	}
	
	@Test
	public void getLocaleFromOffering()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "test_localeprofile.properties");
		
		AssertJUnit.assertEquals(new Locale("hi", "GB"), LocaleGuru.getInstance().getLocale());
	}
	
	@Test
	public void getLocaleFromUserProfile()
	{
		LocaleGuru guru = LocaleGuru.getInstance();
		
		LocaleProfile userProfile = new LocaleProfile();
		
		userProfile.setCountry("CA");
		userProfile.setLanguage("en");
		userProfile.setCurrency("CAD");
		userProfile.setDateFormat("YYYY-MM-DD");
		userProfile.setCurrencyFormat("###,###,###.##");
		userProfile.setTimeZone("America/Edmonton");
		
		guru.setUserProfile(userProfile);
		
		AssertJUnit.assertEquals(new Locale("en", "CA"), guru.getLocale());
		
		// Now set the user profile to NOT specify a country (e.g. the user just specifies language).
		// Should get back user's language with offering's country.
		userProfile.setLanguage("fr");
		
		AssertJUnit.assertEquals(new Locale("fr", "CA"), guru.getLocale());
	}
	
	@Test
	public void testGetNonLatinString()
	{
		String bundle = "com.intuit.sb.payroll.common.i18n.testresources";

		LocaleGuru guru = LocaleGuru.getInstance();
		
		LocaleProfile userProfile = new LocaleProfile();
		
		userProfile.setCountry("IN");
		userProfile.setLanguage("hi");
		userProfile.setCurrency("CAD");
		userProfile.setDateFormat("YYYY-MM-DD");
		userProfile.setCurrencyFormat("###,###,###.##");
		userProfile.setTimeZone("America/Edmonton");
		
		guru.setUserProfile(userProfile);
		
		AssertJUnit.assertEquals("हैलो World", guru.getString(bundle, "hello", "World"));
	}
	
	@Test
	public void testGetOfferingLocaleFromDefault()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "foobar");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		// Make sure the user profile is set to something so we'll be sure the getOfferingLocale call
		// ignores it.
		LocaleProfile userProfile = new LocaleProfile();
		
		userProfile.setCountry("IN");
		userProfile.setLanguage("hi");
		userProfile.setCurrency("CAD");
		userProfile.setDateFormat("YYYY-MM-DD");
		userProfile.setCurrencyFormat("###,###,###.##");
		userProfile.setTimeZone("America/Edmonton");
		
		guru.setUserProfile(userProfile);
		
		AssertJUnit.assertEquals(Locale.getDefault(), guru.getOfferingLocale());
	}
	
	@Test
	public void testGetOfferingLocaleFromOffering()
	{
		System.setProperty(LocaleGuru.CONFIG_LOCATION_PROPERTY, "test_localeprofile.properties");
		
		LocaleGuru guru = LocaleGuru.getInstance();
		
		// Make sure the user profile is set to something so we'll be sure the getOfferingLocale call
		// ignores it.
		LocaleProfile userProfile = new LocaleProfile();
		
		userProfile.setCountry("IN");
		userProfile.setLanguage("hi");
		userProfile.setCurrency("CAD");
		userProfile.setDateFormat("YYYY-MM-DD");
		userProfile.setCurrencyFormat("###,###,###.##");
		userProfile.setTimeZone("America/Edmonton");
		
		guru.setUserProfile(userProfile);
		
		AssertJUnit.assertEquals(new Locale("hi", "GB"), guru.getOfferingLocale());
	}
}
