package com.intuit.sb.payroll.entity.eao.ejb;

import java.lang.reflect.Constructor;
import java.lang.reflect.ParameterizedType;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.ejb.SessionContext;
import javax.persistence.EntityManager;
import javax.persistence.OptimisticLockException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.apache.log4j.Logger;

import com.intuit.sb.dpt.vpd.multitenancy.TenantContext;
import com.intuit.sb.dpt.vpd.multitenancy.TenantContextManager;
import com.intuit.sb.dpt.vpd.multitenancy.TenantContextNoPermissionException;
import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.eao.GenericEAO;

/**
 * Implements the generic CRUD data access operations using EJB3 APIs.
 * <p>
 * To write a EAO (assuming a 1:1 entity:EAO design), subclass and parameterize this class with the interface and
 * concrete class for the entity.
 * <p>
 * All subclasses must be EJB3 Stateless Session Beans. This:
 * <ol>
 * <li>makes the EAO discoverable by clients via JNDI</li>
 * <li>makes the EAO a managed class so the container provides dependency injection services to it</li>
 * <li>enables container managed transaction support for the EAO</li>
 * <li>joins the EAO into any existing transaction</li>
 * </ol>
 * 
 * @param E
 *            The concrete extension of {@code BaseEntity} that implements the entity managed by this EAO.
 * 
 * @see GenericEAO
 */

public abstract class GenericEjbEAO<E extends Identifiable> implements GenericEAO<E> {

	protected static final Logger logger = PayrollLoggerFactory.makeLogger(GenericEAO.class);

	@Resource
	// Inject the EJB session bean context
	private SessionContext mEJBSessionContext;
	// Inject an entity manager instance for our data source
	@PersistenceContext(unitName = "payrollDB")
	private EntityManager mEntityManager;

	private Class<E> mEntityClass;
	private String mFindAllQueryString;

	/**
	 * Constructs an EAO for the provided BaseEntity specialization.
	 */
	@SuppressWarnings("unchecked")
	protected GenericEjbEAO() {
		this.mEntityClass = (Class<E>) ((ParameterizedType) getClass().getGenericSuperclass()).getActualTypeArguments()[0];
		String fqn = getEntityClass().getName();
		int lastDot = fqn.lastIndexOf('.');
		String basename = fqn.substring(lastDot == 0 ? 0 : lastDot + 1);
		mFindAllQueryString = "Select " + basename + " from " + fqn + " as " + basename;
	}

	/**
	 * Gets the EJB3 {@code EntityManager} being used by this EAO. This is not part of the {@code GenericEAO} interface
	 * because
	 * 
	 * @{code EntityManager} is an EJB3 concept and isn't generic to all persistence providers.
	 * @return The {@code EntityManager} used to access the DB.
	 */
	protected EntityManager getEntityManager() {
		return mEntityManager;
	}

	/**
	 * Gets the EJB3 {@code SessionContext} being used by this EAO. This is not part of the {@code GenericEAO} interface
	 * because
	 * 
	 * @{code SessionContext} is an EJB3 concept and isn't generic to all persistence providers.
	 * @return The {@code SessionContext} used by the EJB container.
	 */
	protected SessionContext getEJBSessionContext() {
		return mEJBSessionContext;
	}

	/**
	 * Gets the spcialized entity class this DAO is managing. This is not part of the {@code GenericEAO} interface
	 * because the class type is only know to the DAO implementation, it's not part of the interface.
	 * 
	 * @return The {@code Class} object of the entity class type. this DAO is managing.
	 */
	protected Class<E> getEntityClass() {
		return mEntityClass;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.component.dao.IGenericEAO#create()
	 */
	public E create() {
		try {
			Constructor<E> c = getEntityClass().getConstructor(new Class[] {});
			return c.newInstance(new Object[] {});
		} catch (Exception e) {
			String mesg = MessageFormat.format("Unable to create entity of type ''{0}'': {1}", new Object[] {
					getEntityClass().getName(), ExceptionUtils.throwableMessage(e) });
			logger.error(mesg);
			getEJBSessionContext().setRollbackOnly();
			throw new InfrastructureException(mesg, e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.component.eao.GenericEAO#findById(java.lang.String)
	 */
	public E findById(Long id) {
		E result = null;
		try {
			result = getEntityManager().find(getEntityClass(), id);
		} catch (NumberFormatException ex) {
			handleExceptionMightBeOLE(ex);
		} catch (IllegalStateException ex) {
			handleExceptionMightBeOLE(ex);
		} catch (PersistenceException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.entity.eao.GenericEAO#findByIdentifiable(com.intuit.sb.payroll.entity.domain.Identifiable)
	 */
	public E findByIdentifiable(Identifiable identifiable) {
		E result = null;
		try {
			result = findById(identifiable.getId());
			if (result == null) {
				String mesg = MessageFormat.format("Entity not found: type={0} id={1}", mEntityClass.getName(),
						identifiable.getId());
				throw new IllegalArgumentException(mesg);
			}
			if (result.getVersion() != identifiable.getVersion()) {
				String mesg = MessageFormat.format(
						"Entity is out of date: type={0} id={1} entity version={2} current version={3}", mEntityClass
								.getName(), identifiable.getId(), identifiable.getVersion(), result.getVersion());
				throw new OptimisticLockException(mesg);
			}
		} catch (IllegalStateException ex) {
			handleExceptionMightBeOLE(ex);
		} catch (PersistenceException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.component.dao.IGenericEAO#findAll()
	 */
	public List<E> findAll() {
		return findByQuery(mFindAllQueryString);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.component.dao.IGenericEAO#findByQuery(java.lang.String)
	 */
	protected List<E> findByQuery(String queryString) {
		return findByQuery(queryString, Collections.<String, Object> emptyMap());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.component.dao.IGenericEAO#findByQuery(java.lang.String, java.util.Map)
	 */
	private List<E> findByQuery(String queryString, Map<String, Object> parameters) {
		Query query = getEntityManager().createQuery(queryString);
		return findByQuery(query, parameters);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.component.dao.IGenericEAO#findByNamedQuery(java.lang.String)
	 */
	public List<E> findByNamedQuery(String queryName) {
		return findByNamedQuery(queryName, Collections.<String, Object> emptyMap());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.component.dao.IGenericEAO#findByNamedQuery(java.lang.String, java.util.Map)
	 */
	public List<E> findByNamedQuery(String queryName, Map<String, Object> parameters) {
		Query query = getEntityManager().createNamedQuery(queryName);
		return findByQuery(query, parameters);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.entity.eao.GenericEAO#findByNamedQuery(java.lang.String, java.util.Map, int, int)
	 */
	public List<E> findByNamedQuery(String queryName, Map<String, Object> parameters, int firstResult, int maxResults) {
		Query query = getEntityManager().createNamedQuery(queryName);
		query.setFirstResult(firstResult);
		query.setMaxResults(maxResults);
		return findByQuery(query, parameters);
	}

	@SuppressWarnings("unchecked")
	private List<E> findByQuery(Query query, Map<String, Object> parameters) {
		List<E> result = null;
		try {
			for (String parameterName : parameters.keySet()) {
				query.setParameter(parameterName, parameters.get(parameterName));
			}
			result = query.getResultList();
		} catch (IllegalStateException ex) {
			handleExceptionMightBeOLE(ex);
		} catch (PersistenceException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.component.dao.IGenericEAO#makeManaged(com.intuit.sb.payroll.component.domainmodel.BaseEntity
	 * )
	 */
	public E makeManaged(E entity) {
		E mergedEntity = null;
		try {
			mergedEntity = getEntityManager().merge(entity);
		} catch (IllegalArgumentException ex) {
			handleExceptionMightBeOLE(ex);
		} catch (PersistenceException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return mergedEntity;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.component.dao.IGenericEAO#makePersistent(com.intuit.sb.payroll.component.domainmodel.BaseEntity
	 * )
	 */
	public E makePersistent(E entity) {
		try {
			getEntityManager().persist(entity);
		} catch (IllegalArgumentException ex) {
			handleExceptionMightBeOLE(ex);
		} catch (PersistenceException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return entity;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.entity.eao.GenericEAO#notPersistent(com.intuit.sb.payroll.entity.domain.Identifiable)
	 */
	public boolean notPersistent(E entity) {
		return !getEntityManager().contains(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.component.dao.IGenericEAO#makeTransient(com.intuit.sb.payroll.component.domainmodel.BaseEntity
	 * )
	 */
	public void makeTransient(E entity) {
		try {
			if ( notPersistent(entity))
			{
			  getEntityManager().remove(getEntityManager().merge(entity));
			}
			else
			{
				getEntityManager().remove(entity);
			}
		} catch (IllegalArgumentException ex) {
			handleExceptionMightBeOLE(ex);
		} catch (PersistenceException ex) {
			handleExceptionMightBeOLE(ex);
		}
	}

	public void flush() {
		getEntityManager().flush();
	}

	/**
	 * Rollback when an exception is handled.
	 * 
	 * @throws OptimisticLockException
	 *             If the exception is an OLE convert it to our application OLE.
	 * @throws PayrollServiceFailureException
	 *             Otherwise throw an PayrollServiceFailureException.
	 */
	protected void handleExceptionMightBeOLE(Exception exception) throws OptimisticLockException,
			InfrastructureException {
		if (exception != null) {
			getEJBSessionContext().setRollbackOnly();
			logger.debug("EAO handled exception: " + exception, exception);
			throw (exception instanceof javax.persistence.OptimisticLockException) ? new OptimisticLockException(
					ExceptionUtils.throwableMessage(exception)) : new InfrastructureException(ExceptionUtils
					.throwableMessageChain(exception), exception);
		}
	}

	/** Set the TenantContext for external calls */
	public void setTenantContext(String companyId) throws ServiceFailureException {
		TenantContext tenantContext = TenantContextManager.createContext(companyId);
		try {
			TenantContextManager.setCurrentContext(tenantContext);
		} catch (TenantContextNoPermissionException e) {
			throw new ServiceFailureException(e);
		}
	}

	public void clearTenantContext() {
		TenantContextManager.clearCurrentContext();
	}
}
