package com.intuit.sb.payroll.entity.model;

import java.math.RoundingMode;
import java.util.Currency;

import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Transient;

/**
 * <p>The <i>Datum</i> data type defines a container designed for carrying <i>payroll value</i>s.
 * <br>A single <i>Datum</i> instance carries a single <i>payroll value</i>. 
 * <br>A <i>payroll value</i> is similar to the complex number in mathematics, which has two
 * components: real and imaginary. In a similar way the <i>payroll value</i> is actually 
 * a set of two numbers, each one of them having a type and a value.
 * <p>Thus, we have <i>payroll value</i> = { {<i>type1, value1</i>}, {<i>type2, value2</i>} }
 * <p>The first number ({<i>type1, value1</i>}) is always defined, the second ({<i>type2, value2</i>}) is optional.
 * <br> The first number must have <i>type1</i> that is one of the following:
 * <li>Percentage
 * <li>Quantity
 * <li>Money
 * <br><br>The second value ({<i>type2, value2</i>}), when present, allows us to interpret 
 * the <i>payroll value</i> as a ratio (or rate). The second value often references a period of time.
 * <br>As an example, the <i>payroll value</i> of <i>$1000 <u>per week</u></i> can be represented as:
 * <ul>
 * <li><b>first number</b>: {type1=quantity, value1={money.amount=1000, money.currency=USD}}   (<i>Money</i> us a composite data type - see below)
 * <li><b>second number</b>: {type2=day, value2=5}
 * <li><b><i>payroll value</i>={{<i>type1</i>=money, <i>value1</i>={money.amount=1000, money.currency=USD}},{<i>type2</i>=day, <i>value2</i>=5}}</b> 
 * </ul>
 * or as
 * <ul>
 * <li><b>first number</b>: {type1=money, value1={money.amount=1000, money.currency=USD}}
 * <li><b>second number</b>: {type2=week, value2=1}
 * <li><b><i>payroll value</i>={{<i>type1</i>=money, <i>value1</i>={money.amount=1000, money.currency=USD}},{<i>type2</i>=week, <i>value2</i>=1}}</b> 
 * </ul>
 * Here are some examples of <i>payroll value</i>s that can be represented by <i>Datum</i>:
 * <ul>
 * <li>a monetary amount
 * <li>annual salary: money amount, {year, 1}
 * <li>monthly salary: money amount, {month, 1}
 * <li>hourly rate: money amount, {hour, 1}
 * <li>daily rate: money amount, {day, 1}, or
 * <li>daily rate: money amount, {hour, 8}
 * <li>vacation accrual per year: quantity, {year, 1} 
 * <li>vacation accrual per pay period: quantity, {pay period, 1}
 * <li>percentage deduction per every other pay period: percentage, {pay period, 2}
 * <li>fixed amount deduction per pay period: money amount, {pay period, 1}
 * <li>one time bonus amount: money amount, {occasional pay, <null>}
 * <li>piecework rate per item: money amount, {piecework, 1}
 * <li>piecework rate per 100 items: money amount, {piecework, 100}
 * </ul>
 * <br>Due to JPA/Hibernate restrictions with regard to data type nesting (not allowed), 
 * composite data types (i.e., consisting of several properties) used in <i>Datum</i> had to be 
 * flattened and are represented by a collection of properties.
 * Logically, however, such set of properties is to be considered indivisible.
 * 
 * <br>In <i>Datum</i> the <i>Money</i> data type is a composite data type:
 * <ul>
 * <li>It consists of a set of the following properties: {currency-code, amount}, 
 * e.g., {USD, 100}, i.e., amount of $100. 
 * </ul>
 * <br>At the property level each <i>Datum</i> object contains:
 * <ul>
 * <li><i>type1</i>
 * <li><i>value1</i> which is a set of properties that hold supported value types
 * <li><i>type2</i>
 * <li><i>value2</i> 
 * </ul>
 * <p><p><p><p><br>
 * <font color="red" size=2><b>IMPORTANT:</b></font> When setting the content of the <i>Datum</i> object, please set the values
 * as they appear in <b>{ {<i>type1, value1</i>}, {<i>type2, value2</i>} }</b> left to right.
 * 
 * <p><p><br><b><font color="red" size=3>The above describes how the <i>Datum</i> was intended to be used. This intent differs
 * from the description below, which reflects some of the existing server code implementation.</font></b> 
 * <p><p><p><p><br>
 * A container for payroll "values" such as wages, tax rates, PTO accruals, commissions, etc.
 * The fields in this type vary in their applicability; only certain fields are ever valid
 * in any particular instance of this class. The basic rules are outlined in the table below: 
 * <table border="1">
 * <tr><th>DatumType1Enum</th><th>UnitOfTime</th><th>Applicable Datum Fields</th><th>Example</th></tr>
 * <tr><td>MONEY</td><td>NOTSET</td><td>CurrencyCode, Amount</td><td>Flat $100 bonus</td></tr>
 * <tr><td>MONEY</td><td>Not NOTSET</td><td>CurrencyCode, Amount, UnitOfTime, UnitOfTimeValue</td><td>25 hours at $15/hr</td></tr>
 * <tr><td>QUANTITY</td><td>N/A</td><td>CurrencyCode, Amount, Quantity</td><td>Sold 100 items at $2/item</td></tr>
 * <tr><td>PERCENT</td><td>N/A</td><td>CurrencyCode, Amount, Percentage</td><td>10% of $20K salary commission</td></tr>
 * <tr><td>TIME</td><td>Not NOTSET</td><td>UnitOfTime, UnitOfTimeValue</td><td>PTO accrual</td><td></td></tr>
 * </table>
 * Some users of the Datum type might modify these soft rules. For example, tax rates are encoded as a Percentage
 * without an Amount.
 * 
 */
@javax.persistence.Embeddable
public class Datum extends DatumBase {

	private static final long serialVersionUID = 1791642433L;

	private static final java.util.Currency USD = Currency.getInstance("USD");
	private static final java.util.Currency CAD = Currency.getInstance("CAD");

	/**
	 * Default constructor
	 */
	public Datum() {
		super.setType1(DatumType1Enum.NOTSET);
		super.setType2(DatumType2Enum.NOTSET);
	}

	/**
	 * Gets <i>type1</i> of <i>value</i> in this object.
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return type1 enum
	 */
    @Enumerated(javax.persistence.EnumType.STRING)
	public DatumType1Enum getType1() {
		return super.getType1();
	}

	/**
	 * Gets value1 as <b>quantity</b> (<b>type1</b> checked for correctness)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return quantity
	 * @throws IllegalStateException
	 */
	@Transient
	public Double getValue1AsQuantity() throws IllegalStateException {
		if (!this.getType1().equals(DatumType1Enum.QUANTITY)) {
			throw new IllegalStateException("Can't get value1 as quantity, it is of type " + getType1().name());							
		}
		return super.getQuantity();
	}
	/**
	 * Sets value1 as <b>quantity</b> (<b>type1</b> set to <i>quantity type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param quantity
	 */
	@Transient
	public void setValue1AsQuantity(Double quantity) {
		super.setType1(DatumType1Enum.QUANTITY);
		super.setQuantity(quantity);
	}
	
	/**
	 * Gets value1 as <b>percentage</b> (<b>type1</b> checked for correctness)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return percentage
	 * @throws IllegalStateException
	 */
	@Transient
	public Double getValue1AsPercentage() throws IllegalStateException {
		if (!this.getType1().equals(DatumType1Enum.PERCENT)) {
			throw new IllegalStateException("Can't get value1 as percent, it is of type " + getType1().name());							
		}
		
		return super.getPercentage();
	}
	/**
	 * Sets value1 as <b>percentage</b> (<b>type1</b> set to <i>percentage type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return percentage
	 */
	@Transient
	public void setValue1AsPercentage(Double percentage) {
		super.setType1(DatumType1Enum.PERCENT);
		super.setPercentage(percentage);
	}

	/**
	 * Gets currency code from value1 as <b>money </b> (<b>type1</b> indicates <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return currency code
	 * @throws IllegalStateException
	 */
	@Transient
	public java.util.Currency getValue1AsMoneyCurrencyCode() throws IllegalStateException {
		if (!this.getType1().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Can't get value1 as money, it is of type " + getType1().name());							
		}
		
		return super.getMoney().getCurrency();
	}
	/**
	 * Gets amount from value1 as <b>money</b>  (<b>type1</b> indicates <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return amount
	 * @throws IllegalStateException
	 */
	@Transient
	public java.math.BigDecimal getValue1AsUSDMoneyAmount() throws IllegalStateException {
		if (!this.getType1().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Can't get value1 as money, it is of type " + getType1().name());							
		}
		if (!this.getMoney().getCurrency().equals(USD)) {
			throw new IllegalStateException("Can't get value1 as USD, it has currency code " + super.getMoney().getCurrencyCode());							
		}
		return super.getMoney().getAmount();
	}

	/**
	 * Sets amount in value1 as <b>money</b> (<b>type1</b> to indicate <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param amount
	 */
	@Transient
	public void setValue1AsUSDMoneyAmount(java.math.BigDecimal amount) {
	    super.setType1(DatumType1Enum.MONEY);
	    Money money = super.getMoney();
		money.setCurrencyCode(USD);
		//FIXME Should we do rounding at all here?  QBO/IOP allows up to 5 digits for rates
        // set to 2 decimal places
		money.setAmount(amount.setScale(2, RoundingMode.HALF_UP));
		//super.setMoney(money);
	}
	
	/**
	 * Gets amount from value1 as <b>money</b>  (<b>type1</b> indicates <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return amount
	 * @throws IllegalStateException
	 */
	@Transient
	public java.math.BigDecimal getValue1AsCADMoneyAmount() throws IllegalStateException {
		if (!this.getType1().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Can't get value1 as money, it is of type " + getType1().name());							
		}
		if (!this.getMoney().getCurrency().equals(CAD)) {
			throw new IllegalStateException("Can't get value1 as CAD, it has currency code " + super.getMoney().getCurrencyCode());							
		}
		return super.getMoney().getAmount();
	}

	/**
	 * Sets amount in value1 as <b>money</b> (<b>type1</b> to indicate <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param amount
	 */
	@Transient
	public void setValue1AsCADMoneyAmount(java.math.BigDecimal amount) {
	    super.setType1(DatumType1Enum.MONEY);
	    Money money = new Money();
		money.setCurrencyCode(CAD);
		money.setAmount(amount.setScale(2));
		super.setMoney(money);
	}
	
	/**
	 * Gets amount from value1 as <b>money</b>  (<b>type1</b> indicates <i>money type</i>)
	 * For detailed description see {@linkplain Datum}. Note: this method does not consider 
	 * currency - it will return the amount value of <b>value1</b> regardless of the currency.
	 * 
	 * @return amount
	 * @throws IllegalStateException
	 */
	@Transient
	public java.math.BigDecimal getValue1AsMoneyAmount() throws IllegalStateException {
		if (!this.getType1().equals(DatumType1Enum.MONEY)) {
			throw new IllegalStateException("Can't get value1 as money, it is of type " + getType1().name());							
		}
		return super.getMoney().getAmount();
	}
	
	/**
	 * Gets the <b>type2</b>
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return type2
	 * @throws IllegalStateException
	 */
	@javax.persistence.Enumerated(EnumType.STRING)
	public DatumType2Enum getType2() throws IllegalStateException {
//		if (super.getType2().equals(DatumType2Enum.NOTSET)) {
//            throw new IllegalStateException("Can't get type2, it is not set");                          
//		}

		return super.getType2();
	}
	/**
	 * Gets <b>value2</b>
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return value2
	 * @throws IllegalStateException
	 */
	public Double getValue2() throws IllegalStateException {
//		if (super.getType1().equals(DatumType1Enum.NOTSET)) {
//			throw new IllegalStateException("Can't get value2, type1 is not set");							
//		}
//		if (super.getType2().equals(DatumType2Enum.NOTSET) || super.getType2().equals(DatumType2Enum.NOTAPPLICABLE)) {
//			throw new IllegalStateException("Can't get value2, type2 is not set or not applicable");							
//		}
		return super.getValue2();
	}

	/**
	 * Sets <b>type2</b>
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param type2
	 * @throws IllegalStateException
	 */
	@Transient
	public void setType2(DatumType2Enum type2) throws IllegalStateException {
//		if (super.getType1().equals(DatumType1Enum.NOTSET)) {
//			throw new IllegalStateException("Can't set type2 before setting type1");							
//		}
		super.setType2(type2);
	}
	/**
	 * Sets <b>value2</b> associated with <i>type2</i>
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param value2
	 * @throws IllegalStateException
	 */
	@Transient
	public void setValue2(Double value2) throws IllegalStateException {
//		if (getType1().equals(DatumType1Enum.NOTSET)) {
//			throw new IllegalStateException("Can't set value2 before setting type1");							
//		}
//		if (getType2().equals(DatumType2Enum.NOTSET) || getType2().equals(DatumType2Enum.NOTAPPLICABLE)) {
//			throw new IllegalStateException("Can't set value2 before setting type2");							
//		}
		super.setValue2(value2);
	}
	/**
	 * Sets the optional {<b>type2, value2</b>} payroll value number.
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param {type2, value2}
	 * @throws IllegalStateException
	 */
	@Transient
	public void setType2Value2(DatumType2Enum type2, Double value2) throws IllegalStateException {
		if (getType1().equals(DatumType1Enum.NOTSET)) {
			throw new IllegalStateException("Can't set type2 and value2 before setting type1");							
		}
		super.setType2(type2);
		super.setValue2(value2);
	}

	public Datum clone() {
		//return super.contentCopy();
		return this.contentCopy();
	}
	
	/*
	 * DEPRECATED METHODS
	 * ==================
	 */
	/**
	 * Gets <i>type1</i> of <i>value</i> in this object.
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return type1 enum
	 */
	@Deprecated
	@Transient
	public DatumType1Enum getType() {
		return super.getType1();
	}
	
	/**
	 * Sets <i>type1</i> for <i>value</i> in this object.
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param type1 enum
	 */
	@Deprecated
	public void setType(DatumType1Enum type1) {
		super.setType1(type1);
	}

	/**
	 * Gets the content of the <b>quantity</b> property.
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return quantity
	 */
	@Deprecated
	public Double getQuantity() {
		return super.getQuantity();
	}
	/**
	 * Sets the content of the <b>quantity</b> property.
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param quantity
	 */
	@Deprecated
	public void setQuantity(Double quantity) {
		super.setQuantity(quantity);
	}
	
	/**
	 * Gets the <b>percentage</b> value (<b>type1</b> indicates <i>percentage type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return percentage
	 */
	@Deprecated
	public Double getPercentage() {
		return super.getPercentage();
	}
	/**
	 * Sets the <b>percentage</b> value (<b>type1</b> to indicate <i>percentage type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return percentage
	 */
	@Deprecated
	public void setPercentage(Double percentage) {
		super.setPercentage(percentage);
	}

	/**
	 * Gets the <b>money currency code</b> value (<b>type1</b> indicates <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return currency code
	 */
	@Deprecated
	@Transient
	public java.util.Currency getDatumCurrencyCode() {
		return Currency.getInstance(super.getMoney().getCurrencyCode().value());
	}
	/**
	 * Gets the <b>money amount</b> value (<b>type1</b> indicates <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return amount
	 */
	@Deprecated
	@Transient
	public java.math.BigDecimal getDatumAmount() {
		return super.getMoney().getAmount();
	}

	/**
	 * Sets the <b>money currency code</b> value (<b>type1</b> to indicate <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param currency code
	 */
	@Deprecated
	public void setDatumCurrencyCode(java.util.Currency currencyCode) {
		super.getMoney().setCurrencyCode(currencyCode);
	}
	/**
	 * Sets the <b>money amount</b> value (<b>type1</b> to indicate <i>money type</i>)
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param amount
	 */
	@Deprecated
	public void setDatumAmount(java.math.BigDecimal amount) {
		super.getMoney().setAmount(amount);
	}
	
	/**
	 * Gets <b>type2</b>
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return type2
	 */
	@Deprecated
	@Transient
	public DatumType2Enum getUnitOfTime() {
		return super.getType2();
	}
	/**
	 * Gets the <b>value</b> associated with the <i>unit of time</i>
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @return value2
	 */
	@Deprecated
	@Transient
	public Double getUnitOfTimeValue() {
		return super.getValue2();
	}

	/**
	 * Sets the <b>unit of time type</b> value
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param type2
	 */
	@Deprecated
	public void setUnitOfTime(DatumType2Enum type2) {
		super.setType2(type2);
	}
	/**
	 * Sets the <b>value</b> associated with the <i>unit of time</i>
	 * For detailed description see {@linkplain Datum}.
	 * 
	 * @param value2
	 */
	@Deprecated
	public void setUnitOfTimeValue(Double value2) {
		super.setValue2(value2);
	}

	@Override
	@Deprecated
	public Money getMoney() {
		return super.getMoney();
	}

	@Override
	@Deprecated
	public void setMoney(Money money) {
		super.setMoney(money);
	}	
	
}