package com.intuit.sb.payroll.rules;


import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import com.intuit.sb.core.rules.remote.RuleService;
import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.rules.impl.drools.DroolsHelper;

/**
 * This class is responsible for initializing all the rule sets that exist in Payroll Component
 * Thread safety of this class depends on thread safety of RulesEngine implementations
 * We need to make this a singleton because we don't want to new this everytime from the FunctionalArea as registering rule sets is expensive.
 * TODO: I think we need to move the logic of registering Rule Sets to RulesEngine initialization ie (the dostart code). That's not possible
 * because we have a payroll-rules-cfg.xml file per package. We need to make sure that registration of the rule sets occurs once per package
 * TODO: Rename RulesBridge to PayrollComponentRules
 */
public class RulesBridge
{
    /**
     * TODO: can we have more than one drl file per package?
     * The package URI of the drl file.
     *
     */
    public static final String EMPLOYEE_TAX_SETUP_RULESET_NAME = "EmployeeTaxSetup";

    public static final String WORKER_SETUP_WORKFLOW_RULESET_NAME = "WorkerSetupWorkflow";

    public static final String EMPLOYEE_TAX_WITHHOLDING_RULESET_NAME = "EmployeeTaxWithholding";


    public static final String EMPLOYER_TAX_SETUP_RULESET_NAME = "EmployerTaxSetup";

    private Map<String, LinkedHashMap<String,String>> mRulesetsUriMap = null;

    /**
     * The configuration file to bootstrap all the rules for this package
     */
    public static final String PAYROLL_COMPONENT_RULES_CFG = "/com/intuit/sb/payroll/rules/payroll-rules-cfg.xml";

    public static final String RULES_ENGINE_COMPONENT_NAME = "RulesEngine";

    private static Logger sLogger = PayrollLoggerFactory.makeLogger(RulesBridge.class);

    private static PayrollResourceManager sResMan = PayrollResourceManager.getInstance();

    private static RulesBridge mInstance = null;
    
    private static RuleService ruleService = ServiceLocator.getInstance().getService("RuleService/remote"); 

    public static synchronized RulesBridge getInstance()
    {
        if(mInstance == null)
        {
            mInstance = new RulesBridge();
        }
        return mInstance;
    }

    /**
     * Creates a new instance of RulesBridge
     *@throws RulesException when rules engine is not initialized successfully
     */
    private RulesBridge() throws RulesException
    {
        try
        {
            initalizeRulesSet();
        }
        catch(Exception e)
        {
            throw new RulesException(sResMan.getString(ResourceKeyConstants.RULES_INIT_EXCEPTION), e);
        }
    }

    public Map<String, LinkedHashMap<String,String>> getRulesetMap()
    {
    	return mRulesetsUriMap;
    }

    /**
     * Performs Intialization of the rules sets for Payroll Component
     * @return
     * @throws IOException
     * @throws SAXException
     * @throws RulesException
     * @throws ParserConfigurationException
     */
    private void initalizeRulesSet() throws IOException, SAXException, RulesException, ParserConfigurationException
    {
        sLogger.trace("Entered initalizeRulesSet");
        URL metaUrl = this.getClass().getResource(PAYROLL_COMPONENT_RULES_CFG);
    	if(metaUrl == null)
    	{
            throw new FileNotFoundException(sResMan.getString(ResourceKeyConstants.CPRESOURCE_NOT_FOUND_EXCEPTION) + PAYROLL_COMPONENT_RULES_CFG);
    	}
    	DroolsHelper helper = new DroolsHelper();

        mRulesetsUriMap = helper.initializeRulesMap(metaUrl);

        sLogger.trace("Exited initalizeRulesSet");
    }
    
    private RuleService getRuleServiceBean(){
    	
    	if (ServiceLocator.getInstance().isEmbedded()){
    		// Get the local Rules Service bean
    		return ruleService;
    		
    	}else {
    		// Get the Remote Rules Service bean
    		return ruleService;
    	}

    	
    }

    public RulesResult executeRuleSet(String rulesetName, RulesRequest rulesInput) {

    	RulesResult result = new RulesResult();
    	try
        {
    		RuleService ruleService = getRuleServiceBean();
            LinkedHashMap<String,String> rulesetMap = mRulesetsUriMap.get(rulesetName);
            Set<String> uris = rulesetMap.keySet(); //get all the uris in the ruleset
            Iterator<String> iter = uris.iterator();
            while(iter.hasNext())
            {
            	String uri = iter.next();
            	result.initialize(ruleService.executeRules(uri, rulesInput.getInputs(), null));
            	rulesInput.initialize(result);
            }
        }
        catch(Exception e)
        {
            throw new InfrastructureException(sResMan.getString(ResourceKeyConstants.RULES_EXEC_EXCEPTION), e);
        }
        return result;
    }
    
    

    public RulesResult runRules(String uri, RulesRequest request) throws RulesException
	{
		try
		{
			RuleService ruleService = getRuleServiceBean();
			RulesResult result = new RulesResult();
			result.initialize(ruleService.executeRules(uri, request.getInputs(), null));
			return result;
		} catch (Exception e)
		{
			throw new RulesException(e);
		}
	}

}