package com.intuit.sb.payroll.capability.benefits;


import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.HealthBenefit;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;

@Local
public interface BenefitsComponent {
	
	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "BenefitsComponent/local";

	/**
	 * Creates a new non-persistent benefit detail. All the "immutable" fields in the Dtl are set:
	 * <ul>
	 * <li>The association to its respective Benefits</li>
	 * <li>any empty Datum</li>
	 * <li>a benefit type</li>
	 * <li>it's appropriate law id</li>
	 * </ul>
	 * Additionally, the name and description are set from the values in the corresponding Benefits.
	 * @param benefitTypeEnum
	 * @return the template benefit type ready for the "mutable" fields to be set.
	 * @throws ServiceFailureException 
	 */
	public HealthBenefitDtl createHealthBenefitDtl(HealthBenefitTypeEnum benefitTypeEnum) throws ValidationException, ServiceFailureException;

	/**
	 * Finds a benefit detail.
	 * @param identifiable the identifiable of the existing benefit detail.
	 * @return the benefit detail.
	 * @throws ServiceFailureException 
	 * @throws ValidationException 
	 */
	public HealthBenefitDtl findHealthBenefitDtl(Identifiable identifiable) throws ServiceFailureException, ValidationException;
	
	
	/**
	 * Deletes a benefit detail associated with the worker.
	 * @param worker Valid worker object.
	 * @param benefitDtl the HealthBenefitDtl that needs to be deleted. 
	 * @return worker the updated worker object
	 */
	public Worker deleteHealthBenefitDtl(Worker worker, HealthBenefitDtl benefitDtl) throws ValidationException;

	
	/**
	 * Adds benefit details to the worker.
	 * @param worker worker object.
	 * @param Set<HealthBenefitDtl> the HealthBenefitDtl set to add. 
	 * @return worker the updated worker object
	 * @throws ServiceFailureException 
	 */
	public Worker addHealthBenefitDtls(Worker worker, List<HealthBenefitDtl> dtlSet) throws ValidationException, ServiceFailureException;
    
	/**
	 * Updates benefit details associated with the worker.
	 * @param workerId worker object id.
	 * @param Set<HealthBenefitDtl> the HealthBenefitDtl set to be updated. 
	 * @return worker the updated worker object
	 * @throws ServiceFailureException 
	 */
    public Worker updateHealthBenefitDtls(Long workerId, List<HealthBenefitDtl> dtlSet) throws ValidationException, ServiceFailureException;
    
	/**
	 * Deletes benefit details associated with the worker.
	 * @param workerId worker object id.
	 * @param Set<HealthBenefitDtl> the HealthBenefitDtl set to be removed. 
	 * @return worker the updated worker object
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 */
    public Worker deleteHealthBenefitDtls(Long workerId, List<HealthBenefitDtl> dtlSet) throws ServiceFailureException, ValidationException;

    /**
     * Finds a HealthBenefit by Type.
     * 
     * @param healthBenefitType
     * @return 
     * @throws ServiceFailureException 
     */
	public HealthBenefit findHealthBenefit(HealthBenefitTypeEnum healthBenefitType) throws ServiceFailureException;

	/**
	 * Finds all HealthBenefits.
	 * 
	 * @return a List of Benefits.
	 */
	public List<HealthBenefit> findAllBenefits();

}