package com.intuit.sb.payroll.capability.compensation;

import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;

@Local
public interface CompensationComponent {
	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "CompensationComponent/local";

	/**
	 * Creates a new compensation detail. All the "immutable" fields in the Dtl are set:
	 * <ul>
	 * <li>The association to its respective Compensation</li>
	 * <li>any empty Datum</li>
	 * <li>a compensation type</li>
	 * <li>it's appropriate law id</li>
	 * </ul>
	 * Additionally, the name and description are set from the values in the corresponding Compensation.
	 * Note that the entity returned is not in session.  If you want to save it, use
	 * your friendly EAO's makePersistent call.
	 * 
	 * @param compensationTypeEnum
	 * @return the template compensation type ready for the "mutable" fields to be set.
	 * @throws ValidationException 
	 */
	public CompensationDtl createCompensationDtl(CompensationTypeEnum compensationTypeEnum) throws ValidationException;

	/**
	 * Finds a compensation detail.
	 * @param identifiable the identifiable of the existing compensation detail.
	 * @return the compensation detail.
	 * @throws ServiceFailureException 
	 */
	public CompensationDtl findCompensationDtl(Identifiable identifiable) throws ServiceFailureException;
	
	/**
	 * Find the CompensationDtls that link to the given compensationDtl
	 * @param compensationDtl
	 * @return the List of CompensationDtls linked to the given compensationDtl
	 */
	public List<CompensationDtl> findLinkedCompensationDtls(CompensationDtl compensationDtl);
	
	/**
	 * Removes a compensation detail associated with the worker.
	 * @param worker Valid worker object.
	 * @param compensationDtl the CompensationDtl that needs deleting 
	 * @throws ValidationException if the CompensationDtl has PTD values (has been in a paycheck). 
	 * @throws ServiceFailureException 
	 * @throws WorkflowException 
	 */
	public Worker removeCompensationDtlFromWorker(Worker worker, CompensationDtl compensationDtl) throws ValidationException, WorkflowException, ServiceFailureException;
	
	/**
	 * Adds a CompensationDtl to a Worker.
	 * 
	 * @param compensationDtl the Compensation to add.
	 * @param worker the Worker to add it to.
	 * @return the latest copy of the Worker.
	 * @throws ValidationException if there are negative rates or quantities.
	 * @throws ServiceFailureException 
	 * @throws WorkflowException 
	 */
	public Worker addCompensationToWorker(CompensationDtl compensationDtl, Worker worker) throws ValidationException, WorkflowException, ServiceFailureException;

	public Worker addCompensationsToWorker(List<CompensationDtl> compensationsToAdd, Worker worker) throws ValidationException, WorkflowException, ServiceFailureException;

	public Worker updateCompensationOnWorker(CompensationDtl compensationDtl, Worker worker) throws ValidationException, WorkflowException, ServiceFailureException;

	public Worker removeCompensationsFromWorker(List<CompensationDtl> compensationsToDelete, Worker worker) throws ValidationException, WorkflowException, ServiceFailureException;


}