package com.intuit.sb.payroll.capability.compensation.impl;

import java.text.MessageFormat;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.compensation.CompensationComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.Compensation;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.wages.CompensationDtlEAO;
import com.intuit.sb.payroll.entity.wages.CompensationEAO;

/**
 * Implementation of the CompensationComponent Interface.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
@Stateless(name = "CompensationComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class CompensationComponentBean implements CompensationComponent
{
	@EJB
	private CompensationEAO compensationEAO;
	
	@EJB
	private CompensationDtlEAO compensationDtlEAO;
	
	@EJB
	private EntityService entityService;
	
	@EJB
	private WorkerComponent workerComponent;

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.compensation.CompensationComponent#createCompensationDtl(com.intuit.sb.payroll.entity.model.CompensationTypeEnum)
	 */
	public CompensationDtl createCompensationDtl(CompensationTypeEnum compensationTypeEnum) throws ValidationException {
		Compensation compensation = compensationEAO.findCompensation(compensationTypeEnum);
		if (compensation == null) {
			String mesg = MessageFormat.format("Compensation could not be found for type ''{0}''", compensationTypeEnum.name());
			throw new ValidationException(mesg);
		}
		
		CompensationDtl compensationDtl = CompensationFactory.createCompensationDtlFromCompensation(compensation);
		return compensationDtl;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.compensation.CompensationComponent#findCompensationDtl(com.intuit.sb.payroll.entity.domain.Identifiable)
	 */
	public CompensationDtl findCompensationDtl(Identifiable identifiable) throws ServiceFailureException {
		try {
			CompensationDtl compensationDtl = compensationDtlEAO.findByIdentifiable(identifiable);
			return compensationDtl;
		}
		catch (IllegalArgumentException iae) {
			String mesg = MessageFormat.format("Unable to find compensation: {0}", ExceptionUtils.throwableMessage(iae));
			throw new ServiceFailureException(mesg);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.compensation.CompensationComponent#findLinkedCompensationDtls(com.intuit.sb.payroll.entity.model.CompensationDtl)
	 */
	public List<CompensationDtl> findLinkedCompensationDtls(CompensationDtl compensationDtl) {
		return compensationDtlEAO.findLinkedCompensationDtls(compensationDtl);	
	}

	public Worker removeCompensationsFromWorker(List<CompensationDtl> compensationsToDelete, Worker worker) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(compensationsToDelete, "compensationsToDelete");
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}

		for (CompensationDtl compensationDtl : compensationsToDelete) {
			if(entityService.notPersistent(compensationDtl))
			{
				compensationDtl = entityService.makeManaged(compensationDtl);
			}
			compensationDtl.validateNoPeriodToDateValues();
			if (!compensationDtl.getPtoBaseRateType().equals(PtoBaseRateTypeEnum.NOTSET)) { 
				throw new ValidationException("Set the ptoBaseRate to NOTSET before removal.");
			}
			if(compensationDtl.isDeleted())
			{
				worker = workerComponent.updateWorker(worker);
			}
			else
			{
				worker.removeCompensationDtl(compensationDtl);
				entityService.makeTransient(compensationDtl);
			}
		}
		worker = workerComponent.updateWorker(worker);
		entityService.flush();
		return worker;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.compensation.CompensationComponent#removeCompensationDtlFromWorker(com.intuit.sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.CompensationDtl)
	 */
	public Worker removeCompensationDtlFromWorker(Worker worker, CompensationDtl compensationDtl) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(compensationDtl, "compensationDtl");	
		if(entityService.notPersistent(compensationDtl))
		{
			compensationDtl = entityService.makeManaged(compensationDtl);
		}
		compensationDtl.validateNoPeriodToDateValues();
		if (!compensationDtl.getPtoBaseRateType().equals(PtoBaseRateTypeEnum.NOTSET)) { 
			throw new ValidationException("Set the ptoBaseRate to NOTSET before removal.");
		}
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}
		
		if(!compensationDtl.isDeleted())
		{
			worker.removeCompensationDtl(compensationDtl);
			entityService.makeTransient(compensationDtl);
		}
		worker = workerComponent.updateWorker(worker);
		entityService.flush();
		return worker;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.compensation.CompensationComponent#addCompensationToWorker(com.intuit.sb.payroll.entity.model.CompensationDtl, com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Worker addCompensationToWorker(CompensationDtl compensationDtl, Worker worker) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		CompensationDtlValidator.getInstance().validateEntity(compensationDtl);
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}
		compensationDtl = entityService.makePersistent(compensationDtl);
		//Adding to the worker makes the compensation persistent
		worker.addCompensationDtl(compensationDtl);
		//Variation point for Cash Tips and perhaps others
		LinkedPayrollItemHandler.handleNewCompensationDtl(entityService, compensationDtl, worker);
		worker = workerComponent.updateWorker(worker);
		return worker;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.compensation.CompensationComponent#addCompensationToWorker(com.intuit.sb.payroll.entity.model.CompensationDtl, com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Worker addCompensationsToWorker(List<CompensationDtl> compensationsToAdd, Worker worker) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		CompensationDtlValidator.getInstance().validateEntityCollection(compensationsToAdd);
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}
		
		compensationsToAdd = LinkedPayrollItemHandler.sortCompensationsToAdd(compensationsToAdd);
		for (CompensationDtl compensationDtl : compensationsToAdd) {
			//Adding to the worker makes the compensation persistent
			compensationDtl = entityService.makePersistent(compensationDtl);
			worker.addCompensationDtl(compensationDtl);
			
			//Variation point for Cash Tips and perhaps others
			LinkedPayrollItemHandler.handleNewCompensationDtl(entityService, compensationDtl, worker);			
		}

		worker = workerComponent.updateWorker(worker);
		return worker;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.compensation.CompensationComponent#updateCompensationOnWorker(com.intuit.sb.payroll.entity.model.CompensationDtl, com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Worker updateCompensationOnWorker(CompensationDtl compensationDtl, Worker worker) throws ValidationException, WorkflowException, ServiceFailureException {
		CompensationDtlValidator.getInstance().validateEntity(compensationDtl);
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}
		Set<CompensationDtl> workerCompensations = worker.getCompensationDtls();
		if (workerCompensations.contains(compensationDtl)) {
			compensationDtl = compensationDtlEAO.makeManaged(compensationDtl);			
		} else {
			throw new ValidationException("Can't update a CompensationDtl that is not on the Worker:" + compensationDtl.toString());
		}
		worker = workerComponent.updateWorker(worker);
		return worker;
	}
	
}
