package com.intuit.sb.payroll.capability.employer;

import java.io.IOException;
import java.util.Calendar;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.HistPaycheck;
import com.intuit.sb.payroll.entity.model.HistPaycheckLineItem;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

@Local
public interface EmployerComponent {
	
	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "EmployerComponent/local";

	/**
	 * Returns the full list of WorkLocations for the current Employer.
	 * @return a List of WorkLocations.
	 */
    public List<WorkLocation> findAllWorkLocations();

    /**
     * Returns a specific WorkLocation as specified by the UniqueId.
     * @param uniqueId
     * @return the WorkLocation
     * @throws ServiceFailureException 
     */
    public WorkLocation findWorkLocation(Long uniqueId) throws ServiceFailureException;
	
    /**
     * Adds WorkLocations to the current Employer.
     * 
     * @param workLocations the WorkLocations to add.
     * @return the updated List of all the WorkLocations for the current Employer.
     * @throws ValidationException if there are constraints violated.
     * @throws ServiceFailureException 
     */
    public List<WorkLocation> addWorkLocations(List<WorkLocation> workLocations) throws ValidationException, ServiceFailureException;
    
    /**
     * Adds a Worklocation to the employer.  The new copy of the Employer comes back.
     * 
     * @param workLocation
     * @return
     * @throws ValidationException
     * @throws ServiceFailureException
     */
    public Employer addWorkLocation(WorkLocation workLocation) throws ValidationException, ServiceFailureException;
    
    /**
     * Updates WorkLocations to the current Employer.
     * 
     * @param workLocations the WorkLocations to update.
     * @return the updated List of all the WorkLocations for the current Employer.
     * @throws ValidationException if there are constraints violated, or the WorkLocations were never
     * in the DB in the first place.
     * @throws ServiceFailureException 
     */
	public List<WorkLocation> updateWorkLocations(List<WorkLocation> workLocations) throws ValidationException, ServiceFailureException;
	
	/**
	 * Deletes WorkLocations from the current Employer.
	 * 
	 * @param workLocations the WorkLocations to delete.
	 * @throws ValidationException if there are constraints violated, such as Workers still assigned to 
	 * the WorkLocation.
	 * @throws ServiceFailureException 
	 */
	public List<WorkLocation> deleteWorkLocations(List<WorkLocation> workLocations) throws ValidationException, ServiceFailureException;
    
	/**
	 * Assigns the WorkLocations to the Workers.
	 * 
	 * WorkerId/WorkLocationId
	 * @param workerLocations
	 * @return a List of the updated Workers
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 */
	public List<Worker> assignWorkLocationsToWorkers(WorkerLocation[] workerLocations)
		throws WorkflowException, ValidationException, ServiceFailureException;

	/**
     * Provides a list of Jurisdiction for a given WorkLocation. The WorkLocation Address to Jurisdiction Mapping is currently
     * implemented in Jurisdiction resolver. At a later point in time this will be modified to use some external service.
     * 
     * @param workLocation
     * @return List of Jurisdiction that applies to the work location
	 * @throws ValidationException 
     * @throws PayrollServiceFailureException if something unexpected occurs.
     */
    public List<Jurisdiction> determineAllJurisdictionsWorkLocation(WorkLocation workLocation) throws ValidationException;
    
    /**
     * Create a new Employer in the system from the Employer provided
     * @throws ValidationException if an Employer already exists for the user context
     * @param employer
     * @return Employer
     * @throws ServiceFailureException 
     */
	public Employer createEmployer(Employer employer) throws ValidationException, ServiceFailureException;
   
    /**
     * Save the Employer provided
     * @param employer
     * @return Employer
     * @throws  
     * @throws ServiceFailureException 
     */
	public Employer saveEmployer(Employer employer) throws ValidationException, ServiceFailureException;

    /**
     * Retrieve the current employer
     * TODO [Impl] We should be able to find > 1 companies in the future,
     * 		in the world of Accountant products.
     * @param 
     * @return Employer, or null if the employer does not exist.
     */
	public Employer getEmployer();

	/**
     * Provides a list of Jurisdiction for an employer for ALL WorkLocations 
     * associated with that employer. 
     * The WorkLocation Address to Jurisdiction Mapping is currently
     * implemented in Jurisdiction resolver. 
     * This method also considers worklocation associated with the workers for current employer
     * At a later point in time this will be modified to use some external service.
     * 
     * @return List of Jurisdiction that applies to the current employer
	 * @throws ServiceFailureException 
     */
    public List<Jurisdiction> determineAllJurisdictions() throws ServiceFailureException;
    
	
	/**
	 * Returns the full list of ContactPersons for the current Employer.
	 * @return a Set of ContactPersons.
	 */
    public Set<EmployerContactPerson> getEmployerContactPersons();

    /**
     * Returns a specific EmployerContactPerson as specified by the UniqueId.
     * @param uniqueId
     * @return the EmployerContactPerson
     * @throws ServiceFailureException 
     */
    public EmployerContactPerson findEmployerContactPerson(Long uniqueId) throws ServiceFailureException;
	
    /**
     * Adds ContactPersons to the current Employer.
     * 
     * @param employerContactPersons the ContactPersons to add.
     * @return the updated List of all the ContactPersons for the current Employer.
     * @throws ValidationException if there are constraints violated.
     * @throws ServiceFailureException 
     */
    public Set<EmployerContactPerson> addEmployerContactPersons(List<EmployerContactPerson> employerContactPersons) throws ValidationException, ServiceFailureException;
    
    /**
     * Updates ContactPersons to the current Employer.
     * 
     * @param employerContactPersons the ContactPersons to update.
     * @return the updated List of all the ContactPersons for the current Employer.
     * @throws ValidationException if there are constraints violated, or the ContactPersons were never
     * in the DB in the first place.
     * @throws ServiceFailureException 
     */
	public Set<EmployerContactPerson> updateEmployerContactPersons(List<EmployerContactPerson> employerContactPersons) throws ValidationException, ServiceFailureException;
	
	/**
	 * Deletes ContactPersons from the current Employer.
	 * 
	 * @param employerContactPersons the ContactPersons to delete.
	 * @throws ValidationException if there are constraints violated, 
	 * @throws ServiceFailureException 
	 */
	public void deleteEmployerContactPersons(List<EmployerContactPerson> employerContactPersons) throws ValidationException, ServiceFailureException;
    
	/**
	 * Deletes all ContactPersons from the current Employer.
	 * 
	 * @param contactPersons the ContactPersons to delete.
	 * @throws ValidationException if there are constraints violated
	 * @throws ServiceFailureException 
	 */
	public void deleteAllContactPersons() throws ValidationException, ServiceFailureException;

    /**
     * Add the given Bank Accounts to this company.
     * @param bankAccounts
     * @return an array containing all the added BankAccount objects
     */
    public BankAccount[] addBankAccounts(BankAccount[] bankAccounts) throws ServiceFailureException, ValidationException;
    
    /**
     * Get all Bank Accounts to this company.
     * @return an array containing all the BankAccount objects
     */
    public BankAccount[] getBankAccounts() throws ServiceFailureException, ValidationException;

    /**
     * Update the given Bank Accounts.  Each Bank Account must have a uniqueId.
     * @param bankAccounts
     * @return an array containing all the saved BankAccount objects
     */
    public BankAccount[] updateBankAccounts(BankAccount[] bankAccounts) throws ServiceFailureException, ValidationException;

    /**
     * This component is interested in changes to the PayrollSchedule, as it may affect state and tasks related to the Employer.
     * 
     * @param workflowEventType
     * @throws ServiceFailureException
     */
	public void handlePayrollScheduleEvent(WorkflowEventType workflowEventType) throws ServiceFailureException;
	
 	/**
	 * Get all the valid prefixes for Federal EIN 
	 *  @return an array of strings containg the valid EIN prefixes
	 *  @throws ServiceFailureException
	 */
	// TODO [amalik] this should go into some compliance Bean as not really related to employer
	public String[] getAllValidEinPrefixes() throws ServiceFailureException,IOException;
	
	/**
	 * Get List of Payments for an Employer for a given Date Range
	 * Empty list if no Payments are found
	 * @param dateRange
	 * @return
	 * @throws ValidationException
	 */
	public List<Payment> getPayments(DateRange dateRange) throws ValidationException;
	
	/**
	 * Updates or Add new Payments for the employer
	 * Returns only the updated ones
	 * @param payments
	 * @return
	 * @throws ValidationException
	 */
	public List<Payment> updatePayments(List<Payment> payments) throws ValidationException;
	
	/**
	 * Removes the All Payments for a given Date Range.
	 * @param dateRange
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public void removePayments(DateRange dateRange) throws ValidationException, ServiceFailureException;
	
	/**
	 * Remove a particular Payment
	 * @param uniqueId
	 * @throws ServiceFailureException
	 */
	public void removePayment(Long uniqueId) throws ServiceFailureException;
	
	/**
	 * Get Liabilities from {@link PayrollRun payrollRun} for given period
	 * @param dateRange {@link DateRange} object representing filter period
	 * @return {@link Map Map} with liabilities aggregated and grouped by law id <br>
	 * map key --> lawid, map value --> aggregated liabilities for that lawid
	 * @throws ValidationException
	 */
	public Map<String, Money> getLiabilitiesFromExistingPayrollRuns(DateRange dateRange) 
		throws ValidationException, ServiceFailureException;

	/**
	 * Based on the calendar(date) given this method is called, return a {@link List} of <br>
	 * {@link Payment} objects who's {@see Payment.getPaymentDate() paymentDate} falls <br>
	 * in a quarter before the one in which today is contained.<br>
	 * <code>
	 * Q1 --> Jan 1 - Mar 31
	 * Q2 --> Apr 1 - Jun 30
	 * Q3 --> Jul 1 - Sep 30
	 * Q4 --> Oct 1 - Dec 31
	 * </code>
	 * ex: If the date falls in Q3 of a given year, this method will return the <br>
	 * Payments made in Q1 and Q2 of that year. <br>
	 * If the date falls in Q1, this method will return an empty {@link List}
	 *   
	 * @param calendar The date against which to return Futa Payments
	 * @return {@link List} of {@link Payment} payments
	 * @throws ValidationException 
	 */
	public List<Payment> getFutaPayments(Calendar calendar) throws ValidationException;

	/**
	 * Based on the calendar(date) argument when this method is called, return a {@link List} of <br>
	 * {@link HistPaycheck} objects who's {@see HistPaycheck#getHistPaycheckLineItems()} fall <br>
	 * in a quarter before the one in which today is contained.<br>
	 * <code>
	 * Q1 --> Jan 1 - Mar 31
	 * Q2 --> Apr 1 - Jun 30
	 * Q3 --> Jul 1 - Sep 30
	 * Q4 --> Oct 1 - Dec 31
	 * </code>
	 * ex: If the date falls in Q3 of a given year, this method will return the <br>
	 * HistPaychecks made in Q1 and Q2 of that year. <br>
	 * If the date falls in Q1, this method will return an empty {@link List} <br>
	 * Each {@link HistPaycheck} item will represent a FUTA Liability <br>
	 * Each one will have a single {@link HistPaycheckLineItem lineitem}
	 * @param calendar The {@link Calendar date} from where we need to calculate
	 * @return {@link List} of {@link HistPaycheck} payments
	 * @throws ValidationException 
	 */
	public List<HistPaycheck> getFutaLiabilities(Calendar calendar) throws ValidationException;
	
	public void removeFutaPaymentsAndLiabilitiesYtd(Calendar calendar) throws ValidationException, ServiceFailureException;
	
	/**
	 * Persist data provided (payments and liabilities)
	 * @param payments
	 * @param liabilities
	 * @throws ValidationException 
	 */
	public void updateFutaPaymentsAndLiabilitiesYtd(List<Payment> payments, List<HistPaycheck> liabilities) throws ValidationException;
	
}
