package com.intuit.sb.payroll.capability.employer;

import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.pts.entity.BankAccountState;
import com.intuit.sb.payroll.pts.entity.InteractionEnum;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSWorkflowState;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;

@Local
public interface EmployerPTSWorkflowComponent {

	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "EmployerPTSWorkflowComponent/local";
	
	/**
     * Adds ContactPersons to the current Employer.
     * 
     * @param employerContactPersons the ContactPersons to add.
     * @return the updated List of all the ContactPersons for the current Employer.
     * @throws ValidationException if there are constraints violated.
     * @throws ServiceFailureException 
     */
    public Set<PTSContactPerson> addEmployerContactPersons(List<PTSContactPerson> employerContactPersons) throws ValidationException, ServiceFailureException;
    
    /**
     * Updates ContactPersons to the current Employer.
     * 
     * @param employerContactPersons the ContactPersons to update.
     * @return the updated List of all the ContactPersons for the current Employer.
     * @throws ValidationException if there are constraints violated, or the ContactPersons were never
     * in the DB in the first place.
     * @throws ServiceFailureException 
     */
	public Set<PTSContactPerson> updateEmployerContactPersons(List<PTSContactPerson> employerContactPersons) throws ValidationException, ServiceFailureException;
	
	/**
	 * Deletes ContactPersons from the current Employer.
	 * 
	 * @param employerContactPersons the ContactPersons to delete.
	 * @throws ValidationException if there are constraints violated, 
	 * @throws ServiceFailureException 
	 */
	public void deleteEmployerContactPersons(List<PTSContactPerson> employerContactPersons) throws ValidationException, ServiceFailureException;
    
	/**
	 * Deletes all ContactPersons from the current Employer.
	 * 
	 * @param contactPersons the ContactPersons to delete.
	 * @throws ValidationException if there are constraints violated
	 * @throws ServiceFailureException 
	 */
	public void deleteAllContactPersons() throws ValidationException, ServiceFailureException;
	
	public void updateEmployerTaxServiceState(TaxServiceStatusEnum taxServiceStatus) 
				throws ValidationException, ServiceFailureException;
	
	public void updatePPS_PTS_InteractionState(Identifiable entity, InteractionEnum interactionType) throws ValidationException, ServiceFailureException;
	
	public PTSWorkflowState getEmployerAssistedState(Employer employer) throws ValidationException;

	public PTSContactPerson findEmployerContactPerson(Long empContactPersonuniqueId) throws ServiceFailureException;
	
	public PTSEmployer getPTSEmployer();
	
	public PTSEmployer savePTSEmployer(PTSEmployer employer) throws ValidationException, ServiceFailureException ;

    /**
     * Send Company information to PTS
     * -- Company information, Jurisdiction + Agency information, bank information
     */
	public void sendCompanyInfoToPTS() throws ServiceFailureException, ValidationException;
	
	/**
	 * This component is interested in any activity related to the Assisted setup of the Employer.
	 * 
	 * @param workflowEventType
	 * @param employer
	 * @throws ServiceFailureException
	 */
	public void handleAssistedSetupEvent(WorkflowEventType workflowEventType, PTSEmployer employer) throws ServiceFailureException;

	/**
	 * This component is interested in changes to the Employer as it may affect PTSEmployer state and Tasks.
	 * 
	 * @param eventType
	 * @param employer
	 * @throws ServiceFailureException
	 */
	public void handleEmployerEvent(WorkflowEventType eventType, Employer employer) throws ServiceFailureException;
	
	/**
	 * Get the date when the user sent his/her forms to PTS - basically, <br>
	 * this would be the date of ActivationStatusEnum.ADD_COMPANY_REQUESTED for this user.
	 * @return Date of user sending company info to PTS
	 * @throws ServiceFailureException if there is any problem retrieving this information
	 */
	public Date getTaxPaymentsFormSubmissionDate() throws ServiceFailureException;

	/**
	 * This component is interested in the Employer BankAccount add or update events.
	 * @param workflowEventType
	 * @param bankAccounts
	 */
	public void handleEmployerBankAccountEvent(
			WorkflowEventType workflowEventType, List<BankAccount> bankAccounts) throws ValidationException, ServiceFailureException;

	/**
	 * This will check the state of the EMployer to allow or disallow Data submission to PTS
	 * @return
	 */
	public boolean allowDataSubmissionToPTS();
	
	/**
	 * This method will return the BankAccountState of the BankAccount with the bankAccountID
	 * @param bankAccount
	 * @return BankAccountState
	 */
	public BankAccountState getMostRecentBankAccountState(BankAccount bankAccount);
}
