package com.intuit.sb.payroll.capability.employer.impl;

import java.util.List;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.config.GeminiConfigurationUtility;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.model.TaskTypeEnum;
import com.intuit.sb.payroll.entity.task.TaskEAO;
import com.intuit.sb.payroll.pts.entity.BankAccountStatusEnum;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;

public class EmployerPTSWorkflowTasks {

    /** Generic class to be used for all Tax Payments related TO*DO tasks */
    public static final String TAX_PAYMENTS_TASKS_CLASS = "com.intuit.sb.payroll.task.employer.activation.TaxPayments";

    private static final Logger sLogger = PayrollLoggerFactory.makeLogger(EmployerPTSWorkflowTasks.class);

    public static final String SETUP_ASSISTED_TASK = "setupAssistedTask";
    public static final String ACTIVATION_ACTIVE_TASK = "activationActiveTask";
    public static final String ACTIVATION_ACTIVE_ACCEPTED_TASK = "activationActiveAcceptedTask";
    public static final String REJECTED_EIN_MISMATCH_TASK = "efptsEnrollmentRejectedEINMismatch";
    public static final String PRINT_AND_RESUBMIT_FORMS = "printAndResubmitFormsTask";
    public static final String REJECTED_LEGALNAME_MISMATCH_TASK = "efptsEnrollmentRejecteNameMismatch";
    public static final String REJECTED_BANK_ACCOUNT_TASK = "bankAccountVarificationFailed";

    private static TaskManager taskManager = ServiceLocator.getInstance().getService(TaskManager.JNDI_NAME);

    public static void handlePayrollRunCommitedEvent(TaskManager taskManager, PayrollRun payrollRun)
            throws ServiceFailureException {
        EmployerPTSWorkflowComponent ptsWkflow = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
        PTSEmployer ptsEmployer = ptsWkflow.getPTSEmployer();

        if (GeminiConfigurationUtility.isTaxPaymentsEnabled() &&
                ptsEmployer.getCurrentTaxServiceStatus().equals(TaxServiceStatusEnum.NOT_SIGNED_FOR_ASSISTED)) {
            EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
            Employer employer = employerComponent.getEmployer();
            List<Task> assistedSetupTasks = taskManager.findTaskByTaskNameAndPayLoad(SETUP_ASSISTED_TASK, employer
                    .getId());
            // If there are none (in any state) then create the task.
            if (assistedSetupTasks.size() == 0) {
                Task task = createTask(SETUP_ASSISTED_TASK, new Long(employer.getId()));
                task.setCategory("Scheduled");
                task.setUserManagable(false);
                taskManager.addTask(task);
            }
        }
    }

    public static void handlePTSEmployerSetupComplete(TaskManager taskManager, PTSEmployer employer)
            throws ServiceFailureException {
        // close "Prepare Quarterly Taxes" task
        List<Task> tasks = taskManager.findTaskByTaskNameAndPayLoad(SETUP_ASSISTED_TASK, employer.getEmployer().getId());
        // There will be only one task
        for (Task task : tasks) {
            taskManager.closeTask(task);
        }
    }

    /** Task to show user that PTS Activation is active - user may chose to confirm activation
     *  or reject it*/
    public static void createPtsActivationActiveTask() throws ServiceFailureException {
        createPtsActivationTask(ACTIVATION_ACTIVE_TASK);
    }

    /** Task to show user that PTS Activation is active - AFTER user has confirmed acceptance of the service<br>
     *  Additionally, we close any ACTIVATION_ACTIVE_TASK that is present for this user.
     **/
    public static void createPtsActivationActiveAcceptedTask() throws ServiceFailureException {
        closePtsActivationActiveTask();
        createPtsActivationTask(ACTIVATION_ACTIVE_ACCEPTED_TASK);
    }

    /** Create an "Activation" task of name specified by @param name
     **/
    private static void createPtsActivationTask(String name) throws ServiceFailureException {
        EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
        Employer employer = employerComponent.getEmployer();
        createAndAddTaskToEmployer(name,new Long(employer.getId()));
    }

    /**
     * Close the ACTIVATION_ACTIVE_TASK task
     * @throws ServiceFailureException if there is an error closing the task
     */
    public static void closePtsActivationActiveTask() throws ServiceFailureException {
        TaskManager taskManager = ServiceLocator.getInstance().getService(TaskManager.JNDI_NAME);
        List<Task> tasks = taskManager.findTasksByClass(TAX_PAYMENTS_TASKS_CLASS);
        for (Task task : tasks) {
            //  only ONE task should be there with below name
            if (task.getName().equals(ACTIVATION_ACTIVE_TASK)) {
                taskManager.closeTask(task);
                break;
            }
        }
    }

    /**
     * Task to show user that PTS Enrollment is rejected because of LegalName Mismatch/EIN Mismatch
     */
    public static void handleEFTPSRejectionEvent(EFTPSEnrollmentStatusEnum eftpsEnrollmentStatusEnum) {
        EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
        Employer employer = employerComponent.getEmployer();

        try {
            switch(eftpsEnrollmentStatusEnum){
                case EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH:
                    createPTSRejectionTask(REJECTED_LEGALNAME_MISMATCH_TASK, employer.getId());
                    break;
                case EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH:
                    createPTSRejectionTask(REJECTED_EIN_MISMATCH_TASK, employer.getId());
                    break;
                default:
            }
        }catch (ServiceFailureException e) {
            e.printStackTrace();
        }
    }

    /**
     * When Employer ID/LegalName gets updated by the user, the PTS workflow requires re-submission of <br>
     * the data to PTS server.  This is reminded to the user by a TO*DO Task asking user to PRINT_AND_RESUBMIT
     * forms and here we close the first task of REJECTED_EIN_MISMATCH_TASK/REJECTED_LEGALNAME_MISMATCH_TASK which causes resubmitting of Forms
     */
    public static void handleEmployerUpdatedEvent(PTSEmployer ptsEmployer ,WorkflowEventType eventType) throws ServiceFailureException {
        List<Task> printAndResubmitTask = taskManager.findTaskByTaskNameAndPayLoad(PRINT_AND_RESUBMIT_FORMS,ptsEmployer.getEmployer().getId());
        List<Task> rejectedEFPTSTask = null;  // Can hold either EIN or Name mismatch task

        boolean isPrintAndResubmitTaskRequired = (printAndResubmitTask == null || printAndResubmitTask.size() == 0) ;

        if(WorkflowEventType.UPDATE_EMPLOYERID.equals(eventType)){
            rejectedEFPTSTask = taskManager.findTaskByTaskNameAndPayLoad(REJECTED_EIN_MISMATCH_TASK,ptsEmployer.getEmployer().getId());
        }else if(WorkflowEventType.UPDATE_EMPLOYER.equals(eventType)){
            rejectedEFPTSTask = taskManager.findTaskByTaskNameAndPayLoad(REJECTED_LEGALNAME_MISMATCH_TASK,ptsEmployer.getEmployer().getId());
        }

        if (rejectedEFPTSTask != null && rejectedEFPTSTask.size() > 0){
           boolean isEFTPSTaskBad = rejectedEFPTSTask != null && !rejectedEFPTSTask.get(0).getStatus().equals(TaskStatusEnum.COMPLETED);

            // Create Print form task if needed
            if (isEFTPSTaskBad && isPrintAndResubmitTaskRequired){
                    createAndAddTaskToEmployer(PRINT_AND_RESUBMIT_FORMS, ptsEmployer.getEmployer().getId());
            }else if( isEFTPSTaskBad && printAndResubmitTask.get(0).getStatus().equals(TaskStatusEnum.COMPLETED)){
                 taskManager.reOpenTask(printAndResubmitTask.get(0));
            }

            // Close the task if needed
            if(isEFTPSTaskBad) {
               TaskUtils.closeTask(taskManager, rejectedEFPTSTask.get(0));
            }
        }
    }



    /**
     * Handles the bank account rejection event
     */
    public static void handleBankAccountRejectionEvent(PTSNotificationRequest ptsNotification){
        String bankAccountId = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.SOURCE_BANK_ACCOUNT_ID);
        BankAccountStatusEnum bankAccountStatusEnum = (BankAccountStatusEnum) ptsNotification.getNotificationStatus();

        if(BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED.equals(bankAccountStatusEnum)){
            try {
                createPTSRejectionTask(REJECTED_BANK_ACCOUNT_TASK, Long.valueOf(bankAccountId));
            } catch (NumberFormatException e) {
                e.printStackTrace();
            } catch (ServiceFailureException e) {
                e.printStackTrace();
            }
        }
    }

    /*
     * Handles closing of the Bank Account Rejected task once the bank account is updated
     */
    public static void handleBankAccountUpdate(PTSEmployer ptsEmployer, List<BankAccount> bankAccounts) throws ServiceFailureException{
        // iterate over the updated bank accounts, get the corresponding task for bank account rejection and close it
        for(BankAccount bankAccount:bankAccounts){
        	List<Task> bankAccountRejectedTasks = taskManager.findTaskByTaskNameAndPayLoad(REJECTED_BANK_ACCOUNT_TASK, bankAccount.getId());
        	if(bankAccountRejectedTasks.size() > 1 ){
        		throw new ServiceFailureException("More than one bank account rejected task is there for bankaccount with id " + bankAccount.getId());
        	}
        	if(bankAccountRejectedTasks.size() > 0)
        	{
        		Task taskToBeDeleted = bankAccountRejectedTasks.get(0);
           	 	if(taskToBeDeleted != null){
           		 TaskEAO taskEAO = ServiceLocator.getInstance().getService(TaskEAO.JNDI_NAME);
                 taskEAO.makeTransient(taskToBeDeleted); 
           	 }
        	}
        	
        	
        }
    }

    /**
     *  create an Activation Rejected Task of name specified by @param name
     */
    private static void createPTSRejectionTask(String name, long id) throws ServiceFailureException {
        List<Task> rejectedTasks = taskManager.findTaskByTaskNameAndPayLoad(name, id);
        if(rejectedTasks.size() == 0){
            createAndAddTaskToEmployer(name,id);
        }else if(rejectedTasks.size() > 0 && rejectedTasks.get(0).getStatus().equals(TaskStatusEnum.COMPLETED)){
            try{
                taskManager.reOpenTask(rejectedTasks.get(0));
            }catch (ServiceFailureException e) {
                e.printStackTrace();
            }
        }else {
            sLogger.warn("There is more than one PTS Rejection Task  with the name " + name +" and with payload " + id);
        }
    }

    private static void createAndAddTaskToEmployer(String name, long payload){
        Task task = createTask(name, payload);
        taskManager.addTask(task);
    }

    private static Task createTask(String name, long payload) {
        Task task = new Task();
        task.setName(name);
        task.setPayload(new Long(payload));
        task.setDescription(name);
        task.setType(TaskTypeEnum.SETUP);
        task.setStatus(TaskStatusEnum.NOTSTARTED);
        task.setTaskClass(TAX_PAYMENTS_TASKS_CLASS);
        return task;
    }

    /**
     * Close PRINT_AND_RESUBMIT_FORMS tasks
     * @param ptsEmployer
     */
    public static void closeEmployerUpdateTasks(PTSEmployer ptsEmployer) throws ServiceFailureException {
        taskManager.closeAllTaskByTaskNameAndPayload(PRINT_AND_RESUBMIT_FORMS, ptsEmployer.getEmployer().getId());
    }

    /**
     * For given {@code PTSEmployer} do we have REJECTED_EIN_MISMATCH_TASK and PRINT_AND_RESUBMIT_FORMS<br>
     * in incomplete state
     * @param ptsEmployer
     * @return
     */
    public static boolean updateEmployerTasksPresent(PTSEmployer ptsEmployer) {
        boolean update = false;
        List<Task> printAndResubmitTask = taskManager.findTaskByTaskNameAndPayLoad(PRINT_AND_RESUBMIT_FORMS, ptsEmployer.getEmployer().getId());

        if( printAndResubmitTask != null &&  printAndResubmitTask.size() == 1 &&
                !printAndResubmitTask.get(0).getStatus().equals(TaskStatusEnum.COMPLETED)) {
            update = true;
        }
        return update;
    }
}