package com.intuit.sb.payroll.capability.ppe;

import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import org.apache.commons.lang.StringUtils;

import com.intuit.dataTypes.QBDate;
import com.intuit.payroll.j.CompanyIf;
import com.intuit.payroll.j.EffectiveDatedDatumCollectionIf;
import com.intuit.payroll.j.PayrollDatumIf;
import com.intuit.payroll.j.PayrollFactory;
import com.intuit.payroll.j.PayrollItemCallbackIf;
import com.intuit.payroll.j.PayrollItemIf;
import com.intuit.payroll.j.PayrollItemListCallbackIf;
import com.intuit.payroll.j.PayrollItemListInfoIf;
import com.intuit.payroll.j.UserTaxabilityIf;
import com.intuit.payroll.j.ValidationInfoIf;
import com.intuit.payroll.types.PayrollItemDetailType;
import com.intuit.payroll.types.enumtype.DatumID;
import com.intuit.payroll.types.enumtype.PayrollState;
import com.intuit.sb.payroll.capability.employer.JurisdictionType;
import com.intuit.sb.payroll.capability.workerpayments.CalculationEngineException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Jurisdiction;

/**
 * Class that holds reusable methods for PPE
 * @author rjohn2
 *
 */
public class PPEUtil 
{
	/**
	 * Creates a QBOE equivalent for the Employer
	 * @param employer
	 * @return CompanyIf
	 */
	public static CompanyIf createCompany(Employer employer) 
	{
		CallBackImpl qboeCallback = new CallBackImpl();
		CompanyIf qboeCompany = PayrollFactory.createCompany(new QBDate(employer.getFirstPayrollDate()), null,
				qboeCallback, null);
		qboeCallback.setUserTaxability(qboeCompany.createUserTaxability());
		return qboeCompany;
	}
	
	/**
	 * Returns DTO for min/max/default rates for a given tax item.
	 * @param lawId
	 * @return
	 */
	public static TaxRateDesc getTaxRatesForTaxItem(Employer employer, String lawId) {
		CompanyIf qboCompany = createCompany(employer);
		qboCompany.setDatumValue(DatumID.NEW_COMPANY, Boolean.TRUE);
		PayrollItemDetailType payrollItemDetailType = PayrollItemDetailType.parseStringVal(lawId);
		PayrollItemIf qboPayrollItem = qboCompany.createPayrollItem(payrollItemDetailType);
		PayrollDatumIf rate = qboPayrollItem.getDatum(DatumID.RATE);
		EffectiveDatedDatumCollectionIf rates = (EffectiveDatedDatumCollectionIf) rate.getValue();
		QBDate effectiveDate = new QBDate(DateUtil.getNowDate());
		PayrollDatumIf uiRate = rates.getDatumAsOf(effectiveDate);
		String defaultRate = uiRate.getDefaultValue().toString();
		defaultRate = StringUtils.removeEnd(defaultRate, "%");
		//Min-Max values
		ValidationInfoIf validationInfo = uiRate.getValidationInfo(effectiveDate);
		String min = null;
		Object minValue = validationInfo.getWarnMinValue();
		if (minValue != null) {
			min = minValue.toString();
			min = StringUtils.removeEnd(min, "%");
		} else {
			min = "0.0";
		}
		String max = null;
		Object maxValue = validationInfo.getWarnMaxValue();
		if (maxValue != null) {
			max = maxValue.toString();
			max = StringUtils.removeEnd(max, "%");
		} else {
			max = "100.0";
		}
		TaxRateDesc taxRates = new TaxRateDesc(min, max, defaultRate);
		
		return taxRates;
	}

    /**
     * Creates a unique identifier which is a combination of default (suggested)
     * vendor name and account number label.  
     * @param tax eg FIT, CASUI
     * @return agency key
     */
    public static String getAgencyKey (PayrollItemIf tax)
    {
        String result = tax.getDetailType().getVendorDefaultName ();
        PayrollDatumIf accountNumber = tax.getDatum(DatumID.ACCOUNT_NUMBER);
        if (accountNumber != null) {
            result += accountNumber.getLabel();
        }
        return result;
    }
    
    /**
     * returns the QBOE PPE equivalent of the Jurisdiction passed in
     * @param jur
     * @return
     */
    public static PayrollState mapJurisdiction(Jurisdiction jur)
    {
    	//[rj] - TODO - this is bad way of comparing jurisdictions 
    	// we should do it based on unique Id or something - but there is no other way at this moment
    	if(jur.getName().compareTo(JurisdictionType.UNITED_STATES.getName()) == 0)
    	{
    		return null;
    	}
    	else if(jur.getName().compareTo(JurisdictionType.CALIFORNIA.getName()) == 0)
    	{
    		return PayrollState.CA;
    	}
    	else if(jur.getName().compareTo(JurisdictionType.TEXAS.getName()) == 0)
    	{
    		return PayrollState.TX;
    	}
    	else
    	{
    		throw new CalculationEngineException("Jurisdcition not supported");
    	}
    }
	
	private static class CallBackImpl implements PayrollItemCallbackIf, PayrollItemListCallbackIf 
	{
		private UserTaxabilityIf taxability;
		private Map<Integer, PayrollItemListInfoIf> idMap = new TreeMap<Integer, PayrollItemListInfoIf>();

		public PayrollItemIf getPayrollItem(PayrollItemDetailType arg0) 
		{
			throw new UnsupportedOperationException();
		}

		public PayrollItemIf getPayrollItem(Integer itemId) 
		{
			return (PayrollItemIf) idMap.get(itemId);
		}

		public UserTaxabilityIf getUserTaxability() 
		{
			return taxability;
		}

		public void setUserTaxability(UserTaxabilityIf taxability) 
		{
			this.taxability = taxability;
		}

		public Iterator<PayrollItemListInfoIf> getFullListIterator() 
		{
			return idMap.values().iterator();
		}
	}

}
