package com.intuit.sb.payroll.capability.pto.impl;

import java.text.MessageFormat;
import java.util.Date;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.compensation.CompensationComponent;
import com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOff;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualStartTypeEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffDtl;
import com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.pto.PaidTimeOffEAO;

@Stateless(name = "PaidTimeOffComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class PaidTimeOffComponentBean implements PaidTimeOffComponent {

	protected static final Logger sLogger = PayrollLoggerFactory.makeLogger(PaidTimeOffComponentBean.class);

	@EJB
	private CompensationComponent compensationComponent;
	
	@EJB
	private PaidTimeOffEAO paidTimeOffEAO;
	
	@EJB
	private EntityService entityService;
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent#findPaidTimeOff(com.intuit.sb.payroll.entity.model.PaidTimeOffTypeEnum)
	 */
	public PaidTimeOff findPaidTimeOff(PaidTimeOffTypeEnum paidTimeOffType) throws ServiceFailureException {
		try {
			return paidTimeOffEAO.findPaidTimeOff(paidTimeOffType);
		} catch (Exception ex) {
			throw new ServiceFailureException(ExceptionUtils.throwableMessage(ex));
		}
	}
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent#createPaidTimeOffDtl(com.intuit.sb.payroll.entity.model.CompensationTypeEnum)
	 */
	public PaidTimeOffDtl createPaidTimeOffDtl(PaidTimeOffTypeEnum paidTimeOffTypeEnum) throws ServiceFailureException {
		PaidTimeOff paidTimeOff = findPaidTimeOff(paidTimeOffTypeEnum);
		PaidTimeOffDtl paidTimeOffDtl = new PaidTimeOffDtl();
		paidTimeOffDtl.setPayrollItem(paidTimeOff);
		paidTimeOffDtl.setAccrualStartType(PaidTimeOffAccrualStartTypeEnum.NOTSET);
		paidTimeOffDtl.setPaidTimeOffType(paidTimeOffTypeEnum);
		paidTimeOffDtl.setName(paidTimeOff.getName());
		paidTimeOffDtl.setDescription(paidTimeOff.getDescription());
		
		return paidTimeOffDtl;
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent#addPaidTimeOffDtl(com.intuit.sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.PaidTimeOffDtl)
	 */
	public Worker addPaidTimeOffDtl(Worker worker, PaidTimeOffDtl paidTimeOffDtl) 
			throws WorkflowException, ValidationException, ServiceFailureException {
        PayrollParamValidator.checkIsNotNull(worker, "worker");
		PaidTimeOffValidator.getInstance().validateEntity(paidTimeOffDtl);
		// Rule - Accrual Start can't be before hire date.
		Date accrualStartDate = paidTimeOffDtl.getAccrualStartDate();
		if (accrualStartDate != null) { //Not required if the start type is Immediate, validated above
			Date startDate = worker.getEmploymentPeriod().getStartDate();
			if (startDate.before(accrualStartDate) || DateUtil.isSameDay(accrualStartDate, startDate) ) {
				// All is well
			} else {
				String message = MessageFormat.format("Accrual Start Date of {0} cannot be before the Worker Hire Date of {1}", accrualStartDate, startDate);
				throw new ValidationException(message);
			}
		}
		
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}
		
        // Ensure there is not a PTO of this type already
		Set<PaidTimeOffDtl> paidTimeOffDtls = worker.getPaidTimeOffDtls();
		for (PaidTimeOffDtl existingPTODtl : paidTimeOffDtls) {
			if (existingPTODtl.getPaidTimeOffType() == paidTimeOffDtl.getPaidTimeOffType()) {
				String message = MessageFormat.format("Worker already has a PTO of type {0}", paidTimeOffDtl.getPaidTimeOffType().toString());
				throw new ValidationException(message);
			}
		}
		// See if there is a compensation acting as base rate
		boolean foundBaseCompensation = false;
		Set<CompensationDtl> compensationDtls = worker.getCompensationDtls();
		for (CompensationDtl compensationDtl : compensationDtls) {
			if (compensationDtl.getPtoBaseRateType() == PtoBaseRateTypeEnum.PTO) {
				assertValidPTOBaseType(compensationDtl);
				// Handles both types.
				foundBaseCompensation = true;
				break;
			} else if (compensationDtl.getPtoBaseRateType() == PtoBaseRateTypeEnum.SICK && 
						paidTimeOffDtl.getPaidTimeOffType() == PaidTimeOffTypeEnum.SICK) {
				assertValidPTOBaseType(compensationDtl);
				// Match
				foundBaseCompensation = true;
				break;
			} else if (compensationDtl.getPtoBaseRateType() == PtoBaseRateTypeEnum.VACATION && 
						paidTimeOffDtl.getPaidTimeOffType() == PaidTimeOffTypeEnum.VACATION) {
				assertValidPTOBaseType(compensationDtl);
				// Match
				foundBaseCompensation = true;
				break;
			}
		}
		if (! foundBaseCompensation) {
			throw new ValidationException("No Base Compensation was found for this PTO.");
		}
		
		// Add Disbursement Compensation to the Worker
		CompensationDtl disbursementCompensation = null;
		if (paidTimeOffDtl.getPaidTimeOffType() == PaidTimeOffTypeEnum.SICK) {
			disbursementCompensation = compensationComponent.createCompensationDtl(CompensationTypeEnum.SICKTAKEN);
		} else {
			disbursementCompensation = compensationComponent.createCompensationDtl(CompensationTypeEnum.VACTAKEN);
		}
		//FIXME: the createCompensationDtl above should be an intelligent factory and set the rate based on the type.
		disbursementCompensation.setRate(null);
		compensationComponent.addCompensationToWorker(disbursementCompensation, worker);
		worker.addPaidTimeOffDtl(paidTimeOffDtl);
		
		//FIXME: the worker doesn't seem to get updated when crossing EJB boundaries
		entityService.flush();
		
		return worker;
	}
	
	private void assertValidPTOBaseType(CompensationDtl compensationDtl) throws ValidationException {
		if (compensationDtl.getCompensationType() == CompensationTypeEnum.SALARYREG ||
				compensationDtl.getCompensationType() == CompensationTypeEnum.HOURLYREG) {
			return;
		}
		String message = MessageFormat.format("Invalid Base CompensationType of {0} for PTO", compensationDtl.getCompensationType().toString());
		throw new ValidationException(message);
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent#findPaidTimeOffDtl(com.intuit.sb.payroll.entity.domain.Identifiable)
	 */
	public PaidTimeOffDtl findPaidTimeOffDtl(Identifiable identifiable) throws ServiceFailureException, ValidationException {
		PayrollParamValidator.checkIsNotNull(identifiable, "identifiable");
		try {
			PaidTimeOffDtl paidTimeOffDtl = entityService.findByIdentifiable(PaidTimeOffDtl.class, identifiable);
			return paidTimeOffDtl;
		} catch (Exception ex) {
			throw new ServiceFailureException(ExceptionUtils.throwableMessage(ex));
		}
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent#updatePaidTimeOffDtl(com.intuit.sb.payroll.entity.domain.Identifiable)
	 */
	public Worker updatePaidTimeOffDtl(Worker worker, PaidTimeOffDtl paidTimeOffDtl) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(paidTimeOffDtl, "paidTimeOffDtl");
		PaidTimeOffValidator.getInstance().validateEntity(paidTimeOffDtl);
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}
		
		if (entityService.notPersistent(paidTimeOffDtl)) {
			paidTimeOffDtl = entityService.makeManaged(paidTimeOffDtl);
		}
		
		return worker;
		
	}
	
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.pto.PaidTimeOffComponent#deletePaidTimeOffDtl(com.intuit.sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.PaidTimeOffDtl)
	 */
	public Worker deletePaidTimeOffDtl(Worker worker, PaidTimeOffDtl paidTimeOffDtl) throws ServiceFailureException, ValidationException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(paidTimeOffDtl, "paidTimeOffDtl");
		paidTimeOffDtl.validateNoPeriodToDateValues();
		if (entityService.notPersistent(worker)) {
			worker = entityService.makeManaged(worker);
		}
		
		worker.removePaidTimeOffDtl(paidTimeOffDtl);
		entityService.makeTransient(paidTimeOffDtl);
		return worker;
	}
}
