package com.intuit.sb.payroll.capability.pts.adapter.mapper;

import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.AgencyEnum;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.security.SecurityHelper;
import com.intuit.sbd.payroll.psp.adapters.gemini.AddressAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.CompanyAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.ContactAdapterDTO;
import com.intuit.sbd.payroll.psp.adapters.gemini.OfferingAdapterEnum;
import com.intuit.sbd.payroll.psp.adapters.gemini.PayrollFrequencyAdapterEnum;

/**
 * Mapper for creation of PTS Web Service DTO objects
 * using Gemini basic or PTS Model objects
 */
public class CompanyMapper {

	private static final String NAMESPACE_DELIMITER = "_";

	public static CompanyAdapterDTO mapToCompanyAdapterDTO(PTSEmployer ptsEmployer)
		throws ServiceFailureException
	{
		Employer employer = ptsEmployer.getEmployer();
		
		CompanyAdapterDTO companyDTO = new CompanyAdapterDTO();
		
		String companySourceId = namespaceEmployerId(employer);
		companyDTO.setSourceCompanyId(companySourceId);	
		
		// FIXME companyDTO.setTaxExemptExpirationDate(DateDTO value);
		
		// For now hard coded as we are using this for assisted
		// FIXME get from the employer entity
		companyDTO.setOffering(OfferingAdapterEnum.PAYROLL_PLUS_TAX_SERVICES);
		
		PayrollFrequencyAdapterEnum prlFreqEnum = PayrollFrequencyMapper.mapPayrollFrequency(employer.getPayrollSchedules());
		companyDTO.setPayrollFrequencyCd(prlFreqEnum);
		
		//now set the contact details
		Set<PTSContactPerson> ptsContactPerson = ptsEmployer.getPTSContactPersons(); 

		//primary contact
		ContactAdapterDTO primaryContact = ContactsMapper.getPrimaryContact(ptsContactPerson);
		if (primaryContact != null){
			companyDTO.setPrimaryContact(primaryContact);
		}
		
		
		//secondary contact
		ContactAdapterDTO secondaryContact = ContactsMapper.getSecondayContact(ptsContactPerson);
		if (secondaryContact != null){
			companyDTO.setSecondaryContact(secondaryContact);
		}
		
		//payroll admin contact
		ContactAdapterDTO payrollAdminContact = ContactsMapper.getPayrollAdminContact(ptsContactPerson);
		if (payrollAdminContact != null){
			companyDTO.setPayrollAdminContact(payrollAdminContact);
		}
		
		//set the business and legal addresses
		GlobalPostalAddress businessAddress = employer.getBusinessAddress();
		GlobalPostalAddress legalAddress = employer.getLegalAddress();
		if (legalAddress == null || legalAddress.getAddressLineOne() == null || 
				legalAddress.getPostalCode() == null){
			legalAddress = businessAddress;
		}
		if (businessAddress == null || businessAddress.getAddressLineOne() == null || 
				businessAddress.getPostalCode() == null){
			businessAddress = legalAddress;
		}
		
		AddressAdapterDTO busAddressDTO = AddressMapper.mapAddress(businessAddress);
		companyDTO.setMailingAddress(busAddressDTO);
		
		if(employer.isLegalAndBusAddressSame())
		{
			companyDTO.setLegalAddress(busAddressDTO);
		}
		else
		{
			AddressAdapterDTO legalAddressDTO = AddressMapper.mapAddress(legalAddress);
			companyDTO.setLegalAddress(legalAddressDTO);
		}
		
		companyDTO.setDoingBusinessAsName(employer.getDbaName()==null?"":employer.getDbaName());
		companyDTO.setLegalName(employer.getLegalName()==null?"":employer.getLegalName());
		
		//map the FIEN 
		Set<EmployerId>ids = employer.getEmployerIds();
		String fein = EmployerIDMapper.getFEIN(ids);
		companyDTO.setFein(fein== null?"":fein);
		
		return companyDTO;
	}

	/**
	 * Create a unique identifier for the employer and return it.
	 * The namespace consists the Employer's DB UUID + Realm Id separated by NAMESPACE_DELIMITER
	 * @param employer The Employer entity who is to be namespaced
	 * @return unique identifier for the employer
	 * @throws ServiceFailureException
	 */
	public static String namespaceEmployerId(Employer employer) throws ServiceFailureException {
		if ((employer == null) || (employer.getUuid() == null)){
			// TODO throw Exception 
		}
		StringBuffer sourceCompanyId = new StringBuffer();
		sourceCompanyId.append(employer.getUuid());
		
		String realmId = SecurityHelper.getCurrentRealmId();
		if (realmId != null && realmId.trim().length() > 0) {
			sourceCompanyId.append(NAMESPACE_DELIMITER);
			sourceCompanyId.append(realmId);
		}
        
		return sourceCompanyId.toString();
	}
	
	/**
	 * Method to parse the incoming sourceCompanyId and retrieve the company's REALM ID
	 * from it. {@code namespace()}
	 * @param sourceCompanyId - PPS Source Company ID
	 * @return Realm id of the employer referred to by the sourceCompanyId
	 * @throws ValidationException if the sourceCompanyId does not contain the REALM ID
	 */
	public static String getCompanyRealmIdFromSourceCompanyId(String sourceCompanyId) 
		throws ValidationException {
		if (sourceCompanyId.indexOf(NAMESPACE_DELIMITER) == -1 || sourceCompanyId.endsWith(NAMESPACE_DELIMITER)) {
			throw new ValidationException(sourceCompanyId + " : This sourceCompanyID does not contain" +
					" the realm id of the associated Employer");
		}
		
		int index = sourceCompanyId.indexOf(NAMESPACE_DELIMITER);
		return sourceCompanyId.substring(index+1);
	}
	
	
	/**
	 * Method to parse the incoming sourceCompanyId and retrieve the company's REALM ID
	 * from it. {@code namespace()}
	 * @param sourceCompanyId - PPS Source Company ID
	 * @return UUID of the employer referred to by the sourceCompanyId
	 * @throws ValidationException if the sourceCompanyId does not contain the REALM ID
	 */
	public static String getCompanyIdentifierFromSourceCompanyId(String sourceCompanyId) 
		throws ValidationException {
		if (sourceCompanyId.indexOf(NAMESPACE_DELIMITER) == -1 || sourceCompanyId.endsWith(NAMESPACE_DELIMITER)) {
			throw new ValidationException(sourceCompanyId + " : This sourceCompanyID does not contain" +
					" the realm id of the associated Employer");
		}
		
		int index = sourceCompanyId.indexOf(NAMESPACE_DELIMITER);
		return sourceCompanyId.substring(0,index);
	}

	/**
	 * Mapping Gemini AgencyEnum to PTS AgencyEnum
	 * @param agency Gemini AgencyEnum (US_IRS, CA_EDD, TX_TWC)
	 * @return PTS AgencyEnum - (CAEDD, TXTWC)
	 */
	public static AgencyEnum mapToPtsAgency(com.intuit.sb.payroll.entity.model.AgencyEnum agency) {

		if (agency.equals(com.intuit.sb.payroll.entity.model.AgencyEnum.CA_EDD)) {
			return AgencyEnum.CAEDD;
		}
		if (agency.equals(com.intuit.sb.payroll.entity.model.AgencyEnum.US_IRS)) {
			return AgencyEnum.IRS;
		}
		return null;

	}
}
