package com.intuit.sb.payroll.capability.pts.statehandler;

import java.util.Date;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationException;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationRequest;
import com.intuit.sb.payroll.capability.pts.notification.PTSNotificationTypeEnum;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSAgencyMapper;
import com.intuit.sb.payroll.capability.pts.notification.mapper.PTSNotificationRequestMapper;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentState;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStateEAO;
import com.intuit.sb.payroll.pts.entity.EFTPSEnrollmentStatusEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.PTSWorkflowState;
import com.intuit.sb.payroll.pts.entity.TaxServiceState;


public class EFTPSEnrollmentStatusUpdateHandler extends AbstractStatusUpdateHandler  {

    private static EFTPSEnrollmentStatusUpdateHandler eftpsEnrollStatusHandler =
                    new EFTPSEnrollmentStatusUpdateHandler();

    private EFTPSEnrollmentStatusUpdateHandler(){
        registerStatusChangeWorkers();
    }

    public void validateNotificationRequest(PTSNotificationRequest ptsNotification) throws PTSNotificationException {
        if (!ptsNotification.getNotificationType().equals(PTSNotificationTypeEnum.EFTPS_ENROLLMENT_NOTIFICATION)){
            throw new PTSNotificationException("Wrong Notification being handler here");
        }
    }

    public static StatusUpdateHandler getInstance(){
        return eftpsEnrollStatusHandler;
    }

    // Inner classes for Individual state handlers

    static class GenericEFTPSStatusChangeWorker extends StatusChangeWorker{
        @Override
        public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
            try {
                updateEFTPSEnrollmentStatus(ptsEmployer, ptsNotification);
            } catch (ValidationException e) {
                throw new PTSNotificationException(e.getLocalizedMessage());
            } catch (ServiceFailureException e) {
                throw new PTSNotificationException(e.getLocalizedMessage());
            }
        }
    }

    /**
     * This EINMismatchStatusChangeWorker will handle for "EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH" Status, when Gemini receives
     * the EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH Notification from PTS
     *
     */
    @SuppressWarnings("unused")
    private static class EINMismatchStatusChangeWorker extends GenericEFTPSStatusChangeWorker {
        public EINMismatchStatusChangeWorker() {
            super();
            statusTypeChange.add(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH);
        }
        @Override
        public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
            super.updateStatus(ptsEmployer,  ptsNotification);
            EmployerPTSWorkflowTasks.handleEFTPSRejectionEvent(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_EIN_MISMATCH);
        }
    }
    
    /**
     * This NAMEMismatchStatusChangeWorker will handle for "EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH" Status, when Gemini receives
     * the EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH Notification from PTS
     */
    @SuppressWarnings("unused")
    private static class NAMEMismatchStatusChangeWorker extends GenericEFTPSStatusChangeWorker {
    	public NAMEMismatchStatusChangeWorker() {
    		super();
    		statusTypeChange.add(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH);
    	}
    	@Override
    	public void updateStatus(PTSEmployer ptsEmployer,PTSNotificationRequest ptsNotificationRequest) throws PTSNotificationException {
    		super.updateStatus(ptsEmployer, ptsNotificationRequest);
    		EmployerPTSWorkflowTasks.handleEFTPSRejectionEvent(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_NAME_MISMATCH);
    	}
    } 

    /**
     * Generic Post processing event after ACH Enrollment update has been received from PTS
     */
    @SuppressWarnings("unused")
    private static class CommonStatusChangeWorker extends GenericEFTPSStatusChangeWorker{
        public CommonStatusChangeWorker() {
            super();
            statusTypeChange.add(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_INCOMPLETE);
            statusTypeChange.add(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_INPROGRESS);
            statusTypeChange.add(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_REJECTED_AGEDOUT);
            statusTypeChange.add(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_COMPLETE);
            statusTypeChange.add(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_RESUBMITTED);
            statusTypeChange.add(EFTPSEnrollmentStatusEnum.EFTPS_ENROLLMENT_CANCELLED);
        }

        @Override
        public void updateStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws PTSNotificationException {
            super.updateStatus(ptsEmployer, ptsNotification);

        }
    }

    public synchronized static void updateEFTPSEnrollmentStatus(PTSEmployer ptsEmployer, PTSNotificationRequest ptsNotification) throws ValidationException, ServiceFailureException {
    	ptsEmployer =  getPTSEmployer();
    	EFTPSEnrollmentStateEAO eftpsEnrollStateEAO = ServiceLocator.getInstance().getService(EFTPSEnrollmentStateEAO.JNDI_NAME);
        EFTPSEnrollmentState eftpsEnrollState = new EFTPSEnrollmentState();
        eftpsEnrollState.setEftpsEnrollmentStatus((EFTPSEnrollmentStatusEnum) ptsNotification.getNotificationStatus());
		if (ptsNotification.getTimestamp() != null){
			eftpsEnrollState.setStatusDate(ptsNotification.getTimestamp());
		}else{
			eftpsEnrollState.setStatusDate(new Date());
		}

        String agencyId = ptsNotification.getNotificationAttributes().get(PTSNotificationRequestMapper.AGENCY_ID);
		if (agencyId != null){
			eftpsEnrollState.setAgency(PTSAgencyMapper.mapAgency(agencyId));
		}
        // FIXME achEnrollmentState.setPTSMessages(ptsNotification.getPTSMessage());
		eftpsEnrollState.setActive(Boolean.TRUE);
        PTSWorkflowState ptsWorkflowState = ptsEmployer.getPTSWorkflowState();
        TaxServiceState taxServiceState = ptsWorkflowState.getTaxServiceState();
        eftpsEnrollState = eftpsEnrollStateEAO.makePersistent(eftpsEnrollState);
        taxServiceState.addNotificationStatusEntity(eftpsEnrollState);
    }
}
