package com.intuit.sb.payroll.capability.task;

import java.util.Date;
import java.util.List;

import javax.ejb.Local;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.entity.model.Task;

/**
 * The {@code TaskManager} is responsible for:
 * <ul>
 * <li>managing the list of active and inactive tasks</li>
 * <li>satisfying queries for subsets of tasks</li>
 * </ul>
 */
@Local
public interface TaskManager {
	
	@SuppressWarnings("nls")
	public static final String JNDI_NAME = "TaskManager/local";

	/**
	 * Get the collection of <i>Current</i> Tasks.
	 * <i>Current</i> tasks are defined as those that are:
	 * <ul>
	 * <li>in the date range fromDate to toDate</li>
	 * <li>inactive (complete/cancelled) "date" tasks for a configured past time period</li>
	 * <li>active (notstarted/started) "date" tasks for a configured future time period</li>
	 * <li>active (notstarted/started) "date" tasks that are overdue</li>
	 * <li>active (notstarted/started) "setup" tasks</li>
	 * </ul>
	 * The collection never includes completed "setup" tasks.
	 * 
	 * ### This needs to be exposed via the client API.
	 */
	public List<Task> findCurrentTasksInDateRange(Date fromDate, Date toDate);
	
	/**
	 * Get the collection of <i>Current</i> Tasks.
	 * <i>Current</i> tasks are defined as those that are:
	 * <ul>
	 * <li>inactive (complete/cancelled) "date" tasks for a configured past time period</li>
	 * <li>active (notstarted/started) "date" tasks for a configured future time period</li>
	 * <li>active (notstarted/started) "date" tasks that are overdue</li>
	 * <li>active (notstarted/started) "setup" tasks</li>
	 * </ul>
	 * The collection never includes completed "setup" tasks.
	 * 
	 * ### This needs to be exposed via the client API.
	 */
	public List<Task> findCurrentActiveTasks();
	
	/**
	 * Finds Active Tasks for a class and a particular entity.
	 * @param taskClass
	 * @param payload
	 * @return
	 */
	public List<Task> findActiveTasksByClassAndPayload(String taskClass, Long payload);

	/**
	 * Finds All tasks by class.
	 * 
	 * @param taskClass
	 * @return
	 */
	public List<Task> findTasksByClass(String taskClass);
	
	/**
	 * Re-opens a closed task.
	 * 
	 * @param task
	 * @return
	 * @throws ServiceFailureException
	 */
	public Task reOpenTask(Task task) throws ServiceFailureException;
	
	/**
	 * Get the collection of Tasks with the specified taskName.
	 * @param taskName
	 * @param payload
	 * @return
	 */
	public List<Task> findTaskByTaskNameAndPayLoad(String taskName, Long payload);
		
	/**
	 * 
	 * Changes the task identified by the passed in Id from New/InProgress to Canceled
	 * If the task is already Canceled or Closed no action is taken
	 * If the task corresponding to the  id is not found or when the task status cannot be changed
	 * to Canceled a TaskException is thrown
	 * 
	 * @param id
	 * @throws TaskException
	 */
	public Task cancelTask(long id) throws ServiceFailureException;
	
	/**
	 * 
	 * Changes the task identified by the passed in Id from New/InProgress to Canceled
	 * If the task is already Canceled or Closed no action is taken
	 * If the task corresponding to the  id is not found or when the task status cannot be changed
	 * to Canceled a TaskException is thrown
	 * 
	 * @param task - that Task that needed to be canceled
	 * @throws TaskException
	 */
	public Task cancelTask(Task task) throws ServiceFailureException;

	/**
	 * 
	 * Changes the task identified by Id from New/InProgress to Closed
	 * If the task is already Canceled or Closed no action is taken
	 * If the task corresponding to the  id is not found a TaskException is thrown
	 * 
	 * @param id
	 * @throws TaskException
	 */
	public Task closeTask(long id) throws ServiceFailureException;

	/**
	 * 
	 * Changes the task identified by Id from New/InProgress to Closed
	 * If the task is already Canceled or Closed no action is taken
	 * If the task corresponding to the  id is not found a TaskException is thrown
	 * 
	 * @param task - the Task that need to be changed to closed state
	 * @throws TaskException
	 */
	public Task closeTask(Task task) throws ServiceFailureException;
	
	/**
	 * Changes task status for all the tasks identified by taskName and payload from New/InProgress to Closed
	 * If the task is already Canceled or Closed no action is taken
	 * If the task corresponding to the payload and taskName is not found a TaskException is thrown
	 * @param taskName
	 * @param payload
	 * @throws ServiceFailureException
	 */
	public void closeAllTaskByTaskNameAndPayload(String taskName, long payload) throws ServiceFailureException;
	
	/**
	 * Checks whether the Task represented by Id is userManaged or not
	 * @param id - the unique Id of the Task
	 * @return true if user managable, else false 
	 * @throws TaskException if the Task corresponding to the Id is not found in the system
	 */
	public boolean isUserManagable(long id) throws ServiceFailureException;

	/**
	 * Checks whether the Task is userManaged or not
	 * @param task - the Task that needs to be checked
	 * @return true if user managable, else false 
	 * @throws TaskException if the Task is not found in the system
	 */
	public boolean isUserManagable(Task task) throws ServiceFailureException;
	
	/**
	 * Creates a new Task in the system
	 * @param task - the Task that needs to be added
	 * @return task - the Task added in the system
	 */
	public Task addTask(Task task);
	
	/**
	 * Updates an existing Task in the system.
	 * @param task - the Task to update.
	 * @return task - the updated Task.
	 */
	public Task updateTask(Task task);

	/**
	 * returns the overdue tasks for the user
	 * @return a list of tasks
	 * @throws TaskException
	 */
	public List<Task> getOverdueTasks() throws ServiceFailureException;

	/**
	 * returns the active tasks in the system for the user
	 * Active means Task(s) in New or InProgress state between the specified dates
	 * @return a list of Active Task(s)
	 * @throws TaskException
	 */
	public List<Task> getActiveTasks(Date fromdate, Date toDate) throws ServiceFailureException;

	/**
	 * returns the tasks in the system for the user context in the Date range specified
	 * The Task collection returned will have even the Canceled and Closed tasks
	 * @return a list of Active Task(s)
	 * @throws TaskException
	 */
	public List<Task> getTasks(Date fromDate, Date toDate) throws ServiceFailureException;

	public void deleteTasksByClass(String className);

}