package com.intuit.sb.payroll.capability.task.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.worker.impl.WorkerComponentTasks;
import com.intuit.sb.payroll.capability.workerpayments.impl.WorkerPaymentsComponentTasks;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.task.TaskEAO;

/**
 * General utilities to help with Task management.
 */
public class TaskUtils
{
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(TaskUtils.class);

	protected static TaskEAO taskEAO = ServiceLocator.getInstance().getService(TaskEAO.JNDI_NAME);
	protected static TaskManager taskManager = ServiceLocator.getInstance().getService(TaskManager.JNDI_NAME);

	/**
	 * Set all the tasks in a list to a specified status.
	 * The the task status is set to an inactive state (campleted/cancelled), each task
	 * has its task completion date set to the current date, otherwise it is set to null.
	 * <p>
	 * After setting the new status all the tasks are logged at debug level.
	 * @param tasks The list of tasks whose status will be set.
	 * @param taskStatus The status to set the tasks to.
	 */
	static public void setTaskStatus(List<Task> tasks, TaskStatusEnum taskStatus)
	{
		for (Task task : tasks)
		{
			setTaskStatus(task, taskStatus);
		}
		logTasks(tasks);
	}

	public static void setTaskStatus(Task task, TaskStatusEnum taskStatus) {
		task.setStatus(taskStatus);
		if (taskStatus.equals(TaskStatusEnum.COMPLETED) || taskStatus.equals(TaskStatusEnum.CANCELED))
		{
			Date now = DateUtil.getNowDate(); // Use UTC as this is the time we search on
			task.setCompletionDate(now);
		}
		else
		{
			task.setCompletionDate(null);
		}
	}

	/**
	 * Produce a string representation of a task. It would be nice if the generated Task class did this.
	 * @param task the task to stringiffy.
	 * @return the task in string form.
	 */
	static public String taskToString(Task task)
	{
		StringBuilder sb = new StringBuilder();
		sb.append(task.getType());
		sb.append(" (").append(task.getDeadline()).append(")");
		sb.append(" task '").append(task.getDescription());
		sb.append("' state [").append(task.getStatus());
		if (task.getStatus() == TaskStatusEnum.COMPLETED || task.getStatus() == TaskStatusEnum.CANCELED)
		{
			sb.append(" {").append(task.getCompletionDate()).append("}");
		}
		sb.append("] class [").append(task.getTaskClass());
		sb.append("] payload [" + task.getPayload()); // use + not append to handle null
		sb.append("] category [").append(task.getCategory());
		sb.append("]");
		return sb.toString();
	}

	/**
	 * Log a list of tasks. The tasks are logged using debug level logging.
	 * @param tasks the list of tasks.
	 * @param formatter a TaskFormatter that formats the task into a string.
	 * If null a default TaskFormatter is constructed and used.
	 */
	public static void logTasks(List<Task> tasks)
	{
		if (sLogger.isDebugEnabled())
		{
			for (Task task : tasks)
			{
				TaskUtils.logTask(task);
			}
		}
	}

	/**
	 * Log a list of tasks. The tasks are logged using debug level logging.
	 * @param tasks the list of tasks.
	 * @param formatter a TaskFormatter that formats the task into a string.
	 * If null a default TaskFormatter is constructed and used.
	 */
	public static void logTask(Task task)
	{
		if (sLogger.isDebugEnabled())
		{
			sLogger.debug(task.toString());
		}
	}

	/**
	 * Utility method to clean up all the worker related tasks in DB that are created if a worker is added to DB.
	 * @param workerId the Id of the worker whose tasks have to be clean up.
	 */
	public static void removeAllWorkerTasks(Long workerId)
	{
		//TODO add appropriate methods on TaskManager inteface to achieve the same thing.
		List<Task> workerTasks = taskManager.findTasksByClass(WorkerComponentTasks.SETUP_EMPLOYEE_CLASS);
		for (com.intuit.sb.payroll.entity.model.Task task : workerTasks) {
			if(task.getPayload().compareTo(workerId) == 0)
    		   taskEAO.makeTransient(task);
		}
	}

	/**
	 * Utility method to retrieve all tasks for the given worker.
	 * @param workerId the Id of the worker whose tasks to retrieve.
	 */
	public static List<Task> getAllWorkerTasks(Long workerId)
	{
		List<Task> tasks = new ArrayList<Task>();
		List<Task> workerTasks = taskManager.findTasksByClass(WorkerComponentTasks.SETUP_EMPLOYEE_CLASS);
		for (com.intuit.sb.payroll.entity.model.Task task : workerTasks) {
			if(task.getPayload().compareTo(workerId) == 0)
				tasks.add(task);
		}
		return tasks;
	}

	public static void removeAllWorkerPaymentTasks()
	{
		taskManager.deleteTasksByClass(WorkerPaymentsComponentTasks.EXECUTE_PAYROLL_RUN_CLASS);
	}

	public static List<Task> getTaskByTaskName(List<Task> tasks, String taskName, Identifiable entity){
		Long payload = new Long(0);
		if (entity != null){
			payload = new Long(entity.getId());
		}
		List<Task> foundtasks = new ArrayList<Task>();
		for (Task task : tasks) {
			if ((task.getName().equals(taskName)) &&
				task.getPayload().equals(payload)){
				foundtasks.add(task);
			}
		}
		return foundtasks;
	}

	public static void closeTask(TaskManager taskManager, Task taskToClose) throws ServiceFailureException{
		if (!taskToClose.getStatus().equals(TaskStatusEnum.COMPLETED)) {
			taskToClose = taskManager.closeTask(taskToClose);
		}
	}


	public static Task getLastModifiedTask(List<Task> tasks)
	{
		Task taskToReturn = null;
		if ( tasks != null && tasks.size() > 0){
			Collections.sort(tasks, LAST_MODIFIED);
			taskToReturn = tasks.get(0);
		}
		return taskToReturn;
	}

	/**
	*	Comparator for Task to sort a List of Task objects in DESCENDING order (newest Task at the top)
	*
	*/
	static final Comparator<Task> LAST_MODIFIED =
        	new Comparator<Task>() {
		public int compare(Task t1, Task t2) {
			return t2.getLastModificationDate().compareTo(t1.getLastModificationDate());
		}
	};

}
