package com.intuit.sb.payroll.capability.taxes.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.payroll.types.PayrollException;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.ppe.AgencyView;
import com.intuit.sb.payroll.capability.ppe.PPEUtil;
import com.intuit.sb.payroll.capability.ppe.PayrollProcessingEngine;
import com.intuit.sb.payroll.capability.ppe.TaxRateDesc;
import com.intuit.sb.payroll.capability.ppe.AgencyView.TaxRowView;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.worker.WorkerWorkflowType;
import com.intuit.sb.payroll.capability.worker.impl.WorkerSetupRules;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.AgencyIdType;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.LawId;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.Tax;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.taxes.TaxEAO;
import com.intuit.sb.payroll.entity.taxes.withholding.UnemploymentInsDtlEAO;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.entity.worker.WorkerEAO;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

@Stateless(name = "TaxComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class TaxComponentBean implements TaxComponent {

	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(TaxComponentBean.class);
	
	@EJB
	WorkerEAO workerEAO;

	@EJB
	private EmployerComponent employerComponent;

	@EJB
	private PayrollProcessingEngine ppe;

	@EJB
	private UnemploymentInsDtlEAO unemploymentInsDtlEAO;
	
	@EJB
	private TaxEAO taxEAO;
	
	@EJB
	private EntityService entityService;

	public List<Law> determineApplicableTaxes(Worker worker) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		if (WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.DetermineTaxes, worker)) {
			return determineApplicableTaxesImpl(worker);
		} else {
			ExceptionUtils.generateWorkflowExceptionFromRequiredData(WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.DetermineTaxes, worker));
			return null;
		}
	}

	public Worker applyApplicableTaxes(Worker worker) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		sLogger.debug("Applying applicable taxes for worker " + worker.getId());
		if (WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.DetermineTaxes, worker)) {
			List<Law> lawList = this.determineApplicableTaxesImpl(worker);
			mergeApplicableTaxes(lawList, worker);
		} else {
			// Mark any existing taxes as inactive.
			Set<TaxDtl> taxDtls = worker.getTaxDtls();
			for (TaxDtl taxDtl : taxDtls) {
				taxDtl.setDeleted(true);
			}
		}
		return worker;
	}

	public List<Law> determineApplicableTaxesImpl(Worker worker) throws ValidationException, ServiceFailureException {
		Integer workerAge = determineAgeInYearsOfWorker(worker);
		WorkLocation workLocation = worker.getWorkLocation();
		if (workLocation == null) {
			throw new ValidationException("Failed to find the worklocation for Worker");
		}
		List<Jurisdiction> jurisdictionList = employerComponent.determineAllJurisdictionsWorkLocation(workLocation);
		USPostalAddress taxAddress = AddressConvertor.getUSFromGlobal(workLocation.getTaxResidenceAddress(),
				new USPostalAddressImpl());
		List<Law> lawList = determineApplicableTaxLawsForWorker(worker, workerAge, taxAddress, jurisdictionList);
		return lawList;
	}

	void mergeApplicableTaxes(List<Law> applicableLaws, Worker worker) throws ValidationException {
		// Create Map of the laws that apply to the employee.
		final Map<String, Law> lawMap = new HashMap<String, Law>();
		for (Law law : applicableLaws) {
			lawMap.put(law.getLawId(), law);
		}

		final List<TaxDtl> listToDelete = new ArrayList<TaxDtl>();
		// Create a map of the current payroll Items
		final Map<String, TaxDtl> piMap = new HashMap<String, TaxDtl>();

		final Set<TaxDtl> taxItemIterator = worker.getTaxDtls();
		for (TaxDtl item : taxItemIterator) {
			PayrollItem payrollItem = item.getPayrollItem();
			String lawId = payrollItem.getLawId().getValue();
			// We only want to check against tax items
			if (!lawMap.containsKey(lawId)) {
				listToDelete.add(item);
			}
			piMap.put(payrollItem.getLawId().getValue(), item);
		}

		// Remove bad guys
		for (PayrollItemDtl item : listToDelete) {
			item.setDeleted(true);
		}

		final Map<String, Tax> taxByIdMap = new HashMap<String, Tax>();
		final List<Tax> taxes = taxEAO.findAll();
		for (Tax tax : taxes) {
			taxByIdMap.put(tax.getLawId().getValue(), tax);
		}
		// Add new guys or set inactive to active again.
		final Set<String> keyList = lawMap.keySet();
		for (String key : keyList) {
			if (!piMap.containsKey(key)) {
				Law law = lawMap.get(key);
				Tax tax = taxByIdMap.get(law.getLawId());
				TaxDtl taxDtl = createTaxDtlFromTax(tax);
				addTaxToWorker(taxDtl, worker);
			} else {
				TaxDtl taxDtl = piMap.get(key);
				taxDtl.setDeleted(false);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.taxes.TaxComponent#addTaxToWorker(com.intuit.sb.payroll.entity.model.TaxDtl, com.intuit.sb.payroll.entity.model.Worker)
	 */
	public Worker addTaxToWorker(TaxDtl taxDtl, Worker worker) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(taxDtl, "taxDtl");
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		if (entityService.notPersistent(taxDtl)) {
			taxDtl = entityService.makePersistent(taxDtl);
		}
		worker.addTaxDtl(taxDtl);
		return worker;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.capability.taxes.TaxComponent#createTaxDtlFromTax(com.intuit.sb.payroll.entity.model.Tax)
	 */
	public TaxDtl createTaxDtlFromTax(Tax tax) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(tax, "tax");
		TaxDtl taxDtl = new TaxDtl();
		taxDtl.setPayrollItem(tax);
		taxDtl.setTaxType(tax.getTaxType());
		taxDtl.setName(tax.getName());
		taxDtl.setDescription(tax.getDescription());
		taxDtl.setSequenceNo(tax.getSequenceNo());
		return taxDtl;
	}

	public boolean isSSNAllowedForEIN(Employer employer) throws ServiceFailureException {
		RulesEmployer rulesEmployer = new RulesEmployer();
		rulesEmployer.setIndustryType(employer.getIndustryType());
		rulesEmployer.setLegalStructure(employer.getLegalStructure());

		// get the number of employees for the company
		int numEmployees = workerEAO.getWorkerCount();

		rulesEmployer.setNumEmployees(numEmployees);

		return EmployerTaxSetupRules.isSSNAllowedForEIN(rulesEmployer);
	}

	private List<Law> determineApplicableTaxLawsForWorker(RulesEmployee employee, RulesEmployer employer,
			List<Jurisdiction> jurisdictions) throws ServiceFailureException, ValidationException {
		PayrollParamValidator.checkIsNotNull(employee, "IRulesEmployee");
		PayrollParamValidator.checkIsNotNull(employer, "IRulesEmployer");
		PayrollParamValidator.checkIsNotNull(jurisdictions, "jurisdictions");

		List<Law> allApplicableLaws = getAllApplicableLawsForJusrisdiction(jurisdictions);

		allApplicableLaws = PayrollTaxRules.getApplicableTaxesForEmployee(employee, employer, allApplicableLaws);
		return allApplicableLaws;
	}

	private List<Law> getAllApplicableLawsForJusrisdiction(List<Jurisdiction> jurisdictions) {
		List<Law> applicableLaws = new ArrayList<Law>();
		for (Jurisdiction j : jurisdictions) {
			Set<Law> lawList = j.getLaws();
			for (Law law : lawList) {
				applicableLaws.add(law);
			}
		}
		return applicableLaws;
	}

	/**
	 * Provides a list of Applicable Laws based on Worker and Employer circumstances and Jurisdictions. The Employer is
	 * implicit within the context of the caller's organization.
	 * 
	 * @param worker Worker on whom the applicable tax laws are to be determined
	 * @param ageInYears Age (Integer) of the worker
	 * @param address PostalAddress of the worker
	 * @param jurisdictions List of Jurisdiction for the worker
	 * @see RulesEmployee
	 * @see Jurisdiction
	 * @see Law
	 * @return List of law applicable on the worker
	 * @throws ServiceFailureException 
	 */
	private List<Law> determineApplicableTaxLawsForWorker(Worker worker, Integer ageInYears, USPostalAddress address,
			List<Jurisdiction> jurisdictions) throws ServiceFailureException {
		try {
			sLogger.debug("determineApplicableTaxLawsForWorker "+ worker.getId());
			PayrollParamValidator.checkIsNotNull(worker, "workerEntity");
			PayrollParamValidator.checkIsNotNull(address, "addressEntity");
			PayrollParamValidator.checkIsNotNull(jurisdictions, "jurisdictionsEntity");
			String workCountry = determineWorkCountry(address);
			String workState = determineWorkState(address);

			// We pass in a value object to the Rules engine, based on the Company
			// Entity.
			RulesEmployer rulesEmployer = new RulesEmployer();

			Employer employer = employerComponent.getEmployer();
			IndustryTypeEnum industryType = employer.getIndustryType();
			if (!IndustryTypeEnum.NOTSET.equals(industryType)){
				rulesEmployer.setIndustryType(employer.getIndustryType());
			}
			
			Boolean isNonProfit = employer.isNonProfit();
			if (isNonProfit != null){
				rulesEmployer.setIsForProfit(Boolean.valueOf(!isNonProfit.booleanValue()));
			}
			
			LegalStructureEnum legalStructure = employer.getLegalStructure();
			if (!LegalStructureEnum.NOTSET.equals(legalStructure)){
				rulesEmployer.setLegalStructure(employer.getLegalStructure());
			}

			RulesEmployee rulesEmployee = new RulesEmployee();
			rulesEmployee.setAgeInYears(DateUtil.calculateYearsElapsedFromNow(worker.getDateOfBirth()));
			rulesEmployee.setWorkCountry(workCountry);
			rulesEmployee.setWorkState(workState);
			rulesEmployee.setRelationToOwner(worker.getRelationshipToOwner());
			List<Law> applicableTaxes = determineApplicableTaxLawsForWorker(rulesEmployee, rulesEmployer, jurisdictions);
			return applicableTaxes;
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}

	}

	private String determineWorkState(USPostalAddress address) {
		return address.getRegionOrState();
	}

	private String determineWorkCountry(USPostalAddress address) {
		return address.getCountryISOCode();
	}

	/**
	 * Gets the age of the employee in years.
	 * 
	 * @param worker - Worker object whose age needs to be determined
	 * @return Integer age
	 * @throws ValidationException 
	 * @throws PayrollException if worker entity is null
	 */
	private Integer determineAgeInYearsOfWorker(Worker worker) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(worker, "workerEntity");
		Date dob = worker.getDateOfBirth();
		PayrollParamValidator.checkIsNotNull(dob, "Date of Birth");
		int ageInYears = DateUtil.calculateYearsElapsedFromNow(dob);
		return Integer.valueOf(ageInYears);
	}

	/**
	 * Finds the Agencies and the taxes applicable for the Jurisdictions passed in
	 * 
	 * @param employer - Employer
	 * @return collection of AgencyViews
	 * @throws ValidationException 
	 * @throws PayrollException if employer entity is null
	 */
	public List<AgencyView> getEmployerAgencyDetails(Employer employer, List<Jurisdiction> jurisdictions) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(employer, "employer entity");
		Jurisdiction[] jurisdictionArray = new Jurisdiction[jurisdictions.size()];
		jurisdictionArray = jurisdictions.toArray(jurisdictionArray);

		Set<EmployerId> employerIdSet = employer.getEmployerIds();
		List<UnemploymentInsDocDtl> unemployementInsList = unemploymentInsDtlEAO.findAll();

		List<AgencyView> agencyViews = ppe.getEmployerAgencyDetails(employer, jurisdictionArray);

		// add all the missing data that is not provided by PPE like the Agency URL
		// as well as user editable data, from Gemini db like SUI rate
		//FIXME [Locale] use the rules as a Factory.
		for (AgencyView agencyView : agencyViews) {
			// here add the agencyURL from gemin db
			// for the time being had coding - FIXME [rj]
			String agencyName = agencyView.getDefaultAgencyName();
			AgencyEnum agencyEnum = null;
			if (agencyName.contains("United States Treasury")) {
				agencyView.setAgencyUrl("http://www.irs.gov/");
				agencyEnum = AgencyEnum.US_IRS;
			} else if (agencyName.contains("EDD")) {
				agencyView.setAgencyUrl("http://www.edd.ca.gov/");
				agencyEnum = AgencyEnum.CA_EDD;
			} else if (agencyName.contains("Texas")) {
				agencyView.setAgencyUrl("");
				agencyEnum = AgencyEnum.TX_TWC;
			}

			// Populate the TIN for each Jurisdiction from repo
			agencyView.setTinNumber(getTINInformation(agencyEnum, employerIdSet));

			// here we check any of the tax items is user editable
			// if they are editable, look into Gemini db and get any rates entered by user
			// and merge these rates ion AgnecyView
			Collection<AgencyView.TaxRowView> taxes = agencyView.getOrderedTaxRows();
			for (AgencyView.TaxRowView tax : taxes) {
				// if the tax rate is editable
				if (tax.isEditable()) {
					populateTaxValuesFromDB(tax, unemployementInsList);
				}
			}
		}

		return agencyViews;
	}

	private String getTINInformation(AgencyEnum agencyEnum, Set<EmployerId> employerIdSet) {

		for (EmployerId employerId : employerIdSet) {
			AgencyIdType agencyIdType = employerId.getAgencyIdType();
			if (agencyIdType.getAgencyKey().equals(agencyEnum)) {
				return employerId.getValue();
			}
		}
		return null;
	}

	private void populateTaxValuesFromDB(TaxRowView tax, List<UnemploymentInsDocDtl> unemployementInsList) {
		if (tax == null)
			return;

		// TODO [rj] - for employee this rate is not applicable (except for NY I guess - we will see when we get to NY)
		tax.setEmployeeRate("None"); // FIXME - get this string from resource bundle

		// FIXME - [rj] remove this for loop logic and have a query on lawId to get the SUI tax

		UnemploymentInsDocDtl unempInsDtl = getUnemploymentInsDtl(unemployementInsList, tax.getLawId());

		if (unempInsDtl != null) {
			Double rate = unempInsDtl.getPercentageRate();
			tax.setCompanyRate((rate == null) ? "" : rate.toString() + "%");
			tax.setErTaxGuid(unempInsDtl.getId());
		} else {
			tax.setCompanyRate("");
		}
	}

	private UnemploymentInsDocDtl getUnemploymentInsDtl(List<UnemploymentInsDocDtl> unemployementInsList, int lawId) {

		String lawIdStr = String.valueOf(lawId);

		for (UnemploymentInsDocDtl unemploymentInsDocDtl : unemployementInsList) {
			if (lawIdStr.equals(unemploymentInsDocDtl.getLawId().getValue())) {
				return unemploymentInsDocDtl;
			}
		}
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public List<UnemploymentInsDocDtl> getEmployerUnemploymentInsurances() {
		List<UnemploymentInsDocDtl> insurances = unemploymentInsDtlEAO.findAll();
		Employer employer = employerComponent.getEmployer();
		for (UnemploymentInsDocDtl unemploymentInsDocDtl : insurances) {
			getMinMaxDefaultRates(employer, unemploymentInsDocDtl);
		}
		return insurances;
	}
	
	/**
	 * Get current quarter rate
	 */
	public UnemploymentInsDocDtl getCurrentQuarterRate(Date date) {
		List<UnemploymentInsDocDtl> insurances = getEmployerUnemploymentInsurances();
		UnemploymentInsDocDtl unInsDocDtl = null;
		DateRange currentdateRange = PtdUtils.getQtrRangeForDate(date);
		
		for(UnemploymentInsDocDtl unemploymentInsDocDtl:insurances ) {
			DateRange entityDateRange = unemploymentInsDocDtl.getEffectivePeriod().getDateRange();
			
			if(currentdateRange.equals(entityDateRange)||(currentdateRange.getStartDate().after(entityDateRange.getStartDate()))) 
			{
				unInsDocDtl = unemploymentInsDocDtl;
			}
		}
		return unInsDocDtl;
	}			
	
	/**
	 * {@inheritDoc}
	 */
	public UnemploymentInsDocDtl getEmployerUnemploymentInsurance(Long uniqueId) throws ServiceFailureException { 
		UnemploymentInsDocDtl unemploymentInsDocDtl = unemploymentInsDtlEAO.findById(uniqueId);
		Employer employer = employerComponent.getEmployer();
		if (unemploymentInsDocDtl == null) {
			throw new ServiceFailureException("Could not find the unemployement tax for the Id " + uniqueId);
		}
		getMinMaxDefaultRates(employer, unemploymentInsDocDtl);
		return unemploymentInsDocDtl;
	}
	
	/**
	 * For a given UI setting, get the min/max and default values.
	 * @param employer
	 * @param unemploymentInsDocDtl
	 */
	private void getMinMaxDefaultRates(Employer employer, UnemploymentInsDocDtl unemploymentInsDocDtl) {
		LawId lawId = unemploymentInsDocDtl.getLawId();
		String law = lawId.getValue();
		TaxRateDesc taxRates = PPEUtil.getTaxRatesForTaxItem(employer, law);
		sLogger.info(taxRates.toString());
		if (null == unemploymentInsDocDtl.getPercentageRate()) {
			unemploymentInsDocDtl.setPercentageRate(new Double(taxRates.getDefaultRate()));
			unemploymentInsDocDtl.setDefaultRate(true);
		}
		unemploymentInsDocDtl.setMaxRate(taxRates.getMaxRate());
		unemploymentInsDocDtl.setMinRate(taxRates.getMinRate());
	}

	public List<UnemploymentInsDocDtl> updateUnemploymentInsurances(List<UnemploymentInsDocDtl> unList) throws ServiceFailureException, ValidationException{
		for(UnemploymentInsDocDtl unemInsDocDtl:unList ) {
			updateEmployerUnemploymentInsurance(unemInsDocDtl);
		}
		return unList;
	}
	public UnemploymentInsDocDtl updateEmployerUnemploymentInsurance(UnemploymentInsDocDtl unemploymentInsDocDtl) throws ValidationException{

		// Validate the entity
		Double futureRate = unemploymentInsDocDtl.getPercentageFutureRate();
		Date futureRateStartDate = unemploymentInsDocDtl.getFutureRateStartDate();
		List<UnemploymentInsDocDtl> insurances = getEmployerUnemploymentInsurances();
		UnemploymentInsDocDtl docDtl = null;
		boolean makeManage = true;
		for(UnemploymentInsDocDtl unemInsDocDtl:insurances ) {
			//for the same State if the effective periods of the two SUI are different then  effective period of already existing SUI has to be
			//modified and endDate for it will be actually the startDate of the newly added SUI
			if(unemInsDocDtl.getLawId().equals(unemploymentInsDocDtl.getLawId())) 
			{
				DateRange dateRangeOne = unemInsDocDtl.getEffectivePeriod().getDateRange();
				DateRange dateRangeTwo = unemploymentInsDocDtl.getEffectivePeriod().getDateRange();
				Date startDateOne = dateRangeOne.getStartDate();
				Date startDateTwo = dateRangeTwo.getStartDate();
				Date endDateOne = dateRangeOne.getEndDate();
				Date endDateTwo = dateRangeTwo.getEndDate();
				if(!dateRangeOne.equals(dateRangeTwo))
				{
					//future SUI rate
					if(startDateOne.before(startDateTwo) && endDateOne == null)
					{
						dateRangeOne.setEndDate(dateRangeTwo.getStartDate());
					}
					
					//past SUI rate
					if(startDateTwo.after(startDateOne) && (endDateOne != null && startDateTwo.before(endDateOne)))
					{
						dateRangeTwo.setEndDate(endDateOne);
						dateRangeOne.setEndDate(startDateTwo);
					}
					
					if(startDateOne.equals(startDateTwo))
					{
						unemInsDocDtl.setPercentageRate(unemploymentInsDocDtl.getPercentageRate());
						docDtl = unemInsDocDtl;
						makeManage = false;
					}
				}else{
					if(endDateOne == null && endDateTwo == null)
					{
						DateRange currentdateRange = PtdUtils.getQtrRangeForDate(new Date());
						dateRangeOne.setStartDate(currentdateRange.getStartDate());
						dateRangeTwo.setEndDate(currentdateRange.getStartDate());
					}		
					
				}
			}
		}
		if ((futureRate != null && futureRateStartDate == null) || (futureRate == null && futureRateStartDate != null)) {
			// TODO
			throw new ValidationException("Both futureRate and futureRateStartDate  must be null OR both must be not null"
					+ unemploymentInsDocDtl.getId());
//					+ unemploymentInsDocDtl.getUUID());		// Anduril needs to support UUIDs
		}

		validateRates(futureRate, false);
		validateRates(unemploymentInsDocDtl.getPercentageRate(), true);

		// Update it
		if(makeManage){
			return unemploymentInsDtlEAO.makeManaged(unemploymentInsDocDtl);
		}else{
			return unemploymentInsDtlEAO.makeManaged(docDtl); 
		}
		
	}

	private void validateRates(Double rate, boolean required) throws ValidationException {
		if (required) {
			if (rate == null) {
				throw new ValidationException("Required rate can not be empty or null");
			}
		}

		if ((rate != null) && (rate < 0)) {
			throw new ValidationException("Required rate can not be less than zero");
		}

		if ((rate != null) && (rate > 100)) {
			throw new ValidationException("Required rate can not be more than 100 percent");
		}
	}
}
