package com.intuit.sb.payroll.capability.taxes.withholding.impl;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent;
import com.intuit.sb.payroll.capability.worker.WorkerWorkflowType;
import com.intuit.sb.payroll.capability.worker.impl.WorkerSetupRules;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.ComplianceDoc;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Responsible for Tax Compliance management for Withholdings, WorkStatuses, etc...
 * 
 * $Author$ 
 * $Revision$
 * $DateTime$
 *
 */
@Stateless(name = "TaxWithholdingsComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class TaxWithholdingsComponentBean implements TaxWithholdingsComponent {

	@EJB
	private EntityService entityService;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent#createWorkerComplianceDocDtl(com.
	 * intuit.sb.payroll.entity.model.ComplianceDocIdEnum)
	 */
	public WorkerComplianceDocDtl createWorkerComplianceDocDtl(ComplianceDocIdEnum docType)
			throws ValidationException {
		PayrollParamValidator.checkIsNotNull(docType, "complianceDocType");
		WorkerComplianceDocDtl workCompDtl = WorkerComplianceDocFactory.createWorkerComplianceDocDtlFromType(docType);
		return workCompDtl;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent#addWorkerComplianceDocDtl(com.intuit
	 * .sb.payroll.entity.model.Worker, com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl)
	 */
	public Worker addWorkerComplianceDocDtl(Worker worker, WorkerComplianceDocDtl workerComplianceDocDtl)
			throws WorkflowException, ValidationException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		// If taxes etc have not been set up, we can't go down this road.
		if (!WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.SetupWithholding, worker)) {
			ExceptionUtils.generateWorkflowExceptionFromRequiredData(WorkerSetupRules.getMissingDataForWorkflow(
					WorkerWorkflowType.SetupWithholding, worker));
		}
		// General validation
		WithholdingDataValidator.getInstance().validateEntity(workerComplianceDocDtl);
		// Context-specific validation
		// Must be new
		if (workerComplianceDocDtl.getId() != 0L) {
			throw new ValidationException("addWorkerComplianceDocDtls() called with a saved WorkerComplianceDocDtl");
		}
		// Must not be a duplicate
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		for (WorkerComplianceDocDtl existingCompDtl : workerComplianceDocDtls) {
			if (existingCompDtl.getDocId() == workerComplianceDocDtl.getDocId()) {
				throw new ValidationException(
						"ComplianceDoc already exists - use updateWorkerComplianceDocDtl to change it.");
			}
		}

		worker.addWorkerComplianceDocDtl(workerComplianceDocDtl);
		return worker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent#updateWorkerComplianceDocDtls(com
	 * .intuit.sb.payroll.entity.model.Worker, java.util.Set)
	 */
	public Worker updateWorkerComplianceDocDtl(Worker worker, WorkerComplianceDocDtl workerComplianceDocDtl)
			throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		// If taxes etc have not been set up, we can't go down this road.
		if (!WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.SetupWithholding, worker)) {
			ExceptionUtils.generateWorkflowExceptionFromRequiredData(WorkerSetupRules.getMissingDataForWorkflow(
					WorkerWorkflowType.SetupWithholding, worker));
		}
		// General Validation
		WithholdingDataValidator.getInstance().validateEntity(workerComplianceDocDtl);
		// Context-specific Validation
		if (workerComplianceDocDtl.getId() == 0L) {
			throw new ValidationException(
					"updateWorkerComplianceDocDtl() called with an unsaved WorkerComplianceDocDtl");
		}

		boolean found = false;
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		for (WorkerComplianceDocDtl existingCompDtl : workerComplianceDocDtls) {
			if (existingCompDtl.getId() == workerComplianceDocDtl.getId()) {
				workerComplianceDocDtl.contentCopy(existingCompDtl);
				found = true;
				break;
			}
		}

		if (!found) {
			throw new ServiceFailureException("Unable to find WorkerComplianceDetail to update");
		}

		return worker;
	}

	public Worker deleteWorkerComplianceDocDtl(Worker worker, WorkerComplianceDocDtl workerComplianceDocDtl)
			throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		// Context-specific Validation
		if (workerComplianceDocDtl.getId() == 0L) {
			throw new ValidationException(
					"deleteWorkerComplianceDocDtl() called with an unsaved WorkerComplianceDocDtl");
		}
		//Find the existing doc.
		boolean found = false;
		WorkerComplianceDocDtl compDtlToDelete = null;
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		for (WorkerComplianceDocDtl existingCompDtl : workerComplianceDocDtls) {
			if (existingCompDtl.getId() == workerComplianceDocDtl.getId()) {
				//Can't remove from within the loop, store reference.
				compDtlToDelete = existingCompDtl;
				found = true;
				break;
			}
		}

		if (found) {
			worker.removeWorkerComplianceDocDtl(compDtlToDelete);
			entityService.makeTransient(compDtlToDelete);
		} else {
			throw new ServiceFailureException("Unable to find existing WorkerComplianceDetail to delete");
		}

		return worker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent#findWorkerComplianceDocDtl(com.intuit
	 * .sb.payroll.entity.domain.Identifiable)
	 */
	public WorkerComplianceDocDtl findWorkerComplianceDocDtl(Identifiable identifiable) throws ServiceFailureException {
		// Validations done by upper layer
		try {
			WorkerComplianceDocDtl workerComplianceDtl = entityService.findByIdentifiable(WorkerComplianceDocDtl.class, identifiable);
			return workerComplianceDtl;
		} catch (Exception ex) {
			String mesg = MessageFormat.format("Unable to find WorkerComplianceDtl: {0}", ExceptionUtils
					.throwableMessage(ex));
			throw new ServiceFailureException(mesg);
		}
	}

	public List<WorkerComplianceDocDtl> getCurrentAndDerivedWithholdings(Worker worker) throws ServiceFailureException,
			WorkflowException, ValidationException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		// If taxes etc have not been set up, we can't go down this road.
		if (!WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.SetupWithholding, worker)) {
			ExceptionUtils.generateWorkflowExceptionFromRequiredData(WorkerSetupRules.getMissingDataForWorkflow(
					WorkerWorkflowType.SetupWithholding, worker));
		}
		// Existing Withholding Data
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = worker.getWorkerComplianceDocDtls();
		List<WorkerComplianceDocDtl> results = new ArrayList<WorkerComplianceDocDtl>();
 		//Add existing
		for (WorkerComplianceDocDtl workerComplianceDocDtl : workerComplianceDocDtls) {
			//FIXME [model] remove when we can distinguish Withholdings from other docs.
			if (workerComplianceDocDtl.getDocId() == ComplianceDocIdEnum.W4IRS || 
						workerComplianceDocDtl.getDocId() == ComplianceDocIdEnum.DE4) {
					results.add(workerComplianceDocDtl);
			}
		}

		//Required Withholding Data
		final List<ComplianceDoc> requiredComplianceDocs = 
			WorkerTaxWithholdingRules.getWithholdingFormsApplicableToWorker(worker);

		// Merge, ie, create the docs that don't exist.
		for (ComplianceDoc requiredComplianceDoc : requiredComplianceDocs) {
			ComplianceDocIdEnum docType = requiredComplianceDoc.getDocId();
			boolean haveRequiredDoc = false;
			for (WorkerComplianceDocDtl workerComplianceDocDtl : workerComplianceDocDtls) {
				if (workerComplianceDocDtl.getDocId() == docType) {
					haveRequiredDoc = true;
					break;
				}
			}
			if (!haveRequiredDoc) {
				WorkerComplianceDocDtl newWithholding = this.createWorkerComplianceDocDtl(docType);
				//Add new one
				results.add(newWithholding);
			}
		}

		// Derive the State from the Fed if needed on the results
		WorkerTaxWithholdingRules.deriveStateWithholdingFromFederalWithholding(results);

		return results;
	}
}
