package com.intuit.sb.payroll.capability.taxes.withholding.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.ComplianceDoc;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sb.payroll.entity.taxauthority.LawEAO;

/**
 * This class provides Rules related to WorkerTaxWithholding. It knows how to navigate the Data structures in the Domain
 * model that relate to Withholdings, and it also knows when to delegate policy decisions to an external Rules engine.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class WorkerTaxWithholdingRules {

	/**
	 * Returns the Withholding forms relevant to the Employee. This is derived from the Taxes that the employee is
	 * subject to.
	 * 
	 * It is expected that the caller will create EmployeeWithholdingData instances related to the forms returned.
	 * 
	 * Precondition: This method assumes it is within the context of a transaction.
	 * 
	 * @param worker The employee to get the forms for.
	 * @return The relevant forms for this employee.
	 */
	public static List<ComplianceDoc> getWithholdingFormsApplicableToWorker(Worker worker) throws ServiceFailureException, ValidationException {
		PayrollParamValidator.checkIsNotNull(worker, "Worker");
		List<ComplianceDoc> withholdingForms = new ArrayList<ComplianceDoc>();
		Set<TaxDtl> taxDtls = worker.getTaxDtls();
		LawEAO lawEAO = ServiceLocator.getInstance().getService(LawEAO.JNDI_NAME);
		for (TaxDtl taxDetail : taxDtls) {
			String lawId = taxDetail.getPayrollItem().getLawId().getValue();
			Law law = lawEAO.findByLawId(lawId);
			if (law != null) {
				Set<ComplianceDoc> withholdingFormsFromLaw = law.getComplianceDocs();
				for (ComplianceDoc doc : withholdingFormsFromLaw) {
					//FIXME [model] we need to distinguish between withholding and other types of docs, 
					// then we can get rid of this hard-coded stuff.
					if (doc.getDocId() == ComplianceDocIdEnum.DE4 
							|| doc.getDocId() == ComplianceDocIdEnum.W4IRS) {
						withholdingForms.add(doc);
					}
				}
			}
		}
		return withholdingForms;
	}

	/**
	 * Sets the withholding attributes on a state form from a federal form.
	 * 
	 * FIXME [th] Only supports CA right now. Should have pluggable handlers for different forms.
	 * 
	 * @param existingComplianceData the worker's existing ComplianceData.
	 * @return List<EmployeeWithholdingData> - List into which new Withholding Data, created if any, are returned
	 * @throws ServiceFailureException
	 */
	public static Collection<WorkerComplianceDocDtl> deriveStateWithholdingFromFederalWithholding(
			Collection<WorkerComplianceDocDtl> existingComplianceData) {
		W4Detail w4Detail = null;
		DE4Detail de4Detail = null;
		for (WorkerComplianceDocDtl workerComplianceDocDtl : existingComplianceData) {
			if (workerComplianceDocDtl.getDocId() == ComplianceDocIdEnum.W4IRS) {
				w4Detail = (W4Detail) workerComplianceDocDtl;
			} else if (workerComplianceDocDtl.getDocId() == ComplianceDocIdEnum.DE4) {
				de4Detail = (DE4Detail)  workerComplianceDocDtl;
			}
		}
		if (null == w4Detail || null == de4Detail) {
			throw new IllegalStateException("Developer Error - withholding derivation called without required forms.");
		}
		
		//Leave it alone if it is not derived.
		if (de4Detail.isDerived()) {
			de4Detail.setAdditionalAmount( w4Detail.getAdditionalAmount());
			de4Detail.setTaxExempt( w4Detail.isTaxExempt());
			de4Detail.setNoOfAllowances( w4Detail.getNoOfAllowances());
			de4Detail.setNoOfDeductions(new Integer(0));
			if (w4Detail.getWithholdingStatus() == W4WithholdingStatusEnum.SINGLE ||
					w4Detail.getWithholdingStatus() == W4WithholdingStatusEnum.MARRIEDSINGLERATE) {
				de4Detail.setWithholdingStatus( DE4WithholdingStatusEnum.SINGLE);
			}
			if (w4Detail.getWithholdingStatus() == W4WithholdingStatusEnum.MARRIED) {
				de4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.MARRIEDONEINCOME);
			}
		}
		return existingComplianceData;
	}

}
