package com.intuit.sb.payroll.capability.worker;

import java.util.Date;
import java.util.List;

import javax.ejb.Local;

import com.intuit.payroll.types.PayrollException;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

@Local
public interface WorkerComponent {
	
	@SuppressWarnings("nls")
	public static final String JNDI_NAME =  "WorkerComponent/local";

    /**
     * Fetch worker from the repository based on the worker's unique Id
     * 
     * @param workerUniqueId string representation of worker's unique Id
     * 
     * @return Worker Entity.
     * 
     * @throws ValidationException if WorkerId provided is null
     * @throws ServiceFailureException if no worker exists for the unique Id provided
     */
    public Worker findWorker(Long workerUniqueId) throws ServiceFailureException, ValidationException;
    
    /**
     * Returns a list of all the Workers for the current company.
     * 
     * @return a {@link List} of {@link Worker}s
     * @throws ServiceFailureException
     */
    public List<Worker> findAllWorkers() throws ServiceFailureException;

    /**
     * Returns a list of the workers in various states as defined by the {@link FindWorkersFilter}.
     * 
     * @param filter
     * @return the list of workers.
     * @throws ServiceFailureException
     */
    public List<Worker> findWorkersByFilter(FindWorkersFilter filter) throws ServiceFailureException;
    
    /**
     * Returns a list of workers who are assigned to a particular {@link WorkLocation}.
     * 
     * @param workLocationId
     * @return
     * @throws ServiceFailureException
     */
    public List<Worker> findWorkersByWorkLocation(Long workLocationId) throws ServiceFailureException;
    
    /**
     * Adds a worker to the current company and persists it's current state.
     * 
     * NOTE: adds a default W4 to the worker (this moves to the app later.)
     * 
     * @param worker
     * @return
     * @throws WorkflowException
     * @throws ValidationException
     * @throws ServiceFailureException
     */
    public Worker addWorker(Worker worker) throws WorkflowException, ValidationException, ServiceFailureException;

    /**
     * Updates the state of the worker, makes it managed (if it is out of session).
     * 
     * @param worker Worker to be saved or updated
     * @return Saved worker
     * @throws ValidationException
     * @throws ServiceFailureException
     * @throws WorkflowException 
     */
    public Worker updateWorker(Worker worker) throws WorkflowException, ValidationException, ServiceFailureException;
    
    /**
     * Deletes a worker from the system.  This can only occur if there are no transactions associated with this worker.
     * 
     * @param worker
     * @throws ValidationException if there are Paychecks for this Worker.
     * @throws ServiceFailureException
     */
    public void deleteWorker(Worker worker) throws ValidationException, ServiceFailureException;
    
    /**
     * Terminates the employment of a Worker on the date specified.
     * 
     * @param worker
     * @param terminationDate
     * @return
     * @throws WorkflowException
     * @throws ValidationException
     * @throws ServiceFailureException
     */
    public Worker terminateWorker(Worker worker, Date terminationDate) throws WorkflowException, ValidationException, ServiceFailureException;

    /**
     * If the Worker was previously employed, the system can bring them back, set their status to Employed again and
     * reset the hire date (saving off the prior employment period).  All state evaluation will be re-executed to ensure
     * the Worker is up-to-date.
     * 
     * @param workerId
     * @param hireDate
     * @return
     * @throws WorkflowException
     * @throws ValidationException
     * @throws ServiceFailureException
     */
    public Worker hireExistingWorker(Long workerId, Date hireDate) throws WorkflowException, ValidationException, ServiceFailureException;
    
    /**
     * @param worker
     * @return boolean value of whether worker is ready for pay
     */
    public boolean isWorkerReadyToPay(Worker worker);

    /**
     * Assigns the given {@link WorkLocation} to a {@link Worker}.
     * 
     * @param workLocation
     * @param worker
     * @return the latest copy of the Worker
     * @throws WorkflowException 
     * @throws PayrollValidationException
     */
	public Worker assignWorkLocationToWorker(WorkLocation workLocation, Worker worker) throws WorkflowException, ValidationException, ServiceFailureException;

    /**
     * Fetch worker from the repository based on the worker's unique Id and Version
     * 
     * @param identifiable {@link Identifiable} representation of worker's unique Id and version
     * 
     * @return Worker worker.
     * 
     * @throws PayrollException if WorkerId provided is null
     * @throws PayrollEntityNotFoundException if no worker exists for the unique Id provided
     * @throws PayrollServiceFailureException for another other exception in accessing or fetching entity from the
     *                                  repository 
     */
	public Worker findWorkerByIdentifiable(Identifiable identifiable) throws ServiceFailureException;

	/**
	 * This component is interested in changes to the Employer as it may affect Worker state and Tasks.
	 * 
	 * @param eventType
	 * @param employer
	 * @throws ServiceFailureException
	 */
	public void handleEmployerEvent(WorkflowEventType eventType, Employer employer) throws ServiceFailureException;
	
	/**
	 * This component is interested in changes to the Payroll Schedule as it may affect Worker Tasks.
	 * 
	 * @param workflowEventType
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	public void handlePayrollScheduleEvent(WorkflowEventType workflowEventType) throws ServiceFailureException, ValidationException;
	
	/**
	 * This component is interested in changes to the Payroll Run as it may affect Worker Tasks.
	 * 
	 * @param workflowEventType
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	public void handlePayrollRunEvent(WorkflowEventType workflowEventType) throws ServiceFailureException, ValidationException;
}
