package com.intuit.sb.payroll.capability.worker.impl;

import java.util.Date;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PersonName;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.model.TaskTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * Handles Task management for the WorkerComponent.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class WorkerComponentTasks {
	public static final String SETUP_EMPLOYEE_CLASS = "com.intuit.sb.payroll.worker";
	public static final String SETUP_FIRST_EMPLOYEE_TASK = "firstWorkerSetupTask";
	public static final String WORKER_PAYROLL_READY_TASK = "workerPayrollReadyTask";
	public static final String WORKER_WITHHOLDING_TASK = "workerWithHoldingTask";

	public static void createAddEmployeeTask(TaskManager taskManager, Employer employer) throws ServiceFailureException {
		
		createWorkerSetupTask(taskManager, employer, SETUP_FIRST_EMPLOYEE_TASK, null);
	}
	/**
	 * Manages the "Ready for Pay" task state based on the worker state.
	 * 
	 * @param taskManager
	 * @param tasks
	 * @param worker
	 * @throws ServiceFailureException
	 * @throws TaskException
	 */
	public static void handleWorkerChangedEvent(TaskManager taskManager, List<Task> tasks, Worker worker, Date payrollRunDate)
			throws ServiceFailureException {
		Task workerPayReadyTask = getWorkerSetupTask(tasks, WORKER_PAYROLL_READY_TASK, worker);

		// Create a Worker Payroll Ready task
		if (workerPayReadyTask == null) {
			workerPayReadyTask = createWorkerSetupTask(taskManager, worker, WORKER_PAYROLL_READY_TASK, payrollRunDate);
		}

		if (worker.isReadyForPay()) {
			TaskUtils.closeTask(taskManager, workerPayReadyTask);
		} else {
			// Re-open the task
			if (workerPayReadyTask.getStatus().equals(TaskStatusEnum.COMPLETED)) {
				workerPayReadyTask = taskManager.reOpenTask(workerPayReadyTask);
			}
		}
	}

	/**
	 * Invoked when there is a Worker added to the system.  Determines if any tasks need to be updated.
	 * 
	 * @param taskManager
	 * @param tasks
	 * @param worker
	 * @throws TaskException
	 * @throws ServiceFailureException
	 */
	public static void handleAddFirstEmployeeEvent(TaskManager taskManager, Worker worker)
			throws ServiceFailureException {
		List<Task> tasks = taskManager.findCurrentActiveTasks();
		EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
		Employer employer = employerComponent.getEmployer();
		Task addFirstEmpTask = getWorkerSetupTask(tasks, SETUP_FIRST_EMPLOYEE_TASK, employer);

		if (addFirstEmpTask != null) {
			TaskUtils.closeTask(taskManager, addFirstEmpTask);
		}
	}

	/**
	 * Determines if any tasks need to change state as a result of changes in Worker Withholdings.
	 * 
	 * @param taskManager
	 * @param tasks
	 * @param worker
	 * @throws ServiceFailureException
	 * @throws TaskException
	 */
	public static void handleWorkerWithHoldingEvent(TaskManager taskManager, List<Task> tasks, Worker worker)
			throws ServiceFailureException {
		Task workerWithHoldingTask = getWorkerSetupTask(tasks, WORKER_WITHHOLDING_TASK, worker);

		// Create a worker withholding task is it does not exist
		if (workerWithHoldingTask == null) {
			workerWithHoldingTask = createWorkerSetupTask(taskManager, worker, WORKER_WITHHOLDING_TASK, null);
		}

		if (isWorkerWithHoldingSaveRequired(worker)) {
			// Re-open the task or mark the task as not completed
			if (workerWithHoldingTask.getStatus().equals(TaskStatusEnum.COMPLETED)) {
				taskManager.reOpenTask(workerWithHoldingTask);
			}
		} else {
			// close the task if not already done
			TaskUtils.closeTask(taskManager, workerWithHoldingTask);
		}
	}

	private static boolean isWorkerWithHoldingSaveRequired(Worker entity) {
		Set<WorkerComplianceDocDtl> compDataColl = entity.getWorkerComplianceDocDtls();
		if (compDataColl.size() == 0) {
			return true;
		} else {
			for (WorkerComplianceDocDtl employeeWithholdingData : compDataColl) {
				if ((employeeWithholdingData.getDocId().equals(ComplianceDocIdEnum.W4IRS))
						&& (employeeWithholdingData.isDerived()))
					return true;
			}
		}
		return false;
	}

	private static Task createWorkerSetupTask(TaskManager taskManager, Identifiable entity, String taskname, Date deadline) {
		Task workerPayrollReadyTask = new Task();
		workerPayrollReadyTask.setName(taskname);
		if ((entity != null) && (entity instanceof Worker)) {
			PersonName personName = ((Worker) entity).getPersonName();
			if (personName != null) {
				workerPayrollReadyTask.setDescription(personName.getFirstName());
			}
		}

		workerPayrollReadyTask.setTaskClass(SETUP_EMPLOYEE_CLASS);
		workerPayrollReadyTask.setType(TaskTypeEnum.SETUP);
		workerPayrollReadyTask.setCategory("Setup");
		workerPayrollReadyTask.setUserManagable(false);
		workerPayrollReadyTask.setDeadline(deadline);
		
		Long payload = null;
		if (entity != null) {
			payload = new Long(entity.getId());
		} else {
			payload = new Long(0);
		}

		workerPayrollReadyTask.setPayload(payload);
		workerPayrollReadyTask = taskManager.addTask(workerPayrollReadyTask);
		return workerPayrollReadyTask;
	}

	private static Task getWorkerSetupTask(List<Task> alltasks, String name, Identifiable worker)
			throws ServiceFailureException {
		List<Task> tasks = TaskUtils.getTaskByTaskName(alltasks, name, worker);
		Task task = null;
		if (tasks.size() == 1) {
			task = tasks.get(0);

		} else if (tasks.size() > 1) {
			throw new ServiceFailureException("More than one Task of " + name + " found for a worker");
		}
		return task;
	}
	
	public static void removeTasksForWorker(TaskManager taskManager, Worker worker) {
		TaskUtils.removeAllWorkerTasks(worker.getId());
	}
	
	public static void updateWorkerTasks(TaskManager taskManager, Date deadline) {
		List<Task> tasks = taskManager.findTasksByClass(SETUP_EMPLOYEE_CLASS);
		
		for (Task task : tasks)
		{
			if(task.getName().equals(WORKER_PAYROLL_READY_TASK)) {
				task.setDeadline(deadline);
			}
		}
	}

}
