package com.intuit.sb.payroll.capability.worker.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerWorkflowType;
import com.intuit.sb.payroll.capability.worker.impl.evaluator.WorkerEvaluator;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.RequiredData;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Encapsulates US Worker Setup requirements.
 * 
 * $Author: $ 
 * $Revision: $
 * $DateTime: $
 *
 */
public class WorkerSetupRules {

	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(WorkerSetupRules.class);
	private static final String LIVE_LOCATION_KEY = "live.location.key";
	private static final String WORK_LOCATION_KEY = "work.location.key";
	private static final String WITHHOLDING_INFO_KEY = "federal.withholding.info.key";
	private static final String STATE_WITHHOLDING_INFO_KEY = "state.withholding.info.key";
	private static final String SSN_ID_KEY = "ssn.id.key";
	private static final String HIRE_DATE_KEY = "hire.date.key";
	private static final String COMPENSATION_KEY = "compensation.key";
	private static final String TAX_SETTINGS_KEY = "tax.settings.key";

	public static final RequiredData liveLocation = new RequiredData(LIVE_LOCATION_KEY, "State", "TaxResidenceAddress",
			"Worker Tax Residence");
	public static final RequiredData workLocation = new RequiredData(WORK_LOCATION_KEY, "State", "WorkLocation",
			"Employer Work Location");
	public static final RequiredData withholdingInfo = new RequiredData(WITHHOLDING_INFO_KEY, "FederalWithholdingInformation",
			"FederalWithholdingInformation", "Federal Withholding Information");
	public static final RequiredData stateWithholdingInfo = new RequiredData(STATE_WITHHOLDING_INFO_KEY, "StateWithholdingInformation",
			"StateWithholdingInformation", "State Withholding Information");
	public static final RequiredData worker_ssn = new RequiredData(SSN_ID_KEY, "Value", "PersonTaxId",
	 		"Worker Social Security Number");
	public static final RequiredData hireDate = new RequiredData(HIRE_DATE_KEY, "HireDate", "Worker",
			"Worker Hire Date");
	public static final RequiredData compensation = new RequiredData(COMPENSATION_KEY, "CompensationDtls", "Worker",
			"Compensation Item on the Worker");
	
	public static final RequiredData taxSettings = new RequiredData(TAX_SETTINGS_KEY, "TaxDtls", "Worker",
			"Tax settings on the worker");

	/**
	 * Holds the RequiredData for setting up withholding.
	 */
	private static List<RequiredData> withholdingRequiredData = new ArrayList<RequiredData>();

	static {
		withholdingRequiredData.add(liveLocation);
		withholdingRequiredData.add(workLocation);
		withholdingRequiredData.add(taxSettings);
	}
	
	/**
	 * Holds the RequiredData for deriving State withholding from Federal.
	 */
	private static List<RequiredData> deriveWithholdingRequiredData = new ArrayList<RequiredData>();

	static {
		deriveWithholdingRequiredData.add(liveLocation);
		deriveWithholdingRequiredData.add(workLocation);
		deriveWithholdingRequiredData.add(withholdingInfo);
		deriveWithholdingRequiredData.add(taxSettings);
	}

	/**
	 * Holds the RequiredData for determining applicable taxes.
	 */
	private static List<RequiredData> determineTaxesRequiredData = new ArrayList<RequiredData>();

	static {
		determineTaxesRequiredData.add(liveLocation);
		determineTaxesRequiredData.add(workLocation);
	}
	
	/**
	 * Holds the RequiredData for determining applicable taxes.
	 */
	private static List<RequiredData> ptsTaxReadyRequiredData = new ArrayList<RequiredData>();

	static {
		ptsTaxReadyRequiredData.add(worker_ssn);
		ptsTaxReadyRequiredData.add(liveLocation);
		ptsTaxReadyRequiredData.add(workLocation);
		ptsTaxReadyRequiredData.add(hireDate);
		ptsTaxReadyRequiredData.add(compensation);
		ptsTaxReadyRequiredData.add(taxSettings);
	}

	/**
	 * Holds the RequiredData for calculating paychecks.
	 */
	private static List<RequiredData> payrollRequiredData = new ArrayList<RequiredData>();

	static {
		payrollRequiredData.add(liveLocation);
		payrollRequiredData.add(workLocation);
		payrollRequiredData.add(hireDate);
		payrollRequiredData.add(compensation);
		payrollRequiredData.add(taxSettings);
	}

	/**
	 * Returns the full list of {@link RequiredData} for a given Workflow.
	 * @param workflow
	 * @return the list of {@link RequiredData}
	 */
	public static List<RequiredData> getRequiredDataForWorkflow(WorkerWorkflowType workflow) {
		switch (workflow) {
		case SetupWithholding:
			return withholdingRequiredData;
		case DeriveWithholdings:
			return deriveWithholdingRequiredData;
		case DetermineTaxes:
			return determineTaxesRequiredData;
		case Payroll:
			return payrollRequiredData;
		case PTSTaxReady:
			return ptsTaxReadyRequiredData;
		default:
			return new ArrayList<RequiredData>();
		}
	}

	/**
	 * Given a Worker, this method will return the list of {@link RequiredData} that is missing
	 * for the {@link WorkerWorkflowType}.
	 * 
	 * @param workflow
	 * @param worker
	 * @return
	 * @throws ValidationException
	 * @throws ServiceFailureException 
	 */
	public static List<RequiredData> getMissingDataForWorkflow(WorkerWorkflowType workflow, Worker worker)
			throws ValidationException, ServiceFailureException 
	{
		List<RequiredData> allData = getRequiredDataForWorkflow(workflow);
		WorkerEvaluator evaluator = WorkerEvaluatorFactory.getWorkerEvaluator(LocaleGuru.getInstance().getOfferingLocale());
		List<RequiredData> completedData = evaluator.getCompletedFields(worker, allData);
		return diff(completedData, allData);
	}

	/**
	 * Internal method to return the difference between two lists.
	 * @param completedData
	 * @param allData
	 * @return
	 */
	private static List<RequiredData> diff(List<RequiredData> completedData, List<RequiredData> allData) {
		List<RequiredData> results = new ArrayList<RequiredData>();
		for (RequiredData requiredData : allData) {
			if (!completedData.contains(requiredData)) {
				results.add(requiredData);
			}
		}
		return results;
	}
	
	/**
	 * Called from WorkerComponentBean before saving, perhaps on load as well?
	 * 
	 * Sets the correct state flags on the worker itself.
	 * 
	 * @param worker
	 */
	public static void setWorkerStateProperties(Worker worker) {
		determineEmploymentStatus(worker);
		if(worker.getEmploymentStatus().equals(EmploymentStatusEnum.TERMINATED)) {
			worker.setReadyForPay(false);
			worker.setReadyForTax(false);
		}
		else {
			worker.setReadyForPay(isReadyForWorkflow(WorkerWorkflowType.Payroll, worker));
			worker.setReadyForTax(isReadyForWorkflow(WorkerWorkflowType.DetermineTaxes, worker));
		}
		
	}

	/**
	 * Given a worker, indicates if it is ready for a given Workflow.
	 * @param workflow
	 * @param worker
	 * @return
	 */
	public static boolean isReadyForWorkflow(WorkerWorkflowType workflow, Worker worker) {
		List<RequiredData> leftovers;
		try {
			leftovers = getMissingDataForWorkflow(workflow, worker);
			if (leftovers == null) {
				throw new AssertionError("Don't return null");
			}
			if (leftovers.size() > 0) {
				return false;
			} else {
				return true;
			}
		} catch (ValidationException e) {
			sLogger.debug(e);
			return false;
		} catch (ServiceFailureException se){
			sLogger.debug(se);
			return false;
		}
	}

	/**
	 * [WIP] Employment status determination.
	 * @param worker
	 * @return
	 */
	private static void determineEmploymentStatus(Worker worker) {
		if (worker.getEmploymentStatus() == EmploymentStatusEnum.NOTSET) {
			DateRange employmentPeriod = worker.getEmploymentPeriod();
			if (employmentPeriod != null) {
				Date startDate = employmentPeriod.getStartDate();
				if (startDate != null && DateUtil.isBeforeOrOnSameDay(startDate, DateUtil.getNowDate())) {
					worker.setEmploymentStatus(EmploymentStatusEnum.EMPLOYED);
					worker.setActive(true);
				}
			}
		}
	}
}
