package com.intuit.sb.payroll.capability.worker.impl;

import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.impl.evaluator.WorkerEvaluator;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.common.validation.AbstractEntityDataValidator;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Validates fields in the EmployeeWithholdingData entity that have values. Fields with no values are ignored.
 * 
 * This class does not ensure data integrity or completeness.
 * 
 * @see WorkerEvaluator 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
@SuppressWarnings("nls")
public class WorkerValidator extends AbstractEntityDataValidator<Worker> {

	private static WorkerValidator sInstance = new WorkerValidator();

	private WorkerValidator() {
	}

	public static WorkerValidator getInstance() {
		return sInstance;
	}

	@Override
	public void validateEntityCollection(List<Worker> workerCollection) throws ValidationException {
		for (Worker worker : workerCollection) {
			validateEntity(worker);
		}
	}

	@Override
	public void validateEntity(Worker worker) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(worker.getPersonName(), "worker.personName");
		String firstName = worker.getPersonName().getFirstName();
		PayrollParamValidator.checkIsNotNullOrBlankString(firstName, "worker.personname.firstname");
		//FIXME: what about cases when employment period is unintentionally null?
		if(worker.getEmploymentPeriod() != null && worker.getEmploymentPeriod().getStartDate() != null) {
			Date startDate = worker.getEmploymentPeriod().getStartDate();
			checkHireDateValidity(startDate);
		}
	}
	
	/**
	 * Ensures that the date is within the range i.e. > 1940 and < 1yr + current date.
	 * These checks are currently used for hire date and next pay period ends date.
	 * @param date
	 * @throws ValidationException
	 */
	public static void checkHireDateValidity(Date date) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(date, "start date");
		int minYear = 1941;
		int defaultDay = 1;		
		Calendar now = LocaleGuru.getInstance().getCalendar();
		now.add(Calendar.YEAR, 1);		
		
		Date dateNotLessThan = DateUtil.createCalendar(minYear, Calendar.JANUARY, defaultDay).getTime();
		Date oneYearFromNow = now.getTime();
		if (!DateUtil.isInPeriod(date, dateNotLessThan, oneYearFromNow)) {
			DateFormat format = LocaleGuru.getInstance().getDateFormat();
			String formattedDate = format.format(date);
			//FIXME [I18N] resource translation
			String mesg = MessageFormat.format("Hire Date {0} cannot be before 1941 or after a year from today.", formattedDate);
			throw new ValidationException(mesg);
		}
	}

}
