package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.workerpayments.AdjustmentRequest;
import com.intuit.sb.payroll.capability.workerpayments.AdjustmentResponse;
import com.intuit.sb.payroll.capability.workerpayments.AdjustmentTrigger;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculator;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.DatumType1Enum;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerAdjLineItem;
import com.intuit.sb.payroll.entity.model.EmployerAdjustment;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PayrollItemDtlPtdDatum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerAdjustment;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.entity.workerpayments.CompanyAdjustmentTotals;
import com.intuit.sb.payroll.entity.workerpayments.WorkerTotals;

public class PayrollAdjustmentsController {

	private PayrollRun payrollRun;
	private Employer employer;
	private List<Worker> workerList;
	private PaycheckCalculator calcEngine;
	
	public PayrollAdjustmentsController(Employer employer, List<Worker> workerList, PayrollRun payrollRun, PaycheckCalculator paycheckCalculator) {
		this.employer = employer;
		this.workerList = workerList;
		this.payrollRun = payrollRun;
		this.calcEngine = paycheckCalculator;
	}

	/**
	 * Given a list of workers who had their checks voided, figure out the tax adjustments for them 
	 * and the company.
	 * 
	 * @param affectedWorkers
	 * @throws ServiceFailureException
	 */
	public void adjustForVoid(List<Worker> affectedWorkers) throws ServiceFailureException {
		
		Set<EmployerAdjustment> priorAdjustments = employer.getEmployerAdjustmentsForYear(DateUtil.getYearFromDate(payrollRun.getPayDate()));
		CompanyAdjustmentTotals companyTotals = getCompanyTotals(priorAdjustments);
		//This gets populated when we go through the workers in the next method
		Map<String, PayrollItem> payrollItemMap = new HashMap<String, PayrollItem>();
		WorkerTotals workerTotals = getWorkerTotals(workerList, payrollItemMap);
		
		//Contains all the junk we need to calculated the adjustments.  No data access is done within the adapter.
		AdjustmentRequest adjustmentRequest = new AdjustmentRequest();
		adjustmentRequest.setAdjustmentTrigger(AdjustmentTrigger.PAYCHECK_VOID);
		adjustmentRequest.setCompanyTotals(companyTotals);
		adjustmentRequest.setEmployer(employer);
		adjustmentRequest.setPayrollRun(payrollRun);
		adjustmentRequest.setWorkerTotals(workerTotals);
		adjustmentRequest.setPayrollItemMap(payrollItemMap);
		adjustmentRequest.setAffectedWorkers(affectedWorkers);
		
		AdjustmentResponse response = calcEngine.calculateLiabilityAdjustments(adjustmentRequest);
		Set<EmployerAdjustment> employerAdjustments = response.getEmployerAdjustments();
		if (employerAdjustments.size() > 0) {
			//Set the adjustments on the payrollRun and Employer
			payrollRun.setEmployerAdjustments(employerAdjustments);
			fixupEmployerAdjustmentAssocation(employer, employerAdjustments);
		}
		Set<WorkerAdjustment> workerAdjustments = response.getWorkerAdjustments();
		if (workerAdjustments.size() > 0) {
			payrollRun.setWorkerAdjustments(workerAdjustments);
			//FIXME set up the correct relationships with worker?

		}
	}

	public void adjustForRounding() throws ServiceFailureException {

		Set<EmployerAdjustment> priorAdjustments = employer.getEmployerAdjustmentsForYear(DateUtil.getYearFromDate(payrollRun.getPayDate()));
		CompanyAdjustmentTotals companyTotals = getCompanyTotals(priorAdjustments);
		//This gets populated when we go through the workers in the next method
		Map<String, PayrollItem> payrollItemMap = new HashMap<String, PayrollItem>();
		WorkerTotals workerTotals = getWorkerTotals(workerList, payrollItemMap);
		
		//Contains all the junk we need to calculated the adjustments.  No data access is done within the adapter.
		AdjustmentRequest adjustmentRequest = new AdjustmentRequest();
		adjustmentRequest.setAdjustmentTrigger(AdjustmentTrigger.PAYROLL_RUN);
		adjustmentRequest.setCompanyTotals(companyTotals);
		adjustmentRequest.setEmployer(employer);
		adjustmentRequest.setPayrollRun(payrollRun);
		adjustmentRequest.setWorkerTotals(workerTotals);
		adjustmentRequest.setPayrollItemMap(payrollItemMap);
		adjustmentRequest.setAffectedWorkers(null);
		
		AdjustmentResponse response = calcEngine.calculateLiabilityAdjustments(adjustmentRequest);
		Set<EmployerAdjustment> employerAdjustments = response.getEmployerAdjustments();
		if (employerAdjustments.size() > 0) {
			//Set the adjustments on the payrollRun and Employer
			payrollRun.setEmployerAdjustments(employerAdjustments);
			fixupEmployerAdjustmentAssocation(employer, employerAdjustments);
		}
		
	}
	
	private WorkerTotals getWorkerTotals(List<Worker> workers, Map<String, PayrollItem> payrollItemMap) {
		//Sum up the totals in a Datum container for each lawId.  Each Law will have a Map of Qtr-based totals.
		final Map<String, Map<String, PayrollItemDtlPtdDatum>> totalsByLawIdMap = new HashMap<String, Map<String, PayrollItemDtlPtdDatum>>();
		
		//The first dimension is the worker - each one has totals for the various pay items that they own.
		for (Worker worker : workers) {
			for (PayrollItemDtl payrollItemDtl : worker.getPayrollItemDtls()) {
				//We work in the context of a Law - the second dimension (law:payitem - 1:1)
				final String law = payrollItemDtl.getPayrollItem().getLawId().getValue();
				//ptdDatums are created for each Qtr - they hold WageBase, ISST and TotalAmt information
				for (PayrollItemDtlPtdDatum workerPayItemPtdDatum : payrollItemDtl.getPtdDatums()) {
					//Add the values here to the running total if they are in range
					//The Qtr (or range) is the third dimension
					final PtdDatumType workerPayItemPtdDatumDatum = workerPayItemPtdDatum.getDatum();
					//FIXME PTOs don't have a period.
					if (workerPayItemPtdDatumDatum.getPeriod() != null) {
						Map<String, PayrollItemDtlPtdDatum> lawScopedPtdRunningTotalMap = totalsByLawIdMap.get(law);
						if (null == lawScopedPtdRunningTotalMap) {
							lawScopedPtdRunningTotalMap = new HashMap<String, PayrollItemDtlPtdDatum>();
							totalsByLawIdMap.put(law, lawScopedPtdRunningTotalMap);
							//We keep a map of law to payroll items for processing downstream.
							payrollItemMap.put(law, payrollItemDtl.getPayrollItem());
						}
						//This key is the name of the qtr.
						final String quarterName = workerPayItemPtdDatumDatum.getPtdType().toString();
						//See if there is a Qtrly entry for this law.
						PayrollItemDtlPtdDatum runningTotalForLawForQtr = lawScopedPtdRunningTotalMap.get(quarterName);
						if (null == runningTotalForLawForQtr) {
							//Create an empty entry into the running totals for that law+qtr
							runningTotalForLawForQtr = new PayrollItemDtlPtdDatum();
							PtdDatumType runningTotalDatum = new PtdDatumType();
							runningTotalDatum.setDatumType(workerPayItemPtdDatumDatum.getDatumType());
							runningTotalDatum.setPeriod(PtdUtils.getDateRangeForPeriodType(workerPayItemPtdDatumDatum.getPtdType(), 
									DateUtil.getYearFromDate(workerPayItemPtdDatumDatum.getPeriod().getStartDate())));
							runningTotalDatum.setPtdType(workerPayItemPtdDatumDatum.getPtdType());
							runningTotalForLawForQtr.setDatum(runningTotalDatum);
							lawScopedPtdRunningTotalMap.put(quarterName, runningTotalForLawForQtr);
						}
						//
						addExistingToRunningTotal(runningTotalForLawForQtr, workerPayItemPtdDatum);
					}
				}
			}
		}
		
		//Populate the WorkerTotals
		WorkerTotals workerTotals = new WorkerTotals();
		workerTotals.setQtrTotalsByLawId(totalsByLawIdMap);
		return workerTotals;
	}
	
	/**
	 * Adds a datum to the running total datum.
	 * 
	 * @param runningTotalForLawForQtr
	 * @param ptdDatum
	 */
	private void addExistingToRunningTotal(PayrollItemDtlPtdDatum runningTotalForLawForQtr, PayrollItemDtlPtdDatum ptdDatum) {
		PtdDatumType runningTotalDatum = runningTotalForLawForQtr.getDatum();
		PtdDatumType ptdDatumDatum = ptdDatum.getDatum();
		runningTotalDatum.setMoney(runningTotalDatum.getMoneyValue().add(ptdDatumDatum.getMoneyValue()));
		runningTotalDatum.setIncomeSubjectToTax((runningTotalDatum.getIncomeSubjectToTax().add(ptdDatumDatum.getIncomeSubjectToTax())));
		runningTotalDatum.setWageBase(runningTotalDatum.getWageBaseValue().add(ptdDatumDatum.getWageBaseValue()));	
	}
	
	
	private CompanyAdjustmentTotals getCompanyTotals(Set<EmployerAdjustment> employerAdjustmentsInRange) {
		CompanyAdjustmentTotals totals = new CompanyAdjustmentTotals();
		Map<String, Set<PtdDatumType>> totalsByLawIdMap = totals.getTotalsByLawIdMap();
		for (EmployerAdjustment employerAdjustment : employerAdjustmentsInRange) {
			Date adjustementEffectiveDate = employerAdjustment.getEffectiveDate();
			// What Qtr does this fall into?
			DateRange txDateRange = PtdUtils.getQtrRangeForDate(adjustementEffectiveDate);
			
			Set<EmployerAdjLineItem> employerAdjLineItems = employerAdjustment.getEmployerAdjLineItems();
			for (EmployerAdjLineItem adjLineItem : employerAdjLineItems) {
				String law = adjLineItem.getLawId().getValue();
				Set<PtdDatumType> runningTotalsForLaw = totalsByLawIdMap.get(law);
				if (null == runningTotalsForLaw) {
					runningTotalsForLaw = new HashSet<PtdDatumType>();
					totalsByLawIdMap.put(law, runningTotalsForLaw);
				}
				
				PtdDatumType runningTotalForLawInRange = totals.getDatumForRangeAndLaw(txDateRange, law);
				if (null == runningTotalForLawInRange) {
					runningTotalForLawInRange = new PtdDatumType();
					runningTotalForLawInRange.setDatumType(DatumType1Enum.MONEY);
					runningTotalForLawInRange.setMoneyValue(new Money());
					runningTotalForLawInRange.setPeriod(txDateRange);
					runningTotalsForLaw.add(runningTotalForLawInRange);
				}
				//update the totals for that LawId
				final com.intuit.sb.payroll.entity.model.Money runningAdjustmentAmount = 
					runningTotalForLawInRange.getMoneyValue().add(adjLineItem.getAdjustmentAmount());
				runningTotalForLawInRange.setMoneyValue(runningAdjustmentAmount);
			}
		}
		return totals;
	}
	
	private void fixupEmployerAdjustmentAssocation(Employer employer, Set<EmployerAdjustment> employerAdjustments) {
		for (EmployerAdjustment employerAdjustment : employerAdjustments) {
			employer.addEmployerAdjustment(employerAdjustment);
			employerAdjustment.setEmployer(employer);
		}
	}

}
