package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.LiabilityBalance;
import com.intuit.sb.payroll.capability.pts.adapter.PTSRequestInvoker;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.workerpayments.PayrollPTSWorkflowComponent;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.WorkflowEventType;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.entity.workerpayments.PayrollRunEAO;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;

/**
 * This class provides basic Workflow methods to interact with PTS for every PayrollRun 
 */
@Stateless(name = "PayrollPTSWorkflowComponent")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class PayrollPTSWorkflowComponentBean implements PayrollPTSWorkflowComponent{

	private static Logger sLogger = PayrollLoggerFactory.makeLogger(PayrollPTSWorkflowComponent.class);
	@EJB
	EmployerPTSWorkflowComponent employerPTSComponent;
	
	@EJB
	TaskManager  taskManager;
	
	@EJB
	PayrollRunEAO payrollRunEAO;
	
	public void handlePayrollRunEvent(WorkflowEventType eventType,
			PayrollRun payrollRun) throws ServiceFailureException {
		
		if (eventType == null){
			sLogger.warn("Null WorflowEvent received");
			return;
		}
		sLogger.debug("PayrollRun Event received "+ eventType);
		PTSEmployer ptsEmployer = employerPTSComponent.getPTSEmployer();
		switch(eventType){
			case COMMIT_SCHEDULED_PAYROLL_RUN:
				handlePayrollRunCommitedEvent(payrollRun, ptsEmployer);
				break;
			case PAYROLL_RUN_PAYCHECKS_VOIDED:
				handlePaycheckVoidedEvent(payrollRun, ptsEmployer);
				break;
			case COMMIT_TERMINATION_PAYROLL_RUN:
				/*
				 * WE WILL NOT DO ANYTHING HERE, TERMINATE EMPLOYEE EVENT WILL send
				 * both the terminated PayrollRun and terminate emp request to PSP
				 */
				break;
		}
	}

	/**
	 * @param payrollRun
	 * @param ptsEmployer
	 * @throws ServiceFailureException
	 */
	private void handlePaycheckVoidedEvent(PayrollRun payrollRun,
			PTSEmployer ptsEmployer) throws ServiceFailureException {
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			return;
		}
		
		if(ptsEmployer.getCurrentTaxServiceStatus() == TaxServiceStatusEnum.ACTIVATION_ACTIVE_ST_ACCEPTED) {	
			try {
				PTSRequestInvoker.voidPaychecks(ptsEmployer, payrollRun);
			} catch (ValidationException e) {
				throw new ServiceFailureException(e);
			}
		}
	}


	/**
	 * @param payrollRun
	 * @param ptsEmployer
	 * @throws ServiceFailureException
	 */
	private void handlePayrollRunCommitedEvent(PayrollRun payrollRun,
			PTSEmployer ptsEmployer) throws ServiceFailureException {
		EmployerPTSWorkflowTasks.handlePayrollRunCommitedEvent(taskManager, payrollRun); // creates the SETUP_ASSISTED_TASK if needed
		// Act only when current company is in PTS
		if (StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
			try {
				PTSRequestInvoker.submitPayrollRunInfoToPTS(ptsEmployer, payrollRun);
			} catch (ValidationException e) {
				throw new ServiceFailureException(e);
			}
		}
	}

	
	public void handleAssistedSetupEvent(WorkflowEventType workflowEventType,
			PTSEmployer ptsEmployer) throws ServiceFailureException {
		if (WorkflowEventType.EMPLOYER_ADDED_IN_PTS == workflowEventType){
			// 1. Submit all YTD/QTD Tax Payments
			// 2. Submit all Gemini toDate Payroll Run
			submitToDateTaxPayments(ptsEmployer);
			submitToDatePayrollRuns(ptsEmployer);
		}
		else if(WorkflowEventType.EMPLOYER_IN_PRE_ACTIVATION == workflowEventType){
			submitToDateTaxPayments(ptsEmployer);
		}
	}
	
	public List<LiabilityBalance> getHistoricalLiabilityBalances(PTSEmployer ptsEmployer) throws ServiceFailureException {
		return PTSRequestInvoker.getHistoricalLiabilityBalances(ptsEmployer);
	}
	
	private void submitToDatePayrollRuns(PTSEmployer ptsEmployer) throws ServiceFailureException {
		// Act only when current company is in PTS
		if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(ptsEmployer.getEmployer())){
               return;
        }
		List<PayrollRun> payrollRuns = payrollRunEAO.findAll();
		for (PayrollRun payrollRun:payrollRuns){
			// Check if this To-Date PayrollRun is not sent to PSP
			if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(payrollRun)){
				try {
					PTSRequestInvoker.submitPayrollRunInfoToPTS(ptsEmployer, payrollRun);
				} catch (ValidationException e) {
					throw new ServiceFailureException(e);
				}
			}
		}
	}
		
	private void submitToDateTaxPayments(PTSEmployer ptsEmployer) throws ServiceFailureException {
		// Get current quarter DateRange
		DateRange dateRange = PtdUtils.getQtrRangeForDate(DateUtil.getNowDate());
		
		List<Payment> paymentsToAdd = new ArrayList<Payment>();
		
		// Get all Payments for this quarter
		Employer employer = ptsEmployer.getEmployer();
		Set<Payment> payments = employer.getPayments();
		for (Payment payment:payments){
			if (payment.getPayrollPeriod().equals(dateRange)){
				if (!StatusUpdateHandlerUtil.isEntityAddedToPTS(payment)){
					// This payment is for current quarter, add to send to PTS
					paymentsToAdd.add(payment);
				}
			}
		}
		if (paymentsToAdd.size() > 0){
			PTSRequestInvoker.sendPaymentsToPTS(ptsEmployer,paymentsToAdd);
		}
		
		
	}
}
