package com.intuit.sb.payroll.capability.workerpayments.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.intuit.sb.payroll.capability.agency.AgencyComponent;
import com.intuit.sb.payroll.capability.agency.Holiday;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;

/**
 * All stateless Date calculations are encapsulated in this class. It should have no dependencies on a Database.
 * 
 * Tests are in PayrollScheduleDateUtilsTest.java
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
public class PayrollScheduleDateUtils {

	/**
	 * Returns the Period Start Date based on the End Date and the Frequency.
	 * 
	 * @param currentPeriodEndDate the Period End Date.
	 * @param payFrequency the {@link PayrollFrequencyEnum}
	 * @return Calendar the Start Date
	 */
	public static Calendar getPayPeriodStartDate(Calendar currentPeriodEndDate, PayrollFrequencyEnum payFrequency) {
		return calculateStartOrEndDateBasedOnFrequency(currentPeriodEndDate, payFrequency, false);
	}

	/**
	 * Returns the Period End Date based on the Start Date and the Frequency.
	 * 
	 * @param currentPeriodStartDate
	 * @param payFrequency
	 * @return Calendar for the Next date based on the End date and PayrollFrequency
	 */
	public static Calendar getPayPeriodEndDate(Calendar currentPeriodStartDate, PayrollFrequencyEnum payFrequency) {
		return calculateStartOrEndDateBasedOnFrequency(currentPeriodStartDate, payFrequency, true);
	}

	private static Calendar calculateStartOrEndDateBasedOnFrequency(Calendar workingDate,
			PayrollFrequencyEnum payFrequency, boolean end) {
		if (workingDate == null) {
			throw new IllegalArgumentException("Null workingDate");
		}

		Calendar calculatedDate = DateUtil.copyYMD(workingDate);

		// Go forward or backwards in time
		int multiplier = end ? 1 : -1;
		switch (payFrequency) {
			case DAILY:
				// add one more for the offset
				calculatedDate.add(Calendar.DATE, multiplier * (1 + 1));
				break;
			case WEEKLY:
				calculatedDate.add(Calendar.WEEK_OF_YEAR, multiplier * 1);
				break;
			case BIWEEKLY:
				calculatedDate.add(Calendar.WEEK_OF_YEAR, multiplier * 2);
				break;
			case MONTHLY:
				calculatedDate.add(Calendar.MONTH, multiplier * 1);
				break;
			case QUARTERLY:
				calculatedDate.add(Calendar.MONTH, multiplier * 3);
				break;
			case SEMIMONTHLY:
				throw new UnsupportedOperationException("Cannot use this method for Semi-Monthly");
			case ANNUALLY:
				calculatedDate.add(Calendar.YEAR, multiplier * 1);
				break;
			default:
				break;
		}

		multiplier = multiplier * -1;
		// offset this by one day now one way or the other.
		calculatedDate.add(Calendar.DATE, multiplier);

		return calculatedDate;
	}

	/**
	 * Prepare PayrollSchedule object to be saved by setting calculated fields.
	 * 
	 * @param pSchedule
	 */
	static void determineCalculatedValues(PayrollSchedule payrollSchedule) {
		PayrollFrequencyEnum frequency = payrollSchedule.getFrequency();
		
		switch (payrollSchedule.getFrequency()) {
	        // Execute the calculated value of the PeriodStartDate for DAILY, WEEKLY, and BIWEEKLY
			case DAILY:
			case WEEKLY:
			case BIWEEKLY:
				Date nextPayPeriodEndDate = payrollSchedule.getNextPayPeriodEndDate();
				final Calendar existingEndDate = DateUtil.dateToCalendar(nextPayPeriodEndDate);
				final Calendar calculatedStartDate = PayrollScheduleDateUtils.getPayPeriodStartDate(existingEndDate, frequency);
				payrollSchedule.setPeriodStartDate(calculatedStartDate.getTime());
				break;
			
			//MONTHLY's end date may vary if it's end of the month.
			case MONTHLY:
				Date nextMonthlyPayPeriodEndDate = payrollSchedule.getNextPayPeriodEndDate();
				int day = payrollSchedule.getPayPeriodEndDayofMonth1();
				if (null == nextMonthlyPayPeriodEndDate) {
					final Calendar cal = DateUtil.getNowCalendar();
	
					if (day == PayrollSchedule.LAST_DAY_OF_MONTH) {
						// Modify the day to be last day of the month.
						day = DateUtil.getLastDayOfMonth(cal);
					}
					cal.set(Calendar.DAY_OF_MONTH, day);
					payrollSchedule.setNextPayPeriodEndDate(cal.getTime());
				}
				Calendar endDateCal = DateUtil.dateToCalendar(payrollSchedule.getNextPayPeriodEndDate());
				Calendar monthlyStartDate = null;
				if (payrollSchedule.getPayPeriodEndDayofMonth1() == PayrollSchedule.LAST_DAY_OF_MONTH) {
					endDateCal.set(Calendar.DAY_OF_MONTH, 1);
					monthlyStartDate = DateUtil.copyYMD(endDateCal);
				} else {
					monthlyStartDate = PayrollScheduleDateUtils.getPayPeriodStartDate(endDateCal, frequency);
				}
				
				payrollSchedule.setPeriodStartDate(monthlyStartDate.getTime());
				break;
				
			// SEMIMONTHLY end date varies depending on what half of the month you are in.
			case SEMIMONTHLY:
				final Calendar semiPeriodEndDate = DateUtil.dateToCalendar(payrollSchedule.getNextPayPeriodEndDate());
				DateRange period = getPeriodForSemiMonthlyScheduleFromEndDate(payrollSchedule, semiPeriodEndDate);
				payrollSchedule.setPeriodStartDate(period.getStartDate());
				payrollSchedule.setNextPayPeriodEndDate(period.getEndDate());
				break;
			case ANNUALLY:
				throw new UnsupportedOperationException("Annually is not supported yet.");
		}
	}
	
	static DateRange getPeriodForSemiMonthlyScheduleFromEndDate(PayrollSchedule payrollSchedule, Calendar lastPeriodEndDate) {
		final int day1 = payrollSchedule.getPayPeriodEndDayofMonth1();
		final int day2 = payrollSchedule.getPayPeriodEndDayofMonth2();
		boolean uselastDayOfMonth = false;
		if (day2 == PayrollSchedule.LAST_DAY_OF_MONTH) {
			uselastDayOfMonth = true;
		}
		if (lastPeriodEndDate == null) {
			throw new AssertionError("Must set a nextPayPeriodEndDate");
		}
		final Calendar endCal = DateUtil.copyYMD(lastPeriodEndDate);
		final Calendar startCal = DateUtil.copyYMD(lastPeriodEndDate);
		final int anchorDate = endCal.get(Calendar.DAY_OF_MONTH);
		if (anchorDate == day1) {
			// We are in the first half, use day2 or EOM +1 for start, end is day1
			endCal.set(Calendar.DAY_OF_MONTH, day1);
			if (uselastDayOfMonth) {
				// This will always be the first day of the month in this case.
				startCal.set(Calendar.DAY_OF_MONTH, 1);
			} else {
				startCal.add(Calendar.MONTH, -1);
				final int startDay = day2 +1;
				if (startDay > DateUtil.getLastDayOfMonth(startCal)) {
					throw new AssertionError("Calculated startDay is greater than number of days in the Month");
				} else {
					startCal.set(Calendar.DAY_OF_MONTH, startDay);
				}
			}
		} else if (anchorDate > day1){
			// We are in the second half, use day1 + 1 for start, day2 or EOM for end.
			startCal.set(Calendar.DAY_OF_MONTH, day1+1);
			if (uselastDayOfMonth) {
				endCal.set(Calendar.DAY_OF_MONTH, DateUtil.getLastDayOfMonth(endCal));
			} else {
				endCal.set(Calendar.DAY_OF_MONTH, day2);
			}
		}
		DateRange result = new DateRange(startCal.getTime(), endCal.getTime());
		return result;
	}
	
	static DateRange getPeriodForSemiMonthlyScheduleFromStartDate(PayrollSchedule payrollSchedule, Calendar periodStartDate) {
		final int day1 = payrollSchedule.getPayPeriodEndDayofMonth1();
		final int day2 = payrollSchedule.getPayPeriodEndDayofMonth2();
		boolean uselastDayOfMonth = false;
		if (day2 == PayrollSchedule.LAST_DAY_OF_MONTH) {
			uselastDayOfMonth = true;
		}
		if (periodStartDate == null) {
			throw new AssertionError("Must set a nextPayPeriodEndDate");
		}
		final Calendar startCal = DateUtil.copyYMD(periodStartDate);
		final Calendar endCal = DateUtil.copyYMD(periodStartDate);
		final int anchorDate = startCal.get(Calendar.DAY_OF_MONTH);
		if (anchorDate < day1) {
			endCal.set(Calendar.DAY_OF_MONTH, day1);
		} else if (anchorDate > day1 && anchorDate < day2){
			if (uselastDayOfMonth) {
				endCal.set(Calendar.DAY_OF_MONTH, DateUtil.getLastDayOfMonth(endCal));
			} else {
				endCal.set(Calendar.DAY_OF_MONTH, day2);
			}
		} else {
			endCal.set(Calendar.DAY_OF_MONTH, day1);
			endCal.add(Calendar.MONTH, 1);
		}
		DateRange result = new DateRange(startCal.getTime(), endCal.getTime());
		return result;
	}
		
	/**
	 * Returns a list of {@link PayrollRunInfo} instances that encapsulate the Dates in a {@link PayrollRun}, based on a
	 * {@link PayrollSchedule}.
	 * 
	 * @param payrollSchedule the {@link PayrollSchedule} on which to base the calculations.
	 * @param startDate {@link Date} the start day of the period.
	 * @param year the year for which {@link PayrollRun}s to process. If it is null it returns the 
	 * {@link PayrollRunInfo} immediately after the passed payrollSchedule
	 * @return the {@link List} of {@link PayrollRunInfo}s if any, or empty {@link List}.
	 */
	static List<PayrollRunInfo> getPayrollRunInfosForSchedule(PayrollSchedule payrollSchedule, Calendar startDate,
			Calendar year) {
		List<PayrollRunInfo> results = new ArrayList<PayrollRunInfo>();
		final PayrollFrequencyEnum frequency = payrollSchedule.getFrequency();
		Calendar startCal = DateUtil.copyYMD(startDate);
		Calendar payrollRunDate;
		do {
			Calendar endDate = null;
			PayrollRunInfo payrollRunInfo = new PayrollRunInfo();			

			switch (frequency) {
				case DAILY:
				case WEEKLY:
				case BIWEEKLY:
				case QUARTERLY:
				case ANNUALLY:
					endDate = getPayPeriodEndDate(startCal, frequency);
					break;
				case MONTHLY:
					int day = payrollSchedule.getPayPeriodEndDayofMonth1();
	
					if (day == PayrollSchedule.LAST_DAY_OF_MONTH) {
						endDate = DateUtil.copyYMD(startCal);
						day = DateUtil.getLastDayOfMonth(endDate);
						endDate.set(Calendar.DAY_OF_MONTH, day);
					} else {
						endDate = getPayPeriodEndDate(startCal, frequency);
					}
					break;
				case SEMIMONTHLY:
					DateRange semiRange = getPeriodForSemiMonthlyScheduleFromStartDate(payrollSchedule, startCal);
					endDate = DateUtil.dateToCalendar(semiRange.getEndDate());
					break;
				default:
					break;
			}
			DateRange payPeriod = new DateRange(startCal.getTime(), endDate.getTime());	
			
			// Adjust the paycheck date first and then base the run date off of that.
			// Moves forwards
			Calendar payDate = PayrollScheduleDateUtils.adjustDateForOffsetWithHolidaysAndWeekends(endDate, 
					payrollSchedule.getPayDayOffset());
			// Moves backwards
			payrollRunDate = PayrollScheduleDateUtils.adjustDateForOffsetWithHolidaysAndWeekends(payDate, 
					payrollSchedule.getPayrollRunDayOffset() - payrollSchedule.getPayDayOffset());
			
			
			
			if(year == null) {
				payrollRunInfo.setFrequency(frequency);
				payrollRunInfo.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);
				payrollRunInfo.setPayPeriod(payPeriod);
				payrollRunInfo.setPayDate(payDate.getTime());
				payrollRunInfo.setPayrollRunDate(payrollRunDate.getTime());
				results.add(payrollRunInfo);
				break;
			} 
			else if(payrollRunDate.get(Calendar.YEAR) == year.get(Calendar.YEAR)) {
				payrollRunInfo.setFrequency(frequency);
				payrollRunInfo.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);
				payrollRunInfo.setPayPeriod(payPeriod);
				payrollRunInfo.setPayDate(payDate.getTime());
				payrollRunInfo.setPayrollRunDate(payrollRunDate.getTime());
				results.add(payrollRunInfo);
			}			
			
			// Move to beginning of next period if this is not the last loop
			startCal = DateUtil.copyYMD(endDate);
			startCal.add(Calendar.DAY_OF_MONTH, 1);

		} while(payrollRunDate.get(Calendar.YEAR) <= year.get(Calendar.YEAR));

		return results;
	}

	/**
	 * Adjusts a date by an offset of business days moving forward.
	 * Accounts for holidays or weekends.
	 * 
	 * @param date
	 * @param offset number of days to move. + means forward, - means backwards
	 * @return the new Calendar instance with the offset applied.
	 */
	public static Calendar adjustDateForOffsetWithHolidaysAndWeekends(Calendar date, int offset) {
		if (null == date) {
			throw new IllegalArgumentException("No null dates as inputs.");
		}
		
		if (offset == 0) {
			return DateUtil.copyYMD(date);
		}
		
		int step = offset > 0 ? 1 : -1;
		int amount = Math.abs(offset);
		Calendar result = DateUtil.copyYMD(date);
		int i = 0;
		
		do {
			result.add(Calendar.DAY_OF_MONTH, step);
			
			// Count it as a business day
			if (!isHoliday(result) && !DateUtil.isWeekend(result)) {
				i++;
			}
		
		// Make sure not to end on a holiday or weekend.
		}while (i < amount || isHoliday(result) || DateUtil.isWeekend(result));
		
		return result;
	}
	
	/**
	 * Checks if the current date is a holiday in the US.
	 * 
	 * @param date Date to check for holiday status
	 * @return true if the date is a holiday.
	 */
	public static boolean isHoliday(Calendar date) {

		Calendar copy = DateUtil.copyYMD(date);
		
		// Create dateRange from yesterday to tomorrow to make sure to get the holidays.
		copy.add(Calendar.DATE, -1);
		Date yesterday = copy.getTime();
		copy.add(Calendar.DATE, 2);
		Date tomorrow = copy.getTime();
		DateRange dateRange = new DateRange(yesterday, tomorrow);
		
		AgencyComponent agencyComponent = ServiceLocator.getInstance().getService(AgencyComponent.JNDI_NAME);		
		List<Holiday> holidays = agencyComponent.getHolidaysForPeriod(dateRange);
		
		for (Holiday holiday : holidays) {
			if (DateUtil.isSameDay(date.getTime(), holiday.getDate())){
				return true;
			}
		}
		
		return false;
	}

	/**
	 * Filters existing runs from a list of candidate runs.
	 * 
	 * @param allRuns the list to filter from.
	 * @param existingRuns the PayrollRuns to remove.
	 * @return
	 */
	static List<PayrollRunInfo> filterCompletedPayrollRuns(List<PayrollRunInfo> allRuns, List<PayrollRun> existingRuns) {
		final List<PayrollRunInfo> results = new ArrayList<PayrollRunInfo>();
		// Map to search for existing values in.
		final Map<DateRange, PayrollRun> existingRunsMap = new HashMap<DateRange, PayrollRun>(existingRuns.size());
		for (PayrollRun payrollRun : existingRuns) {
			existingRunsMap.put(payrollRun.getPayPeriod(), payrollRun);
		}
		// Check for the existence of the pay period in the existing runs
		for (PayrollRunInfo payrollRunInfo : allRuns) {
			if (!existingRunsMap.containsKey(payrollRunInfo.getPayPeriod())) {
				results.add(payrollRunInfo);
			} else {
				// Even if there's a match on Period, need to ensure it's the same frequency or ad-hocs can hide
				// scheduled.
				PayrollRun candidatePayrollRun = existingRunsMap.get(payrollRunInfo.getPayPeriod());
				if (!candidatePayrollRun.getPayrollFrequency().equals(payrollRunInfo.getFrequency())) {
					results.add(payrollRunInfo);
				}
			}
		}
		return results;
	}
}
