package com.intuit.sb.payroll.calcengine;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Currency;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.CalculationInputs;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckInputs;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInputs;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.DeductionDtl;
import com.intuit.sb.payroll.entity.model.DeductionTypeEnum;
import com.intuit.sb.payroll.entity.model.HealthBenefit;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;


/**
 *  <P>The <CODE>CAPaycheckCalcTest</CODE> class contains tests for testing the paycheck calculation functionality
 *     for a California employee</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CAPaycheckCalcTest extends CalcEngineTestBase {
    
	
	/**
     * <P>Test biweekly paycheck with hourly wage and all applicable taxes for California 
     * using withholding status of Married 4 and Health Benefit</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with rate of 15.00/hour and 80 hours</LI>
     * <LI>Create a health benefit item with per paycheck deduction rate of 21.12</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1HealthBenefit_WithholdCAMarried4_PaidBiweekly() throws Exception {
    	PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 4, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIEDONEINCOME, 4, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        HealthBenefitDtl pretaxHealthBenefit = createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125HEALTH, 
        			new BigDecimal("21.12"), DatumType2Enum.PERPAYCHECK);
        employee.addHealthBenefitDtl(pretaxHealthBenefit);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
		
        BigDecimal amount = new BigDecimal("1178.88");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(PayrollItemType.SEC125HEALTH.getLawId(), new BigDecimal("21.12")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("33.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("73.09"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.09"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.43"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.43"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("73.09"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.09"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.08"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.18"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 salary wage and all applicable taxes for California 
     * using withholding status of Single 1 and health benefit</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Create a health benefit item with per paycheck rate of 50.00</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1SalaryHealthBenefit_WithholdCASingle1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        HealthBenefitDtl pretaxHealthBenefit = createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125HEALTH, 
    			new BigDecimal("50.00"), DatumType2Enum.PERPAYCHECK);
        employee.addHealthBenefitDtl(pretaxHealthBenefit);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
       
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("1719.36"));
        
        BigDecimal amount = new BigDecimal("2488.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
        	new LineItemAmount(PayrollItemType.SEC125HEALTH.getLawId(), new BigDecimal("50.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("428.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("154.28"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.08"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("19.91"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("154.28"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.08"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("84.61"), amount, amount),
        	new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.83"), amount, amount),
        	new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("19.91"), amount, amount),
        	new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.49"), amount, amount)
        	
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }


	/**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California using withholding status of CA Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdCASingle1_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);

        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
      // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("65.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("12.77"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage, 1 piecework and all applicable taxes for California using withholding
     *  status of CA Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Create a piecework wage item with a rate of 1.00 per unit and 300 units</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_1Piecework_WithholdCASingle1_PaidWeekly() throws Exception{
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);

        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
      // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        
        CompensationDtl pieceworkWage = createPieceworkWageItemDtl(new BigDecimal("1.0"), 1.0, 300.0);
        pieceworkWage.setId(2L);
        
        employee.addCompensationDtl(hourlyWage);
        employee.addCompensationDtl(pieceworkWage);
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(pieceworkWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        BigDecimal amount = new BigDecimal("900.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(PayrollItemType.PIECEWORK.getLawId(), new BigDecimal("300.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("128.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("55.80"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("13.05"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("33.78"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("7.20"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("7.20"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("55.80"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("13.05"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("30.60"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.90"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 salary wage, 1 Piecework wage and all applicable taxes for California 
     * using withholding status of Single 1 and health benefit</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Create a piecework rate of $3.00 / 1 unit with 80 units worked</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_1Piecework_WithholdCASingle1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        CompensationDtl pieceworkWage = createPieceworkWageItemDtl(new BigDecimal("3.0"), 1.0, 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        pieceworkWage.setId(1L);
        employee.addCompensationDtl(pieceworkWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(pieceworkWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
       
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("1885.88"));
        
        BigDecimal amount = new BigDecimal("2778.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
        	new LineItemAmount(PayrollItemType.PIECEWORK.getLawId(), new BigDecimal("240.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("500.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("172.26"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("40.29"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.23"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("172.26"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("40.29"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("94.47"), amount, amount),
        	new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("157.80"), amount, amount),
        	new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.23"), amount, amount),
        	new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.78"), amount, amount)
        	
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }


    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California using withholding status of MarriedOneIncome 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use withholding of Married 1 for Federal and MarriedOneIncome 1 for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdCAMarriedOneIncome1_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        BigDecimal amount = new BigDecimal("600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("42.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("6.21"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California 
     * using withholding status of MarriedTwoIncome 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use withholding of Married 1 for Federal and MarriedTwoIncome 1 for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdCAMarriedTwoIncome1_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIED, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("42.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("12.77"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California using withholding status of HeadOfHousehold 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use withholding of MarriedUsingSingleRate 1 for Federal and HeadOfHousehold 1 for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdCAHeadOfHousehold1_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIEDSINGLERATE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.HEADOFHOUSEHOLD, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("65.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("4.86"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California 
     * using withholding status of DontWithhold 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use withholding of DontWithhold for Federal and DontWithhold for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdCADontWithhold_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.NOTSET, 0, EXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.NOTSET, 0, sZeroMoney, 0, EXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California 
     * using withholding status of CA Single 1 and extra withholdings</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 60.00 and 40 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add extra withholding of 30.00 for Federal, 10.00 and estimated deduction of 2 for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdCASingle1_ExtraWithhold_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        Money thirty = new Money();
        thirty.setAmount(new BigDecimal("30.00"));
        thirty.setCurrencyCode(Currency.getInstance("USD"));
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, thirty, NOTEXEMPT);
        Money ten = new Money();
        ten.setAmount(new BigDecimal("10.00"));
        ten.setCurrencyCode(Currency.getInstance("USD"));
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, ten, 2, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("60.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("2400.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("557.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("148.80"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("34.80"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("179.32"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("19.20"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("19.20"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("148.80"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("34.80"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("81.60"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.40"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage, all applicable taxes for California 
     * and a rate based pre-tax deduction</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Add a pre-tax deduction to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test(enabled=false, groups={"broken"})
    public void test_1Hourly_WithRateBasedPreTaxDeduction_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);
        // add employee deductions
        HealthBenefitDtl sec125Health = new HealthBenefitDtl();
        HealthBenefit healthBenefit = healthBenefitEAO.findHealthBenefit(HealthBenefitTypeEnum.SEC125HEALTH);
        sec125Health.setPayrollItem(healthBenefit);
// TODO Fix following code when deductions are supported        
//        sec125Health.setRate("8%");
        employee.addHealthBenefitDtl(sec125Health);
        
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("58.00")),
            new LineItemAmount(sec125Health.getPayrollItem().getLawId().getValue(), new BigDecimal("48.00")),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("34.22")),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.00")),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("10.03")),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.42")),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.42")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("34.22")),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.00")),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("18.77")),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.55"))};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage, all applicable taxes for California and a amount based pre-tax deduction</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Add a pre-tax deduction to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithAmountBasedPreTaxDeduction_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);
        // add employee deductions
        HealthBenefitDtl sec125Health = createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125HEALTH, 
                                                          	   new BigDecimal("50.00"), DatumType2Enum.PERPAYCHECK);
        sec125Health.setId(124L);
        employee.addHealthBenefitDtl(sec125Health);
        
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("550.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
        	new LineItemAmount(sec125Health.getPayrollItem().getLawId().getValue(), new BigDecimal("50.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("57.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("34.10"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("7.98"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("9.95"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.40"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.40"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("34.10"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("7.98"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("18.70"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.55"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 overtime and all applicable taxes for California 
     * using withholding status of CA Single 2</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create an overtime wage item with a rate of 22.50 and 10 hours</LI>
     * <LI>Use withholding of Single 2 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1Overtime_WithholdCASingle2_PaidBiweekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 2, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 2, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        CompensationDtl overtimeWage = createOvertimeWageItemDtl(CompensationTypeEnum.HOURLYOVT, 
        		hourlyWage, 1.5);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        overtimeWage.setId(2L);
        //Add the quantity
        Datum quantityDatum = new Datum();
        quantityDatum.setValue1AsQuantity(10.0);
        overtimeWage.setQuantity(quantityDatum);
        employee.addCompensationDtl(overtimeWage);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(overtimeWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("1425.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(PayrollItemType.OVERTIME.getLawId(), new BigDecimal("225.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("143.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("88.35"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("20.66"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("35.43"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("11.40"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("11.40"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("88.35"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("20.66"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("48.45"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.43"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

	/**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 overtime, 1 double-time and all applicable taxes for California 
     * using withholding status of CA Single 2</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create an overtime wage item with a rate of 22.50 and 10 hours</LI>
     * <LI>Create a double-time wage item with a rate of 30.00 and 2 hours</LI>
     * <LI>Use withholding of Single 2 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1Overtime1Doubletime_WithholdCASingle2_PaidBiweekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 2, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 2, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        //10 hrs
        CompensationDtl overtimeWage = createOvertimeWageItemDtl(CompensationTypeEnum.HOURLYOVT, 
                hourlyWage, 1.5);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        overtimeWage.setId(2L);
        //Add the quantity
        Datum quantityDatum = new Datum();
        quantityDatum.setValue1AsQuantity(10.0);
        overtimeWage.setQuantity(quantityDatum);
        // 2 hrs
        CompensationDtl doubletimeWage = createOvertimeWageItemDtl(CompensationTypeEnum.HOURLYDBL, 
                                               hourlyWage, 2.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        doubletimeWage.setId(3L);
        //Add the quantity
        Datum dblQuantityDatum = new Datum();
        dblQuantityDatum.setValue1AsQuantity(2.0);
        doubletimeWage.setQuantity(dblQuantityDatum);
        employee.addCompensationDtl(doubletimeWage);
        employee.addCompensationDtl(hourlyWage);
        employee.addCompensationDtl(overtimeWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(overtimeWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(doubletimeWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("1485.00");
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(PayrollItemType.OVERTIME.getLawId(), new BigDecimal("225.00")),
            new LineItemAmount(PayrollItemType.DOUBLETIME.getLawId(), new BigDecimal("60.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("152.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("92.07"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("21.53"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("39.03"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("11.88"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("11.88"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("92.07"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("21.53"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("50.49"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.49"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 commission and all applicable taxes for California 
     * using withholding status of CA Married 4</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create a commission wage item with a rate of 400.00</LI>
     * <LI>Use withholding of Married 4 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1Commission_WithholdCAMarried4_PaidBiweekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 4, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIED, 4, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        CompensationDtl commissionWage = createAmountBasedWageItemDtl(CompensationTypeEnum.COMMISSION, 
        		new BigDecimal("400.00"), DatumType2Enum.PERPAYCHECK);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        commissionWage.setId(2L);
        employee.addCompensationDtl(commissionWage);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(commissionWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);

        BigDecimal amount = new BigDecimal("1600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(PayrollItemType.COMMISSION.getLawId(), new BigDecimal("400.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("83.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("99.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("23.20"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("40.72"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("12.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("12.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("99.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("23.20"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("54.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.60"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 commission and all applicable taxes for California 
     * using withholding status of CA Married 4</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create a fixed commission wage item with a rate of 400.00</LI>
     * <LI>Use withholding of Married 4 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1FixedCommission_WithholdCAMarried4_PaidBiweekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 4, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIED, 4, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        CompensationDtl commissionWage = createAmountBasedWageItemDtl(CompensationTypeEnum.COMMISSION, 
        		new BigDecimal("400.00"), DatumType2Enum.PERPAYCHECK);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        commissionWage.setId(2L);
        commissionWage.setFixedRate(true);
        employee.addCompensationDtl(commissionWage);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(commissionWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);

        BigDecimal amount = new BigDecimal("1600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(PayrollItemType.COMMISSION.getLawId(), new BigDecimal("400.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("83.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("99.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("23.20"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("40.72"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("12.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("12.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("99.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("23.20"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("54.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.60"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 Tip and all applicable taxes for California using withholding status of CA Married 3</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 8.00 and 40 hours</LI>
     * <LI>Create a Tip wage item with a rate of 600.00</LI>
     * <LI>Use withholding of Married 3 for Federal and Married_One_Income for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1TipPaycheck_WithholdCAMarried3_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 3, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIEDONEINCOME, 3, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("8.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        CompensationDtl tipWage = createAmountBasedWageItemDtl(CompensationTypeEnum.TIPSPAYCHECK, 
        		new BigDecimal("600.00"), DatumType2Enum.PERPAYCHECK);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        tipWage.setId(2L);
        employee.addCompensationDtl(tipWage);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(tipWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("920.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("320.00")),
            new LineItemAmount(PayrollItemType.TIPS_PAYCHECK.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("70.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("57.04"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("13.34"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("10.89"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("57.04"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("13.34"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("31.28"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.92"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 Tip and all applicable taxes for California using withholding status of CA Married 3</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 8.00 and 40 hours</LI>
     * <LI>Create a Tip (CASH) wage item with a rate of 600.00</LI>
     * <LI>Use withholding of Married 3 for Federal and Married_One_Income for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test(enabled=true)
    public void test_1Hourly1TipCash_WithholdCAMarried3_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 3, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIEDONEINCOME, 3, sZeroMoney, 0, NOTEXEMPT);
        // add compensations
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("8.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        CompensationDtl tipWage = createAmountBasedWageItemDtl(CompensationTypeEnum.TIPSCASH, 
        		new BigDecimal("600.00"), DatumType2Enum.PERPAYCHECK);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        tipWage.setId(2L);
        employee.addCompensationDtl(tipWage);
        employee.addCompensationDtl(hourlyWage);
        // add deductions
        DeductionDtl cashTipsOffsetDtl = WorkerTestUtils.createDeductionDtl(DeductionTypeEnum.CASHTIPSOFFSET);
        employee.addDeductionDtl(cashTipsOffsetDtl);
        tipWage.setLinkedPayrollItemDtl(cashTipsOffsetDtl);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(tipWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("920.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("320.00")),
            new LineItemAmount(PayrollItemType.TIPS_CASH.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(PayrollItemType.TIPS_CASHOFFSET.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("70.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("57.04"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("13.34"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("10.89"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("7.36"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("57.04"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("13.34"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("31.28"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.92"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 Tip and all applicable taxes for California using withholding status of CA Married 3</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 8.00 and 40 hours</LI>
     * <LI>Create an allocated Tip wage item with a rate of 600.00</LI>
     * <LI>Use withholding of Married 3 for Federal and Married_One_Income for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1TipsAllocated_WithholdCAMarried3_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 3, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIEDONEINCOME, 3, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("8.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        CompensationDtl tipWage = createAmountBasedWageItemDtl(CompensationTypeEnum.TIPSALLOCATED, 
        		new BigDecimal("600.00"), DatumType2Enum.PERPAYCHECK);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        tipWage.setId(2L);
        employee.addCompensationDtl(tipWage);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(tipWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("320.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("320.00")),
            new LineItemAmount(PayrollItemType.TIPS_ALLOCATED.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("19.84"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("4.64"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("2.56"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("2.56"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("19.84"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("4.64"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.88"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.32"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage, 1 Tip and all applicable taxes for California 
     * using withholding status of Federal Married 3, CA Married (two incomes) 3, all items contain YTD values</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 80.00 and 40 hours (containing YTD value)</LI>
     * <LI>Create a Tip wage item with a rate of 2000.00 (containing YTD value)</LI>
     * <LI>Use withholding of Married 3 for Federal and Married (two incomes) 3 CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes (all containing YTD values) to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1Tip_WithholdCAMarried3_WithYTD_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 3, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIED, 3, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("80.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        CompensationDtl tipWage = createAmountBasedWageItemDtl(CompensationTypeEnum.TIPSPAYCHECK, 
        		new BigDecimal("2000.00"), DatumType2Enum.PERPAYCHECK);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        tipWage.setId(2L);
        
        setupPaycheckDates(frequency);
        Date paycheckDate = mPaycheckDate.getTime();
        
        setYtdAmount(hourlyWage, paycheckDate, new BigDecimal("3200.00"), new BigDecimal("3200.00"),
                     new BigDecimal("3200.00"), 40.0);
        // add hourly wages
        employee.addCompensationDtl(hourlyWage);
        // add amount based wages
        setYtdAmount(tipWage, paycheckDate, new BigDecimal("2000.00"), new BigDecimal("2000.00"),
                     new BigDecimal("2000.00"), 0.0);
        employee.addCompensationDtl(tipWage);
        // add employee federal taxes
        BigDecimal ytdWages = new BigDecimal("5200.00");
        setYtdAmount(mUSFIT, paycheckDate, new BigDecimal("1193.00"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSFIT);
        setYtdAmount(mUSSSTE, paycheckDate, new BigDecimal("322.40"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSSSTE);
        setYtdAmount(mUSMEDTE, paycheckDate, new BigDecimal("75.40"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        setYtdAmount(mUSFUI, paycheckDate, new BigDecimal("41.60"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSFUI);
        setYtdAmount(mUSSSTER, paycheckDate, new BigDecimal("322.40"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSSSTER);
        setYtdAmount(mUSMEDTER, paycheckDate, new BigDecimal("75.40"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        setYtdAmount(mCAPIT, paycheckDate, new BigDecimal("429.68"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mCAPIT);
        setYtdAmount(mCASDI, paycheckDate, new BigDecimal("41.60"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        setYtdAmount(mCASUI, paycheckDate, new BigDecimal("176.80"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mCASUI);
        setYtdAmount(mCAETT, paycheckDate, new BigDecimal("5.20"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mCAETT);

        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(tipWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("5200.00");
        
        // verify paycheck line items
        //Note: Limits: FUI, CASUI, CAETT - $7000
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("3200.00"), new BigDecimal("6400.00")),
            new LineItemAmount(PayrollItemType.TIPS_PAYCHECK.getLawId(), new BigDecimal("2000.00"), new BigDecimal("4000.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("1193.00"), new BigDecimal("2386.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("322.40"), new BigDecimal("644.80"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("75.40"), new BigDecimal("150.80"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("429.68"), new BigDecimal("859.36"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("41.60"), new BigDecimal("83.20"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("14.40"), new BigDecimal("56.00"), amount, new BigDecimal("1800.00")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("322.40"), new BigDecimal("644.80"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("75.40"), new BigDecimal("150.80"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("61.20"), new BigDecimal("238.00"), amount, new BigDecimal("1800.00")),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.80"), new BigDecimal("7.00"), amount, new BigDecimal("1800.00"))};
        AssertJUnit.assertTrue("LineItem mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test semimonthly paycheck with 1 hourly wage, 401K and all applicable taxes for California 
     * using withholding status of CA Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a semimonthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 20.00 and 80 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add 401K items</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
//TODO  Fix this when deductions are supported   
/*    
    @Test
    public void test_1Hourly_With401K_WithholdCASingle1_PaidSemimonthly() {
        Worker employee = createEmployee(mCALocation, mCALocation, PayrollFrequencyEnum.SEMIMONTHLY);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItem(CompensationTypeEnum.HOURLYREG, 
        		PayrollItemType.HOURLY.getLawId().getValue(), new BigDecimal("20.00"), 80.0);
        // add hourly wages
        employee.addCompensationDtl(hourlyWage);
        // add deductions
        BenefitDtl four01k = createPercentageBasedRetirementDeductionDtl(PayrollItemType.FOUR01K.getLawId().getValue(), 10.0);
        employee.addDeductionDtl(four01k);
        BenefitDtl four01kMatch = createPercentageBasedRetirementDeductionDtl(PayrollItemType.FOUR01KMATCH.getLawId().getValue(), 3.0);
        employee.addDeductionDtl(four01kMatch);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(employee);
        PayrollRun payrollRunData = createPayrollRun(mPaycheckDate, sPeriodStartDate, mPeriodEndDate);       
        Paycheck paycheck = new Paycheck();
        paycheck.setPayPeriod(payrollRunData.getPayPeriod());
        paycheck = mCalcEngine.calculatePaycheck(employee, mEmployer, payrollRunData, paycheck);
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId().getValue(), new BigDecimal("1600.00")),
            new LineItemAmount(PayrollItemType.FOUR01K.getLawId().getValue(), new BigDecimal("160.00")),
            new LineItemAmount(mUSFIT.getLawId().getValue(), new BigDecimal("162.00")),
            new LineItemAmount(mUSSSTE.getLawId().getValue(), new BigDecimal("99.20")),
            new LineItemAmount(mUSMEDTE.getLawId().getValue(), new BigDecimal("23.20")),
            new LineItemAmount(mCAPIT.getLawId().getValue(), new BigDecimal("36.07")),
            new LineItemAmount(mCASDI.getLawId().getValue(), new BigDecimal("12.80")),
//            TODO: Re-enable this when the non-taxable company payments is returned
//            new LineItemAmount(PayrollItemType.FOUR01KMATCH.getLawId().getValue(), new BigDecimal("48.00")),
            new LineItemAmount(mUSFUI.getLawId().getValue(), new BigDecimal("12.80")),
            new LineItemAmount(mUSSSTER.getLawId().getValue(), new BigDecimal("99.20")),
            new LineItemAmount(mUSMEDTER.getLawId().getValue(), new BigDecimal("23.20")),
            new LineItemAmount(mCASUI.getLawId().getValue(), new BigDecimal("54.40")),
            new LineItemAmount(mCAETT.getLawId().getValue(), new BigDecimal("1.60"))};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
*/
    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California 
     * using withholding status of MarriedOneIncome 4, with AEIC</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 10.00 and 40 hours</LI>
     * <LI>Use withholding of Married 4 for Federal and MarriedOneIncome 4 for CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee, with AEIC</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test(enabled=false, groups={"broken"})
    public void test_1Hourly_WithholdCAMarriedOneIncome4_WithAEIC_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 4, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIEDONEINCOME, 4, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("10.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employee tax credit
        employee.addTaxDtl(mUSAEIC); 
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("400.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00")),
            new LineItemAmount(mUSAEIC.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00")),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("24.80")),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("5.80")),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00")),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("3.20")),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("3.20")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("24.80")),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("5.80")),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("13.60")),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.40"))};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California 
     * using withholding status of Single 1, with AEIC</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 10.00 and 40 hours</LI>
     * <LI>Use withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee, with AEIC</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test(enabled=false, groups={"broken"})
    public void test_1Hourly_WithholdCASingle1_WithAEIC_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("10.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employee tax credit
        employee.addTaxDtl(mUSAEIC); 
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("400.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("35.00")),
            new LineItemAmount(mUSAEIC.getPayrollItem().getLawId().getValue(), new BigDecimal("24.00")),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("24.80")),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("5.80")),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("3.95")),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("3.20")),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("3.20")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("24.80")),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("5.80")),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("13.60")),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.40"))};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly, 1 overtime, 1 tip and all applicable taxes for California 
     * using withholding status of Single 1, with AEIC, all items contain YTD values</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 8.00 and 40 hours (containing YTD value)</LI>
     * <LI>Create an overtime wage item with a rate of 12.00 and 5 hours (containing YTD value)</LI>
     * <LI>Create a tip wage item with an amount of 250.00 (containing YTD value)</LI>
     * <LI>Use withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee, with AEIC (all containing YTD values)</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
// TODO  Fix this when AEIC is supported
/*    
    @Test
    public void test_1Hourly1Overtime1Tip_WithholdCASingle1_WithAEIC_WithYTD_PaidWeekly() {
        Worker employee = createEmployee(mCALocation, mCALocation, PayrollFrequencyEnum.WEEKLY);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItem(CompensationTypeEnum.HOURLYREG, 
        		PayrollItemType.HOURLY.getLawId().getValue(), new BigDecimal("8.00"), 40.0);
        hourlyWage.setYtdAmount(new BigDecimal("320.00"));
        CompensationDtl overtimeWage = createRateBasedWageItem(CompensationTypeEnum.HOURLYOVT, 
        		PayrollItemType.OVERTIME.getLawId().getValue(), new BigDecimal("12.00"), 5.0);
        overtimeWage.setYtdAmount(new BigDecimal("60.00"));
        CompensationDtl tipWage = createAmountBasedWageItem(CompensationTypeEnum.TIPSPAYCHECK, 
        		PayrollItemType.TIPS_PAYCHECK.getLawId().getValue(), new BigDecimal("200.00"));
        tipWage.setYtdAmount(new BigDecimal("200.00"));
        // add hourly wages
        employee.addCompensationDtl(hourlyWage);
        employee.addCompensationDtl(overtimeWage);
        employee.addCompensationDtl(tipWage);
        // add employee federal taxes
        mUSFIT.setYtdAmount(new BigDecimal("62.00"));
        employee.addTaxDtl(mUSFIT);
        mUSSSTE.setYtdAmount(new BigDecimal("35.96"));
        employee.addTaxDtl(mUSSSTE);
        mUSMEDTE.setYtdAmount(new BigDecimal("8.41"));
        employee.addTaxDtl(mUSMEDTE);
        // add employee tax credit
        mUSAEIC.setYtdAmount(new BigDecimal("7.00"));
        employee.addTaxDtl(mUSAEIC); 
        // add employer federal taxes
        mUSFUI.setYtdAmount(new BigDecimal("4.64"));
        employee.addTaxDtl(mUSFUI);
        mUSSSTER.setYtdAmount(new BigDecimal("35.96"));
        employee.addTaxDtl(mUSSSTER);
        mUSMEDTER.setYtdAmount(new BigDecimal("8.41"));
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        mCAPIT.setYtdAmount(new BigDecimal("11.57"));
        employee.addTaxDtl(mCAPIT);
        mCASDI.setYtdAmount(new BigDecimal("4.64"));
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        mCASUI.setYtdAmount(new BigDecimal("19.72"));
        employee.addTaxDtl(mCASUI);
        mCAETT.setYtdAmount(new BigDecimal("0.58"));
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(employee);
        PayrollRun payrollRunData = createPayrollRun(mPaycheckDate, sPeriodStartDate, mPeriodEndDate);       
        Paycheck paycheck = new Paycheck();
        paycheck.setPayPeriod(payrollRunData.getPayPeriod());
        paycheck = mCalcEngine.calculatePaycheck(employee, mEmployer, payrollRunData, paycheck);
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId().getValue(), new BigDecimal("320.00"), new BigDecimal("640.00")),
            new LineItemAmount(PayrollItemType.OVERTIME.getLawId().getValue(), new BigDecimal("60.00"), new BigDecimal("120.00")),
            new LineItemAmount(PayrollItemType.TIPS_PAYCHECK.getLawId().getValue(), new BigDecimal("200.00"), new BigDecimal("400.00")),
            new LineItemAmount(mUSFIT.getLawId().getValue(), new BigDecimal("62.00"), new BigDecimal("124.00")),
            new LineItemAmount(mUSAEIC.getLawId().getValue(), new BigDecimal("7.00"), new BigDecimal("14.00")),
            new LineItemAmount(mUSSSTE.getLawId().getValue(), new BigDecimal("35.96"), new BigDecimal("71.92")),
            new LineItemAmount(mUSMEDTE.getLawId().getValue(), new BigDecimal("8.41"), new BigDecimal("16.82")),
            new LineItemAmount(mCAPIT.getLawId().getValue(), new BigDecimal("11.57"), new BigDecimal("23.14")),
            new LineItemAmount(mCASDI.getLawId().getValue(), new BigDecimal("4.64"), new BigDecimal("9.28")),
            new LineItemAmount(mUSFUI.getLawId().getValue(), new BigDecimal("4.64"), new BigDecimal("9.28")),
            new LineItemAmount(mUSSSTER.getLawId().getValue(), new BigDecimal("35.96"), new BigDecimal("71.92")),
            new LineItemAmount(mUSMEDTER.getLawId().getValue(), new BigDecimal("8.41"), new BigDecimal("16.82")),
            new LineItemAmount(mCASUI.getLawId().getValue(), new BigDecimal("19.72"), new BigDecimal("39.44")),
            new LineItemAmount(mCAETT.getLawId().getValue(), new BigDecimal("0.58"), new BigDecimal("1.16"))};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
*/
    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for California 
     * using withholding status of CA Single 1, with mileage reimbursement</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a Weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Add a mileage reimbursement item to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test(enabled=false, groups={"broken"})
    public void test_1Hourly_WithholdCASingle1_WithMileage_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);
        // add employee additions
/*        TODO: Enable the following code when feature is implemented
        IAmountBasedAddition mileage = new AmountBasedAddition();
        mileage.setLawId(PayrollItemType.MILEAGEREIMBURSEMENT.getLawId());
        mileage.setAmount("100.00");
        employee.addAddition(mileage);
*/        

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("65.00")),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20")),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70")),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("12.77")),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80")),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20")),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70")),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40")),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"))};
//       TODO: Add following code to the list above when feature is implemented
//        new LineItemAmount(PayrollItemType.MILEAGEREIMBURSEMENT.getLawId().getValue(), new BigDecimal("100.00"))};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage and only federal and state income taxes using withholding status of CA Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 20.00 and 80 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and only federal and state income taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_USFIT_CAPIT_CAETT_WithholdCASingle1_PaidBiweekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;

    	Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("20.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        // add employer federal taxes
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        // add employer state taxes
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
		BigDecimal amount = new BigDecimal("1600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("206.00"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("51.56"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.60"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 hourly wage and only federal and state income taxes using withholding status
     *  of CA Single 1.  For this paycheck, the annual wage limits for FUTA, SST, CAUI, CAETT has been reached</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 1,500.00 and 80 hours</LI>
     * <LI>Use default withholding of Single 1 for both Federal and CA taxes</LI>
     * <LI>Add the wage and only federal and state income taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdCASingle1_PaidBiweekly_LimitsReached() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;

    	Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("1500.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
		BigDecimal amount = new BigDecimal("120000.00");
		BigDecimal limit1 = new BigDecimal("7000.00");
		BigDecimal limit2 = new BigDecimal("102000.00");
		BigDecimal limit3 = new BigDecimal("86698.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("41103.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("6324.00"), amount, limit2),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("1740.00"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("11873.43"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("693.58"), amount, limit3),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("56.00"), amount, limit1),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("6324.00"), amount, limit2),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("1740.00"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("238.00"), amount, limit1),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("7.00"), amount, limit1)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 salary wage and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_WithholdCASingle1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
       
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("1748.48"));
        
        BigDecimal amount = new BigDecimal("2538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
        	new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
        	new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
        	new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        	
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 salary wage and all applicable taxes for California 
     * using no withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_DontWithholdCASingle1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.NOTSET, 1, EXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.NOTSET, 1, sZeroMoney, 0, EXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
       
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("2323.96"));
        
        BigDecimal amount = new BigDecimal("2538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
        	new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
        	new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
        	new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        	
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }


    /**
     * <P>Test biweekly paycheck with 1 salary wage and all applicable taxes for California 
     * using withholding status of Married Two incomes 2</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_WithholdCAMarriedOneIncome2_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 2, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIEDONEINCOME, 2, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
       
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("2000.56"));
        
        BigDecimal amount = new BigDecimal("2538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("264.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
        	new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("59.40"), amount, amount),
        	new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
        	new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        	
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    
    /**
     * <P>Test biweekly paycheck with 1 salary wage and all applicable taxes for California 
     * using withholding status of Married Two incomes 2</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_WithholdCAMarriedTwoIncome1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIED, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
       
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("1903.48"));
        
        BigDecimal amount = new BigDecimal("2538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("285.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
        	new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
        	new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
        	new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        	
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 salary wage, a fixed Bonus and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Create a one time fixed bonus of $1000.00</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_1FixedBonus_WithholdCASingle1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);
        salaryWage.setId(1L);
        employee.addCompensationDtl(salaryWage);
        
        CompensationDtl bonusWage = createAmountBasedWageItemDtl(CompensationTypeEnum.BONUS, 
        		new BigDecimal("1000.00"), DatumType2Enum.PERPAYCHECK);
        bonusWage.setId(2L);
        //The magic setting.
        bonusWage.setFixedRate(true);
        employee.addCompensationDtl(bonusWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
       
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("2310.98"));
        
        BigDecimal amount = new BigDecimal("3538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
        	new LineItemAmount(PayrollItemType.BONUS.getLawId(), new BigDecimal("1000.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("700.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("219.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("51.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("28.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("219.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("51.31"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("120.31"), amount, amount),
        	new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("228.48"), amount, amount),
        	new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("28.31"), amount, amount),
        	new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("3.54"), amount, amount)
        	
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, a variable bonus and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Create a variable bonus payroll item</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_1VariableBonus_WithholdCASingle1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.SINGLE, 1, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);
        salaryWage.setId(1L);
        employee.addCompensationDtl(salaryWage);
        
        CompensationDtl bonusWage = createAmountBasedWageItemDtl(CompensationTypeEnum.BONUS, 
        		new BigDecimal(1000.00), DatumType2Enum.PERPAYCHECK);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        bonusWage.setId(2L);
        bonusWage.setFixedRate(false);
        employee.addCompensationDtl(bonusWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(bonusWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
       
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("2310.98"));
        
        BigDecimal amount = new BigDecimal("3538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
        	new LineItemAmount(PayrollItemType.BONUS.getLawId(), new BigDecimal("1000.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("700.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("219.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("51.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("28.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("219.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("51.31"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("120.31"), amount, amount),
        	new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("228.48"), amount, amount),
        	new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("28.31"), amount, amount),
        	new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("3.54"), amount, amount)
        	
        };
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

	/**
     * <P>Test biweekly paycheck with hourly wage and all applicable taxes for California 
     * using withholding status of Married 4 and Health Benefit</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with rate of 15.00/hour and 80 hours</LI>
     * <LI>Create a health benefit item with per paycheck deduction rate of 21.12</LI>
     * <LI>Add the wage and all applicable California taxes to the employee</LI>
     * <LI>Create a healthBenefit override as an input</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1HealthBenefitOverride_WithholdCAMarried4_PaidBiweekly() throws Exception {
    	PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mCALocation, mCALocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 4, NOTEXEMPT);
        addWorkerCAPITComplianceInfo(employee, DE4WithholdingStatusEnum.MARRIEDONEINCOME, 4, sZeroMoney, 0, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        HealthBenefitDtl pretaxHealthBenefit = createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125HEALTH, 
        			new BigDecimal("21.12"), DatumType2Enum.PERPAYCHECK);
        pretaxHealthBenefit.setId(123L);
        
        employee.addHealthBenefitDtl(pretaxHealthBenefit);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        employee.addTaxDtl(mCAPIT);
        employee.addTaxDtl(mCASDI);
        // add employer state taxes
        employee.addTaxDtl(mCASUI);
        employee.addTaxDtl(mCAETT);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        //Create an override
        CalculationInputs healthBenefitOverride = new CalculationInputs();
        healthBenefitOverride.setQuantity(null);
		Datum rateDatum = new Datum();
        rateDatum.setValue1AsUSDMoneyAmount(new BigDecimal("6.66"));
        rateDatum.setType2(DatumType2Enum.PERPAYCHECK);
        healthBenefitOverride.setRate(rateDatum);
        healthBenefitOverride.setSourcePayrollItemDtlId(pretaxHealthBenefit.getId());
        calculationInputCollection.add(healthBenefitOverride);
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
		
        BigDecimal amount = new BigDecimal("1193.34");
        // verify paycheck line items
        LineItemAmount lineItems[] = {new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(PayrollItemType.SEC125HEALTH.getLawId(), new BigDecimal("6.66")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("35.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("73.99"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.30"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.55"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.55"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("73.99"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.30"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.57"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.19"), amount, amount)};
        AssertJUnit.assertTrue(verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
}
