package com.intuit.sb.payroll.calcengine;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.workerpayments.CalculationInputs;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckInputs;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInputs;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.HealthBenefitTypeEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;



/**
 *  <P>The <CODE>TXPaycheckCalcTest</CODE> class contains tests for testing the paycheck calculation functionality
 *     for a Texas employee</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class TXPaycheckCalcTest extends CalcEngineTestBase {
    /**
     * <P>Test daily paycheck with 1 hourly wage and all applicable taxes for Texas 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a daily pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 8 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdSingle1_PaidDaily() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.DAILY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 8.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("120.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("13.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("7.44"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("1.74"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("0.96"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("7.44"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("1.74"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("3.24"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for Texas using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdSingle1_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);
        
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("65.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("16.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test semimonthly paycheck with 1 hourly wage and all applicable taxes for Texas 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a semimonthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdSingle1_PaidSemimonthly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.SEMIMONTHLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("1200.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("126.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("32.40"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 hourly wage and all applicable taxes for Texas using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a monthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 176 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdSingle1_PaidMonthly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 176.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("2640.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("2640.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("287.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("163.68"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("38.28"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("21.12"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("163.68"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("38.28"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("71.28"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test quarterly paycheck with 1 hourly wage and all applicable taxes for Texas using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a quarterly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdSingle1_PaidQuarterly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.QUARTERLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("600.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("16.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test annually paycheck with 1 hourly wage and all applicable taxes for Texas 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a annually pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdSingle1_PaidAnnually() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.ANNUALLY;

        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 96.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("1440.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1440.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("89.28"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("20.88"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("11.52"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("89.28"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("20.88"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("38.88"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and all applicable taxes for Texas 
     * using withholding status of Single 1, all items contain YTD values</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes (all containing YTD values) to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     */
    @Test
    public void test_1Hourly_WithholdSingle1_WithYTD_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("120.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        setupPaycheckDates(frequency);
        Date paycheckDate = mPaycheckDate.getTime();
        
        BigDecimal ytdWages = new BigDecimal("4800.00");
        
		setYtdAmount(hourlyWage, paycheckDate, ytdWages, ytdWages, ytdWages, 40.0);
        // add hourly wages
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        setYtdAmount(mUSFIT, paycheckDate, new BigDecimal("1275.00"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSFIT);
        setYtdAmount(mUSSSTE, paycheckDate, new BigDecimal("297.60"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSSSTE);
        setYtdAmount(mUSMEDTE, paycheckDate, new BigDecimal("69.60"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        setYtdAmount(mUSFUI, paycheckDate, new BigDecimal("38.40"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSFUI);
        setYtdAmount(mUSSSTER, paycheckDate, new BigDecimal("297.60"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSSSTER);
        setYtdAmount(mUSMEDTER, paycheckDate, new BigDecimal("69.60"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        setYtdAmount(mTXSUI, paycheckDate, new BigDecimal("129.60"), ytdWages, ytdWages, 0.0);
        employee.addTaxDtl(mTXSUI);

        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("4800.00");
        
        // verify paycheck line items
        //Note: Limits: FUI - $7000, TXSUI - $9000
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("4800.00"), new BigDecimal("9600.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("1275.00"), new BigDecimal("2550.00"),
                               amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("297.60"), new BigDecimal("595.20"),
                               amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("69.60"), new BigDecimal("139.20"),
                               amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("17.60"), new BigDecimal("56.00"),
                               amount, new BigDecimal("2200.00")),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("297.60"), new BigDecimal("595.20"),
                               amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("69.60"), new BigDecimal("139.20"),
                               amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("113.40"), new BigDecimal("243.00"),
                               amount, new BigDecimal("4200.00"))
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 2 hourly wage and all applicable taxes for Texas using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a weekly pay frequency</LI>
     * <LI>Create two hourly wage item with a rate of 15.00 and 30 hours and 12.00 and 10 hours respectively</LI>
     * <LI>Add the wages and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_2Hourly_WithholdSingle1_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage1 = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 30.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage1.setId(1L);
        CompensationDtl hourlyWage2 = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("12.00"), 10.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage2.setId(2L);
        hourlyWage2.setDescription("Hourly 2");
        employee.addCompensationDtl(hourlyWage1);
        employee.addCompensationDtl(hourlyWage2);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage1));
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage2));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("570.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("450.00"), new BigDecimal("450.00")),
            new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("120.00"), new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("60.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("35.34"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.27"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.56"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("35.34"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.27"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("15.39"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage and all applicable taxes for Texas 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_WithholdSingle1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("1904.27"));
        
        BigDecimal amount = new BigDecimal("2538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("68.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage and all applicable taxes for Texas 
     * using withholding status of Married 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_WithholdMarried1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 1, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("2059.27"));
        
        BigDecimal amount = new BigDecimal("2538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("285.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("68.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }
    
    /**
     * <P>Test biweekly paycheck with 1 salary wage and all applicable taxes for Texas 
     * using no withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Add the wage with no withholding and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary_DontWithholdSingle1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.NOTSET, 1, EXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("2344.27"));
        
        BigDecimal amount = new BigDecimal("2538.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("68.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }


    /**
     * <P>Test biweekly paycheck with 1 hourly wage and all applicable taxes for Texas using withholding status of Married 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdMarried1_PaidBiweekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("1200.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("84.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("32.40"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage and all applicable taxes for Texas 
     * using withholding status of MarriedUsingSingleRate 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdMarriedUsingSingleRate1_PaidBiweekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIEDSINGLERATE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("1200.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("32.40"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage and all applicable taxes for Texas 
     * using withholding status of MarriedUsingSingleRate 1.  For this paycheck, the annual wage limits for FUTA, SST
     * and TXUI has been reached</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 1500.00 and 80 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_WithholdMarriedUsingSingleRate1_PaidBiweekly_LimitsReached() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIEDSINGLERATE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("1500.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("120000.00");
		BigDecimal limit1 = new BigDecimal("7000.00");
		BigDecimal limit2 = new BigDecimal("102000.00");
		BigDecimal limit3 = new BigDecimal("9000.00");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), amount),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("41103.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("6324.00"), amount, limit2),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("1740.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("56.00"), amount, limit1),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("6324.00"), amount, limit2),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("1740.00"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("243.00"), amount, limit3)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage and all applicable taxes for Texas using withholding status of Don't Withhold</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly_DontWithhold_PaidBiweekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.NOTSET, 1, EXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 80.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        BigDecimal amount = new BigDecimal("1200.00");
        
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("1200.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("32.40"), amount, amount)
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage and 1 health benefit and all applicable taxes for Texas using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 40 hours</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Create an health benefit wage item with a rate of 21.12 per paycheck</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Hourly1HealthBenefit_WithholdSingle1_PaidWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.SINGLE, 1, NOTEXEMPT);
        CompensationDtl hourlyWage = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
        		new BigDecimal("15.00"), 40.0);
        //NOTE we have to set an ID on the wage or it won't be resolved on the worker.
        hourlyWage.setId(1L);
        employee.addCompensationDtl(hourlyWage);
        HealthBenefitDtl pretaxHealthBenefit = createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125HEALTH, 
    			new BigDecimal("21.12"), DatumType2Enum.PERPAYCHECK);
        employee.addHealthBenefitDtl(pretaxHealthBenefit);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);
        
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        List<CalculationInputs> calculationInputCollection = new ArrayList<CalculationInputs>();
        calculationInputCollection.add(createCalculationInputsFromCompensationDtl(hourlyWage));
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	calculationInputCollection);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("472.60"));

        BigDecimal amount = new BigDecimal("578.88");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.HOURLY.getLawId(), new BigDecimal("600.00")),
        	new LineItemAmount(PayrollItemType.SEC125HEALTH.getLawId(), new BigDecimal("21.12")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("62.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("35.89"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.39"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.63"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("35.89"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.39"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("16.20"), new BigDecimal("600.00"), new BigDecimal("600.00"))
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage and 1 health benefit and all applicable taxes for Texas 
     * using withholding status of Married 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Texas employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with annual rate of 66000.00</LI>
     * <LI>Add the wage and all applicable Texas taxes to the employee</LI>
     * <LI>Create a health benefit wage item with rate of 50.00 per paycheck</LI>
     * <LI>Call CalcEngine to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     *
     */
    @Test
    public void test_1Salary1HealthBenefit_WithholdMarried1_PaidBiWeekly() throws Exception {
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        Worker employee = createEmployee(mTXLocation, mTXLocation);
        addWorkerFITComplianceInfo(employee, W4WithholdingStatusEnum.MARRIED, 1, NOTEXEMPT);
        CompensationDtl salaryWage = createAmountBasedWageItemDtl(CompensationTypeEnum.SALARYREG, 
        		new BigDecimal("66000.00"), DatumType2Enum.YEAR);

        employee.addCompensationDtl(salaryWage);
        
        HealthBenefitDtl pretaxHealthBenefit = createHealthBenefitDtl(HealthBenefitTypeEnum.SEC125HEALTH, 
    			new BigDecimal("50.00"), DatumType2Enum.PERPAYCHECK);
        employee.addHealthBenefitDtl(pretaxHealthBenefit);
        
        // add employee federal taxes
        employee.addTaxDtl(mUSFIT);
        employee.addTaxDtl(mUSSSTE);
        employee.addTaxDtl(mUSMEDTE);
        // add employer federal taxes
        employee.addTaxDtl(mUSFUI);
        employee.addTaxDtl(mUSSSTER);
        employee.addTaxDtl(mUSMEDTER);
        // add employee state taxes
        // add employer state taxes
        employee.addTaxDtl(mTXSUI);

        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        Date paycheckDate = mPaycheckDate.getTime();
        
        final PayrollRunInfo payrollRunInfo = new PayrollRunInfo(payPeriod, paycheckDate, paycheckDate, frequency, 
				null);
		final PaycheckInputs paycheckInputs = new PaycheckInputs(employee,	null);
		final PayrollRunInputs payRunInputs = new PayrollRunInputs(mEmployer, payrollRunInfo);
		final Paycheck paycheck = mCalcEngine.calculatePaycheck(payRunInputs, 	paycheckInputs);
        
        // Verify paycheck net amount
        AssertJUnit.assertEquals(paycheck.getNetAmount().getAmount(), new BigDecimal("2021.10"));
        
        BigDecimal amount = new BigDecimal("2488.46");
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(PayrollItemType.SALARIED.getLawId(), new BigDecimal("2538.46")),
        	new LineItemAmount(PayrollItemType.SEC125HEALTH.getLawId(), new BigDecimal("50.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("277.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("154.28"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.08"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("19.91"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("154.28"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.08"), amount, amount),
            new LineItemAmount(mTXSUI.getPayrollItem().getLawId().getValue(), new BigDecimal("68.54"), new BigDecimal("2538.46"), new BigDecimal("2538.46"))
        };
        AssertJUnit.assertTrue("Paycheck line item(s) mismatch!",
                          verifyLineItems(paycheck.getPaycheckLineItems(), lineItems, "paycheck line items"));
    }

}
