package com.intuit.sb.payroll.capability.common;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.hibernate.cfg.Configuration;
import org.hibernate.ejb.Ejb3Configuration;
import org.hibernate.tool.hbm2ddl.SchemaExport;

/**
 * Drops and creates the DB tables for the EJB3 Entities in a given persistence unit
 * using the DB connection specified in that unit.
 * 
 * Connection properties can be overridden by specifying Java System properties with
 * the property name prefix of "dropandcreate.". All such properties will be passed,
 * after removing the "dropandcreate." prefix to the Hibernate configuration start up.
 */
public class DropAndCreateTables {

	/**
	 * @param args must contain a single arg that is the persistence unit name.
	 */
	public static void main(String[] args) {
		if (args.length != 1) {
			System.err.println("Usage error - supply a Persistence Unit name.");
			System.exit(1);
		}
		String persistenceUnitName = args[0];
		
		try {
			dropAndCreateTables(persistenceUnitName, getSystemPropertiesWithPrefix("dropandcreate.", true));
			System.out.println("Done!");
		}
		catch (Exception e) {/* ignored */}
	}

    /**
     * Bootstraps the Hibernate Ejb3 configuration and then drops and creates the tables for all
     * the entities defined for the given persistence unit.
     *
     * @param persistenceUnitName the name of the persistence unit from {@code persistence.xml}.
     * @param configuration The {@code Properties} used to override {@code persistence.xml}.
     * @throws Exception if there is a failure.
     */
    private static void dropAndCreateTables(String persistenceUnitName, Properties configuration) throws Exception
    {
        try
        {
            Map<?, ?> configOverrides = new HashMap<Object, Object>(configuration);

            Ejb3Configuration ejb3cfg = new Ejb3Configuration();
            ejb3cfg.configure(persistenceUnitName, configOverrides);
            Configuration cfg = ejb3cfg.getHibernateConfiguration();
            SchemaExport schemaExport = new SchemaExport(cfg);
            schemaExport.create(false, true); // false=do not report to stdout. true=immediately execute the DDL on the DB
        }
        catch (Throwable ex)
        {
            // We catch and wrap Throwable because we'd also like to catch
            // NoClassDefFoundError and other subclasses of Error
            String mesg = MessageFormat.format(
                    "Building EntityManagerFactory failed: {0}",
                    new Object[] {throwableMessageChain(ex)});
            System.err.println(mesg);
            throw new Exception(mesg);
        }
    }

    /**
     * Returns a string containing the throwable message or, if it's null,
     * the string representation of the throwable (i.e. the exception
     * class name).
     * @param t the throwable
     * @return a non-null message from the throwable
     */
    public static String throwableMessage(Throwable t)
    {
        return (t.getMessage() != null) ? t.getMessage() : t.toString();
    }

    /**
     * Returns a string containing all the nested messages in the throwable or, if it's null,
     * the string representation of the throwable (i.e. the exception class name).
     * @param t the throwable
     * @return a non-null message from the throwable
     */
    public static String throwableMessageChain(Throwable t)
    {
        StringBuffer mesg = new StringBuffer(throwableMessage(t));
        for (Throwable chainedThrowable = t.getCause(); chainedThrowable != null; chainedThrowable = chainedThrowable.getCause()) {
        	mesg.append(": ").append(throwableMessage(chainedThrowable));
        }
    	return mesg.toString();
    }

    /**
     * Gets a {@code Properties} set of all Java system properties whose name starts with the
     * specified {@code prefix}. For example, a prefix of "intuit." will return "intuit.debug"
     * but will not return "intuition.guess".
     * 
     * @param prefix The property name prefix to search for.
     * @param removePrefix if set to true the property name is stripped of the prefix when being
     * added to the returned {@code Properties} set. Otherwise the property name is used as is. 
     * @return The properties that match the prefix.
     */
    public static java.util.Properties getSystemPropertiesWithPrefix(String prefix, boolean removePrefix) {
        java.util.Properties matchedProps = new java.util.Properties();
        java.util.Enumeration<?> propNames = System.getProperties().keys();
        while (propNames.hasMoreElements()) {
            String propName = (String)propNames.nextElement();
            if (propName.startsWith(prefix)) {
                matchedProps.setProperty(
                	removePrefix ? propName.substring(prefix.length()) : propName,
                	System.getProperty(propName));
            }
        }
        return matchedProps;
    }
    public static java.util.Properties getSystemPropertiesWithPrefix(String prefix) {
        return getSystemPropertiesWithPrefix(prefix, false);
    }
}
