package com.intuit.sb.payroll.capability.common;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.jboss.deployers.spi.DeploymentException;
import org.jboss.embedded.Bootstrap;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;

/**
 * This base TestCase class will automatically bootstrap the Embedded JBoss container.
 * 
 * The subclass should implement a static @BeforeClass (and optionally @AfterClass) method
 * to handle deployment of the archives into the container. This method must add URLs for the
 * deployment archives via {@code addDeployJarFromClasspath} and then call @{code deploy()}.
 * Undeployment is automatically handled by this class before the container is shutdown.
 * 
 * This class was designed so that all tests within the class can be run at once from your IDE, or just one test method.
 *
 */
public abstract class EmbeddedContainerTestBase
{
	private static final Class<EmbeddedContainerTestBase> THIS = EmbeddedContainerTestBase.class;
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(THIS);
	private static final List<URL> jeeArchiveURLs = new ArrayList<URL>();
	private static List<URL> jeeDeployedURLs = new ArrayList<URL>();
	
	
	public EmbeddedContainerTestBase()
	{
	}

	/**
	 * Start the JBoss container.
	 * <p>
	 * Subclasses should also implement an @BeforeClass - see class doc above.
	 * @throws Exception if it fails - good luck decoding the console output and exception X(.
	 */
	public static void bootstrapEmbeddedContainer() throws Exception
	{
		bootstrap();
		ServiceLocator.setEmbedded(true);
	}

	/**
	 * Undeploy the application archives and shutdown the container.
	 * @throws Exception if something bad happens.
	 */
	public static void classTearDown() throws Exception
	{
		sLogger.debug("afterClass");
		try
		{
			for (URL jeeArchiveURL : jeeDeployedURLs)
			{
				Bootstrap.getInstance().undeploy(jeeArchiveURL);
			}
		}
		catch (DeploymentException e)
		{
			throw new InfrastructureException("Unable to undeploy", e);
		}
		if (System.getProperty("shutdown.embedded.jboss") != null) Bootstrap.getInstance().shutdown();
	}
	
	/**
	 * Adds a resource from the classpath to the list of jars to deploy to the JEE container.
	 * @param resource the resource path.
	 */
	public static void addDeployJarFromClasspath(String resource)
	{
	    URL url = THIS.getClassLoader().getResource(resource);
//	    AssertJUnit.assertNotNull("Unable to build URL for '" + resource + "'", url);
	    if (null != url)
	    {
	        jeeArchiveURLs.add(url);
	    }
	}

	/**
	 * Adds a jar from the file system to the list of jars to deploy to the JEE container.
	 * @param jarFile the file path.
	 */
	public static void addDeployJarFromFileSystem(String jarFile)
	{
		try
		{
			File newfile = new File(jarFile);
			Boolean exists = newfile.exists();
			if (exists)
			{
			    URL url = newfile.toURL(); 
			    if (null != url)
			    {
			        jeeArchiveURLs.add(url);
			    }
			}
		}
		catch(Exception ex)
		{
		    //do nothing. If the URL is not valid, the jar won't be deployed	
		}
	}

	/**
	 * Deploy the archives specified in calls to {@code addDeployJarFromClasspath}.
	 * @throws PayrollServiceFailureException if it's unable to deploy an archive.
	 */
	synchronized public static void deploy()
	{
		URL urlToDeploy = null;
	    try
		{
			for (URL jeeArchiveURL : jeeArchiveURLs)
			{
				//if the jar is already deployed, skip
				if ( ! jeeDeployedURLs.contains(jeeArchiveURL) )
				{
					urlToDeploy = jeeArchiveURL;
					sLogger.debug("Deploying Jar: " + jeeArchiveURL.toString());
					Bootstrap.getInstance().deploy(jeeArchiveURL);
					sLogger.debug("Deployed Jar: " + jeeArchiveURL.toString());
					jeeDeployedURLs.add(jeeArchiveURL);
				}
			}
		}
		catch (DeploymentException e)
		{
			sLogger.error("Error Deploying Jars", e);
			throw new InfrastructureException("Unable to deploy url " + urlToDeploy, e);
		}
	}

	synchronized protected static void deploy(String resource)
	{
	    try
		{
	    	URL url = THIS.getClassLoader().getResource(resource);
	    	//if the jar is already deployed, skip
			if ( ! jeeDeployedURLs.contains(url) )
			{
				Bootstrap.getInstance().deploy(url);
				jeeDeployedURLs.add(url);
			}
		}
		catch (DeploymentException e)
		{
			throw new InfrastructureException("Unable to deploy", e);
		}
	}
	synchronized private static void bootstrap()
	{
		sLogger.debug("startContainer");
		if (Bootstrap.getInstance().isStarted()) return;

		try
		{
			Bootstrap.getInstance().bootstrap();		
		}
		catch (Exception error)
		{
			sLogger.error(error);
			throw new RuntimeException("Failed to bootstrap", error);
		}
	}

	synchronized public static String getVersionString(String filename) {
		String version = "";
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document dom = db.parse(filename);
			Element docElement = dom.getDocumentElement();
			NodeList nl = docElement.getElementsByTagName("parent");
			if (null != nl && nl.getLength() == 1) {
				Element element = (Element)nl.item(0);
				NodeList nodeList = element.getElementsByTagName("version");
				if (null != nodeList && nodeList.getLength() == 1) {
					Element versionElement = (Element)nodeList.item(0);
					version = versionElement.getFirstChild().getNodeValue();
				}
			}
		} catch(ParserConfigurationException pce) {
    		pce.printStackTrace();
    	} catch(SAXException se) {
    		se.printStackTrace();
    	} catch(IOException ioe) {
    		ioe.printStackTrace();
    	}
		return version;
	}
	
}