package com.intuit.sb.payroll.capability.employer;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Set;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponentTestBase;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.HistPaycheck;
import com.intuit.sb.payroll.entity.model.HistPaycheckLineItem;
import com.intuit.sb.payroll.entity.model.LawId;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentLineItemSplit;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.entity.utils.PtdUtils;

public class FutaPaymentsAndLiabilitiesTest extends WorkerPaymentsComponentTestBase {

	private static Money sAmount1 = new Money("35.46");
	private static Money sAmount2 = new Money("75.11");

	// Create a Payment - test that it falls in expected region
	@Test
	public void testSaveAndGetFutaPayments() throws Exception {
		
		Calendar now = LocaleGuru.getInstance().getCalendar();
		int thisYear = now.get(Calendar.YEAR);

		Payment futaPayment = createDummyFutaPayment(PtdTypeEnum.QUARTER2, thisYear);	// payment on April 6th
		List<Payment> payments = new ArrayList<Payment>();
		payments.add(futaPayment);
		payments = sEmployerComponent.updatePayments(payments);
		// payment added to employer

		DateRange range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER1, thisYear);
		Date date = range.getStartDate();
		now.setTime(date);
		// payments based on Q1 - should return nothing
		payments = sEmployerComponent.getFutaPayments(now);
		Assert.assertNotNull(payments);
		Assert.assertEquals(payments.size(), 0);
		
		range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, thisYear);
		date = range.getStartDate();
		now.setTime(date);
		// payments based on Q3 - should return ONE
		payments = sEmployerComponent.getFutaPayments(now);
		Assert.assertNotNull(payments);
		Assert.assertEquals(payments.size(), 1);
		
		futaPayment = payments.get(0);
		validateFutaPayment(futaPayment, PtdTypeEnum.QUARTER2, thisYear);		
	}
	
	/** Check a Payment, its PaymentLineItem and PaymentLineItemSplit */
	private void validateFutaPayment(Payment payment, PtdTypeEnum quarter, int year) {
		Assert.assertNotNull(payment);
		DateRange range = PtdUtils.getDateRangeForPeriodType(quarter, year);
		Assert.assertEquals(payment.getPayrollPeriod(), range);
		Assert.assertNotNull(payment.getPaymentLineItems());
		Assert.assertEquals(payment.getPaymentLineItems().size(), 1);
		
		PaymentLineItem pli = payment.getPaymentLineItems().iterator().next();
		Assert.assertNotNull(pli);
		Assert.assertEquals(pli.getAmount(), sAmount1);
		Assert.assertEquals(pli.getPaymentType(), PaymentType.FORM940FUTA);
				
		Set<PaymentLineItemSplit> splits = pli.getPaymentLineItemSplits();
		Assert.assertNotNull(splits);
		Assert.assertEquals(splits.size(), 1);
		PaymentLineItemSplit split = splits.iterator().next();
		
		Assert.assertNotNull(split);
		Assert.assertEquals(split.getSplitAmount(), sAmount1);
	}

	@Test
	public void testGetFutaLiabilities() throws Exception {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		int thisYear = now.get(Calendar.YEAR);

		Employer employer = sEmployerComponent.getEmployer();
		HistPaycheck futaLiability = createDummyFutaLiability(PtdTypeEnum.QUARTER2, thisYear);
		if (entityService.notPersistent(futaLiability)){
			entityService.makePersistent(futaLiability);
		}else{
			entityService.makeManaged(futaLiability);
		}
		employer.addHistPaycheck(futaLiability);
		List<HistPaycheck> liabilities = new ArrayList<HistPaycheck>();
		// payment added to employer

		DateRange range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER1, thisYear);
		Date date = range.getStartDate();
		now.setTime(date);
		// payments based on Q1 - should return nothing
		liabilities = sEmployerComponent.getFutaLiabilities(now);
		Assert.assertNotNull(liabilities);
		Assert.assertEquals(liabilities.size(), 0);
		
		range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, thisYear);
		date = range.getStartDate();
		now.setTime(date);
		// payments based on Q3 - should return ONE
		liabilities = sEmployerComponent.getFutaLiabilities(now);
		Assert.assertNotNull(liabilities);
		Assert.assertEquals(liabilities.size(), 1);
		
		validateFutaLiabilities(liabilities.get(0), sAmount1, PtdTypeEnum.QUARTER2, thisYear);
	}
	
	private void validateFutaLiabilities(HistPaycheck liability, Money amount, PtdTypeEnum quarter, int year) {
		Assert.assertNotNull(liability);
		Assert.assertEquals(liability.getGrossAmount(), amount);
		DateRange range = PtdUtils.getDateRangeForPeriodType(quarter, year);
		Assert.assertEquals(liability.getPayrollPeriod(), range);
		Assert.assertEquals(liability.getHistPaycheckLineItems().size(), 1);
		
		HistPaycheckLineItem hpli = liability.getHistPaycheckLineItems().iterator().next();
		Assert.assertNotNull(hpli);
		Assert.assertEquals(hpli.getValue(), amount);
		Assert.assertEquals(hpli.getLawId().getValue(), QboeLawIdConstants.sFUTALawId);
	}

	@Test( enabled = true, expectedExceptions=ValidationException.class)
	public void testFutaPaymentsWithNullArg() throws Exception {
		sEmployerComponent.getFutaPayments(null);
	}

	@Test( enabled = true, expectedExceptions=ValidationException.class)
	public void testFutaLiabilitiesWithNullArg() throws Exception {
		sEmployerComponent.getFutaLiabilities(null);
	}
	
	@Test
	public void updateFutaPaymentsAndLiabilitiesTest() throws Exception {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		int thisYear = now.get(Calendar.YEAR);

		Payment fPayment = createDummyFutaPayment(PtdTypeEnum.QUARTER2, thisYear);
		List<Payment> fPayments = new ArrayList<Payment>();
		fPayments.add(fPayment);
		List<Payment> payments = new ArrayList<Payment>();
		payments.add(createDummyFutaPayment(PtdTypeEnum.QUARTER3, thisYear));
		
		HistPaycheck futaLiability = createDummyFutaLiability(PtdTypeEnum.QUARTER3, thisYear);
		List<HistPaycheck> liabilities = new ArrayList<HistPaycheck>();
		liabilities.add(futaLiability);
		
		sEmployerComponent.updateFutaPaymentsAndLiabilitiesYtd(payments, liabilities);
		
		DateRange range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER4, thisYear);
		Date date = range.getStartDate();
		now.setTime(date);
		payments = sEmployerComponent.getFutaPayments(now);
		liabilities = sEmployerComponent.getFutaLiabilities(now);
		
		
		Assert.assertNotNull(payments);
		Assert.assertEquals(payments.size(), 1);
		validateFutaPayment(payments.get(0), PtdTypeEnum.QUARTER3, thisYear);
		Assert.assertNotNull(liabilities);
		Assert.assertEquals(liabilities.size(), 1);
		validateFutaLiabilities(liabilities.get(0), sAmount1, PtdTypeEnum.QUARTER3, thisYear);
		
		HistPaycheck liability = liabilities.get(0);
		liability.setGrossAmount(sAmount2);
		liability.getHistPaycheckLineItems().iterator().next().setValue(sAmount2);
		sEmployerComponent.updateFutaPaymentsAndLiabilitiesYtd(payments, liabilities);
		validateFutaLiabilities(liabilities.get(0), sAmount2, PtdTypeEnum.QUARTER3, thisYear);
		
	}
	
	@Test
	public void testRemoveFutaPayments() throws Exception {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		int year = now.get(Calendar.YEAR);
		Payment fPayment = createDummyFutaPayment(PtdTypeEnum.QUARTER2, year);
		List<Payment> fPayments = new ArrayList<Payment>();
		fPayments.add(fPayment);

		HistPaycheck fLiability = createDummyFutaLiability(PtdTypeEnum.QUARTER1, year);
		List<HistPaycheck> fLiabilities = new ArrayList<HistPaycheck>();
		fLiabilities.add(fLiability);

		// add the payment
		sEmployerComponent.updateFutaPaymentsAndLiabilitiesYtd(fPayments, fLiabilities);
		
		DateRange range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, year);
		Date date = range.getStartDate();
		now.setTime(date);
		fPayments = sEmployerComponent.getFutaPayments(now);
		Assert.assertNotNull(fPayments);
		Assert.assertNotNull(fPayments.get(0));
		validateFutaPayment(fPayments.get(0), PtdTypeEnum.QUARTER2, year);
		
		sEmployerComponent.removeFutaPaymentsAndLiabilitiesYtd(now);
		fPayments = sEmployerComponent.getFutaPayments(now);
		Assert.assertNotNull(fPayments);
		Assert.assertEquals(fPayments.size(), 0);
	}
	
	@Test
	public void testRemoveFutaLiabilities() throws Exception {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		int year = now.get(Calendar.YEAR);

		Payment fPayment = createDummyFutaPayment(PtdTypeEnum.QUARTER2, year);
		List<Payment> fPayments = new ArrayList<Payment>();
		fPayments.add(fPayment);
		
		HistPaycheck fLiability = createDummyFutaLiability(PtdTypeEnum.QUARTER1, year);
		List<HistPaycheck> fLiabilities = new ArrayList<HistPaycheck>();
		fLiabilities.add(fLiability);
		// add the HistPaycheck
		sEmployerComponent.updateFutaPaymentsAndLiabilitiesYtd(fPayments, fLiabilities);
		
		DateRange range = PtdUtils.getDateRangeForPeriodType(PtdTypeEnum.QUARTER3, year);
		Date date = range.getStartDate();
		now.setTime(date);
		fLiabilities = sEmployerComponent.getFutaLiabilities(now);
		Assert.assertNotNull(fLiabilities);
		Assert.assertNotNull(fLiabilities.get(0));
		validateFutaLiabilities(fLiabilities.get(0), sAmount1, PtdTypeEnum.QUARTER1, year);
		
		sEmployerComponent.removeFutaPaymentsAndLiabilitiesYtd(now);
		fLiabilities = sEmployerComponent.getFutaLiabilities(now);
		Assert.assertNotNull(fLiabilities);
		Assert.assertEquals(fLiabilities.size(), 0);
	}
	
	private PayrollItem getPayrollItemForGivenLawid(String lawid) {
		List<PayrollItem> payrollItems = entityService.findAll(PayrollItem.class);
		PayrollItem pItem = null;
		for (PayrollItem item : payrollItems) {
			if (item.getLawId().getValue().equals(QboeLawIdConstants.sFUTALawId)) {
				pItem = item;
				break;
			}
		}
		return pItem;
	}
	
	private Payment createDummyFutaPayment(PtdTypeEnum quarter, int year) {
		
		DateRange range = PtdUtils.getDateRangeForPeriodType(quarter, year);
		Date date = range.getStartDate();
		date = DateUtil.getNextDate(date, 5);	//	5 days after quarter start
		
		PayrollItem pItem = getPayrollItemForGivenLawid(QboeLawIdConstants.sFUTALawId);
		
		Payment payment = new Payment();
		payment.setPaymentDate(date);
		payment.setPayrollPeriod(range);
		payment.setRealmOwner(sEmployer);
		
		PaymentLineItem pli = new PaymentLineItem();
		pli.setAmount(sAmount1);
		pli.setPaymentType(PaymentType.FORM940FUTA);
		pli.setRealmOwner(sEmployer);
		
		PaymentLineItemSplit pliSplit = new PaymentLineItemSplit();
		pliSplit.setDerived(false);
		pliSplit.setPayrollItem(pItem);
		pliSplit.setRealmOwner(sEmployer);
		pliSplit.setSplitAmount(sAmount1);
		
		pli.addPaymentLineItemSplit(pliSplit);
		payment.addPaymentLineItem(pli);
		
		return payment;
	}
	
	private HistPaycheck createDummyFutaLiability(PtdTypeEnum ptdTypeEnum, int thisYear) {
		HistPaycheck liability = new HistPaycheck();
		HistPaycheckLineItem hpli = new HistPaycheckLineItem();
		
		liability.addHistPaycheckLineItem(hpli);
		liability.setEmployer(sEmployer);
		liability.setGrossAmount(sAmount1);
		liability.setPayrollPeriod(PtdUtils.getDateRangeForPeriodType(ptdTypeEnum, thisYear));
		liability.setRealmOwner(sEmployer);

		LawId lawid = new LawId();
		lawid.setValue(QboeLawIdConstants.sFUTALawId);
		hpli.setLawId(lawid);
		hpli.setValue(sAmount1);
		PayrollItem pItem = getPayrollItemForGivenLawid(QboeLawIdConstants.sFUTALawId);
		hpli.setPayrollItem(pItem);

		return liability;
	}

	@Test(enabled=false, expectedExceptions = ValidationException.class)
	public void testUpdateNullPayments() throws Exception {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		int year = now.get(Calendar.YEAR);
		HistPaycheck fLiability = createDummyFutaLiability(PtdTypeEnum.QUARTER1, year);
		List<HistPaycheck> fLiabilities = new ArrayList<HistPaycheck>();
		fLiabilities.add(fLiability);
		sEmployerComponent.updateFutaPaymentsAndLiabilitiesYtd(null, fLiabilities);		
	}

	@Test(enabled=false, expectedExceptions = ValidationException.class)
	public void testUpdateNullLiabilities() throws Exception {
		Calendar now = LocaleGuru.getInstance().getCalendar();
		int year = now.get(Calendar.YEAR);
		Payment fPayment = createDummyFutaPayment(PtdTypeEnum.QUARTER1, year);
		List<Payment> fPayments = new ArrayList<Payment>();
		fPayments.add(fPayment);
		sEmployerComponent.updateFutaPaymentsAndLiabilitiesYtd(fPayments, null);		
	}
}




