package com.intuit.sb.payroll.capability.task;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.impl.EmployerComponentTasks;
import com.intuit.sb.payroll.capability.task.impl.TaskUtils;
import com.intuit.sb.payroll.capability.worker.impl.WorkerComponentTasks;
import com.intuit.sb.payroll.capability.workerpayments.CalculationInputs;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCommitRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponentTestBase;
import com.intuit.sb.payroll.capability.workerpayments.impl.WorkerPaymentsComponentTasks;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.task.TaskEAO;

/**
 * The tests below will attempt to ensure the correctness of
 *  
 * a. EnityListeners for Employer, Worker & PayrollSchedule <br>
 * b. Payroll Message Driven bean 
 * c. TaskManager (sending events) and ensuring that tasks are created. 
 *
 */
public class PayrollTasksTest extends BasicTestBase {
	
	protected TaskManager taskManager;
	protected TaskEAO taskEAO;
	protected PayrollScheduleComponent psComponent;
	protected WorkerPaymentsComponent sWorkerPaymentsComponent;

	@BeforeMethod
	public void initializePayrollTasks()
	{
		taskManager = qaBootstrap.getEjbFromClass(TaskManager.class);
		taskEAO = qaBootstrap.getEjbFromClass(TaskEAO.class);
		psComponent = qaBootstrap.getEjbFromClass(PayrollScheduleComponent.class);
		sWorkerPaymentsComponent = qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
		
	}
	
	
	@Test
	public void testHappyPathTaskSequence() throws Exception {
    	EmployerComponentTestUtils.deleteCurrentEmployer();
    	EmployerComponentTestUtils.createAndSaveNewEmployer();
		// Get the PayrollSchedule setup Task
		List<Task> companySetupTasks = taskManager.findTasksByClass(EmployerComponentTasks.COMPANY_SETUP_TASKS_CLASS);
		// 1. CompanySetup
		// 2. WorkLocation
		// 3. PayrollSchedule
		
		AssertJUnit.assertEquals("Expected 3 Tasks", 3, companySetupTasks.size());
		
		Employer employer = sEmployerComponent.getEmployer();
		Task companySetupTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.COMPANY_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("Company Setup Task does not exist", companySetupTask);

		Task payrollScheduleTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("PayrollSchedule Setup Task does not exist", payrollScheduleTask);

		Task workLocationTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.WORKLOCATION_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("Worklocation Setup Task does not exist", workLocationTask);
		
		// Create a WorkLocation and verify task state changes.
    	EmployerComponentTestUtils.createWorkLocation();
		companySetupTasks = taskManager.findTasksByClass(EmployerComponentTasks.COMPANY_SETUP_TASKS_CLASS);
		AssertJUnit.assertEquals("Expected 3 Tasks", 3, companySetupTasks.size());
		workLocationTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.WORKLOCATION_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("Worklocation Setup Task does not exist", workLocationTask);
		AssertJUnit.assertEquals(TaskStatusEnum.COMPLETED, workLocationTask.getStatus());
    	
    	//Create a Payroll Schedule and verify task state changes.
		PayrollScheduleTestUtils.createDefaultSchedule();
		companySetupTasks = taskManager.findTasksByClass(EmployerComponentTasks.COMPANY_SETUP_TASKS_CLASS);
		AssertJUnit.assertEquals("Expected 3 Tasks", 3, companySetupTasks.size());
		payrollScheduleTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("PayrollSchedule Setup Task does not exist", payrollScheduleTask);
		AssertJUnit.assertEquals(TaskStatusEnum.COMPLETED, payrollScheduleTask.getStatus());
		
		// Ensure that the setup company task is set to completed.  We're done!
		companySetupTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.COMPANY_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("Company Setup Task does not exist", companySetupTask);
		AssertJUnit.assertEquals(TaskStatusEnum.COMPLETED, companySetupTask.getStatus());

		// Get the WorkerSetup Tasks
		List<Task> workerSetups = taskManager.findTasksByClass(WorkerComponentTasks.SETUP_EMPLOYEE_CLASS);
		AssertJUnit.assertEquals("Worker Setup Tasks do not exist", 1, workerSetups.size());
		
		// Get the First Employee Setup Task
		List<Task> addFirstEmpTasks = TaskUtils.getTaskByTaskName(workerSetups, WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK, 
										sEmployerComponent.getEmployer());
		AssertJUnit.assertEquals("Setup First worker Task should not be empty", 1, addFirstEmpTasks.size());

		// Create a worker ready for pay.
    	EmployerComponentTestUtils.createPayrollReadyWorker();
		addFirstEmpTasks = TaskUtils.getTaskByTaskName(workerSetups, WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK, 
				sEmployerComponent.getEmployer());
		AssertJUnit.assertEquals("Setup First worker Task should not be empty", 1, addFirstEmpTasks.size());
		Task addFirstEmpTask = addFirstEmpTasks.get(0);
		AssertJUnit.assertEquals(TaskStatusEnum.COMPLETED, addFirstEmpTask.getStatus());
    	
		//Ready for Pay worker should yield a task to run payroll
		List<Task> runPayrollTasks = taskManager.findTasksByClass(WorkerPaymentsComponentTasks.EXECUTE_PAYROLL_RUN_CLASS);
		AssertJUnit.assertEquals("Expected one Run Payroll Task", 1, runPayrollTasks.size());
		Task runPayrollTask = runPayrollTasks.get(0);
		AssertJUnit.assertEquals(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK, runPayrollTask.getName());
		
		//Run Payroll as directed
		PayrollSchedule aSchedule = sPayrollScheduleComponent.getDefaultPayrollSchedule();
    	PayrollRunInfo payrollRunInfo = sPayrollScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
    	List<Worker> workers = sWorkerComponent.findAllWorkers();
    	Worker worker = workers.get(0);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(worker.getId(),
    					payrollRunInfo, new BigDecimal(10), new Double(80), new Double(10), new BigDecimal(0),
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
  
    	@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
    
    	// Assert that we have 3 run payroll tasks - one completed, one not and one distribute
		runPayrollTasks = taskManager.findTasksByClass(WorkerPaymentsComponentTasks.EXECUTE_PAYROLL_RUN_CLASS);
		AssertJUnit.assertEquals("Expected three Run Payroll Tasks", 3, runPayrollTasks.size());
    	List<Task> activePayrunTasks = taskManager.findActiveTasksByClassAndPayload(WorkerPaymentsComponentTasks.EXECUTE_PAYROLL_RUN_CLASS, aSchedule.getId());
    	AssertJUnit.assertEquals("Expected one active payrun tasks", 1, activePayrunTasks.size());
    	
    	//Update the payroll schedule and assert that it is updated
    	Long id = aSchedule.getId(); 
    	
    	aSchedule.setFrequency(PayrollFrequencyEnum.WEEKLY);
    	aSchedule = psComponent.updatePayrollSchedule(aSchedule);
    	AssertJUnit.assertEquals("Schedule Id did not match", id, new Long(aSchedule.getId()));
    	AssertJUnit.assertEquals("Schedule frequency not updated to Weekly", PayrollFrequencyEnum.WEEKLY, aSchedule.getFrequency());
    	activePayrunTasks = taskManager.findActiveTasksByClassAndPayload(WorkerPaymentsComponentTasks.EXECUTE_PAYROLL_RUN_CLASS, aSchedule.getId());
    	AssertJUnit.assertEquals("Expected one active payrun tasks", 1, activePayrunTasks.size());
	}
	
	@Test
	public void testDeleteLastPayrollReadyWorker() throws Exception {
		EmployerComponentTestUtils.deleteCurrentEmployer();
    	EmployerComponentTestUtils.createAndSaveNewEmployer();
		// Get the PayrollSchedule setup Task
		List<Task> companySetupTasks = taskManager.findTasksByClass(EmployerComponentTasks.COMPANY_SETUP_TASKS_CLASS);
		// 1. CompanySetup
		// 2. WorkLocation
		// 3. PayrollSchedule
		
		AssertJUnit.assertEquals("Expected 3 Tasks", 3, companySetupTasks.size());
		
		Employer employer = sEmployerComponent.getEmployer();
		Task companySetupTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.COMPANY_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("Company Setup Task does not exist", companySetupTask);

		Task payrollScheduleTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("PayrollSchedule Setup Task does not exist", payrollScheduleTask);

		Task workLocationTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.WORKLOCATION_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("Worklocation Setup Task does not exist", workLocationTask);
		
		// Create a WorkLocation and verify task state changes.
    	EmployerComponentTestUtils.createWorkLocation();
		companySetupTasks = taskManager.findTasksByClass(EmployerComponentTasks.COMPANY_SETUP_TASKS_CLASS);
		AssertJUnit.assertEquals("Expected 3 Tasks", 3, companySetupTasks.size());
		workLocationTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.WORKLOCATION_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("Worklocation Setup Task does not exist", workLocationTask);
		AssertJUnit.assertEquals(TaskStatusEnum.COMPLETED, workLocationTask.getStatus());
    	
    	//Create a Payroll Schedule and verify task state changes.
		PayrollScheduleTestUtils.createDefaultSchedule();
		companySetupTasks = taskManager.findTasksByClass(EmployerComponentTasks.COMPANY_SETUP_TASKS_CLASS);
		AssertJUnit.assertEquals("Expected 3 Tasks", 3, companySetupTasks.size());
		payrollScheduleTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("PayrollSchedule Setup Task does not exist", payrollScheduleTask);
		AssertJUnit.assertEquals(TaskStatusEnum.COMPLETED, payrollScheduleTask.getStatus());
		
		// Ensure that the setup company task is set to completed.  We're done!
		companySetupTask = getCompanySetupTask(companySetupTasks, EmployerComponentTasks.COMPANY_SETUP_TASK, employer);
		AssertJUnit.assertNotNull("Company Setup Task does not exist", companySetupTask);
		AssertJUnit.assertEquals(TaskStatusEnum.COMPLETED, companySetupTask.getStatus());

		// Get the WorkerSetup Tasks
		List<Task> workerSetups = taskManager.findTasksByClass(WorkerComponentTasks.SETUP_EMPLOYEE_CLASS);
		AssertJUnit.assertEquals("Worker Setup Tasks do not exist", 1, workerSetups.size());
		
		// Get the First Employee Setup Task
		List<Task> addFirstEmpTasks = TaskUtils.getTaskByTaskName(workerSetups, WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK, 
										sEmployerComponent.getEmployer());
		AssertJUnit.assertEquals("Setup First worker Task should not be empty", 1, addFirstEmpTasks.size());

		// Create a worker ready for pay.
    	EmployerComponentTestUtils.createPayrollReadyWorker();
		addFirstEmpTasks = TaskUtils.getTaskByTaskName(workerSetups, WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK, 
				sEmployerComponent.getEmployer());
		AssertJUnit.assertEquals("Setup First worker Task should not be empty", 1, addFirstEmpTasks.size());
		Task addFirstEmpTask = addFirstEmpTasks.get(0);
		AssertJUnit.assertEquals(TaskStatusEnum.COMPLETED, addFirstEmpTask.getStatus());
    	
		//Ready for Pay worker should yield a task to run payroll
		List<Task> runPayrollTasks = taskManager.findTasksByClass(WorkerPaymentsComponentTasks.EXECUTE_PAYROLL_RUN_CLASS);
		AssertJUnit.assertEquals("Expected one Run Payroll Task", 1, runPayrollTasks.size());
		Task runPayrollTask = runPayrollTasks.get(0);
		AssertJUnit.assertEquals(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK, runPayrollTask.getName());
		
		sWorkerComponent.deleteWorker(sWorker);
		List<Task> activeTasks = taskManager.findCurrentActiveTasks();
		AssertJUnit.assertEquals("Expected only one task", 1, activeTasks.size());
		AssertJUnit.assertEquals("Expected Add Employee Task",WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK  , activeTasks.get(0).getName());		
	}
	
    private static PaycheckCalculationRequest createPaycheckCalculationRequest(Long workerId, PayrollRunInfo payrollRunInfo,
			BigDecimal hourlyRate, Double hours, Double overtimeHours, BigDecimal bonusAmount,
			PayrollDistributionMethodEnum distributionMethod)
		throws Exception {
		PaycheckCalculationRequest paycheckCalculationRequest = new PaycheckCalculationRequest();
		Worker worker = sWorkerComponent.findWorker(workerId);
		Set<CompensationDtl> compensationDtls = worker.getCompensationDtls();
		List<CalculationInputs> calculationInputs = new ArrayList<CalculationInputs>();
		for (CompensationDtl compensationDtl : compensationDtls) {
			// Copy the datums to their transient siblings for calculation purposes.
			CalculationInputs calcInputs = new CalculationInputs();
			Datum calcRate = new Datum();
			calcInputs.setRate(calcRate);
			Datum calcQty = new Datum();
			calcInputs.setQuantity(calcQty);
			WorkerPaymentsComponentTestBase.setCompensationRateAndQuantity(compensationDtl, calcRate, calcQty, hourlyRate, hours, overtimeHours,
							bonusAmount);
			calcInputs.setSourcePayrollItemDtlId(compensationDtl.getId());
			calculationInputs.add(calcInputs);
		}
		// Need to clone the dates here, otherwise changing values in PayrollRunInfo will affect the dates in the
		//   PaycheckCalculationRequest
		paycheckCalculationRequest.setPaycheckDate((Date)payrollRunInfo.getPayDate().clone());
		DateRange period = payrollRunInfo.getPayPeriod();
		DateRange payPeriod = new DateRange(period.getStartDate(), period.getEndDate());
		paycheckCalculationRequest.setPayPeriod(payPeriod);
		paycheckCalculationRequest.setCalculationInputs(calculationInputs);
		paycheckCalculationRequest.setWorkerId(workerId);
		paycheckCalculationRequest.setFrequency(payrollRunInfo.getFrequency());
		paycheckCalculationRequest.setDistributionMethod(distributionMethod);
		
		return paycheckCalculationRequest;     
}
	
	private Task getCompanySetupTask(List<Task> alltasks, String companySetupTask,
			Employer employer) throws ServiceFailureException {
		List<Task> tasks = TaskUtils.getTaskByTaskName(alltasks, companySetupTask, employer);
		Task task = null;
		if (tasks.size() == 1){
			task = tasks.get(0);
			
		}else if (tasks.size() > 1){
			throw new ServiceFailureException("More than one Payroll Ready Task found for a worker");
		}
		return task;
	}
}
