package com.intuit.sb.payroll.capability.task;

import java.util.Date;
import java.util.List;

import javax.transaction.NotSupportedException;
import javax.transaction.SystemException;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.common.PayrollTestBase;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.model.TaskTypeEnum;
import com.intuit.sb.payroll.entity.task.TaskEAO;

public class TaskManagerTest extends PayrollTestBase {

	protected static Logger sLogger = LogManager.getLogger(TaskManagerTest.class);
	
	protected TaskManager sTaskManager;
	
	protected TaskEAO sTaskEao;
	
	static private final long ONE_DAY = 1000 * 60 * 60 * 24; // number of milliseconds in a day
	static private final long ONE_WEEK = ONE_DAY * 7; // number of milliseconds in a week 
	
	static private final long CURRENT_TASKS_LOOK_BEHIND_TIME_DEFAULT = ONE_WEEK; // look backwards 1 week 
	static private final long CURRENT_TASKS_LOOK_AHEAD_TIME_DEFAULT = ONE_WEEK * 8; // look forward 8 weeks
	
    @BeforeMethod
    public void setUpTaskService() {
    	sTaskEao = qaBootstrap.getEjbFromClass(TaskEAO.class);
    	sTaskManager = qaBootstrap.getEjbFromClass(TaskManager.class);
    	try {
			getUserTransaction().begin();
		} catch (NotSupportedException e) {
			sLogger.error("Error Rolling back user transaction\n" + e.getLocalizedMessage());
		} catch (SystemException e) {
			sLogger.error("Error Rolling back user transaction\n" + e.getLocalizedMessage());		
		}
	}
    
    @AfterMethod
    public void teardown() {
    	try {
			getUserTransaction().rollback();
		} catch (IllegalStateException e) {
			sLogger.error("Error Rolling back user transaction\n" + e.getLocalizedMessage());
		} catch (SecurityException e) {
			sLogger.error("Error Rolling back user transaction\n" + e.getLocalizedMessage());
		} catch (SystemException e) {
			sLogger.error("Error Rolling back user transaction\n" + e.getLocalizedMessage());
		}
    }
	
    /**
     * Test to create new task in db and then cancel it
     */
	@Test
	public void addAndCancelTaskTest() throws Exception {
		Task task = createNonScheduledTask(new Date());
		task = sTaskManager.addTask(task);
		AssertJUnit.assertTrue("Able to persist task", task.getName().equals("Test NonScheduled Task"));
		task = sTaskManager.cancelTask(task.getId());
		AssertJUnit.assertEquals("Cancelled Task successfully", TaskStatusEnum.CANCELED, task.getStatus());
		AssertJUnit.assertNotNull(task);
	}
	
	@Test
	public void currentTasksTest() {
		Long today = System.currentTimeMillis(); // Use UTC as that's the time we store in the Task records
		Date fromDate = new Date(today - CURRENT_TASKS_LOOK_BEHIND_TIME_DEFAULT);
		Date toDate = new Date(today + CURRENT_TASKS_LOOK_AHEAD_TIME_DEFAULT);
		List<Task> oldtasks = sTaskManager.findCurrentTasksInDateRange(fromDate, toDate);
		Task task1 = createNonScheduledTask(DateUtil.getNowDate());
		Task task2 = createNonScheduledTask(DateUtil.createDate(2008, 5, 11));	//	11th June 2008
		
		task1.setStatus(TaskStatusEnum.STARTED);
		task2.setStatus(TaskStatusEnum.STARTED);
		task1.setDeadline(DateUtil.createDate(2008, 10, 25));
		task2.setDeadline(DateUtil.createDate(2009, 10, 26));
		
		task1 = sTaskManager.addTask(task1);
		task2 = sTaskManager.addTask(task2);
		AssertJUnit.assertNotNull(task1);
		AssertJUnit.assertNotNull(task2);
		
		List<Task> tasks = sTaskManager.findCurrentTasksInDateRange(fromDate, toDate);
		AssertJUnit.assertEquals(1+ oldtasks.size(), tasks.size());
	}
	
	
	@Test
	public void overdueTasksTest() throws Exception {
		List<Task> oldtasks = sTaskManager.getOverdueTasks();
		Task task1 = createNonScheduledTask(DateUtil.getNowDate());
		Task task2 = createNonScheduledTask(DateUtil.createDate(2008, 5, 11));	//	11th June 2008
		
		task1.setStatus(TaskStatusEnum.STARTED);
		task2.setStatus(TaskStatusEnum.STARTED);
		task1.setDeadline(DateUtil.createDate(2008, 5, 25));
		task2.setDeadline(DateUtil.createDate(2009, 10, 26));
	
	
		task1 = sTaskManager.addTask(task1);
		task2 = sTaskManager.addTask(task2);
		AssertJUnit.assertNotNull(task1);
		AssertJUnit.assertNotNull(task2);
		
		List<Task> tasks = sTaskManager.getOverdueTasks();
		AssertJUnit.assertEquals(1+oldtasks.size(), tasks.size());
			
	}
	
    /**
     * Test to create new task in db and close it
     */
	@Test
	public void addAndCloseTaskTest() throws Exception {
		Task task = createNonScheduledTask(new Date());
		task = sTaskManager.addTask(task);
		AssertJUnit.assertTrue("Able to persist task", task.getName().equals("Test NonScheduled Task"));
		task = sTaskManager.closeTask(task.getId());
		AssertJUnit.assertEquals("Closed Task successfully", TaskStatusEnum.COMPLETED, task.getStatus());
		AssertJUnit.assertNotNull(task);
	}
	/**
	 * Method to create Task bean to be used by test methods with given due date
	 * @param dueDate
	 * @return
	 */
    protected Task createNonScheduledTask(Date dueDate) {
		Task task = new Task();
		task.setName("Test NonScheduled Task");
		task.setType(TaskTypeEnum.NONSCHEDULED);
		task.setDescription("Sample non scheduled test task");
		task.setCompletionDate(dueDate);
		return task;
	}
}
