package com.intuit.sb.payroll.capability.taxes.withholding;

import java.math.BigDecimal;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * <P>
 * The <CODE>AddWorkerComplianceDocDtlTest</CODE> class contains tests for testing the Exceptions that can be thrown
 * by addWorkerComplianceDocDtl() in TaxWithholdingsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class AddWorkerComplianceDocDtlExceptionsTest extends TaxWithholdingsComponentTestBase {
	private static Worker sEmployee;

    @BeforeMethod
    public static void initializeBeforeTest() {
    	sEmployee = WorkerTestUtils.createEmployee();
    }
    
    @AfterMethod
    public static void cleanUpAfterTest() {
    	sEmployee = null;
    }

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with a null Worker</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with a null Worker</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWorker() throws Exception {
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(null, workerComplianceDocDtl);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with a null WorkerComplianceDocDtl</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with a null WorkerComplianceDocDtl</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWorkerComplianceDocDtl() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, null);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with no taxes applied</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with no taxes applied</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that WorkflowException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = WorkflowException.class)
	public void testTaxesNotApplied() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, workerComplianceDocDtl);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with no WorkLocation set</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with no WorkLocation set</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that WorkflowException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = WorkflowException.class)
	public void testNoWorkLocationSet() throws Exception {
		// Setup
		WorkerComplianceDocDtl workerComplianceDocDtl = sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, workerComplianceDocDtl);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with a negative W4 allowance</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with a negative W4 allowance</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeW4Allowance() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		w4Detail.setNoOfAllowances(-1);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with a negative W4 extra withholding</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with a negative W4 extra withholding</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeW4ExtraWithholding() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		w4Detail.setNoOfAllowances(1);
		Money w4AdditionalAmount = new Money();
		w4AdditionalAmount.setAmount(new BigDecimal("-100.00"));
		w4Detail.setAdditionalAmount(w4AdditionalAmount);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with the WithholdingStatus set to null</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with the WithholdingStatus set to null</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullW4WithholdingStatus() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		w4Detail.setWithholdingStatus(null);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with the WithholdingStatus not set</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with the WithholdingStatus not set</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testW4WithholdingStatusNotSet() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.NOTSET);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with a negative DE4 allowance</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with a negative DE4 allowance</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeDE4Allowance() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	DE4Detail de4Detail = (DE4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		de4Detail.setNoOfAllowances(-1);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with the WithholdingStatus set to null</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with the WithholdingStatus set to null</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullDE4WithholdingStatus() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	DE4Detail de4Detail = (DE4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		de4Detail.setWithholdingStatus(null);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with the WithholdingStatus not set</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with the WithholdingStatus not set</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testDE4WithholdingStatusNotSet() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	DE4Detail de4Detail = (DE4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		de4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.NOTSET);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with a negative DE4 extra withholding</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with a negative DE4 extra withholding</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNegativeDE4ExtraWithholding() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	DE4Detail de4Detail = (DE4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		de4Detail.setNoOfAllowances(1);
		Money de4AdditionalAmount = new Money();
		de4AdditionalAmount.setAmount(new BigDecimal("-100.00"));
		de4Detail.setAdditionalAmount(de4AdditionalAmount);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with a previously saved WorkerComplianceDocDtl</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4Detail to the worker
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with a previously saved WorkerComplianceDocDtl</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testWithSavedComplianceDocDtl() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		w4Detail.setNoOfAllowances(1);
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(false);
		w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
		// Add it to the worker
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
		sEmployee = sWorkerComponent.updateWorker(sEmployee);
		W4Detail savedW4Detail = getW4Detail(sEmployee);
    	// Exercise
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, savedW4Detail);
	}

	/**
	 * <P>Test calling addWorkerComplianceDocDtl() with a WorkerComplianceDocDtl that exists on the worker already</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a W4Detail to the worker
	 * <LI>Call TaxWithholdingsComponent.addWorkerComplianceDocDtl() with a WorkerComplianceDocDtl that exists on the
	 *  worker already</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testWithComplianceDocDtlThatExistsAlready() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.W4IRS);
		w4Detail.setNoOfAllowances(1);
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(false);
		w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
		// Add it to the worker
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
		sEmployee = sWorkerComponent.updateWorker(sEmployee);
    	// Exercise
		// Try to add it when one exists already
		sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
	}

}
