package com.intuit.sb.payroll.capability.taxes.withholding;

import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;

/**
 * <P>
 * The <CODE>UpdateWorkerComplianceDocDtlTest</CODE> class contains tests for testing the
 * addWorkerComplianceDocDtl() in TaxWithholdingsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class UpdateWorkerComplianceDocDtlTest extends TaxWithholdingsComponentTestBase {
	private static Worker sEmployee;

    @BeforeMethod
    public static void initializeBeforeTest() {
    	sEmployee = WorkerTestUtils.createEmployee();
    }
    
    @AfterMethod
    public static void cleanUpAfterTest() {
    	sEmployee = null;
    }

	/**
	 * <P>Test calling UpdateWorkerComplianceDocDtl() to update a W4Detail with no existing DE4Detail</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Add a W4Detail to the worker</LI>
	 * <LI>Call the TaxWithholdingsComponent.updateWorkerComplianceDocDtl() to update a W4Detail</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the W4Detail is updated on the worker</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateW4WithNoDE4Detail() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Setup a W4Detail
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(ComplianceDocIdEnum.W4IRS);
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(false);
		w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
		w4Detail.setNoOfAllowances(1);
		// Add the W4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
		WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		sEmployee = workerComponent.updateWorker(sEmployee);
		W4Detail savedW4Detail = getW4Detail(sEmployee);
    	// Exercise
		savedW4Detail.setTaxExempt(true);
		sEmployee = sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, savedW4Detail);
		// Verify
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = sEmployee.getWorkerComplianceDocDtls();
		AssertJUnit.assertEquals("Number of ComplianceDocDtls mismatch!", 1, workerComplianceDocDtls.size());
		W4Detail actual = getW4Detail(sEmployee);
		AssertJUnit.assertTrue("ComplianceDocDtl mismatch!", verifyW4Detail(savedW4Detail, actual));
	}

	/**
	 * <P>Test calling UpdateWorkerComplianceDocDtl() to update a DE4Detail</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Add a DE4Detail to the worker</LI>
	 * <LI>Call the TaxWithholdingsComponent.updateWorkerComplianceDocDtl() to update a DE4Detail</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a DE4Detail is updated on the worker</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateDE4() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Setup a DE4Detail
    	DE4Detail de4Detail = (DE4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		de4Detail.setDerived(false);
		de4Detail.setTaxExempt(false);
		de4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.SINGLE);
		de4Detail.setNoOfAllowances(1);
		// Add the DE4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
		WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		sEmployee = workerComponent.updateWorker(sEmployee);
		DE4Detail savedDe4Detail = getDe4Detail(sEmployee);
		AssertJUnit.assertNotNull("Can't find DE4Detail that was just added!", savedDe4Detail);
    	// Exercise
		savedDe4Detail.setNoOfAllowances(2);
		sEmployee = sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, savedDe4Detail);
		// Verify
		DE4Detail actual = getDe4Detail(sEmployee);
		AssertJUnit.assertTrue("ComplianceDocDtl mismatch!", verifyDE4Detail(savedDe4Detail, actual));
	}

	/**
	 * <P>Test calling UpdateWorkerComplianceDocDtl() to update a W4Detail with an existing non-derived DE4Detail</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the worker's WorkLocation</LI>
	 * <LI>Apply the applicable taxes</LI>
	 * <LI>Add a W4Detail to the worker</LI>
	 * <LI>Call the TaxWithholdingsComponent.updateWorkerComplianceDocDtl() to update a W4Detail</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the W4Detail is updated and the non-derived DE4Detail is not affected on the worker</LI>
	 * </UL>
	 */
	@Test
	public void testUpdateW4WithExistingNonDerivedDE4Detail() throws Exception {
		// Setup
    	// Set the workLocation
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	sEmployee.setWorkLocation(workLocation);
    	// Apply applicable taxes
    	sEmployee = sTaxComponent.applyApplicableTaxes(sEmployee);
    	// Setup a W4Detail
    	W4Detail w4Detail = (W4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(ComplianceDocIdEnum.W4IRS);
		w4Detail.setDerived(false);
		w4Detail.setTaxExempt(false);
		w4Detail.setWithholdingStatus(W4WithholdingStatusEnum.SINGLE);
		w4Detail.setNoOfAllowances(1);
		// Add the W4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, w4Detail);
    	// Setup a DE4Detail
    	DE4Detail de4Detail = (DE4Detail)sTaxWithholdingsComponent.createWorkerComplianceDocDtl(
				ComplianceDocIdEnum.DE4);
		de4Detail.setDerived(false);
		de4Detail.setTaxExempt(false);
		de4Detail.setWithholdingStatus(DE4WithholdingStatusEnum.SINGLE);
		de4Detail.setNoOfAllowances(1);
		// Add the DE4Detail
		sEmployee = sTaxWithholdingsComponent.addWorkerComplianceDocDtl(sEmployee, de4Detail);
		WorkerComponent workerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
		sEmployee = workerComponent.updateWorker(sEmployee);
		W4Detail savedW4Detail = getW4Detail(sEmployee);
    	// Exercise
		savedW4Detail.setWithholdingStatus(W4WithholdingStatusEnum.MARRIED);
		savedW4Detail.setNoOfAllowances(2);
		sEmployee = sTaxWithholdingsComponent.updateWorkerComplianceDocDtl(sEmployee, savedW4Detail);
		// Verify
		Set<WorkerComplianceDocDtl> workerComplianceDocDtls = sEmployee.getWorkerComplianceDocDtls();
		AssertJUnit.assertEquals("Number of ComplianceDocDtls mismatch!", 2, workerComplianceDocDtls.size());
		W4Detail actual = getW4Detail(sEmployee);
		AssertJUnit.assertTrue("ComplianceDocDtl mismatch!", verifyW4Detail(savedW4Detail, actual));
		DE4Detail actualDe4Detail = getDe4Detail(sEmployee);
		AssertJUnit.assertTrue("ComplianceDocDtl mismatch!", verifyDE4Detail(de4Detail, actualDe4Detail));
	}

}
