package com.intuit.sb.payroll.capability.worker;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>FindWorkersByWorkLocationTest</CODE> class contains tests for testing findWorkersByWorkLocation()
 *   in WorkerComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class FindWorkersByWorkLocationTest extends WorkerComponentTestBase
{
	private static Long sReadyForPayWorkerId = 0L;
	private static Long sReadyForPayWorkerId2 = 0L;

	/**
	 * One-time set up for this class.
	 *   Creates 2 payroll ready workers (1 active and 1 inactive, both in "Mountain View, CA")
	 */
    @BeforeClass
    public static void setUpFindWorkersByWorkLocationTestClass() {
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            // Create an active payroll ready worker
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
            // Create an inactive payroll ready worker
			Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Paul", sMountainViewLocation);
			sReadyForPayWorkerId2 = worker.getId();
			worker.setActive(false);
			sWorkerComponent.updateWorker(worker);
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpFindWorkersByWorkLocationTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * One-time tear down for this class.
     *   Deletes the 2 workers that were created in this class
     */
    @AfterClass
    public static void tearDownFindWorkersByWorkLocationTestClass() {
        try {
            getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			if (0 != sReadyForPayWorkerId2) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId2);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId2 = 0L;
			}
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownFindWorkersByWorkLocationTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * <P>Test finding all workers in "San Francisco, CA"</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Determine the id of the "San Francisco, CA" work location</LI>
     * <LI>Call WorkerComponent.findWorkersByWorkLocation() with the work location id to find all workers</LI>
     * <LI>Verify the number of workers returned match the expected value</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.findWorkersByWorkLocation"})
    public void testFindWorkersInSanFrancisco() throws Exception {
    	// There are 3 workers (1 in San Francisco, 1 in Mountain View and 1 in Mesquite) that were seeded by SeedData,
    	//   and 2 payroll ready employees (both in Mountain View) created in setUpFindWorkersByFilterTestClass()
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sSanFranciscoLocation);
    	List<Worker> workers = sWorkerComponent.findWorkersByWorkLocation(workLocation.getId());
        AssertJUnit.assertEquals("Expected number of workers mismatch!", 1, workers.size());
    }

    /**
     * <P>Test finding all workers in "Mountain View, CA"</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Determine the id of the "Mountain View, CA" work location</LI>
     * <LI>Call WorkerComponent.findWorkersByWorkLocation() with the work location id to find all workers</LI>
     * <LI>Verify the number of workers returned match the expected value</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.findWorkersByWorkLocation"})
    public void testFindWorkersInMountainView() throws Exception {
    	// There are 3 workers (1 in San Francisco, 1 in Mountain View and 1 in Mesquite) that were seeded by SeedData,
    	//   and 2 payroll ready employees (both in Mountain View) created in setUpFindWorkersByFilterTestClass()
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMountainViewLocation);
    	List<Worker> workers = sWorkerComponent.findWorkersByWorkLocation(workLocation.getId());
        AssertJUnit.assertEquals("Expected number of workers mismatch!", 3, workers.size());
    }


    /**
     * <P>Test finding all workers in "Mesquite, TX"</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Determine the id of the "Mesquite, TX" work location</LI>
     * <LI>Call WorkerComponent.findWorkersByWorkLocation() with the work location id to find all workers</LI>
     * <LI>Verify the number of workers returned match the expected value</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.findWorkersByWorkLocation"})
    public void testFindWorkersInMesquite() throws Exception {
    	// There are 3 workers (1 in San Francisco, 1 in Mountain View and 1 in Mesquite) that were seeded by SeedData,
    	//   and 2 payroll ready employees (both in Mountain View) created in setUpFindWorkersByFilterTestClass()
    	WorkLocation workLocation = WorkerTestUtils.getNamedWorkLocation(sMesquiteLocation);
    	List<Worker> workers = sWorkerComponent.findWorkersByWorkLocation(workLocation.getId());
        AssertJUnit.assertEquals("Expected number of workers mismatch!", 1, workers.size());
    }

    /**
     * <P>Test finding all workers in a non-existent location</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerComponent.findWorkersByWorkLocation() with an unknown work location id</LI>
     * <LI>Verify the number of workers returned is zero</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.findWorkersByWorkLocation"})
    public void testFindWorkersWithNonExistentId() throws Exception {
    	// There are 3 workers (1 in San Francisco, 1 in Mountain View and 1 in Mesquite) that were seeded by SeedData,
    	//   and 2 payroll ready employees (both in Mountain View) created in setUpFindWorkersByFilterTestClass()
    	List<Worker> workers = sWorkerComponent.findWorkersByWorkLocation(9198493849L);
        AssertJUnit.assertEquals("Number of workers returned is not zero!", 0, workers.size());
    }

}
