package com.intuit.sb.payroll.capability.worker;

import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>HireExistingWorkerExceptionsTest</CODE> class contains tests for testing exceptions that can be thrown
 *     by hireExistingWorker() in WorkerComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class HireExistingWorkerExceptionsTest extends WorkerComponentTestBase
{

	/**
	 * One-time set up for this class.
	 */
    @BeforeClass
    public static void setUpHireExistingWorkerExceptionsTestClass() {
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpHireExistingWorkerExceptionsTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * One-time tear down for this class.
     */
    @AfterClass
    public static void tearDownHireExistingWorkerExceptionsTestClass() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownHireExistingWorkerExceptionsTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * <P>Test hiring a worker when the worker's hire date is null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() with a null hireDate</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"}, expectedExceptions=ValidationException.class)
    public void testHiringWorkerWithNullHireDate() throws Exception {
    	// Setup
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		// Terminate the worker
		Date terminationDate = DateUtil.getNowDate();
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    	// Exercise
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), null);
    }

    /**
     * <P>Test hiring a worker when the worker's EmploymentPeriod is null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() with the worker's EmploymentPeriod set to null</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"}, expectedExceptions=ValidationException.class)
    public void testHiringWorkerWithNullEmploymentPeriod() throws Exception {
    	// Setup
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		// Terminate the worker
		Date terminationDate = DateUtil.getNowDate();
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    	// Exercise
    	worker.setEmploymentPeriod(null);
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), DateUtil.getNowDate());
    }

    /**
     * <P>Test hiring a worker that was deleted</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Delete the worker</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() with the deleted worker's id</LI>
     * <LI>Verify that ServiceFailureException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"}, expectedExceptions=ServiceFailureException.class)
    public void testHiringDeletedWorker() throws Exception {
    	// Setup
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		// Terminate the worker
		Date terminationDate = DateUtil.getNowDate();
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    	// Delete the worker
    	WorkerTestUtils.cleanUpWorker(worker);
    	// Exercise
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), DateUtil.getNowDate());
    }

    /**
     * <P>Test hiring a worker when the worker is employed</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() with the employed worker</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"}, expectedExceptions=ValidationException.class)
    public void testHiringEmployedWorker() throws Exception {
    	// Setup
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
    	// Exercise
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), DateUtil.getNowDate());
    }

    /**
     * <P>Test hiring a worker when the new hireDate is before the previous hireDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call WorkerComponent.hireExistingWorker() with a hireDate before the previous hireDate</LI>
     * <LI>Verify that ValidationException is thrown</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.hireExistingWorker"}, expectedExceptions=ValidationException.class)
    public void testHiringWorkerWithHireDateBeforePreviousHireDate() throws Exception {
    	// Setup
        // Create an active payroll ready worker
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		// Calculate a hireDate that is before the original hireDate
		Date hireDate = DateUtil.getNextDate(worker.getEmploymentPeriod().getStartDate(), -7);
		// Terminate the worker
		Date terminationDate = DateUtil.getNowDate();
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    	// Exercise
    	worker = sWorkerComponent.hireExistingWorker(worker.getId(), hireDate);
    }

}
