package com.intuit.sb.payroll.capability.worker;

import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.EmploymentStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 *  <P>The <CODE>TerminateWorkerTest</CODE> class contains tests for testing terminateWorker() in WorkerComponent</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class TerminateWorkerTest extends WorkerComponentTestBase
{

	/**
	 * One-time set up for this class.
	 */
    @BeforeClass
    public static void setUpTerminateWorkerTestClass() {
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        
    	try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpTerminateWorkerTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * One-time tear down for this class.
     */
    @AfterClass
    public static void tearDownTerminateWorkerTestClass() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownTerminateWorkerTestClass(): " + t.getMessage());
        }
    }
    
    /**
     * <P>Test terminating a worker using today's date as the termination date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Call WorkerComponent.terminateWorker() to terminate the worker with today's date</LI>
     * <LI>Verify expected results</LI>
     * </OL>
     * 
     */
    @Test(groups = {"worker.terminateWorker"})
    public void testTerminateWorkerToday() throws Exception {
    	// Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		Date terminationDate = DateUtil.getNowDate();
		// Exercise
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    	// Verify
    	AssertJUnit.assertFalse("Worker should have been marked inactive!", worker.isActive());
    	AssertJUnit.assertTrue("Current EmploymentPeriod start date mismatch!",
    			DateUtil.isSameDay(WorkerTestUtils.HIRE_DATE, worker.getEmploymentPeriod().getStartDate()));
    	AssertJUnit.assertTrue("Current EmploymentPeriod end date mismatch!", DateUtil.isSameDay(terminationDate,
    			worker.getEmploymentPeriod().getEndDate()));
    	AssertJUnit.assertEquals("Worker's employment status was not set to TERMINATED!",
    			EmploymentStatusEnum.TERMINATED, worker.getEmploymentStatus());
    }

    /**
     * <P>Test terminating a worker using a termination date in the future</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Call WorkerComponent.terminateWorker() to terminate the worker with a future date</LI>
     * <LI>Verify expected results</LI>
     * </OL>
     * 
     */
    //TODO  Enable this test when the engine code handle changing the status on the actual termination date
    @Test(groups = {"worker.terminateWorker"}, enabled=false)
    public void testTerminateWorkerWithFutureDate() throws Exception {
    	// Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Johnny", sMountainViewLocation);
		// Set the terminationDate to 2 days from today
		Date terminationDate = DateUtil.getNextDate(DateUtil.getNowDate(), 2);
		// Exercise
    	worker = sWorkerComponent.terminateWorker(worker, terminationDate);
    	// Verify
    	AssertJUnit.assertTrue("Worker should still be marked active!", worker.isActive());
    	AssertJUnit.assertTrue("Current EmploymentPeriod start date mismatch!",
    			DateUtil.isSameDay(WorkerTestUtils.HIRE_DATE, worker.getEmploymentPeriod().getStartDate()));
    	AssertJUnit.assertTrue("Current EmploymentPeriod end date mismatch!", DateUtil.isSameDay(terminationDate,
    			worker.getEmploymentPeriod().getEndDate()));
    	AssertJUnit.assertEquals("Worker's employment status should still be EMPLOYED!",
    			EmploymentStatusEnum.EMPLOYED, worker.getEmploymentStatus());
    }

}
