package com.intuit.sb.payroll.capability.worker;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.impl.WorkerSetupRules;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.RequiredData;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.PersonTaxId;
import com.intuit.sb.payroll.entity.model.PersonTaxIdEnum;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * Tests WorkerSetup
 * 
 */
public class WorkerSetupTest {
	@Test
	public void testGetRequiredDataForWorkflow() throws Exception {
		List<RequiredData> requiredDataForWorkflow = WorkerSetupRules
				.getRequiredDataForWorkflow(WorkerWorkflowType.DetermineTaxes);
		AssertJUnit.assertEquals(2, requiredDataForWorkflow.size());

		requiredDataForWorkflow = WorkerSetupRules.getRequiredDataForWorkflow(WorkerWorkflowType.Payroll);
		AssertJUnit.assertEquals(5, requiredDataForWorkflow.size());
	}

	@Test
	public void testGetMissingDataForTaxWorkflow() throws Exception {
		Worker worker = new Employee();
		List<RequiredData> missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(
				WorkerWorkflowType.DetermineTaxes, worker);
		AssertJUnit.assertEquals(2, missingDataForWorkflow.size());

		// Add an address
		GlobalPostalAddress taxResidenceGlobalAddress = new GlobalPostalAddress();
		taxResidenceGlobalAddress.setCountrySubDivisionCode("CA");
		worker.setTaxResidenceAddress(taxResidenceGlobalAddress);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.DetermineTaxes, worker);
		AssertJUnit.assertEquals(1, missingDataForWorkflow.size());

		// Add a work location
		WorkLocation workLocation = new WorkLocation();
		workLocation.setTaxResidenceAddress(taxResidenceGlobalAddress);
		worker.setWorkLocation(workLocation);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.DetermineTaxes, worker);
		AssertJUnit.assertEquals(0, missingDataForWorkflow.size());

		boolean actual = WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.DetermineTaxes, worker);
		AssertJUnit.assertTrue("Worker should be ready for Taxes", actual);
	}

	@Test
	public void testGetMissingDataForEmployeeTaxReadyWorkflow() throws Exception {
		Worker worker = new Employee();
		List<RequiredData> missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.PTSTaxReady,worker);
		AssertJUnit.assertEquals(6, missingDataForWorkflow.size());
		
		//Add Social Security number
		PersonTaxId personTaxId = new PersonTaxId();
		personTaxId.setType(PersonTaxIdEnum.SSN);
		personTaxId.setValue("123456789");
		worker.setPersonTaxId(personTaxId);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.PTSTaxReady, worker);
		AssertJUnit.assertEquals(5, missingDataForWorkflow.size());
				
		// Add an address (liveLocation)
		GlobalPostalAddress taxResidenceGlobalAddress = new GlobalPostalAddress();
		taxResidenceGlobalAddress.setCountrySubDivisionCode("CA");
		worker.setTaxResidenceAddress(taxResidenceGlobalAddress);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.PTSTaxReady, worker);
		AssertJUnit.assertEquals(4, missingDataForWorkflow.size());
		
		// Add a work location
		WorkLocation workLocation = new WorkLocation();
		workLocation.setTaxResidenceAddress(taxResidenceGlobalAddress);
		worker.setWorkLocation(workLocation);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.PTSTaxReady, worker);
		AssertJUnit.assertEquals(3, missingDataForWorkflow.size());

		// Hire Date
		worker.setEmploymentPeriod(new DateRange(DateUtil.getNowDate(), null));
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.PTSTaxReady, worker);
		AssertJUnit.assertEquals(2, missingDataForWorkflow.size());

		// Compensation
		CompensationDtl compensation = new CompensationDtl();
		worker.addCompensationDtl(compensation);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.PTSTaxReady, worker);
		AssertJUnit.assertEquals(1, missingDataForWorkflow.size());

		// Taxes
		TaxDtl tax = new TaxDtl();
		worker.addTaxDtl(tax);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.PTSTaxReady, worker);
		AssertJUnit.assertEquals(0, missingDataForWorkflow.size());
		
		boolean actual = WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.PTSTaxReady, worker);
		AssertJUnit.assertTrue("Worker is not ready for PTS Enrollment", actual);
	}
	
	@Test
	public void testGetMissingDataForPayrollWorkflow() throws Exception {
		Worker worker = new Employee();
		List<RequiredData> missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.Payroll,
				worker);
		AssertJUnit.assertEquals(5, missingDataForWorkflow.size());

		// Add an address
		GlobalPostalAddress taxResidenceGlobalAddress = new GlobalPostalAddress();
		taxResidenceGlobalAddress.setCountrySubDivisionCode("CA");
		worker.setTaxResidenceAddress(taxResidenceGlobalAddress);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.Payroll, worker);
		AssertJUnit.assertEquals(4, missingDataForWorkflow.size());

		// Add a work location
		WorkLocation workLocation = new WorkLocation();
		workLocation.setTaxResidenceAddress(taxResidenceGlobalAddress);
		worker.setWorkLocation(workLocation);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.Payroll, worker);
		AssertJUnit.assertEquals(3, missingDataForWorkflow.size());

		// Hire Date
		worker.setEmploymentPeriod(new DateRange(DateUtil.getNowDate(), null));
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.Payroll, worker);
		AssertJUnit.assertEquals(2, missingDataForWorkflow.size());

		// Withholding - This is not required as we will use the default withholding settings if none are set.
//		W4Detail whDoc = new W4Detail();
//		whDoc.setDocId(ComplianceDocIdEnum.W4IRS);
//		whDoc.setNoOfAllowances(1);
//		worker.addWorkerComplianceDocDtl(whDoc);
//		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.Payroll, worker);
//		AssertJUnit.assertEquals(1, missingDataForWorkflow.size());

		// Compensation
		CompensationDtl compensation = new CompensationDtl();
		worker.addCompensationDtl(compensation);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.Payroll, worker);
		AssertJUnit.assertEquals(1, missingDataForWorkflow.size());

		// Taxes
		TaxDtl tax = new TaxDtl();
		worker.addTaxDtl(tax);
		missingDataForWorkflow = WorkerSetupRules.getMissingDataForWorkflow(WorkerWorkflowType.Payroll, worker);
		AssertJUnit.assertEquals(0, missingDataForWorkflow.size());

		boolean actual = WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.Payroll, worker);
		AssertJUnit.assertTrue("Worker should be ready for Pay", actual);

		WorkerSetupRules.setWorkerStateProperties(worker);
		AssertJUnit.assertTrue("Expected worker to be ready for Tax", worker.isReadyForTax());
		AssertJUnit.assertTrue("Expected worker to be ready for Pay", worker.isReadyForPay());
	}

	@Test
	public void testIsReadyForWorkflow() {
		Worker worker = new Employee();
		boolean actual = WorkerSetupRules.isReadyForWorkflow(WorkerWorkflowType.DetermineTaxes, worker);
		AssertJUnit.assertFalse("Worker should not have been ready for Taxes", actual);
		// FIXME add the various workflows and states the worker can be in.
	}

	@Test
	public void testSetWorkerStateProperties() {
		Worker worker = new Employee();
		WorkerSetupRules.setWorkerStateProperties(worker);
		AssertJUnit.assertFalse("Expected worker to not be ready for Tax", worker.isReadyForTax());
		AssertJUnit.assertFalse("Expected worker to not be ready for Pay", worker.isReadyForPay());
	}

}
