package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualTypeEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.Worker;



/**
 *  <P>The <CODE>CalculatePaycheckPTOTest</CODE> class contains tests for testing the calculatePaycheck() in
 *   WorkerPaymentsComponent with an hourly wage and PTO items</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CalculatePaycheckPTOTest extends WorkerPaymentsComponentTestBase {
    private static WorkerPaymentsComponent sWorkerPaymentsComponent;
	private static WorkerComponent sWorkerComponent;

    @BeforeClass
    public static void setUpCalculatePaycheckTest() {
    	sWorkerPaymentsComponent = 
            qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
    	
        try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCalculatePaycheckTest(): " + t.getMessage());
        }
    }
    
    @AfterClass
    public static void tearDownCalculatePaycheckTest() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCalculatePaycheckTest(): " + t.getMessage());
        }
    }
    
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
    }
    
    @AfterMethod
    public void tearDown() throws Exception
    {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
            // Remove withholdingInfo associated with the employee
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
        }
        super.tearDown();
    }
    
    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1, no PTO disbursed on this paycheck</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 0 hours</LI>
     * <LI>Set the vacation PTO disburse to 0 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerPaycheck_ZeroTimeTaken_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 80.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 0.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 0.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1200.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("943.05"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("256.95"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("143.40"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1200.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("0.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("0.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1, no PTO disbursed (nothing set in calcInput) on this paycheck</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 80 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerPaycheck_DontSetCalcInput_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 80.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
        // Purposely not setting the CalculationInputs for the PTO disbursement items
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1200.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("943.05"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("256.95"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("143.40"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1200.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("0.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("0.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 sick PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 72 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1SickPTO_AccruePerPaycheck_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 72.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.SICK);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1200.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("943.05"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("256.95"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("143.40"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1080.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 72 hours</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1VacationPTO_AccruePerPaycheck_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 72.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.VACATION);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);

        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1200.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("943.05"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("256.95"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("143.40"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1080.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 24 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.769 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 1.538 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerPaycheck_PaidWeekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 24.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.769 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.769,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 1.538 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.538,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("600.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("600.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("471.53"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("128.47"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("71.70"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("360.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("65.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("12.77"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 64 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerPaycheck_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 64.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1200.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("943.05"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("256.95"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("143.40"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("960.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 2 hourly wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 56 hours, this is the rate that the PTO is based on</LI>
     * <LI>Create an hourly wage item with a rate of 20.00 and 8 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     */
    @Test
    public void test_2Hourly_1Sick1VacationPTO_AccruePerPaycheck_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 56.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
    	// Add a second hourly compensation item
        CompensationDtl hourlyDtl2 = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                                new BigDecimal("20.00"), 8.0);
        hourlyDtl2.setDescription("Hourly2");
    	worker.addCompensationDtl(hourlyDtl2);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1240.00");
        //TODO: verify PTO balance
        // Sick earned - 1:32, Balance - -6:27;  Vacation earned - 3:04, Balance - -4:55
        AssertJUnit.assertEquals("Gross total", newMoney("1240.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("971.27"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("268.73"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("148.18"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("840.00")),
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("160.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("136.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("76.88"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.98"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("27.95"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.92"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.92"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("76.88"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.98"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("42.16"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.24"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test semimonthly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a semimonthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 72 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.66666 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.33333 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerPaycheck_PaidSemimonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 72.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.66666 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.66666,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.33333 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.33333,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.SEMIMONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1320.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1320.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1035.59"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("284.41"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("157.74"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1080.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("144.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("81.84"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("19.14"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("28.87"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.56"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.56"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("81.84"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("19.14"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.88"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.32"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 168 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 3.33333 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 6.66666 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerPaycheck_PaidMonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 168.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 3.33333 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.33333,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 6.66666 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 6.66666,
    	                                        DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2760.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2760.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("2156.83"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("603.17"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("329.82"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("2520.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("305.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("171.12"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("40.02"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("64.95"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.08"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.08"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("171.12"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("40.02"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("93.84"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.76"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO accrual per year and all applicable taxes
     * for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 24 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 40 hours/year</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 80 hours/year</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerYearProrated_PaidWeekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 24.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 40 hours/year
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 40.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
    	// Add a vacation PTO item
    	// Set the accrual rate to 80 hours/year
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 80.0,
    	                                        DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("600.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("600.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("471.53"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("128.47"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("71.70"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("360.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("65.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("12.77"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 hourly wage, 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 72 hours</LI>
     * <LI>Create an hourly vacation item with 8 hours</LI>
     * <LI>Create an hourly vacation accrual PTO item with an accrual rate of 80 hours/year</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1VacationPTO_AccruePerYearProrated_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 72.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.VACATION);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a vacation PTO item
    	// Set the accrual rate to 80 hours/year
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 80.0,
    	                                        DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);

        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1200.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("943.05"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("256.95"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("143.40"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1080.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test semimonthly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO accrual per year and all applicable
     * taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a semimonthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 72 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 40 hours/year</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 80 hours/year</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerYearProrated_PaidSemimonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 72.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 40 hours/year
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 40.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
    	// Add a vacation PTO item
    	// Set the accrual rate to 80 hours/year
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 80.0,
    	                                        DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.SEMIMONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1320.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1320.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1035.59"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("284.41"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("157.74"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1080.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("144.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("81.84"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("19.14"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("28.87"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.56"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.56"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("81.84"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("19.14"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.88"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.32"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO accrual per year and all applicable taxes
     * for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 168 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 40 hours/year</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 80 hours/year</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerYearProrated_PaidMonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 168.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 40 hours/year
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 40.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
    	// Add a vacation PTO item
    	// Set the accrual rate to 80.0 hours/year
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 80.0,
    	                                        DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2760.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2760.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("2156.83"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("603.17"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("329.82"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("2520.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("305.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("171.12"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("40.02"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("64.95"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.08"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.08"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("171.12"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("40.02"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("93.84"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.76"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO accrual per hour worked and all applicable
     * taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 24 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.01923 hours/hour accrual per hour worked</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 0.03846 hours/hour accrual per hour worked</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerHourWorked_PaidWeekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 24.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.01923 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.01923,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
    	// Add a vacation PTO item
    	// Set the accrual rate to 0.03846 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.03846,
    	                                        DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("600.00");
        //TODO: verify PTO balance
        // Sick earned - 0:27, Balance - -7:32;  Vacation earned - 0:55, Balance - -7:04
        AssertJUnit.assertEquals("Gross total", newMoney("600.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("471.53"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("128.47"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("71.70"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("360.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("65.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("12.77"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("4.80"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("37.20"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("8.70"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.40"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("0.60"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO accrual per hour and all applicable taxes
     * for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 64 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.01923 hours/year accrual per hour worked</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 0.03846 hours/hour accrual per hour worked</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerHourWorked_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 64.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.01923 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.01923,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
    	// Add a vacation PTO item
    	// Set the accrual rate to 0.03846 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.03846,
    	                                        DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1200.00");
        //TODO: verify PTO balance
        // Sick earned - 1:13, Balance - -6:46;  Vacation earned - 2:27, Balance - -5:32
        AssertJUnit.assertEquals("Gross total", newMoney("1200.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("943.05"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("256.95"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("143.40"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("960.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("130.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("25.55"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("9.60"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("74.40"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("17.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("40.80"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.20"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test semimonthly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO accrual per hour worked and all
     * applicable taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a semimonthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 72 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.01923 hours/hour accrual per hour worked</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 0.03846 hours/hour accrual per hour worked</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerHourWorked_PaidSemimonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 72.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.01923 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.01923,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
    	// Add a vacation PTO item
    	// Set the accrual rate to 0.03846 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.03846,
    	                                        DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.SEMIMONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1320.00");
        //TODO: verify PTO balance
        // Sick earned - 1:23, Balance - -6:36;  Vacation earned - 2:46, Balance - -5:13
        AssertJUnit.assertEquals("Gross total", newMoney("1320.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1035.59"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("284.41"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("157.74"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("1080.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("144.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("81.84"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("19.14"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("28.87"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.56"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.56"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("81.84"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("19.14"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.88"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.32"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 hourly wage, 1 sick and 1 vacation PTO accrual per hour worked and all applicable
     * taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 168 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.01923 hours/hour accrual per hour worked</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 0.03846 hours/hour accrual per hour worked</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Hourly_1Sick1VacationPTO_AccruePerHourWorked_PaidMonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 168.0);
        hourlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.01923 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.01923,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
    	// Add a vacation PTO item
    	// Set the accrual rate to 0.03846 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.03846,
    	                                        DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2760.00");
        //TODO: verify PTO balance
        // Sick earned - 3:13, Balance - -4:46;  Vacation earned - 6:27, Balance - -1:32
        AssertJUnit.assertEquals("Gross total", newMoney("2760.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("2156.83"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("603.17"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("329.82"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("2520.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("120.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("305.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("171.12"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("40.02"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("64.95"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.08"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.08"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("171.12"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("40.02"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("93.84"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.76"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

}