package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.CompensationTypeEnum;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.DatumType2Enum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.PaidTimeOffAccrualTypeEnum;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PtoBaseRateTypeEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.Worker;



/**
 *  <P>The <CODE>CalculatePaycheckSalaryPTOTest</CODE> class contains tests for testing the calculatePaycheck() in
 *   WorkerPaymentsComponent with a salary wage and PTO items</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CalculatePaycheckSalaryPTOTest extends WorkerPaymentsComponentTestBase {
    private static WorkerPaymentsComponent sWorkerPaymentsComponent;       
	private static WorkerComponent sWorkerComponent;

    @BeforeClass
    public static void setUpCalculatePaycheckTest() {
    	sWorkerPaymentsComponent = 
            qaBootstrap.getEjbFromClass(WorkerPaymentsComponent.class);
    	sWorkerComponent = qaBootstrap.getEjbFromClass(WorkerComponent.class);
        try {
            getUserTransaction().begin();
            saveEmployerInfo();
            saveWorkLocationAddress();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in CalculatePaycheckSalaryPTOTest(): " + t.getMessage());
        }
    }
    
    @AfterClass
    public static void tearDownCalculatePaycheckTest() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in CalculatePaycheckSalaryPTOTest(): " + t.getMessage());
        }
    }
    
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
    }
    
    @AfterMethod
    public void tearDown() throws Exception
    {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
            // Remove withholdingInfo associated with the employee
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
        }
        super.tearDown();
    }
    
    /**
     * <P>Test biweekly paycheck with 1 salary wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1, no PTO disbursed on this paycheck</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 0 hours</LI>
     * <LI>Set the vacation PTO disburse to 0 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerPaycheck_ZeroTimeTaken_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 0.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 0.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2538.46");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2538.46") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1748.48"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("789.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("303.35"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2538.46")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("0.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("0.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, 1 sick PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1SickPTO_AccruePerPaycheck_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.SICK);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2538.46");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2538.46") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1748.48"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("789.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("303.35"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2284.61")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1VacationPTO_AccruePerPaycheck_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.VACATION);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2538.46");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2538.46") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1748.48"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("789.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("303.35"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2284.61")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 salary wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.769 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 1.538 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerPaycheck_PaidWeekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.769 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.769,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 1.538 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.538,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1269.23");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1269.23") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("874.25"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("394.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("151.66"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("761.53")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("220.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("78.69"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("18.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("67.74"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.15"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.15"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("78.69"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("18.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("43.15"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.27"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerPaycheck_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2538.46");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2538.46") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1748.48"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("789.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("303.35"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2030.76")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, 1 hourly wage, 1 sick and 1 vacation PTO and all applicable taxes
     * for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year, this is the rate that the PTO is based on</LI>
     * <LI>Create an hourly wage item with a rate of 15.00 and 8 hours</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.53846 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.07692 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
     public void test_1Salary1Hourly_1Sick1VacationPTO_AccruePerPaycheck_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
    	// Add an hourly compensation item
        CompensationDtl hourlyDtl = createRateBasedWageItemDtl(CompensationTypeEnum.HOURLYREG, 
                                                               new BigDecimal("15.00"), 8.0);
    	worker.addCompensationDtl(hourlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.53846 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.53846,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.07692 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.07692,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        //TODO: Figure out why QBO calculated the paycheck total as 2658.46 (2538.46+120.00), why doesn't the salary
        //        amount get reduced??
        BigDecimal amount = new BigDecimal("2658.46");
        //TODO: verify PTO balance
        // Sick earned - 1:32, Balance - -6:27;  Vacation earned - 3:04, Balance - -4:55
        AssertJUnit.assertEquals("Gross total", newMoney("2658.46") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1817.18"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("841.28"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("317.69"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2030.76")),
        	new LineItemAmount(QboeLawIdConstants.sHourlyLawId, new BigDecimal("120.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("470.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("164.82"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("38.55"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("146.64"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("21.27"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("21.27"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("164.82"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("38.55"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("90.39"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.66"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test semimonthly paycheck with 1 salary wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a semimonthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 1.66666 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 3.33333 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerPaycheck_PaidSemimonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 1.66666 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 1.66666,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 3.33333 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.33333,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.SEMIMONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2750.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2750.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1893.85"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("856.15"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("328.63"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2250.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("250.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("250.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("477.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("170.50"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("39.88"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("146.77"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.00"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("170.50"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("39.88"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("93.50"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.75"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 salary wage, 1 sick and 1 vacation PTO and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 3.33333 hours/paycheck</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 6.66666 hours/paycheck</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerPaycheck_PaidMonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 3.33333 hours/paycheck
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 3.33333,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
    	// Add a vacation PTO item
    	// Set the accrual rate to 6.66666 hours/paycheck
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 6.66666,
    	                                    DatumType2Enum.PERPAYCHECK, PaidTimeOffAccrualTypeEnum.EACH_PAYCHECK);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("5500.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("5500.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("3787.72"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("1712.28"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("657.25"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("5021.74")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("239.13")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("239.13")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("954.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("293.53"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("187.00"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("5.50"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 salary wage, 1 sick and 1 vacation PTO accrual per year and all applicable taxes
     * for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 40 hours/year</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 80 hours/year</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerYearProrated_PaidWeekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 40 hours/year
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 40.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
    	// Add a vacation PTO item
    	// Set the accrual rate to 80 hours/year
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 80.0,
    	                                        DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1269.23");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("1269.23") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("874.25"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("394.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("151.66"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("761.53")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("220.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("78.69"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("18.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("67.74"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.15"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.15"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("78.69"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("18.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("43.15"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.27"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, 1 sick PTO accrual per year and all applicable taxes for California 
     * using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 40 hours/year</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1SickPTO_AccruePerYearProrated_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.SICK);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 40 hours/year
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 40.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2538.46");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2538.46") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1748.48"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("789.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("303.35"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2284.61")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test semimonthly paycheck with 1 salary wage, 1 sick and 1 vacation PTO accrual per year and all applicable
     * taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a semimonthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 40 hours/year</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 80 hours/year</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerYearProrated_PaidSemimonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 40 hours/year
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 40.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
    	// Add a vacation PTO item
    	// Set the accrual rate to 80 hours/year
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 80.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.SEMIMONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2750.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("2750.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1893.85"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("856.15"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("328.63"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2250.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("250.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("250.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("477.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("170.50"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("39.88"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("146.77"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.00"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("170.50"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("39.88"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("93.50"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.75"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 salary wage, 1 sick and 1 vacation PTO accrual per year and all applicable taxes
     * for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 40 hours/year</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 80 hours/year</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerYearProrated_PaidMonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 40 hours/year
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 40.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
    	// Add a vacation PTO item
    	// Set the accrual rate to 80 hours/year
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 80.0,
    	                                    DatumType2Enum.YEAR, PaidTimeOffAccrualTypeEnum.PER_YEAR_PRORATED);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("5500.00");
        //TODO: verify PTO balance
        AssertJUnit.assertEquals("Gross total", newMoney("5500.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("3787.72"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("1712.28"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("657.25"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("5021.74")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("239.13")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("239.13")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("954.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("293.53"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("187.00"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("5.50"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test weekly paycheck with 1 salary wage, 1 sick and 1 vacation PTO accrual per hour worked and all applicable
     * taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a weekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.01923 hours/hour accrual per hour worked</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 0.03846 hours/hour accrual per hour worked</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerHourWorked_PaidWeekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.01923 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.01923,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
    	// Add a vacation PTO item
    	// Set the accrual rate to 0.03846 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.03846,
    	                                        DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.WEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("1269.23");
        //TODO: verify PTO balance
        // Sick earned - 0:46, Balance - -7:13;  Vacation earned - 1:32, Balance - -6:27
        AssertJUnit.assertEquals("Gross total", newMoney("1269.23") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("874.25"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("394.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("151.66"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("761.53")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("220.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("78.69"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("18.40"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("67.74"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.15"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("10.15"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("78.69"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("18.40"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("43.15"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("1.27"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test biweekly paycheck with 1 salary wage, 1 sick and 1 vacation PTO accrual per hour worked and all applicable
     * taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a biweekly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.01923 hours/hour accrual per hour worked</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 0.03846 hours/hour accrual per hour worked</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerHourWorked_PaidBiweekly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.01923 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.01923,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
    	// Add a vacation PTO item
    	// Set the accrual rate to 0.03846 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.03846,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.BIWEEKLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2538.46");
        //TODO: verify PTO balance
        // Sick earned - 1:32, Balance - -6:27;  Vacation earned - 3:04, Balance - -4:55
        AssertJUnit.assertEquals("Gross total", newMoney("2538.46") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1748.48"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("789.98"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("303.35"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2030.76")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("253.85")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("440.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("135.48"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("20.31"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("157.38"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("36.81"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("86.31"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.54"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test semimonthly paycheck with 1 salary wage, 1 sick and 1 vacation PTO accrual per hour worked and all
     * applicable taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a semimonthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.01923 hours/hour accrual per hour worked</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 0.03846 hours/hour accrual per hour worked</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerHourWorked_PaidSemimonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.01923 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.01923,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
    	// Add a vacation PTO item
    	// Set the accrual rate to 0.03846 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.03846,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.SEMIMONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("2750.00");
        //TODO: verify PTO balance
        // Sick earned - 1:41, Balance - -6:18;  Vacation earned - 3:23, Balance - -4:36
        AssertJUnit.assertEquals("Gross total", newMoney("2750.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("1893.85"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("856.15"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("328.63"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("2250.00")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("250.00")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("250.00")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("477.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("170.50"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("39.88"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("146.77"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("22.00"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("170.50"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("39.88"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("93.50"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("2.75"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

    /**
     * <P>Test monthly paycheck with 1 salary wage, 1 sick and 1 vacation PTO accrual per hour worked and all applicable
     * taxes for California using withholding status of Single 1</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create California employee with a monthly pay frequency</LI>
     * <LI>Create a salary wage item with a rate of $66,000/year</LI>
     * <LI>Create a sick accrual PTO item with an accrual rate of 0.01923 hours/hour accrual per hour worked</LI>
     * <LI>Create a vacation accrual PTO item with an accrual rate of 0.03846 hours/hour accrual per hour worked</LI>
     * <LI>Set the sick PTO disburse to 8 hours</LI>
     * <LI>Set the vacation PTO disburse to 8 hours</LI>
     * <LI>Setup list of compensations</LI>
     * <LI>Call the WorkerPaymentsComponent to calculate the paycheck</LI>
     * <LI>Verify calculated paycheck values</LI>
     * </OL>
     * 
     */
    @Test
    public void test_1Salary_1Sick1VacationPTO_AccruePerHourWorked_PaidMonthly() throws Exception {
    	Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("James", sMountainViewLocation);
    	// Add a salary compensation item
        // Set up compensation for a worker
        CompensationDtl salarlyDtl = WorkerTestUtils.createCompensationDtl(CompensationTypeEnum.SALARYREG, true);
        salarlyDtl.setPtoBaseRateType(PtoBaseRateTypeEnum.PTO);
    	worker.addCompensationDtl(salarlyDtl);
        worker = sWorkerComponent.updateWorker(worker);
    	// Add a sick PTO item
    	// Set the accrual rate to 0.01923 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addSickPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.01923,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
    	// Add a vacation PTO item
    	// Set the accrual rate to 0.03846 hours/hour accrual per hour worked
    	worker = WorkerTestUtils.addVacationPTO(worker, WorkerTestUtils.ACCRUAL_START_DATE, 0.03846,
    	                                    DatumType2Enum.HOUR, PaidTimeOffAccrualTypeEnum.HOURLY);
        
        PayrollFrequencyEnum frequency = PayrollFrequencyEnum.MONTHLY;
        setupPaycheckDates(frequency);
        DateRange payPeriod = new DateRange(sPeriodStartDate.getTime(), mPeriodEndDate.getTime());
        
        List<CalculationInputs> calculationInputs = createCalcInputsFromCompensationDtls(worker.getCompensationDtls());
    	// Set the number of sick hours taken
        setSickDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
    	// Set the number of vacation hours taken
        setVacationDisburseHours(worker.getCompensationDtls(), calculationInputs, 8.0);
        
        PaycheckCalculationRequest paycheckCalculationRequest = createPaycheckRequest(worker, frequency, payPeriod,
				calculationInputs);
        
        Paycheck paycheck = sWorkerPaymentsComponent.calculatePaycheck(paycheckCalculationRequest);
        BigDecimal amount = new BigDecimal("5500.00");
        //TODO: verify PTO balance
        // Sick earned - 3:32, Balance - -4:27;  Vacation earned - 7:04, Balance - -0:55
        AssertJUnit.assertEquals("Gross total", newMoney("5500.00") , paycheck.getGrossAmount());
        AssertJUnit.assertEquals("Net pay", newMoney("3787.72"), paycheck.getNetAmount());
        AssertJUnit.assertEquals("Employee taxes", newMoney("1712.28"), paycheck.getWorkerPdTaxes());
        AssertJUnit.assertEquals("Employer taxes", newMoney("657.25"), paycheck.getEmployerPdTaxes());
        // verify paycheck line items
        LineItemAmount lineItems[] = {
        	new LineItemAmount(QboeLawIdConstants.sSalaryLawId, new BigDecimal("5021.74")),
            new LineItemAmount(QboeLawIdConstants.sSickTimeDisbursementLawId, new BigDecimal("239.13")),
            new LineItemAmount(QboeLawIdConstants.sVacationDisbursementLawId, new BigDecimal("239.13")),
            new LineItemAmount(mUSFIT.getPayrollItem().getLawId().getValue(), new BigDecimal("954.00"), amount, amount),
            new LineItemAmount(mUSSSTE.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTE.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCAPIT.getPayrollItem().getLawId().getValue(), new BigDecimal("293.53"), amount, amount),
            new LineItemAmount(mCASDI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSFUI.getPayrollItem().getLawId().getValue(), new BigDecimal("44.00"), amount, amount),
            new LineItemAmount(mUSSSTER.getPayrollItem().getLawId().getValue(), new BigDecimal("341.00"), amount, amount),
            new LineItemAmount(mUSMEDTER.getPayrollItem().getLawId().getValue(), new BigDecimal("79.75"), amount, amount),
            new LineItemAmount(mCASUI.getPayrollItem().getLawId().getValue(), new BigDecimal("187.00"), amount, amount),
            new LineItemAmount(mCAETT.getPayrollItem().getLawId().getValue(), new BigDecimal("5.50"), amount, amount)
        };
        AssertJUnit.assertTrue("Expected lineItem value mismatch!", verifyLineItems(paycheck.getPaycheckLineItems(),
                                                                               lineItems, "paycheck line items"));
    }

}