package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>CancelPayrollRunByIdTest</CODE> class contains tests for testing the
 * cancelPayrollRunById() in WorkerPaymentsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class CancelPayrollRunByIdTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@BeforeClass
	public static void setUpCancelPayrollRunByIdTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpCancelPayrollRunByIdTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownCancelPayrollRunByIdTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownCancelPayrollRunByIdTest(): " + t.getMessage());
		}
	}

    /**
     * <P>Test canceling a PayrollRun</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values for a worker</LI>
     * <LI>Setup a PayrollRunRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() with a valid PayrollRunRequest</LI>
     * <LI>Change the PayrollRun workflow status to NotSet</LI>
     * <LI>Call WorkerPaymentsComponent.cancelPayrollRunById() with the payrollRun id returned from the commitPayrollRun()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the payroll run is canceled/deleted</LI>
     * </UL>
     */
	@Test
	public void testCancelPayrollRunById() throws Exception {
    	PayrollRun payrollRun = null;
    	try {
        	//Setup
    		// Remember the existing number of PayrollRuns
    		List<PayrollRun> existingPayrollRuns = sWorkerPaymentsComponent.findAllPayrollRuns();
    		int existingCount = existingPayrollRuns.size();
        	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
        	                                   sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
        	// Change the workflow status to a value other than COMMITTED
        	payrollRun.setWorkflowStatus(PayrollRunWorkflowStatusEnum.NOTSET);
        	//Exercise
        	sWorkerPaymentsComponent.cancelPayrollRunById(payrollRun.getId());

        	//Verify
    		List<PayrollRun> payrollRuns = sWorkerPaymentsComponent.findAllPayrollRuns();
        	AssertJUnit.assertEquals("PayrollRun not canceled/deleted!", existingCount, payrollRuns.size());
        	payrollRun = null;
     	} finally {
        	//Tear down
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    	}
	}

	/**
	 * <P>Test calling cancelPayrollRunById() with a null PayrollRunId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.cancelPayrollRunById() with a null PayrollRunId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayrollRunId() throws Exception {
    	//Exercise
		sWorkerPaymentsComponent.cancelPayrollRunById(null);
	}

	/**
	 * <P>Test calling cancelPayrollRunById() with a non-existent PayrollRunId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.cancelPayrollRunById() with a non-existent PayrollRunId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testNonExistentPayrollRunId() throws Exception {
    	//Exercise
		sWorkerPaymentsComponent.cancelPayrollRunById(934893943L);
	}

}
