package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>CommitPayrollRunExceptionsTest</CODE> class contains Exception tests for testing the
 * commitPayrollRun() in WorkerPaymentsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class CommitPayrollRunExceptionsTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@BeforeClass
	public static void setUpCommitPayrollRunExceptionsTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpCommitPayrollRunExceptionsTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownCommitPayrollRunExceptionsTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownCommitPayrollRunExceptionsTest(): " + t.getMessage());
		}
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null PayrollRunRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null PayrollRunRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayrollRunRequest() throws Exception {
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(null);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null PayrollRunInfo in the PayrollRunRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null PayrollRunInfo in the PayrollRunRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayrollRunInfo() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    	// Purposely set the PayrollRunInfo to null
        payrollRunRequest.setPayrollRunInfo(null);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null periodStartDate in the PayrollRunInfo</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null periodStartDate in the PayrollRunInfo</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPeriodStartDate() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    	// Purposely set the periodStartDate in PayrollRunInfo to null
    	DateRange payPeriod = payrollRunInfo.getPayPeriod();
    	payPeriod.setStartDate(null);
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null periodEndDate in the PayrollRunInfo</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null periodEndDate in the PayrollRunInfo</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPeriodEndDate() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    	// Purposely set the periodEndDate in PayrollRunInfo to null
    	DateRange payPeriod = payrollRunInfo.getPayPeriod();
    	payPeriod.setEndDate(null);
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null payrollRunDate in the PayrollRunInfo</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null payrollRunDate in the PayrollRunInfo</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayrollRunDate() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    	// Purposely set the payrollRunDate in PayrollRunInfo to null
    	payrollRunInfo.setPayrollRunDate(null);
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null payDate in the PayrollRunInfo</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null payDate in the PayrollRunInfo</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayDate() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    	// Purposely set the payDate in PayrollRunInfo to null
    	payrollRunInfo.setPayDate(null);
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a periodStartDate after the periodEndDate in the PayrollRunInfo</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a periodStartDate after the periodEndDate in the
	 *  PayrollRunInfo</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testPeriodStartDateAfterPeriodEndDate() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    	// Purposely set the periodStartDate after the periodEndDate in PayrollRunInfo
    	DateRange payPeriod = payrollRunInfo.getPayPeriod();
    	Date tempDate = payPeriod.getStartDate();
    	payPeriod.setStartDate(payPeriod.getEndDate());
    	payPeriod.setEndDate(tempDate);
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}


	/**
	 * <P>Test calling commitPayrollRun() with a payDate before the payrollRunDate in the PayrollRunInfo</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a payDate before the payrollRunDate in the
	 *  PayrollRunInfo</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testPayDateBeforePayrollRunDate() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    	// Purposely set the payDate before the payrollRunDate in PayrollRunInfo
		Calendar tempDate = DateUtil.getNowCalendar();
		tempDate.setTime(payrollRunInfo.getPayrollRunDate());
		tempDate.add(Calendar.DAY_OF_MONTH, -1);
    	payrollRunInfo.setPayDate(tempDate.getTime());
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with an invalid frequency in the PayrollRunInfo</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with an invalid frequency in the PayrollRunInfo</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testInvalidFrequency() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    	// Purposely set an invalid frequency in PayrollRunInfo
    	payrollRunInfo.setFrequency(PayrollFrequencyEnum.ANNUALLY);
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null PaycheckCalculationRequest list in payrollRunRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null PaycheckCalculationRequest list in
	 *  payrollRunRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPaycheckCalculationRequestList() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
    	// Purposely set it to null
        payrollRunRequest.setCalculationRequests(null);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null PaycheckCalculationRequest in the PaycheckCalculationRequest
	 *  list</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null PaycheckCalculationRequest in the
	 *  PaycheckCalculationRequest list</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	paycheckCalculationRequests.add(null);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null Paycheck date in the PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null Paycheck date in the
	 *  PaycheckCalculationRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPaycheckDateInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the paycheck date to null
    	calcRequest.setPaycheckDate(null);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null PayPeriod in the PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null PayPeriod in the
	 *  PaycheckCalculationRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayPeriodInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the pay period to null
    	calcRequest.setPayPeriod(null);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null PayPeriod StartDate in the PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null PayPeriod StartDate in the
	 *  PaycheckCalculationRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayPeriodStartDateInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the pay period startDate to null
    	DateRange payPeriod = calcRequest.getPayPeriod();
    	payPeriod.setStartDate(null);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null PayPeriod EndDate in the PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null PayPeriod EndDate in the
	 *  PaycheckCalculationRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayPeriodEndDateInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the pay period endDate to null
    	DateRange payPeriod = calcRequest.getPayPeriod();
    	payPeriod.setEndDate(null);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a PayPeriod EndDate before the PayPeriod StartDate in the
	 *  PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a PayPeriod EndDate before the PayPeriod StartDate
	 *  in the PaycheckCalculationRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testPayPeriodEndDateBeforeStartDateInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the pay period endDate to null
    	DateRange payPeriod = calcRequest.getPayPeriod();
    	Date tempDate = payPeriod.getEndDate();
    	payPeriod.setEndDate(payPeriod.getStartDate());
    	payPeriod.setStartDate(tempDate);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null worker id in the PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null worker id in the
	 *  PaycheckCalculationRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullWorkerIdInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the worker id to null
    	calcRequest.setWorkerId(null);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a worker id value of zero in the PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a worker id value of zero in the
	 *  PaycheckCalculationRequest.  An id value of zero means that the worker hasn't been saved to the database</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testZeroWorkerIdInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the worker id to zero
    	calcRequest.setWorkerId(0L);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() with a null frequency in the PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() with a null frequency in the
	 *  PaycheckCalculationRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullFrequencyInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the frequency to null
    	calcRequest.setFrequency(null);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

	/**
	 * <P>Test calling commitPayrollRun() without setting a frequency in the PaycheckCalculationRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.commitPayrollRun() without setting a frequency in the
	 *  PaycheckCalculationRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testFrequencyNotSetInPaycheckCalculationRequest() throws Exception {
    	//Setup
    	aSchedule = createBiweeklySchedule(DateUtil.getNowDate(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
    	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    	// Purposely set the frequency to NOTSET
    	calcRequest.setFrequency(PayrollFrequencyEnum.NOTSET);
    	paycheckCalculationRequests.add(calcRequest);
    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

        //Exercise
		@SuppressWarnings("unused")
		PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	}

}
