package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>CreatePayrollRunExceptionsTest</CODE> class contains tests for testing the
 * exceptions thrown by createPayrollRun() in WorkerPaymentsComponent
 * </P>
 * @author smadduri
 *
 */
public class CreatePayrollRunExceptionsTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private static Long sReadyForPayWorkerId2 = 0L;
	private static Long sNotReadyForPayWorkerId = 0L;
	private static Long sSalaryWorkerId = 0L;

	@BeforeClass
	public static void setUpCreatePayrollRunTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			sReadyForPayWorkerId2 = WorkerTestUtils.createPayrollReadyWorker("Paul", sMountainViewLocation);
			sNotReadyForPayWorkerId = WorkerTestUtils.createNoWagePayrollReadyWorker("Brad", sMountainViewLocation);
			sSalaryWorkerId = WorkerTestUtils.createSalaryPayrollReadyWorker("Madoff", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpFindPaycheckTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownCreatePayrollRunTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			if (0 != sReadyForPayWorkerId2) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId2);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId2 = 0L;
			}
			if(0 != sNotReadyForPayWorkerId)
			{
				Worker worker = entityService.findById(Worker.class, sNotReadyForPayWorkerId);
				WorkerTestUtils.cleanUpWorker(worker);
				sNotReadyForPayWorkerId = 0L;
			}
			if(0 != sSalaryWorkerId)
			{
				Worker worker = entityService.findById(Worker.class, sSalaryWorkerId);
				WorkerTestUtils.cleanUpWorker(worker);
				sSalaryWorkerId = 0L;
			}

			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownFindPaycheckTest(): " + t.getMessage());
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run with a non payroll ready worker</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add non payroll ready worker ID to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since we tried to create a payroll run with
	 * a non payroll ready worker</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithNotReadyToPayWorker() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sNotReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing a empty workerIdList</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add an empty workerIdList to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since there are no workers</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunEmptyWorkerIdsList() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing a bogus workerId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a bogus workerIdList to it</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException is thrown since there are no workers with the ID
	 * we passed since it is bogus.</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ServiceFailureException.class)
	public void testCreatePayrollRunWithBogusWorkerIdList() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            Long bogusWorkerId = new Long("12345");
            workerIdList.add(bogusWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing null payrollRunInfo</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a null payrollRunInfo</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since payrollRunInfo is null</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithNullPayrollInfo() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	PayrollRunInfo payrollRunInfo = null;
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing PayrollRunTypeEnum NOTSET</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a null payrollRunInfo with PayrollRunTypeEnum NOTSET</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since PayrollRunTypeEnum is NOTSET</LI>
	 * </UL>
	 * 	
	 */
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithPayrollRunTypeEnumNOTSET() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	payrollRunInfo.setPayrollRunType(PayrollRunTypeEnum.NOTSET);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing null PayDate</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a null PayDate to payrollRunInfo</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since PayDate is null</LI>
	 * </UL>
	 * 	
	 */	
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithNullPayDate() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	
        	payrollRunInfo.setPayDate(null);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing null RunDate</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a null RunDate to payrollRunInfo</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since RunDate is null</LI>
	 * </UL>
	 * 	
	 */		
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithNullRunDate() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	
        	payrollRunInfo.setPayrollRunDate(null);
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing null PayPeriod</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a null PayPeriod to payrollRunInfo</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since PayPeriod is null</LI>
	 * </UL>
	 * 	
	 */		
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithNullPayPeriod() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	
        	payrollRunInfo.setPayPeriod(null);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run with PayPeriodStartDate After EndDate</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a payrollRunInfo with PayPeriodStartDate After EndDate</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since PayPeriodStartDate is After EndDate</LI>
	 * </UL>
	 * 	
	 */	
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithPayPeriodStartDateAfterEndDate() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	
        	// Purposely set the pay period endDate before startDate
        	DateRange payPeriod = payrollRunInfo.getPayPeriod();
        	Date tempDate = payPeriod.getEndDate();
        	payPeriod.setEndDate(payPeriod.getStartDate());
        	payPeriod.setStartDate(tempDate);
        	
        	payrollRunInfo.setPayPeriod(payPeriod);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing null PayPeriodStartDate</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a null PayPeriodStartDate to payrollRunInfo</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since PayPeriodStartDate is null</LI>
	 * </UL>
	 * 	
	 */	
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithNullPayPeriodStartDate() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	
        	DateRange payPeriod = payrollRunInfo.getPayPeriod();
        	payPeriod.setStartDate(null);
        	
        	payrollRunInfo.setPayPeriod(payPeriod);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll run by passing null PayPeriodEndDate</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a null PayPeriodEndDate to payrollRunInfo</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since PayPeriodEndDate is null</LI>
	 * </UL>
	 * 	
	 */	
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithNullPayPeriodEndDate() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	
        	DateRange payPeriod = payrollRunInfo.getPayPeriod();
        	payPeriod.setEndDate(null);
        	
        	payrollRunInfo.setPayPeriod(payPeriod);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll with PaycheckDate Before RunDate</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a payrollRunInfo with PaycheckDate Before RunDate</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since PaycheckDate is Before RunDate</LI>
	 * </UL>
	 * 	
	 */	
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithPaycheckDateBeforeRunDate() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	
        	// Purposely set the paycheckDate before runDate
        	Date tempDate = payrollRunInfo.getPayrollRunDate();
        	payrollRunInfo.setPayrollRunDate(payrollRunInfo.getPayDate());
        	payrollRunInfo.setPayDate(tempDate);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
	
	/**
	 * <P>This test is to create a scheduled payroll with PaycheckDate Before today</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a biweekly payroll schedule</LI>
	 * <LI>Create a payroll run create request and add a payrollRunInfo with PaycheckDate Before today</LI>
	 * <LI>Call the createPayrollRun() API</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ValidationException is thrown since PaycheckDate is Before today</LI>
	 * </UL>
	 * 	
	 */	
	@Test(groups = {"worker.workerPayments", "worker.createPayrollRun"}, expectedExceptions = ValidationException.class)
	public void testCreatePayrollRunWithPaycheckDateBeforeToday() throws Exception
	{
		@SuppressWarnings("unused")
		PayrollRun payrollRun = null;
		try
		{
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	        	
    		Date pastDate = DateUtil.createDate(2007, 6, 27);
    		
        	// Purposely set the paycheckDate to history
        	payrollRunInfo.setPayDate(pastDate);


            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
            
            List<Long> workerIdList = new ArrayList<Long>();
            workerIdList.add(sReadyForPayWorkerId);
            
            //Create a PayrollRunCreateRequest
            PayrollRunCreateRequest payrollRunCreateRequest = new PayrollRunCreateRequest(workerIdList, payrollRunInfo);
            getUserTransaction().commit();
            
            //Execute
            payrollRun = sWorkerPaymentsComponent.createPayrollRun(payrollRunCreateRequest);            
        }
		finally
		{
			//Nothing to do here
		}
	}
}
