package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>FindMostRecentPaycheckTest</CODE> class contains tests for testing the
 * findMostRecentPaycheck() in WorkerPaymentsComponent
 * </P>
 */
public class FindMostRecentPaycheckTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private static Long sReadyForPayWorkerId2 = 0L;
	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@BeforeClass
	public static void setUpFindPaycheckTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			sReadyForPayWorkerId2 = WorkerTestUtils.createPayrollReadyWorker("Paul", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpFindPaycheckTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownFindPaycheckTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			if (0 != sReadyForPayWorkerId2) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId2);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId2 = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownFindPaycheckTest(): " + t.getMessage());
		}
	}

	/**
	 * <P>Test calling findMostRecentPaycheck() with a worker who has no paychecks</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the sWorkerPaymentsComponent.findMostRecentPaycheck() with a worker who
	 * has not been part of a payroll run.</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that null is returned
	 * </UL>
	 */
	@Test(groups = {"worker.findMostRecentPaycheck", "worker.workerPayments"})
	public void testWorkerHasNoPaychecks() throws Exception {
    	//Exercise
		Paycheck paycheck = sWorkerPaymentsComponent.findMostRecentPaycheck(entityService.findById(Worker.class, sReadyForPayWorkerId));
		AssertJUnit.assertNull("worker has no paychecks", paycheck);
		paycheck = sWorkerPaymentsComponent.findMostRecentPaycheck(entityService.findById(Worker.class, sReadyForPayWorkerId2));
		AssertJUnit.assertNull("worker has no paychecks", paycheck);
	}

    /**
     * <P>Test findMostRecentPaycheck() API with 2 hourly workers, only one of which
     * has a single paycheck</P>
     * 
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a Biweekly pay schedule</LI>
     * <LI>Create a payroll run for the pay schedule</LI>
     * <LI>Create a paycheck calculation request for sReadyForPayWorkerId</LI>
     * <LI>Commit the payroll run</LI>
     * <LI>Call sWorkerPaymentsComponent.findMostRecentPaycheck(worker) to verify the paycheck</LI>
     * </OL>
     * 
     * <UL>
     * verify that the paycheck returned form the API is the same paycheck that is 
     * created while committing the payroll run
     * </UL>
     */
	@Test(groups = {"worker.findMostRecentPaycheck", "worker.workerPayments"})
	public void testOneWorkerHasAPaycheck() throws Exception {

    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());

            List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
            PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);

        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
        	PayrollRun payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);

            Paycheck inputPaycheck = null;
            for (Paycheck paycheck : payrollRun.getPaychecks()) {
            	if (paycheck.getWorker().getId() == sReadyForPayWorkerId) {
            		inputPaycheck = paycheck;
            		break;
            	}
            }
            
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

         	//Exercise & Verify
    		Paycheck paycheck = sWorkerPaymentsComponent.findMostRecentPaycheck(entityService.findById(Worker.class, sReadyForPayWorkerId));
    		AssertJUnit.assertTrue("paycheck is the expected one", validatePaycheck(paycheck, inputPaycheck));

    		paycheck = sWorkerPaymentsComponent.findMostRecentPaycheck(entityService.findById(Worker.class, sReadyForPayWorkerId2));
    		AssertJUnit.assertNull("worker has no paychecks", paycheck);
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().rollback();
            }
    	}
	}
    
    /**
     * <P>Test findMostRecentPaycheck() API with 2 hourly workers both
     * have a single paycheck</P>
     * 
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a Biweekly pay schedule</LI>
     * <LI>Create a payroll run for the pay schedule</LI>
     * <LI>Create a paycheck calculation request for "sReadyForPayWorkerId" and "sReadyForPayWorkerId2"</LI>
     * <LI>Commit the payroll run</LI>
     * <LI>Call sWorkerPaymentsComponent.findMostRecentPaycheck(worker) to verify the paycheck</LI>
     * </OL>
     * 
     * <UL>
     * verify that the paycheck returned form the API is the same paycheck that is 
     * created while committing the payroll run
     * </UL>
     */
	@Test(groups = {"worker.findMostRecentPaycheck", "worker.workerPayments"})
	public void testTwoWorkersHavePaychecksFromSameRuns() throws Exception {

    	PayrollRun payrollRuns[] = new PayrollRun[2];
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	for (int i = 1; i <= 2; i++) {
	        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
	            // Set the Payroll system date to the payroll run date
	            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
	
	            List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
	            PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
	        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount.multiply(new BigDecimal(i)),
	        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
	        	paycheckCalculationRequests.add(calcRequest);
	        	PaycheckCalculationRequest calcRequest2 = createPaycheckCalculationRequest(sReadyForPayWorkerId2,
	        					payrollRunInfo, mHourlyRate, new Double(80), new Double(5), mBonusAmount,
	        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
	        	paycheckCalculationRequests.add(calcRequest2);
	        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
	            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
	            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
	        	payrollRuns[i - 1] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
        	}
        	
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
         	
            Paycheck inputPaycheck = null;
            for (Paycheck paycheck : payrollRuns[1].getPaychecks()) {
            	if (paycheck.getWorker().getId() == sReadyForPayWorkerId) {
            		inputPaycheck = paycheck;
                	//Exercise
            		Paycheck retunedPaycheck = sWorkerPaymentsComponent.findMostRecentPaycheck(entityService.findById(Worker.class, sReadyForPayWorkerId));

                	//Verify
            		AssertJUnit.assertTrue("paycheck is the expected one", validatePaycheck(retunedPaycheck, inputPaycheck));
            	}
            	if (paycheck.getWorker().getId() == sReadyForPayWorkerId2) {
            		inputPaycheck = paycheck;
                	//Exercise
            		Paycheck retunedPaycheck = sWorkerPaymentsComponent.findMostRecentPaycheck(entityService.findById(Worker.class, sReadyForPayWorkerId2));

                	//Verify
            		AssertJUnit.assertTrue("paycheck is the expected one", validatePaycheck(retunedPaycheck, inputPaycheck));
            	}
            }
            

    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().rollback();
            }
    	}
	}

    /**
     * <P>Test findMostRecentPaycheck() API with 2 hourly workers both
     * have a single paycheck but from different payruns</P>
     * 
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a Biweekly pay schedule</LI>
     * <LI>Create a payroll run for the pay schedule</LI>
     * <LI>Create a paycheck calculation request for "sReadyForPayWorkerId" and "sReadyForPayWorkerId2"</LI>
     * <LI>Commit the payroll run</LI>
     * <LI>Call sWorkerPaymentsComponent.findMostRecentPaycheck(worker) to verify the paycheck</LI>
     * </OL>
     * 
     * <UL>
     * verify that the paycheck returned form the API is the same paycheck that is 
     * created while committing the payroll run
     * </UL>
     */
	@Test(groups = {"worker.findMostRecentPaycheck", "worker.workerPayments"})
	public void testTwoWorkersHavePaychecksFromDifferentRuns() throws Exception {

    	PayrollRun payrollRuns[] = new PayrollRun[2];
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	for (int i = 1; i <= 2; i++) {
	        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
	            // Set the Payroll system date to the payroll run date
	            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
	
	            List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
	            PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
	        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount.multiply(new BigDecimal(i)),
	        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
	        	paycheckCalculationRequests.add(calcRequest);
	        	if (i == 1) { // worker 2 only has a check in the first run
		        	PaycheckCalculationRequest calcRequest2 = createPaycheckCalculationRequest(sReadyForPayWorkerId2,
		        					payrollRunInfo, mHourlyRate, new Double(80), new Double(5), mBonusAmount,
		        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
		        	paycheckCalculationRequests.add(calcRequest2);
	        	}
	        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
	            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
	            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
	        	payrollRuns[i - 1] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
        	}

            // Find worker2's paycheck from the input
        	Paycheck inputPaycheck = null;
            for (Paycheck paycheck : payrollRuns[0].getPaychecks()) {
            	if (paycheck.getWorker().getId() == sReadyForPayWorkerId2) {
            		inputPaycheck = paycheck;
            		break;
            	}
            }
            
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	//Exercise
    		Paycheck paycheck = sWorkerPaymentsComponent.findMostRecentPaycheck(entityService.findById(Worker.class, sReadyForPayWorkerId2));

        	//Verify
    		AssertJUnit.assertTrue("paycheck is the expected one", validatePaycheck(paycheck, inputPaycheck));
    	} finally {
            if (Status.STATUS_ACTIVE != getUserTransaction().getStatus()) {
            	getUserTransaction().rollback();
            }
    	}
	}

}
