package com.intuit.sb.payroll.capability.workerpayments;

import java.util.Calendar;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;


/**
 * The <CODE>GetNextScheduledPayrollRunInfoBiweeklyTest</CODE> class contains tests for testing the
 * getNextScheduledPayrollRunInfo() in PayrollScheduleComponent using a biweekly schedule with no existing PayrollRuns
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetNextScheduledPayrollRunInfoBiweeklyTest extends WorkerPaymentsComponentTestBase {
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period is from Monday to Sunday</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a biweekly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_MondayToSunday() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 4);
    	aSchedule = createBiweeklySchedule(periodEndDate.getTime(), sDEFAULT_PAYROLL_RUN_OFFSET,
    	                                   sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 22),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 4)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 5),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo, Pay period is from Wednesday to Tuesday</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a biweekly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_WednesdayToTuesday() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2009, Calendar.JANUARY, 6);
    	aSchedule = createBiweeklySchedule(periodEndDate.getTime(), sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2008, Calendar.DECEMBER, 24),
		                                                           DateUtil.createDate(2009, Calendar.JANUARY, 6)),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 9),
		                                             DateUtil.createDate(2009, Calendar.JANUARY, 7),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
    /**
     * <P>Test getting the next up-coming PayrollRunInfo in a leap year</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollSchedule with a biweekly pay frequency</LI>
     * <LI>Call PayrollScheduleComponent.getNextScheduledPayrollRunInfo() to get the next up-coming PayrollRunInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
	@Test
	public void testGettingUpcomingPayrollRunInfo_LeapYear() throws Exception {
		// Setup
		Calendar periodEndDate = DateUtil.createCalendar(2012, Calendar.MARCH, 4);
    	aSchedule = createBiweeklySchedule(periodEndDate.getTime(), sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
		// Exercise
		PayrollRunInfo computed = pScheduleComponent.getNextScheduledPayrollRunInfo(aSchedule);
		// Verify
		PayrollRunInfo expected = new PayrollRunInfo(new DateRange(DateUtil.createDate(2012, Calendar.FEBRUARY, 20),
		                                                           DateUtil.createDate(2012, Calendar.MARCH, 4)),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 7),
		                                             DateUtil.createDate(2012, Calendar.MARCH, 5),
		                                             PayrollFrequencyEnum.BIWEEKLY,
		                                             PayrollRunTypeEnum.SCHEDULED);
 		AssertJUnit.assertTrue("Expected PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
	}
	
}
