package com.intuit.sb.payroll.capability.workerpayments;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.EmployerAdjustment;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>PayrollRunAdjustmentsTest</CODE> class contains tests for testing adjustment values after running a
 *  Payroll Run.
 * </P>
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 * 
 */
public class PayrollRunAdjustmentsTest extends WorkerPaymentsComponentTestBase {
	private static Long sWorkerIds[] = {0L, 0L, 0L, 0L};

	@BeforeClass
	public static void setUpPayrollRunAdjustmentsTest() throws Exception {
		getUserTransaction().begin();
		saveEmployerInfo();
		saveWorkLocationAddress();
		setUpSanMateoAddress();
		getUserTransaction().commit();
	}

	@AfterClass
	public static void tearDownPayrollRunAdjustmentsTest() throws Exception {
		getUserTransaction().begin();
		restoreWorkLocationAddress();
		restoreEmployerInfo();
		getUserTransaction().commit();
	}
	
	@BeforeMethod
	public void initialize() throws Exception {
		sWorkerIds[0] = WorkerTestUtils.createSalaryPayrollReadyWorker("Abdul", sMountainViewLocation);
		sWorkerIds[1] = WorkerTestUtils.createSalaryPayrollReadyWorker("Lawrence", sMountainViewLocation);
		sWorkerIds[2] = WorkerTestUtils.createSalaryPayrollReadyWorker("Levi", sMountainViewLocation);
		sWorkerIds[3] = WorkerTestUtils.createSalaryPayrollReadyWorker("Renaldo", sMountainViewLocation);
	}
	
	@AfterMethod
	public void cleanUp() throws Exception {
		getUserTransaction().begin();
		for (int i = 0; i < sWorkerIds.length; i++) {
			if (0 != sWorkerIds[i]) {
				Worker worker = entityService.findById(Worker.class, sWorkerIds[i]);
				WorkerTestUtils.cleanUpWorker(worker);
				sWorkerIds[i] = 0L;
			}
		}
		getUserTransaction().commit();
		getUserTransaction().begin();
	}
	
    /**
     * <P>Test adjustment values after running a single Payroll Run</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object for each worker</LI>
     * <LI>Add all the PaycheckCalculationRequest objects to the list of PaycheckCalculationRequest</LI>
     * <LI>Setup a PayrollRunCommitRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() to commit a Payroll Run</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the payroll run state is committed, the calculated values are correct and the adjustment
     * values are correct</LI>
     * </UL>
     */
	@Test
	public void testSinglePayrollRun() throws Exception {
		getUserTransaction().commit();
    	PayrollRun payrollRun = null;
    	try {
	    	//Setup
	    	aSchedule = createBiweeklySchedule(DateUtil.createDate(sTaxYear, Calendar.JANUARY, 14),
	    	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
	    	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
	    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
	    	for (int i = 0; i < sWorkerIds.length; i++) {
		    	PaycheckCalculationRequest calcRequest = createSalaryPaycheckCalculationRequest(sWorkerIds[i],
		    					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
		    	paycheckCalculationRequests.add(calcRequest);
	    	}
	    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
	        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
	        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
	
	        // Set the Payroll system date to the payroll run date
	        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
	    	//Exercise
	    	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
	     	// Restore the Payroll system date to the current date
	     	resetPayrollSystemDate();
	
	    	//Verify
	    	AssertJUnit.assertTrue(validatePayrollRun(payrollRun, payrollRunRequest));
	    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 4, payrollRun.getPaychecks().size());
	    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
	    					payrollRun.getWorkflowStatus());
	    	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "6993.92"), payrollRun.getTotalNetPay());
	    	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "3159.92"),
	    					payrollRun.getTotalWorkerPdTaxes());
	    	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "1213.38"),
	    					payrollRun.getTotalEmployerPdTaxes());
	    	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
	    					payrollRun.getTotalWorkerPdBenefits());
	    	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
	    					payrollRun.getTotalEmployerPdBenefits());
	    	AssertJUnit.assertEquals("# of EmployerAdjustment returned != 1!", 1,
	    			payrollRun.getEmployerAdjustments().size());
	    	EmployerAdjustment calculatedAdjustment = payrollRun.getEmployerAdjustments().iterator().next();
	    	AssertJUnit.assertEquals("PayrollRun mismatch!", payrollRun, calculatedAdjustment.getPayrollRun());
	    	AssertJUnit.assertEquals("PayrollPeriod mismatch!", payrollRunInfo.getPayPeriod(),
	    			calculatedAdjustment.getPayrollPeriod());
	    	
	    	HashSet<AdjLineItem> expectedAdjLineItems = new HashSet<AdjLineItem>();
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sFUTALawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mUSFUI.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sSST_EELawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.02"),
	    			mUSSSTE.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sSST_ERLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.02"),
	    			mUSSSTER.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sMED_EELawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mUSMEDTE.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sMED_ERLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mUSMEDTER.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sCASUILawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mCASUI.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sCASDILawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mCASDI.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sCAETTLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mCAETT.getPayrollItem()));
	    	AssertJUnit.assertTrue("EmployerAdjustment mismatch!", verifyEmployerAdjustment(expectedAdjLineItems,
	    			calculatedAdjustment.getEmployerAdjLineItems()));
    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
    		getUserTransaction().begin();
        	//Tear down
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    		getUserTransaction().commit();
    	}
	}

    /**
     * <P>Test adjustment values after running Payroll Runs for the first quarter in 2008, Payroll start date of
     * 01/01/2008</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object for each worker</LI>
     * <LI>Add all the PaycheckCalculationRequest objects to the list of PaycheckCalculationRequest</LI>
     * <LI>Setup a PayrollRunCommitRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() to commit Payroll Runs for an entire quarter</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the adjustment values are correct</LI>
     * </UL>
     */
	@Test
	public void testPayrollRunsForFirstQuarter2008() throws Exception {
		getUserTransaction().commit();
		final int numOfPayrollRuns = 6;
		PayrollRunInfo payrollRunInfo = null;
    	PayrollRun payrollRuns[] = new PayrollRun[numOfPayrollRuns];
    	try {
	    	//Setup
			// There are a total of 6 biweekly payroll runs in the first quarter of 2008:
			//   1st paycheck - pay period 01/01/2008 - 01/14/2008, paycheck date of 01/17/2008
			//   2nd paycheck - pay period 01/15/2008 - 01/28/2008, paycheck date of 01/31/2008
			//   3rd paycheck - pay period 01/29/2008 - 02/11/2008, paycheck date of 02/14/2008
			//   4th paycheck - pay period 02/12/2008 - 02/25/2008, paycheck date of 02/28/2008
			//   5th paycheck - pay period 02/26/2008 - 03/10/2008, paycheck date of 03/13/2008
			//   6th paycheck - pay period 03/11/2008 - 03/24/2008, paycheck date of 03/27/2008
			Date nextPayPeriodEndDate = DateUtil.createDate(2007, Calendar.DECEMBER, 31);
			for (int j = 0; j < numOfPayrollRuns; j++) {
				// Calculate the next pay period end date
				nextPayPeriodEndDate = DateUtil.getNextDate(nextPayPeriodEndDate, 14);
		    	aSchedule = createBiweeklySchedule(nextPayPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		    			sDEFAULT_PAY_DAY_OFFSET);
		    	payrollRunInfo = createAPayrollRunInfo(aSchedule);
		    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
		    	for (int i = 0; i < sWorkerIds.length; i++) {
			    	PaycheckCalculationRequest calcRequest = createSalaryPaycheckCalculationRequest(sWorkerIds[i],
			    					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
			    	paycheckCalculationRequests.add(calcRequest);
		    	}
		    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
		        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
		        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
		
		        // Set the Payroll system date to the payroll run date
		        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
		    	//Exercise
		    	payrollRuns[j] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
		     	// Restore the Payroll system date to the current date
		     	resetPayrollSystemDate();
		
		    	//Verify
		    	AssertJUnit.assertTrue(validatePayrollRun(payrollRuns[j], payrollRunRequest));
		    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 4, payrollRuns[j].getPaychecks().size());
		    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
		    					payrollRuns[j].getWorkflowStatus());
			}    	
			PayrollRun payrollRun = payrollRuns[numOfPayrollRuns-1];
	    	AssertJUnit.assertEquals("# of EmployerAdjustment returned != 1!", 1,
	    			payrollRun.getEmployerAdjustments().size());
	    	EmployerAdjustment calculatedAdjustment = payrollRun.getEmployerAdjustments().iterator().next();
	    	AssertJUnit.assertEquals("PayrollRun mismatch!", payrollRun, calculatedAdjustment.getPayrollRun());
	    	AssertJUnit.assertEquals("PayrollPeriod mismatch!", payrollRunInfo.getPayPeriod(),
	    			calculatedAdjustment.getPayrollPeriod());
	    	
	    	HashSet<AdjLineItem> expectedAdjLineItems = new HashSet<AdjLineItem>();
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sSST_EELawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.02"),
	    			mUSSSTE.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sSST_ERLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.02"),
	    			mUSSSTER.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sMED_EELawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mUSMEDTE.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sMED_ERLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mUSMEDTER.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sCASDILawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mCASDI.getPayrollItem()));
	    	AssertJUnit.assertTrue("EmployerAdjustment mismatch!", verifyEmployerAdjustment(expectedAdjLineItems,
	    			calculatedAdjustment.getEmployerAdjLineItems()));
    	} catch (Exception e) {
    		e.printStackTrace();
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
    		getUserTransaction().begin();
        	//Tear down
    		for (int j = 0; j < numOfPayrollRuns; j++) {
	    		if (payrollRuns[j] != null) {
	            	Long payrollRunId = payrollRuns[j].getId();
	            	cleanUpPayrollRun(payrollRunId);
	    		}
    		}
    		getUserTransaction().commit();
    	}
	}

    /**
     * <P>Test adjustment values after running Payroll Runs into the 2nd quarter in 2008, Payroll start date of
     * 01/01/2008</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object for each worker</LI>
     * <LI>Add all the PaycheckCalculationRequest objects to the list of PaycheckCalculationRequest</LI>
     * <LI>Setup a PayrollRunCommitRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() to commit Payroll Runs into the 2nd quarter</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the adjustment values are correct</LI>
     * </UL>
     */
	//TODO  Enable this test when the code is implemented.
	@Test(enabled = false)
	public void testPayrollRunsIntoSecondQuarter2008() throws Exception {
		getUserTransaction().commit();
		final int numOfPayrollRuns = 8;
		PayrollRunInfo payrollRunInfo = null;
    	PayrollRun payrollRuns[] = new PayrollRun[numOfPayrollRuns];
    	try {
	    	//Setup
			// There are a total of 7 biweekly payroll runs that puts the last payroll run into the 2nd quarter:
			//   1st paycheck - pay period 01/01/2008 - 01/14/2008, paycheck date of 01/17/2008
			//   2nd paycheck - pay period 01/15/2008 - 01/28/2008, paycheck date of 01/31/2008
			//   3rd paycheck - pay period 01/29/2008 - 02/11/2008, paycheck date of 02/14/2008
			//   4th paycheck - pay period 02/12/2008 - 02/25/2008, paycheck date of 02/28/2008
			//   5th paycheck - pay period 02/26/2008 - 03/10/2008, paycheck date of 03/13/2008
			//   6th paycheck - pay period 03/11/2008 - 03/24/2008, paycheck date of 03/27/2008
			//   7th paycheck - pay period 03/25/2008 - 04/07/2008, paycheck date of 04/10/2008
			Date nextPayPeriodEndDate = DateUtil.createDate(2007, Calendar.DECEMBER, 31);
			for (int j = 0; j < numOfPayrollRuns; j++) {
				// Calculate the next pay period end date
				nextPayPeriodEndDate = DateUtil.getNextDate(nextPayPeriodEndDate, 14);
		    	aSchedule = createBiweeklySchedule(nextPayPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		    			sDEFAULT_PAY_DAY_OFFSET);
		    	payrollRunInfo = createAPayrollRunInfo(aSchedule);
		    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
		    	for (int i = 0; i < sWorkerIds.length; i++) {
			    	PaycheckCalculationRequest calcRequest = createSalaryPaycheckCalculationRequest(sWorkerIds[i],
			    					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
			    	paycheckCalculationRequests.add(calcRequest);
		    	}
		    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
		        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
		        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
		
		        // Set the Payroll system date to the payroll run date
		        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
		    	//Exercise
		        payrollRuns[j] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
		     	// Restore the Payroll system date to the current date
		     	resetPayrollSystemDate();
		
		    	//Verify
		    	AssertJUnit.assertTrue(validatePayrollRun(payrollRuns[j], payrollRunRequest));
		    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 4, payrollRuns[j].getPaychecks().size());
		    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
		    			payrollRuns[j].getWorkflowStatus());
			}    	
			PayrollRun payrollRun = payrollRuns[numOfPayrollRuns-1];
	    	AssertJUnit.assertEquals("# of EmployerAdjustment returned != 1!", 1,
	    			payrollRun.getEmployerAdjustments().size());
	    	EmployerAdjustment calculatedAdjustment = payrollRun.getEmployerAdjustments().iterator().next();
	    	AssertJUnit.assertEquals("PayrollRun mismatch!", payrollRun, calculatedAdjustment.getPayrollRun());
	    	AssertJUnit.assertEquals("PayrollPeriod mismatch!", payrollRunInfo.getPayPeriod(),
	    			calculatedAdjustment.getPayrollPeriod());
	    	
	    	HashSet<AdjLineItem> expectedAdjLineItems = new HashSet<AdjLineItem>();
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sSST_EELawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.02"),
	    			mUSSSTE.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sSST_ERLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.02"),
	    			mUSSSTER.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sMED_EELawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.03"),
	    			mUSMEDTE.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sMED_ERLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.03"),
	    			mUSMEDTER.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sCASDILawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.03"),
	    			mCASDI.getPayrollItem()));
	    	AssertJUnit.assertTrue("EmployerAdjustment mismatch!", verifyEmployerAdjustment(expectedAdjLineItems,
	    			calculatedAdjustment.getEmployerAdjLineItems()));
    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
    		getUserTransaction().begin();
        	//Tear down
    		for (int j = 0; j < numOfPayrollRuns; j++) {
	    		if (payrollRuns[j] != null) {
	            	Long payrollRunId = payrollRuns[j].getId();
	            	cleanUpPayrollRun(payrollRunId);
	    		}
    		}
    		getUserTransaction().commit();
    	}
	}

    /**
     * <P>Test adjustment values after running Payroll Runs for the entire year in 2008, Payroll start date of
     * 01/01/2008</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object for each worker</LI>
     * <LI>Add all the PaycheckCalculationRequest objects to the list of PaycheckCalculationRequest</LI>
     * <LI>Setup a PayrollRunCommitRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() to commit Payroll Runs for the entire year in 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the adjustment values are correct</LI>
     * </UL>
     */
	//TODO  Enable this test when the code is implemented.
	@Test(enabled = false)
	public void testPayrollRunsForYear2008() throws Exception {
		getUserTransaction().commit();
		final int numOfPayrollRuns = 25;
		PayrollRunInfo payrollRunInfo = null;
    	PayrollRun payrollRuns[] = new PayrollRun[numOfPayrollRuns];
    	try {
	    	//Setup
			// There are a total of 25 biweekly payroll runs for the year 2008:
			//    1st paycheck - pay period 01/01/2008 - 01/14/2008, paycheck date of 01/17/2008
			//    2nd paycheck - pay period 01/15/2008 - 01/28/2008, paycheck date of 01/31/2008
			//    3rd paycheck - pay period 01/29/2008 - 02/11/2008, paycheck date of 02/14/2008
			//    4th paycheck - pay period 02/12/2008 - 02/25/2008, paycheck date of 02/28/2008
			//    5th paycheck - pay period 02/26/2008 - 03/10/2008, paycheck date of 03/13/2008
			//    6th paycheck - pay period 03/11/2008 - 03/24/2008, paycheck date of 03/27/2008
			//    7th paycheck - pay period 03/25/2008 - 04/07/2008, paycheck date of 04/10/2008
			//    8th paycheck - pay period 04/08/2008 - 04/21/2008, paycheck date of 04/27/2008
			//    9th paycheck - pay period 04/22/2008 - 05/05/2008, paycheck date of 05/08/2008
			//   10th paycheck - pay period 05/06/2008 - 05/19/2008, paycheck date of 05/22/2008
			//   11th paycheck - pay period 05/20/2008 - 06/02/2008, paycheck date of 06/05/2008
			//   12th paycheck - pay period 06/03/2008 - 06/16/2008, paycheck date of 06/19/2008
			//   13th paycheck - pay period 06/17/2008 - 06/30/2008, paycheck date of 07/03/2008
			//   14th paycheck - pay period 07/01/2008 - 07/14/2008, paycheck date of 07/17/2008
			//   15th paycheck - pay period 07/15/2008 - 07/28/2008, paycheck date of 07/31/2008
			//   16th paycheck - pay period 07/29/2008 - 08/11/2008, paycheck date of 08/14/2008
			//   17th paycheck - pay period 08/12/2008 - 08/25/2008, paycheck date of 08/28/2008
			//   18th paycheck - pay period 08/26/2008 - 09/08/2008, paycheck date of 09/11/2008
			//   19th paycheck - pay period 09/09/2008 - 09/22/2008, paycheck date of 09/25/2008
			//   20th paycheck - pay period 09/23/2008 - 10/06/2008, paycheck date of 10/09/2008
			//   21th paycheck - pay period 10/07/2008 - 10/20/2008, paycheck date of 10/23/2008
			//   22th paycheck - pay period 10/21/2008 - 11/03/2008, paycheck date of 11/06/2008
			//   23th paycheck - pay period 11/04/2008 - 11/17/2008, paycheck date of 11/20/2008
			//   24th paycheck - pay period 11/18/2008 - 12/01/2008, paycheck date of 12/04/2008
			//   25th paycheck - pay period 12/02/2008 - 12/15/2008, paycheck date of 12/18/2008
			Date nextPayPeriodEndDate = DateUtil.createDate(2007, Calendar.DECEMBER, 31);
			for (int j = 0; j < numOfPayrollRuns; j++) {
				// Calculate the next pay period end date
				nextPayPeriodEndDate = DateUtil.getNextDate(nextPayPeriodEndDate, 14);
		    	aSchedule = createBiweeklySchedule(nextPayPeriodEndDate, sDEFAULT_PAYROLL_RUN_OFFSET,
		    			sDEFAULT_PAY_DAY_OFFSET);
		    	payrollRunInfo = createAPayrollRunInfo(aSchedule);
		    	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
		    	for (int i = 0; i < sWorkerIds.length; i++) {
			    	PaycheckCalculationRequest calcRequest = createSalaryPaycheckCalculationRequest(sWorkerIds[i],
			    					payrollRunInfo, PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
			    	paycheckCalculationRequests.add(calcRequest);
		    	}
		    	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
		        payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
		        payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
		
		        // Set the Payroll system date to the payroll run date
		        setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
		    	//Exercise
		        payrollRuns[j] = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
		     	// Restore the Payroll system date to the current date
		     	resetPayrollSystemDate();
		
		    	//Verify
		    	AssertJUnit.assertTrue(validatePayrollRun(payrollRuns[j], payrollRunRequest));
		    	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 4, payrollRuns[j].getPaychecks().size());
		    	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
		    			payrollRuns[j].getWorkflowStatus());
			}    	
			PayrollRun payrollRun = payrollRuns[numOfPayrollRuns-1];
	    	AssertJUnit.assertEquals("# of EmployerAdjustment returned != 1!", 1,
	    			payrollRun.getEmployerAdjustments().size());
	    	EmployerAdjustment calculatedAdjustment = payrollRun.getEmployerAdjustments().iterator().next();
	    	AssertJUnit.assertEquals("PayrollRun mismatch!", payrollRun, calculatedAdjustment.getPayrollRun());
	    	AssertJUnit.assertEquals("PayrollPeriod mismatch!", payrollRunInfo.getPayPeriod(),
	    			calculatedAdjustment.getPayrollPeriod());
	    	
	    	HashSet<AdjLineItem> expectedAdjLineItems = new HashSet<AdjLineItem>();
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sSST_EELawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.02"),
	    			mUSSSTE.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sSST_ERLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "0.02"),
	    			mUSSSTER.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sMED_EELawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mUSMEDTE.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sMED_ERLawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mUSMEDTER.getPayrollItem()));
	    	expectedAdjLineItems.add(new AdjLineItem(QboeLawIdConstants.sCASDILawId,
	    			new Money(LocaleGuru.getInstance().getCurrency(), "-0.01"),
	    			mCASDI.getPayrollItem()));
	    	AssertJUnit.assertTrue("EmployerAdjustment mismatch!", verifyEmployerAdjustment(expectedAdjLineItems,
	    			calculatedAdjustment.getEmployerAdjLineItems()));
    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	} finally {
    		getUserTransaction().begin();
        	//Tear down
    		for (int j = 0; j < numOfPayrollRuns; j++) {
	    		if (payrollRuns[j] != null) {
	            	Long payrollRunId = payrollRuns[j].getId();
	            	cleanUpPayrollRun(payrollRunId);
	    		}
    		}
    		getUserTransaction().commit();
    	}
	}

}
