package com.intuit.sb.payroll.capability.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.WorkerPaymentsComponentTasks;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PaycheckWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.Worker;

/**
 * <P>
 * The <CODE>UpdatePayrollRunTest</CODE> class contains tests for testing the
 * updatePayrollRun() in WorkerPaymentsComponent
 * </P>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class UpdatePayrollRunTest extends WorkerPaymentsComponentTestBase {
	private static Long sReadyForPayWorkerId = 0L;
	private BigDecimal mHourlyRate = new BigDecimal("10.0");
	private BigDecimal mBonusAmount = new BigDecimal("200.0");

	@BeforeClass
	public static void setUpUpdatePayrollRunTest() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			saveWorkLocationAddress();
			setUpSanMateoAddress();
			sReadyForPayWorkerId = WorkerTestUtils.createPayrollReadyWorker("Johnny", sMountainViewLocation);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpUpdatePayrollRunTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownUpdatePayrollRunTest() {
		try {
			getUserTransaction().begin();
			if (0 != sReadyForPayWorkerId) {
    			Worker worker = entityService.findById(Worker.class, sReadyForPayWorkerId);
    			WorkerTestUtils.cleanUpWorker(worker);
    			sReadyForPayWorkerId = 0L;
			}
			restoreWorkLocationAddress();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownUpdatePayrollRunTest(): " + t.getMessage());
		}
	}

    /**
     * <P>Test updatePayrollRun() API with 1 hourly worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a PayrollRunInfo with a biweekly pay frequency using default payroll run values</LI>
     * <LI>Setup a PaycheckCalculationRequest object with some compensation values for the worker</LI>
     * <LI>Setup a PayrollRunRequest object</LI>
     * <LI>Call WorkerPaymentsComponent.commitPayrollRun() with a valid PayrollRunRequest</LI>
     * <LI>Set the CheckNumber and the PaycheckStatus for the calculated paycheck</LI>
     * <LI>Call WorkerPaymentsComponent.updatePayrollRun() with the payrollRun id returned from the commitPayrollRun()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the paycheck for the PayrollRun is updated</LI>
     * </UL>
     */
	@Test
	public void testUpdatePayrollRunWith1HourlyWorker() throws Exception {
    	PayrollRun payrollRun = null;
    	String checkNumber = "123";
    	String memo = "testmemo";
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.JANUARY, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	
        	//Exercise
    		PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    		payrollRunUpdateRequest.setPayrollRunId(payrollRun.getId());
    		List<PaycheckUpdateRequest> paycheckUpdateRequests = new ArrayList<PaycheckUpdateRequest>();
    		PaycheckUpdateRequest paycheckUpdateRequest = new PaycheckUpdateRequest();
    		paycheckUpdateRequest.setCheckNumber(checkNumber);
    		paycheckUpdateRequest.setMemo(memo);
    		Set<Paycheck> paychecks = payrollRun.getPaychecks();
    		Paycheck paycheck = paychecks.iterator().next();
    		paycheckUpdateRequest.setPaycheckId(paycheck.getId());
    		paycheckUpdateRequest.setPaycheckStatus(PaycheckWorkflowStatusEnum.DISTRIBUTED);
    		paycheckUpdateRequests.add(paycheckUpdateRequest);
    		payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
        	PayrollRun savedPayrollRun = sWorkerPaymentsComponent.updatePayrollRun(payrollRunUpdateRequest);

        	//Verify
        	AssertJUnit.assertTrue(validatePayrollRun(savedPayrollRun, payrollRunRequest));
        	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, savedPayrollRun.getPaychecks().size());
        	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
        					savedPayrollRun.getWorkflowStatus());
        	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "908.27"), savedPayrollRun.getTotalNetPay());
        	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "241.72"),
        	                savedPayrollRun.getTotalWorkerPdTaxes());
        	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "137.43"),
        	                savedPayrollRun.getTotalEmployerPdTaxes());
        	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
        	                savedPayrollRun.getTotalWorkerPdBenefits());
        	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
        	                savedPayrollRun.getTotalEmployerPdBenefits());
        	
    		Set<Paycheck> updatedPaychecks = savedPayrollRun.getPaychecks();
    		Paycheck updatedPaycheck = updatedPaychecks.iterator().next();
        	AssertJUnit.assertEquals("CheckNumber mismatch!", checkNumber, updatedPaycheck.getTxnNo());
        	AssertJUnit.assertEquals("Memo mismatch", memo, updatedPaycheck.getMemo());
        	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.DISTRIBUTED,
        	                    updatedPaycheck.getWorkflowStatus());
    	} finally {
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    	}
	}
	
	@Test
	public void testUpdatePayrollRunTasks() throws Exception {
    	PayrollRun payrollRun = null;
    	String checkNumber = "123";
    	String memo = "testmemo";
    	try {
        	//Setup
        	aSchedule = createBiweeklySchedule(DateUtil.createDate(2008, Calendar.DECEMBER, 14),
        	                                   sDEFAULT_PAYROLL_RUN_OFFSET, sDEFAULT_PAY_DAY_OFFSET);
        	PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
        	List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();
        	PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
        					payrollRunInfo, mHourlyRate, new Double(80), new Double(10), mBonusAmount,
        					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
        	paycheckCalculationRequests.add(calcRequest);
        	PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
            payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
            payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
        	payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
        	
         	//Verify that we have a task to finish the distribution.
         	TaskManager taskManager = qaBootstrap.getEjbFromClass(TaskManager.class);
         	List<Task> tasks = taskManager.findActiveTasksByClassAndPayload(WorkerPaymentsComponentTasks.EXECUTE_PAYROLL_RUN_CLASS, new Long(payrollRun.getId()));
         	AssertJUnit.assertTrue(tasks.size() == 1);
         	Task completeDistributionTask = tasks.get(0);
         	AssertJUnit.assertEquals(WorkerPaymentsComponentTasks.FINISH_PAYCHECK_DISTRIBUTION_TASK, completeDistributionTask.getName());
         	
        	//Exercise
    		PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
    		payrollRunUpdateRequest.setPayrollRunId(payrollRun.getId());
    		List<PaycheckUpdateRequest> paycheckUpdateRequests = new ArrayList<PaycheckUpdateRequest>();
    		PaycheckUpdateRequest paycheckUpdateRequest = new PaycheckUpdateRequest();
    		paycheckUpdateRequest.setCheckNumber(checkNumber);
    		paycheckUpdateRequest.setMemo(memo);
    		Set<Paycheck> paychecks = payrollRun.getPaychecks();
    		Paycheck paycheck = paychecks.iterator().next();
    		paycheckUpdateRequest.setPaycheckId(paycheck.getId());
    		paycheckUpdateRequest.setPaycheckStatus(PaycheckWorkflowStatusEnum.DISTRIBUTED);
    		paycheckUpdateRequests.add(paycheckUpdateRequest);
    		payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
        	PayrollRun savedPayrollRun = sWorkerPaymentsComponent.updatePayrollRun(payrollRunUpdateRequest);

        	//Verify the task to finish the distribution is gone.
         	tasks = taskManager.findActiveTasksByClassAndPayload(WorkerPaymentsComponentTasks.EXECUTE_PAYROLL_RUN_CLASS, new Long(payrollRun.getId()));
         	AssertJUnit.assertTrue(tasks.size() == 0);
 
         	// Verify the rest of the run.
        	AssertJUnit.assertTrue(validatePayrollRun(savedPayrollRun, payrollRunRequest));
        	AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, savedPayrollRun.getPaychecks().size());
        	AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
        					savedPayrollRun.getWorkflowStatus());
        	AssertJUnit.assertEquals("totalNetPay mismatch!", new Money("USD", "908.27"), savedPayrollRun.getTotalNetPay());
        	AssertJUnit.assertEquals("totalWorkerPdTaxes mismatch!", new Money("USD", "241.72"),
        	                savedPayrollRun.getTotalWorkerPdTaxes());
        	AssertJUnit.assertEquals("totalEmployerPdTaxes mismatch!", new Money("USD", "137.43"),
        	                savedPayrollRun.getTotalEmployerPdTaxes());
        	AssertJUnit.assertEquals("totalWorkerPdBenefits mismatch!", new Money("USD", "0.00"),
        	                savedPayrollRun.getTotalWorkerPdBenefits());
        	AssertJUnit.assertEquals("totalEmployerPdBenefits mismatch!", new Money("USD", "0.00"),
        	                savedPayrollRun.getTotalEmployerPdBenefits());
        	
    		Set<Paycheck> updatedPaychecks = savedPayrollRun.getPaychecks();
    		Paycheck updatedPaycheck = updatedPaychecks.iterator().next();
        	AssertJUnit.assertEquals("CheckNumber mismatch!", checkNumber, updatedPaycheck.getTxnNo());
        	AssertJUnit.assertEquals("Memo mismatch", memo, updatedPaycheck.getMemo());
        	AssertJUnit.assertEquals("PaycheckStatus mismatch!", PaycheckWorkflowStatusEnum.DISTRIBUTED,
        	                    updatedPaycheck.getWorkflowStatus());
    	} finally {
        	//Tear down
    		if (payrollRun != null) {
            	Long payrollRunId = payrollRun.getId();
            	cleanUpPayrollRun(payrollRunId);
    		}
    	}
	}

	/**
	 * <P>Test calling updatePayrollRun() with a null PayrollrunUpdateRequest</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.updatePayrollRun() with a null PayrollrunUpdateRequest</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPayrollRunUpdateRequest() throws Exception {
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.updatePayrollRun(null);
	}

	/**
	 * <P>Test calling updatePayrollRun() with a null PaycheckUpdateRequest list</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.updatePayrollRun() with a null PaycheckUpdateRequest list</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPaycheckUpdateRequestList() throws Exception {
		// Setup
		PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
		payrollRunUpdateRequest.setPayrollRunId(1L);
		payrollRunUpdateRequest.setPaycheckUpdateRequests(null);
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.updatePayrollRun(payrollRunUpdateRequest);
	}

	/**
	 * <P>Test calling updatePayrollRun() with an empty PaycheckUpdateRequest list</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.updatePayrollRun() with an empty PaycheckUpdateRequest list</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testEmptyPaycheckUpdateRequestList() throws Exception {
		// Setup
		PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
		payrollRunUpdateRequest.setPayrollRunId(1L);
		List<PaycheckUpdateRequest> paycheckUpdateRequests = new ArrayList<PaycheckUpdateRequest>();
		payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.updatePayrollRun(payrollRunUpdateRequest);
	}

	/**
	 * <P>Test calling updatePayrollRun() with a null PaycheckUpdateRequest list element</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.updatePayrollRun() with a null PaycheckUpdateRequest list element</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testNullPaycheckUpdateRequestListElement() throws Exception {
		// Setup
		PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
		payrollRunUpdateRequest.setPayrollRunId(1L);
		List<PaycheckUpdateRequest> paycheckUpdateRequests = new ArrayList<PaycheckUpdateRequest>();
		paycheckUpdateRequests.add(null);
		payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.updatePayrollRun(payrollRunUpdateRequest);
	}

	/**
	 * <P>Test calling updatePayrollRun() with a non-existent PayrollRunId</P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call the WorkerPaymentsComponent.updatePayrollRun() with a non-existent PayrollRunId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that PayrollServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testNonExistentPayrollRunId() throws Exception {
		// Setup
		PayrollRunUpdateRequest payrollRunUpdateRequest = new PayrollRunUpdateRequest();
		payrollRunUpdateRequest.setPayrollRunId(934893943L);
		List<PaycheckUpdateRequest> paycheckUpdateRequests = new ArrayList<PaycheckUpdateRequest>();
		PaycheckUpdateRequest paycheckUpdateRequest = new PaycheckUpdateRequest();
		paycheckUpdateRequest.setCheckNumber(null);
		paycheckUpdateRequest.setPaycheckId(0L);
		paycheckUpdateRequest.setPaycheckStatus(PaycheckWorkflowStatusEnum.NOTSET);
		paycheckUpdateRequests.add(paycheckUpdateRequest);
		payrollRunUpdateRequest.setPaycheckUpdateRequests(paycheckUpdateRequests);
    	//Exercise
		@SuppressWarnings("unused")
		PayrollRun payollRun = sWorkerPaymentsComponent.updatePayrollRun(payrollRunUpdateRequest);
	}

}
