package com.intuit.sb.payroll.capability.workerpayments.impl;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;

/**
* Tests the logic in the PayrollScheduleValidator
*
* $Author$
* $Revision$
* $DateTime$
*
*/

public class PayrollScheduleValidatorTest {
	private static PayrollScheduleValidator validator;
	private PayrollSchedule aSchedule = null;
	
	@BeforeClass
	public static void initializePayrollScheduleValidator()
	{
		validator = PayrollScheduleValidator.getInstance();
	}
	
    @BeforeMethod
    public void setUp() throws Exception {
    	aSchedule = PayrollScheduleTestUtils.createPayrollSchedule(DateUtil.getNowCalendar(),
    	                                                           PayrollSchedule.LAST_DAY_OF_MONTH,
																   PayrollFrequencyEnum.SEMIMONTHLY, 0, 2);
    }
    
    /**
     * <P>
     * Test with a null PayrollSchedule
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a null PayrollSchedule</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testNullPayrollSchedule() throws Exception {
        // Exercise
        validator.validateEntity(null);
    }
    
    /**
     * <P>
     * Test with a null PayrollFrequencyEnum in the PayrollSchedule
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a null PayrollFrequencyEnum</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testNullPayrollFrequencyEnum() throws Exception {
    	// Setup
    	aSchedule.setFrequency(null);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with PayrollFrequencyEnum.NOTSET in the PayrollSchedule
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with PayrollFrequencyEnum.NOTSET</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testPayrollFrequencyEnumNotSet() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.NOTSET);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with null NextPayPeriodEndDate in the PayrollSchedule using a weekly schedule
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a null NextPayPeriodEndDate</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testWeeklyNullNextPayPeriodEndDate() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.WEEKLY);
    	aSchedule.setNextPayPeriodEndDate(null);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test PayrollRunDate after PayDate using a weekly schedule
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with the PayrollRunDate after the PayDate</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testWeeklyPayrollRunDateAfterPayDate() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.WEEKLY);
    	aSchedule.setPayDayOffset(1);
    	aSchedule.setPayrollRunDayOffset(2);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a monthly schedule, but the PayPeriodEndDayofMonth1 value is 0
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a monthly schedule and
     * PayPeriodEndDayofMonth1 set to 0</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testMonthlyPayPeriodEndDayofMonth1ValueOfZero() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.MONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth1(0);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a monthly schedule, but the PayPeriodEndDayofMonth1 value is 29
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a monthly schedule and
     * PayPeriodEndDayofMonth1 set to 29</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testMonthlyPayPeriodEndDayofMonth1ValueOf29() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.MONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth1(29);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a monthly schedule, and the PayPeriodEndDayofMonth1 value is LAST_DAY_OF_MONTH
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a monthly schedule and
     * PayPeriodEndDayofMonth1 set to LAST_DAY_OF_MONTH</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that no ValidationException is thrown</L1>
     * </UL>
     */
    @Test
    public void testMonthlyPayPeriodEndDayofMonth1ValueOfLastDayOfMonth() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.MONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth1(PayrollSchedule.LAST_DAY_OF_MONTH);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a semimonthly schedule, but the PayPeriodEndDayofMonth1 value is 0
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a semimonthly schedule and
     * PayPeriodEndDayofMonth1 set to 0</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testSemimonthlyPayPeriodEndDayofMonth1ValueOfZero() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth1(0);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a semimonthly schedule, but the PayPeriodEndDayofMonth1 value is 29
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a semimonthly schedule and
     * PayPeriodEndDayofMonth1 set to 29</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testSemimonthlyPayPeriodEndDayofMonth1ValueOf29() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth1(29);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a semimonthly schedule, and the PayPeriodEndDayofMonth1 value is LAST_DAY_OF_MONTH
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a semimonthly schedule and
     * PayPeriodEndDayofMonth1 set to LAST_DAY_OF_MONTH</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testSemimonthlyPayPeriodEndDayofMonth1ValueOfLastDayOfMonth() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth1(PayrollSchedule.LAST_DAY_OF_MONTH);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a semimonthly schedule, but the PayPeriodEndDayofMonth2 value is 0
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a semimonthly schedule and
     * PayPeriodEndDayofMonth2 set to 0</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testSemimonthlyPayPeriodEndDayofMonth2ValueOfZero() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth2(0);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a semimonthly schedule, but the PayPeriodEndDayofMonth2 value is 29
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a semimonthly schedule and
     * PayPeriodEndDayofMonth2 set to 29</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testSemimonthlyPayPeriodEndDayofMonth2ValueOf29() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth2(29);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a semimonthly schedule, and the PayPeriodEndDayofMonth1 & PayPeriodEndDayofMonth2 value are
     *  LAST_DAY_OF_MONTH
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a semimonthly schedule and
     * PayPeriodEndDayofMonth1 & PayPeriodEndDayofMonth2 set to LAST_DAY_OF_MONTH</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that a ValidationException is thrown</L1>
     * </UL>
     */
    @Test(expectedExceptions = ValidationException.class)
    public void testSemimonthlyPayPeriodEndDayofMonth1PayPeriodEndDayofMonth2ValueOfLastDayOfMonth() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth1(PayrollSchedule.LAST_DAY_OF_MONTH);
    	aSchedule.setPayPeriodEndDayofMonth2(PayrollSchedule.LAST_DAY_OF_MONTH);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
    /**
     * <P>
     * Test with a semimonthly schedule, and the PayPeriodEndDayofMonth2 value is LAST_DAY_OF_MONTH
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call PayrollScheduleValidator.validateEntity() with a semimonthly schedule and
     * PayPeriodEndDayofMonth2 set to LAST_DAY_OF_MONTH</LI>
     * </OL>
     * 
     * <UL>
     * <L1>Verify that no ValidationException is thrown</L1>
     * </UL>
     */
    //@Test
    public void testSemimonthlyPayPeriodEndDayofMonth2ValueOfLastDayOfMonth() throws Exception {
    	// Setup
    	aSchedule.setFrequency(PayrollFrequencyEnum.SEMIMONTHLY);
    	aSchedule.setPayPeriodEndDayofMonth2(PayrollSchedule.LAST_DAY_OF_MONTH);
    	
        // Exercise
    	validator.validateEntity(aSchedule);
    }
    
}
