package com.intuit.sb.payroll.services.benefits.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.benefits.BenefitsComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.HealthBenefitDtl;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.benefits.BenefitsService;
import com.intuit.sb.payroll.services.benefits.HealthBenefit;
import com.intuit.sb.payroll.services.benefits.HealthBenefitResponse;
import com.intuit.sb.payroll.services.benefits.HealthBenefitTemplate;
import com.intuit.sb.payroll.services.benefits.HealthBenefitType;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;

/**
 *
 */
@Stateless(name = "BenefitsService")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class BenefitsServiceBean implements BenefitsService {
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(BenefitsServiceBean.class);

	@EJB
	private WorkerComponent workerComponent;
	
	@EJB
	private BenefitsComponent benefitsComponent;
	
	/**
	 * Find the Health Benefit template of a given type for the employer.
     * @param HealthBenefitType type
     * @return a HealthBenefit template
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 */
	public HealthBenefitTemplate findEmployerHealthBenefit(HealthBenefitType type) throws ServiceFailureException, ValidationException {
	    if (null == type) {
            throw new ServiceFailureException("HealthBenefitType cannot be null");
	    }
       com.intuit.sb.payroll.entity.model.HealthBenefit benefit = benefitsComponent.findHealthBenefit(BenefitMapper.mapToHealthBenefitTypeEnum(type));
       return BenefitMapper.mapToHealthBenefitTemplate(benefit);
	}

	/**
	 * Find the Health Benefit templates for the employer.
     * @param 
     * @return a HealthBenefitTemplate dto array
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 */
	public HealthBenefitTemplate[] findEmployerHealthBenefits() throws ServiceFailureException, ValidationException {
        List<com.intuit.sb.payroll.entity.model.HealthBenefit> benefits = benefitsComponent.findAllBenefits();
        List<HealthBenefitTemplate> benefitTemplates = new ArrayList<HealthBenefitTemplate>();
        for (com.intuit.sb.payroll.entity.model.HealthBenefit benefit : benefits)
        {
        	benefitTemplates.add(BenefitMapper.mapToHealthBenefitTemplate(benefit));
        }
        
        HealthBenefitTemplate[] benefitArray = new HealthBenefitTemplate[benefits.size()];
        return benefitTemplates.toArray(benefitArray);
	}

	
	/**
	 * Provides a listing of all the Health Benefits for the given worker.
     * @param workerId
     * @return an array containing the added HealthBenefits
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 */
	public HealthBenefit[] findAllHealthBenefits(ObjectHandle workerHandle) throws ServiceFailureException, ValidationException {
		CommonValidators.validateObjectHandle(workerHandle);
		if (sLogger.isDebugEnabled()) sLogger.debug("findAllHealthBenefits for" + workerHandle.toString());
		Long workerId = workerHandle.getId();
        Worker eiWorker = workerComponent.findWorker(workerId);
		HealthBenefit[] benefits = BenefitMapper.mapToHealthBenefits(eiWorker.getHealthBenefitDtls());
		if (sLogger.isDebugEnabled()) sLogger.debug("Found Benefits: " + ToStringBuilder.reflectionToString(benefits, ToStringStyle.SIMPLE_STYLE));
		return benefits;
	}

    /**
     * Add the given HealthBenefits to the given worker.
     * @param workerHandle
     * @param healthBenefits array
     * @return an response dto containing the worker and their HealthBenefits
     */
	public HealthBenefitResponse addHealthBenefits(ObjectHandle workerHandle, HealthBenefit[] healthBenefits) throws ServiceFailureException, ValidationException  {
		CommonValidators.validateObjectHandle(workerHandle);
    	validateHealthBenefits(healthBenefits);
        // Convert to HealthBenefitDtl set
        List<HealthBenefitDtl> dtlSet = BenefitMapper.mapToHealthBenefitDtls(healthBenefits);
        
        Worker worker = workerComponent.findWorker(workerHandle.getId());
        // Add the benefit details to the worker
        Worker eiWorker = benefitsComponent.addHealthBenefitDtls(worker, dtlSet);

        // Return the worker HealthBenefits response
        Set<HealthBenefitDtl> responseSet = eiWorker.getHealthBenefitDtls();
        HealthBenefitResponse response = new HealthBenefitResponse(WorkerMapper.mapFromEntityToPojo(eiWorker), BenefitMapper.mapToHealthBenefits(responseSet));
        return response; 
	}
    
    /**
     * Update the given HealthBenefits for a worker.  
     * @param workerHandle
     * @param healthBenefits array
     * @return an response dto containing the worker and their HealthBenefits
     */
    public HealthBenefitResponse updateHealthBenefits(ObjectHandle workerHandle, HealthBenefit[] healthBenefits) throws ServiceFailureException, ValidationException {
		CommonValidators.validateObjectHandle(workerHandle);
    	validateHealthBenefits(healthBenefits);
        // Convert to HealthBenefitDtl set
        List<HealthBenefitDtl> dtlSet = BenefitMapper.mapToHealthBenefitDtls(healthBenefits);

        // Update the worker health benefit details
        Worker eiWorker = benefitsComponent.updateHealthBenefitDtls(workerHandle.getId(), dtlSet);

        // Return the worker HealthBenefits response
        Set<HealthBenefitDtl> responseSet = eiWorker.getHealthBenefitDtls();
        HealthBenefitResponse response = new HealthBenefitResponse(WorkerMapper.mapFromEntityToPojo(eiWorker), BenefitMapper.mapToHealthBenefits(responseSet));
        return response; 
	}

    /**
     * Delete the given HealthBenefits for a worker.  
     * @param workerHandle
     * @param healthBenefits array
     * @return an response dto containing the worker and their HealthBenefits
     */
    public HealthBenefitResponse deleteHealthBenefits(ObjectHandle workerHandle, HealthBenefit[] healthBenefits) throws ServiceFailureException, ValidationException {
		CommonValidators.validateObjectHandle(workerHandle);
    	validateHealthBenefits(healthBenefits);
        // Convert to HealthBenefitDtl set
        List<HealthBenefitDtl> dtlSet = BenefitMapper.mapToHealthBenefitDtls(healthBenefits);
        
        // Delete the worker health benefit details
        Worker eiWorker = benefitsComponent.deleteHealthBenefitDtls(workerHandle.getId(), dtlSet);

        // Return the worker HealthBenefits response
        Set<HealthBenefitDtl> responseSet = eiWorker.getHealthBenefitDtls();
        HealthBenefitResponse response = new HealthBenefitResponse(WorkerMapper.mapFromEntityToPojo(eiWorker), BenefitMapper.mapToHealthBenefits(responseSet));
        return response; 
	}

	private static void validateHealthBenefits(HealthBenefit[] benefits) throws ServiceFailureException {
		if (null == benefits) {
			throw new ServiceFailureException("HealthBenefits cannot be null");
		}
		if (0 == benefits.length) {
            throw new ServiceFailureException("HealthBenefits cannot be empty");
		}
		for (HealthBenefit healthBenefit : benefits) {
	        if (null == healthBenefit) {
	            throw new ServiceFailureException("HealthBenefit cannot be null");
	        }
            if (null == healthBenefit.getType()) {
                throw new ServiceFailureException("HealthBenefitType cannot be null");
            }
		}
	}
}