package com.intuit.sb.payroll.services.employer.impl;

import org.springframework.beans.BeanUtils;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.BankAccount;
import com.intuit.sb.payroll.pts.entity.BankAccountState;
import com.intuit.sb.payroll.services.common.BankAccountStatusEnum;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;

/**
 * Mapper to handle conversion between DTO and model
 * BankAccount object
 *
 */
public class BankAccountMapper {

     private static EmployerPTSWorkflowComponent sEmployerPTSWorkflowComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);

    /** Map the given DTO BankAccount object to a model
     * @param bankAccount : DTO BankAccount object
     * @return Model BankAccount object
     * @throws ValidationException
     * @throws ServiceFailureException
     * */
    public static BankAccount mapDtoBankAccountToModel(com.intuit.sb.payroll.services.common.BankAccount bankAccount)
            throws ServiceFailureException, ValidationException {

        BankAccount account = null;

        if (bankAccount != null) {
            ObjectHandle uniqueId = bankAccount.getObjectHandle();

            if (uniqueId == null || uniqueId.getId() == 0L){
                // create new account
                account = new BankAccount();
            }else{
                // existing account - retrieve it
                CommonValidators.validateObjectHandle(uniqueId);
                EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
                BankAccount[] accounts = employerComponent.getBankAccounts();
                for (BankAccount modelAccount : accounts) {
                    if (modelAccount.getId() == bankAccount.getObjectHandle().getId()) {
                        account = modelAccount;
                        break;
                    }
                }
            }

            // since properties between the model and DTO are identical, we can use BeanUtils.copyProperties
            BeanUtils.copyProperties(bankAccount, account);
            // handle any additional property now
            if (bankAccount.getObjectHandle() != null) {
                account.setVersion(bankAccount.getObjectHandle().getVersion()); 
            }
        }

        return account;
    }

    /** Map the given model BankAccount object to a DTO
     * @param Model BankAccount object
     *  @return bankAccount : DTO BankAccount object
     * */
    public static com.intuit.sb.payroll.services.common.BankAccount
            mapModelBankAccountToDto(BankAccount bankAccount) {

        com.intuit.sb.payroll.services.common.BankAccount account = null;

        if (bankAccount != null){
            account = new com.intuit.sb.payroll.services.common.BankAccount();
            BeanUtils.copyProperties(bankAccount, account);
            //  set ObjectHandle for BankAccount
            account.setObjectHandle(new ObjectHandle(bankAccount.getId(), bankAccount.getVersion()));

            // set whether the bank Account can be updated/edited
            account.setLocked(!StatusUpdateHandlerUtil.isBankAccountEditAllowed(bankAccount));
            
            //set weather the BankAccount is already added or not
            account.setAddedToPTS(StatusUpdateHandlerUtil.isEntityAddedToPTS(bankAccount));
            
            //set the Status of the BankAccount(i.e)verified, pending or rejected
            BankAccountState bankAccountState = sEmployerPTSWorkflowComponent.getMostRecentBankAccountState(bankAccount);
            account.setBankAccountStatus(getBankAccountStatusEnum(bankAccountState));
          }
        return account;
    }
    
    private static BankAccountStatusEnum getBankAccountStatusEnum(BankAccountState bankAccountState)
    {
    	if(bankAccountState == null){
    		return BankAccountStatusEnum.BANKACCOUNT_UNDER_VERIFICATION;
    	}
        switch (bankAccountState.getStatus()) {
               case BANKACCOUNT_UNDER_VERIFICATION:
               	return BankAccountStatusEnum.BANKACCOUNT_UNDER_VERIFICATION;
                case BANKACCOUNT_VERIFICATION_FAILED_RECEIVED:
                	return BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED;
                case BANKACCOUNT_VERIFIED_RECEIVED:
                	return BankAccountStatusEnum.BANKACCOUNT_VERIFIED_RECEIVED;
                default:
                	return null;
       }
    }
}
