package com.intuit.sb.payroll.services.employer.impl;

import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.cdm.basetypes.WebSiteAddress;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.impl.PhysicalAddressMapper;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.employer.PrintPreferencesDTO;
import com.intuit.sb.payroll.services.employer.TakeHomeCompanyYTDChoicesDTO;
import com.intuit.sb.payroll.services.employer.WorkerLocation;
import com.intuit.sb.payroll.takehome.entity.PrintPreferences;
import com.intuit.sb.payroll.takehome.entity.TakeHomeCompanyYTDChoices;
import com.intuit.sb.payroll.takehome.entity.TakehomeCompany;
import com.intuit.sb.payroll.takehome.entity.TakehomeCompanyEAO;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Mapper class to handle EmployerInfo Mappings between the DTO/Service layer and the Entity Model.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
public class EmployerInfoMapper {

	/**
	 * Maps the Business Address info from an Entity Employer to a DTO EmployerInfo. Null input yields null output.
	 * 
	 * @param employer
	 * @return
	 * @throws ServiceFailureException 
	 */
	public static EmployerInfo mapEmployerInfo(com.intuit.sb.payroll.entity.model.Employer employer) throws ServiceFailureException {
		if (employer != null) {
			EmployerInfo eInfo = new EmployerInfo();
			eInfo.setObjectHandle(new ObjectHandle(employer.getId(), employer.getVersion()));
			eInfo.setCompanyName(employer.getCompanyName());
			eInfo.setDBAName(employer.getDbaName());
			WebSiteAddress webAddress = employer.getWebAddress();
			if (webAddress != null) {
				eInfo.setWebAddress(webAddress.getURI());
			}

			// Map company address
			eInfo.setCompanyAddress(PhysicalAddressMapper.mapAddress(AddressConvertor.getUSFromGlobal(employer
					.getBusinessAddress(), new USPostalAddressImpl())));

			eInfo.setLegalName(employer.getLegalName());

			eInfo.setLegalAddressSameAsBusinessAddress(employer.isLegalAndBusAddressSame());

			// [as] Legal address always always stands on its own
			USPostalAddress entityLegalAddress = AddressConvertor.getUSFromGlobal(employer.getLegalAddress(),
					new USPostalAddressImpl());
			eInfo.setLegalAddress(PhysicalAddressMapper.mapAddress(entityLegalAddress));

			eInfo.setIndustryType(IndustryTypeMapper.mapIndustryType(employer.getIndustryType()));

			eInfo.setCompanyType(CompanyTypeMapper.mapCompanyType(employer.getLegalStructure()));

			TelephoneNumber legalPhone = employer.getLegalPhone();
			if (legalPhone != null) {
				eInfo.setLegalPhone(legalPhone.getFreeFormNumber());
				eInfo.setExtn(legalPhone.getExtension());
			}
			
			TakehomeCompany takeHomeCompany = getTakeHomeCompany(eInfo);
			if (takeHomeCompany != null) {
				mapTakeHomeEntityToPojo(eInfo, takeHomeCompany);
			}
			
			eInfo.setAssistedStateType(EmployerAssistedImpl.getAssistedState());
			return eInfo;

		} else {
			return null;
		}
	}

	private static TakehomeCompany getTakeHomeCompany(EmployerInfo eInfo) {
		TakehomeCompanyEAO companyEAO = ServiceLocator.getInstance().getService(TakehomeCompanyEAO.JNDI_NAME);
		TakehomeCompany	takeHomeCompany = companyEAO.findByPayrollEmployerId(eInfo.getObjectHandle().getId());
		return takeHomeCompany;
	}

	private static void mapTakeHomeEntityToPojo(EmployerInfo eInfo,
			TakehomeCompany takeHomeCompany) {
		Long takehomeEmployerId = takeHomeCompany.getId();
		Long takehomeEmployerVersion = takeHomeCompany.getVersion();
		ObjectHandle takehomeEmployerObjectHandle = new ObjectHandle(takehomeEmployerId, takehomeEmployerVersion);
		eInfo.setObjectHandle(takehomeEmployerObjectHandle);
		eInfo.setPrintPreferences(mapPrintPreferencesToDTO(takeHomeCompany.getPrintPreferences()));
		eInfo.setTakeHomeCompanyYTDChoices(mapTakeHomeCompanyYTDChoicesToDTO(takeHomeCompany.getYTDPaymentChoices()));
		eInfo.setCompanyLogo(takeHomeCompany.getCompanyLogo());		
	}
	
	public static TakeHomeCompanyYTDChoicesDTO mapTakeHomeCompanyYTDChoicesToDTO(TakeHomeCompanyYTDChoices takeHomeCompanyYTDChoices){
		TakeHomeCompanyYTDChoicesDTO result= null;
		if(takeHomeCompanyYTDChoices != null){
			result = new TakeHomeCompanyYTDChoicesDTO();
			result.setIsTaxPaid(takeHomeCompanyYTDChoices.getIsTaxPaid());
			result.setIsOutsourced(takeHomeCompanyYTDChoices.getIsOutsourced());
			result.setIsFUTAPaid(takeHomeCompanyYTDChoices.getIsFUTAPaid());
			result.setIsPaymentsAccurate(takeHomeCompanyYTDChoices.getIsPaymentsAccurate());
			result.setStatus(takeHomeCompanyYTDChoices.getStatus());
		}
		return result;
	}
	
	public static TakeHomeCompanyYTDChoices mapTakeHomeCompanyYTDChoicesFromDTO(TakeHomeCompanyYTDChoicesDTO takeHomeCompanyYTDChoices){
		TakeHomeCompanyYTDChoices result=null;
		if(takeHomeCompanyYTDChoices != null){
			result = new TakeHomeCompanyYTDChoices();
			result.setIsTaxPaid(takeHomeCompanyYTDChoices.getIsTaxPaid());
			result.setIsOutsourced(takeHomeCompanyYTDChoices.getIsOutsourced());
			result.setIsFUTAPaid(takeHomeCompanyYTDChoices.getIsFUTAPaid());
			result.setIsPaymentsAccurate(takeHomeCompanyYTDChoices.getIsPaymentsAccurate());
			result.setStatus(takeHomeCompanyYTDChoices.getStatus());
		}
		return result;
	}
	
	public static PrintPreferencesDTO mapPrintPreferencesToDTO(PrintPreferences printPreferences) {
		PrintPreferencesDTO result = null;
		if (printPreferences != null) {
			result = new PrintPreferencesDTO();
			result.setXalignment(printPreferences.getXalignment());
			result.setYalignment(printPreferences.getYalignment());
		}
		return result;
	}
	
	public static PrintPreferences mapPrintPreferencesFromDTO(PrintPreferencesDTO printPreferences) {
		PrintPreferences result = null;
		if (printPreferences != null) {
			result = new PrintPreferences();
			result.setXalignment(printPreferences.getXalignment());
			result.setYalignment(printPreferences.getYalignment());
		}
		return result;
	}

	/**
	 * Maps from a DTO EmployerInfo to an Entity Employer. Null input = null output.
	 * 
	 * @param dto
	 * @return
	 * @throws ServiceFailureException 
	 */
	public static com.intuit.sb.payroll.entity.model.Employer mapEmployerInfo(EmployerInfo dto) throws ServiceFailureException {
		if (dto != null) {

			EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
			com.intuit.sb.payroll.entity.model.Employer employer = employerComponent.getEmployer();
			if (employer == null) {
				throw new IllegalStateException("Employer has not been configured.");
			}

			employer.setCompanyName(dto.getCompanyName());
			employer.setDbaName(dto.getDBAName());

			WebSiteAddress webAddress = new WebSiteAddress();
			webAddress.setURI(dto.getWebAddress());
			employer.setWebAddress(webAddress);

			USPostalAddress busAddress = new USPostalAddressImpl();
			PhysicalAddress dtoCompanyAddress = dto.getCompanyAddress();
			PhysicalAddressMapper.mapAddress(dtoCompanyAddress, busAddress);

			employer.setBusinessAddress(AddressConvertor.getGlobalFromUS(busAddress, new GlobalPostalAddress()));

			employer.setLegalName(dto.getLegalName());

			employer.setLegalStructure(CompanyTypeMapper.mapCompanyType(dto.getCompanyType()));

			employer.setIndustryType(IndustryTypeMapper.mapIndustryType(dto.getIndustryType()));

			TelephoneNumber entityLegalPhone = new TelephoneNumber();
			entityLegalPhone.setFreeFormNumber(dto.getLegalPhone());
			entityLegalPhone.setExtension(dto.getExtn());
			employer.setLegalPhone(entityLegalPhone);

			employer.setLegalAndBusAddressSame(dto.isLegalAddressSameAsBusinessAddress());

			// [as] Legal address always always stands on its own
			PhysicalAddress dtoAddress = dto.getLegalAddress();
			if (dtoAddress != null) {
				USPostalAddress entityLegalAddress = new USPostalAddressImpl();
				PhysicalAddressMapper.mapAddress(dtoAddress, entityLegalAddress);
				employer.setLegalAddress(AddressConvertor
						.getGlobalFromUS(entityLegalAddress, new GlobalPostalAddress()));
			}
			return employer;
		} else {
			return null;
		}
	}

	/**
	 * Map WorkerLocation DTO to capability WorkerLocation
	 * @param workerLocation
	 * @return
	 */
	public static com.intuit.sb.payroll.capability.employer.WorkerLocation 
			mapWorkerLocation(WorkerLocation workerLocation) {
		
		if (workerLocation == null) {
			return null;
		}
		
		com.intuit.sb.payroll.capability.employer.WorkerLocation wLocation = 
			new com.intuit.sb.payroll.capability.employer.WorkerLocation();
		wLocation.workerId = workerLocation.getWorkerId();
		wLocation.workLocationId = workerLocation.getWorkLocationId();
		
		return wLocation;
	}
}
