package com.intuit.sb.payroll.services.employer.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.agency.AgencyComponent;
import com.intuit.sb.payroll.capability.agency.Holiday;
import com.intuit.sb.payroll.capability.config.GeminiConfigurationUtility;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent;
import com.intuit.sb.payroll.capability.workerpayments.impl.PayrollScheduleDateUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.security.SecurityHelper;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.common.SaveWorkLocationRequest;
import com.intuit.sb.payroll.services.common.UpdateWorkLocationsResponse;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.common.WorkLocationOperation;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.CalendarEntry;
import com.intuit.sb.payroll.services.employer.CalendarEntryType;
import com.intuit.sb.payroll.services.employer.CompanyLegalTaxFilingInfo;
import com.intuit.sb.payroll.services.employer.CompanyType;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.employer.EmployerService;
import com.intuit.sb.payroll.services.employer.IndustryType;
import com.intuit.sb.payroll.services.employer.LoadApplicationResponse;
import com.intuit.sb.payroll.services.employer.PrimaryPrincipal;
import com.intuit.sb.payroll.services.employer.SaveEmployerInfoRequest;
import com.intuit.sb.payroll.services.employer.SaveEmployerInfoResponse;
import com.intuit.sb.payroll.services.employer.TaxPaymentsChecklist;
import com.intuit.sb.payroll.services.employer.TaxPaymentsChecklistItem;
import com.intuit.sb.payroll.services.employer.TaxPaymentsChecklistItemStatus;
import com.intuit.sb.payroll.services.employer.WorkerLocation;
import com.intuit.sb.payroll.services.worker.GetWorkersFilter;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerService;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;
import com.intuit.sb.payroll.services.workerpayments.PaycheckStatus;
import com.intuit.sb.payroll.services.workerpayments.PaycheckSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService;
import com.intuit.sb.payroll.services.workerpayments.impl.PayrollRunMapper;
import com.intuit.sb.payroll.takehome.entity.TakehomeCompanyEAO;
import com.intuit.sb.payroll.task.TaskService;


@Stateless(name = "EmployerService")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class EmployerServiceBean implements EmployerService {
	
	private static final Logger logger = PayrollLoggerFactory.makeLogger(EmployerServiceBean.class);
	
	@EJB
	EmployerComponent employerComponent;
	
	@EJB
	WorkerComponent workerComponent;
	
	@EJB
	AgencyComponent agencyComponent;
	
	@EJB
	TakehomeCompanyEAO takehomeCompanyEAO;
	
	@EJB
	PayrollScheduleComponent payrollScheduleComponent;
	
	@EJB
	WorkerPaymentsComponent workerPaymentsComponent;
				
	public EmployerInfo getEmployerInfo() throws ServiceFailureException, ValidationException {
		return EmployerInfoImpl.getEmployerInfo();
	}

	/**
	 * {@inheritDoc}
	 */
	public SaveEmployerInfoResponse saveEmployerInfo(SaveEmployerInfoRequest employerInfoRequest) throws ServiceFailureException, ValidationException {
		SaveEmployerInfoResponse employerInfoResponse = new SaveEmployerInfoResponse();
		employerInfoResponse.setWorkLocations(setWorkLocations(employerInfoRequest.getSaveWorkLocationRequests()));
		employerInfoResponse.setEmployerInfo(EmployerInfoImpl.saveEmployerInfo(employerInfoRequest.getEmployerInfo()));
		return employerInfoResponse;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public LoadApplicationResponse loadApplication() throws ValidationException, ServiceFailureException {
		LoadApplicationResponse loadApplicationResponse = new LoadApplicationResponse();

		loadApplicationResponse.setNewCompany(CompanyCreateImpl.createCompany());
		
		EmployerInfo employerInfo = getEmployerInfo();
		
		// First time (after creating the company) we default the companyName to
		// the value they entered in during the sign up process.
		if (loadApplicationResponse.isNewCompany()) {
			try {
				String companyName = SecurityHelper.getCompanyNameFromIAM();
				employerInfo.setCompanyName(companyName);
				employerInfo = EmployerInfoImpl.saveEmployerInfo(employerInfo);
	        }
	        catch (Exception e) {
	        	logger.error(ExceptionUtils.throwableMessageChain(e));
	        	throw new ServiceFailureException(e);
	        }
	        // Proposed Changes for GeminiAdmin story
	        //SecurityHelper.registerEmployerWithGeminiAdmin();
		}

		if (!GeminiConfigurationUtility.isTaxPaymentsEnabled()) {
			employerInfo.setAssistedStateType(AssistedStateType.ASSISTED_NOT_SUPPORTED);
		}
		
		String userName;
		try {
			userName = SecurityHelper.getUserNameromFromIAM();
		} catch (Exception e) {
			logger.error(ExceptionUtils.throwableMessageChain(e));
        	throw new ServiceFailureException(e);
		}
		loadApplicationResponse.setUserName(userName);
		
		loadApplicationResponse.setEmployerInfo(employerInfo);

    	//FIXME use @EJB refs
		WorkerPaymentService workerPaymentService = ServiceLocator.getInstance().getService(WorkerPaymentService.JNDI_NAME);
    	loadApplicationResponse.setPayrollSchedule(workerPaymentService.getPayrollSchedule());
    	
		//FIXME use @EJB refs
    	WorkerService workerService = ServiceLocator.getInstance().getService(WorkerService.JNDI_NAME);
    	loadApplicationResponse.setWorkers(workerService.getWorkers(GetWorkersFilter.ALL));
    	
		//FIXME use @EJB refs
    	TaskService taskService = ServiceLocator.getInstance().getService(TaskService.JNDI_NAME);
    	loadApplicationResponse.setCurrentTasks(taskService.getCurrentTasks());
    	
    	//FIXME the manifest service doesn't work in the EJB layer :( -- [th] Nor should it - it is depends on servlet resources.  Propose that if we want
    	// version information at the EJB tier we use mvn to slam the application.properties into the ejb.jar, then use classloader to get the resource rather
    	// than flex.
//    	ManifestService manifestService = new ManifestService();
//    	loadApplicationResponse.setBuildTime(manifestService.getBuildTime());
//    	loadApplicationResponse.setVersion(manifestService.getVersion());
    	
    	// Send back three year's worth of calendar entries to get things going
    	Calendar today = new GregorianCalendar();
    	loadApplicationResponse.setCalendarEntryYears(new CalendarEntry[3][]);
    	CalendarEntry[] calendarEntries = getCalendarEntries(today.get(Calendar.YEAR) - 1);
    	loadApplicationResponse.getCalendarEntryYears()[0] = calendarEntries;
    	calendarEntries = getCalendarEntries(today.get(Calendar.YEAR));
    	loadApplicationResponse.getCalendarEntryYears()[1] = calendarEntries;
    	calendarEntries = getCalendarEntries(today.get(Calendar.YEAR) + 1);
    	loadApplicationResponse.getCalendarEntryYears()[2] = calendarEntries;
    	
		return loadApplicationResponse;
	}

	/**
	 * {@inheritDoc}
	 */
	public void saveCompanyInfo(EmployerInfo info) throws ServiceFailureException, ValidationException {
		EmployerInfoImpl.saveEmployerInfo(info);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.composite.entityinterface.worker.IWorkerService#getWorkLocations()
	 */
	public WorkLocation[] getWorkLocations() throws ServiceFailureException, ValidationException {
		return WorkLocationsImpl.getWorkLocations();
	}

	private WorkLocation[] setWorkLocations(SaveWorkLocationRequest[] workLocationRequests) throws ServiceFailureException,	ValidationException {
		WorkLocation[] finalWorkLocations = null;
		if (workLocationRequests != null) {
			List<WorkLocation> addWorkLocations = new ArrayList<WorkLocation>();
			List<WorkLocation> updateWorkLocations = new ArrayList<WorkLocation>();
			List<WorkLocation> deleteWorkLocations = new ArrayList<WorkLocation>();
			for (SaveWorkLocationRequest workLocationRequest : workLocationRequests) {
				int opcode = workLocationRequest.getWorkLocationOperation().ordinal();
				if (opcode == WorkLocationOperation.ADD.ordinal()) {
					addWorkLocations.add(workLocationRequest.getWorkLocation());
				}
				else if (opcode == WorkLocationOperation.UPDATE.ordinal()) {
					updateWorkLocations.add(workLocationRequest.getWorkLocation());
				}
				else if (opcode == WorkLocationOperation.DELETE.ordinal()) {
					deleteWorkLocations.add(workLocationRequest.getWorkLocation());
				}
			}
			if (addWorkLocations.size() != 0) {
				finalWorkLocations = addWorkLocations(addWorkLocations.toArray(new WorkLocation[0])).getWorkLocations();
			}
			if (updateWorkLocations.size() != 0) {
				finalWorkLocations = updateWorkLocations(updateWorkLocations.toArray(new WorkLocation[0])).getWorkLocations();
			}
			if (deleteWorkLocations.size() != 0) {
				finalWorkLocations = deleteWorkLocations(deleteWorkLocations.toArray(new WorkLocation[0]));
			}
		}
		if (finalWorkLocations == null) {
			finalWorkLocations = getWorkLocations();
		}
		return finalWorkLocations;
	}

	/**
	 * {@inheritDoc}
	 */
	public UpdateWorkLocationsResponse addWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException,
			ValidationException {
		UpdateWorkLocationsResponse updateWorkLocationsResponse = new UpdateWorkLocationsResponse();
		
		updateWorkLocationsResponse.setWorkLocations(WorkLocationsImpl.addWorkLocations(workLocations));
		
		return updateWorkLocationsResponse;
	}

	/**
	 * {@inheritDoc}
	 */
	public UpdateWorkLocationsResponse updateWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException,
			ValidationException {
		UpdateWorkLocationsResponse updateWorkLocationsResponse = new UpdateWorkLocationsResponse();
		
		updateWorkLocationsResponse.setWorkLocations(WorkLocationsImpl.updateWorkLocations(workLocations));
		
		List<com.intuit.sb.payroll.entity.model.Worker> workerEntityList = new ArrayList<com.intuit.sb.payroll.entity.model.Worker>();
		
		for(WorkLocation workLocation : workLocations) {
			List<com.intuit.sb.payroll.entity.model.Worker> workerList = workerComponent.findWorkersByWorkLocation(workLocation.getObjectHandle().getId());
			if(workerList.size() > 0) {
				workerEntityList.addAll(workerList);
			}
		}
		
		updateWorkLocationsResponse.setWorkers(WorkerMapper.mapFromEntitiesToPojos(workerEntityList));
		
		return updateWorkLocationsResponse;
	}

	/**
	 * {@inheritDoc}
	 */
	public WorkLocation[] deleteWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException,
			ValidationException {
		return WorkLocationsImpl.deleteWorkLocations(workLocations);
	}

	/**
	 * (@ineritDoc)
	 */
	public Worker[] assignWorkLocationsToWorkers(WorkerLocation[] workerLocations) throws WorkflowException,
			ServiceFailureException, ValidationException {
		return WorkLocationsImpl.assignWorkLocationsToWorkers(workerLocations);
	}

	/**
	 * {@inheritDoc}
	 */
	public EmployerContact[] addEmployerContacts(EmployerContact[] contacts) throws ServiceFailureException,
			ValidationException {
		return EmployerContactsImpl.addEmployerContacts(contacts);
	}

	/**
	 * {@inheritDoc}
	 */
	public void deleteEmployerContacts(EmployerContact[] contacts) throws ServiceFailureException,
			ValidationException {
		EmployerContactsImpl.deleteEmployerContacts(contacts);

	}

	/**
	 * {@inheritDoc}
	 */
	public EmployerContact[] updateEmployerContacts(EmployerContact[] contacts) throws ServiceFailureException,
			ValidationException {
		return EmployerContactsImpl.updateEmployerContacts(contacts);
	}

	/**
	 * {@inheritDoc}
	 */
	public EmployerContact findEmployerContact(Long id) throws ServiceFailureException, ValidationException {
		return EmployerContactsImpl.getEmployerContact(id);
	}

	/**
	 * {@inheritDoc}
	 */
	public EmployerContact[] getEmployerContacts() throws ServiceFailureException, ValidationException {
		return EmployerContactsImpl.getEmployerContacts();
	}

	/**
	 * {@inheritDoc}
	 */
	public void deleteAllEmployerContacts() throws ServiceFailureException, ValidationException {
		EmployerContactsImpl.deleteAllEmployerContacts();
	}

	public CompanyLegalTaxFilingInfo getCompanyLegalTaxFilingInfo() throws ServiceFailureException, ValidationException {
		return CompanyLegalTaxFilingInfoImpl.getCompanyInfo();
	}

	public void saveCompanyLegalTaxFilingInfo(CompanyLegalTaxFilingInfo info) throws ServiceFailureException,
			ValidationException {
		CompanyLegalTaxFilingInfoImpl.saveCompanyInfo(info);
	}

	// TODO - [rj] - now only Deep mocking implementation
	// change to actual implementation later
	public CompanyType[] getCompanyTypes() {
		return CompanyType.values();
	}

	// TODO - [rj] - now only Deep mocking implementation
	// change to actual implementation later
	public IndustryType[] getIndustryTypes() {
		return IndustryType.getIndustryTypeList();
	}

	/**
	 * {@inheritDoc}
	 */
	public void updateAssistedState(AssistedStateType assistedStateType) throws ServiceFailureException,
			ValidationException {
		EmployerAssistedImpl.updateAssistedState(assistedStateType);
	}

	/**
	 * {@inheritDoc}
	 */
	public TaxPaymentsChecklistItem[] getTaxPaymentsChecklist() throws ServiceFailureException {
		TaxPaymentsChecklist checklist = new TaxPaymentsChecklist();

		// TODO: Statuses of checklist item to be based of Employer's Assited state
		checklist.finishEmployeeSetup.setChecklistItemStatus(TaxPaymentsChecklistItemStatus.NOT_SET);
		checklist.enterTaxPaymentsInfo.setChecklistItemStatus(TaxPaymentsChecklistItemStatus.NOT_STARTED);
		checklist.verifyBankAccount.setChecklistItemStatus(TaxPaymentsChecklistItemStatus.NOT_SET);
		checklist.completeReturnForms.setChecklistItemStatus(TaxPaymentsChecklistItemStatus.NOT_SET);

		return checklist.toArray(new TaxPaymentsChecklistItem[checklist.size()]);
	}

	/**
	 * {@inheritDoc}
	 */
	public BankAccount[] addBankAccounts(BankAccount[] bankAccounts)
			throws ServiceFailureException, ValidationException {
		
		return EmployerBankImpl.addBankAccounts(bankAccounts);
	}

	/**
	 * {@inheritDoc}
	 */
	public BankAccount[] getBankAccounts()
			throws ServiceFailureException, ValidationException {
		return EmployerBankImpl.getBankAccounts();
	}

	/**
	 * {@inheritDoc}
	 */
	public BankAccount[] updateBankAccounts(BankAccount[] bankAccounts)
			throws ServiceFailureException, ValidationException {
		return EmployerBankImpl.updateBankAccounts(bankAccounts);
	}

	/**
	 * {@inheritDoc}
	 * @return 
	 */
	public void sendCompanyInfoToPTS() throws ServiceFailureException, ValidationException {
		EmployerAssistedImpl.sendCompanyInfoToPTS();
	}

	/**
	 * {@inheritDoc}
	 */
	public PrimaryPrincipal getPrimaryPrincipal() throws ServiceFailureException {
		return EmployerContactsImpl.getPrimaryPrincipal();
	}

	/**
	 * {@inheritDoc}
	 */
	public PrimaryPrincipal addPrimaryPrincipal(PrimaryPrincipal primaryPrincipal) throws ServiceFailureException,
			ValidationException {
		return EmployerContactsImpl.addPrimaryPrincipal(primaryPrincipal); 
	}

	/**
	 * {@inheritDoc}
	 */
	public PrimaryPrincipal updatePrimaryPrincipal(PrimaryPrincipal primaryPrincipal)
			throws ServiceFailureException, ValidationException {
		return EmployerContactsImpl.updatePrimaryPrincipal(primaryPrincipal);
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean allowDataSubmissionToPTS() {
		return EmployerAssistedImpl.allowDataSubmissionToPTS();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public CalendarEntry[] getCalendarEntries(Integer year) throws ServiceFailureException, ValidationException
	{
		ArrayList<CalendarEntry> result = new ArrayList<CalendarEntry>();
		
		// Get holidays for the current year.
		DateRange dateRange = new DateRange(DateUtil.createDate(year, 0, 1) , DateUtil.createDate(year, 11, 31));
		List<Holiday> holidays = agencyComponent.getHolidaysForPeriod(dateRange);
		
		// Convert the holidays for the given year into CalendarEntries for the app
		mapHolidaysToCalendarEntries(result, holidays);
		
		AssistedStateType employerAssistedType = getEmployerInfo().getAssistedStateType();
		if(employerAssistedType == AssistedStateType.NOT_ASSISTED) {
			addTaxDueDates(result, year);
		}		
		
		// Get the default payroll schedule in order to find the past and future payroll runs.
		PayrollSchedule currentPaySchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
		
		if (currentPaySchedule != null) {
			// Find future payroll runs
			List<PayrollRunInfo> futurePayRollRuns = 
					payrollScheduleComponent.getUpcomingScheduledPayrollRunInfosByYear(currentPaySchedule, DateUtil.createDate(year, 0, 1));
			
			// Find past payroll runs
			List<com.intuit.sb.payroll.entity.model.PayrollRun> payrollRuns = null;
			List<PayrollRunSummary> pastPayRollSummaries = new ArrayList<PayrollRunSummary>();
			payrollRuns = workerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.SCHEDULED, year);
			payrollRuns.addAll(workerPaymentsComponent.findRecentPayrollRunsForYear(PayrollRunTypeEnum.ADHOC, year));
			if (payrollRuns != null) {
				for (com.intuit.sb.payroll.entity.model.PayrollRun payrollRun : payrollRuns) {
					PayrollRunSummary summary = PayrollRunMapper.mapPayrollRunToPayrollRunSummary(payrollRun);
					pastPayRollSummaries.add(summary);
				}
			}
			
			// Convert collections of PayRollInfo into the necessary collection of CalendarEntries
			mapPastPayRollRunSummariesToCalendarEntries(result, pastPayRollSummaries);
			mapFuturePayRollRunInfoToCalendarEntries(result, futurePayRollRuns);
		}
		
		return result.toArray(new CalendarEntry[0]);
	}
	
	private void addTaxDueDates(Collection<CalendarEntry> result, Integer year) {
		CalendarEntry taxEntry;
		Calendar date;
		
		date = DateUtil.createCalendar(year, Calendar.JANUARY, 31);
		adjustForHolidaysAndWeekend(date);
		taxEntry = new CalendarEntry(date.get(Calendar.MONTH) + 1, date.get(Calendar.DATE), year, "Annual and quarterly forms due. W-2 due to employee", CalendarEntryType.TAXES, 0, null);
		result.add(taxEntry);
		
		date = DateUtil.createCalendar(year, Calendar.MARCH, 1);
		adjustForHolidaysAndWeekend(date);
		taxEntry = new CalendarEntry(date.get(Calendar.MONTH) + 1, date.get(Calendar.DATE), year, "W-2 due for paper filers", CalendarEntryType.TAXES, 0, null);
		result.add(taxEntry);
		
		date = DateUtil.createCalendar(year, Calendar.MARCH, 31);
		adjustForHolidaysAndWeekend(date);
		taxEntry = new CalendarEntry(date.get(Calendar.MONTH) + 1, date.get(Calendar.DATE), year, "W-2 due for electronic filers", CalendarEntryType.TAXES, 0, null);
		result.add(taxEntry);
		
		date = DateUtil.createCalendar(year, Calendar.APRIL, 30);
		adjustForHolidaysAndWeekend(date);
		taxEntry = new CalendarEntry(date.get(Calendar.MONTH) + 1, date.get(Calendar.DATE), year, "Quarterly forms due", CalendarEntryType.TAXES, 0, null);
		result.add(taxEntry);
		
		date = DateUtil.createCalendar(year, Calendar.JULY, 31);
		adjustForHolidaysAndWeekend(date);
		taxEntry = new CalendarEntry(date.get(Calendar.MONTH) + 1, date.get(Calendar.DATE), year, "Quarterly forms due", CalendarEntryType.TAXES, 0, null);
		result.add(taxEntry);
		
		date = DateUtil.createCalendar(year, Calendar.OCTOBER, 31);
		adjustForHolidaysAndWeekend(date);
		taxEntry = new CalendarEntry(date.get(Calendar.MONTH) + 1, date.get(Calendar.DATE), year, "Quarterly forms due", CalendarEntryType.TAXES, 0, null);
		result.add(taxEntry);
	}
	
	private Calendar adjustForHolidaysAndWeekend(Calendar date) {
		while (PayrollScheduleDateUtils.isHoliday(date) || DateUtil.isWeekend(date)) {
			date.add(Calendar.DAY_OF_MONTH, 1);			
		}
		return date;
	}

	private void mapPastPayRollRunSummariesToCalendarEntries(Collection<CalendarEntry> result, 
			List<PayrollRunSummary> summaries)
	{
		for(PayrollRunSummary paySummary:summaries)
		{
			Calendar payRollRunDate = DateUtil.dateToCalendar(paySummary.getPayrollPeriod().getPayrollRunDate());

			// Figure out if this is distributed or not for the description
			String description = "View payroll";
			PaycheckSummary[] paychecks = paySummary.getPaycheckSummaries();
			for (PaycheckSummary paycheck: paychecks){
				if (!PaycheckStatus.COMPLETE.equals(paycheck.getPaycheckStatus())
						&& !PaycheckStatus.VOIDED.equals(paycheck.getPaycheckStatus())) {
					description = "Distribute paychecks";
					break;
				}
			}
			
			CalendarEntry payRollRunEntry = new CalendarEntry(payRollRunDate.get(Calendar.MONTH) + 1, payRollRunDate.get(Calendar.DATE), 
					payRollRunDate.get(Calendar.YEAR), description, CalendarEntryType.PAYROLL_RUN, paySummary.getPayrollRunId(), null);
			
			result.add(payRollRunEntry);
		}
	}
	
	private void mapFuturePayRollRunInfoToCalendarEntries(Collection<CalendarEntry> result, List<PayrollRunInfo> dates)
	{
		for(PayrollRunInfo payInfo:dates)
		{
			Calendar payRollRunDate = DateUtil.dateToCalendar(payInfo.getPayrollRunDate());
			
			CalendarEntry payRollRunEntry = new CalendarEntry(payRollRunDate.get(Calendar.MONTH) + 1, payRollRunDate.get(Calendar.DATE), 
					payRollRunDate.get(Calendar.YEAR), "Scheduled payroll", CalendarEntryType.PAYROLL_RUN, 0, PayrollRunMapper.createPayrollRunPeriodFromPayrollRunInfo(payInfo));
			
			result.add(payRollRunEntry);
		}
	}
	
	private void mapHolidaysToCalendarEntries(Collection<CalendarEntry> result, List<Holiday> holidays)
	{
		Calendar date;
		Holiday holiday;
		CalendarEntry entry;
		
		for(int i = 0; i < holidays.size(); ++i)
		{
			holiday = holidays.get(i);
			date = DateUtil.dateToCalendar(holiday.getDate());
			entry = new CalendarEntry(date.get(Calendar.MONTH) + 1, date.get(Calendar.DATE), date.get(Calendar.YEAR), holiday.getName(),
					CalendarEntryType.HOLIDAY, 0, null);
			result.add(entry);
		}
			
	}
}
