package com.intuit.sb.payroll.services.employer.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.common.impl.WorkLocationMapper;
import com.intuit.sb.payroll.services.employer.WorkerLocation;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;

/**
 * Class that implements the getWorkLocations service interface. It is expected
 * that this class will be directly invoked by the implementor of IWorkerService
 * to realize the getWorkLocations API.
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
class WorkLocationsImpl {
	private WorkLocationsImpl() {
	}

	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(WorkLocationsImpl.class);
	private static EmployerComponent employerComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);


	static WorkLocation[] getWorkLocations() throws ServiceFailureException {
		try {

			List<com.intuit.sb.payroll.entity.model.WorkLocation> entityWorkLocations = employerComponent.findAllWorkLocations();

			List<WorkLocation> resultWorkLocations = new ArrayList<WorkLocation>();
			//Map from Entity
			for (com.intuit.sb.payroll.entity.model.WorkLocation workLocation : entityWorkLocations) {
				WorkLocation dto = WorkLocationMapper.mapWorkLocation(workLocation);
				resultWorkLocations.add(dto);
			}
			
			WorkLocation[] locations = new WorkLocation[resultWorkLocations.size()];
			return resultWorkLocations.toArray(locations);
		} catch (Exception e) {
			sLogger.error("Error retrieving Work Locations", e);
			throw new ServiceFailureException(e);
		}
	}

	/**
	 * Adds the passed in worklocations into the system returns the aggregate of
	 * the worklocations available in the system afetr adding the new ones.
	 * 
	 * @param workLocations
	 * @return
	 * @throws ServiceFailureException
	 * @throws ValidationException 
	 */
	static WorkLocation[] addWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException, ValidationException {
		validateWorkLocations(workLocations);

		List<com.intuit.sb.payroll.entity.model.WorkLocation> mappedWorkLocations = new ArrayList<com.intuit.sb.payroll.entity.model.WorkLocation>();

		//Map to Entity
		try {
			for (WorkLocation workLocation : workLocations) {
				com.intuit.sb.payroll.entity.model.WorkLocation entityLocation = WorkLocationMapper.mapWorkLocation(workLocation);
				mappedWorkLocations.add(entityLocation);
			}
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}


		List<com.intuit.sb.payroll.entity.model.WorkLocation> allWorkLocations;
		allWorkLocations = employerComponent.addWorkLocations(mappedWorkLocations);
		
		List<WorkLocation> resultWorkLocations = new ArrayList<WorkLocation>();
		//Map from Entity
		for (com.intuit.sb.payroll.entity.model.WorkLocation entityLocation : allWorkLocations) {
			WorkLocation workLocation = WorkLocationMapper.mapWorkLocation(entityLocation);
			resultWorkLocations.add(workLocation);
		}
		
		WorkLocation[] locations = new WorkLocation[resultWorkLocations.size()];
		return resultWorkLocations.toArray(locations);
	}

	/**
	 * Updates the work locations passed in the system. If any of the work
	 * locations can not be found an ValidationException exception is thrown and
	 * none of the WorkLocations is updated.
	 * 
	 * @param workLocations
	 * @return
	 * @throws ServiceFailureException
	 */
	static WorkLocation[] updateWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException,
			ValidationException {
		validateWorkLocations(workLocations);

		List<com.intuit.sb.payroll.entity.model.WorkLocation> mappedWorkLocations = new ArrayList<com.intuit.sb.payroll.entity.model.WorkLocation>();

		//Map to Entity
		try {
			for (WorkLocation workLocation : workLocations) {
				com.intuit.sb.payroll.entity.model.WorkLocation entityLocation = WorkLocationMapper.mapWorkLocation(workLocation);
				mappedWorkLocations.add(entityLocation);
			}
		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}

		List<com.intuit.sb.payroll.entity.model.WorkLocation> allWorkLocations;
		allWorkLocations = employerComponent.updateWorkLocations(mappedWorkLocations);
		
		List<WorkLocation> resultWorkLocations = new ArrayList<WorkLocation>();
		//Map from Entity
		for (com.intuit.sb.payroll.entity.model.WorkLocation entityWL : allWorkLocations) {
			WorkLocation dto = WorkLocationMapper.mapWorkLocation(entityWL);
			resultWorkLocations.add(dto);
		}
		
		WorkLocation[] locations = new WorkLocation[resultWorkLocations.size()];
		return resultWorkLocations.toArray(locations);
	}

	/**
	 * deletes the WorkLocation collection passed in. If any of the
	 * worklocations are not found a ValidationException is thrown and none of
	 * the workLOcations is deleted from the system
	 * 
	 * @param workLocations
	 * @return
	 * @throws ServiceFailureException
	 * @throws ValidationException
	 */
	static WorkLocation[] deleteWorkLocations(WorkLocation[] workLocations) throws ServiceFailureException, ValidationException {
		validateWorkLocations(workLocations);

		List<com.intuit.sb.payroll.entity.model.WorkLocation> mappedWorkLocations = new ArrayList<com.intuit.sb.payroll.entity.model.WorkLocation>();
		List<com.intuit.sb.payroll.entity.model.WorkLocation> allWorkLocations;

		for (WorkLocation workLocation : workLocations) {
			if (null == workLocation.getObjectHandle()) {
				throw new ValidationException("Cannot delete an unsaved WorkLocation");
			}
			com.intuit.sb.payroll.entity.model.WorkLocation entityLocation = WorkLocationMapper.mapWorkLocation(workLocation);
			mappedWorkLocations.add(entityLocation);
		}
		
		allWorkLocations = employerComponent.deleteWorkLocations(mappedWorkLocations);
		
		List<WorkLocation> resultWorkLocations = new ArrayList<WorkLocation>();
		//Map from Entity
		for (com.intuit.sb.payroll.entity.model.WorkLocation entityWL : allWorkLocations) {
			WorkLocation dto = WorkLocationMapper.mapWorkLocation(entityWL);
			resultWorkLocations.add(dto);
		}
		
		WorkLocation[] locations = new WorkLocation[resultWorkLocations.size()];
		return resultWorkLocations.toArray(locations);
	}

	static Worker[] assignWorkLocationsToWorkers(WorkerLocation[] workerLocations)
		throws WorkflowException, ValidationException, ServiceFailureException {

	    if (null == workerLocations) {
            throw new ValidationException("workerLocations HashMap cannot be null");
	    }
	    int i = 0;
	    com.intuit.sb.payroll.capability.employer.WorkerLocation[] wLocations = 
	    	new com.intuit.sb.payroll.capability.employer.WorkerLocation[workerLocations.length];
	    for (WorkerLocation wLocation : workerLocations) {
	    	wLocations[i++] = EmployerInfoMapper.mapWorkerLocation(wLocation);
	    }
	    return WorkerMapper.mapFromEntitiesToPojos(employerComponent.assignWorkLocationsToWorkers(wLocations));
	}
	
	private static void validateWorkLocations(WorkLocation[] workLocations) throws ValidationException {
		if (null == workLocations) {
			throw new ValidationException("WorkLocations array cannot be null");
		}
		
		if (workLocations.length == 0) {
			throw new ValidationException("WorkLocations array cannot be empty");
		}
		
		for (WorkLocation workLocation : workLocations) {
			if (null == workLocation) {
				throw new ValidationException("WorkLocations can't be null");
			}
		}
		
	}
}