package com.intuit.sb.payroll.services.taxes.impl;

import java.util.ArrayList;
import java.util.List;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent;
import com.intuit.sb.payroll.common.PayrollResourceManager;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.W4Detail;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;
import com.intuit.sb.payroll.services.taxes.FormType;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;

/**
 * Holds the functions for the mapping between the Entity and the POJO withholding information
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 */
public class WithholdingInformationEIMapper {
	protected static final PayrollResourceManager sResourceManager = PayrollResourceManager.getInstance();
	private static TaxWithholdingsComponent taxWithholdingsComponent = ServiceLocator.getInstance().getService(
			TaxWithholdingsComponent.JNDI_NAME);

	/**
	 * Map an array of WorkerComplianceDocDtl entities to an array of WithholdingInformation DTOs
	 * 
	 * @param worker
	 * @param workerComplianceDtls
	 * @param taxId
	 * @return array of WithholdingInformation
	 * @throws ServiceFailureException
	 */
	public static WithholdingInformation[] mapFromEntitiesToPojos(Worker worker,
			WorkerComplianceDocDtl[] workerComplianceDtls) throws ServiceFailureException {
		List<WithholdingInformation> withholdingInfos = new ArrayList<WithholdingInformation>();
		for (WorkerComplianceDocDtl workerComplianceDocDtl : workerComplianceDtls) {
			WithholdingInformation withholdingInfo = mapFromEntityToPojo(worker, workerComplianceDocDtl);
			//We only map docs we care about so null is valide.
			if (withholdingInfo != null) {
				withholdingInfos.add(withholdingInfo);
			}
		}
		return withholdingInfos.toArray(new WithholdingInformation[withholdingInfos.size()]);
	}

	/**
	 * Map from WorkerComplianceDocDtl entity to WithholdingInformation DTO.
	 * 
	 * @param worker
	 * @param workerCompDtl
	 * @param taxId
	 * @return WithholdingInformation
	 * @throws ServiceFailureException
	 */
	public static WithholdingInformation mapFromEntityToPojo(Worker worker, WorkerComplianceDocDtl workerCompDtl) throws ServiceFailureException {

		WithholdingInformation withholdingInformation = new WithholdingInformation();

		if (workerCompDtl instanceof W4Detail) {
			W4Detail newW4Dtl = (W4Detail) workerCompDtl;
			withholdingInformation.setAllowances(newW4Dtl.getNoOfAllowances());
			withholdingInformation.setExempt(newW4Dtl.isTaxExempt());
			if (newW4Dtl.getAdditionalAmount() != null) {
				withholdingInformation.setExtraWithholding(newW4Dtl.getAdditionalAmount().getAmount());
			}
			withholdingInformation.setFormType(FormType.US_W4);
			withholdingInformation.setObjectHandle(ObjectHandleMapper.mapToObjectHandle(newW4Dtl));
			withholdingInformation.setWithholdingStatus(WithholdingStatusMapper
					.mapW4WithholdingStatusEnumToWithholdingStatus(newW4Dtl.getWithholdingStatus()));
		} else if (workerCompDtl instanceof DE4Detail) {
			DE4Detail newDE4Dtl = (DE4Detail) workerCompDtl;
			withholdingInformation.setAllowances(newDE4Dtl.getNoOfAllowances());
			withholdingInformation.setDeductions(newDE4Dtl.getNoOfDeductions());
			withholdingInformation.setExempt(newDE4Dtl.isTaxExempt());
			if (newDE4Dtl.getAdditionalAmount() != null) {
				withholdingInformation.setExtraWithholding(newDE4Dtl.getAdditionalAmount().getAmount());
			}
			withholdingInformation.setFormType(FormType.CA_DE4);
			withholdingInformation.setObjectHandle(ObjectHandleMapper.mapToObjectHandle(newDE4Dtl));
			withholdingInformation.setWithholdingStatus(WithholdingStatusMapper
					.mapDE4WithholdingStatusEnumToWithholdingStatus(newDE4Dtl.getWithholdingStatus()));
		} else {
			return null;
		}

		return withholdingInformation;
	}
	
	public static WorkerComplianceDocDtl mapFromPojoToEntity(WithholdingInformation withholdingInfo,
			WorkerComplianceDocDtl existingComplianceDoc) {

		mapFromPojoToEntityImpl(withholdingInfo, existingComplianceDoc);
		return existingComplianceDoc;
	}
	

	/**
	 * Set a worker's collection of WithholdingInformation.
	 * 
	 * For each of the supplied withholdingInformations, either update the respective withholdingData for the worker (if
	 * one exists) or create a new withholdingData in the worker's compliance data collection.
	 * 
	 * @param worker the worker who's withholding data is being set.
	 * @param withholdingInformation the withholding information to set into withholding data.
	 * @param complianceDtl an existing record.
	 * @throws ServiceFailureException
	 */
	static WorkerComplianceDocDtl mapNewPojoToEntity(WithholdingInformation withholdingInformation)
			throws ServiceFailureException, ValidationException {
		
		WorkerComplianceDocDtl workCompDtl = null;
		if (null == withholdingInformation.getFormType()) {
			throw new ValidationException("FormType value must be set");
		}
		if (withholdingInformation.getFormType().equals(FormType.US_W4)) {
			workCompDtl = taxWithholdingsComponent.createWorkerComplianceDocDtl(ComplianceDocIdEnum.W4IRS);
		} else if (withholdingInformation.getFormType().equals(FormType.CA_DE4)) {
			workCompDtl = taxWithholdingsComponent.createWorkerComplianceDocDtl(ComplianceDocIdEnum.DE4);
		} else {
			throw new ValidationException("FormType value must be set");
		}

		mapFromPojoToEntityImpl(withholdingInformation, workCompDtl);

		return workCompDtl;
	}

	private static void mapFromPojoToEntityImpl(WithholdingInformation withholdingInformation,
			WorkerComplianceDocDtl complianceDtl) {
		complianceDtl.setDerived(false);

		if (complianceDtl instanceof W4Detail) {
			W4Detail w4d = (W4Detail) complianceDtl;
			w4d.setTaxExempt(withholdingInformation.isExempt());
			w4d.setNoOfAllowances(withholdingInformation.getAllowances());
			if (null != withholdingInformation.getExtraWithholding()) {
				Money money = new Money();
				money.setAmount(withholdingInformation.getExtraWithholding());
				w4d.setAdditionalAmount(money);
			}
			w4d.setWithholdingStatus(WithholdingStatusMapper.mapWithholdingStatusToW4Detail(withholdingInformation
					.getWithholdingStatus()));
		} else if (complianceDtl instanceof DE4Detail) {
			DE4Detail de4 = (DE4Detail) complianceDtl;
			de4.setTaxExempt(withholdingInformation.isExempt());
			de4.setNoOfAllowances(withholdingInformation.getAllowances());
			de4.setNoOfDeductions(withholdingInformation.getDeductions());
			if (null != withholdingInformation.getExtraWithholding()) {
				Money money = new Money();
				money.setAmount(withholdingInformation.getExtraWithholding());
				de4.setAdditionalAmount(money);
			}
			de4.setWithholdingStatus(WithholdingStatusMapper.mapWithholdingStatusToDE4Detail(withholdingInformation
					.getWithholdingStatus()));
		}
	}

}
