package com.intuit.sb.payroll.services.taxpayments;

import java.util.List;
import java.util.Map;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.PayrollRun;

/**
 * This interface contains apis for managing or retrieving 
 * Tax specific information
 * <p>
 *
 * $Author: dkulkarni $
 * $Revision: #1 $
 * $DateTime: 2009/06/15 14:09:31 $
 */
@Remote
public interface TaxPaymentService
{
	@SuppressWarnings("nls")
    public static final String JNDI_NAME = "TaxPaymentService/remote";
	
	/**
	 * Gets a list of {@link TaxPayments TaxPayments} the user has made.
	 * @param paymentPeriod
	 * PaymentPeriod consists of the start and end dates between which the API will retrieve <br>
	 * TaxPayments.  The search will include ALL TaxPayments:
	 * <ul><li> BETWEEN the start and end date</li>
	 * 	   <li> ON the start date but <b>NOT ON THE END DATE </b></li></ul>
	 * @return Array of TaxPayments for the given period
	 * @throws ValidationException
	 */
	public ToDateTaxPayments[] getToDateTaxPayments(PaymentPeriod paymentPeriod) throws ValidationException;
	
	/**
	 * Update or add the new list of {@link TaxPayments TaxPayments} for the employer
	 * Returns the same updated list
	 * @param taxPayments
	 * @return the updated list
	 * @throws ValidationException
	 * @throws ServiceFailureException 
	 */
	public ToDateTaxPayments[] updateTaxPayments(ToDateTaxPayments[] taxPayments) throws ValidationException, ServiceFailureException;
	
	/**
	 * Remove all the {@link TaxPayments TaxPayments} for the given date period
	 * @param paymentPeriod
	 * @throws ValidationException
	 */
	public void removeTaxPayments(PaymentPeriod paymentPeriod) throws ValidationException, ServiceFailureException ;
	
	/**
	 * Remove the {@link TaxPayments TaxPayments} for the given Payment identifier
	 * @param paymentPeriod
	 * @throws ValidationException
	 */
	public void removeTaxPayment(Long paymentId) throws ValidationException, ServiceFailureException ;
	
	/**
	 * Get Liabilities from {@link PayrollRun payrollRun} for given period
	 * A {@link List} is being returned since {@link Map} cannot be accepted by Flex
	 * @param payment {@link PaymentPeriod paymentperiod} object to check for liabilities<br>
	 * PaymentPeriod consists of the start and end dates between which the API will retrieve <br>
	 * liabilities.  The search will include ALL liabilites:
	 * <ul><li> BETWEEN the start and end date</li>
	 * 	   <li> ON the start date but <b>NOT ON THE END DATE </b></li></ul>
	 * @return {@link List list} containing 
	 * 
	 * @throws ValidationException
	 * @throws ServiceFailureException 
	 */
	public List<PaymentTypeLiability> getLiabilitiesFromExistingPayrollRuns(PaymentPeriod paymentPeriod) throws ValidationException, ServiceFailureException;
	
	/**
	 * Retrieve {@link List List} of {@link FutaPaymentsAndLiabilities FutaPaymentsAndLiabilities} for previous quarters of this 
	 * financial year
	 * @return {@link List List} of {@link FutaPaymentsAndLiabilities FutaPaymentsAndLiabilities} 
	 */
	public List<FutaPaymentsAndLiabilities> getFutaPaymentsAndLiabilitiesYtd() throws ValidationException;
	
	/**
	 * Remove all Futa Payments and Liabilities for this Financial year
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 */
	public void removeFutaPaymentsAndLiabilitiesYtd() throws ValidationException, ServiceFailureException;
	
	/**
	 * Update data for Futa Payments and liabilities
	 * @param paymentsAndLiabilities
	 * @throws ValidationException 
	 * @throws ServiceFailureException 
	 */
	public List<FutaPaymentsAndLiabilities> updateFutaPaymentsAndLiabilitiesYtd(List<FutaPaymentsAndLiabilities> paymentsAndLiabilities) throws ValidationException, ServiceFailureException;

	/**
	 * Method returns a list of liability balances based on the payments made 
	 * @return a list of balance liabilities
	 * @throws ServiceFailureException
	 */
	public LiabilityBalance[] getHistoricalLiabilityBalances()
			throws ServiceFailureException;

	/**
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#getHistoricalLiabilityBalances()
	 */
	public LiabilityBalance[] getTaxPaymentBalances() throws ServiceFailureException;

	/**
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.taxpayments.TaxPaymentService#getHistoricalLiabilityBalances()
	 */
	public LiabilityBalance[] getFUTABalances() throws ServiceFailureException;
}
