package com.intuit.sb.payroll.services.taxpayments.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Currency;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.taxes.impl.LawUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.HistPaycheck;
import com.intuit.sb.payroll.entity.model.HistPaycheckLineItem;
import com.intuit.sb.payroll.entity.model.LawId;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentLineItemSplit;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.taxpayments.FutaAmountType;
import com.intuit.sb.payroll.services.taxpayments.FutaPaymentLiability;
import com.intuit.sb.payroll.services.taxpayments.FutaPaymentsAndLiabilities;
import com.intuit.sb.payroll.services.taxpayments.PaymentPeriod;

/**
 * Mapper to translate {@link Payment payment} and {@link HistPaycheck liability} objects <br>
 * into corresponding {@link FutaPaymentLiability FutaPaymentLiability} object <br>
 * and vice versa
 */
public class FutaPaymentsAndLiabilitiesMapper {

	private static void validateMultipleFutaPandLinSameQuarter(
			List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities)
		throws ValidationException {
		
		PayrollParamValidator.checkListAndContents(futaPaymentsAndLiabilities, "Futa Payments and Liabilities");
		FutaPaymentsAndLiabilities[] futaArray = new FutaPaymentsAndLiabilities[futaPaymentsAndLiabilities.size()];
		futaArray = futaPaymentsAndLiabilities.toArray(futaArray);
		
		for (int i=0; i<futaArray.length; i++) {
			PaymentPeriod period1 = futaArray[i].getQuarterPaymentPeriod();
			for (int j=0; j<futaArray.length; j++) {
				PaymentPeriod period2 = futaArray[j].getQuarterPaymentPeriod();
				if (i != j && period1.equals(period2)) {
					throw new ValidationException("Cannot have multiple Futa Payments or Liabilities in same quarter");
				}
			}
		}
	}

	/**
	 * Translate given {@link FutaPaymentsAndLiabilities FutaPaymentsAndLiabilities} object to {@link Payment Payment} entity
	 * @param futaPaymentsAndLiabilities
	 * @return
	 */
	public static List<Payment> getPaymentEntityFromFutaPaymentsAndLiabilities(
			List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities) 
				throws ValidationException {
		validateMultipleFutaPandLinSameQuarter(futaPaymentsAndLiabilities);
		PayrollParamValidator.checkListAndContents(futaPaymentsAndLiabilities, "Futa Payments and Liabilities");
		List<Payment> payments = new ArrayList<Payment>();
		Payment payment = null;
		PaymentLineItem pli = null;
		Set<PaymentLineItemSplit> splits = null;
		PaymentLineItemSplit pliSplit = null;
		List<PayrollItem> payrollItems = null;
		for (FutaPaymentsAndLiabilities paymentsAndLiabilities : futaPaymentsAndLiabilities) {
			if (paymentsAndLiabilities != null && paymentsAndLiabilities.getFutaPayment() != null) {
				FutaPaymentLiability futaPayment = paymentsAndLiabilities.getFutaPayment();
				PayrollParamValidator.checkIsNotNull(
						paymentsAndLiabilities.getQuarterPaymentPeriod(), "FutaPaymentsAndLiabilities.getQuarterPaymentPeriod()");
				Money money = getMoney(futaPayment.getAmount());
				if (futaPayment.getObjectHandle() != null && futaPayment.getObjectHandle().getId() > 0L) {
					// get from persistence
					EmployerComponent empComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
					Employer er = empComponent.getEmployer();
					Set<Payment> dbpayments = er.getPayments();
					PayrollParamValidator.checkIsNotNull(dbpayments, "Payments");
					PayrollParamValidator.checkIsNotNullOrEmptyArray(dbpayments.toArray(), "Payments");
					
					for (Payment payment2 : dbpayments) {
						if (payment2.getId() == futaPayment.getObjectHandle().getId()) {
							payment = payment2;
							break;
						}
					}
					if (payment != null) {
						Set<PaymentLineItem> lineitems = payment.getPaymentLineItems();
						if (lineitems == null || lineitems.size() == 0 || lineitems.size() > 1) {
							throw new ValidationException("Invalid PaymentLineItems for FUTA PAYMENT i DB");
						}
						for (PaymentLineItem li : lineitems) {
							pli = li;
							break;	//only one item
						}
						if (pli != null) {
							splits = pli.getPaymentLineItemSplits();
							if (splits == null || splits.size() == 0 || splits.size() > 1) {
								throw new ValidationException("Invalid PaymentLineItemSplits for FUTA PAYMENT i DB");
							}
						}
						for (PaymentLineItemSplit split : splits) {
							pliSplit = split;
							break;	//only one item
						}
						pli.setAmount(money);
						pliSplit.setSplitAmount(money);
					}
				} else {
					payment = new Payment();
					/* Default Payment Date to first day of quarter of the payment */
					payment.setPaymentDate(paymentsAndLiabilities.getQuarterPaymentPeriod().getPeriodStartDate());
					
					/* BEGIN === PaymentLineItem for this PaymentLine ==*/
					pli = new PaymentLineItem();
					
					pli.setAmount(money);
					/* BEGIN === PaymentLineItemSplit for this PaymentLineItem ==*/
					pliSplit = new PaymentLineItemSplit();
					pliSplit.setDerived(false);

					payrollItems = PaymentTypeLawMapper.mapPaymentTypeToPayrollItem(PaymentType.FORM940FUTA);
					PayrollItem pItem = LawUtils.getPayrollItem(payrollItems, QboeLawIdConstants.sFUTALawId);
					pliSplit.setPayrollItem(pItem);
					
					pliSplit.setSplitAmount(money);
					
					pli.addPaymentLineItemSplit(pliSplit);
					pli.setPaymentType(PaymentType.FORM940FUTA);
					/* END === PaymentLineItemSplit for this PaymentLineItem ==*/

					payment.addPaymentLineItem(pli);
					/* END === PaymentLineItem for this PaymentLine ==*/
				}
				
				
				payment.setPayrollPeriod(DateRangeMapper.mapPaymentPeriodToDateRange(paymentsAndLiabilities.getQuarterPaymentPeriod()));
				payments.add(payment);
				
			}
		}		
		return payments;
	}

	/**
	 * Translate given {@link FutaPaymentLiability} object to {@link HistPaycheck} entity
	 * @param futaPaymentsAndLiabilities
	 * @return
	 * @throws ServiceFailureException 
	 */
	public static List<HistPaycheck> getHistPaycheckEntityFromFutaPaymentsAndLiabilities(
			List<FutaPaymentsAndLiabilities> futaPaymentsAndLiabilities) 
				throws ValidationException, ServiceFailureException {
		validateMultipleFutaPandLinSameQuarter(futaPaymentsAndLiabilities);
		PayrollParamValidator.checkListAndContents(futaPaymentsAndLiabilities, "Futa Payments and Liabilities");
		List<HistPaycheck> list = new ArrayList<HistPaycheck>();
		EmployerComponent empComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
		Employer employer = empComponent.getEmployer();

		HistPaycheck histPaycheck = null;
		HistPaycheckLineItem hpli = null;

		for (FutaPaymentsAndLiabilities payAndLiab : futaPaymentsAndLiabilities) {
			if (payAndLiab != null && payAndLiab.getFutaLiability() != null) {
				PayrollParamValidator.checkIsNotNull(
						payAndLiab.getQuarterPaymentPeriod(), "FutaPaymentsAndLiabilities.getQuarterPaymentPeriod()");
				FutaPaymentLiability futaPL = payAndLiab.getFutaLiability();
				DateRange period = DateRangeMapper.mapPaymentPeriodToDateRange(payAndLiab.getQuarterPaymentPeriod());
				Money amt = new Money();
				amt.setAmount(futaPL.getAmount());
				if (futaPL.getObjectHandle() != null && futaPL.getObjectHandle().getId() > 0L) {
					// get from persistence
					Set<HistPaycheck> histPaychecks = employer.getHistPaychecks();
					PayrollParamValidator.checkIsNotNull(histPaychecks, "HistoricPaychecks");
					PayrollParamValidator.checkIsNotNullOrEmptyArray(histPaychecks.toArray(), "HistoricPaychecks");
					for (HistPaycheck histPaycheck2 : histPaychecks) {
						if (histPaycheck2.getId() == futaPL.getObjectHandle().getId()) {
							histPaycheck = histPaycheck2;
							break;
						}
					}
					
					if (histPaycheck != null) {
						Set<HistPaycheckLineItem> lineitems = histPaycheck.getHistPaycheckLineItems();
						if (lineitems == null || lineitems.size() == 0 || lineitems.size() > 1) {
							throw new ValidationException("Lineitems not correct for FUTA Liability found in DB");
						}
						for (HistPaycheckLineItem item : lineitems) {
							hpli = item;
							break;		// only one item
						}
						hpli.setValue(amt);
					}
				} else {
					histPaycheck = new HistPaycheck();
					histPaycheck.setPayrollPeriod(period);
					hpli = new HistPaycheckLineItem();
					histPaycheck.addHistPaycheckLineItem(hpli);
					hpli.setValue(amt);
					histPaycheck.setGrossAmount(amt);

					LawId lawid = new LawId();
					lawid.setValue(QboeLawIdConstants.sFUTALawId);
					hpli.setLawId(lawid);
					
					List<PayrollItem> pItems = PaymentTypeLawMapper.mapPaymentTypeToPayrollItem(PaymentType.FORM940FUTA);
					PayrollItem pItem = LawUtils.getPayrollItem(pItems, lawid.getValue());
					hpli.setPayrollItem(pItem);
					histPaycheck.setEmployer(employer);
				}
				
				list.add(histPaycheck);
			}
		}

		
		return list;
	}
	
	private static Money getMoney(BigDecimal amount) {
		Money money = new Money("0.00");
		String curr = LocaleGuru.getInstance().getCurrency();
		money.setCurrencyCode(Currency.getInstance(curr));
		money.setAmount(amount);
		return money;
	}

	private static BigDecimal getZeroBigDecimal() {
		BigDecimal amt = new BigDecimal("0.00");
		amt = amt.setScale(2, RoundingMode.HALF_UP);
		return amt;
	}

	/**
	 *
	 * @param quarter Quarter under consideration - {@see PtdUtils} Q1, Q2, Q3, Q4
	 * @param payments {@link List} of {@link Payment} objects
	 * @return Payment amount for payments in this quarter
	 * @throws ValidationException
	 */
	private static BigDecimal getFutaPaymentAmountForGivenQuarter
			(DateRange quarter, List<Payment> payments) throws ValidationException {
		BigDecimal amt = getZeroBigDecimal();
		for (Payment payment : payments) {
			if (payment.getPayrollPeriod().equals(quarter)) {
				amt = amt.add(getPaymentAmount(payment).getAmount());
			}
		}
		return amt;
	}

	/**
	 *
	 * @param quarter Quarter under consideration - {@see PtdUtils} Q1, Q2, Q3, Q4
	 * @param payments {@link List} of {@link Payment} objects
	 * @return Payment in this quarter
	 * @throws ValidationException
	 */
	private static Payment getFutaPaymentsForGivenQuarter
			(DateRange quarter, List<Payment> payments) throws ValidationException {
		for (Payment payment : payments) {
			if (payment.getPayrollPeriod().equals(quarter)) {
				return payment;
			}
		}
		return null;
	}

	/**
	 *
	 * @param quarter Quarter under consideration - {@see PtdUtils} Q1, Q2, Q3, Q4
	 * @param liabilities {@link List} of {@link HistPaycheck} items
	 * @return value of {@link HistPaycheck} for this quarter
	 * @throws ValidationException
	 */
	private static BigDecimal getFutaLiabilitiesAmountForGivenQuarter
			(DateRange quarter, List<HistPaycheck> liabilities) throws ValidationException {
		BigDecimal amt = getZeroBigDecimal();
		for (HistPaycheck histPaycheck : liabilities) {
			if (histPaycheck.getPayrollPeriod().equals(quarter)) {
				// assume only ONE lineitem
				Set<HistPaycheckLineItem> lineitems = histPaycheck.getHistPaycheckLineItems();
				if (lineitems == null || lineitems.size() == 0 || lineitems.size() > 1) {
					throw new ValidationException(
							"Futa Liabilities - HistPaycheck should have One and Only One HistPaycheckLineItem");
				}
				// update LIABILITY value of this FutaPaymentLiability
				for (HistPaycheckLineItem li : lineitems) {
					amt = amt.add(li.getValue().getAmount());
				}
			}
		}
		return amt;
	}

	/**
	 *
	 * @param quarter Quarter under consideration - {@see PtdUtils} Q1, Q2, Q3, Q4
	 * @param liabilities {@link List} of {@link HistPaycheck} items
	 * @return {@link HistPaycheck} for this quarter
	 * @throws ValidationException
	 */
	private static HistPaycheck getFutaLiabilitiesForGivenQuarter
			(DateRange quarter, List<HistPaycheck> liabilities) throws ValidationException {
		for (HistPaycheck histPaycheck : liabilities) {
			if (histPaycheck.getPayrollPeriod().equals(quarter)) {
				return histPaycheck;
			}
		}
		return null;
	}
	/**
	 * Translate {@link List} of {@link Payment} and {@link HistPaycheck} objects into equivalent list<br>
	 * of {@link FutaPaymentLiability}
	 * @param payments
	 * @param liabilities
	 * @throws ValidationException
	 * @return
	 */
	public static List<FutaPaymentsAndLiabilities> mapPaymentAndHistPaycheckEntityToFutaPaymentsAndLiabilities(
			List<Payment> payments, List<HistPaycheck> liabilities)
			throws ValidationException {
		List<FutaPaymentsAndLiabilities> list = new ArrayList<FutaPaymentsAndLiabilities>();
		FutaPaymentsAndLiabilities fpAndLs = null;
		FutaPaymentLiability fpayment = null;
		FutaPaymentLiability fliability = null;
		List<DateRange> quarterList = PtdUtils.getQuartersBeforeCurrentQuarter();

		for (DateRange range : quarterList) {
			Payment pay = getFutaPaymentsForGivenQuarter(range, payments);			
			HistPaycheck liab = getFutaLiabilitiesForGivenQuarter(range, liabilities);
			
			fpAndLs = new FutaPaymentsAndLiabilities();

			fpayment = new FutaPaymentLiability();
			fpayment.setFutaAmountType(FutaAmountType.FUTAPAYMENT);
			fpayment.setAmount(getFutaPaymentAmountForGivenQuarter(range, payments));	
			if (pay != null && pay.getId() > 0L) {
				fpayment.setObjectHandle(new ObjectHandle(pay.getId(), pay.getVersion()));
			}
			
			fliability = new FutaPaymentLiability();
			fliability.setFutaAmountType(FutaAmountType.FUTALIABILITY);
			fliability.setAmount(getFutaLiabilitiesAmountForGivenQuarter(range, liabilities));
			if (liab != null && liab.getId() > 0L) {
				fliability.setObjectHandle(new ObjectHandle(liab.getId(), liab.getVersion()));
			}

			fpAndLs.setQuarterPaymentPeriod(DateRangeMapper.mapDateRangeToPaymentPeriod(range));
			fpAndLs.setFutaLiability(fliability);
			fpAndLs.setFutaPayment(fpayment);
			list.add(fpAndLs);
		}

		return list;
	}

	/**
	 * Aggregate the input {@link Payment} object lineitems to get the corresponding payment amount
	 * @param payment
	 * @return Payment Amount
	 */
	private static Money getPaymentAmount(Payment payment) throws ValidationException {
		PayrollParamValidator.checkIsNotNull(payment, "payment");
		Set<PaymentLineItem> set = payment.getPaymentLineItems();
		Money amount = new Money("0.00");
		for (PaymentLineItem lineItem : set) {
			amount = amount.add(lineItem.getAmount());
		}

		return amount;
	}

}
