package com.intuit.sb.payroll.services.taxpayments.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.LawId;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.workerpayments.PayrollItemEAO;

/**
 * Utility class to map {@code PaymentType} enum to the corresponding
 * Law.
 * In most cases this is 1 <--> 1
 * ex: CAUI maps to law id 87
 * 
 * However in some cases the mapping is 1 <--> *
 * ex: FORM94x maps to law id's 64 and 62
 *
 */
public class PaymentTypeLawMapper {

	// store for mapping between law-id and paymenttype
	private static Map<String, PaymentType> sPaymentLawIdMap = new HashMap<String, PaymentType>();
	
	private PaymentTypeLawMapper() {
		// empty constructor
	}
	
	public static Set<String> getAllMappedLawIds() {
		if (sPaymentLawIdMap.size() == 0) {
			buildLawIdPaymentTypeMap();
		}
		return sPaymentLawIdMap.keySet();
	}
	
	static {
		buildLawIdPaymentTypeMap();
	}
	
	/**
	 * Build the {@link Map} - this has to be updated whenever new Tax laws are to be considered
	 */
	private static void buildLawIdPaymentTypeMap() {
		sPaymentLawIdMap.put(QboeLawIdConstants.sSST_EELawId, PaymentType.FORM94x);
		sPaymentLawIdMap.put(QboeLawIdConstants.sSST_ERLawId, PaymentType.FORM94x);
		sPaymentLawIdMap.put(QboeLawIdConstants.sMED_EELawId, PaymentType.FORM94x);
		sPaymentLawIdMap.put(QboeLawIdConstants.sMED_ERLawId, PaymentType.FORM94x);
		sPaymentLawIdMap.put(QboeLawIdConstants.sFICALawId, PaymentType.FORM94x);		
		sPaymentLawIdMap.put(QboeLawIdConstants.sFUTALawId, PaymentType.FORM940FUTA);
		sPaymentLawIdMap.put(QboeLawIdConstants.sCAETTLawId, PaymentType.CAETT);
		sPaymentLawIdMap.put(QboeLawIdConstants.sCAPITLawId, PaymentType.CAPIT);
		sPaymentLawIdMap.put(QboeLawIdConstants.sCASDILawId, PaymentType.CASDI);
		sPaymentLawIdMap.put(QboeLawIdConstants.sCASUILawId, PaymentType.CAUI);
		sPaymentLawIdMap.put(QboeLawIdConstants.sCobraPremiumCredit, PaymentType.COBRA);
	}
	
	/**
	 * @param lawId {@link LawId LawId} whose {@link PaymentType} needs to be retrieved
	 * @return {@link PaymentType} for given lawId - null if none found
	 */
	public static PaymentType mapLawIdToPaymentType(String lawId) {
		if (sPaymentLawIdMap.size() == 0) {
			buildLawIdPaymentTypeMap();
		}
		return sPaymentLawIdMap.get(lawId);
	}
	
	/**
	 * @param paymentType {@code PaymentType} object
	 * @return {@code List} of {@code String} objects with LawId(s) corresponding to the given paymentType
	 */
	public static List<String> mapPaymentTypeToLawId(PaymentType paymentType) {
		if (sPaymentLawIdMap.size() == 0) {
			buildLawIdPaymentTypeMap();
		}
		List<String> lawIds = new ArrayList<String>();
		Set<String> keyset = sPaymentLawIdMap.keySet();
		for (String key : keyset) {
			if (sPaymentLawIdMap.get(key).equals(paymentType)) {
				lawIds.add(key);
			}
		}
		
		return lawIds;
	}

	/**
	 * Loop through the list of Law Id's found for given PaymentType.<br>
	 * For each one, loop through the Set of {@code PayrollItem} objects for this user and<br>
	 * capture the ones with the law id corresponding to those found for the paymenttype.
	 * @param paymentType {@code PaymentType} object
	 * @return {@code List} of {@code PayrollItem} objects with LawId(s) corresponding to the given paymentType
	 */
	public static List<PayrollItem> mapPaymentTypeToPayrollItem(PaymentType paymentType) {
		List<String> lawIds = mapPaymentTypeToLawId(paymentType);
		List<PayrollItem> payrollItems = null;
		List<PayrollItem> payrollItemsFinal = new ArrayList<PayrollItem>();
		if (lawIds.size() > 0) {
			PayrollItemEAO payrollItemEao = ServiceLocator.getInstance().getService(PayrollItemEAO.JNDI_NAME);
			payrollItems = payrollItemEao.findAll();
			for (String lawId : lawIds) {
				// for every law id find the corresponding payroll item
				for (PayrollItem payrollItem : payrollItems) {
					if (payrollItem.getLawId().getValue().equals(lawId)) {
						payrollItemsFinal.add(payrollItem);
						break;
					}
				}

			}
		}
		
		return payrollItemsFinal;
	}
}
