package com.intuit.sb.payroll.services.taxpayments.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Currency;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponent;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.common.i18n.LocaleGuru;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.LawId;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Payment;
import com.intuit.sb.payroll.entity.model.PaymentLineItem;
import com.intuit.sb.payroll.entity.model.PaymentLineItemSplit;
import com.intuit.sb.payroll.entity.model.PaymentType;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.utils.PtdUtils;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.taxpayments.TaxPaymentLineItem;
import com.intuit.sb.payroll.services.taxpayments.TaxPayments;
import com.intuit.sb.payroll.services.taxpayments.ToDateTaxPayments;

/**
 * Utility class to map entity Payment entity
 * to Composed TaxPayments object
 */
public class TaxPaymentsMapper {
	
	private static final String[] lawIdsFor94X = {
			QboeLawIdConstants.sFICALawId, 
			QboeLawIdConstants.sSST_ERLawId, 
			QboeLawIdConstants.sSST_EELawId, 
			QboeLawIdConstants.sMED_ERLawId, 
			QboeLawIdConstants.sMED_EELawId};

	public static ToDateTaxPayments mapPaymentEntityToTaxPaymentsDto(Set<Payment> payments) 
			throws ValidationException {
		if (payments == null || payments.size() == 0) {
			return null;
		}

		int i = 0;
		Payment[] paymentsArray = new Payment[payments.size()];
		for (Payment payment : payments) {
			paymentsArray[i++] = payment;
		}
		
		return mapPaymentEntityToTaxPaymentsDto(paymentsArray);
	}

	/**
	 * 
	 * @param paymentsMap {@link Map Map} object with key as {@link DateRange DateRange} and <br>
	 * value as {@link List List} of {@link Payment Payment} objects that fall within that range
	 * @return Array of {@link ToDateTaxPayments ToDateTaxPayments}
	 * @throws ValidationException
	 */
	public static ToDateTaxPayments[] mapPaymentEntityToTaxPaymentsDto(
			Map<DateRange, List<Payment>> paymentsMap) throws ValidationException {
//		Now our Payment objects are distributed by their payroll period in the map			
		Set<DateRange> rangeSet = paymentsMap.keySet();
		ToDateTaxPayments[] todatePayments = new ToDateTaxPayments[rangeSet.size()];
		ToDateTaxPayments todatePayment = null;
		int i = 0;
		for (DateRange range : rangeSet) {
			todatePayment = mapPaymentEntityToTaxPaymentsDto(paymentsMap.get(range));
			todatePayments[i++] = todatePayment;
		}
		
		return todatePayments;
	}
	
	public static ToDateTaxPayments mapPaymentEntityToTaxPaymentsDto(List<Payment> payments) 
			throws ValidationException {
		if (payments == null || payments.size() == 0) {
			return null;
		}

		int i = 0;
		Payment[] paymentsArray = new Payment[payments.size()];
		for (Payment payment : payments) {
			paymentsArray[i++] = payment;
		}
		
		return mapPaymentEntityToTaxPaymentsDto(paymentsArray);
	}

	/**
	 * 
	 * @param payments Array of {@code Payment} objects
	 * @return {@code ToDateTaxPayments} object
	 * @throws ValidationException
	 */
	public static ToDateTaxPayments mapPaymentEntityToTaxPaymentsDto(Payment[] payments) 
		throws ValidationException {
		
		if (payments == null || payments.length == 0) {
			return null;
		}

		ToDateTaxPayments todate = new ToDateTaxPayments();
		TaxPayments[] taxPayments = new TaxPayments[payments.length];	// TaxPayments <==> Payment
		todate.setTaxPayments(taxPayments);	//	Assign TaxPayments Array to ToDateTaxPayments

		int i = 0;
		LawId lawid = null;
		PayrollItem payrollItem = null;

		for (Payment payment : payments) {
			if (payment == null) {
				throw new ValidationException("Invalid / NULL Payment component in ToDateTaxPayments");
			}
			if (todate.getTaxPaymentsEndDate() == null) {	// do this only once
				//  Assign Tax Payments start/end date 
				//	(usually corresponding to beginning and end of a quarter)
				todate.setTaxPaymentsStartDate(payment.getPayrollPeriod().getStartDate());
				todate.setTaxPaymentsEndDate(payment.getPayrollPeriod().getEndDate());
			}

			TaxPayments taxPayment = new TaxPayments();
			taxPayments[i++] = taxPayment;
			if (payment.getId() > 0) {
				taxPayment.setObjectHandle(new ObjectHandle(payment.getId(), payment.getVersion()));
			}
			taxPayment.setTaxPaymentDate(payment.getPaymentDate());
			taxPayment.setSentToPTS(StatusUpdateHandlerUtil.isEntityAddedToPTS(payment));

			Set<PaymentLineItem> paymentLineItems = payment.getPaymentLineItems();
			TaxPaymentLineItem[] tpLineItems = new TaxPaymentLineItem[paymentLineItems.size()];
			taxPayment.setTaxPaymentLineItems(tpLineItems);
			
			
			int j = 0;
			for (PaymentLineItem paymentLineItem : paymentLineItems) {
				TaxPaymentLineItem tpli = new TaxPaymentLineItem();
				if (paymentLineItem.getId() > 0) {
					tpli.setObjectHandle(new ObjectHandle(paymentLineItem.getId(), paymentLineItem.getVersion()));
				}
				tpLineItems[j++] = tpli;
				
				Set<PaymentLineItemSplit> splits = paymentLineItem.getPaymentLineItemSplits();
				
				for (PaymentLineItemSplit split : splits) {
//					Get the "LawId" from the first PaymentLineItemSplit's PayrollItem
//					All the Split Items will have the same Law reference.
					payrollItem = split.getPayrollItem();
					lawid = payrollItem.getLawId();
					break;
				}
				
				tpli.setAmount(paymentLineItem.getAmount().getAmount());
				tpli.setLawId(lawid.getValue());
				tpli.setPaymentType(paymentLineItem.getPaymentType().name());
			}
		}
		
		return todate;
	}
	
	/**
	 * @param payment {@code TaxPayments} object
	 * @return {@code Payment} object corresponding the input {@link TaxPayments} object <br>
	 * Retrieve existing object from DB or create a new object as required.
	 */
	private static Payment getPaymentForGivenTaxPayments (TaxPayments taxpayment) 
			throws ValidationException, ServiceFailureException {
		Payment payment = null;
		Set<PaymentLineItem> paymentLineItems = null;
		
		if (taxpayment == null || taxpayment.getTaxPaymentDate() == null) {
			throw new ValidationException("Invalid TaxPayments object - either the Taxpayment is null" +
					" or it has no Tax Payment Date selected.");
		}
		
		if (taxpayment.getObjectHandle() == null || 
				taxpayment.getObjectHandle().getId() == 0L) {	//	create new object
			payment = new Payment();
			payment.setPaymentDate(taxpayment.getTaxPaymentDate());
			payment.setPayrollPeriod(PtdUtils.getQtrRangeForDate(taxpayment.getTaxPaymentDate()));

			// initially empty set of PaymentLineItem objects
			paymentLineItems = getPaymentLineItems(taxpayment);
			payment.setPaymentLineItems(paymentLineItems);
		} else {	//	get existing object
			payment = loadPaymentFromPersistence(taxpayment);
		}
		
		return payment;
	}

	/**
	 * Load the {@code Payment} and its linked {@code PaymentLineItem} and {@code PaymentLineItemSplit} objects
	 * since these are NOT defined as FetchType.EAGER in the model.
	 * @param id
	 * @return
	 * @throws ValidationException
	 */
	private static Payment loadPaymentFromPersistence(TaxPayments taxpayment) throws ValidationException {
		long taxPaymentId = taxpayment.getObjectHandle().getId();
		EmployerComponent empComponent = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
		// FIXME Use of PaymentEAO could have reduced the code here
		
		Set<Payment> payments = empComponent.getEmployer().getPayments();
		Payment payment = null;
		for (Payment paymentDb: payments){
			if (paymentDb.getId() == taxPaymentId){
				payment = paymentDb; 
				break;
			}
		}
		if (payment == null) {
			throw new ValidationException("Unable to find Payment with ID " + taxPaymentId);
		}
		
		payment.setPaymentDate(taxpayment.getTaxPaymentDate());
		payment.setPayrollPeriod(PtdUtils.getQtrRangeForDate(taxpayment.getTaxPaymentDate()));
		
		TaxPaymentLineItem[] taxPaymentLineItems = taxpayment.getTaxPaymentLineItems();
		
		// Now the individuals PaymentLineItems
		Set<PaymentLineItem> paymentLineItems = payment.getPaymentLineItems();
		
		for (PaymentLineItem paymentLineItem : paymentLineItems){
			boolean found = false;
			for (int i = 0; i < taxPaymentLineItems.length; i++){
				TaxPaymentLineItem taxPaymentLineItem = taxPaymentLineItems[i];
				
				if ((taxPaymentLineItem.getObjectHandle() != null) &&
					(taxPaymentLineItem.getObjectHandle().getId() == paymentLineItem.getId())){
					found = true;;
					if (taxPaymentLineItem.getAmount().compareTo(BigDecimal.ZERO) == -1) {
						throw new ValidationException("Negative amounts not allowed for Payments: " +
								taxPaymentLineItem.getAmount());
					}
					paymentLineItem.setAmount(new Money(taxPaymentLineItem.getAmount().toString()));
				}
			}
			
			if (!found){
				throw new ValidationException("Unable to find PaymentLineItem with ID " + paymentLineItem.getId());
			}
		}
		
		return payment;
	}
	
	private static Set<PaymentLineItem> getPaymentLineItems(TaxPayments taxpayment) 
				throws ValidationException, ServiceFailureException {
		Set<PaymentLineItem> paymentLineItems = null;
		PaymentLineItem pli = null;
		
		if (taxpayment.getObjectHandle() == null || 
				taxpayment.getObjectHandle().getId() == 0L) {	//	create new object
			// initially empty set of PaymentLineItem objects
			paymentLineItems = new HashSet<PaymentLineItem>();
			TaxPaymentLineItem[] tpLineItems = taxpayment.getTaxPaymentLineItems();	// TaxPaymentLineItem <==> PaymentLineItem
			EmployerComponent eComp = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);
			DateRange range = PtdUtils.getQtrRangeForDate(taxpayment.getTaxPaymentDate());
			Map<String, Money> map = eComp.getLiabilitiesFromExistingPayrollRuns(range); //  liabilities data map
				
			// now that we have liabilities, split the items
			for (TaxPaymentLineItem tpli : tpLineItems) {
				if (tpli.getAmount().compareTo(BigDecimal.ZERO) == -1) {
					// invalid - negative amount
					throw new ValidationException("Negative amounts not allowed for Payments: " +
							tpli.getAmount());
				}
				// each line item represents a PaymentType - FUTA/94X/940/etc.
				pli = new PaymentLineItem();
				paymentLineItems.add(pli);
				pli.setAmount(getMoneyFromAmount(tpli.getAmount()));
				pli.setPaymentType(PaymentType.fromValue(tpli.getPaymentType()));

				Set<PaymentLineItemSplit> pliSplits = splitPaymentLineItems(pli, map);
				for (PaymentLineItemSplit payLineItemSplit : pliSplits){
					pli.addPaymentLineItemSplit(payLineItemSplit);
				}
				
			}
		} else {	//	load from DB
			Payment payment = getPaymentForGivenTaxPayments(taxpayment);
			paymentLineItems = payment.getPaymentLineItems();
		}
		return paymentLineItems;
	}
		
	/** Given a {@link ToDateTaxPayments} object, retrieve an Array of {@link Payment} objects */
	public static Payment[] mapTaxPaymentsDtoToPaymentEntity (ToDateTaxPayments todatetaxPayments) 
		throws ValidationException, ServiceFailureException {

		TaxPayments[] taxpayments = todatetaxPayments.getTaxPayments();
		Payment[] payments = new Payment[taxpayments.length];	//	Payment <==> TaxPayments

		int i = 0;
		Payment payment = null;

		for (TaxPayments tp : taxpayments) {
			payment = getPaymentForGivenTaxPayments(tp);
			payments[i++] = payment;
		}
		
		List<PayrollItem> payrollItems = PaymentTypeLawMapper.mapPaymentTypeToPayrollItem(PaymentType.FORM94x);
		Money[][] splitAmounts = new Money[0][0];
		if (payments != null && payments.length > 0) {
			splitAmounts = do94xSplit(payments, payrollItems);
		}
		
		i = 0;
		// clear and replace existing split items with new ones always
		PaymentLineItemSplit splitItem = null;
		for (Payment payment2 : payments) {
			PaymentLineItem li = get94xPaymentLineItem(payment2);
			if (li != null) {
				li.removeAllPaymentLineItemSplits();
				
				for (int j = 0; j < splitAmounts[i].length; j++) {
					splitItem = new PaymentLineItemSplit();
					splitItem.setDerived(true);
					splitItem.setSplitAmount(splitAmounts[i][j]);
					splitItem.setPayrollItem(getPayrollItemForGivenLawId(payrollItems, lawIdsFor94X[j]));
					li.addPaymentLineItemSplit(splitItem);
				}
				i++;
			}
		}
		
		return payments;
	}
	
	private static PayrollItem getPayrollItemForGivenLawId(List<PayrollItem> payrollItems, String lawId) {
		for (PayrollItem item : payrollItems) {
			if (item.getLawId().getValue().equals(lawId)) {
				return item;
			}
		}
		return null;
	}
	
	
	/**
	 * When liabilities for 94X are greater than payment made, find the balance.<br>
	 * Redistribute it as per following example<br>
	 * <code>
	 * 
	 * 		Total liabilities - $5800 with breakup
	 * 		1.	Federal Income Tax 			= 3000
	 *		2.	Social Security Employee 	= 600
	 *		3.	Social Security Employer 	= 600
	 *		4.	Medicare Employee 			= 800
	 *		5.	Medicare Employer 			= 800
	 *
	 * 		======== CASE: Liabilities > Payments ============
	 * 		Total payments on 94X: $7000 ==> Balance = 7000 - 5800 = $1200
	 * 		So we redistribute the payments as below (in reverse order):
	 * 		5.	Medicare Employer 			= 0 	($800 < 1200 ==>> 1200 - 800: Balance: $400)
	 * 		4.	Medicare Employee 			= 400	($800 > 400  ==>>  800 - 400: Balance: 0)
	 * 		3.	Social Security Employer 	= 600
	 * 		2.	Social Security Employee 	= 600
	 * 		1.	Federal Income Tax 			= 3000
	 * 
	 * 		======== CASE: Liabilities <= Payments ============
	 * 		Total payments on 94X - $6800 ==> Balance = -VE $1000
	 * 		So we redistribute the payments as below:
	 * 		(Balance / no. of items(5)) = 1000/5 = $200
	 * 		1.	Federal Income Tax 			= 3000 + 200 = $3200
	 * 		2.	Social Security Employee 	= 600 + 200 = $800
	 * 		3.	Social Security Employer 	= 600 + 200 = $800
	 * 		4.	Medicare Employee 			= 800 + 200 = $1000
	 * 		5.	Medicare Employer 			= 800 + 200 = $1000
	 * 		=====================================================
	 * 
	 * 
	 * 
	 * </code>
	 * @param payments			{@link Payment payment} Array
	 * @param payrollItems		{@link PayrollItem payrollItem} objects
	 * @return Double array of split amounts
	 */
	public static Money[][] do94xSplit(Payment[] payments, List<PayrollItem> payrollItems) 
			throws ValidationException, ServiceFailureException {
		
		// now create/manage Line Item Split for Payment Line Items
		// for most PaymentTypes there will be a single split item for a line item.
		// for PaymentType Form94X, the split will be determined by the liabilities.
		//  array of lawids, payments and liabilities - there will be a 1 <--> 1 mapping
		//  between the elements

		//  array of lawids, payments and liabilities - there will be a 1 <--> 1 mapping
		//  between the elements
		// 2 dimensional array - columns=Payments, row=split (5)
		Money[][] paymentContents = new Money[payments.length][lawIdsFor94X.length];
		Money[] liabilities = new Money[lawIdsFor94X.length];
		EmployerComponent eComp = ServiceLocator.getInstance().getService(EmployerComponent.JNDI_NAME);

		int i = 0; int j = 0;
		for (i=0; i<payments.length; i++) {
			for (j=0; j<lawIdsFor94X.length; j++) {
				paymentContents[i][j] = new Money();	//	initialize to zero
			}
		}
	
		for (i=0; i<lawIdsFor94X.length; i++) {
			liabilities[i] = new Money();	//	initialize to zero
		}

		Map<String, Money> liabilitiesMap = //  liabilities data map for this quarter
			eComp.getLiabilitiesFromExistingPayrollRuns(payments[0].getPayrollPeriod());			
			
		for (int k = 0; k < lawIdsFor94X.length; k++) {
			if (liabilitiesMap.get(lawIdsFor94X[k]) != null){
				liabilities[k] = liabilitiesMap.get(lawIdsFor94X[k]);	//	store actual liability line item
			}
		}
		
		Money totalPaymentLineItemAmount = new Money();	//  value of a given payment
		Money paymentSubTotal = new Money();			//  sum of payment splits at a given point
		Money splitTotal = new Money();	//  sum of given split item across payments
		Money split1 = new Money();	//  calculation placeholder
		Money split2 = new Money();	//  calculation placeholder
		Money splitMin = new Money();
		Money splitBal = new Money();
		for (i=0; i<payments.length; i++) {	//  loop through each payment
			PaymentLineItem plitem = get94xPaymentLineItem(payments[i]);
			if (plitem == null) {
				totalPaymentLineItemAmount = new Money("0.00");
			} else {
				totalPaymentLineItemAmount = plitem.getAmount();
			}
			for (j=0; j<lawIdsFor94X.length; j++) {	//  loop through each split item
				paymentSubTotal = getThis94xPaymentTotal(paymentContents, i);
				// if sum of splits for this payment is already equal to 
				// total payment amount, move to next payment
				if (paymentSubTotal.getAmount().equals(totalPaymentLineItemAmount.getAmount())) {
					break;	// move to next payment
				}
				// total of this split item across all payments
				splitTotal = getThisSplitItemTotal(paymentContents, j);
				if (splitTotal.getAmount().equals(liabilities[j].getAmount())) {
					continue;
				} else {
					split1 = liabilities[j].subtract(splitTotal);
					split2 = totalPaymentLineItemAmount.subtract(paymentSubTotal);
					// take minimum of split1 and split2
					splitMin = split1.getAmount().compareTo(split2.getAmount()) == -1? split1 : split2;
					paymentContents[i][j] = splitMin;
				}
			}
			// now for the case of PAYMENTS > LIABILITIES
			// distribute excess payment across split items of the current payment
			Money excessPaymentRemaining = 
				totalPaymentLineItemAmount.subtract(getThis94xPaymentTotal(paymentContents, i));
			if (excessPaymentRemaining.getAmount().compareTo(BigDecimal.ZERO) == 1) {
				// balance to divide equally across each split
				BigDecimal dividedExcessPayment = 
					excessPaymentRemaining.getAmount().divide(new BigDecimal(lawIdsFor94X.length), 2, RoundingMode.HALF_UP);
				splitBal.setAmount(dividedExcessPayment);
				for (int k=0; k<lawIdsFor94X.length; k++) {
					paymentContents[i][k] = paymentContents[i][k].add(splitBal);
				}
			}
		}
		return paymentContents;
	}
	
	//  get the sum of 94x split items for this payment calculated so far
	private static Money getThis94xPaymentTotal(Money[][] payments, int paymentIndex) {
		Money total = new Money();
		for (int i = 0; i < payments[paymentIndex].length; i++) {
			total = total.add(payments[paymentIndex][i]);
		}
		return total;
	}
	
	
	private static Money getThisSplitItemTotal(Money[][] payments, int itemIndex) {
		Money total = new Money();
		for (int i = 0; i < payments.length; i++) {
			total = total.add(payments[i][itemIndex]);
		}
		return total;
	}
	
	private static PaymentLineItem get94xPaymentLineItem(Payment payment) {
		for (PaymentLineItem li : payment.getPaymentLineItems()) {
			if (li.getPaymentType().equals(PaymentType.FORM94x)) {
				return li;
			}
		}
		return null;
	}
	
	/**
	 * Based on the PaymentLineItem TaxPaymentType, split it into multiple items
	 * @param map 
	 */
	private static Set<PaymentLineItemSplit> splitPaymentLineItems(
			PaymentLineItem paymentLineItem, Map<String, Money> liabilitiesMap) {
		PaymentType type = paymentLineItem.getPaymentType();
		Set<PaymentLineItemSplit> split = new HashSet<PaymentLineItemSplit>();
		List<PayrollItem> payrollItems = PaymentTypeLawMapper.mapPaymentTypeToPayrollItem(type);
		
		switch (type) {
		case FORM94x:
//			this split is handled separately at the end of payments creation
			break;
		default:
			split = doDefaultSplit(payrollItems, paymentLineItem.getAmount());
			break;
		}
		
		return split;
	}
	
	/**
	 * Default split - payrollItems should be of {@code List} with only ONE {@code PayrollItem}
	 * @param payrollItems
	 * @param money 
	 * @param type
	 * @return
	 */
	private static Set<PaymentLineItemSplit> doDefaultSplit(List<PayrollItem> payrollItems, Money money) {
		Set<PaymentLineItemSplit> split = new HashSet<PaymentLineItemSplit>();
		if (payrollItems == null || payrollItems.size() == 0) {
			return null;
		}
		
		PaymentLineItemSplit splitItem = new PaymentLineItemSplit();
		splitItem.setDerived(false);
		splitItem.setSplitAmount(money);
		splitItem.setPayrollItem(payrollItems.get(0));

		split.add(splitItem);

		return split;
	}

	private static Money getMoneyFromAmount(BigDecimal amount) {
		Money money = new Money();
		money.setAmount(amount);
		Currency curr = Currency.getInstance(LocaleGuru.getInstance().getCurrency());
		money.setCurrencyCode(curr);
		return money;
	}
	
    /**
	 *	Payments may be spread across different quarters/date ranges.
	 *	group them by their quarter
	 *	use a Map with key as the guidance payment period/date range
	 *	the value is a list of Payment objects that fall in that period/range
	 */
	public static Map<DateRange, List<Payment>> groupPaymentsByPayrollPeriod(List<Payment> payments) {
		Map<DateRange, List<Payment>> map = new HashMap<DateRange, List<Payment>>();
		List<Payment> newList = null;
		for (Payment payment : payments) {
			DateRange period = payment.getPayrollPeriod();
			if (map.keySet().contains(period)) {
				map.get(period).add(payment);
			} else { 
				newList = new ArrayList<Payment>();
				newList.add(payment);
				map.put(period, newList);
			}
		}
		
		return map;
	}
	
}
