package com.intuit.sb.payroll.services.wages;

import java.io.Serializable;
import java.math.BigDecimal;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.services.common.ObjectHandle;

/**
 * A compensation item. Each CompensationItem specifies a single pay item for a worker. It is
 * the actual amount the worker is to be paid for a specific type of earning.
 */
public class CompensationItem implements Serializable {

	private static final long serialVersionUID = 5256205605687331907L;

	private ObjectHandle objectHandle;
	private BigDecimal rate;
	private double units;
	private CompensationType compensationType;
	private String name;
	private CompensationItem linkedCompensationItem; 
	private boolean fixedRate;
	private PtoBaseRateType ptoBaseRateType = PtoBaseRateType.NOTSET;
	private boolean deleted;
	

	/**
	 * Minimal constructor (required by LCDS)
	 */
	public CompensationItem() {
	}

	/**
	 * Constructs a CompensationItem.
	 * @param objectHandle
	 * @param rate rate in $/hr for hourly
	 * @param units
	 * @param compensationType
	 * @param name
	 */
	public CompensationItem(ObjectHandle objectHandle, BigDecimal rate, double units, 
			CompensationType compensationType, String name, Boolean fixedRate) {
		this.objectHandle = objectHandle;
		this.rate = rate;
		if (null != rate) {
            // set to 2 decimal places
    		this.rate = this.rate.setScale(2);
		}
		this.units = units;
		this.compensationType = compensationType;
		this.name = name;
		this.fixedRate = fixedRate;
	}
	
	/**
	 * Constructs a CompensationItem linked to another CompensationItem.  Use
	 * this for overtime and double-time since they need to be linked to the
	 * hourly CompensationItem.
	 * @param objectHandle
	 * @param rate multiplication factor for overtime/double time (e.g. 1.5/2.0)
	 * @param units
	 * @param compensationType
	 * @param name
	 * @param linkedCompensationItem CompensationItem this CompensationItem links to, e.g. overtime links to hourly
	 */
	public CompensationItem(ObjectHandle objectHandle, BigDecimal rate, double units, 
			CompensationType compensationType, String name, CompensationItem linkedCompensationItem,Boolean fixedRate) {
		this(objectHandle, rate, units, compensationType, name, fixedRate);
		this.linkedCompensationItem = linkedCompensationItem;
	}

	/**
	 * Gets the handle to this object.
	 * For a new CompensationItem this must be null. For an
	 * existing item it must be the handle obtained when the COmpensationItem
	 * was originally produced; it must not be modified.
	 * @return the objectHandle
	 */
	public ObjectHandle getObjectHandle() {
		return objectHandle;
	}

	/**
	 * Sets the handle to this object.
	 * @param objectHandle the objectHandle to set
	 */
	public void setObjectHandle(ObjectHandle objectHandle) {
		this.objectHandle = objectHandle;
	}

	/**
	 * Gets the CompensationItem this CompensationItem is linked to.
	 * This allows one compensation to be based on, or associated with, another.
	 * e.g. a bonus might be expressed as a percentage and linked to the salary
	 * compensation. If there is no link this property is null.
	 * @return the linkedCompensationItem
	 */
	public CompensationItem getLinkedCompensationItem() {
		return linkedCompensationItem;
	}

	/**
	 * Sets the CompensationItem this one is linked to.
	 * @param linkedCompensationItem the linkedCompensationItem to set
	 */
	public void setLinkedCompensationItem(CompensationItem linkedCompensationItem) {
		this.linkedCompensationItem = linkedCompensationItem;
	}

	/**
	 * The rate of pay. This is in $/hr for hourly, multiplication factor for overtime/double time (e.g. 1.5/2.0)
	 * @return the rate
	 */
	public BigDecimal getRate() {
		return rate;
	}

	/**
	 * @param rate the rate to set
	 */
	public void setRate(BigDecimal rate) {
		this.rate = rate;
		if (null != rate) {
            // set to 2 decimal places
    		this.rate = this.rate.setScale(2);
		}
	}

	/**
	 * The number of units. These are in terms of the rate. For example,
	 * 20 units at $15/unit, 20 hours at $15/hour, 20% commission on $15,
	 * would all have a unit of 20 and a rate of 15.
	 * @return the units
	 */
	public double getUnits() {
		return units;
	}

	/**
	 * @param units the units to set
	 */
	public void setUnits(double units) {
		this.units = units;
	}

	public CompensationType getCompensationType() {
		return compensationType;
	}

	public void setCompensationType(CompensationType compensationType) {
		this.compensationType = compensationType;
	}

	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * This attribute corresponds to the fixed/varied options
	 * Fixed type has an associated rate.
	 * Compensation types Bonus, tips, reimbursement,commission can be fixed or varied
	 * Compensation types Hourly, Salary, Piecework are all fixed
	 * @return IsFixedRate
	 */
	public boolean isFixedRate() {
		return fixedRate;
	}

	public void setFixedRate(boolean fixedRate) {
		this.fixedRate = fixedRate;
	}
	
	public void setPtoBaseRateType(PtoBaseRateType ptoBaseRateType) {
		this.ptoBaseRateType = ptoBaseRateType;
	}

	public PtoBaseRateType getPtoBaseRateType() {
		return ptoBaseRateType;
	}

	/**
	 * @param deleted the deleted to set
	 */
	public void setDeleted(boolean deleted) {
		this.deleted = deleted;
	}

	/**
	 * @return the deleted
	 */
	public boolean isDeleted() {
		return deleted;
	}

	@Override
	public boolean equals(Object other) {
		 return EqualsBuilder.reflectionEquals(this, other);
	}

	@Override
	public int hashCode() {
		 return HashCodeBuilder.reflectionHashCode(this);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}
}
