package com.intuit.sb.payroll.services.worker;

import java.util.Date;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;

/**
 * The Worker Service exposes Worker Management functionality,
 * including Setup, Maintenance, and Dismissals.
 * 
 */
@Remote
public interface WorkerService {

	public static final String JNDI_NAME = "WorkerService/remote";

	/**
	 * Gets a collection of Worker instances.
	 * 
	 * @param filter Specifies a filter that governs which workers are returned.
	 * @return The collection of Workers for the current company that match the filter.
	 * The collection can be empty.
	 * @throws ValidationException if there is a constraint that is violated. For example,
	 * the workerId is a well formed value but the worker does not exist (perhaps it has been deleted).
	 * @throws ServiceFailureException the indicated method argument is an invalid value (eg. null).
	 * The provided value can never, in any context, be valid and result in a successful call. 
	 * This is an application coding error and not something the client user can correct.
	 */
	public Worker[] getWorkers(GetWorkersFilter filter) throws ServiceFailureException, ValidationException;
	
	/**
	 * Gets a collection of Worker instances assigned to a given WorkLocation.
	 * 
	 * @param uniqueId The uniqueId of the WorkLocation for the Workers that are returned.
	 * @return The collection of Workers for the current company that match the WorkLocation.
	 * The collection can be empty.
	 * @throws ValidationException if there is a constraint that is violated. For example,
	 * the workerId is a well formed value but the worker does not exist (perhaps it has been deleted).
	 * @throws ServiceFailureException the indicated method argument is an invalid value (eg. null).
	 * The provided value can never, in any context, be valid and result in a successful call. 
	 * This is an application coding error and not something the client user can correct.
	 */
	public Worker[] getWorkersByWorkLocation(Long uniqueId) throws ServiceFailureException, ValidationException;


	/**
	 * Adds or Updates a Worker and their associated withholding configuration.
	 * 
	 * At a minimum, the worker's first and last name must
	 * be provided. All other information is optional (use a null to
	 * indicate an unspecified value). Specifically, for a new worker,
	 * the objectHandle value must be null. 
	 *  
	 * @param worker the worker to add/update.
	 * @return the Worker which, for a new worker, will include the new unique identifier.
	 * @throws ValidationException if there is a constraint that is violated.
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 * @throws WorkflowException if some action has to occur before this function can be executed.
	 */
	public Worker saveWorker(Worker worker) throws ValidationException, WorkflowException, ServiceFailureException;
	
	/**
	 * Deletes a worker from the system.  ValidationException if there are any paychecks issued for this worker.
	 * @param workerId
	 * @throws ValidationException
	 * @throws ServiceFailureException
	 */
	public void deleteWorker(Long workerId) throws ValidationException, ServiceFailureException;
	
	/**
	 * Terminates the Workers employment as of the terminationDate
	 * @param worker
	 * @param terminationDate
	 * @return
	 * @throws ValidationException
	 * @throws WorkflowException
	 * @throws ServiceFailureException
	 */
	public Worker terminateWorker(Worker worker, Date terminationDate) throws ValidationException, WorkflowException, ServiceFailureException;

	/**
	 * Executes any rehire workflows required to make a Terminated Worker Employed again.
	 * 
	 * @param workerId
	 * @param hireDate
	 * @return
	 * @throws ValidationException
	 * @throws WorkflowException
	 * @throws ServiceFailureException
	 */
	public Worker hireExistingWorker(Long workerId, Date hireDate) throws ValidationException, WorkflowException, ServiceFailureException;

	/**
	 * Retrieves a Worker by their unique identifier.
	 * 
	 * @param uniqueId The unique identifier.
	 * @return all kinds of interesting data on the Worker.
	 * @throws ServiceFailureException if the uniqueId does not match an existing worker.
	 * @throws ServiceFailureException if there is a problem with the database etc...
	 */
	public Worker getWorkerById(Long uniqueId) throws ServiceFailureException, ValidationException;
	
	public boolean isWorkerReadyForPay(Worker worker) throws ServiceFailureException, ValidationException;
	
}
