package com.intuit.sb.payroll.services.worker.impl;

import java.util.List;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.worker.FindWorkersFilter;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.worker.GetWorkersFilter;

/**
 * This class is responsible for Implementing the the Service to get a list of Workers. It is expected that this class
 * will be directly invoked by the implementor of IWorkerService to realize the getWorkers API.
 * 
 * $Author: thobson $ 
 * $Revision: #5 $ 
 * $DateTime: 2008/12/17 20:05:18 $
 */
class GetWorkersImpl {

	private GetWorkersImpl() {
	}

	/**
	 * Flow control method that controls the sequence of calls to back-end services for the implementation.
	 * @param workerComponent 
	 * 
	 * @param filter the Filter on which to base the query.
	 * @return An array of Workers. May be empty.
	 * @throws ValidationException thrown if the filter is null.
	 * @throws ServiceFailureException if there is a server error.
	 */
	static com.intuit.sb.payroll.services.worker.Worker[] getWorkers(WorkerComponent workerComponent, GetWorkersFilter filter) throws ValidationException,
			ServiceFailureException {
		validate(filter);
		FindWorkersFilter capabilityFilter = mapFilter(filter);
		List<Worker> workerList = 	workerComponent.findWorkersByFilter(capabilityFilter);
		com.intuit.sb.payroll.services.worker.Worker[] result = WorkerMapper.mapFromEntitiesToPojos(workerList);
		return result;
	}

	private static FindWorkersFilter mapFilter(GetWorkersFilter filter) {
		if (filter.equals(GetWorkersFilter.ALL)) {
			return FindWorkersFilter.ALL;
		} else if (filter.equals(GetWorkersFilter.NOT_READY_FOR_PAYROLL)) {
			return FindWorkersFilter.NOT_READY_FOR_PAYROLL;
		} else if (filter.equals(GetWorkersFilter.READY_FOR_PAYROLL)) {
			return FindWorkersFilter.READY_FOR_PAYROLL;
		} else if (filter.equals(GetWorkersFilter.ACTIVE)) {
			return FindWorkersFilter.ACTIVE;
		} else if (filter.equals(GetWorkersFilter.INACTIVE)) {
			return FindWorkersFilter.INACTIVE;
		}
		return null;
	}

	/**
	 * Gets the list of Workers and returns a collection of Worker objects for a given WorkLocation
	 * 
	 * @param uniqueId The uniqueId of the WorkLocation for the Workers
	 * @return An array of Worker. May be empty.
	 * @throws ValidationException thrown if the filter is null.
	 * @throws ServiceFailureException if there is a server error.
	 */
	static com.intuit.sb.payroll.services.worker.Worker[] getWorkersByWorkLocation(WorkerComponent workerComponent, Long uniqueId) throws ValidationException,
			ServiceFailureException {
		CommonValidators.validateId(uniqueId);
		List<Worker> workerList = workerComponent.findWorkersByWorkLocation(uniqueId);
		return WorkerMapper.mapFromEntitiesToPojos(workerList);
	}

	/**
	 * Validates that the required inputs are provided.
	 * 
	 * @param filter
	 * @throws ValidationException thrown if the filter is null.
	 */
	private static void validate(GetWorkersFilter filter) throws ValidationException {
		if (null == filter) {
			throw new ValidationException("GetWorkersFilter is null");
		}
	}
}