package com.intuit.sb.payroll.services.worker.impl;

import java.util.Date;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.common.validation.PayrollParamValidator;
import com.intuit.sb.payroll.entity.domain.Identifiable;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;
import com.intuit.sb.payroll.services.worker.GetWorkersFilter;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerService;

/**
 * Implementation of the IWorkerService. Delegates most implementation to other
 * classes.
 * 
 * This Bean will create a transaction or participate in the current one.
 *
 * $Author: rjohn2 $
 * $Revision: #10 $
 * $DateTime: 2008/06/06 14:58:09 $
 *
 */
@Stateless(name = "WorkerService")
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public class WorkerServiceBean implements WorkerService {
	
	@EJB
	WorkerComponent mWorkerComponent;
		
	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(WorkerServiceBean.class);
	
    /**
     * No-arg constructor.
     */
    public WorkerServiceBean() {
    }
    
    public Worker getWorkerById(Long uniqueId) 
    		throws ServiceFailureException, ValidationException {
    	
        return GetWorkerByIdImpl.getWorkerById(uniqueId);
    }

    /* (non-Javadoc)
     * @see com.intuit.sb.payroll.services.worker.WorkerService#getWorkers(com.intuit.sb.payroll.services.worker.GetWorkersFilter)
     */
    public Worker[] getWorkers(GetWorkersFilter filter) throws ServiceFailureException, ValidationException {
    	return GetWorkersImpl.getWorkers(mWorkerComponent, filter);
    }
    
    public Worker[] getWorkersByWorkLocation(Long uniqueId) 
    		throws ServiceFailureException, ValidationException {
    	
    	return GetWorkersImpl.getWorkersByWorkLocation(mWorkerComponent, uniqueId);
    }

    @SuppressWarnings("nls")
    public Worker saveWorker(Worker worker) 
    		throws ServiceFailureException, ValidationException, WorkflowException {
    	
    	if (worker != null && sLogger.isDebugEnabled()) {
    		sLogger.debug("[saveWorker Input: "+ worker.toString() + "]");
    	}
        Worker saveWorkerResponse = SaveWorkerImpl.saveWorker(mWorkerComponent, worker);
        if (sLogger.isDebugEnabled() && saveWorkerResponse != null) {
        	sLogger.debug("[saveWorker Output: " + saveWorkerResponse.toString() + "]");
        }
        return saveWorkerResponse;
    }
    
    /*
     * (non-Javadoc)
     * @see com.intuit.sb.payroll.services.worker.WorkerService#deleteWorker(java.lang.Long)
     */
    public void deleteWorker(Long workerId) throws ServiceFailureException, ValidationException{
		CommonValidators.validateId(workerId);
    	com.intuit.sb.payroll.entity.model.Worker worker = mWorkerComponent.findWorker(workerId);
    	mWorkerComponent.deleteWorker(worker);
    }
    
    /*
     * (non-Javadoc)
     * @see com.intuit.sb.payroll.services.worker.WorkerService#terminateWorker(com.intuit.sb.payroll.services.worker.Worker, java.util.Date)
     */
	public Worker terminateWorker(Worker worker, Date terminationDate) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(worker, "worker");
		PayrollParamValidator.checkIsNotNull(terminationDate, "terminationDate");
		ObjectHandle workerUniqueId = worker.getObjectHandle();
		if (workerUniqueId == null || workerUniqueId.getId() == 0L) {
			throw new ValidationException("Cannot terminate a new worker.  Hire them first!");
		} else {
			CommonValidators.validateObjectHandle(workerUniqueId);
			Identifiable identifiable = ObjectHandleMapper.mapToIdentifiable(workerUniqueId);
			com.intuit.sb.payroll.entity.model.Worker workerEntity = mWorkerComponent.findWorkerByIdentifiable(identifiable);
			workerEntity = mWorkerComponent.terminateWorker(workerEntity, terminationDate);
			Worker terminatedWorker = WorkerMapper.mapFromEntityToPojo(workerEntity);
			return terminatedWorker;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.worker.WorkerService#hireExistingWorker(java.lang.Long, java.util.Date)
	 */
	public Worker hireExistingWorker(Long workerId, Date hireDate) throws ValidationException, WorkflowException, ServiceFailureException {
		PayrollParamValidator.checkIsNotNull(workerId, "workerId");
		PayrollParamValidator.checkIsNotNull(hireDate, "hireDate");

		com.intuit.sb.payroll.entity.model.Worker workerEntity = mWorkerComponent.hireExistingWorker(workerId, hireDate);
		Worker rehiredWorker = WorkerMapper.mapFromEntityToPojo(workerEntity);
		return rehiredWorker;

	}

    public boolean isWorkerReadyForPay(Worker worker) throws ValidationException {
        try {
        	com.intuit.sb.payroll.entity.model.Worker workerEntity = SaveWorkerImpl.setEntityWorkerProperties(worker, new Employee());
            boolean result = mWorkerComponent.isWorkerReadyToPay(workerEntity);
            return result;
        } catch (Exception e) {
            // should never happen as we are calculating the same
            return false;
        } finally {
        }
    }
}
