package com.intuit.sb.payroll.services.workerpayments;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import com.intuit.sb.payroll.services.common.ObjectHandle;

/**
 * The properties that define the schedule for a payroll run. This might be a single run or a recurring set of runs as
 * governed by the <code>payrollPayDateOffset</code> and <code>frequency</code> properties.
 */
public class PayrollSchedule implements Serializable
{

	private static final long serialVersionUID = -4715322282162984857L;

	// periodStartDate is the starting date of the time period of work, the worker will get paid for.
    private Date periodStartDate;

    // periodEndDate is the end date of the time period of work, the worker will get paid for
    private Date periodEndDate;

    // This is offset number of days between Payroll Run Date and Pay Period End Date This offset will determine the
    // Payroll Run Date that the user will normally run their payroll. On this date, the user run payroll, create
    // paychecks with payDate PayrollRunDate = PeriodEndDate + payrollRunDateOffset
    private int payrollRunDateOffset;

    // This is offset number of days between the payroll run date and payDate. PayDate can be calculated from
    // payrollPayDateOffset, payDate = Payroll RunDate + payrollPayDateOffset
    private int payrollPayDateOffset;

    // This is the interval on which the user normally pays their employees
    private PayrollScheduleFrequency frequency;

    // Payroll Schedule object handle
    private ObjectHandle objectHandle;

    // PayrollSchedule description
    private String memo;

    /**
     * Used only for Monthly and SemiMonthly PayrollSchedule Frequency. Represents the pay Period End day of the month,
     * 1st, 16th, etc. Allowed values are 1 - 28 & LAST_DAY_OF_MONTH (any day later than 28th is treated LAST DAY OF THE
     * MONTH)
     */
    private int payPeriodEndDayofMonth1;

    // Used only for SemiMonthly PayrollSchedule Frequency. Represents the second pay Period End day of the month, 1st,
    // 16th, etc. Allowed values are 1 - 28 & LAST_DAY_OF_MONTH (any day later than 28th is treated LAST DAY OF THE
    // MONTH)
    private int payPeriodEndDayofMonth2;
    
    /**
	 * If there are many schedules, one of them may be designated as the default one.
	 */
	private boolean isDefault;

    /**
     * This represent Last Day or any Month. Any day values more than 28 should be considered as Last Day of the Month
     */
    public static int LAST_DAY_OF_MONTH = 100;

    /**
     * Default constructor.
     */
    public PayrollSchedule()
    {
        super();
    }

    /**
     * Returns the Pay Period Start Date {@code periodStartDate} is the starting date of the time period of work, the
     * worker will get paid for.
     *
     * @return the periodStartDate
     */
    public Date getPeriodStartDate()
    {
        return periodStartDate;
    }

    /**
     * Set the Pay Period Start Date {@code periodStartDate} is the starting date of the time period of work, the worker
     * will get paid for.
     *
     * @param periodStartDate the periodStartDate to set
     */
    public void setPeriodStartDate(Date periodStartDate)
    {
        this.periodStartDate = periodStartDate;
    }

    /**
     * Get the Pay Period End Date {@code periodEndDate} is the end date of the time period of work, the worker will get
     * paid for
     *
     * @return the periodEndDate
     */
    public Date getPeriodEndDate()
    {
        return periodEndDate;
    }

    /**
     * Set the Pay Period End Date {@code periodEndDate} is the end date of the time period of work, the worker will get
     * paid for
     *
     * @param periodEndDate the periodEndDate to set
     */
    public void setPeriodEndDate(Date periodEndDate)
    {
        this.periodEndDate = periodEndDate;
    }

    /**
     * Set the PayrollRunDate offSet
     * <p>
     * PayrollRunDateOffset is offset number of days between Payroll Run Date and Pay Period End Date.<br>
     * This offset will determine the Payroll Run Date that the user will normally run their payroll. On this date, the
     * user run payroll, create paychecks with <code>payDate</code> PayrollRunDate = PeriodEndDate +
     * payrollRunDateOffset
     *
     * @param payrollRunDayOffset the payrollRunDate offset value to set
     */
    public void setPayrollRunDateOffset(int payrollRunDayOffset)
    {
        this.payrollRunDateOffset = payrollRunDayOffset;
    }

    /**
     * Get the PayrollRunDate offSet
     * <p>
     * PayrollRunDateOffset is offset number of days between Payroll Run Date and Pay Period End Date.<br>
     * This offset will determine the Payroll Run Date that the user will normally run their payroll. On this date, the
     * user run payroll, create paychecks with <code>payDate</code> PayrollRunDate = PeriodEndDate +
     * payrollRunDateOffset
     *
     */
    public int getPayrollRunDateOffset()
    {
        return this.payrollRunDateOffset;
    }

    /**
     * Get the Payroll Pay Date Offset <br>
     * PayrollPayDateOffset is offset number of days between the payroll run date and payDate. PayDate can be calculated
     * from payrollPayDateOffset,<br>
     * payDate = Payroll RunDate + payrollPayDateOffset
     *
     * @return the payrollPayDateOffset
     */
    public int getPayrollPayDateOffset()
    {
        return payrollPayDateOffset;
    }

    /**
     * Set the Payroll Pay Date Offset <br>
     * PayrollPayDateOffset is offset number of days between the payroll run date and payDate. PayDate can be calculated
     * from payrollPayDateOffset,<br>
     * payDate = Payroll RunDate + payrollPayDateOffset
     *
     * @param payrollPayDateOffset the payrollPayDateOffset to set
     */
    public void setPayrollPayDateOffset(int payrollPayDateOffset)
    {
        this.payrollPayDateOffset = payrollPayDateOffset;
    }

    /**
     * Get the the interval on which the user normally pays their employees
     *
     * @return PayrollScheduleFrequency
     */
    public PayrollScheduleFrequency getFrequency()
    {
        return frequency;
    }

    /**
     * Set the the interval on which the user normally pays their employees
     *
     * @param frequency PayrollScheduleFrequency
     */
    public void setFrequency(PayrollScheduleFrequency frequency)
    {
        this.frequency = frequency;
    }


    /**
     * @return the objectHandle
     */
    public ObjectHandle getObjectHandle() {
        return objectHandle;
    }

    /**
     * @param objectHandle the objectHandle to set
     */
    public void setObjectHandle(ObjectHandle objectHandle) {
        this.objectHandle = objectHandle;
    }

    /**
     * Get the PeriodEndDayofMonth1 PeriodEndDayofMonth1 is Used only for Monthly and SemiMonthly PayrollSchedule
     * Frequency.<br>
     * Represents the pay Period End day of the month, 1st, 16th, etc. <br>
     * Allowed values are 1 - 28 & LAST_DAY_OF_MONTH (any day later than 28th is treated LAST DAY OF THE MONTH)
     *
     * @return int representing PayPeriodEnd Of Month
     */
    public int getPayPeriodEndDayofMonth1()
    {
        return payPeriodEndDayofMonth1;
    }

    /**
     * Set the PeriodEndDayofMonth1 PeriodEndDayofMonth1 is Used only for Monthly and SemiMonthly PayrollSchedule
     * Frequency.<br>
     * Represents the pay Period End day of the month, 1st, 16th, etc. <br>
     * Allowed values are 1 - 28 & LAST_DAY_OF_MONTH (any day later than 28th is treated LAST DAY OF THE MONTH)
     *
     * @param payPeriodEndDayofMonth1 PayPeriodEnd Of Month
     */

    public void setPayPeriodEndDayofMonth1(int payPeriodEndDayofMonth1)
    {
        this.payPeriodEndDayofMonth1 = payPeriodEndDayofMonth1;
    }

    /**
     * Get the PayPeriodEndDayofMonth2 PayPeriodEndDayofMonth2 is used only for SemiMonthly PayrollSchedule Frequency.
     * <br>
     * Represents the second pay Period End day of the month, 1st, 16th, etc. <br>
     * Allowed values are 1 - 28 & LAST_DAY_OF_MONTH (any day later than 28th is treated LAST DAY OF THE MONTH)
     *
     * @return payPeriodEndDayofMonth2 PayPeriodEnd Of Month for Semi Monthly Pay Frequency
     */
    public int getPayPeriodEndDayofMonth2()
    {
        return payPeriodEndDayofMonth2;
    }

    /**
     * Set the PayPeriodEndDayofMonth2 PayPeriodEndDayofMonth2 is used only for SemiMonthly PayrollSchedule Frequency.
     * <br>
     * Represents the second pay Period End day of the month, 1st, 16th, etc. <br>
     * Allowed values are 1 - 28 & LAST_DAY_OF_MONTH (any day later than 28th is treated LAST DAY OF THE MONTH)
     *
     * @param payPeriodEndDayofMonth2 PayPeriodEnd Of Month for Semi Monthly Pay Frequency
     */
    public void setPayPeriodEndDayofMonth2(int payPeriodEndDayofMonth2)
    {
        this.payPeriodEndDayofMonth2 = payPeriodEndDayofMonth2;
    }


    public String getPayrollScheduleMemo()
    {
        return memo;
    }


    public void setPayrollScheduleMemo(String memo)
    {
        this.memo = memo;
    }
    
    /**
     * This boolean method is determined if the Payroll Schedule is default one or not
     * @return
     */
    public boolean isDefaultt() {
		return isDefault;
	}

	public void setDefaultt(boolean isDefault) {
		this.isDefault = isDefault;
	}

    @Override
    public boolean equals(Object other)
    {
        return EqualsBuilder.reflectionEquals(this, other);
    }

    @Override
    public int hashCode()
    {
       return HashCodeBuilder.reflectionHashCode(this);
    }

	@Override
	public String toString()
	{
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}

	
}
