package com.intuit.sb.payroll.services.workerpayments.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.CalculationInputs;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest;
import com.intuit.sb.payroll.entity.model.CompensationDtl;
import com.intuit.sb.payroll.entity.model.DateRange;
import com.intuit.sb.payroll.entity.model.Datum;
import com.intuit.sb.payroll.entity.model.Money;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.PtdDatumType;
import com.intuit.sb.payroll.entity.model.PtdTypeEnum;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.impl.CompensationItemMapper;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckLineItem;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;

/**
 * Utility class to
 * <ul>
 * <li>Convert Services PaycheckRequest to CalcRequest</li>
 * <li>Convert CalcRequest PaycheckDetails to Services PaycheckDetails</li>
 * <li>Create a Paycheck from PaycheckDetails and CompensationItems</li>
 * </ul>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
class PaycheckMapper {
	
	private PaycheckMapper() {}
    
	static PaycheckDetails mapPaycheckToPaycheckDetails(Paycheck paycheck) {
		PaycheckDetails checkDetails = new PaycheckDetails();
		checkDetails.setDistributionMethod(PayrollDistributionMethodMapper.mapEntityToInterface(
				paycheck.getDistributionMethod()));
		checkDetails.setGrossTotal(paycheck.getGrossAmount().getAmount());
		checkDetails.setNetTotal(paycheck.getNetAmount().getAmount());
		checkDetails.setPaycheckId(paycheck.getId());
		checkDetails.setCheckNumber(paycheck.getTxnNo());
		checkDetails.setMemo(paycheck.getMemo());
		checkDetails.setWorkerId(paycheck.getWorker().getId());
		checkDetails.setPaycheckStatus(PayrollRunMapper.mapPaycheckWorkflowStatus(paycheck.getWorkflowStatus()));
        
        Set<com.intuit.sb.payroll.entity.model.PaycheckLineItem> entityPaycheckLineItemSet = 
        	paycheck.getPaycheckLineItems();

        List<PaycheckLineItem> paycheckLineItemList = new ArrayList<PaycheckLineItem>(entityPaycheckLineItemSet.size());
        for (com.intuit.sb.payroll.entity.model.PaycheckLineItem entityPaycheckLineItem : entityPaycheckLineItemSet) {
        	PaycheckLineItem lineItem = mapPaycheckLineItem(entityPaycheckLineItem);
            paycheckLineItemList.add(lineItem);
        }
        PaycheckLineItem[] eiPaycheckLineItemArray = paycheckLineItemList.toArray(new PaycheckLineItem[paycheckLineItemList.size()]);
        checkDetails.setPaycheckLineItemsList(eiPaycheckLineItemArray);
		return checkDetails;
	}

	static PaycheckLineItem mapPaycheckLineItem(com.intuit.sb.payroll.entity.model.PaycheckLineItem entityPaycheckLineItem) {
		PaycheckLineItem lineItem = new PaycheckLineItem();
		lineItem.setTitle(entityPaycheckLineItem.getName());
		lineItem.setLawId(entityPaycheckLineItem.getLawId().getValue());
		lineItem.setPayrollItemCategory(
				PayrollItemCategoryTypeMapper.mapFromDEntityToEI(entityPaycheckLineItem.getSortSequence()));
		PayrollItemDtl sourcePayrollItemDtl = entityPaycheckLineItem.getSourcePayrollItemDtl();
		lineItem.setPayrollItemId(sourcePayrollItemDtl.getId());
		if (sourcePayrollItemDtl instanceof CompensationDtl) {      					  
			CompensationDtl sourceCompensationDtl = (CompensationDtl)sourcePayrollItemDtl;
			CompensationItem baseCompensationItem = CompensationItemMapper.mapToCompensationItem(sourceCompensationDtl);
			//FIXME [LinkedCompensation]
			if (sourceCompensationDtl.getLinkedPayrollItemDtl() != null &&
					sourceCompensationDtl.getLinkedPayrollItemDtl() instanceof CompensationDtl) {
				CompensationDtl linkedCompensationDtl = (CompensationDtl)sourceCompensationDtl.getLinkedPayrollItemDtl();
				CompensationItem linkedCompensationItem = CompensationItemMapper.mapToCompensationItem(linkedCompensationDtl);
				baseCompensationItem.setLinkedCompensationItem(linkedCompensationItem);
			}
			lineItem.setCompensationItem(baseCompensationItem);
		}
		  
		Money lineItemAmount = entityPaycheckLineItem.getValue();
		if (lineItemAmount != null) {
			lineItem.setAmount(lineItemAmount.getAmount());
		}
		
		//Quantity
		Datum qtyDatum = entityPaycheckLineItem.getQuantity();
		if (qtyDatum != null) {
			Double qty = qtyDatum.getValue1AsQuantity();
			if (qty != null) {
				lineItem.setUnits(qty.doubleValue());
			}
		}
		
		//YTD
		PtdDatumType ytdDatum = entityPaycheckLineItem.getYtdDatum().getDatum();
		if (ytdDatum != null) {
			BigDecimal ptdAmount = ytdDatum.getMoneyValue().getAmount();
			if (ptdAmount != null && ytdDatum.getPtdType()==PtdTypeEnum.YEAR) {
				lineItem.setYtd(ptdAmount);
			}
		}
		
		//Rate
		Datum rateDatum = entityPaycheckLineItem.getRate();
		if (rateDatum != null) {
			BigDecimal rate = rateDatum.getValue1AsUSDMoneyAmount();
			if (rate != null) {
				lineItem.setRate(rate);
			}
		}
		
		return lineItem;
	}
    

	/**
	 * New API for calcs, to be integrated later.
	 * 
	 * @param calcRequest
	 * @return
	 * @throws ValidationException
	 */
	public static PaycheckCalculationRequest mapPaycheckRequestToPaycheckCalculationRequest(PaycheckRequest calcRequest) throws ValidationException {
    	DateRange dateRange = new DateRange(calcRequest.getPayrollRunPeriod().getPeriodStartDate(), calcRequest.getPayrollRunPeriod().getPeriodEndDate());
        
        CalcInputs[] compensationItems = calcRequest.getCalcInputs();
        List<CalculationInputs> calculationInputs = createCalculationInputsFromCalcInputs(compensationItems);
        
    	PaycheckCalculationRequest capabilityRequest = new PaycheckCalculationRequest();
    	capabilityRequest.setWorkerId(calcRequest.getWorkerId());
        capabilityRequest.setPayPeriod(dateRange);
        capabilityRequest.setPaycheckDate(calcRequest.getPayrollRunPeriod().getPayDate());
        capabilityRequest.setFrequency(PayrollFrequencyMapper.mapFromEIToDEntity(
        		calcRequest.getPayrollRunPeriod().getFrequency()));
        capabilityRequest.setCalculationInputs(calculationInputs);
        capabilityRequest.setDistributionMethod(PayrollDistributionMethodMapper.mapInterfaceToEntity(calcRequest.getDistributionMethod()));
        capabilityRequest.setPartialPayPeriod(calcRequest.getPayrollRunPeriod().isPartialPeriod());
        
        return capabilityRequest;
	}

	/**
	 * New API for calcs, to be integrated later.
	 * 
	 * @param calcInputs
	 * @return
	 * @throws ValidationException
	 */
	static List<CalculationInputs> createCalculationInputsFromCalcInputs(CalcInputs[] calcInputs) throws ValidationException {
		List<CalculationInputs> results = new ArrayList<CalculationInputs>(calcInputs.length);
		for (CalcInputs calcInput : calcInputs) {
			if (calcInput.getRate() == null) {
				throw new ValidationException("Compensation Item rate cannot be null for fixed type");
			}
			CalculationInputs calculationInputs = new CalculationInputs();
        	Datum calcRate = new Datum();
			calcRate.setValue1AsUSDMoneyAmount(calcInput.getRate());
        	calculationInputs.setRate(calcRate);
        	
        	Datum calcQty = new Datum();
    		calcQty.setValue1AsQuantity(calcInput.getUnits());	
        	calculationInputs.setQuantity(calcQty);
        	calculationInputs.setSourcePayrollItemDtlId(calcInput.getPayItemObjectHandle().getId());
        	results.add(calculationInputs);
		}
		return results;
	}
		

}
