package com.intuit.sb.payroll.services.workerpayments.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.capability.worker.WorkerComponent;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponent;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.model.Paycheck;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.services.common.impl.CommonValidators;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunUpdateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollSchedule;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService;

/**
 * Implementation of WorkerPaymentService interface.
 * 
 * $Author$ $Revision$ $DateTime$
 */
@Stateless(name = "WorkerPaymentService")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class WorkerPaymentServiceBean implements WorkerPaymentService {

	@EJB
	private PayrollScheduleComponent payrollScheduleComponent;

	@EJB
	private WorkerPaymentsComponent workerPaymentsComponent;
	
	@EJB
	private WorkerComponent workerComponent;

	private static final Logger sLogger = PayrollLoggerFactory.makeLogger(WorkerPaymentServiceBean.class);

	/**
	 * {@inheritDoc}
	 */
	public PayrollSchedule getPayrollSchedule() throws ServiceFailureException {
		try {
			List<com.intuit.sb.payroll.entity.model.PayrollSchedule> payrollSchedules = payrollScheduleComponent
					.getAllPayrollSchedules();
			 
			// TakeHome application will have only one PayrollSchedule throw exception if thats not the case
			if (payrollSchedules.size() == 1){
				PayrollSchedule payrollSchedule = PayrollScheduleMapper.mapFromDEntityToEI(payrollSchedules.get(0));

				//setting the payrollschedule to match with the Next payrollRun Dates
				PayrollRunInfo payRollRunInfo = getNextScheduledPayrollRunInfo();
				payrollSchedule.setPeriodStartDate(payRollRunInfo.getPayrollRunPeriod().getPeriodStartDate());
				payrollSchedule.setPeriodEndDate(payRollRunInfo.getPayrollRunPeriod().getPeriodEndDate());
				
				return payrollSchedule;
			}else{
				return null;
			}
			
		} catch (Exception pe) {
			sLogger.error("Error occured while retriving PayrollSchedule", pe);
			throw new ServiceFailureException(pe);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	public PayrollSchedule createPayrollSchedule(PayrollSchedule payrollSchedule) throws ValidationException,
			ServiceFailureException {
		PayrollScheduleValidator.validatePayrollSchedule(payrollSchedule);
		
		com.intuit.sb.payroll.entity.model.PayrollSchedule entitySchedule = PayrollScheduleMapper
				.mapFromEIToDEntity(payrollSchedule);
		entitySchedule = payrollScheduleComponent.createPayrollSchedule(entitySchedule);
		payrollScheduleComponent.assignPayrollScheduleToEmployer(entitySchedule);
		PayrollSchedule result = PayrollScheduleMapper.mapFromDEntityToEI(entitySchedule);
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	public PayrollSchedule updatePayrollSchedule(PayrollSchedule payrollSchedule) throws ValidationException,
			ServiceFailureException {
		PayrollScheduleValidator.validatePayrollSchedule(payrollSchedule);
		
		com.intuit.sb.payroll.entity.model.PayrollSchedule entitySchedule = PayrollScheduleMapper
				.mapFromEIToDEntity(payrollSchedule);
		entitySchedule = payrollScheduleComponent.updatePayrollSchedule(entitySchedule);
		payrollScheduleComponent.assignPayrollScheduleToEmployer(entitySchedule);
		PayrollSchedule result = PayrollScheduleMapper.mapFromDEntityToEI(entitySchedule);
		return result;
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService#getNextScheduledPayrollRunInfo()
	 */
	public PayrollRunInfo getNextScheduledPayrollRunInfo() throws ValidationException, ServiceFailureException {
		com.intuit.sb.payroll.entity.model.PayrollSchedule payrollSchedule = payrollScheduleComponent.getDefaultPayrollSchedule();
		if (null != payrollSchedule) {
			com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo nextScheduledPayrollRunInfo = payrollScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
			PayrollRunInfo response = PayrollRunMapper.mapPayrollRunInfo(nextScheduledPayrollRunInfo);
			return response;
		} else {
			return null;
		}
	}
		
	/**
	 * {@inheritDoc}
	 */
	public PaycheckDetails calcPaycheckDetails(PaycheckRequest request) throws ValidationException, WorkflowException,
			ServiceFailureException {

		PaycheckRequestValidator.validate(request);
		if (sLogger.isTraceEnabled()) {
			sLogger.trace("[calcPaycheckDetails Input: " + request.toString() + "]");
		}

		com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest capabilityRequest = PaycheckMapper
				.mapPaycheckRequestToPaycheckCalculationRequest(request);

		com.intuit.sb.payroll.entity.model.Paycheck paycheck = workerPaymentsComponent.calculatePaycheck(capabilityRequest);

		PaycheckDetails result = PaycheckMapper.mapPaycheckToPaycheckDetails(paycheck);
		if (sLogger.isTraceEnabled()) {
			sLogger.trace("[calcPaycheck Output: " + result.toString() + "]");
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService#getPaycheckById(java.lang.Long)
	 */
	public PaycheckDetails getPaycheckById(Long paycheckId) throws ServiceFailureException, ValidationException {
		CommonValidators.validateId(paycheckId);
		com.intuit.sb.payroll.entity.model.Paycheck paycheck = workerPaymentsComponent.getPaycheckById(paycheckId);
		PaycheckDetails result = PaycheckMapper.mapPaycheckToPaycheckDetails(paycheck);
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	public PayrollRunSummary getPayrollRunSummaryById(Long payrollRunId) throws ValidationException,
			ServiceFailureException {
		CommonValidators.validateId(payrollRunId);
		PayrollRunSummary summary = null;
		com.intuit.sb.payroll.entity.model.PayrollRun payrollRun = workerPaymentsComponent.findPayrollRun(payrollRunId);
		summary = PayrollRunMapper.mapPayrollRunToPayrollRunSummary(payrollRun);
		return summary;
	}

	/**
	 * {@inheritDoc}
	 * TODO next default should be it's own API.
	 */
	public PayrollRunResponse getPayrollRunById(Long payrollRunId) throws ValidationException, ServiceFailureException {
		CommonValidators.validateId(payrollRunId);
		PayrollRunResponse response = null;
		com.intuit.sb.payroll.entity.model.PayrollRun payrollRun = null;
		if (payrollRunId == 0) {
			payrollRun = workerPaymentsComponent.getNextDefaultScheduledPayrollRun();
		} else {
			payrollRun = workerPaymentsComponent.findPayrollRun(payrollRunId);
		}
		if (payrollRun != null) {
			response = PayrollRunMapper.mapPayrollRunToPayrollRunResponse(payrollRun);
		}
		return response;
	}

    /* (non-Javadoc)
     * @see com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService#getRecentScheduledPayrollRuns(java.lang.Long)
     */
    public PayrollRunInfo[] getRecentScheduledPayrollRuns(int numberOfRuns) throws ServiceFailureException, ValidationException
    {
    	if (numberOfRuns <= 0) {
    		throw new ValidationException("numberOfRuns must be greater than zero.");
    	}

		List<PayrollRunInfo> responses = new ArrayList<PayrollRunInfo>();
		List<com.intuit.sb.payroll.entity.model.PayrollRun> payrollRuns = null;
		payrollRuns = workerPaymentsComponent.findRecentPayrollRuns(PayrollRunTypeEnum.SCHEDULED, numberOfRuns);
		if (payrollRuns != null) {
			for (com.intuit.sb.payroll.entity.model.PayrollRun payrollRun : payrollRuns) {
				PayrollRunInfo payrollRunInfo = PayrollRunMapper.mapPayrollRunToPayrollRunInfo(payrollRun);
				responses.add(payrollRunInfo);
			}
		}
		return responses.toArray(new PayrollRunInfo[responses.size()]);
    }

    /*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService#getAllPayrollRuns()
	 */
	public PayrollRunInfo[] getAllPayrollRuns() throws ServiceFailureException {
		List<PayrollRunInfo> responses = new ArrayList<PayrollRunInfo>();
		List<com.intuit.sb.payroll.entity.model.PayrollRun> payrollRuns = null;
		payrollRuns = workerPaymentsComponent.findAllPayrollRuns();
		if (payrollRuns != null) {
			for (com.intuit.sb.payroll.entity.model.PayrollRun payrollRun : payrollRuns) {
				PayrollRunInfo dude = PayrollRunMapper.mapPayrollRunToPayrollRunInfo(payrollRun);
				responses.add(dude);
			}
		}
		return responses.toArray(new PayrollRunInfo[responses.size()]);
	}

	/**
	 * {@inheritDoc}
	 */
	public PayrollRunSummary saveDraftPayrollRun(PayrollRunSaveRequest payrollRunRequest) throws ValidationException {
		throw new ValidationException("Not Supported Yet, sorry");
	}

	/**
	 * {@inheritDoc}
	 */
	public PayrollRunSaveResponse commitPayrollRun(PayrollRunSaveRequest payrollRunRequest) throws ValidationException,
			ServiceFailureException, WorkflowException {
		PayrollRunSaveRequestValidator.validate(payrollRunRequest);
		com.intuit.sb.payroll.capability.workerpayments.PayrollRunCommitRequest capabilityRequest = PayrollRunMapper
				.mapPayrollRunSaveRequestToPayrollRunRequest(payrollRunRequest);
		com.intuit.sb.payroll.entity.model.PayrollRun payrollRun = workerPaymentsComponent.commitPayrollRun(capabilityRequest);
		PayrollRunSummary payrollRunSummary = PayrollRunMapper.mapPayrollRunToPayrollRunSummary(payrollRun);
		
		/*
		 * Terminate any workers that are part of a termination run.
		 * We have to do this in two loops because the terminate call somehow
		 * modifies the payrollRun paychecks collection resulting in a
		 * ConcurrentModificationException on the for loop if we do it in one loop.
		 */
		Set<Paycheck> paychecks = payrollRun.getPaychecks();
		if (payrollRunRequest.getPayrollRunType() == PayrollRunTypeEnum.TERMINATION) {
			List<com.intuit.sb.payroll.entity.model.Worker> workersToTerm = new ArrayList<com.intuit.sb.payroll.entity.model.Worker>();
			for (Paycheck paycheck : paychecks) {
				workersToTerm.add(paycheck.getWorker());
			}
			for (com.intuit.sb.payroll.entity.model.Worker worker : workersToTerm) {
				workerComponent.terminateWorker(worker, payrollRunRequest.getTerminationDate());
			}
		}
		/*
		 * Now that the workers have been paid and terminated (if needed)
		 * they're in their final form and can be mapped for returning.
		 * So, for the third time in this method, loop over the paychecks/workers...
		 */
		Worker[] workers = new Worker[paychecks.size()];
		int i = 0;
		for (Paycheck paycheck : paychecks) {
			workers[i++] = WorkerMapper.mapFromEntityToPojo(paycheck.getWorker());
		}
		
    	//FIXME use @EJB refs
		PayrollRunSaveResponse payrollRunSaveResponse = new PayrollRunSaveResponse(payrollRunSummary, getPayrollSchedule(), workers);
		return payrollRunSaveResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService#updatePayrollRun(com.intuit.sb.payroll.services
	 * .workerpayments.PayrollRunUpdateRequest)
	 */
	public PayrollRunSummary updatePayrollRun(PayrollRunUpdateRequest updateRequest) throws ValidationException,
			ServiceFailureException, WorkflowException {
		PayrollRunSaveRequestValidator.validate(updateRequest);
		com.intuit.sb.payroll.capability.workerpayments.PayrollRunUpdateRequest capabilityRequest = PayrollRunMapper
				.mapPayrollRunUpdateRequest(updateRequest);
		com.intuit.sb.payroll.entity.model.PayrollRun payrollRun = workerPaymentsComponent.updatePayrollRun(capabilityRequest);
		PayrollRunSummary payrollRunSummary = PayrollRunMapper.mapPayrollRunToPayrollRunSummary(payrollRun);
		return payrollRunSummary;
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService#voidPaychecks(com.intuit.sb.payroll.services.workerpayments.VoidPaychecksRequest)
	 */
	public PayrollRunSummary voidPaychecks(Long[] paycheckIds)
			throws ValidationException, ServiceFailureException,
			WorkflowException {
		
		VoidPaychecksRequestValidator.validate(paycheckIds);
		List<Long> paycheckIdsList = PayrollRunMapper
				.mapVoidPaychecksRequest(paycheckIds);
		com.intuit.sb.payroll.entity.model.PayrollRun payrollRun = workerPaymentsComponent.voidPaychecks(paycheckIdsList);
		PayrollRunSummary payrollRunSummary = PayrollRunMapper.mapPayrollRunToPayrollRunSummary(payrollRun);
		return payrollRunSummary;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void cancelPayrollRunById(Long payrollRunId) throws ValidationException, ServiceFailureException {
		CommonValidators.validateId(payrollRunId);
		workerPaymentsComponent.cancelPayrollRunById(payrollRunId);
	}

	/*
	 * (non-Javadoc)
	 * @see com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService#getPayrollRunCSVById(java.lang.Long)
	 */
	public String getPayrollRunCSVById(Long payrollRunId) throws ValidationException, ServiceFailureException {
		PayrollRun payrollRun = workerPaymentsComponent.findPayrollRun(payrollRunId);
		String csv = PayrollRunReports.createCSV(payrollRun);
		return csv;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public PayrollRunResponse createPayrollRun(PayrollRunCreateRequest payrollRunCreateRequest)
			throws ValidationException, ServiceFailureException {
		
		PayrollRunSaveRequestValidator.validate(payrollRunCreateRequest);
		com.intuit.sb.payroll.capability.workerpayments.PayrollRunCreateRequest capabilityCreateRequest = 
			PayrollRunMapper.mapPayrollRunCreateRequest(payrollRunCreateRequest);
		com.intuit.sb.payroll.entity.model.PayrollRun payrollRun = workerPaymentsComponent.createPayrollRun(capabilityCreateRequest);
		PayrollRunResponse payrollRunResponse = PayrollRunMapper.mapPayrollRunToPayrollRunResponse(payrollRun);
		return payrollRunResponse;
	}
}
