package com.intuit.sb.payroll.task;

import java.util.Date;

import javax.ejb.Remote;

import com.intuit.sb.payroll.ServiceFailureException;

@Remote
public interface TaskService {
	
	public static final String JNDI_NAME = "TaskService/remote";
	
	/**
	 * Returns a collection of Tasks that is relevant to the current context.  Context
	 * in this case means the company/user that is logged in.  It will typically include
	 * up-coming tasks for a certain date range, and some recently completed tasks up to
	 * a certain date range.
	 * 
	 * The thresholds will be configured by the offering, and possibly overridden by the user.
	 * @return a collection of Tasks that meet the configured criteria.
	 */
	public Task[] getCurrentTasks() throws ServiceFailureException;
	
	/**
	 * Some Tasks require human gestures to mark as complete.  If permitted, the user can 
	 * set a Task as complete.  If the task is a System task then a WorkflowException may be 
	 * thrown.
	 * 
	 * @param task
	 * @return the Task with any state changes applied to it.
	 * @throws ServiceFailureException if this task cannot be set to the Complete State.
	 */
	public Task markTaskComplete(Task task) throws ServiceFailureException;
	
	/**
	 * Dummy send mail
	 * @throws ServiceFailureException
	 */
	public void sendDummyEmail() throws ServiceFailureException;	

	/**
	 * Returns a collection of Tasks that is relevant to the current context.  Context
	 * in this case means the company/user that is logged in.  It will typically include
	 * for the date range specified. The tasks returned will include even the canceled and closed tasks. 
	 * 

	 * @return a collection of Tasks that meet the configured criteria.
	 */
	public Task[] getTasks(Date fromdate, Date toDate) throws ServiceFailureException;
	
	/**
	 * Returns a collection of Tasks that is relevant to the current context and which are active in the system.
	 * Context in this case means the company/user that is logged in.  Active - means only 'New' and 'In Progress' tasks.
	 * 

	 * @return a collection of Tasks that meet the configured criteria.
	 */
	public Task[] getActiveTasks(Date fromdate, Date toDate) throws ServiceFailureException;
	
	/**
	 * Returns a collection of Tasks that are past due date and is relevant to the current context.
	 * Obviously only Active tasks in the system are returned.
	 * Context in this case means the company/user that is logged in.  
	 * Active - means only 'New' and 'In Progress' tasks.

	 * @return an array of overdue Tasks that meet the configured criteria. The array could be empty if there are no
	 * overdue tasks
	 */
	public Task[] getOverdueTasks() throws ServiceFailureException;
	
	/**
	 * Some tasks could be canceled by the user, like a scheduled payroll run. If the task can not be 
	 * canceled by user because it is a system created task, then WorkFlowWxception may be thrown
	 * @param task
	 * @return
	 */
	public Task cancelTask(Task task) throws ServiceFailureException;

	/**
	 * Creates a new user generated task in the system - an example will be a Reminder task
	 * @param task
	 * @return
	 * @throws ServiceFailureException
	 */
	public Task createTask(Task task) throws ServiceFailureException;
}
