package com.intuit.sb.payroll.task.impl;

import java.util.Date;
import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.employer.impl.EmployerComponentTasks;
import com.intuit.sb.payroll.capability.task.TaskManager;
import com.intuit.sb.payroll.notification.ContentFormatType;
import com.intuit.sb.payroll.notification.EmailRecipient;
import com.intuit.sb.payroll.notification.PayrollNotificationService;
import com.intuit.sb.payroll.notification.template.EmailTemplate;
import com.intuit.sb.payroll.notification.template.RunPayrollReminderTemplate;
import com.intuit.sb.payroll.task.Task;
import com.intuit.sb.payroll.task.TaskService;
import com.intuit.sb.payroll.task.TaskStatus;

@Stateless(name = "TaskService")
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class TaskServiceBean implements TaskService {

	@EJB
	private TaskManager taskManager;

	/**
	 * Returns a collection of Tasks that is relevant to the current context. Context in this case means the
	 * company/user that is logged in. It will typically include up-coming tasks for a certain date range, and some
	 * recently completed tasks up to a certain date range.
	 * 
	 * The thresholds will be configured by the offering, and possibly overridden by the user.
	 * 
	 * @return a collection of Tasks that meet the configured criteria.
	 */
	public Task[] getCurrentTasks() throws ServiceFailureException {
		Task[] tasks;
		List<com.intuit.sb.payroll.entity.model.Task> taskDTOs = taskManager.findCurrentActiveTasks();

		taskDTOs = adjustCompanySetupTasks(taskDTOs);

		tasks = new Task[taskDTOs.size()];
		int i = 0;
		for (com.intuit.sb.payroll.entity.model.Task taskDTO : taskDTOs) {
			tasks[i] = TaskMapper.mapFromDEntityToEI(taskDTO);
			i++;
		}
		return tasks;
	}

	private List<com.intuit.sb.payroll.entity.model.Task> adjustCompanySetupTasks(
			List<com.intuit.sb.payroll.entity.model.Task> taskDTOs) {
		// If the list of tasks contains company setup task then remove the
		// Add worklocation and update payroll Schedule
		com.intuit.sb.payroll.entity.model.Task companySetupTask = getCompanySetupTask(taskDTOs);
		com.intuit.sb.payroll.entity.model.Task payrollScheduleTask = null;
		com.intuit.sb.payroll.entity.model.Task addworklocationTask = null;

		if (companySetupTask != null) {

			for (com.intuit.sb.payroll.entity.model.Task task : taskDTOs) {
				if (task.getName().equals(EmployerComponentTasks.WORKLOCATION_SETUP_TASK)) {
					addworklocationTask = task;
				} else if (task.getName().equals(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK)) {
					payrollScheduleTask = task;
				}
			}
		}
		if (payrollScheduleTask != null) {
			taskDTOs.remove(payrollScheduleTask);
		}
		if (addworklocationTask != null) {
			taskDTOs.remove(addworklocationTask);
		}
		return taskDTOs;
	}

	private com.intuit.sb.payroll.entity.model.Task getCompanySetupTask(
			List<com.intuit.sb.payroll.entity.model.Task> taskDTOs) {
		for (com.intuit.sb.payroll.entity.model.Task task : taskDTOs) {
			if (task.getName().equals(EmployerComponentTasks.COMPANY_SETUP_TASK)) {
				return task;
			}
		}
		return null;
	}

	/**
	 * Returns a collection of Tasks that is relevant to the current context. Context in this case means the
	 * company/user that is logged in. It will typically include for the date range specified. The tasks returned will
	 * include even the canceled and closed tasks.
	 * 
	 * @return an array of Tasks that meet the configured criteria. The contents of the array could be empty if no Tasks
	 *         found in the system matching the criteria.
	 */
	public Task[] getTasks(Date fromdate, Date toDate) throws ServiceFailureException {
		Task[] tasks;
		List<com.intuit.sb.payroll.entity.model.Task> taskDTOs = taskManager.getTasks(fromdate, toDate);
		tasks = new Task[taskDTOs.size()];
		int i = 0;
		for (com.intuit.sb.payroll.entity.model.Task taskDTO : taskDTOs) {
			tasks[i] = TaskMapper.mapFromDEntityToEI(taskDTO);
			i++;
		}
		return tasks;
	}

	/**
	 * Returns a collection of Tasks that is relevant to the current context and which are active in the system. Context
	 * in this case means the company/user that is logged in. Active - means only 'New' and 'In Progress' tasks.
	 * 
	 * 
	 * @return a collection of Tasks that meet the configured criteria.
	 */
	public Task[] getActiveTasks(Date fromdate, Date toDate) throws ServiceFailureException {
		Task[] activeTasks;
		List<com.intuit.sb.payroll.entity.model.Task> taskDTOs = taskManager.getActiveTasks(fromdate, toDate);
		activeTasks = new Task[taskDTOs.size()];
		int i = 0;
		for (com.intuit.sb.payroll.entity.model.Task taskDTO : taskDTOs) {
			activeTasks[i] = TaskMapper.mapFromDEntityToEI(taskDTO);
			i++;
		}
		return activeTasks;
	}

	/**
	 * Returns a collection of Tasks that are past due date and is relevant to the current context. Obviously only
	 * Active tasks in the system are returned. Context in this case means the company/user that is logged in. Active -
	 * means only 'New' and 'In Progress' tasks.
	 * 
	 * @return an array of overdue Tasks that meet the configured criteria. The array could be empty if there are no
	 *         overdue tasks
	 */
	public Task[] getOverdueTasks() throws ServiceFailureException {

		Task[] overdueTasks;
		List<com.intuit.sb.payroll.entity.model.Task> taskDTOs = taskManager.getOverdueTasks();
		overdueTasks = new Task[taskDTOs.size()];
		int i = 0;
		for (com.intuit.sb.payroll.entity.model.Task taskDTO : taskDTOs) {
			overdueTasks[i] = TaskMapper.mapFromDEntityToEI(taskDTO);
			i++;
		}
		return overdueTasks;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.task.TaskService#markTaskComplete(com.intuit.sb.payroll.task.Task)
	 */
	public Task markTaskComplete(Task task) throws ServiceFailureException {
		if (task == null || task.getObjectHandle() == null) {
			throw new ServiceFailureException("Can not close a task which does not exist in the system"); // TODO - move
																											// to
																											// resource
																											// bundle
		}
		long taskId = task.getObjectHandle().getId();
		if (taskManager.isUserManagable(taskId)) {
			com.intuit.sb.payroll.entity.model.Task taskDo = taskManager.closeTask(taskId);
			task = TaskMapper.mapFromDEntityToEI(taskDo);
		} else {
			throw new ServiceFailureException("Can not change the state of this Task"); // TODO [rj] resource bundle
		}
		return task;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.task.TaskService#cancelTask(com.intuit.sb.payroll.task.Task)
	 */
	public Task cancelTask(Task task) throws ServiceFailureException {
		if (task == null || task.getObjectHandle() == null) {
			throw new ServiceFailureException("Can not cancel a task which does not exist in the system"); // TODO -
																											// move to
																											// resource
																											// bundle
		}
		long taskId = task.getObjectHandle().getId();
		if (taskManager.isUserManagable(taskId)) {
			com.intuit.sb.payroll.entity.model.Task taskDo = taskManager.cancelTask(taskId);
			task = TaskMapper.mapFromDEntityToEI(taskDo);
		} else {
			throw new ServiceFailureException("Can not change the state of this Task"); // TODO [rj] resource bundle
		}
		return task;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.task.TaskService#createTask(com.intuit.sb.payroll.task.Task)
	 */
	public Task createTask(Task task) throws ServiceFailureException {

		if (task == null) {
			throw new ServiceFailureException("Can not create a null task"); // TODO - move this to a resource bundle
		}
		// always set the status to new, since we are going to create a 'New' task in the system
		task.setStatus(TaskStatus.NEW);
		com.intuit.sb.payroll.entity.model.Task taskDo = TaskMapper.mapFromEIToDEntity(task);
		taskDo = taskManager.addTask(taskDo); // this serializes the task in the system
		task = TaskMapper.mapFromDEntityToEI(taskDo);
		return task;
	}

	/*
	 * This is a temp service exposed to test Email Notifications from the UI (non-Javadoc)
	 * 
	 * @see com.intuit.sb.payroll.task.TaskService#sendDummyEmail()
	 */
	public void sendDummyEmail() throws ServiceFailureException {

		PayrollNotificationService notificationWSClient = PayrollNotificationService.getInstance();

		EmailRecipient emailRecipient = new EmailRecipient();
		emailRecipient.setEmailAddress("avnish_malik@intuit.com");
		emailRecipient.setFormat(ContentFormatType.HTML);
		emailRecipient.setRecipientId("amalik");
		emailRecipient.setRecipientName("Avnish Malik");

		EmailRecipient emailRecipient1 = new EmailRecipient();
		emailRecipient1.setEmailAddress("dheerendra_kulkarni@intuit.com");
		emailRecipient1.setFormat(ContentFormatType.HTML);
		emailRecipient1.setRecipientId("dkulkarni");
		emailRecipient1.setRecipientName("Dheerendra Kulkarni");

		EmailRecipient emailRecipient2 = new EmailRecipient();
		emailRecipient2.setEmailAddress("raviiyer100@gmail.com");
		emailRecipient2.setFormat(ContentFormatType.HTML);
		emailRecipient2.setRecipientId("riyer");
		emailRecipient2.setRecipientName("Ravi Iyer");

		EmailRecipient emailRecipient3 = new EmailRecipient();
		emailRecipient3.setEmailAddress("Avnish.Malik@gmail.com");
		emailRecipient3.setFormat(ContentFormatType.HTML);
		emailRecipient3.setRecipientId("amalik");
		emailRecipient3.setRecipientName("Avnish Malik");

		EmailTemplate emailTemplate = new RunPayrollReminderTemplate();
		emailTemplate.setProperty(RunPayrollReminderTemplate.PRIMARY_CONTACT_NAME, "AVNISH_MALIK");
		emailTemplate.setProperty(RunPayrollReminderTemplate.PRIMARY_CONTACT_EMAIL, "avnish_malik@intuit.com");
		emailTemplate.setProperty(RunPayrollReminderTemplate.PAY_PERIOD_STARTDATE, "10th Oct 2009");
		emailTemplate.setProperty(RunPayrollReminderTemplate.PAY_PERIOD_ENDDATE, "17th Oct 2009");
		emailTemplate.setProperty(RunPayrollReminderTemplate.PAYROLL_RUN_DATE_MINUS_1, "16th Oct 2009");
		emailTemplate.setProperty(RunPayrollReminderTemplate.PAYROLL_RUN_DATE_MINUS_2, "15th Oct 2009");
		emailTemplate.setProperty(RunPayrollReminderTemplate.SEND_DATE, "14 Oct 2008");
		try {
			notificationWSClient.sendMail(emailRecipient, emailTemplate);

			notificationWSClient.sendMail(emailRecipient1, emailTemplate);
			notificationWSClient.sendMail(emailRecipient2, emailTemplate);
			notificationWSClient.sendMail(emailRecipient3, emailTemplate);

		} catch (Exception e) {
			throw new ServiceFailureException(e);
		}

	}
}
