package com.intuit.sb.payroll.services.taxes;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;



/**
 *  <P>The <CODE>ApplyApplicableTaxesSanMateoTest</CODE> class contains test methods for getting applicable tax items for San Mateo</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class ApplyApplicableTaxesSanMateoTest extends TaxesTestBase
{
	@BeforeClass
	public static void setUpApplyApplicableTaxesSanMateo()
	{
        try
        {
            getUserTransaction().begin();
    		saveEmployerInfo();
            setUpSanMateoAddress();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUpApplyApplicableTaxesSanMateo(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}
	
	@AfterClass
	public static void tearDownApplyApplicableTaxesSanMateo()
	{
        try
        {
            getUserTransaction().begin();
            restoreEmployerInfo();
            getUserTransaction().commit();
        }
        catch (Throwable t)
        {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in tearDownApplyApplicableTaxesSanMateo(): " + t.getMessage());
            try
            {
                getUserTransaction().rollback();
            }
            catch (Exception e)
            {
            }
        }
	}

    /************* Test cases for employee not related to the employer *************/
    
    /**
     * <P>Tests applicable payroll items for non profit organization in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
	@Test
    public void testNonProfit501c3InSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "CA Personal Income Tax", "CA State Disability Insurance", "CA Employment Training Tax",
            "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for church in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testNonProfit501c3ChurchInSanMateoAge22()throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "CA Personal Income Tax", "CA Employment Training Tax"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for C-Corp in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to C-Corp</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testC_CorpInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "Federal Unemployment Insurance", "CA Personal Income Tax", "CA State Disability Insurance",
            "CA Employment Training Tax", "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for S-Corp in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to S-Corp</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testS_CorpInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "Federal Unemployment Insurance", "CA Personal Income Tax", "CA State Disability Insurance", "CA Employment Training Tax",
            "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for LLC in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to LLC</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testLLCInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.LLC);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "Federal Unemployment Insurance", "CA Personal Income Tax", "CA State Disability Insurance", "CA Employment Training Tax",
            "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for Partnership in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to Partnership</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testPartnershipInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
        sEmployer.setNonProfit(false);
        
       // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "Federal Unemployment Insurance", "CA Personal Income Tax", "CA State Disability Insurance", "CA Employment Training Tax",
            "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for SoleProprietor in San Mateo, CA; employee is not related to employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to SoleProprietor</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to NotRelated</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testSoleProprietorInSanMateoAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "Federal Unemployment Insurance", "CA Personal Income Tax", "CA State Disability Insurance", "CA Employment Training Tax",
            "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
                                    expectedPayrollItems);
    }
    
    /************* Test cases for employee being a partner of the employer *************/
    
    /**
     * <P>Tests applicable payroll items for non profit organization in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testNonProfit501c3InSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "CA Personal Income Tax", "CA State Disability Insurance", "CA Employment Training Tax",
            "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for church in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to 501c3</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testNonProfit501c3ChurchInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
        sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
        sEmployer.setNonProfit(true);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "CA Personal Income Tax", "CA Employment Training Tax"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for C-Corp in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to C-Corp</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testC_CorpInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "Federal Unemployment Insurance", "CA Personal Income Tax", "CA State Disability Insurance", "CA Employment Training Tax",
            "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for S-Corp in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to S-Corp</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testS_CorpInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {"Federal Income Tax", "Medicare Tax (Employee)", "Medicare Tax (Employer)", "Social Security Tax (Employee)", "Social Security Tax (Employer)",
            "Federal Unemployment Insurance", "CA Personal Income Tax", "CA State Disability Insurance", "CA Employment Training Tax",
            "CA State Unemployment Insurance"};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for LLC in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to LLC</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testLLCInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.LLC);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for Partnership in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to Partnership</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testPartnershipInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedPayrollItems);
    }
    
    /**
     * <P>Tests applicable payroll items for SoleProprietor in San Mateo, CA; and employee is partner of employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set company legal structure to SoleProprietor</LI>
     * <LI>Set work location in San Mateo, CA</LI>
     * <LI>Set up list of expected payroll items</LI>
     * <LI>Set relationship to partner</LI>
     * <LI>Set birthdate to age 22</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testSoleProprietorInSanMateoForPartnerAge22() throws Exception
    {
		// Setup
		//   - Implicit setup performed in setUpClass() and setUp()
		//   - Additional setup
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
        sEmployer.setNonProfit(false);
        
        // Set up expected list of Payroll items
        String[] expectedPayrollItems = {};
        
        performApplyApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
                                    expectedPayrollItems);
    }
    
}
