package com.intuit.sb.payroll.services.taxes;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;

/**
 * <P>
 * The <CODE>GetApplicableTaxesTexasTest</CODE> class contains test methods for getting applicable taxes in Texas
 * </P>
 * 
 * <DL>
 * <DT><B>Requirements Document:</B></DT>
 * <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d"
 * >Gemini requirements</A></DD>
 * </DL>
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class GetApplicableTaxesTexasTest extends TaxesTestBase {

	@BeforeClass
	public static void setUpGetApplicableTaxesTexas() {
		try {
			getUserTransaction().begin();
			saveEmployerInfo();
			setUpTexasAddress();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in setUpGetApplicableTaxesTexas(): " + t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
		}
	}

	@AfterClass
	public static void tearDownGetApplicableTaxesTexas() {
		try {
			getUserTransaction().begin();
			restoreEmployerInfo();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in tearDownGetApplicableTaxesTexas(): " + t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
		}
	}

	/************* Test cases for employee not related to the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to NotRelated</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to NotRelated</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to NotRelated</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to NotRelated</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to NotRelated</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to NotRelated</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; employee is not related to employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to NotRelated</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED,
				expectedTaxes);
	}

	/************* Test cases for employee being a partner of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = {};

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = {};

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is partner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to partner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = {};

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.OWNER,
				expectedTaxes);
	}

	/************* Test cases for employee being a Spouse of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is Spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is Spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is Spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is Spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is Spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is Spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is Spouse of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Spouse</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForSpouseAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.SPOUSE,
				expectedTaxes);
	}

	/************* Test cases for employee being a DomesticPartner of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is DomesticPartner of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to DomesticPartner</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForDomesticPartnerAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate,
				RelationshipToOwnerEnum.REGISTERED_DOMESTIC_PARTNER, expectedTaxes);
	}

	/************* Test cases for employee being a Child of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 22</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForChildAge22() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 21</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForChildAge21() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge21BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 20</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForChildAge20() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge20BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	// TODO: Tests for Child, age < 21, legalStructure in (Partnership, LLC), All Partners = Parents of Child

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 18</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForChildAge18() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge18BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is Child of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Child</LI>
	 * <LI>Set birthdate to age 17</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForChildAge17() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge17BirthDate, RelationshipToOwnerEnum.CHILD,
				expectedTaxes);
	}

	// TODO: Tests for Child, age < 18, legalStructure in (Partnership, LLC), All Partners = Parents of Child

	/************* Test cases for employee being a Parent of the employer *************/

	/**
	 * <P>
	 * Tests applicable taxes for non profit organization in Mesquite, TX; and employee is Parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3InTexasForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for church in Mesquite, TX; and employee is Parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to 501c3</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testNonProfit501c3ChurchInTexasForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.CHURCH);
		sEmployer.setLegalStructure(LegalStructureEnum.STATUS501C3);
		sEmployer.setNonProfit(true);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for C-Corp in Mesquite, TX; and employee is Parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to C-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testC_CorpInTexasForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for S-Corp in Mesquite, TX; and employee is Parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to S-Corp</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testS_CorpInTexasForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SCORP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for LLC in Mesquite, TX; and employee is Parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to LLC</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testLLCInTexasForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.LLC);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for Partnership in Mesquite, TX; and employee is Parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to Partnership</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testPartnershipInTexasForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.PARTNERSHIP);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE",
				"US-FED-FUI", "US-TX-SUI" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	/**
	 * <P>
	 * Tests applicable taxes for SoleProprietor in Mesquite, TX; and employee is Parent of employer
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set company legal structure to SoleProprietor</LI>
	 * <LI>Set work location in Mesquite, TX</LI>
	 * <LI>Set up list of expected taxes</LI>
	 * <LI>Set relationship to Parent</LI>
	 * <LI>Set birthdate to age 40</LI>
	 * <LI>Call utility routine to perform the actual work</LI>
	 * </OL>
	 * 
	 */
	@Test
	public void testSoleProprietorInTexasForParentAge40() throws Exception {
		// Setup
		// - Implicit setup performed in setUpClass() and setUp()
		// - Additional setup
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.SOLEPROPRIETOR);
		sEmployer.setNonProfit(false);

		// Set up expected list of taxes
		String[] expectedTaxes = { "US-FED-FIT", "US-FED-MEDT", "US-FED-MEDT-EE", "US-FED-SST", "US-FED-SST-EE" };

		performGetApplicableTaxes(sEmployer, sPhysicalAddress, sAge40BirthDate, RelationshipToOwnerEnum.PARENT,
				expectedTaxes);
	}

	// TODO: Tests for Parent, legalStructure = Partnership, All Partners = Siblings

}
