package com.intuit.sb.payroll.services.taxes;

import java.util.Calendar;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;

import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.capability.taxes.withholding.TaxWithholdingsComponent;
import com.intuit.sb.payroll.capability.taxes.withholding.impl.WorkerTaxWithholdingRules;
import com.intuit.sb.payroll.common.ResourceKeyConstants;
import com.intuit.sb.payroll.entity.model.ComplianceDoc;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.Law;
import com.intuit.sb.payroll.entity.model.PayrollItem;
import com.intuit.sb.payroll.entity.model.PayrollItemDtl;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.wages.CompensationItemTestBase;
import com.intuit.sb.payroll.services.worker.WorkerService;

/**
 *  <P>The <CODE>TaxesTestBase</CODE> class contains helper methods for testing Taxes related services</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class TaxesTestBase extends CompensationItemTestBase {
	
	protected static TaxComponent sTaxComponent;
	protected static WorkerService sWorkerService;
	protected static TaxWithholdingsService sTaxWithholdingsService;
	protected static TaxWithholdingsComponent sTaxWithholdingsComponent;
	
	public static void initializeEAOs()
	{
		sTaxComponent = 
			qaBootstrap.getEjbFromClass(TaxComponent.class);
		sWorkerService = 
			qaBootstrap.getEjbFromClass(WorkerService.class);
		sTaxWithholdingsService = 
			qaBootstrap.getEjbFromClass(TaxWithholdingsService.class);
		sTaxWithholdingsComponent = qaBootstrap.getEjbFromClass(TaxWithholdingsComponent.class);
	}
	
	@BeforeClass
	public static void setUpTaxes() {
		initializeEAOs();
        try {
            getUserTransaction().begin();
            saveWorkLocationAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
        	t.printStackTrace();
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpTaxes(): " + t.getMessage());
        }
	}
	
	@AfterClass
	public static void tearDownTaxes() {
        try {
            getUserTransaction().begin();
            restoreWorkLocationAddress();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownTaxes(): " + t.getMessage());
        }
	}
	
	private boolean verifyApplicableTaxes(List<Law> computedTaxes, String [] expectedTaxes) {
		
		if ( computedTaxes == null || expectedTaxes == null) {
			throw new RuntimeException("Null parameter");
		}
		
        boolean compareResult = (computedTaxes.size() == expectedTaxes.length);
        if (true == compareResult) {
            // Only compare the tax items if the number of taxes matches the expected number of taxes
    		HashSet<String> computedMnemonicSet = new HashSet<String>();
    		for(int i=0; i < computedTaxes.size(); i++) {
                computedMnemonicSet.add(computedTaxes.get(i).getName());
            } 
    		HashSet<String> expectedDescriptionSet = new HashSet<String>();
    		for(int i=0; i < expectedTaxes.length; i++) {
    			expectedDescriptionSet.add(expectedTaxes[i]);
            }
            compareResult = computedMnemonicSet.equals(expectedDescriptionSet);
        }
		
        if (false == compareResult) {
            sLogger.error("Computed Taxes:");            
            for(int i=0; i < computedTaxes.size(); i++) {
                sLogger.error(computedTaxes.get(i).getName());            
            }            
            sLogger.error("Expected Taxes:");            
            for(int i=0; i < expectedTaxes.length; i++) {
                sLogger.error(expectedTaxes[i].toString());            
            }            
        }
		
		return compareResult;
	}
	
    /**
     * <P>Verifies that the specified exception is a PayrollException containing ID_NOT_FOUND_EXCEPTION</P>
     *
     */
    protected boolean verifyIdNotFoundException(Exception ex) {
        String expectedMsg = sResourceManager.getString(ResourceKeyConstants.ID_NOT_FOUND_EXCEPTION);
        return (ex.getMessage().contains(expectedMsg));
    }
    
    /**
     * <P>Verifies that the specified exception is a PayrollException containing ID_NOT_FOUND_EXCEPTION</P>
     *
     */
    protected boolean verifyInvalidAgeException(Exception ex) {
        String expectedMsg = sResourceManager.getString(ResourceKeyConstants.INVALID_AGE_EXCEPTION);
        return (ex.getMessage().contains(expectedMsg));
    }
    
    /**
     * <P>Perform the actual work of getting the applicable taxes and verifying against the expected results</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call utility routine to set up the test data</LI>
     * <LI>Get the applicable taxes and verify against the expected results</LI>
     * </OL>
     *
     * <H3>Result: <FONT COLOR="GREEN">Success</FONT></H3>
     * <UL>
     * <LI>Verify list of taxes returned matches the expected list</LI>
     * </UL>
     */
    protected void performGetApplicableTaxes(Employer org, USPostalAddress address, Calendar dateOfBirth,
                                             RelationshipToOwnerEnum relationship, String[] expectedTaxes) throws Exception {       
    	
            setUpTestData(org, address, dateOfBirth, relationship);
            sWorker = entityService.findById(Worker.class, sWorkerId);
            List<Law> computedTaxes = sTaxComponent.determineApplicableTaxes(sWorker);
    
            // Verify the applicable taxes
            AssertJUnit.assertTrue("Applicable taxes mismatch!", verifyApplicableTaxes(computedTaxes, expectedTaxes));
    }
    
    protected boolean verifyApplicableTaxItems(Set<TaxDtl> computedPayrollItems, String [] expectedPayrollItems) {
    	
        Iterator<TaxDtl> iterator;
        if ( computedPayrollItems == null || expectedPayrollItems == null) {
            throw new RuntimeException("Null parameter");
        }
        
        boolean compareResult = (computedPayrollItems.size() == expectedPayrollItems.length);
        if (true == compareResult) {
            // Only compare the payroll items if the number of computed items matches the expected number of items
            HashSet<String> computedPayrollItemSet = new HashSet<String>();
            
            iterator = computedPayrollItems.iterator();
            while (iterator.hasNext()) {
                PayrollItemDtl itemDetail = iterator.next();
                PayrollItem item = itemDetail.getPayrollItem();
                computedPayrollItemSet.add(item.getDescription());
            } 
            HashSet<String> expectedPayrollItemSet = new HashSet<String>();
            for(int i=0; i < expectedPayrollItems.length; i++) {
                expectedPayrollItemSet.add(expectedPayrollItems[i]);
            }
            compareResult = computedPayrollItemSet.equals(expectedPayrollItemSet);
        }
        
        if (false == compareResult) {
            sLogger.error("Computed Taxes:");            
            iterator = computedPayrollItems.iterator();
            while (iterator.hasNext()) {
                PayrollItemDtl itemDetail = iterator.next();
                PayrollItem item = itemDetail.getPayrollItem();
                sLogger.error(item.getDescription());
            } 
            sLogger.error("Expected Taxes:");            
            for(int i=0; i < expectedPayrollItems.length; i++) {
                sLogger.error(expectedPayrollItems[i].toString());            
            }            
        }
        
        return compareResult;
    }
    
    /**
     * <P>Apply the applicable taxes to sWorker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call the component to apply the applicable taxes</LI>
     * </OL>
     *
     */
    protected static void applyApplicableTaxes() throws Exception {
        sWorker = entityService.findById(Worker.class, sWorkerId);
        // Get the worker, and hence creating a session (which is needed in serviceImpl.deriveStateWithholdingFromFederalWithholding) 
        sWorker = sTaxComponent.applyApplicableTaxes(sWorker);
        sWorker = sWorkerComponent.updateWorker(sWorker);
    }
    
    /**
     * <P>Perform the actual work of applying the applicable taxes and verifying against the expected results</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call utility routine to set up the test data</LI>
     * <LI>Apply the applicable taxes and verify against the expected results</LI>
     * </OL>
     *
     * <H3>Result: <FONT COLOR="GREEN">Success</FONT></H3>
     * <UL>
     * <LI>Verify list of taxes returned matches the expected list</LI>
     * </UL>
     */
    protected void performApplyApplicableTaxes(Employer org, USPostalAddress address, Calendar dateOfBirth,
    		RelationshipToOwnerEnum relationship, String[] expectedPayrollItems) throws Exception {       
    	
        setUpTestData(org, address, dateOfBirth, relationship);
        
        // Apply the applicable taxes
        applyApplicableTaxes();
        sWorker = entityService.findById(Worker.class, sWorkerId);
        Set<TaxDtl> computedPayrollItems = sWorker.getTaxDtls();

        // Verify the applicable payroll items
        boolean applicableTaxesMatched = verifyApplicableTaxItems(computedPayrollItems, expectedPayrollItems);
        AssertJUnit.assertTrue("Applicable tax items mismatch!", applicableTaxesMatched);
    }

    protected boolean verifyWithholdingForms(List<ComplianceDoc> computedWithholdingForms,
                                             String [] expectedWithholdingForms) {
    	
        Iterator<ComplianceDoc> iterator;
        if ( computedWithholdingForms == null || expectedWithholdingForms == null) {
            throw new RuntimeException("Null parameter");
        }
        
        boolean compareResult = (computedWithholdingForms.size() == expectedWithholdingForms.length);
        if (true == compareResult) {
            // Only compare the Withholding forms if the number of computed items matches the expected number of items
            HashSet<String> computedPayrollItemSet = new HashSet<String>();
            iterator = computedWithholdingForms.iterator();
            while (iterator.hasNext()) {
                ComplianceDoc item = iterator.next();
                computedPayrollItemSet.add(item.getName());
            } 
            HashSet<String> expectedPayrollItemSet = new HashSet<String>();
            for(int i=0; i < expectedWithholdingForms.length; i++)
            {
                expectedPayrollItemSet.add(expectedWithholdingForms[i]);
            }
            compareResult = computedPayrollItemSet.equals(expectedPayrollItemSet);
        }
        
        if (false == compareResult) {
            sLogger.error("Computed Withholding forms:");            
            iterator = computedWithholdingForms.iterator();
            while (iterator.hasNext()) {
                ComplianceDoc item = iterator.next();
                sLogger.error(item.getName());
            } 
            sLogger.error("Expected Withholding forms:");            
            for(int i=0; i < expectedWithholdingForms.length; i++) {
                sLogger.error(expectedWithholdingForms[i].toString());            
            }            
        }
        
        return compareResult;
    }
    
    /**
     * <P>Perform the actual work of applying the applicable taxes, 
     * getting withholding forms and verifying against the expected results</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call utility routine to set up the test data</LI>
     * <LI>Apply the applicable taxes and get withholding forms, then verify against the expected results</LI>
     * </OL>
     *
     * <H3>Result: <FONT COLOR="GREEN">Success</FONT></H3>
     * <UL>
     * <LI>Verify list of withholding forms returned matches the expected list</LI>
     * </UL>
     */
    protected void performGetWithholdingForms(Employer org, USPostalAddress address, Calendar dateOfBirth,
    			RelationshipToOwnerEnum relationship, String[] expectedWithholdingForms) throws Exception {

        setUpTestData(org, address, dateOfBirth, relationship);
        
        // Get the applicable taxes
        applyApplicableTaxes();
        sWorker = entityService.findById(Worker.class, sWorkerId);

        List<ComplianceDoc> computedWithholdingForms = 
        	WorkerTaxWithholdingRules.getWithholdingFormsApplicableToWorker(sWorker);
        
        // Verify the applicable withholding forms
        boolean withholdingFormsMatched = verifyWithholdingForms(computedWithholdingForms, expectedWithholdingForms);
        AssertJUnit.assertTrue("WithholdingForms mismatch!", withholdingFormsMatched);
    }

}
