package com.intuit.sb.payroll.services.taxes;

import java.math.BigDecimal;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4Detail;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;


/**
 *  <P>The <CODE>W4ToDE4Test</CODE> class contains test methods for testing the W4 to DE4 mapping</P>
 *
 *  <DL>
 *  <DT><B>Requirements Document:</B></DT>
 *  <DD><A HREF="http://spitfire.corp.intuit.net/sites/merlinpayroll/Shared%20Documents/Forms/AllItems.aspx?RootFolder=%2fsites%2fmerlinpayroll%2fShared%20Documents%2fRequirements&View=%7b146B0034%2d58AF%2d4E76%2d93B1%2dA00364A96B7E%7d">Gemini requirements</A></DD>
 *  </DL>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class W4ToDE4Test extends TaxesTestBase {
	@BeforeClass
	public static void setUpW4ToDE4() {
        try {
            getUserTransaction().begin();
    		saveEmployerInfo();
            // Make sure that the worker used by these tests have an address in CA
            setUpSanFranciscoAddress();
            setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
            applyApplicableTaxes();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUpW4ToDE4(): " + t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
        }
	}
	
	@AfterClass
	public static void tearDownW4ToDE4() {
        try {
            getUserTransaction().begin();
            removeAllWorkerPayrollItems(sWorkerId);
            restoreEmployerInfo();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in tearDownW4ToDE4(): " + t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
        }
	}
	
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
    }

    @AfterMethod
    public void tearDown() throws Exception {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
	        // Remove withholdingInfo associated with the employee
	        removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
	        removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
        }
        super.tearDown();
    }
    
    /**
     * <P>Tests the W4 to DE4 mapping using Federal Single, DE4 withholding info has not been setup yet</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up W4 withholding</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
	@Test
    public void testFederalSingleMapping() throws Exception {
        setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, 
        		W4WithholdingStatusEnum.SINGLE, new Integer(1), new BigDecimal("0.00"));
        
        performW4toDE4Mapping(sWorkerId, sDE4DocId, QboeLawIdConstants.sCAPITLawId, sNotExemptFromTax, 
        		DE4WithholdingStatusEnum.SINGLE, new Integer(1), new Integer(0), new BigDecimal("0.00"));
    }
    
    /**
     * <P>Tests the W4 to DE4 mapping using Federal Married, DE4 withholding info has not been setup yet</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up W4 withholding</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testFederalMarriedMapping() throws Exception {
        setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, 
        		W4WithholdingStatusEnum.MARRIED, new Integer(1), new BigDecimal("30.00"));
        
        performW4toDE4Mapping(sWorkerId, sDE4DocId, QboeLawIdConstants.sCAPITLawId, sNotExemptFromTax, 
        		DE4WithholdingStatusEnum.MARRIEDONEINCOME, new Integer(1), new Integer(0), new BigDecimal("30.00"));
    }
    
    /**
     * <P>Tests the W4 to DE4 mapping using Federal Married Using Single, DE4 withholding info has not been setup yet</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up W4 withholding</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testFederalMarriedUsingSingleMapping() throws Exception {
        setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, 
        		W4WithholdingStatusEnum.MARRIEDSINGLERATE, new Integer(2), new BigDecimal("0.00"));
        
        performW4toDE4Mapping(sWorkerId, sDE4DocId, QboeLawIdConstants.sCAPITLawId, sNotExemptFromTax, 
        		DE4WithholdingStatusEnum.SINGLE, new Integer(2), new Integer(0), new BigDecimal("0.00"));
    }
    
    /**
     * <P>Tests the W4 to DE4 mapping using Federal don't withhold, DE4 withholding info has not been setup yet</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up W4 withholding</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testFederalDontWithholdMapping() throws Exception {
        setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sExemptFromTax, 
        		W4WithholdingStatusEnum.MARRIEDSINGLERATE, new Integer(2), new BigDecimal("0.00"));
        performW4toDE4Mapping(sWorkerId, sDE4DocId, QboeLawIdConstants.sCAPITLawId, sExemptFromTax,
        		DE4WithholdingStatusEnum.SINGLE, new Integer(2), new Integer(0), new BigDecimal("0.00"));
    }
    
    /**
     * <P>Tests the W4 to DE4 mapping using Federal Single, DE4 withholding info has been setup already</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up W4 withholding</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testFederalSingleMappingWithExistingDE4() throws Exception {
        setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, 
        		W4WithholdingStatusEnum.SINGLE, new Integer(1), new BigDecimal("0.00"));
        performW4toDE4Mapping(sWorkerId, sDE4DocId, QboeLawIdConstants.sCAPITLawId, sNotExemptFromTax, 
        		DE4WithholdingStatusEnum.SINGLE, new Integer(1), new Integer(0), new BigDecimal("0.00"));
    }
    
    /**
     * <P>Tests the W4 to DE4 mapping using Federal Married, DE4 withholding info has been setup already</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up W4 withholding</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testFederalMarriedMappingWithExistingDE4() throws Exception {
        setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, 
        		W4WithholdingStatusEnum.MARRIED, new Integer(2), new BigDecimal("10.00"));
        performW4toDE4Mapping(sWorkerId, sDE4DocId, QboeLawIdConstants.sCAPITLawId, sNotExemptFromTax, 
        		DE4WithholdingStatusEnum.MARRIEDONEINCOME, new Integer(2), new Integer(0), new BigDecimal("10.00"));
    }
    
    /**
     * <P>Tests the W4 to DE4 mapping using Federal Married Using Single rate, DE4 withholding info has been setup already</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up W4 withholding</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testFederalMarriedUsingSingleMappingWithExistingDE4() throws Exception {
        setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, 
        		W4WithholdingStatusEnum.MARRIEDSINGLERATE, new Integer(2), new BigDecimal("10.00"));
        performW4toDE4Mapping(sWorkerId, sDE4DocId, QboeLawIdConstants.sCAPITLawId, sNotExemptFromTax, 
        		DE4WithholdingStatusEnum.SINGLE, new Integer(2), new Integer(0), new BigDecimal("10.00"));
    }
    
    /**
     * <P>Tests the W4 to DE4 mapping using Federal Don't withhold, DE4 withholding info has been setup already</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up W4 withholding</LI>
     * <LI>Call utility routine to perform the actual work</LI>
     * </OL>
     *
     */
    @Test
    public void testFederalDontWithholdMappingWithExistingDE4() throws Exception {
        setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sExemptFromTax, 
        		W4WithholdingStatusEnum.MARRIEDSINGLERATE, new Integer(2), new BigDecimal("10.00"));
        performW4toDE4Mapping(sWorkerId, sDE4DocId, QboeLawIdConstants.sCAPITLawId, sExemptFromTax, 
        		DE4WithholdingStatusEnum.SINGLE, new Integer(2), new Integer(0), new BigDecimal("10.00"));
    }
    
    /**
     * <P>Perform the actual work of mapping W4 to DE4 and verifying against the expected results</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call utility routine to set up the test data</LI>
     * <LI>Perform the W4 to DE4 mapping, and verify against the expected results</LI>
     * </OL>
     *
     * <H3>Result: <FONT COLOR="GREEN">Success</FONT></H3>
     * <UL>
     * <LI>Verify the mapping values match the expected values</LI>
     * </UL>
     */
    protected void performW4toDE4Mapping(Long workerId, String mappedDocId, String mappedLawId, 
    		boolean isIncomeTaxExcempt, DE4WithholdingStatusEnum withholdingStatus, Integer allowance,
            Integer deductions, BigDecimal additonalAmount) throws Exception {
    	
        // Get the worker, and hence creating a session 
    	// (which is needed in serviceImpl.deriveStateWithholdingFromFederalWithholding) 
        sWorker = entityService.findById(Worker.class, sWorkerId);
        List<WorkerComplianceDocDtl> computedWithholdingForms = sTaxWithholdingsComponent.getCurrentAndDerivedWithholdings(sWorker);
        
        // Verify the withholding info
        AssertJUnit.assertTrue("Withholding info mapping mismatch!", verifyDE4WithholdingInfo(computedWithholdingForms,
        		mappedDocId, mappedLawId, isIncomeTaxExcempt, withholdingStatus, allowance, 
        		deductions, additonalAmount));
    }
    
    private boolean verifyDE4WithholdingInfo(List<WorkerComplianceDocDtl> computedComplianceData,
    		String docId, String lawId, boolean isIncomeTaxExcempt, DE4WithholdingStatusEnum withholdStatus,
    		Integer allowance, Integer deductions, BigDecimal additonalAmount) {
    	
        boolean compareResult = false;
        DE4Detail eWithholdingData = null;
        for (WorkerComplianceDocDtl computedData : computedComplianceData) {
            if (computedData.getDocId() == ComplianceDocIdEnum.DE4) {
                eWithholdingData = (DE4Detail)computedData;
                break;
            }
        }
        
        if (null != eWithholdingData) {
            boolean withholdStatusMatched = eWithholdingData.getWithholdingStatus().equals(withholdStatus);
            boolean allowanceMatched = eWithholdingData.getNoOfAllowances().equals(allowance);
            boolean deductionsMatched = eWithholdingData.getNoOfDeductions().equals(deductions);
            boolean isIncomeTaxExcemptMatched = eWithholdingData.isTaxExempt() == isIncomeTaxExcempt;
            boolean additonalAmountMatched = 
            	(0 == eWithholdingData.getAdditionalAmount().getAmount().compareTo(additonalAmount));
            // TODO  Add verification for docId when that association is set up by the engine
            compareResult = withholdStatusMatched && allowanceMatched && deductionsMatched && isIncomeTaxExcemptMatched
                            && additonalAmountMatched;
            if (false == compareResult) {
                sLogger.error("Computed Withholding info:");            
                sLogger.error("\twithholdStatus: " + eWithholdingData.getWithholdingStatus().toString());
                sLogger.error("\tallowance: " + eWithholdingData.getNoOfAllowances());
                sLogger.error("\tdeductions: " + eWithholdingData.getNoOfDeductions());
                sLogger.error("\tisTaxExempt: " + eWithholdingData.isTaxExempt());
                sLogger.error("\tadditionalAmount: " + eWithholdingData.getAdditionalAmount().toString());
                sLogger.error("\tlawId: " + eWithholdingData.getLawId().getValue().toString());
                sLogger.error("Expected Withholding info:");            
                sLogger.error("\twithholdStatus: " + withholdStatus.toString());
                sLogger.error("\tallowance: " + allowance.toString());
                sLogger.error("\tdeductions: " + deductions.toString());
                sLogger.error("\tisTaxExempt: " + isIncomeTaxExcempt);
                sLogger.error("\tadditionalAmount: " + additonalAmount.toString());
                sLogger.error("\tlawId: " + lawId);
            }
        } else {
            sLogger.error("Cannot find compliance data for " + lawId);            
        }
        
        return compareResult;
    }
}
