package com.intuit.sb.payroll.services.worker;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.entity.model.WorkLocation;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Contains tests for testing WorkerService APIs
 *
 */
public class WorkerServiceTest extends WorkerServiceTestBase {

    @Test
    public void testGetWorkers() throws Exception {
        //create some workers
        List<Worker> workers = createSomeWorkers();

        
        // Get the initial # of workers in db
        Worker[] workersFromDb = sWorkerService.getWorkers(GetWorkersFilter.ALL);
        int initNoOfWorker = workersFromDb.length;
        
        //save these workers in db
        for (Worker worker: workers) {
            Worker savedWorker = sWorkerService.saveWorker(worker);
            worker.setObjectHandle(savedWorker.getObjectHandle());
        }

        //now get the workers from the db
        workersFromDb = sWorkerService.getWorkers(GetWorkersFilter.ALL);

        if((workers.size()+initNoOfWorker) != workersFromDb.length) {
           AssertJUnit.fail("The number of workers retrived does not match with the ones saved");
        }
        
        //TODO we may need to add more tests here fro checking the data 
        // retrieved for each worker matches with what we saved in db
        
        //cleanup the db from the data created for this test
        for (Worker worker:workers) {
            cleanup(worker.getObjectHandle().getId());
        }
    }
	
    @SuppressWarnings("unused")
    @Test(expectedExceptions = ValidationException.class)
    public void testGetWorkersWithNullFilter() throws Exception
    {
        Worker[] workersFromDb = sWorkerService.getWorkers(null);
    }
	
    @Test
    public void testGetWorkersByWorkLocationId() throws Exception
    {
        List<WorkLocation>  workLocations = entityService.findAll(WorkLocation.class);
        
        WorkLocation loc = workLocations.get(0);
        Worker[] workers = sWorkerService.getWorkersByWorkLocation(loc.getId());
        	
        AssertJUnit.assertNotNull(workers);
    }
	
    @SuppressWarnings("unused")
    @Test(expectedExceptions = ValidationException.class)
    public void testGetWorkersWithNullWorkLocationId() throws Exception
    {
	Worker[] workers = sWorkerService.getWorkersByWorkLocation(null);
    }

	// TODO: Move to some withholding test class
	@Test (expectedExceptions=WorkflowException.class)
	public void testSaveWithholdingsWithoutWorkLocation() throws Exception {
		Worker worker = createWorker();
		Worker savedWorker = sWorkerService.saveWorker(worker);
		WithholdingInformation withholding = createWithholdingInfo(savedWorker);
		SaveWithholdingInfoRequest request = new SaveWithholdingInfoRequest();
		request.setWithholdingInfo(withholding);
		request.setWorkerId(savedWorker.getObjectHandle().getId());
		request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
		sTaxWithholdingsService.saveWithholdingInformation(request);
		// No cleanup as the call fails and we never commit anything
	}

	@Test
    public void testUpdateWorker() throws Exception {
        Worker worker = null;

        try {
	        worker = createWorker();
	        PhysicalAddress taxResidenceAddress = PojoUtils.createUSCAPhysicalAddress();
	        worker.setTaxResidenceAddress(taxResidenceAddress );
	        worker.setWorkLocation(getWorkLocation());
	        Worker workerFromDb = sWorkerService.saveWorker(worker);

	        WithholdingInformation withholding = createWithholdingInfo(workerFromDb);
			SaveWithholdingInfoRequest request = new SaveWithholdingInfoRequest();
			request.setWithholdingInfo(withholding);
			request.setWorkerId(workerFromDb.getObjectHandle().getId());
			request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
			sTaxWithholdingsService.saveWithholdingInformation(request);
            
            AssertJUnit.assertTrue(workerFromDb.getFirstName().equals(worker.getFirstName()));
            
            worker = sWorkerService.getWorkerById(workerFromDb.getObjectHandle().getId());
            AssertJUnit.assertTrue(worker != null);
            //Now update the worker
            worker.setFirstName("Julia");
            worker.setLastName("Roberts");

            Worker updatedWorkerFromDb = sWorkerService.saveWorker(worker);
            
            AssertJUnit.assertTrue("Names differ: " + updatedWorkerFromDb.getFirstName() + 
            		" vs " + worker.getFirstName(), 
            		updatedWorkerFromDb.getFirstName().equals(worker.getFirstName()));
            //cleanup
            cleanup(updatedWorkerFromDb.getObjectHandle().getId());
        } catch (Exception e) {
        	sLogger.error("Error during test: ", e);
        	throw e;
        }
    }

	@Test(enabled=false, groups={"broken"})
	public void testWorkerLevelCompensationItems() throws Exception {
		//FIXME [th] why the heck does this work in eclipse but not mvn?
        //create a new worker
		Worker worker = createWorker(); //create a worker with minimum data

		// Test SaveWorker with no compensations
		Worker savedWorker = sWorkerService.saveWorker(worker);
        
        CompensationItem[] noCompensations = sWageService.getCompensationItems(
        		savedWorker.getObjectHandle().getId());
		AssertJUnit.assertEquals("Initial Compensation Items should be empty", 0, noCompensations.length);
		
		//create the wages 
		CompensationItem[] inputCompensations = createCompensationItems();

		// Test inserting compensations
		sWageService.saveCompensationItems(savedWorker.getObjectHandle().getId(), mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, inputCompensations));
		
		getUserTransaction().commit();
		//get the wage items from the db
		CompensationItem[] insertedCompensations = sWageService.getCompensationItems(savedWorker.getObjectHandle().getId());
		AssertJUnit.assertEquals("Number of compensation items does not match", 
				inputCompensations.length, insertedCompensations.length);
		// We don't control the order of the collection returned from the engine so we have to search for each CI
	    for (int i = 0; i < inputCompensations.length; i++) {
			boolean found = false;
	    	for (CompensationItem insertedCompensation : insertedCompensations) {
	    		// Set the PK in the DB object to null so we can compare the whole objects
		    	insertedCompensation.setObjectHandle(null);
		    	if (inputCompensations[i].equals(insertedCompensation)) {
	    			found = true;
	    			break;
	    		}
	    	}
            AssertJUnit.assertTrue("Compensation item not found in results set", found);
	    }
		// Refetch so we have the object handles
	    insertedCompensations = sWageService.getCompensationItems(savedWorker.getObjectHandle().getId());

		// Test updating compensations
		CompensationItem[] updateCompensations = createWageDefaults2(insertedCompensations);
        sWageService.saveCompensationItems(savedWorker.getObjectHandle().getId(), 
        		mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE, updateCompensations));
		
        //get the wage items from the db
		CompensationItem[] updatedCompensations = sWageService.getCompensationItems(savedWorker.getObjectHandle().getId());
	    AssertJUnit.assertEquals("Number of compensation items does not match", 
	    		insertedCompensations.length, updatedCompensations.length);
//	    for (int i = 0; i < updateCompensations.length; i++) {
//			// Refresh the update versions set by the DB into the original compensations 
//	    	// so we can compare the whole objects
//	    	updateCompensations[i].getObjectHandle().setVersion(updatedCompensations[i].getObjectHandle().getVersion());
//            AssertJUnit.assertEquals("Compensation items does not match", updateCompensations[i], updatedCompensations[i]);
//	    }
	    
	    // Negative test: try removing hourly without removing overtime
	    CompensationItem hourlyItem = null;
	    for (int i = 0; i < updatedCompensations.length; i++) {
	    	hourlyItem = updatedCompensations[i];
	    	if (hourlyItem.getCompensationType().equals(CompensationType.Hourly)) {
	    		break;
	    	}
	    }
	    
	    AssertJUnit.assertNotNull("Hourly compensation item not found", hourlyItem);
	    SaveCompensationItemRequest deleteRequest = new SaveCompensationItemRequest();
	    deleteRequest.setCompensationItemOperation(SaveCompensationItemOperation.DELETE);
	    deleteRequest.setCompensationItem(hourlyItem);
		SaveCompensationItemRequest[] deleteRequests = new SaveCompensationItemRequest[1];
		deleteRequests[0] = deleteRequest;
	    try {
	    	sWageService.saveCompensationItems(savedWorker.getObjectHandle().getId(), deleteRequests);
	    	AssertJUnit.fail("Expected ServiceFailureException not caught");
	    } catch (Exception sfe) {
	    	// Do nothing - expect this exception
	    	sLogger.info("Expected ServiceFailureException was thrown");
	    }
	    
	    // Have to start a transaction again since the transaction was rolled back due to the Exception above
		getUserTransaction().begin();
	    // re-fetch the wage items from the db
		updatedCompensations = sWageService.getCompensationItems(savedWorker.getObjectHandle().getId());
   
		// Test removing compensations
	    CompensationItem[] removedCompensations = new CompensationItem[] {
	    		updatedCompensations[0], updatedCompensations[1]}; 
	    sWageService.saveCompensationItems(savedWorker.getObjectHandle().getId(), 
	    		mapToSaveCompensationItemRequest(SaveCompensationItemOperation.DELETE, removedCompensations));

	    CompensationItem[] remainingWageItems = sWageService.getCompensationItems(savedWorker.getObjectHandle().getId());
		for (int i = 0; i < remainingWageItems.length; i++) {
			Long tmpId1 = new Long(remainingWageItems[i].getObjectHandle().getId());
			Long tmpId2 = new Long(updatedCompensations[i+2].getObjectHandle().getId());			
			if (tmpId1.compareTo(tmpId2) != 0) {
				AssertJUnit.fail("The Compensation Item removed should match");
			}
		}

		//cleanup the database from the objects created for this test
		cleanup(savedWorker.getObjectHandle().getId());
		getUserTransaction().commit();
	}
	
    private void cleanup(Long id) throws Exception {
        if (id != null) {
        	cleanUpEIWorker(id);
        }
    }
}
