package com.intuit.sb.payroll.services.worker;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.testng.AssertJUnit;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.taxes.FormType;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.CompensationUtils;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * <P>
 * The <CODE>WorkerServiceTestBase</CODE> class is the base class for the IWorkerService Interface tests.
 * 
 * $Author$ $Revision$ $DateTime$
 * 
 */
public class WorkerServiceTestBase extends EntityUtilsTestBase {

	/*
	 * Comparator for sorting compensation items.
	 */
	private static class CompensationItemComparator implements Comparator<CompensationItem> {
		public int compare(CompensationItem wageItem1, CompensationItem wageItem2) {
			if (wageItem1.getCompensationType().ordinal() == wageItem2.getCompensationType().ordinal()) {
				if (null != wageItem1.getObjectHandle() && (null != wageItem2.getObjectHandle())) {
					Long tmpId1 = new Long(wageItem1.getObjectHandle().getId());
					Long tmpId2 = new Long(wageItem2.getObjectHandle().getId());
					return tmpId1.compareTo(tmpId2);
				} else {
					if (null == wageItem1.getObjectHandle()) {
						return (null == wageItem2.getObjectHandle() ? 0 : -1);
					} else {
						return (null == wageItem2.getObjectHandle() ? 1 : 0);
					}
				}
			} else { // handle > and < cases
				return new Integer(wageItem1.getCompensationType().ordinal()).compareTo(wageItem2.getCompensationType()
						.ordinal());
			}
		}
	}

	protected boolean verifyCompensationItems(CompensationItem[] computedCompensationItems,
			List<CompensationItem> expectedCompensationItems) {
		if (null == computedCompensationItems || null == expectedCompensationItems) {
			throw new RuntimeException("Null parameter");
		}
		Set<CompensationItem> compensationItems = new TreeSet<CompensationItem>(new CompensationItemComparator());
		for (CompensationItem compensationItem : expectedCompensationItems) {
			compensationItems.add(compensationItem);
		}
		CompensationItem[] expectedWageItemsArray = compensationItems
				.toArray(new CompensationItem[expectedCompensationItems.size()]);
		boolean compareResult = (computedCompensationItems.length == expectedWageItemsArray.length);
		if (compareResult) {
			// Only compare the wage items if the number of wage items matches
			// the expected number
			for (int i = 0; i < computedCompensationItems.length; i++) {
				// Not comparing the ObjectHandle value, so set both fields to
				// null
				computedCompensationItems[i].setObjectHandle(null);
				expectedWageItemsArray[i].setObjectHandle(null);
				compareResult = computedCompensationItems[i].equals(expectedWageItemsArray[i]);
				if (!compareResult) {
					break;
				}
			}
		}

		if (false == compareResult) {
			sLogger.error("Computed CompensationItems:");
			for (int i = 0; i < computedCompensationItems.length; i++) {
				printCompensationItem(computedCompensationItems[i]);
			}
			sLogger.error("Expected CompensationItems:");
			for (int i = 0; i < expectedWageItemsArray.length; i++) {
				printCompensationItem(expectedWageItemsArray[i]);
			}
		}

		return compareResult;
	}

	protected boolean verifyCompensationItemsWithLinkedItems(CompensationItem[] computedCompensationItems,
			List<CompensationItem> expectedCompensationItems, String linkedType) {
		if (null == computedCompensationItems || null == expectedCompensationItems) {
			throw new RuntimeException("Null parameter");
		}
		Set<CompensationItem> compensationItems = new TreeSet<CompensationItem>(new CompensationItemComparator());
		for (CompensationItem compensationItem : expectedCompensationItems) {
			compensationItems.add(compensationItem);
		}
		CompensationItem[] expectedWageItemsArray = compensationItems
				.toArray(new CompensationItem[expectedCompensationItems.size()]);
		boolean compareResult = (computedCompensationItems.length == expectedWageItemsArray.length);
		if (compareResult) {
			// Only compare the wage items if the number of wage items matches
			// the expected number
			for (int i = 0; i < computedCompensationItems.length; i++) {
				// Not comparing the ObjectHandle value, so set both fields to
				// null
				computedCompensationItems[i].setObjectHandle(null);
				expectedWageItemsArray[i].setObjectHandle(null);

				AssertJUnit.assertEquals("WageItemsDontMatch", expectedWageItemsArray[i], computedCompensationItems[i]);

				// COMPARE IF LINKED ITEMS ARE ATTACHED TO COMPENSATION ITEM
				// Check the compensation type, if it is double or overtime then
				// get the Linked Compensation Item
				// and check if it is hourly
				// if the compensation is of type "Hourly" , skip
				CompensationItem hourlyItem = null;
				if (linkedType.equalsIgnoreCase("Both")) {
					// Check if compensation is of type Overtime, if yes, check
					// if getLinkedCompensation is of type Hourly
					if (CompensationType.Overtime.equals(computedCompensationItems[i].getCompensationType())) {
						hourlyItem = computedCompensationItems[i].getLinkedCompensationItem();
						if (null == hourlyItem) {
							compareResult = false;
							break;
						} else {
							if (CompensationType.Hourly.equals(hourlyItem.getCompensationType())) {
								compareResult = true;
							} else {
								compareResult = false;
								break;
							}
						}
					}
					if (CompensationType.Doubletime.equals(computedCompensationItems[i].getCompensationType())) {
						{
							hourlyItem = computedCompensationItems[i].getLinkedCompensationItem();
						}
						if (hourlyItem == null) {
							compareResult = false;
							break;
						} else {
							if (CompensationType.Hourly.equals(hourlyItem.getCompensationType())) {
								compareResult = true;
							} else {
								compareResult = false;
								break;
							}
						}
					}
				}
				if (linkedType.equalsIgnoreCase("Overtime")) {
					if (CompensationType.Overtime.equals(computedCompensationItems[i].getCompensationType())) {
						hourlyItem = computedCompensationItems[i].getLinkedCompensationItem();
						if (hourlyItem == null) {
							compareResult = false;
							break;
						} else {
							if (CompensationType.Hourly.equals(hourlyItem.getCompensationType()))
								compareResult = true;
							else {
								compareResult = false;
								break;
							}
						}
					}
				}
				if (linkedType.equalsIgnoreCase("Doubletime")) {
					if (CompensationType.Doubletime.equals(computedCompensationItems[i].getCompensationType())) {
						hourlyItem = computedCompensationItems[i].getLinkedCompensationItem();
						if (hourlyItem == null) {
							compareResult = false;
							break;
						} else {
							if (CompensationType.Hourly.equals(hourlyItem.getCompensationType()))
								compareResult = true;
							else {
								compareResult = false;
								break;
							}
						}
					}
				}
			}

			if (false == compareResult) {
				sLogger.error("Computed WageItem items:");
				for (int j = 0; j < computedCompensationItems.length; j++) {
					printCompensationItem(computedCompensationItems[j]);
				}
				sLogger.error("Expected WageItem items:");
				for (int j = 0; j < expectedWageItemsArray.length; j++) {
					printCompensationItem(expectedWageItemsArray[j]);
				}
			}
		}
		return compareResult;
	}

	protected Worker createWorker() {
		Worker worker = new Worker();
		worker.setFirstName("Reuben");
		worker.setLastName("John");
		worker.setHireDate(new Date()); // set current date
		worker.setBirthDate(new Date()); // set current date
		worker.setJobTitle("SW Engineer");
		worker.setWorkerType(WorkerType.EMPLOYEE);
		return worker;
	}

	protected List<Worker> createSomeWorkers() {
		List<Worker> workerList = new ArrayList<Worker>();

		Worker worker1 = new Worker();
		worker1.setFirstName("Joe");
		worker1.setLastName("Chan");
		worker1.setHireDate(new Date()); // set current date
		worker1.setJobTitle("Sr Manager");
		worker1.setWorkerType(WorkerType.EMPLOYEE);

		Worker worker2 = new Worker();
		worker2.setFirstName("Tim");
		worker2.setLastName("Hobson");
		worker2.setHireDate(new Date()); // set current date
		worker2.setJobTitle("SW Engineer");
		worker2.setWorkerType(WorkerType.EMPLOYEE);

		Worker worker3 = new Worker();
		worker3.setFirstName("Rajan");
		worker3.setLastName("Mahadevan");
		worker3.setHireDate(new Date()); // set current date
		worker3.setJobTitle("SW Engineer");
		worker3.setWorkerType(WorkerType.EMPLOYEE);

		Worker worker4 = new Worker();
		worker4.setFirstName("Brian");
		worker4.setLastName("Sterling");
		worker4.setHireDate(new Date()); // set current date
		worker4.setJobTitle("SW Engineer");
		worker4.setWorkerType(WorkerType.EMPLOYEE);

		workerList.add(worker1);
		workerList.add(worker2);
		workerList.add(worker3);
		workerList.add(worker4);

		return workerList;
	}

	protected WorkLocation getWorkLocation() throws Exception {
		// get the work locations from db
		WorkLocation[] locations = sEmployerService.getWorkLocations();
		if (locations.length == 0) {
			throw new Exception("Worklocations not available in db");
		}
		return locations[0];
	}

	protected CompensationItem[] createWageDefaults2(CompensationItem[] old) {
		CompensationItem[] wageItems = new CompensationItem[1];

		wageItems[0] = old[0]; // hourly
		wageItems[0].setRate(new BigDecimal("500.0"));

		return wageItems;
	}

	protected WithholdingInformation createWithholdingInfo(Worker worker) throws ServiceFailureException,
			ValidationException {
		WithholdingInformation w4 = createEIWithholdingInformation(worker, FormType.US_W4);
		w4.setExtraWithholding(new BigDecimal("500.00"));

		return w4;
	}

	/**
	 * This utility methods compares 2 Arrays of Workers for equality.
	 * 
	 * @param expectedWorkers
	 *            Array of <CODE>com.intuit.sb.payroll.services.worker.Worker</CODE> Entities
	 * @param computedWorkers
	 *            Array of <CODE>com.intuit.sb.payroll.services.worker.Worker</CODE> Entities
	 * @return True if computedWorker matches expectedWorker
	 */
	protected boolean verifyEIWorkers(Worker[] expectedWorkers, Worker[] computedWorkers) {
		boolean isEqual = expectedWorkers.length == computedWorkers.length;
		if (isEqual) {
			// Only compare the Worker items if the number of items matches
			// the expected number

			// Use TreeSet so both lists will be sorted
			TreeSet<Worker> computedSet = new TreeSet<Worker>();

			for (int i = 0; i < computedWorkers.length; i++) {
				computedSet.add(computedWorkers[i]);
			}
			TreeSet<Worker> expectedSet = new TreeSet<Worker>();
			for (int i = 0; i < expectedWorkers.length; i++) {
				expectedSet.add(expectedWorkers[i]);
			}
			Iterator<Worker> computedIter = computedSet.iterator();
			for (Worker expectedWorker : expectedSet) {
				Boolean result = verifyEIWorker(expectedWorker, computedIter.next());
				if (!result) {
					isEqual = false;
				}
			}
		}

		return isEqual;
	}

	protected boolean verifyEIWorker(Worker expectedWorker, Worker computedWorker) {
		boolean compareResult = expectedWorker.equals(computedWorker);
		if (false == compareResult) {
			sLogger.error("Computed Worker:");
			sLogger.error(computedWorker.toString());
			sLogger.error("Expected Worker:");
			sLogger.error(expectedWorker.toString());
		}
		return compareResult;
	}

	protected Worker initEIWorkerValues(Long workerId) throws ValidationException, ServiceFailureException {
		return sWorkerService.getWorkerById(workerId);
	}

	public static SaveCompensationItemRequest[] mapToSaveCompensationItemRequest(
			SaveCompensationItemOperation operation, CompensationItem[] compensationItems) {
		return CompensationUtils.mapToSaveCompensationItemRequest(operation, compensationItems);
	}

}
