package com.intuit.sb.payroll.task;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.common.DateUtil;

public class TaskServiceTest extends TaskServiceTestBase {
	@Test(expectedExceptions = ServiceFailureException.class)
	public void createNullTaskTest() throws ServiceFailureException {
		Task task = null;
		taskService.createTask(task);
	}

	@Test
	public void createTaskTest() throws Exception {
		Date dueDate = DateUtil.createDate(2008, Calendar.DECEMBER, 25); // 25th Dec 2008
		Task task = createNonScheduledTask(dueDate);
		Task taskFromDb = taskService.createTask(task);
		AssertJUnit.assertNotNull(taskFromDb);
		AssertJUnit.assertNotNull(taskFromDb.getObjectHandle());
		AssertJUnit.assertNotNull(taskFromDb.getDueDate());
	}

	@Test
	public void findTasksTest() throws Exception {
		Date today = DateUtil.getNowDate();
		Calendar cal = DateUtil.dateToCalendar(today);
		int nextYear = cal.get(Calendar.YEAR) + 1;
		int lastYear = cal.get(Calendar.YEAR) - 1;
		Date olddueDate = DateUtil.createDate(nextYear, Calendar.DECEMBER, 25); // 25th Dec of next year
		Task[] oldtasks = taskService.getTasks(today, olddueDate);

		// create an old task
		Date dueDate = DateUtil.createDate(lastYear, Calendar.DECEMBER, 25); // 25th Dec 2008
		Task task = createNonScheduledTask(dueDate);
		taskService.createTask(task);

		task = createNonScheduledTask(today);
		taskService.createTask(task);

		// There should only be 1 more task within this period
		Task[] tasks = taskService.getTasks(today, olddueDate);
		AssertJUnit.assertEquals(1 + oldtasks.length, tasks.length);
	}

	@Test
	public void closeTaskTest() throws Exception {
		Date fromDate = DateUtil.createDate(2007, Calendar.DECEMBER, 25); // 25th Dec 2007
		Date toDate = DateUtil.createDate(2009, Calendar.DECEMBER, 25); // 25th Dec 2009
		Date dueDate = DateUtil.createDate(2008, Calendar.DECEMBER, 25); // 25th Dec 2008

		Task[] oldTasks = taskService.getTasks(fromDate, toDate);
		Task[] oldActivetasks = taskService.getActiveTasks(fromDate, toDate);

		// first create at least one task in the system

		Task task = createNonScheduledTask(dueDate);
		Task task1Frondb = taskService.createTask(task);

		// now close the task
		Task closedTask = taskService.markTaskComplete(task1Frondb);
		TaskStatus status = closedTask.getstatus();
		AssertJUnit.assertEquals(TaskStatus.COMPLETE, status); // check the task status as canceled

		// now get the task back and see the status of the task
		// now get the active tasks, we should get only one active task

		Task[] tasks = taskService.getActiveTasks(fromDate, toDate);
		AssertJUnit.assertEquals(oldActivetasks.length, tasks.length);

		// the total number of tasks in the system should only be increased by 1
		tasks = taskService.getTasks(fromDate, toDate);
		tasks = removeSetupTasks(tasks);
		AssertJUnit.assertEquals(1 + oldTasks.length, tasks.length);
	}

	// Remove SetUp related tasks with no due date
	private Task[] removeSetupTasks(Task[] tasks) {
		ArrayList<Task> taskList = new ArrayList<Task>();
		for (Task task : tasks) {
			if ((task.getType().equals(TaskType.NON_SCHEDULED)) && (task.getDueDate() == null)) {
				continue;
			} else {
				taskList.add(task);
			}
		}
		Task[] tmpTasks = new Task[taskList.size()];
		return taskList.toArray(tmpTasks);
	}

	@Test
	public void cancelTaskTest() throws Exception {
		Date fromDate = DateUtil.createDate(2007, Calendar.DECEMBER, 25); // 25th Dec 2008
		Date toDate = DateUtil.createDate(2009, Calendar.DECEMBER, 25); // 25th Dec 2009
		Date dueDate = DateUtil.createDate(2008, Calendar.DECEMBER, 25); // 25th Dec 2008

		Task[] oldActivetasks = taskService.getActiveTasks(fromDate, toDate);
		Task[] oldtasks = taskService.getTasks(fromDate, dueDate);

		// first create at least one task in the system

		Task task = createNonScheduledTask(dueDate);
		Task task1Frondb = taskService.createTask(task);

		// now cancel the task
		Task cancelledTask = taskService.cancelTask(task1Frondb);
		TaskStatus status = cancelledTask.getstatus();
		AssertJUnit.assertEquals(TaskStatus.CANCELLED, status); // check the task status as canceled

		// now get the task back and see the status of the task
		// now get the active tasks, we should get only one active task

		Task[] tasks = taskService.getActiveTasks(fromDate, toDate);
		tasks = removeSetupTasks(tasks);
		AssertJUnit.assertEquals(oldActivetasks.length, tasks.length);

		// the total number of tasks in the system will still be 2
		tasks = taskService.getTasks(fromDate, dueDate);
		tasks = removeSetupTasks(tasks);
		AssertJUnit.assertEquals(1 + oldtasks.length, tasks.length);
	}

	@Test
	public void findActiveTasksTest() throws Exception {
		Date today = DateUtil.getNowDate();
		Calendar cal = DateUtil.dateToCalendar(today);
		int nextYear = cal.get(Calendar.YEAR) + 1;
		Date toDate = DateUtil.createDate(nextYear, Calendar.DECEMBER, 25); // 25th Dec of next year
		Task[] oldtasks = taskService.getTasks(today, toDate);
		Task[] oldActiveTasks = taskService.getActiveTasks(today, toDate);
		// first create at least one task in the system

		Task task = createNonScheduledTask(toDate);
		taskService.createTask(task);

		Date dueDate = DateUtil.createDate(2010, Calendar.JUNE, 25); // 25th June 2009
		task = createNonScheduledTask(dueDate);
		Task task2FromDb = taskService.createTask(task);

		// now close the first task
		taskService.markTaskComplete(task2FromDb);

		// now get the active tasks, we should get only one active task

		Task[] activeTasks = taskService.getActiveTasks(today, toDate);
		AssertJUnit.assertEquals(1 + oldActiveTasks.length, activeTasks.length);

		// the total number of tasks in the system will still be +2
		Task[] tasks = taskService.getTasks(today, toDate);

		AssertJUnit.assertEquals(2 + oldtasks.length, tasks.length);
	}

	@Test
	public void getOverdueTaskTest() throws Exception {
		Date today = DateUtil.getNowDate();
		Date dueDate = DateUtil.createDate(2010, Calendar.DECEMBER, 25);
		Task[] oldtasks = taskService.getTasks(today, dueDate);
		Task[] oldoverDueTasks = taskService.getOverdueTasks();

		// first create at least one task in the system
		dueDate = DateUtil.createDate(2007, Calendar.DECEMBER, 25); // 25th Dec 2007 task
		Task task = createNonScheduledTask(dueDate);
		taskService.createTask(task);

		dueDate = DateUtil.createDate(2010, Calendar.JUNE, 25); // 25th June 2009 task
		task = createNonScheduledTask(dueDate);
		taskService.createTask(task);

		Task[] overDueTasks = taskService.getOverdueTasks();
		AssertJUnit.assertEquals(1 + oldoverDueTasks.length, overDueTasks.length); // 25th Dec 2007 - only one task

		// the total number of tasks in the system will still be 2
		// now get the active tasks, we should get only one active task
		dueDate = DateUtil.createDate(2010, Calendar.DECEMBER, 25); // 25th Dec 2009

		Task[] tasks = taskService.getTasks(today, dueDate);
		AssertJUnit.assertEquals(1 + oldtasks.length, tasks.length); // 25th June 2009 task
	}
}
