package com.intuit.sb.payroll.entity.common.impl;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.OptimisticLockException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.apache.log4j.Logger;
import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.dpt.vpd.multitenancy.TenantContext;
import com.intuit.sb.dpt.vpd.multitenancy.TenantContextManager;
import com.intuit.sb.dpt.vpd.multitenancy.TenantContextNoPermissionException;
import com.intuit.sb.payroll.InfrastructureException;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.common.PayrollLoggerFactory;
import com.intuit.sb.payroll.entity.common.EntityService;
import com.intuit.sb.payroll.entity.domain.Identifiable;

@Stateless(name = "EntityService")
@TransactionAttribute(TransactionAttributeType.MANDATORY)
@SecurityDomain("BHPIAM")
@RolesAllowed( { "Authenticated" })
public class EntityServiceBean implements EntityService {
	
	protected static final Logger logger = PayrollLoggerFactory.makeLogger(EntityServiceBean.class);
	
	@Resource
	// Inject the EJB session bean context
	private SessionContext mEJBSessionContext;
	// Inject an entity manager instance for our data source
	@PersistenceContext(unitName = "payrollDB")
	private EntityManager mEntityManager;
	
	@SuppressWarnings("unchecked")
	public <T> List<T> findAll(Class<T> type) {
		String fqn = type.getName();
		int lastDot = fqn.lastIndexOf('.');
		String basename = fqn.substring(lastDot == 0 ? 0 : lastDot + 1);
		String queryString = "Select " + basename + " from " + fqn + " as " + basename;
		Query queryImpl = mEntityManager.createQuery(queryString);
		List<?> results = queryImpl.getResultList();
		if (results != null) {
			List<T> stResults = new ArrayList<T>(results.size());
			for (Object object : results) {
				stResults.add((T) object);
			}
			return stResults;
		} else {
			return new ArrayList<T>();
		}
	}

	public <T> T findById(Class<T> type, Long id) {
		return mEntityManager.find(type, id);
	}

	public <T> T findByIdentifiable(Class<T> type, Identifiable identifiable) {
		T result = null;
		try {
			result = findById(type, identifiable.getId());
			if (result == null) {
				String mesg = MessageFormat.format("Entity not found: type={0} id={1}", type.getName(),
						identifiable.getId());
				throw new IllegalArgumentException(mesg);
			}
			if (((Identifiable)result).getVersion() != identifiable.getVersion()) {
				String mesg = MessageFormat.format(
						"Entity is out of date: type={0} id={1} entity version={2} current version={3}", type
								.getName(), identifiable.getId(), identifiable.getVersion(), ((Identifiable)result).getVersion());
				throw new OptimisticLockException(mesg);
			}
		} catch (IllegalStateException ex) {
			handleExceptionMightBeOLE(ex);
		} catch (PersistenceException ex) {
			handleExceptionMightBeOLE(ex);
		}
		return result;
	}

	public void flush() {
		mEntityManager.flush();
	}

	public <T> T makeManaged(T entity) {
		return mEntityManager.merge(entity);
	}

	public <T> T makePersistent(T entity) {
		mEntityManager.persist(entity);
		return entity;
	}

	public <T> void makeTransient(T entity) {
		mEntityManager.remove(entity);

	}

	public <T> boolean notPersistent(T entity) {
		return ! mEntityManager.contains(entity);
	}

	/** Set the TenantContext for external calls */
	public void setTenantContext(String companyId) throws ServiceFailureException {
		TenantContext tenantContext = TenantContextManager.createContext(companyId);
		try {
			TenantContextManager.setCurrentContext(tenantContext);
		} catch (TenantContextNoPermissionException e) {
			throw new ServiceFailureException(e);
		}
	}

	public void clearTenantContext() {
		TenantContextManager.clearCurrentContext();
	}
	
	/**
	 * Rollback when an exception is handled.
	 * 
	 * @throws OptimisticLockException
	 *             If the exception is an OLE convert it to our application OLE.
	 * @throws PayrollServiceFailureException
	 *             Otherwise throw an PayrollServiceFailureException.
	 */
	protected void handleExceptionMightBeOLE(Exception exception) throws OptimisticLockException,
			InfrastructureException {
		if (exception != null) {
			mEJBSessionContext.setRollbackOnly();
			logger.debug("EAO handled exception: " + exception, exception);
			throw (exception instanceof javax.persistence.OptimisticLockException) ? new OptimisticLockException(
					ExceptionUtils.throwableMessage(exception)) : new InfrastructureException(ExceptionUtils
					.throwableMessageChain(exception), exception);
		}
	}

}
