package com.intuit.sb.payroll.entity.task.impl;


import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;

import org.jboss.annotation.security.SecurityDomain;

import com.intuit.sb.payroll.entity.eao.ejb.GenericEjbEAO;
import com.intuit.sb.payroll.entity.model.Task;
import com.intuit.sb.payroll.entity.model.TaskStatusEnum;
import com.intuit.sb.payroll.entity.task.TaskEAO;


/**
 * The EJB3 implementation of the {@link TaskEAO TaskEAO} interface.
 * As required by the superclass, this concrete EAO implementation
 * is an EJB3 SSB. It publishs itself in JNDI under its interface,
 * not implementation, name.
 * 
 * @see Task, TaskEAO, GenericEAO
 */

/* We obviously can't annotate GenericEJB3EAO as stateless because we want to use the
 * name of the specialised interface. But there's another reason; the EJB spec doesn't
 * allow inheritance of @Stateless from a superclass (unlike most other annotations
 * which *are* inherited from superclasses into specializations!)
 */
@Stateless(name = "TaskEAO")
@TransactionManagement(TransactionManagementType.CONTAINER) // Use CMT
// Clients must start the Txn so multiple EAO calls in a single request are within a single txn
@TransactionAttribute(TransactionAttributeType.MANDATORY) 
@SecurityDomain("BHPIAM")
@RolesAllowed({"Authenticated"})
public final class TaskEJB3EAO
        extends GenericEjbEAO<Task>
        implements TaskEAO
{
    /**
     * EJB required no-arg constructor.
     */
    public TaskEJB3EAO()
    {
        super();
    }
    
    // ********************** Business Queries ********************** //

    /* (non-Javadoc)
	 * @see com.intuit.sb.payroll.task.foo#findActiveTasksByClass(java.lang.String)
	 */    
    public List<Task> findActiveTasksByClass(String taskClass)
    {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("taskClass", taskClass);
        
        List<Task> results = null;
		try
		{
			// named query is in orm.xml
			results = findByNamedQuery("findActiveTasksByClass", queryParameters);
		}
		catch(RuntimeException ex)
		{
			handleExceptionMightBeOLE(ex);
		}     
		return results;
    }

    /* (non-Javadoc)
	 * @see com.intuit.sb.payroll.task.foo#findActiveTasksByClassAndPayload(java.lang.String, java.lang.String)
	 */    
    public List<Task>  findActiveTasksByClassAndPayload(String taskClass, Long payload)
    {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("taskClass", taskClass);
        queryParameters.put("payload", payload);
        
        List<Task> results = null;
		try
		{
			// named query is in orm.xml
			results = findByNamedQuery("findActiveTasksByClassAndPayload", queryParameters);
		}
		catch(RuntimeException ex)
		{
			handleExceptionMightBeOLE(ex);
		}     
		return results;
    }

    /* (non-Javadoc)
	 * @see com.intuit.sb.payroll.task.foo#findCurrentTasks(java.util.Date, java.util.Date)
	 */    
    public List<Task>  findCurrentTasks(Date completionStartDate, Date deadlineEndDate)
    {
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("completionStartDate", completionStartDate);
        queryParameters.put("deadlineEndDate", deadlineEndDate);
        
        List<Task> results = null;
		try
		{
			// named query is in orm.xml
			results = findByNamedQuery("findCurrentTasksInDateRange", queryParameters);
		}
		catch(RuntimeException ex)
		{
			handleExceptionMightBeOLE(ex);
		}     
		return results;
    }
    
    /* (non-Javadoc)
	 * @see com.intuit.sb.payroll.task.foo#findCurrentActiveTasks()
	 */
    public List<Task> findCurrentActiveTasks() {
        
        List<Task> results = null;
		try
		{
			// named query is in orm.xml
			results = findByNamedQuery("findCurrentActiveTasks");
		}
		catch(RuntimeException ex)
		{
			handleExceptionMightBeOLE(ex);
		}     
		return results;
	}
    
    /* (non-Javadoc)
	 * @see com.intuit.sb.payroll.task.foo#findTaskByTaskNameAndPayLoad(java.lang.String, java.lang.Long)
	 */ 
	public List<Task> findTaskByTaskNameAndPayLoad(String taskName, Long payload) {
	    Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("taskName", taskName);
        queryParameters.put("payload", payload);

        List<Task> results = null;
		try
		{
			// named query is in orm.xml - will get all the tasks that whose has the name same as taskName and payload same as payload(passed value)
			results = findByNamedQuery("findTaskByTaskNameAndPayLoad", queryParameters);
		}
		catch(RuntimeException ex)
		{
			handleExceptionMightBeOLE(ex);
		}
		return results;
	}
	


    /*************************************************** [rj] added from here *******************************/
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.task.impl.TaskEAO2#getOverdueTasks()
	 */
	public List<Task> getOverdueTasks()
	{
		Date deadlineEndDate = new Date(); //today is the cut off date
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("deadlineEndDate", deadlineEndDate);

        List<Task> results = null;
		try
		{
			// named query is in orm.xml - will get all the tasks that are New/InProgress whose deadline is less than provided deadlineEndDate(which is today)
			results = findByNamedQuery("findOverdueTasks", queryParameters);
		}
		catch(RuntimeException ex)
		{
			handleExceptionMightBeOLE(ex);
		}
		return results;
	}
	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.task.impl.TaskEAO2#getActiveTasks(java.util.Date, java.util.Date)
	 */
	public List<Task> getActiveTasks(Date fromDate, Date toDate)
	{
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("fromDate", fromDate);
        queryParameters.put("toDate", toDate);

        List<Task> results = null;
		try
		{
			// named query is in orm.xml - will get all the tasks that are New/InProgress whose deadline is between the fromDate and toDate
			results = findByNamedQuery("findActiveTasks", queryParameters);
		}
		catch(RuntimeException ex)
		{
			handleExceptionMightBeOLE(ex);
		}
		return results;
	}

	/* (non-Javadoc)
	 * @see com.intuit.sb.payroll.entity.task.impl.TaskEAO2#getTasks(java.util.Date, java.util.Date)
	 */
	public List<Task> getTasks(Date fromDate, Date toDate)
	{
        Map<String, Object> queryParameters = new HashMap<String, Object>();
        queryParameters.put("fromDate", fromDate);
        queryParameters.put("toDate", toDate);

        List<Task> results = null;
		try
		{
			// named query is in orm.xml - will get all the tasks that whose deadline is between the fromDate and toDate
			results = findByNamedQuery("findTasks", queryParameters);
		}
		catch(RuntimeException ex)
		{
			handleExceptionMightBeOLE(ex);
		}
		return results;
	}
	
	public Task updateTaskStatus(Task task, TaskStatusEnum taskStatusEnum){
		Date now = new Date(System.currentTimeMillis()); // Use UTC as this is the time we search on
		task.setStatus(taskStatusEnum);
		if (taskStatusEnum.equals(TaskStatusEnum.COMPLETED) || taskStatusEnum.equals(TaskStatusEnum.CANCELED))
		{
			task.setCompletionDate(now);
		}
		else
		{
			task.setCompletionDate(null);
		}
		return makePersistent(task);
	}	
}
