package com.intuit.sb.payroll.capability.employer;

import java.util.List;
import java.util.Set;

import org.testng.Assert;
import org.testng.AssertJUnit;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.pts.adapter.mapper.EmployerIDMapper;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.entity.model.Employer;
import com.intuit.sb.payroll.entity.model.EmployerContactPerson;
import com.intuit.sb.payroll.entity.model.EmployerId;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.pts.entity.ActivationState;
import com.intuit.sb.payroll.pts.entity.ActivationStateEAO;
import com.intuit.sb.payroll.pts.entity.PTSContactPerson;
import com.intuit.sb.payroll.pts.entity.PTSContactPersonTypeEnum;
import com.intuit.sb.payroll.pts.entity.PTSEmployer;
import com.intuit.sb.payroll.pts.entity.TaxServiceStatusEnum;


/**
 * Contain test for creating and updating assisted company
 * 
 * @author amalik
 * 
 */
@Test(groups = "ptsAddCompanyTests")
public class PTSAddEmployerTest extends EmployerComponentTestBase {

	protected EmployerPTSWorkflowComponent sEmployerPTSWorkflowComponent;
	protected ActivationStateEAO activationStateEAO;
	

	
	@BeforeMethod
	public void initializeAssitedEmployerTest()
	{
		sEmployerPTSWorkflowComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
		
		activationStateEAO = ServiceLocator.getInstance().getService(ActivationStateEAO.JNDI_NAME);
	}
	
	/**
	 * Based on the Gemini-PSP contract, creating a company with minimal data
	 * <ol>
	 *   <li> sourceCompanyId </li>
	 *   <li> Offeringcode </li>
	 *   <li> PrimaryContact </li> 
	 *   <li> Mailing Address </li>
	 *   <li> Legal Address </li> 
	 *   <li> LegalName </li> 
	 *   <li>FEIN</li>
	 * </ol>
	 */
	@Test(enabled=true, groups = {"ptsAddCompanyTests"})
	public void testMinimalRequirementForCompanyCreation() {
		try {
			// Setup Employer with all required data sets
			// 1. Delete the current employer
			EmployerComponentTestUtils.deleteCurrentEmployer();

			// 2. Create a Basic Employer with legalName, DBA Name and EIN
			Employer employer = EmployerComponentTestUtils
					.createNewEmployerInMemory("testCompany", "testDBAName", "testlegalName",
							null, IndustryTypeEnum.RESTAURANT,
							LegalStructureEnum.PARTNERSHIP);

			// Add Primary Contact only
			String firstName = "CPerson";
			String lastName = "Test";
			String title = "Mr";
			String phone = "408-555-1234";
			EmployerContactPerson person = createEmployerContactPerson(
					firstName, lastName, title, phone);
			person.setOwner(Boolean.TRUE);
			employer.addEmployerContactPerson(person);

			PTSContactPerson ptsContactPerson = createPTSEmployerContactPerson(person, PTSContactPersonTypeEnum.PRIMARY_CONTACT);
			
			// Legal Address
			
			GlobalPostalAddress legalAddress = EmployerComponentTestUtils
					.createGlobalPostalAddress("44 Elm Street", "San Carlos",
							"CA", "93122");
			employer.setLegalAddress(legalAddress);

			// Mailing address
			GlobalPostalAddress mailingAddress = EmployerComponentTestUtils
					.createGlobalPostalAddress("45 Elm Street", "San Carlos",
							"CA", "93122");
			employer.setBusinessAddress(mailingAddress);

			employer = sEmployerComponent.createEmployer(employer);
			PTSEmployer ptsEmployer = sEmployerPTSWorkflowComponent.getPTSEmployer();
			ptsEmployer.addPTSContactPerson(ptsContactPerson);
			List<ActivationState> sortedActiveState = activationStateEAO.findAllInSortedOrder();
			Assert.assertEquals(1, sortedActiveState.size(), "Size of ActivationState is not expected for new Employer");
			

			// Start Activation process
			sEmployerPTSWorkflowComponent.updateEmployerTaxServiceState(TaxServiceStatusEnum.SIGNED_UP_FOR_ASSISTED);
			
			sEmployerPTSWorkflowComponent.sendCompanyInfoToPTS();
			
			AssertJUnit.assertEquals("Add Company Call is not sent to PSP", true,
					StatusUpdateHandlerUtil.isEntityAddedToPTS(sEmployerPTSWorkflowComponent.getPTSEmployer().getEmployer()));
			
		} catch (ServiceFailureException e) {
			AssertJUnit.fail("ServiceFailureException " + e.getLocalizedMessage());
			e.printStackTrace();
		} catch (ValidationException e) {
			AssertJUnit.fail("ValidationException " + e.getLocalizedMessage());
			e.printStackTrace();
		}
	}
	
	private PTSContactPerson createPTSEmployerContactPerson(EmployerContactPerson employerContactPerson, PTSContactPersonTypeEnum contactPersonType) {
		
		PTSContactPerson ptsContactPerson = new PTSContactPerson();
		ptsContactPerson.setEmployerContact(employerContactPerson);
		ptsContactPerson.setContactPersonType(contactPersonType);
		
		return ptsContactPerson;
	}

	/**
	 * Send complete DTO with all Required and optional values
	 */
	@Test(enabled=false, groups = {"ptsAddCompanyTests"})
	public void testAssistedCompanyCreation() {
		try {
			// Setup Employer with all required data sets
			// 1. Delete the current employer
			EmployerComponentTestUtils.deleteCurrentEmployer();

			// 2. Create a Basic Employer
			Employer employer = EmployerComponentTestUtils
					.createNewEmployerInMemory("testCompany","testDBAName", "testDBAName",
							"123456789", IndustryTypeEnum.RESTAURANT,
							LegalStructureEnum.PARTNERSHIP);

			// PrimaryContact
			String firstName = "CPerson";
			String lastName = "Test";
			String title = "Mr";
			String phone = "408-555-1234";
			EmployerContactPerson person = createEmployerContactPerson(
					firstName, lastName, title, phone);
			person.setOwner(Boolean.TRUE);
			employer.addEmployerContactPerson(person);

			// Add Secondary Contact ( & PayrollAdmin)

			EmployerContactPerson secperson = createEmployerContactPerson(
					"SPerson", "Parker", "Jr", "6501234567");
			secperson.setOwner(Boolean.FALSE);
			employer.addEmployerContactPerson(secperson);

			// Legal Address
			GlobalPostalAddress legalAddress = EmployerComponentTestUtils
					.createGlobalPostalAddress("44 Elm Street", "San Carlos",
							"CA", "93122");
			employer.setLegalAddress(legalAddress);

			// Mailing address
			GlobalPostalAddress mailingAddress = EmployerComponentTestUtils
					.createGlobalPostalAddress("45 Elm Street", "San Carlos",
							"CA", "93122");
			employer.setBusinessAddress(mailingAddress);

			employer = sEmployerComponent.createEmployer(employer);

			// We make sure ONLY required values for Employer is available
			employer = setMinimalPropertiesWithEmployerForAssisted(employer);


			// Start Activation process
			sEmployerPTSWorkflowComponent.updateEmployerTaxServiceState(TaxServiceStatusEnum.SIGNED_UP_FOR_ASSISTED);

		} catch (ServiceFailureException e) {
			AssertJUnit.fail("ServiceFailureException " + e.getLocalizedMessage());
			e.printStackTrace();
		} catch (ValidationException e) {
			AssertJUnit.fail("ValidationException " + e.getLocalizedMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Here we check for DBAName LegalName EIN LegalAddess PrimaryContact
	 * 
	 * @param employer
	 * @return
	 */
	private Employer setMinimalPropertiesWithEmployerForAssisted(
			Employer employer) {
		// Check and ensure all legal information is available
		String dbaName = employer.getDbaName();
		if ((dbaName == null) || (dbaName.length() == 0)) {
			employer.setDbaName("dummy DBAName");
		}

		String legalName = employer.getLegalName();
		if ((legalName == null) || (legalName.length() == 0)) {
			employer.setLegalName("My Bogus Company");
		}

		Set<EmployerId> employerIds = employer.getEmployerIds(); // For EIN

		String fein = EmployerIDMapper.getFEIN(employerIds);

		if (fein == null) {
			// Set EIN for this Employer
		}

		GlobalPostalAddress legalAddress = employer.getLegalAddress();

		if (legalAddress == null) {
			USPostalAddress uslegalAddress = EmployerComponentTestUtils
					.createLegalAddress();
			employer.setLegalAddress(AddressConvertor.getGlobalFromUS(
					uslegalAddress, new GlobalPostalAddress()));
		}

		// ensure Primary Contact is available
		Set<EmployerContactPerson> employerContactPersons = employer
				.getEmployerContactPersons();

		if (employerContactPersons.size() == 0) {
			AssertJUnit
					.fail("Employer Not ready for assisted as PrimaryContact not specified");
		}

		return employer;
	}
}
