package com.intuit.sb.payroll.qa.services.employer;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.PTSEmployerServiceTestBase;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.CompanyActivationStatusEnum;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationTypeEnum;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.employer.SaveEmployerInfoRequest;
import com.intuit.sb.payroll.services.taxes.EmployerTaxIdInfo;
import com.intuit.sb.payroll.task.Task;


public class EFTPSRejectionTest extends PTSEmployerServiceTestBase{
	
	 /**
	 * <P>
	 * Test getting the Tasks from Gemini , when PTS sends EFTPS - Rejected (wrong EIN) notification to Gemini
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer 
	 * <LI>Create new Employer with random EIN</LI>
	 * <LI>Add current Employer to PTS
	 * <LI>Construct PTSNotificationDTO based on added Employer detail</LI>
	 * <LI>call updateCompanyActivationStatus() of PTSNotificationAdapter passing above PTSNotificationDTO</LI>
	 * <LI>Get the current tasks listing of Gemini</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that "Make correction to EIN" task is obtained
	 * <LI> Verify that Legal Info is unlocked : UI 
	 * </LI>
	 * </OL>
	 * @throws Exception 
	 */

	@Test(enabled = true)
	public void testEFTPSRejectInvalidEINNotification() throws Exception{
		
		prepareCurrentEmployerForAssisted(sEmployerService,sEmployerTaxSettingsService);
		
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		EmployerTaxIdInfo[] taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
		
		Assert.assertTrue(taxIdInfos[0].isLocked(), "UNEXPECTED : Legal card is not locked");		
		
		// verify that data submission is not allowed
		Assert.assertFalse(sEmployerService.allowDataSubmissionToPTS(), "UNEXPECTED : data submission to PTS is allowed");
		
		//Mock the EFTPS rejection notification for invalid EIN
		mockUpdateCompanyActivationStatus(createPTSNotiticationDTO(NotificationTypeEnum.EFTPS_NOTIFICATION,CompanyActivationStatusEnum.EFTPS_ENRL_REJECTED_EIN_MIS_MATCH));
		
		Task[] tasks = sTaskService.getCurrentTasks();
		
		// Verify the expected task
		String[] expectedTaskNames = {EmployerPTSWorkflowTasks.REJECTED_EIN_MISMATCH_TASK };
		
		verifyTaskName(tasks, expectedTaskNames);
		
		taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
		
		//verify if card is not locked
		Assert.assertFalse(taxIdInfos[0].isLocked(), "UNEXPECTED : Legal card is locked");
		
		// verify that data submission is allowed
		Assert.assertTrue(sEmployerService.allowDataSubmissionToPTS(), "UNEXPECTED : data submission to PTS is allowed");
		
		
		// update the current employer EIN
		updateCurrentEmployerEIN();
		
		//verify if card is no locked
		Assert.assertFalse(taxIdInfos[0].isLocked(), "UNEXPECTED : Legal card is locked");
		
		// verify that data submission is not allowed
		Assert.assertTrue(sEmployerService.allowDataSubmissionToPTS(), "UNEXPECTED : data submission to PTS not is allowed");
		
		String[] expectedTaskNames2 = {EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS};
		
		tasks = sTaskService.getCurrentTasks();
		
		verifyTaskName(tasks, expectedTaskNames2);
		
		for(int i = 0 ; i < tasks.length ; i++ ){
			
			if(tasks[i].getName().equalsIgnoreCase(EmployerPTSWorkflowTasks.REJECTED_EIN_MISMATCH_TASK)){
				Assert.fail("UNEXPECTED : REJECTED_EIN_MISMATCH_TASK is found");
			}
			
		}
		
	}
	

	/**
	 * <P>
	 * Test getting the Tasks from Gemini , when PTS sends EFTPS - Rejected (wrong Name) notification to Gemini
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer 
	 * <LI>Create new Employer with random EIN</LI>
	 * <LI>Add current Employer to PTS
	 * <LI>Construct PTSNotificationDTO based on added Employer detail</LI>
	 * <LI>call updateCompanyActivationStatus() of PTSNotificationAdapter passing above PTSNotificationDTO</LI>
	 * <LI>Get the current tasks listing of Gemini</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that "Make correction to Legal Name" task is obtained
	 * </LI>
	 * </OL>
	 * @throws Exception 
	 */

	@Test(enabled=true)
	public void testEFTPSRejectInvalidLegalName() throws Exception{
		
		prepareCurrentEmployerForAssisted(sEmployerService,sEmployerTaxSettingsService);
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);
		
		EmployerTaxIdInfo[] taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
		
		Assert.assertTrue(taxIdInfos[0].isLocked(), "UNEXPECTED : Legal card is not locked");		
		
		// verify that data submission is not allowed
		Assert.assertFalse(sEmployerService.allowDataSubmissionToPTS(), "UNEXPECTED : data submission to PTS is allowed");
		
		//Mock the EFTPS rejection notification for legal name
		mockUpdateCompanyActivationStatus(createPTSNotiticationDTO(NotificationTypeEnum.EFTPS_NOTIFICATION,CompanyActivationStatusEnum.EFTPS_ENRL_REJECTED_NAME_MIS_MATCH));
		
		Task[] tasks = sTaskService.getCurrentTasks();
		
		String[] expectedTaskNames = {EmployerPTSWorkflowTasks.REJECTED_LEGALNAME_MISMATCH_TASK };
		
		verifyTaskName(tasks, expectedTaskNames);
		
		taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
		
		Assert.assertFalse(taxIdInfos[0].isLocked(), "UNEXPECTED : Legal card is not locked");
		
		
		// verify that data submission is not allowed
		Assert.assertTrue(sEmployerService.allowDataSubmissionToPTS(), "UNEXPECTED : data submission to PTS is allowed");
		
		// update the current employer EIN
		updateCurrentEmployerLegalName();
		
		//verify if card is no locked
		Assert.assertFalse(taxIdInfos[0].isLocked(), "UNEXPECTED : Legal card is locked");
		
		// verify that data submission is not allowed
		Assert.assertTrue(sEmployerService.allowDataSubmissionToPTS(), "UNEXPECTED : data submission to PTS not is allowed");
		
		String[] expectedTaskNames2 = {EmployerPTSWorkflowTasks.PRINT_AND_RESUBMIT_FORMS};
		
		tasks = sTaskService.getCurrentTasks();
		
		verifyTaskName(tasks, expectedTaskNames2);
		
		for(int i = 0 ; i < tasks.length ; i++ ){
			
			if(tasks[i].getName().equalsIgnoreCase(EmployerPTSWorkflowTasks.REJECTED_LEGALNAME_MISMATCH_TASK)){
				Assert.fail("UNEXPECTED : REJECTED_EIN_MISMATCH_TASK is found");
			}
			
		}

		
	}
	
	private void updateCurrentEmployerLegalName() throws ServiceFailureException, ValidationException{
		
		EmployerInfo eiInfo = sEmployerService.getEmployerInfo();
		eiInfo.setLegalName("Random Legal Name");
		SaveEmployerInfoRequest sreq = new SaveEmployerInfoRequest();
		sreq.setEmployerInfo(eiInfo);
		sEmployerService.saveEmployerInfo(sreq);
		
	}
	
	private void updateCurrentEmployerEIN() throws ValidationException, ServiceFailureException{
		
		EmployerTaxIdInfo[] taxIdInfos = null;
		
		taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
		for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos)
		{
			if (employerTaxIdInfo.getTaxAgency().name.contains("US_IRS"))
			{
				employerTaxIdInfo.setTaxId("156111786");
			}
		}
		sEmployerTaxSettingsService.updateEmployerTaxIDs(taxIdInfos);
	}

}
