package com.intuit.sb.payroll.qa.services.employer;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.common.PTSEmployerServiceTestBase;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.CompanyActivationStatusEnum;
import com.intuit.sb.payroll.capability.pts.adapter.dtos.NotificationTypeEnum;
import com.intuit.sb.payroll.entity.model.BankAccountTypeEnum;
import com.intuit.sb.payroll.services.common.BankAccount;
import com.intuit.sb.payroll.services.common.BankAccountStatusEnum;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.task.Task;


public class PTSEmployerBankValidationTest extends PTSEmployerServiceTestBase{
	
	
		
	/**
     * <P>Test GEMINI behavior when Bank Account Validation Success Notification is received</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Add all the required info for company to be assisted ready</LI>
     * <LI>Add a bank account</LI>
     * <LI>Mock the Bank Validation Success Notification call</LI>
     * </OL>
     * <UL>
     * <LI>Verify that status of Bank is "BANKACCOUNT_VERIFIED_RECEIVED"</LI>
     * <LI>Verify that the card status is still locked i.e "true"</LI>
     * <LI>Verify that updating bank account not possible</LI>
     * </UL>
     */
	@Test(enabled=false)// to be fixed by Dev
	public void testBankValidationSuccessNotification() throws Exception {
		
		prepareCurrentEmployerForAssisted(null,null);
		
		//create a bank Account
		BankAccount[] bankAccounts=new BankAccount[1];		
		BankAccount bankAccount = createBankAccount(BankAccountTypeEnum.SAVINGS);
		bankAccounts[0] = bankAccount;
		
		//add bankAccount to the employer
		BankAccount[] savedBankAccounts = addBanksToCurrentEmployer(bankAccounts);
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED); //Sends company Info to PSP
		
		//get bank id
		BankAccount bankAccountRetrieved = getBankAccountBasedOnAccountNo(savedBankAccounts, bankAccount.getAccountNo());
		String id = getBankIdFromBankAccount(bankAccountRetrieved);
		
		//verify the bankAccount Status
		verifyBankVerifiedStatusInDB(BankAccountStatusEnum.BANKACCOUNT_UNDER_VERIFICATION,bankAccount.getAccountNo());
		
		//verify the lock status
		Assert.assertEquals(getBankAccountBasedOnAccountNo(savedBankAccounts, bankAccounts[0].getAccountNo()).getLocked(),true);
		
		//Mock the Bank Account Verification success Notification
		mockBankAccountInboundCall(NotificationTypeEnum.ACTIVATION_ITEM_BANK_ACCOUNT_VERIFY,CompanyActivationStatusEnum.ACTIVATION_ACTIVE,id);		
		
		// verify the retrieved bankAccountStatus
		verifyBankVerifiedStatusInDB(BankAccountStatusEnum.BANKACCOUNT_VERIFIED_RECEIVED,bankAccount.getAccountNo());
		
		//verify the lock status
		Assert.assertEquals(getBankAccountBasedOnAccountNo(savedBankAccounts, bankAccounts[0].getAccountNo()).getLocked(),true);
		
		//verify the update
		bankAccountRetrieved.setAccountNo(EmployerComponentTestUtils.generateBankAccountNumber());
		bankAccounts[0] = bankAccountRetrieved;
		
		try{
			sEmployerService.updateBankAccounts(bankAccounts);
			Assert.fail("This should have thrown ValidationException");
		}
		catch(ValidationException ve){
			
		}
	}
	
	/**
     * <P>Test GEMINI behavior when Bank Account Validation failure Notification is received</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Add all the required info for company to be assisted ready</LI>
     * <LI>Add a bank account</LI>
     * <LI>Mock the Bank Validation failure Notification call</LI>
     * </OL>
     * <UL>
     * <LI>Verify that TO DO task list has "REJECTED_BANK_ACCOUNT_TASK" task added</LI>
     * <LI>Verify that status of Bank is "BANKACCOUNT_VERIFICATION_FAILED_RECEIVED"</LI>
     * <LI>Verify that the card status is unlocked i.e "false"</LI>
     * <LI>Verify that after updating bank account the "Modify Bank" closed and card status is locked i.e "true"</LI>
     * </UL>
     */	
	@Test(enabled=false)// to be fixed by Dev
	public void testInvalidBankAccountNotification() throws Exception{
		
		long oldBankid = 0L;
		long newBankId= 0L;
		
		prepareCurrentEmployerForAssisted(null,null);
		
		//create a bank Account
		BankAccount[] bankAccounts=new BankAccount[1];		
		bankAccounts[0]= createBankAccount(BankAccountTypeEnum.SAVINGS);
		
		
		//add bankAccount to the employer
		BankAccount[] savedBankAccounts = addBanksToCurrentEmployer(bankAccounts);
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED); //Sends company Info to PSP
		
		//get the saved bank account
    	BankAccount bankAccountRetrieved = getBankAccountBasedOnAccountNo(savedBankAccounts, bankAccounts[0].getAccountNo());
		String id = getBankIdFromBankAccount(bankAccountRetrieved);
		
		oldBankid = bankAccountRetrieved.getObjectHandle().getId();
	
		//TO DO tasks before failure Notification is received
		Task[] toDoList = sTaskService.getCurrentTasks();
		
		//Mock the Bank Account Verification Failure Notification
		mockBankAccountInboundCall(NotificationTypeEnum.ACTIVATION_ITEM_BANK_ACCOUNT_VERIFY, CompanyActivationStatusEnum.ACTIVATION_CANCELLED, id);		
		
		// verify the retrieved bankAccountStatus
		verifyBankVerifiedStatusInDB(BankAccountStatusEnum.BANKACCOUNT_VERIFICATION_FAILED_RECEIVED, bankAccounts[0].getAccountNo());
		
		//verify the lock
		Assert.assertEquals(getBankAccountBasedOnAccountNo(savedBankAccounts, bankAccounts[0].getAccountNo()).getLocked(),false);		
		
		// Verify the TO-DO task 
		Task[] toDoListLatest = sTaskService.getCurrentTasks();
		Assert.assertEquals(toDoListLatest.length, toDoList.length+1,"Bank Account Correction TODO task is not created");
		String[] expectedTasks = {EmployerPTSWorkflowTasks.REJECTED_BANK_ACCOUNT_TASK};
		verifyTaskName(toDoListLatest, expectedTasks);
		
		// Update the bank account
		sEmployerService.updateBankAccounts(bankAccounts);
		
		//Verify the TO-DO Task after Updating Bank Account
		toDoListLatest = sTaskService.getCurrentTasks();
		
		//Assert that the Modify the Bank Account task is closed
		Assert.assertEquals(toDoListLatest.length, toDoList.length,"Bank Account Correction TODO task is not closed");
		
		// verify that card is locked
		Assert.assertEquals(getBankAccountBasedOnAccountNo(savedBankAccounts, bankAccounts[0].getAccountNo()).getLocked(),true);
				
		//update bank account again
		savedBankAccounts = sEmployerService.getBankAccounts();
		BankAccount bankAccountAfterSuccess = getBankAccountBasedOnAccountNo(savedBankAccounts, bankAccounts[0].getAccountNo());		
		newBankId = bankAccountAfterSuccess.getObjectHandle().getId();
		
		Assert.assertTrue(oldBankid==newBankId, "UNEXPECTED : Bank Ids are same");	
		
		bankAccountAfterSuccess.setAccountNo(EmployerComponentTestUtils.generateBankAccountNumber());
		bankAccounts[0] = bankAccountAfterSuccess;		
		
		try{
			sEmployerService.updateBankAccounts(bankAccounts);
			Assert.fail("UNEXPECTED : Able to update the Bank");
		}
		catch(ValidationException ve){
			
		}	
	} 
}
