package com.intuit.sb.payroll.qa.services.employer;

 


import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.EmployerPTSWorkflowComponent;
import com.intuit.sb.payroll.capability.pts.statehandler.StatusUpdateHandlerUtil;
import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.CompanyType;
import com.intuit.sb.payroll.services.employer.EmployerContact;
import com.intuit.sb.payroll.services.employer.EmployerContactType;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.services.employer.SaveEmployerInfoRequest;
import com.intuit.sb.payroll.services.employer.impl.EmployerInfoMapper;
import com.intuit.sb.payroll.services.taxes.EmployerTaxIdInfo;


/**
 * <P>The <CODE>PTSSendCompanyInfoTests</CODE> class contains test methods for adding a company to PTS and ValidationException for various error path
 * scenario 
 * 
 * $Author: kvenkatraman $
 */

public class PTSSendCompanyInfoTest extends EmployerServiceTestBase {
	
	protected static EmployerPTSWorkflowComponent empPTSWfComponent;
	
	 @BeforeClass
	 public static void initializePTSSendCompanyInfoTestsClass()
	 {
		 empPTSWfComponent = ServiceLocator.getInstance().getService(EmployerPTSWorkflowComponent.JNDI_NAME);
	 }
	 /**
		 * <P>
		 * Test adding Company Info to PTS with minimum mandatory fields
		 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
		 * </P>
		 * <P>
		 * Fields that can be set through EmployerService
		 * </P>
		 * <OL>
		 * <LI>Primary Contact</LI>
		 * <LI>Mailing Address [ Main Worklocation ]</LI>
		 * <LI>Legal Address</LI>
		 * <LI>Legal Name</LI>
		 * <LI>FEIN</LI>
		 * </OL>
		 * 
		 * <H3>Steps:</H3>
		 * <OL>
		 * <LI>Delete current employer and Create new Employer with random EIN</LI>
		 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
		 * <LI>Add Primary Contact to the Employer</LI>
		 * <LI>Add Legal Information to the Employer</LI>
		 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
		 * <LI>Initiate PSP call workflow</LI>
		 * </OL>
		 * <H3>
		 * <OL>
		 * <LI>Verify that Employer State is changed to ADD_COMPANY_REQUESTED ==> Add Company Call to PTS is successful
		 * </LI>
		 * </OL>
		 */

	@Test(enabled=true)
	public void testAddEmployerCompanyWithMinimalMandatoryInfo()throws Exception{
		
		deleteCurrentEmployerAndCreateNewEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();

		// Add work location
		addWorklocationToCurrentEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
		
 		Assert.assertEquals(StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()),
 				true,"Add Company Call is not sent to PSP");
	}
	
	/**
	 * <P>
	 * Test adding Company Info to PTS with All the fields mentioned in the Contract Document
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <P>
	 * Fields that can be set through EmployerService
	 * </P>
	 * <OL>
	 * <LI>Primary Contact</LI>
	 * <LI>Secondary Contact</LI>
	 * <LI>PayrollAdmin Contact</LI>
	 * <LI>Mailing Address [ Main Worklocation ]</LI>
	 * <LI>Legal Address</LI>
	 * <LI>Legal Name</LI>
	 * <LI>FEIN</LI>
	 * <LI>TaxEmpentExpiration Date</LI>
	 * <LI>Payroll Frequency Code</LI>
	 * <LI>DBA</LI>
	 * </OL>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Primary Contact to the Employer</LI>
	 * <LI>Add Secondary Contact to the Employer</LI>
	 * <LI>Add PayrollAdmin Contact to the Employer</LI>
	 * <LI>Add Legal Information to the Employer</LI>
	 * <LI>Add Tax Expiration Date</LI>
	 * <LI>Add Payroll Frequency</LI>
	 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call workflow</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that Add Company Call to PTS is successful
	 * </LI>
	 * </OL>
	 */

	@Test(enabled=true)
	public void testAddEmployerCompanyWithAllInfo()throws Exception{
		
		deleteCurrentEmployerAndCreateNewEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// add secondary contact
		
		addEmployerContactToCurrentEmployer(EmployerContactType.SecondaryContact);
		
		// add payrollAdmin contact
		
		addEmployerContactToCurrentEmployer(EmployerContactType.PayrollAdminContact);
		
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();

		// Add worklocation
		addWorklocationToCurrentEmployer();
		
		// TODO add TaxExpirationDate
		// TODO add Payroll Frequency Code
		// TODO add DBA Name - different than Legal name
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
		
    	Assert.assertEquals(StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()),
    			true,"Add Company Call is not sent to PSP");	
	}
	
	//TODO:TOIMPLEMENT
	@Test(enabled=true)// implement this after IP001109 fix
	public void testAddCompanyInfoWithLegalNameSameAsDBAName(){
		
	}
	
	//TODO:TOIMPLEMENT
	@Test(enabled=true)// Test to ensure that Main address goes as mailing address to PSP 
	//, when both Main and Other worklocation is available in the system. Should this go under capability test 
	public void testAddCompanyInfoWithSecondaryWorkLocation(){
		
	}
	
	/**
	 * <P>
	 * Test adding Company Info to PTS Without Primary Contact (Primary Contact is mandatory for PTS call to succeed)
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Legal Information to the Employer</LI>
	 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call workflow</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * </LI>
	 * </OL>
	 */
	@Test (enabled=true)
	public void testAddCompanyInfoWithOutPrimaryContact()throws Exception{

		deleteCurrentEmployerAndCreateNewEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();

		// Add work location
		addWorklocationToCurrentEmployer();
		
		try{
			//update the assisted state
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
			Assert.fail("ValidationException is expected while sending Company info without Primary Contact");

		}
		catch(ValidationException ve){
			//TODO hook to PSP Test Web Service for data validation	
		}
			
	}
	
	/**
	 * <P>
	 * Test adding Company Info to PTS Without Legal Name , Legal Address (Legal Name , Legal Address are mandatory for PTS call to succeed)
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Primary Contact to the Employer</LI>
	 * <LI>Add Work location to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call work flow</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * </LI>
	 * </OL>
	 */
	@Test (enabled=true)
	public void testAddCompanyInfoWithOutLegalAddress()throws Exception{

		deleteCurrentEmployerAndCreateNewEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// Add work location
		addWorklocationToCurrentEmployer();
		
		// Remove Legal address
		sEmployerComponent.getEmployer().setLegalAddress(null);
		
		try{
			//update the assisted state
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP 		
			Assert.fail("ValidationException is expected while sending Company info without Legal Address");
			
		}catch(ValidationException ve){
			//TODO hook to PSP Test Web Service for data validation	
		}
	
	}
	
	/**
	 * <P>
	 * Test adding Company Info to PTS With empty EIN (EIN is mandatory for PTS call to succeed)
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Set Employer EIN to empty string ""</LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Primary Contact to the Employer</LI>
	 * <LI>Add Legal info to the Employer</LI>
	 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call workflow</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * </LI>
	 * </OL>
	 */
	@Test (enabled=true)
	public void testAddCompanyInfoWithOutEIN()throws Exception{		

		deleteCurrentEmployerAndCreateNewEmployer();		
		//reset EIN to ""
		setEINToEmployer("");
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();

		// Add work location
		addWorklocationToCurrentEmployer();
		
		try{
			//update the assisted state
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
			Assert.fail("ValidationException is expected while sending Company info without EIN");

		}catch(ValidationException ve){
			//TODO hook to PSP Test Web Service for data validation			
		}

		
	}
	/**
	 * <P>
	 * Test adding Company Info to PTS With null EIN (EIN is mandatory for PTS call to succeed)
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Set Employer EIN to null string </LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Primary Contact to the Employer</LI>
	 * <LI>Add Legal info to the Employer</LI>
	 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call workflow</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * </LI>
	 * </OL>
	 */
	@Test (enabled=true)
	public void testAddCompanyInfoWithNullEIN()throws Exception{
		
	
		deleteCurrentEmployerAndCreateNewEmployer();
		

		//reset EIN to null
		setEINToEmployer(null);
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();

		
		// Add worklocation
		addWorklocationToCurrentEmployer();
		
		try{
			//update the assisted state
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
			Assert.fail("ValidationException is expected while sending Company info with null EIN");

		}
		catch(ValidationException ve){
			//TODO hook to PSP Test Web Service for data validation
		}
		
	}
	
	/**
	 * <P>
	 * Test adding Company Info to PTS With blank EIN (EIN is mandatory for PTS call to succeed)
	 * <href a="http://teamrooms.reno.intuit.com/pdpm/SBDPayroll_CP1/payrollservices/Shared%20Documents/New%20Initiatives/Engineering/PSP/PSP%20%20Gemini%20Integration%20Contract.doc">Gemini-PSP Contract document</href>
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Set Employer EIN to blank string "    "</LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Primary Contact to the Employer</LI>
	 * <LI>Add Legal info to the Employer</LI>
	 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call workflow</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * </LI>
	 * </OL>
	 */
	@Test(enabled=true)
	public void testAddCompanyInfoWithBlankEIN()throws Exception{
		
		deleteCurrentEmployerAndCreateNewEmployer();
		
		//reset EIN to "    "
		setEINToEmployer("      ");
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();
		// Add worklocation
		addWorklocationToCurrentEmployer();
		
		try{
			//update the assisted state
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
			Assert.fail("ValidationException is expected while sending Company info with blank EIN");

		}catch(ValidationException ve){
			//TODO hook to PSP Test Web Service for data validation
		}		
	}
	
	/**
	 * <P>
	 * Test Updating the Legal Info of the Employer after Company has been added to PTS
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Primary Contact to the Employer</LI>
	 * <LI>Add Legal Information to the Employer</LI>
	 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call workflow</LI>
	 * <LI>Assert that PSP call is through</LI>
	 * <LI>Update the Legal Info of the Employer</LI>
	 * </OL>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=false)// AMALIK Need to change the api to kick the workflow of legal name change
	public void testUpdateLegalInfoAfterAddCompanyPTSCall()throws Exception{
		
		deleteCurrentEmployerAndCreateNewEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();
		
		// Add work location
		addWorklocationToCurrentEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
		
    	Assert.assertEquals(StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()),
    			true,"Add Company Call is not sent to PSP");
    	
    	// update the legal info after PSP call
    	EmployerInfo empinfo = sEmployerService.getEmployerInfo();
    	empinfo.setLegalName("BigBlue Corp");
    	empinfo.setLegalAddress(createPhysicalAddress());
    	try{
    		SaveEmployerInfoRequest saveEmployerInfoRequest = new SaveEmployerInfoRequest();
    		saveEmployerInfoRequest.setEmployerInfo(empinfo);		
    		sEmployerService.saveEmployerInfo(saveEmployerInfoRequest);
    		Assert.fail("ValidationException is expected while updating the Legal Info after Add Company PSP call ");
    	}catch(ValidationException ve){
    		
    	}
    	
	}
	
	/**
	 * <P>
	 * Test Updating the Employer Contact after Company has been added to PTS
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Primary Contact to the Employer</LI>
	 * <LI>Add Legal Information to the Employer</LI>
	 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call workflow</LI>
	 * <LI>Assert that PSP call is through</LI>
	 * <LI>Update the Employer Contact</LI>
	 * </OL>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * </LI>
	 * </OL>
	 */

	@Test (enabled=true)
	public void testUpdateEmployerContactAfterAddCompanyPTSCall()throws Exception{
		
		deleteCurrentEmployerAndCreateNewEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();
		
		// Add work location
		addWorklocationToCurrentEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
		
	   	Assert.assertEquals(StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()),
	   			true,"Add Company Call is not sent to PSP");

    	// Update the Primary Contact
		EmployerContact[] primaryContactsFromDB =sEmployerService.getEmployerContacts();
		primaryContactsFromDB[0]= createEmployerContact("Jim", "Pitt", "6568751212", EmployerContactType.PrimaryContact);	
		try{
			sEmployerService.updateEmployerContacts(primaryContactsFromDB);
			Assert.fail("ValidationException is expected while updating the Primary Contact after Add Company PSP call ");
		}
		catch(ValidationException ve){
			
		}
		

	}
	//TODO:TOIMPLEMENT
	@Test(enabled=true)	
	public void testUpdateEINAfterAddCompanyPTSCall()throws Exception{
  
	}
	
	/**
	 * <P>
	 * Test Add Company Info to PTS multiple times
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Delete current employer and Create new Employer with random EIN</LI>
	 * <LI>Update the assisted state of the employer to InitiatedOnBoarding</LI>
	 * <LI>Add Primary Contact to the Employer</LI>
	 * <LI>Add Legal Information to the Employer</LI>
	 * <LI>Add Worklocation to the Employer[Mailing Address]</LI>
	 * <LI>Initiate PSP call workflow</LI>
	 * <LI>Assert that PSP call is through</LI>
	 * <LI>Initiate PSP call workflow again</LI>
	 * </OL>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>Verify that ValidationException is thrown
	 * </LI>
	 * </OL>
	 */
	@Test(enabled=true)
	public void testAddCompanyToPTSMultipleTimes() throws Exception{
		
		deleteCurrentEmployerAndCreateNewEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);		
		
		//add primary contact
		addEmployerContactToCurrentEmployer(EmployerContactType.PrimaryContact);
		
		// add legal info to Employer 
		addLegalInfoToCurrentEmployer();

		// Add worklocation
		addWorklocationToCurrentEmployer();
		
		//update the assisted state
		sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
		
    	Assert.assertEquals(StatusUpdateHandlerUtil.isEntityAddedToPTS(empPTSWfComponent.getPTSEmployer().getEmployer()),
    			true,"Add Company Call is not sent to PSP");
    	
    	try{
        	// send 2nd time
    		//update the assisted state
			sEmployerService.updateAssistedState(AssistedStateType.ASSISTED_FORMS_PRINTED);	//send company info to PSP
        	Assert.fail("ValidationException is expected when Company is added 2nd time for the same employer");
    	}catch(ValidationException ve){
    		
    	}
		
	}
	
	/**Helper Method to delete the current employer and create a new employer with random EIN 
	 * */
	
	private void deleteCurrentEmployerAndCreateNewEmployer() throws Exception{
		//Delete an existing employer
		EmployerComponentTestUtils.deleteCurrentEmployer();
		// create a new employer with EIN
		EmployerComponentTestUtils.createAndSaveNewEmployer();

	}
	
	/**Helper Method to add employer contact to current employer 
	 * */
	
	private void addEmployerContactToCurrentEmployer(EmployerContactType contactType) throws ServiceFailureException, ValidationException{
		
		EmployerContact[] primaryContacts = new EmployerContact[1];
		primaryContacts[0]= createEmployerContact("Brad", "Pitt", "6505551212", contactType);		
		sEmployerService.addEmployerContacts(primaryContacts);
		
	}
	
	/**Helper Method to add Legal Info to current employer 
	 * */
	private void addLegalInfoToCurrentEmployer() throws ServiceFailureException, ValidationException{
		
		EmployerInfo employerInfo = EmployerInfoMapper.mapEmployerInfo(empPTSWfComponent.getPTSEmployer().getEmployer());
		
		PhysicalAddress legalAddress = createPhysicalAddress();
		employerInfo.setLegalAddress(legalAddress);
		
		//add legal Name
		employerInfo.setLegalName("Red Chilli Corp");
		
		// Company Type is mandatory field to save the company
		employerInfo.setCompanyType(CompanyType.CORPORATION);
		//save
		SaveEmployerInfoRequest saveEmployerInfoRequest = new SaveEmployerInfoRequest();
		saveEmployerInfoRequest.setEmployerInfo(employerInfo);		
		sEmployerService.saveEmployerInfo(saveEmployerInfoRequest);
		
	}
	
	/**Helper Method to add work location to current employer 
	 * */
	private void addWorklocationToCurrentEmployer() throws ServiceFailureException, ValidationException{
		WorkLocation[] wrkLocations= new WorkLocation[1];
		wrkLocations[0]=createWorkLocation("MV", "2234567656");
		
		sEmployerService.addWorkLocations(wrkLocations);

		
	}
	
	/**Helper Method to set EIN to current employer 
	 * */
	private void setEINToEmployer(String ein) throws ServiceFailureException{
		
		try {
			EmployerTaxIdInfo[] taxIdInfos = sEmployerTaxSettingsService.getEmployerTaxIDs();
			//assertEmployerTaxIdInfo(taxIdInfos);
			for (EmployerTaxIdInfo employerTaxIdInfo : taxIdInfos) {
				if (employerTaxIdInfo.getTaxAgency().name.contains("US_IRS")) {
					employerTaxIdInfo.setTaxId("");				
					break;
				}
			}
			sEmployerTaxSettingsService.updateEmployerTaxIDs(taxIdInfos);
		} catch (ValidationException ex) {
			Assert.fail("Validation Exception thrown " + ex.getMessage());
		}

		
	}

}
