package com.intuit.sb.payroll.qa.reports;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.common.NameValuePair;
import com.intuit.sb.payroll.capability.common.RowData;
import com.intuit.sb.payroll.capability.common.XMLUtil;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponentTestBase;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.reports.ReportDetail;
import com.intuit.sb.payroll.reports.ReportRequest;
import com.intuit.sb.payroll.reports.ReportResponse;
import com.intuit.sb.payroll.reports.ReportService;
import com.intuit.sb.payroll.reports.ReportSubtotals;
import com.intuit.sb.payroll.reports.ReportType;

/**
 * <P>
 * The <CODE>GetReportTest</CODE> class contains tests for testing getReport in ReportService
 * </P>
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 * 
 */
public class GetReportTest extends WorkerPaymentsComponentTestBase {
	
	private static Long sWorkerIds[] = {0L, 0L};
	private static ReportService reportService = null;
	
	@BeforeClass
	public static void setUpGetReportTest() throws Exception {
    	reportService = qaBootstrap.getEjbFromClass(ReportService.class);
		
		getUserTransaction().begin();
		saveEmployerInfo();
		saveWorkLocationAddress();
		setUpSanMateoAddress();
		
		// Create 2 workers
		sWorkerIds[0] = WorkerTestUtils.createSalaryPayrollReadyWorker("Bonnie", sMountainViewLocation);
		sWorkerIds[1] = WorkerTestUtils.createSalaryPayrollReadyWorker("Clyde", sMountainViewLocation);
		getUserTransaction().commit();
		
		// Call static method to create paychecks, pass in the 2 workers
		WorkerPaymentsComponentTestBase.createScheduledPayrollRuns(sWorkerIds);
		
		    	
	}

	@AfterClass
	public static void tearDownGetReportTest() throws Exception {
		getUserTransaction().begin();
    	//Tear down
		// Call static tear down method from base class - pass in 2 workers
		WorkerPaymentsComponentTestBase.deleteCreatedScheduledPayrollRuns(sWorkerIds);
		restoreWorkLocationAddress();
		restoreEmployerInfo();
		getUserTransaction().commit();
	}
	
	/***************************** Report sub-total by Paycheck date *****************************/
	
    /**
     * <P>Test getting report for 1st quarter 2008, sub-totaled by paycheck date.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the 1st quarter in 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_1stQuarter2008() throws Exception {
		// Setup
		String reportTitle = "Q1 2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.MARCH, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_PAYCHECK, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"01/17/2008", "$135.48", "$20.31", "$86.31", "$2.54", "$244.64"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"01/31/2008", "$135.48", "$20.31", "$86.31", "$2.54", "$244.64"};
		expectedList.add(initRowData(column3Values));
		// 4th column
		String[] column4Values = {"02/14/2008", "$135.48", "$20.30", "$65.38", "$1.92", "$223.08"};
		expectedList.add(initRowData(column4Values));
		// 5th column
		String[] column5Values = {"02/28/2008", "$135.48", "$20.31", "$0.00", "$0.00", "$155.79"};
		expectedList.add(initRowData(column5Values));
		// 6th column
		String[] column6Values = {"03/13/2008", "$135.48", "$20.31", "$0.00", "$0.00", "$155.79"};
		expectedList.add(initRowData(column6Values));
		// 7th column
		String[] column7Values = {"03/27/2008", "$270.96", "$40.62", "$86.31", "$2.54", "$400.43"};
		expectedList.add(initRowData(column7Values));
		// 8th column
		String[] column8Values = {"TOTAL", "$948.36", "$142.16", "$324.31", "$9.54", "$1,424.37"};
		expectedList.add(initRowData(column8Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for January 2008, sub-totaled by paycheck date.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for January 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_Jan2008() throws Exception {
		// Setup
		String reportTitle = "January 2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.JANUARY, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_PAYCHECK, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"01/17/2008", "$135.48", "$20.31", "$86.31", "$2.54", "$244.64"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"01/31/2008", "$135.48", "$20.31", "$86.31", "$2.54", "$244.64"};
		expectedList.add(initRowData(column3Values));
		// 4th column
		String[] column4Values = {"TOTAL", "$270.96", "$40.62", "$172.62", "$5.08", "$489.28"};
		expectedList.add(initRowData(column4Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for 03/14/2008 to 03/27/2008, sub-totaled by paycheck date.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the period 03/14/2008 to 03/27/2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_03142008_To_03272008() throws Exception {
		// Setup
		String reportTitle = "03/14/2008 - 03/27/2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.MARCH, 14);
		Date toDate = DateUtil.createDate(2008, Calendar.MARCH, 27);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_PAYCHECK, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"03/27/2008", "$270.96", "$40.62", "$86.31", "$2.54", "$400.43"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"TOTAL", "$270.96", "$40.62", "$86.31", "$2.54", "$400.43"};
		expectedList.add(initRowData(column3Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for 03/13/2008 to 03/13/2008, sub-totaled by paycheck date.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the period 03/13/2008 to 03/13/2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_03132008_To_03132008() throws Exception {
		// Setup
		String reportTitle = "03/13/2008 - 03/13/2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.MARCH, 13);
		Date toDate = DateUtil.createDate(2008, Calendar.MARCH, 13);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_PAYCHECK, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"03/13/2008", "$135.48", "$20.31", "$0.00", "$0.00", "$155.79"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"TOTAL", "$135.48", "$20.31", "$0.00", "$0.00", "$155.79"};
		expectedList.add(initRowData(column3Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for 12/18/2008 to 01/01/2009, sub-totaled by paycheck date.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the period 12/18/2008 to 01/01/2009</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_12182008_To_01012009() throws Exception {
		// Setup
		String reportTitle = "12/18/2008 - 01/01/2009";
		Date fromDate = DateUtil.createDate(2008, Calendar.DECEMBER, 18);
		Date toDate = DateUtil.createDate(2009, Calendar.JANUARY, 1);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_PAYCHECK, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"12/18/2008", "$270.96", "$40.62", "$0.00", "$0.00", "$311.58"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"01/01/2009", "$136.43", "$27.92", "$86.31", "$2.54", "$253.20"};
		expectedList.add(initRowData(column3Values));
		// 4th column
		String[] column4Values = {"TOTAL", "$407.39", "$68.54", "$86.31", "$2.54", "$564.78"};
		expectedList.add(initRowData(column4Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for 03/17/2008 to 03/21/2008, sub-totaled by paycheck date.  NOTE: There aren't any
     *  paychecks in this period.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the period 03/17/2008 to 03/21/2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_03172008_To_03212008() throws Exception {
		// Setup
		String reportTitle = "03/17/2008 - 03/21/2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.MARCH, 17);
		Date toDate = DateUtil.createDate(2008, Calendar.MARCH, 21);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_PAYCHECK, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"TOTAL", "$0.00", "$0.00", "$0.00", "$0.00", "$0.00"};
		expectedList.add(initRowData(column2Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

	/***************************** Report sub-total by month *****************************/
	
    /**
     * <P>Test getting report for January 2008, sub-totaled by month.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for January 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_Jan2008_ByMonth() throws Exception {
		// Setup
		String reportTitle = "January 2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.JANUARY, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_MONTH, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"Jan 2008", "$270.96", "$40.62", "$172.62", "$5.08", "$489.28"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"TOTAL", "$270.96", "$40.62", "$172.62", "$5.08", "$489.28"};
		expectedList.add(initRowData(column3Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for 1st quarter 2008, sub-totaled by month.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the 1st quarter in 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_1stQuarter2008_ByMonth() throws Exception {
		// Setup
		String reportTitle = "Q1 2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.MARCH, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_MONTH, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"Jan 2008", "$270.96", "$40.62", "$172.62", "$5.08", "$489.28"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"Feb 2008", "$270.96", "$40.61", "$65.38", "$1.92", "$378.87"};
		expectedList.add(initRowData(column3Values));
		// 4th column
		String[] column4Values = {"Mar 2008", "$406.44", "$60.93", "$86.31", "$2.54", "$556.22"};
		expectedList.add(initRowData(column4Values));
		// 5th column
		String[] column5Values = {"TOTAL", "$948.36", "$142.16", "$324.31", "$9.54", "$1,424.37"};
		expectedList.add(initRowData(column5Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for year 2008, sub-totaled by month.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the year 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_Year2008_ByMonth() throws Exception {
		// Setup
		String reportTitle = "2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.DECEMBER, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_MONTH, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"Jan 2008", "$270.96", "$40.62", "$172.62", "$5.08", "$489.28"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"Feb 2008", "$270.96", "$40.61", "$65.38", "$1.92", "$378.87"};
		expectedList.add(initRowData(column3Values));
		// 4th column
		String[] column4Values = {"Mar 2008", "$406.44", "$60.93", "$86.31", "$2.54", "$556.22"};
		expectedList.add(initRowData(column4Values));
		// 5th column
		String[] column5Values = {"Apr 2008", "$270.96", "$40.61", "$0.00", "$0.00", "$311.57"};
		expectedList.add(initRowData(column5Values));
		// 6th column
		String[] column6Values = {"May 2008", "$270.96", "$40.62", "$0.00", "$0.00", "$311.58"};
		expectedList.add(initRowData(column6Values));
		// 7th column
		String[] column7Values = {"Jun 2008", "$406.44", "$60.92", "$86.31", "$2.54", "$556.21"};
		expectedList.add(initRowData(column7Values));
		// 8th column
		String[] column8Values = {"Jul 2008", "$406.44", "$60.93", "$0.00", "$0.00", "$467.37"};
		expectedList.add(initRowData(column8Values));
		// 9th column
		String[] column9Values = {"Aug 2008", "$270.96", "$40.61", "$0.00", "$0.00", "$311.57"};
		expectedList.add(initRowData(column9Values));
		// 10th column
		String[] column10Values = {"Sep 2008", "$406.44", "$60.92", "$65.38", "$1.92", "$534.66"};
		expectedList.add(initRowData(column10Values));
		// 11th column
		String[] column11Values = {"Oct 2008", "$270.96", "$40.61", "$0.00", "$0.00", "$311.57"};
		expectedList.add(initRowData(column11Values));
		// 12th column
		String[] column12Values = {"Nov 2008", "$270.96", "$40.62", "$0.00", "$0.00", "$311.58"};
		expectedList.add(initRowData(column12Values));
		// 13th column
		String[] column13Values = {"Dec 2008", "$406.44", "$60.92", "$0.00", "$0.00", "$467.36"};
		expectedList.add(initRowData(column13Values));
		// 14th column
		String[] column14Values = {"TOTAL", "$3,928.92", "$588.92", "$476.00", "$14.00", "$5,007.84"};
		expectedList.add(initRowData(column14Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for 12/01/2008 to 01/31/2009, sub-totaled by month.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for 12/01/2008 to 01/31/2009</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_12012008_To01312009_ByMonth() throws Exception {
		// Setup
		String reportTitle = "12/01/2008 - 01/31/2009";
		Date fromDate = DateUtil.createDate(2008, Calendar.DECEMBER, 1);
		Date toDate = DateUtil.createDate(2009, Calendar.JANUARY, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_MONTH, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column13Values = {"Dec 2008", "$406.44", "$60.92", "$0.00", "$0.00", "$467.36"};
		expectedList.add(initRowData(column13Values));
		// 3rd column
		String[] column2Values = {"Jan 2009", "$136.43", "$27.92", "$86.31", "$2.54", "$253.20"};
		expectedList.add(initRowData(column2Values));
		// 4th column
		String[] column14Values = {"TOTAL", "$542.87", "$88.84", "$86.31", "$2.54", "$720.56"};
		expectedList.add(initRowData(column14Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

	/***************************** Report sub-total by quarter *****************************/
	
    /**
     * <P>Test getting report for 1st quarter 2008, sub-totaled by quarter.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the 1st quarter in 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_1stQuarter2008_ByQuarter() throws Exception {
		// Setup
		String reportTitle = "Q1 2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.MARCH, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_QUARTER, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"Q1 2008", "$948.36", "$142.16", "$324.31", "$9.54", "$1,424.37"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"TOTAL", "$948.36", "$142.16", "$324.31", "$9.54", "$1,424.37"};
		expectedList.add(initRowData(column3Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for year 2008, sub-totaled by quarter.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the year 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_Year2008_ByQuarter() throws Exception {
		// Setup
		String reportTitle = "2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.DECEMBER, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_QUARTER, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"Q1 2008", "$948.36", "$142.16", "$324.31", "$9.54", "$1,424.37"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"Q2 2008", "$948.36", "$142.15", "$86.31", "$2.54", "$1,179.36"};
		expectedList.add(initRowData(column3Values));
		// 4th column
		String[] column4Values = {"Q3 2008", "$1,083.84", "$162.46", "$65.38", "$1.92", "$1,313.60"};
		expectedList.add(initRowData(column4Values));
		// 5th column
		String[] column5Values = {"Q4 2008", "$948.36", "$142.15", "$0.00", "$0.00", "$1,090.51"};
		expectedList.add(initRowData(column5Values));
		// 6th column
		String[] column6Values = {"TOTAL", "$3,928.92", "$588.92", "$476.00", "$14.00", "$5,007.84"};
		expectedList.add(initRowData(column6Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for 01/01/2008 to 03/31/2009, sub-totaled by quarter.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for 01/01/2008 to 03/31/2009</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_01012008_To_03312009_ByQuarter() throws Exception {
		// Setup
		String reportTitle = "01/01/2008 - 03/31/2009";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2009, Calendar.MARCH, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_QUARTER, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"Q1 2008", "$948.36", "$142.16", "$324.31", "$9.54", "$1,424.37"};
		expectedList.add(initRowData(column2Values));
		// 3rd column
		String[] column3Values = {"Q2 2008", "$948.36", "$142.15", "$86.31", "$2.54", "$1,179.36"};
		expectedList.add(initRowData(column3Values));
		// 4th column
		String[] column4Values = {"Q3 2008", "$1,083.84", "$162.46", "$65.38", "$1.92", "$1,313.60"};
		expectedList.add(initRowData(column4Values));
		// 5th column
		String[] column5Values = {"Q4 2008", "$948.36", "$142.15", "$0.00", "$0.00", "$1,090.51"};
		expectedList.add(initRowData(column5Values));
		// 6th column
		String[] column6Values = {"Q1 2009", "$136.43", "$27.92", "$86.31", "$2.54", "$253.20"};
		expectedList.add(initRowData(column6Values));
		// 7th column
		String[] column7Values = {"TOTAL", "$4,065.35", "$616.84", "$562.31", "$16.54", "$5,261.04"};
		expectedList.add(initRowData(column7Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for year 2007, sub-totaled by quarter.  NOTE: There aren't any
     *  paychecks in this period.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for year 2007</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_Year2007_ByQuarter() throws Exception {
		// Setup
		String reportTitle = "2007";
		Date fromDate = DateUtil.createDate(2007, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2007, Calendar.DECEMBER, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.PER_QUARTER, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"TOTAL", "$0.00", "$0.00", "$0.00", "$0.00", "$0.00"};
		expectedList.add(initRowData(column2Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

	/***************************** Report with no sub-total *****************************/
	
    /**
     * <P>Test getting report for August 2008, no sub-total.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for August 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_August2008_NoSubtotal() throws Exception {
		// Setup
		String reportTitle = "Aug 2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.AUGUST, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.AUGUST, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.NOT_SET, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"TOTAL", "$270.96", "$40.61", "$0.00", "$0.00", "$311.57"};
		expectedList.add(initRowData(column2Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for 1st quarter 2008, no sub-total.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for the 1st quarter in 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_1stQuarter2008_NoSubtotal() throws Exception {
		// Setup
		String reportTitle = "Q1 2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.MARCH, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.NOT_SET, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"TOTAL", "$948.36", "$142.16", "$324.31", "$9.54", "$1,424.37"};
		expectedList.add(initRowData(column2Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for year 2008, no sub-total.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for year 2008</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_Year2008_NoSubtotal() throws Exception {
		// Setup
		String reportTitle = "2008";
		Date fromDate = DateUtil.createDate(2008, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2008, Calendar.DECEMBER, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.NOT_SET, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"TOTAL", "$3,928.92", "$588.92", "$476.00", "$14.00", "$5,007.84"};
		expectedList.add(initRowData(column2Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

    /**
     * <P>Test getting report for year 2007, no sub-total.  NOTE: There aren't any paychecks in this period.
     * </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call ReportService.getReport() to get a report for year 2007</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the returned data are correct</LI>
     * </UL>
     */
	@Test
	public void testGetReport_Year2007_NoSubtotal() throws Exception {
		// Setup
		String reportTitle = "2007";
		Date fromDate = DateUtil.createDate(2007, Calendar.JANUARY, 1);
		Date toDate = DateUtil.createDate(2007, Calendar.DECEMBER, 31);
		ReportRequest reportRequest = new ReportRequest(ReportType.CA_TAX_LIABILITIES, fromDate, toDate, 
				ReportSubtotals.NOT_SET, ReportDetail.COMPLETE, reportTitle);
		// Exercise
		ReportResponse response = reportService.getReport(reportRequest);
		// Verify
		List<RowData> computedList = XMLUtil.getRowData(response.getXmlData());
		List<RowData> expectedList = new ArrayList<RowData>();
		// 1st column
		String[] column1Values = {reportTitle, "CA Personal Income Tax", "CA State Disability Insurance",
				"CA State Unemployment Insurance", "CA Employment Training Tax", "TOTAL DUE"};
		expectedList.add(initRowData(column1Values));
		// 2nd column
		String[] column2Values = {"TOTAL", "$0.00", "$0.00", "$0.00", "$0.00", "$0.00"};
		expectedList.add(initRowData(column2Values));
		
		AssertJUnit.assertTrue("Returned data mismatch!", verifyRowDataList(expectedList, computedList));
	}

	private RowData initRowData(String[] values) {
		RowData rowData = new RowData();
		for (String value : values) {
			rowData.addNameValuePair(new NameValuePair(XMLUtil.columnTag, value));
		}
		return rowData;
	}
	
	private boolean verifyRowDataList(List<RowData> expected, List<RowData> computed) {
		boolean result = expected.equals(computed);
		if (false == result) {
			sLogger.error("Expected RowData values:");
			for (RowData rowData : expected) {
				sLogger.error(rowData.toString());
			}
			sLogger.error("Computed RowData values:");
			for (RowData rowData : computed) {
				sLogger.error(rowData.toString());
			}
		}
		return result;
	}
}
