package com.intuit.sb.payroll.qa.services.benefits;

import java.math.BigDecimal;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.ExceptionUtils;
import com.intuit.sb.payroll.services.benefits.BenefitsServiceTestBase;
import com.intuit.sb.payroll.services.benefits.HealthBenefit;
import com.intuit.sb.payroll.services.benefits.HealthBenefitResponse;
import com.intuit.sb.payroll.services.benefits.HealthBenefitTemplate;
import com.intuit.sb.payroll.services.benefits.HealthBenefitType;
import com.intuit.sb.payroll.services.common.ObjectHandle;

/**
 * Tests the HealthBenefits operation of the BenefitService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class HealthBenefitsTest extends BenefitsServiceTestBase {

    /**
     * <P>Test calling addHealthBenefits with a null workerHandle and a valid array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's addHealthBenefits() with a null workerHandle and a valid array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testAddHealthBenefitsWithNullObjectHandle() throws Exception {
        // Setup
        HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[1];
        healthBenefitsToAdd[0] = createHealthBenefit();
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.addHealthBenefits(null, healthBenefitsToAdd);
    }

    /**
     * <P>Test calling addHealthBenefits with a valid workerHandle and an empty array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's addHealthBenefits() with a valid workerHandle and an empty array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testAddHealthBenefitsWithEmptyArray() throws Exception {
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, new HealthBenefit[0]);
    }

    /**
     * <P>Test calling addHealthBenefits with a valid workerHandle and a null array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's addHealthBenefits() with a valid workerHandle and a null array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testAddHealthBenefitsWithNullArray() throws Exception {
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, null);
    }

    /**
     * <P>Test calling addHealthBenefits with a valid workerHandle and a null array element</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a HealthBenefit array with 2 elements, 1 valid and 1 null element</LI>
     * <LI>Call BenefitService's addHealthBenefits() with a valid workerHandle and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testAddHealthBenefitsWithNullArrayElement() throws Exception {
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[2];
        healthBenefitsToAdd[0] = createHealthBenefit();
        healthBenefitsToAdd[1] = null;
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
    }

    /**
     * <P>Test calling addHealthBenefits with a non-existent workerId in the workerHandle</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a HealthBenefit array with 1 valid element</LI>
     * <LI>Call BenefitService's addHealthBenefits() with a non-existent workerId in the workerHandle and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testAddHealthBenefitsWithNonExistentWorkerId() throws Exception {
        // Setup
        ObjectHandle workerHandle = new ObjectHandle(934893442L, 0L);
        HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[1];
        healthBenefitsToAdd[0] = createHealthBenefit();
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
    }

    /**
     * <P>Test adding a HealthBenefit with a valid workerHandle and a Null BenefitType</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Retrieve the Employer's HealthBenefitTemplate</LI>
     * <LI>Create a HealthBenefit array with 1 element using the HealthBenefitTemplate, but don't set the BenefitType value</LI>
     * <LI>Call BenefitService's addHealthBenefits() with a valid workerHandle and and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testAddHealthBenefitWithNullBenefitType() throws Exception {
        boolean added = false;
        HealthBenefitResponse healthBenefitResponse = null;
        HealthBenefit[] healthBenefits = null;
        try {
            // Setup
            ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
            HealthBenefitTemplate healthBenefitTemplate = sBenefitsService.findEmployerHealthBenefit(HealthBenefitType.HEALTH);
            HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[1];
            healthBenefitsToAdd[0] = new HealthBenefit();
            healthBenefitsToAdd[0].setObjectHandle(null);
            healthBenefitsToAdd[0].setName(healthBenefitTemplate.getName());
            healthBenefitsToAdd[0].setRate(new BigDecimal(150.00));
            // Exercise
            healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
            healthBenefits = healthBenefitResponse.getBenefits();
            added = true;
        } finally {
            // Cleanup
            if (added) {
                sBenefitsService.deleteHealthBenefits(healthBenefitResponse.getWorker().getObjectHandle(), healthBenefits);
            }
        }
    }

    /**
     * <P>Test adding a HealthBenefit to a worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a HealthBenefit array with 1 valid element</LI>
     * <LI>Call BenefitService's addHealthBenefits() with a valid workerHandle and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the HealthBenefit is added to the worker
     * </UL>
     */
    @Test
    public void testAddHealthBenefit() throws Exception {
        getUserTransaction().commit();
        boolean added = false;
        HealthBenefitResponse healthBenefitResponse = null;
        HealthBenefit[] healthBenefits = null;
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[1];
        healthBenefitsToAdd[0] = createHealthBenefit();
        try {
            // Exercise
            healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
            healthBenefits = healthBenefitResponse.getBenefits();
            added = true;
            // Verify
            AssertJUnit.assertEquals("Only expecting 1 health benefit item!", 1, healthBenefits.length);
            // Fix up the objectHandle value
            healthBenefitsToAdd[0].setObjectHandle(healthBenefits[0].getObjectHandle());
            AssertJUnit.assertTrue("HealthBenefit added does not match expected!", verifyHealthBenefits(healthBenefitsToAdd,
                                                                                                   healthBenefits));
        } finally {
            // Cleanup
            if (added) {
                sBenefitsService.deleteHealthBenefits(healthBenefitResponse.getWorker().getObjectHandle(), healthBenefits);
            }
        }
    }

    /**
     * <P>Test adding a HealthBenefit to a worker using the HealthBenefitTemplate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Retrieve the Employer's HealthBenefitTemplate</LI>
     * <LI>Create a HealthBenefit array with 1 valid element using the HealthBenefitTemplate</LI>
     * <LI>Call BenefitService's addHealthBenefits() with a valid workerHandle and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the HealthBenefit is added to the worker
     * </UL>
     */
    @Test
    public void testAddHealthBenefitUsingTemplate() throws Exception {
        getUserTransaction().commit();
        boolean added = false;
        HealthBenefitResponse healthBenefitResponse = null;
        HealthBenefit[] healthBenefits = null;
        try {
            // Setup
            ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
            HealthBenefitTemplate healthBenefitTemplate = sBenefitsService.findEmployerHealthBenefit(HealthBenefitType.HEALTH);
            HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[1];
            healthBenefitsToAdd[0] = new HealthBenefit();
            healthBenefitsToAdd[0].setObjectHandle(null);
            healthBenefitsToAdd[0].setName(healthBenefitTemplate.getName());
            healthBenefitsToAdd[0].setType(healthBenefitTemplate.getType());
            BigDecimal bigDecimal = new BigDecimal(150.00);
            // set to 2 decimal places
            bigDecimal = bigDecimal.setScale(2);
            healthBenefitsToAdd[0].setRate(bigDecimal);
            // Exercise
            healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
            healthBenefits = healthBenefitResponse.getBenefits();
            added = true;
            // Verify
            AssertJUnit.assertEquals("Only expecting 1 health benefit item!", 1, healthBenefits.length);
            // Fix up the objectHandle value
            healthBenefitsToAdd[0].setObjectHandle(healthBenefits[0].getObjectHandle());
            AssertJUnit.assertTrue("HealthBenefit added does not match expected!", verifyHealthBenefits(healthBenefitsToAdd,
                                                                                                   healthBenefits));
        } finally {
            // Cleanup
            if (added) {
                sBenefitsService.deleteHealthBenefits(healthBenefitResponse.getWorker().getObjectHandle(), healthBenefits);
            }
        }
    }

    /**
     * <P>Test adding a duplicate HealthBenefit to a worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a HealthBenefit array with 1 valid element</LI>
     * <LI>Call BenefitService's addHealthBenefits() with a valid workerHandle and the HealthBenefit array</LI>
     * <LI>Call BenefitService's addHealthBenefits() again to add the duplicate HealthBenefit</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (enabled=false, expectedExceptions=ValidationException.class)
    public void testAddDuplicateHealthBenefit() throws Exception {
    	getUserTransaction().commit();
        boolean added = false;
        HealthBenefitResponse healthBenefitResponse = null;
        HealthBenefit[] healthBenefits = null;
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[1];
        healthBenefitsToAdd[0] = createHealthBenefit();
        try {
            // Exercise
            healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
            healthBenefits = healthBenefitResponse.getBenefits();
            added = true;
            @SuppressWarnings("unused")
            HealthBenefitResponse healthBenefitResponse2 = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
        } finally {
            // Cleanup
            if (added) {
                sBenefitsService.deleteHealthBenefits(healthBenefitResponse.getWorker().getObjectHandle(), healthBenefits);
            }
        }
    }

    /**
     * <P>Test calling deleteHealthBenefits with a null workerHandle and a valid array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's deleteHealthBenefits() with a null workerHandle and a valid array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testDeleteHealthBenefitsWithNullObjectHandle() throws Exception {
        // Setup
        HealthBenefit[] healthBenefitsToDelete = new HealthBenefit[1];
        healthBenefitsToDelete[0] = createHealthBenefit();
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.deleteHealthBenefits(null, healthBenefitsToDelete);
    }

    /**
     * <P>Test calling deleteHealthBenefits with a valid workerHandle and an empty array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's deleteHealthBenefits() with a valid workerHandle and an empty array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testDeleteHealthBenefitsWithEmptyArray() throws Exception {
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.deleteHealthBenefits(workerHandle, new HealthBenefit[0]);
    }

    /**
     * <P>Test calling deleteHealthBenefits with a valid workerHandle and a null array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's deleteHealthBenefits() with a valid workerHandle and a null array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testDeleteHealthBenefitsWithNullArray() throws Exception {
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.deleteHealthBenefits(workerHandle, null);
    }

    /**
     * <P>Test calling deleteHealthBenefits with a valid workerHandle and a null array element</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a HealthBenefit array with 2 elements, 1 valid and 1 null element</LI>
     * <LI>Call BenefitService's deleteHealthBenefits() with a valid workerHandle and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that {@link ServiceFailureException} is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testDeleteHealthBenefitsWithNullArrayElement() throws Exception {
        HealthBenefit[] healthBenefits = null;
        try {
            // Setup
            healthBenefits = addHealthBenefitToWorker(sWorkerId);
            ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
            HealthBenefit[] healthBenefitsToDelete = new HealthBenefit[2];
            healthBenefitsToDelete[0] = healthBenefits[0];
            healthBenefitsToDelete[1] = null;
            // Exercise
            @SuppressWarnings("unused")
            HealthBenefitResponse healthBenefitResponse = sBenefitsService.deleteHealthBenefits(workerHandle, healthBenefitsToDelete);
        } finally {
            // Cleanup
            if (null != healthBenefits) {
                sBenefitsService.deleteHealthBenefits(getWorkerObjectHandle(sWorkerId), healthBenefits);
            }
        }
    }

    /**
     * <P>Test calling deleteHealthBenefits with a non-existent workerId in the workerHandle</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a HealthBenefit array with 1 valid element</LI>
     * <LI>Call BenefitService's deleteHealthBenefits() with a non-existent workerId in the workerHandle and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testDeleteHealthBenefitsWithNonExistentWorkerId() throws Exception {
    	getUserTransaction().commit();
        HealthBenefit[] healthBenefits = null;
        try {
            // Setup
            healthBenefits = addHealthBenefitToWorker(sWorkerId);
            HealthBenefit[] healthBenefitsToDelete = new HealthBenefit[1];
            healthBenefitsToDelete[0] = healthBenefits[0];
            ObjectHandle workerHandle = new ObjectHandle(934893443L, 0L);
            // Exercise
            @SuppressWarnings("unused")
            HealthBenefitResponse healthBenefitResponse = sBenefitsService.deleteHealthBenefits(workerHandle, healthBenefitsToDelete);
        } finally {
            getUserTransaction().begin();
            // Cleanup
            if (null != healthBenefits) {
                sBenefitsService.deleteHealthBenefits(getWorkerObjectHandle(sWorkerId), healthBenefits);
            }
            getUserTransaction().commit();
        }
    }

    /**
     * <P>Test deleting HealthBenefits</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a HealthBenefit for a worker</LI>
     * <LI>Call BenefitService's deleteHealthBenefits() to delete the healthBenefit</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the HealthBenefit is deleted
     * </UL>
     */
    @Test
    public void testDeleteHealthBenefits() throws Exception {
        getUserTransaction().commit();
        // Setup
        HealthBenefit[] healthBenefits = addHealthBenefitToWorker(sWorkerId);
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.deleteHealthBenefits(workerHandle, healthBenefits);
        // Verify
        AssertJUnit.assertEquals("HealthBenefit not deleted!", 0, healthBenefitResponse.getBenefits().length);
    }

    /**
     * <P>Test calling findAllHealthBenefits with a null workerHandle</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's findAllHealthBenefits() with a null workerHandle</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testFindAllHealthBenefitsWithNullObjectHandle() throws Exception {
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefit[] healthBenefits = sBenefitsService.findAllHealthBenefits(null);
    }

    /**
     * <P>Test calling findAllHealthBenefits with a non-existent workerId in the workerHandle</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's findAllHealthBenefits() with a non-existent workerId in the workerHandle</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testFindAllHealthBenefitsWithNonExistentWorkerId() throws Exception {
        // Setup
        ObjectHandle workerHandle = new ObjectHandle(934893442L, 0L);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefit[] healthBenefits = sBenefitsService.findAllHealthBenefits(workerHandle);
    }

    /**
     * <P>Test getting HealthBenefit for a worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup HealthBenefit for a worker</LI>
     * <LI>Call BenefitService's findAllHealthBenefits() with a valid workerHandle</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the HealthBenefit is as expected
     * </UL>
     */
    @Test
    public void testFindAllHealthBenefit() throws Exception {
        getUserTransaction().commit();
        boolean added = false;
        HealthBenefit[] healthBenefitsAdded = null;
        try {
            // Setup
            healthBenefitsAdded = addHealthBenefitToWorker(sWorkerId);
            added = true;
            ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
            // Exercise
            HealthBenefit[] healthBenefits = sBenefitsService.findAllHealthBenefits(workerHandle);
            // Verify
            AssertJUnit.assertTrue("Expected HealthBenefit mismatch!", verifyHealthBenefits(healthBenefitsAdded,
                                                                                       healthBenefits));
        } finally {
            // Cleanup
            if (added) {
                sBenefitsService.deleteHealthBenefits(getWorkerObjectHandle(sWorkerId), healthBenefitsAdded);
            }
        }
    }

    /**
     * <P>Test getting HealthBenefit for a worker with no HealthBenefits</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's findAllHealthBenefits() with a valid workerHandle</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the HealthBenefit is empty
     * </UL>
     */
    @Test
    public void testFindHealthBenefitForWorkerWithNoHealthBenefit() throws Exception {
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        HealthBenefit[] healthBenefits = sBenefitsService.findAllHealthBenefits(workerHandle);
        // Verify
        AssertJUnit.assertEquals("Expected HealthBenefit is not empty!", 0, healthBenefits.length);
     }

    /**
     * <P>Test calling updateHealthBenefits with a null workerHandle and a valid array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's updateHealthBenefits() with a null workerHandle and a valid array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testUpdateHealthBenefitsWithNullObjectHandle() throws Exception {
        // Setup
        HealthBenefit[] healthBenefitsToUpdate = new HealthBenefit[1];
        healthBenefitsToUpdate[0] = createHealthBenefit();
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.updateHealthBenefits(null, healthBenefitsToUpdate);
    }

    /**
     * <P>Test calling updateHealthBenefits with a valid workerHandle and an empty array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's updateHealthBenefits() with a valid workerHandle and an empty array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testUpdateHealthBenefitsWithEmptyArray() throws Exception {
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.updateHealthBenefits(workerHandle, new HealthBenefit[0]);
    }

    /**
     * <P>Test calling updateHealthBenefits with a valid workerHandle a null array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's updateHealthBenefits() with a valid workerHandle a null array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testUpdateHealthBenefitsWithNullArray() throws Exception {
        // Setup
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.updateHealthBenefits(workerHandle, null);
    }

    /**
     * <P>Test calling updateHealthBenefits with a valid workerHandle a null array element</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a HealthBenefit array with 2 elements, 1 valid and 1 null element</LI>
     * <LI>Call BenefitService's updateHealthBenefits() with a valid workerHandle the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testUpdateHealthBenefitsWithNullArrayElement() throws Exception {
        // Setup
        HealthBenefit[] healthBenefitsToUpdate = new HealthBenefit[1];
        healthBenefitsToUpdate[0] = null;
        ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.updateHealthBenefits(workerHandle, healthBenefitsToUpdate);
    }

    /**
     * <P>Test calling updateHealthBenefits with a non-existent workerId in the workerHandle</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a HealthBenefit for a worker</LI>
     * <LI>Call BenefitService's updateHealthBenefits() with a non-existent workerId in the workerHandle and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testUpdateHealthBenefitsWithNonExistentWorkerId() throws Exception {
        // Setup
        HealthBenefit[] healthBenefitsToUpdate = addHealthBenefitToWorker(sWorkerId);
        ObjectHandle workerHandle = new ObjectHandle(934893446L, 0L);
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitResponse healthBenefitResponse = sBenefitsService.updateHealthBenefits(workerHandle, healthBenefitsToUpdate);
    }

    /**
     * <P>Test updating a HealthBenefit for a worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup a HealthBenefit for a worker</LI>
     * <LI>Change some values in the HealthBenefit</LI>
     * <LI>Call BenefitService's updateHealthBenefits() with a valid workerHandle and the HealthBenefit array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the HealthBenefit is updated
     * </UL>
     */
    @Test
    public void testUpdateHealthBenefit() throws Exception {
        getUserTransaction().commit();
        boolean added = false;
        HealthBenefit[] healthBenefits = null;
        try {
            // Setup
            healthBenefits = addHealthBenefitToWorker(sWorkerId);
            added = true;
            ObjectHandle workerHandle = getWorkerObjectHandle(sWorkerId);
            HealthBenefit[] healthBenefitsToUpdate = new HealthBenefit[1];
            healthBenefitsToUpdate[0] = healthBenefits[0];
            // Make some changes
            healthBenefitsToUpdate[0].setName("Updated Name");
            BigDecimal bigDecimal = new BigDecimal(200.00);
            // set to 2 decimal places
            bigDecimal = bigDecimal.setScale(2);
            healthBenefitsToUpdate[0].setRate(bigDecimal);
            // Exercise
            HealthBenefitResponse healthBenefitResponse = sBenefitsService.updateHealthBenefits(workerHandle, healthBenefitsToUpdate);
            healthBenefits = healthBenefitResponse.getBenefits();
            // Verify
            AssertJUnit.assertEquals("Only expecting 1 health benefit item!", 1, healthBenefits.length);
            // Fix up the objectHandle value
            healthBenefitsToUpdate[0].setObjectHandle(healthBenefits[0].getObjectHandle());
            AssertJUnit.assertTrue("HealthBenefit added does not match expected!", verifyHealthBenefits(healthBenefitsToUpdate,
                                                                                                   healthBenefits));
        } finally {
            // Cleanup
            if (added) {
                sBenefitsService.deleteHealthBenefits(getWorkerObjectHandle(sWorkerId), healthBenefits);
            }
        }
    }

    /**
     * <P>Test getting HealthBenefitTemplate(s) for an Employer</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's findEmployerHealthBenefits()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the HealthBenefitTemplate is as expected
     * </UL>
     */
    @Test
    public void testFindEmployerHealthBenefits() throws Exception {
        // Setup
        HealthBenefitTemplate[] expectedTemplates = setUpExpectedEmployerHealthBenefitTemplates();
        // Exercise
        HealthBenefitTemplate[] healthBenefitTemplates = sBenefitsService.findEmployerHealthBenefits();
        // Verify
        AssertJUnit.assertTrue("Expected HealthBenefitTemplate mismatch!",
                          verifyHealthBenefitTemplates(expectedTemplates, healthBenefitTemplates));
    }

    /**
     * <P>Test getting the health HealthBenefitTemplate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's findEmployerHealthBenefit() with HealthBenefitType.Health</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the HealthBenefitTemplate is as expected
     * </UL>
     */
    @Test
    public void testFindEmployerHealthBenefitForHealth() throws Exception {
        // Setup
        HealthBenefitTemplate expectedTemplate = setUpExpectedEmployerHealthBenefitTemplate(HealthBenefitType.HEALTH);
        // Exercise
        HealthBenefitTemplate healthBenefitTemplate = sBenefitsService.findEmployerHealthBenefit(HealthBenefitType.HEALTH);
        // Verify
        AssertJUnit.assertEquals("Expected HealthBenefitTemplate mismatch!", expectedTemplate, healthBenefitTemplate);
    }

    /**
     * <P>Test calling findEmployerHealthBenefit() with HealthBenefitType.DependentCare </P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's findEmployerHealthBenefit() with HealthBenefitType.DependentCare</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    //NOTE: This test needs to be rewritten when HealthBenefitType.DependentCare is supported
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testFindEmployerHealthBenefitForDependentCare() throws Exception {
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitTemplate healthBenefitTemplate = sBenefitsService.findEmployerHealthBenefit(HealthBenefitType.DEPENDENT_CARE);
    }

    /**
     * <P>Test calling findEmployerHealthBenefit() with null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call BenefitService's findEmployerHealthBenefit() with null</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testFindEmployerHealthBenefitWithNull() throws Exception {
        // Exercise
        @SuppressWarnings("unused")
        HealthBenefitTemplate healthBenefitTemplate = sBenefitsService.findEmployerHealthBenefit(null);
    }

    private HealthBenefit[] addHealthBenefitToWorker(Long workerId) throws ServiceFailureException {
        HealthBenefitResponse healthBenefitResponse = null;
        HealthBenefit[] healthBenefits = null;
        HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[1];
        healthBenefitsToAdd[0] = createHealthBenefit();
        try {
            ObjectHandle workerHandle = getWorkerObjectHandle(workerId);
            healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
            healthBenefits = healthBenefitResponse.getBenefits();
        } catch (Exception e) {
            String mesg = ExceptionUtils.throwableMessageChain(e);
            sLogger.error(mesg);
            AssertJUnit.fail("Error in addHealthBenefitToWorker(): " + mesg);
        }
        return healthBenefits;
    }
}
