package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Tests Bonus handling in the CompensationService.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class BonusCompensationTest extends CompensationTestBase {

	/**
	 * <P>
	 * Test adding a Fixed Bonus Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up a fixed bonus wage item</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify that saved and expected bonus items are the same</LI>
	 * </OL>
	 */
	@Test
	public void testAddFixedBonusWage() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpBonusWage(true);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test adding a Varied Bonus Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up a varied bonus wage item</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify that saved and expected bonus items are the same</LI>
	 * </OL>
	 */
	@Test
	public void testAddVariedBonusWage() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpBonusWage(false);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}

	/**
	 * <P>
	 * Test updating a Varied Bonus Wage Item to a Fixed Bonus Wage
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up a varied bonus wage item</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Update varied to fixed type
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify that updated and expected bonus items are the same</LI>
	 * </OL>
	 */
	@Test
	public void testUpdateVariedToFixedBonusWage() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpBonusWage(false);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Bonus)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Bonus cannot have a rate");
				}
				compItem.setFixedRate(true);
				compItem.setRate(new BigDecimal("155.97"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Bonus Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Bonus.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(new BigDecimal("155.97"))) {
					AssertJUnit.fail("Varied Wage Item not updated to Fixed");
				}
			} else
				AssertJUnit.fail("Bonus Comp Item not saved");
		}
	}

	/**
	 * <P>
	 * Test updating a Fixed to Varied Bonus Wage
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up a fixed bonus wage item</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Update fixed to varied type
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify that updated and expected bonus items are the same</LI>
	 * </OL>
	 */
	@Test
	public void testUpdateFixedToVariedBonusWage() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpBonusWage(true);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the fixed bonus wage to varied rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Bonus)) {
				compItem.setFixedRate(false);
				compItem.setRate(new BigDecimal("155.97"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Bonus Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Bonus has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Bonus.equals(cItem.getCompensationType())) {
				if (cItem.getRate() == null) {
					AssertJUnit.fail("Varied  wage item should not be null");
				}
			} else
				AssertJUnit.fail("Bonus Comp Item not saved");
		}
	}

	/**
	 * <P>
	 * Test adding null rate for fixed bonus wage
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up a fixed bonus with rate null</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testFixedBonusWithNullRate() throws Exception {
		// Setup
		CompensationItem compItem = new CompensationItem();
		compItem.setCompensationType(CompensationType.Bonus);
		compItem.setFixedRate(true);
		compItem.setName("Bonus");
		compItem.setRate(null);

		CompensationItem[] wageItems = new CompensationItem[1];
		wageItems[0] = compItem;
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
	}

	/**
	 * <P>
	 * Test Deleting Fixed Bonus Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Fixed Bonus Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved Salary</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Bonus Wage item is deleted</LI>
	 * 
	 */
	@Test
	public void testRemoveFixedBonusWageItem() throws Exception {
		List<CompensationItem> compensationItemList = setUpBonusWage(true);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Bonus)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Bonus Wage Item not deleted");
		}
	}

	/**
	 * <P>
	 * Test Deleting Varied Bonus Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Varied Bonus Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved Salary</LI>
	 * <LI>Call WageService saveCompensationItems command to perform the actual
	 * save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Bonus wage is deleted</LI>
	 * 
	 */
	@Test
	public void testRemoveVariedBonusWageItem() throws Exception {
		List<CompensationItem> compensationItemList = setUpBonusWage(false);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Bonus)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Bonus Wage Item not deleted");
		}
	}
}
