package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Tests the CompensationItem CRUD of the CompensationService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CompensationCRUDTest extends CompensationTestBase {

	/**
	 * <P>Test getting empty wageItems for a worker, no exception should be thrown</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Do not set up any wage items for worker</LI>
	 * <LI>Call WorkerService's getCompensationItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned is empty 
	 * </UL>
	 */

	@Test
	public void testGettingEmptyWageItems() throws Exception {
		// Setup
		List<CompensationItem> wageItemList = new ArrayList<CompensationItem>();
		// Exercise
		CompensationItem[] wageItems = sWageService
				.getCompensationItems(sWorkerId);
		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(wageItems,
				wageItemList));
		// Teardown - nothing to clean up
	}

	/**
	 * <P>Test getting wageItems for a worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testGettingWageItems() throws Exception {
		// Setup
		setupWages(sWorkerId);
		// Exercise
		CompensationItem[] wageItems = sWageService
				.getCompensationItems(sWorkerId);
		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(wageItems,
				compensationItems));
		// Teardown - done in base class
	}


	/**
	 * <P>Test adding wageItems for a worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testAddingWageItems() throws Exception {
		// Setup
		setupWages(sWorkerId);
		// Add a TipsPaycheck wageItem
		CompensationItem wageItem = new CompensationItem();
		wageItem.setRate(new BigDecimal("23"));
		wageItem.setCompensationType(CompensationType.TipsPaycheck);
		wageItem.setUnits(0);
		wageItem.setFixedRate(true);
		compensationItems.add(wageItem);
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[1];
		compensationItemRequests[0] = new SaveCompensationItemRequest(wageItem, SaveCompensationItemOperation.ADD);
		
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, compensationItemRequests);
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItems));
		// Teardown - done in base class
	}

	/**
	 * <P>Test adding second hourly wageItem for a worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testAddSecondHourlyWageItem() throws Exception {
		// Setup
		setupWages(sWorkerId);
		// Add a second hourly wageItem
		CompensationItem wageItem = new CompensationItem();
		wageItem.setRate(new BigDecimal("12"));
		wageItem.setCompensationType(CompensationType.Hourly);
		wageItem.setName("New Hourly");
		wageItem.setUnits(40);
		wageItem.setFixedRate(true);
		compensationItems.add(wageItem);
		CompensationItem[] wageItems = new CompensationItem[compensationItems
				.size()];
		wageItems = compensationItems.toArray(wageItems);
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[1];
		compensationItemRequests[0] = new SaveCompensationItemRequest(wageItem, SaveCompensationItemOperation.ADD);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, compensationItemRequests);
		// Verify
		CompensationItem[] updatedWageItems = sWageService
				.getCompensationItems(sWorkerId);
		// First fix up the ObjectHandle value first
		for (CompensationItem compensationItem : updatedWageItems) {
			if (compensationItem.getName().equals(wageItem.getName())) {
				wageItem.setObjectHandle(compensationItem.getObjectHandle());
			}
		}
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(
				updatedWageItems, compensationItems));
		// Teardown - done in base class
	}

	/**
	 * <P>Test updating wageItems for a worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items for the worker</LI>
	 * <LI>Update the list of wage items' rates</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testUpdatingWageItemsValues() throws Exception {
		// Setup
		setupWages(sWorkerId);
		// Update the wageItems' rates
		for (int i = 0; i < compensationItems.size(); i++) {
			CompensationItem wageItem = compensationItems.get(i);
			if (CompensationType.Hourly.equals(wageItem.getCompensationType())) {
				wageItem.setRate(new BigDecimal("12.0"));
			} else if (CompensationType.Overtime.equals(wageItem
					.getCompensationType())) {
				wageItem.setRate(new BigDecimal("1.5"));
			} else if (CompensationType.Bonus.equals(wageItem
					.getCompensationType())) {
				wageItem.setRate(null);
			}
		}
		CompensationItem[] wageItems = new CompensationItem[compensationItems
				.size()];
		wageItems = compensationItems.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId,
				mapToSaveCompensationItemRequest(
						SaveCompensationItemOperation.UPDATE, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService
				.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(
				updatedWageItems, compensationItems));
		// Teardown - done in base class
	}

	  
	/**
	 * <P>Test adding and updating wageItems for a worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items for the worker</LI>
	 * <LI>Modify the list of wage items</LI>
	 * <LI>Add a new wage item</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testAddAndUpdateWageItemsValues() throws Exception {
		// Setup
		setupWages(sWorkerId);
		// Update the wageItems' rates
		for (int i = 0; i < compensationItems.size(); i++) {
			CompensationItem wageItem = compensationItems.get(i);
			if (CompensationType.Hourly.equals(wageItem.getCompensationType())) {
				wageItem.setRate(new BigDecimal("12.0"));
			} else if (CompensationType.Overtime.equals(wageItem
					.getCompensationType())) {
				wageItem.setRate(new BigDecimal("1.5"));
			} else if (CompensationType.Bonus.equals(wageItem
					.getCompensationType())) {
				wageItem.setRate(null);
			}
		}
		// Now add a TipsPaycheck wageItem
		CompensationItem wageItem = new CompensationItem();
		wageItem.setRate(new BigDecimal("123"));
		wageItem.setCompensationType(CompensationType.TipsPaycheck);
		wageItem.setName("Tips");
		wageItem.setUnits(0);
		wageItem.setFixedRate(true);
		compensationItems.add(wageItem);
		CompensationItem[] wageItems = new CompensationItem[compensationItems
				.size()];
		wageItems = compensationItems.toArray(wageItems);
		SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[wageItems.length];
		for (int i = 0; i < compensationItemRequests.length; i++) {
			compensationItemRequests[i] = new SaveCompensationItemRequest(
					wageItems[i],
					(null == wageItems[i].getObjectHandle()) ? SaveCompensationItemOperation.ADD
							: SaveCompensationItemOperation.UPDATE);
		}
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, compensationItemRequests);
		// Verify
		CompensationItem[] updatedWageItems = sWageService
				.getCompensationItems(sWorkerId);
		// First fix up the ObjectHandle value first
		for (CompensationItem compensationItem : updatedWageItems) {
			if (compensationItem.getName().equals(wageItem.getName())) {
				wageItem.setObjectHandle(compensationItem.getObjectHandle());
			}
		}
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(
				updatedWageItems, compensationItems));
		// Teardown - done in base class
	}


	/**
	 * <P>Test saving of unchanged wageItems for a worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveWageItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testSavingUnchangedWageItems() throws Exception {
		// Setup
		setupWages(sWorkerId);
		CompensationItem[] wageItems = sWageService
				.getCompensationItems(sWorkerId);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId,
				mapToSaveCompensationItemRequest(
						SaveCompensationItemOperation.UPDATE, wageItems));
		// Verify
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(wageItems,
				compensationItems));
		// Teardown - done in base class
	}

	/**
	 * <P>Test replacing wageItems for a worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Create new list of wage items containing only 1 new wageItem</LI>
	 * <LI>Call WageService's saveWageItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testReplacingWageItems() throws Exception {
		// Setup
		//setupWages(sWorkerId);
		// Create a TipsPaycheck wageItem
		CompensationItem wageItem = new CompensationItem();
		wageItem.setRate(new BigDecimal("123"));
		wageItem.setCompensationType(CompensationType.TipsPaycheck);
		wageItem.setFixedRate(true);
		wageItem.setUnits(0);
		List<CompensationItem> wageItemList = new ArrayList<CompensationItem>();
		wageItemList.add(wageItem);
		CompensationItem[] wageItems = new CompensationItem[wageItemList.size()];
		wageItems = wageItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId,
				mapToSaveCompensationItemRequest(
						SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService
				.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(
				updatedWageItems, wageItemList));
		// Teardown - done in base class
	}


	/**
	 * <P>Test removing a wageItem from a worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items for the worker</LI>
	 * <LI>Remove the last wage item (Bonus)</LI>
	 * <LI>Call WageService's removeWageItems command to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testRemoveWageItems() throws Exception {
		// Setup
		setupWages(sWorkerId);
		CompensationItem[] wageItems = new CompensationItem[1];
		CompensationItem wageItem = compensationItems.get(2);
		wageItems[0] = wageItem;
		compensationItems.remove(wageItem);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId,
				mapToSaveCompensationItemRequest(
						SaveCompensationItemOperation.DELETE, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService
				.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(
				updatedWageItems, compensationItems));
		// Teardown - done in base class
	}






	


	
 

}
