package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;

/**
 * Tests exception handling of the CompensationService.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class CompensationServiceExceptionsTest extends CompensationTestBase {

	/**
	 * <P>
	 * Test Exception caused by null workerId
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WorkerService's getCompensationItems command with a null
	 * workerId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown since workerId is
	 * null
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testGetWageItemsWithNullWorkerId() throws Exception {
		sWageService.getCompensationItems(null);
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test Exception caused by non existent worker id
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WorkerService's getCompensationItems command with a non existent
	 * workerId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown since workerId is non
	 * existent
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testGetWageItemsWithNonExistentWorkerId() throws Exception {
		// Exercise
		sWageService.getCompensationItems(new Long("39439480"));
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test Exception caused by null workerId
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command with a null workerId
	 * </LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ValidationException is thrown since workerId is
	 * null
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testAddWageItemsWithNullWorkerId() throws Exception {
		setupWages(sWorkerId);
		CompensationItem[] wageItems = new CompensationItem[compensationItems.size()];
		wageItems = compensationItems.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(null, mapToSaveCompensationItemRequest(SaveCompensationItemOperation.ADD,
				wageItems));
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test Exception caused by null wageItems
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command with a null
	 * wageItems</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown since wageItems is
	 * null
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testAddOrUpdateWageItemsWithNullWageItems() throws Exception {
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, null);
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test Exception caused by null element in wageItems array
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command with a null element
	 * in wageItems array</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown since an element in
	 * wageItems is null
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testAddOrUpdateWageItemsWithNullWageItemsElement() throws Exception {
		// Setup
		setupWages(sWorkerId);

		CompensationItem[] wageItems = new CompensationItem[2];
		// Add a TipsPaycheck wageItem
		CompensationItem wageItem = new CompensationItem();
		wageItem.setRate(new BigDecimal("23"));
		wageItem.setCompensationType(CompensationType.TipsPaycheck);
		wageItem.setFixedRate(true);
		wageItem.setUnits(1);
		wageItems[0] = wageItem;
		wageItems[1] = null;
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test Exception caused by invalid element in wageItems array
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command with an invalid
	 * element in wageItems array</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown since an element in
	 * wageItems is invalid
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testAddOrUpdateWageItemsWithInvalidWageItemInWageItemsElement() throws Exception {
		// Setup
		setupWages(sWorkerId);

		CompensationItem[] wageItems = new CompensationItem[compensationItems.size()];
		wageItems = compensationItems.toArray(wageItems);
		// Now purposely set the first item's CompensationType to null
		wageItems[0].setCompensationType(null);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test adding empty wageItems for a worker
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Set up new empty wage items array</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown since wageItems is
	 * empty
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testAddingEmptyWageItems() throws Exception {
		// Setup

		CompensationItem[] wageItems = new CompensationItem[0];
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test Exception caused by null workerId
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveWageItems command with a null workerId</LI>
	 * </OL>
	 * 
	 * <UL>
	 * /
	 * <LI>Verify that a ValidationException is thrown since workerId is null
	 * </UL>
	 */

	@Test(expectedExceptions = ValidationException.class)
	public void testSaveWageItemsWithNullWorkerId() throws Exception {
		CompensationItem[] wageItems = new CompensationItem[compensationItems.size()];
		wageItems = compensationItems.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(null, mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
				wageItems));
	}

	/**
	 * <P>
	 * Test Exception caused by null wageItems
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command with a null
	 * wageItems</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ValidationException is thrown since wageItems is null
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testSaveWageItemsWithNullWageItems() throws Exception {
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, null);
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test Exception caused by null element in wageItems array
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command with a null element
	 * in wageItems array</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown since an element in
	 * wageItems is null
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testSaveWageItemsWithNullWageItemsElement() throws Exception {
		try {
			// Setup
			setupWages(sWorkerId);

			CompensationItem[] wageItems = new CompensationItem[2];
			// Add a TipsPaycheck wageItem
			CompensationItem wageItem = new CompensationItem();
			wageItem.setRate(new BigDecimal("23"));
			wageItem.setCompensationType(CompensationType.TipsPaycheck);
			wageItem.setFixedRate(true);
			wageItem.setUnits(1);
			wageItems[0] = wageItem;
			wageItems[1] = null;
			// Exercise
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));
			AssertJUnit.fail("ValidationException was not thrown!");
		} finally {

			// Verify that no new wageItem has been added to the worker
			CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
			AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItems));
		}
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test inserting wageItems for a worker where the wageitem already has a
	 * key
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Create new list of wage items containing only 1 new wageItem</LI>
	 * <LI>Call WageService's saveWageItems command to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testInsertingWageItemsWithKey() throws Exception {
		// Setup
		setupWages(sWorkerId);
		// Create a TipsPaycheck wageItem
		CompensationItem wageItem = new CompensationItem();
		wageItem.setObjectHandle(new ObjectHandle(1L, 1L));
		wageItem.setRate(new BigDecimal("123"));
		wageItem.setCompensationType(CompensationType.TipsPaycheck);
		wageItem.setFixedRate(true);
		wageItem.setUnits(1);
		List<CompensationItem> wageItemList = new ArrayList<CompensationItem>();
		wageItemList.add(wageItem);
		CompensationItem[] wageItems = new CompensationItem[wageItemList.size()];
		wageItems = wageItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test replacing wageItems for a worker where the wageitem doesn't have a
	 * key
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up list of wage items</LI>
	 * <LI>Create new list of wage items containing only 1 new wageItem</LI>
	 * <LI>Call WageService's saveWageItems command to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that a ServiceFailureException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testReplacingWageItemsWithoutKey() throws Exception {
		// Setup
		setupWages(sWorkerId);
		// Create a TipsPaycheck wageItem
		CompensationItem wageItem = new CompensationItem();
		wageItem.setRate(new BigDecimal("123"));
		wageItem.setCompensationType(CompensationType.TipsPaycheck);
		wageItem.setUnits(1);
		wageItem.setFixedRate(true);
		List<CompensationItem> wageItemList = new ArrayList<CompensationItem>();
		wageItemList.add(wageItem);
		CompensationItem[] wageItems = new CompensationItem[wageItemList.size()];
		wageItems = wageItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.UPDATE, wageItems));
		// Teardown - done in base class
	}

	/**
	 * <P>
	 * Test Deleting Hourly Wage set up with Double Time and Overtime
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage, DoubleTime, Overtime . Link Overtime, DoubleTime
	 * to hourly</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to return the saved
	 * compensations</LI>
	 * <LI>Call WageService's saveCompensationItems command to delete the saved
	 * Hourly Item</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of wage items returned matches what's expected
	 * </UL>
	 */
	@Test(expectedExceptions = ServiceFailureException.class)
	public void testDeleteOnlyHourlyWageItem() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpHourlyWageWithExtraTime("Both");
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Delete only hourly wage item

		CompensationItem[] workerWageItems = sWageService.getCompensationItems(sWorkerId);
		CompensationItem[] newWageItems = new CompensationItem[1];
		for (CompensationItem cItem : workerWageItems) {
			if (CompensationType.Hourly.equals(cItem.getCompensationType())) {
				newWageItems[0] = cItem;
				sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
						SaveCompensationItemOperation.DELETE, newWageItems));
			}
		}
	}

	/**
	 * <P>
	 * Test Adding Hourly Wage with Rate equal to negative number
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Hourly Wage . Setup rate as a negative number</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify ValidationException is thrown
	 * </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testHourlyRateAsNegative() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		// Create Hourly Compensation Item
		CompensationItem hourlyCompensationItem = new CompensationItem();
		hourlyCompensationItem.setCompensationType(CompensationType.Hourly);
		hourlyCompensationItem.setRate(new BigDecimal(-9));
		hourlyCompensationItem.setName("Hourly");
		hourlyCompensationItem.setFixedRate(true);
		hourlyCompensationItem.setUnits(new Double(10));
		compensationItemList.add(hourlyCompensationItem);
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
	}

	// test to verify the Negative cash fixed Rate
	/**
	 * <P>
	 * Test Adding Tips Wage Item with varied rate to negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up CashTips Wage with fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify CashTips Wage Item gives validation exception</LI>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testAddTipsWithVariedNegativeCashRate() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(true, false);
		wageItem.setRate(new BigDecimal("-20"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));

	}
}
