package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Tests Reimbursement handling in the CompensationService.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class ReimbursementCompensationTest extends CompensationTestBase {

	/**
	 * <P>
	 * Test Adding Reimbursement Wage Item with fixed rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Reimbursement Wage with fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is Saved</LI>
	 */
	@Test
	public void testAddFixedReimbursement() throws Exception {

		// SetUp
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpReimbursementWage(true, false);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(savedWageItems, compensationItemList));

		validateWageItem(wageItems);
	}

	/**
	 * <P>
	 * Test Adding Reimbursement Wage Item with varied rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Reimbursement Wage with varied rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is Saved</LI>
	 */
	@Test
	public void testAddVariedReimbursement() throws Exception {

		// SetUp
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpReimbursementWage(false, false);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		for (CompensationItem comp : savedWageItems) {
			comp.setRate(null);
		}

		validateWageItem(savedWageItems);
	}

	/**
	 * <P>
	 * Test Adding Reimbursement Wage Item with fixed rate to negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Reimbursement Wage with negative fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item throws validation exception</LI>
	 */
	@Test
	public void testAddFixedReimbursementWithdNegativeRate() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;

		CompensationItem[] compensationItem = sWageService.getCompensationItems(sWorkerId);
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem wageItem = setUpReimbursementWage(true, false);
		wageItem.setRate(new BigDecimal("-20"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		try {
			// Exercise
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		validateWageItem(compensationItem);

	}

	/**
	 * <P>
	 * Test Adding Reimbursement Wage Item with varied rate to negative
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Reimbursement Wage with negative varied rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item throws validation exception</LI>
	 */
	@Test
	public void testAddVariedReimbursementWithNegativeRate() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;

		CompensationItem[] compensationItem = sWageService.getCompensationItems(sWorkerId);
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpReimbursementWage(false, false);
		wageItem.setRate(new BigDecimal("-20"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		try {
			sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
					SaveCompensationItemOperation.ADD, wageItems));
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		validateWageItem(compensationItem);
	}

	/**
	 * <P>
	 * Test Adding Reimbursement Wage Item with fixed rate to zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Reimbursement Wage with zero fixed rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item throws validation exception</LI>
	 */
	@Test
	public void testAddFixedReimbursementWithZeroRate() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpReimbursementWage(true, false);
		wageItem.setRate(new BigDecimal("0"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		for (CompensationItem comp : wageItems) {
			comp.setRate(new BigDecimal("0"));
		}
		validateWageItem(wageItems);
	}

	/**
	 * <P>
	 * Test Updating fixed Reimbursement Wage Item to zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed Reimbursement Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedReimbursementZeroRate() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem ReimbursementComp = setUpReimbursementWage(true, false);
		compensationItemList.add(ReimbursementComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : savedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {

				if (compItem.isFixedRate()) {
					compItem.setRate(new BigDecimal("0"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));

				} else
					AssertJUnit.fail("Reimbursement Wage Item is not saved as Fixed Rate");

			}
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		for (CompensationItem comp : savedWageItems) {
			comp.setRate(new BigDecimal("0"));
		}
		validateWageItem(savedWageItems);

	}

	/**
	 * <P>
	 * Test Updating fixed Reimbursement Wage Item to zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed Reimbursement Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedReimbursement() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem ReimbursementComp = setUpReimbursementWage(true, false);
		compensationItemList.add(ReimbursementComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : savedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {

				if (compItem.isFixedRate()) {
					compItem.setRate(new BigDecimal("20.22"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));

				} else
					AssertJUnit.fail("Reimbursement Wage Item is not saved ");

			}
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {

			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		for (CompensationItem comp : savedWageItems) {
			comp.setRate(new BigDecimal("20.22"));
		}
		validateWageItem(savedWageItems);

	}

	/**
	 * <P>
	 * Test Updating fixed Reimbursement Wage Item to zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed Reimbursement Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedReimbursement() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem ReimbursementComp = setUpReimbursementWage(false, false);
		compensationItemList.add(ReimbursementComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : savedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {

				if (!compItem.isFixedRate()) {
					compItem.setRate(new BigDecimal("20"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));

				} else
					AssertJUnit.fail("Reimbursement Wage Item is not saved as Fixed Rate");

			}
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		for (CompensationItem comp : savedWageItems) {
			comp.setRate(new BigDecimal("20"));
		}
		validateWageItem(savedWageItems);

	}

	/**
	 * <P>
	 * Test Adding Reimbursement Wage Item with varied rate to zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Reimbursement Wage with zero varied rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item throws validation exception</LI>
	 */
	@Test
	public void testAddVariedReimbursementZeroRate() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpTipsWage(false, false);
		wageItem.setRate(new BigDecimal("0"));

		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		for (CompensationItem comp : wageItems) {
			comp.setRate(new BigDecimal("0"));
		}
		validateWageItem(wageItems);

	}

	/**
	 * <P>
	 * Test creating duplicate Reimbursement Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Reimbursement wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement wage item throws validation exception</LI>
	 * 
	 */

	@Test
	public void testAddDuplicateVariedReimbursementWithSameName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = setUpReimbursementWage(false, false);
		CompensationItem wageItem2 = setUpReimbursementWage(false, false);

		compensationItemList.add(wageItem1);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		for (int i = 0; i <= 1; i++) {

			AssertJUnit.assertTrue(updatedWageItems[i].getRate() == null && wageItems[i].getRate() == null);
			AssertJUnit.assertTrue(updatedWageItems[i].getName().equals(wageItems[i].getName()));
			AssertJUnit.assertTrue(updatedWageItems[i].getCompensationType().equals(wageItems[i].getCompensationType()));
		}
	}

	/**
	 * <P>
	 * Test creating duplicate Reimbursement Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Reimbursement wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateReimbursementWithUniqueName() throws Exception {

		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = setUpReimbursementWage(false, false);
		wageItem1.setName("Reimbursement1");

		CompensationItem wageItem2 = setUpReimbursementWage(false, false);
		wageItem2.setName("Reimbursement2");

		compensationItemList.add(wageItem1);
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : savedWageItems) {
			if (CompensationType.Reimbursement.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);

	}

	@Test
	public void testUpdateVariedReimbursementToFixed() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem ReimbursementComp = setUpReimbursementWage(false, false);
		compensationItemList.add(ReimbursementComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : savedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
				if (compItem.getRate() != null) {
					AssertJUnit.fail("Varied Reimbursement cannot have a rate");
				}
				compItem.setFixedRate(true);
				compItem.setRate(new BigDecimal("155.97"));
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Reimbursement Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Reimbursement has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Reimbursement.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(new BigDecimal("155.97"))) {
					AssertJUnit.fail("Varied Reimbursement Wage Item not updated to Fixed");
				}
			} else
				AssertJUnit.fail(" Reimbursement Wage Item not saved");
		}

		validateWageItem(savedWageItems);

	}

	/**
	 * <P>
	 * Test Updating fixed Reimbursement Wage Item to varied
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed Reimbursement Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to varied rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedReimbursementToVaried() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem ReimbursementComp = setUpReimbursementWage(true, false);
		compensationItemList.add(ReimbursementComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : savedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
				if (compItem.getRate() == null) {
					AssertJUnit.fail("Fixed Reimbursement should have a rate");
				} else
					compItem.setFixedRate(false);
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			} else
				AssertJUnit.fail("Reimbursement Wage Item not saved as expected");
		}

		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Reimbursement has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Reimbursement.equals(cItem.getCompensationType())) {
				if (!cItem.isFixedRate()) {
					if (cItem.getRate() == null) {
						AssertJUnit.fail("Varied  wage item should not be null");
					}

				} else {
					AssertJUnit.fail(" Reimbursement Wage Item not saved");
				}
			}
		}
		// validateWageItem(savedWageItems);
	}

	/**
	 * <P>
	 * Test Updating fixed Reimbursement Wage Item to negative rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed Reimbursement Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e -20</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateFixedReimbursementNegativeRate() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem ReimbursementComp = setUpReimbursementWage(true, false);
		compensationItemList.add(ReimbursementComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : savedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {

				if (compItem.isFixedRate()) {
					BigDecimal value = compItem.getRate();

					if (!value.equals(new BigDecimal("122.50")))
						AssertJUnit.fail("Reimbursement Wage Item not saved with expected rate");

					else {
						compItem.setRate(new BigDecimal("-20"));
						compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
								compItem));
					}
				} else
					AssertJUnit.fail("Reimbursement Wage Item is not saved as Fixed Rate");

			} else
				AssertJUnit.fail("Reimbursement Wage Item not saved as expected");
		}

		try {
			// If the list has values , then call operation to update it
			if (compItemList.size() > 0) {
				SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
				newCompItems = compItemList.toArray(newCompItems);

				sWageService.saveCompensationItems(sWorkerId, newCompItems);
			} else
				AssertJUnit.fail("No Compensation Items to Update");
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		for (CompensationItem comp : savedWageItems) {
			comp.setRate(new BigDecimal("122.50"));
		}
		validateWageItem(savedWageItems);
		// Cleanup the compensation item(s)
		savedWageItems = sWageService.getCompensationItems(sWorkerId);
		compItemList.clear();
		for (CompensationItem cItem : savedWageItems) {
			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, cItem));
		}
		SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
		newCompItems = compItemList.toArray(newCompItems);
		sWageService.saveCompensationItems(sWorkerId, newCompItems);
		getUserTransaction().commit();
	}

	/**
	 * <P>
	 * Test Updating fixed Reimbursement Wage Item to negative rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up fixed Reimbursement Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e -20</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is not Updated</LI>
	 * 
	 */
	@Test
	public void testUpdateVariedReimbursementNegativeRate() throws Exception {
		getUserTransaction().commit();
		boolean validationExceptionThrown = false;
		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();
		CompensationItem ReimbursementComp = setUpReimbursementWage(false, false);
		compensationItemList.add(ReimbursementComp);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);

		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		// Update the varied bonus wage to fixed rate
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : savedWageItems) {

			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
				if (compItem.isFixedRate())
					AssertJUnit.fail("Reimbursement Wage Item not saved with expected rate");
				else {
					compItem.setRate(new BigDecimal("-20"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));
				}
			} else
				AssertJUnit.fail("Reimbursement Wage Item not saved as expected");
		}

		try {
			// If the list has values , then call operation to update it
			if (compItemList.size() > 0) {
				SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
				newCompItems = compItemList.toArray(newCompItems);

				sWageService.saveCompensationItems(sWorkerId, newCompItems);
			} else
				AssertJUnit.fail("No Compensation Items to Update");
		} catch (ValidationException ve) {

			validationExceptionThrown = true;
		}

		AssertJUnit.assertTrue("Expected ValidationException is not thrown", validationExceptionThrown);
		getUserTransaction().begin();
		for (CompensationItem comp : savedWageItems) {
			comp.setRate(null);
		}
		validateWageItem(savedWageItems);
		// Cleanup the compensation item(s)
		savedWageItems = sWageService.getCompensationItems(sWorkerId);
		compItemList.clear();
		for (CompensationItem cItem : savedWageItems) {
			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, cItem));
		}
		SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
		newCompItems = compItemList.toArray(newCompItems);
		sWageService.saveCompensationItems(sWorkerId, newCompItems);
		getUserTransaction().commit();
	}

	/**
	 * <P>
	 * Test Deleting Reimbursement Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Reimbursement Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved Reimbursement</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteVariedReimbursement() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpReimbursementWage(false, false);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Reimbursement Wage Item not deleted");
		}

	}

	/**
	 * <P>
	 * Test Deleting Reimbursement Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Reimbursement Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved Reimbursement</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Reimbursement Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteFixedReimbursement() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem = setUpReimbursementWage(true, false);
		compensationItemList.add(wageItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Reimbursement)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);

		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Reimbursement Wage Item not deleted");
		}

	}

	/**
	 * <P>
	 * Test Deleting Reimbursement Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 Reimbursement Wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved Reimbursement wage items</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify one of the Reimbursement Wage Item is Deleted</LI>
	 * 
	 */
	@Test
	public void testDeleteOneReimbursementWageItemAmongMultiple() throws Exception {

		// Setup
		List<CompensationItem> compensationItemList = new ArrayList<CompensationItem>();

		CompensationItem wageItem1 = setUpReimbursementWage(false, false);
		wageItem1.setName("Reimbursement1");
		compensationItemList.add(wageItem1);

		CompensationItem wageItem2 = setUpReimbursementWage(false, true);
		wageItem2.setName("Reimbursement2");
		compensationItemList.add(wageItem2);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] savedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : savedWageItems) {

			compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			break;

		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length == 2) {
			AssertJUnit.fail("Reimbursement Wage Item not deleted");
		}
	}

}
