package com.intuit.sb.payroll.qa.services.compensation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;

/**
 * Tests Salary handling in the CompensationService.
 *
 * $Author$ 
 * $Revision$
 * $DateTime$
 */
public class SalaryCompensationTest extends CompensationTestBase {

	/**
	 * <P>
	 * Test Adding Salary Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Salary Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Salary Wage Item is Saved</LI>
	 * 
	 */
	@Test
	public void testAddSalaryWageItem() throws Exception {
		List<CompensationItem> compensationItemList = setUpSalaryWage();
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		AssertJUnit.assertTrue("Wage items mismatch!", verifyCompensationItems(updatedWageItems, compensationItemList));
	}

	/**
	 * <P>
	 * Test Deleting Salary Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Salary Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Retrieve the saved Salary</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work with operation as Delete</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Salary Wage Item is Saved</LI>
	 * 
	 */
	@Test
	public void testRemoveSalaryWageItem() throws Exception {
		List<CompensationItem> compensationItemList = setUpSalaryWage();
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));

		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);

		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Salary)) {
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE, compItem));
			}
		}
		// If the list has values , then call operation to delete it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);
			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		}

		// Verify
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		if (workerWageItemsNew.length > 0) {
			AssertJUnit.fail("Salary Wage Item not deleted");
		}
	}

	/**
	 * <P>
	 * Test Updating Salary Wage Item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Salary Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Salary Wage Item is Saved</LI>
	 * 
	 */
	@Test
	public void testUpdateSalaryWageItem() throws Exception {
		List<CompensationItem> compensationItemList = setUpSalaryWage();
		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update the salary wage rate to a new value
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Salary)) {
				BigDecimal rate = compItem.getRate();
				if (rate.equals(new BigDecimal("1200.50"))) {
					compItem.setRate(new BigDecimal("1500.97"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));
				} else
					AssertJUnit.fail("Salary Wage Item not saved as expected");
			}
		}
		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Salary has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Salary.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				if (!value.equals(new BigDecimal("1500.97"))) {
					AssertJUnit.fail("Salary Wage Item not updated");
				}
			} else
				AssertJUnit.fail("Salary Comp Item not saved");
		}
	}

	/**
	 * <P>
	 * Test Updating Salary Wage Item to zero
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up Salary Wage</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update the wage item rate to a new value i.e zero</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify Salary Wage Item is Saved</LI>
	 * 
	 */

	@Test
	public void testUpdateSalaryWageItemAsZero() throws Exception {
		List<CompensationItem> compensationItemList = setUpSalaryWage();

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		// Update the salary wage rate to a new value
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();

		BigDecimal bigDecimal = new BigDecimal("1200.50");
		// set to 2 decimal places
		bigDecimal = bigDecimal.setScale(2);
		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getCompensationType().equals(CompensationType.Salary)) {
				BigDecimal rate = compItem.getRate();
				if (rate.equals(new BigDecimal("1200.50"))) {
					compItem.setRate(new BigDecimal("0"));
					compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
							compItem));
				} else
					AssertJUnit.fail("Salary Wage Item not saved as expected");
			}
		}
		// If the list has values , then call operation to update it
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else
			AssertJUnit.fail("No Compensation Items to Update");

		// Verify that Salary has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Salary.equals(cItem.getCompensationType())) {
				BigDecimal value = cItem.getRate();
				BigDecimal zero = new BigDecimal("0.00");
				// set to 2 decimal places
				zero = zero.setScale(2);
				if (!value.equals(zero)) {
					AssertJUnit.fail("Salary Wage Item not updated");
				}
			} else
				AssertJUnit.fail("Salary Comp Item not saved");
		}

	}

	/**
	 * <P>
	 * Test creating duplicate Salary Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 salary wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateSalaryWageWithSameName() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpSalaryWage();
		// create duplicate salary wage with same name and rate
		CompensationItem salaryCompItem = new CompensationItem();
		salaryCompItem.setCompensationType(CompensationType.Salary);
		salaryCompItem.setRate(new BigDecimal("1200.43"));
		salaryCompItem.setName("Salary");
		salaryCompItem.setFixedRate(true);
		compensationItemList.add(salaryCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Salary.equals(compItem.getCompensationType())) {
				count++;
			}
		}
		AssertJUnit.assertEquals(2, count);
	}

	/**
	 * <P>
	 * Test creating duplicate Salary Wage Item with unique names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 salary wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Compare the saved and expected compensation items</LI>
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify saved and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testAddDuplicateSalaryWageWithUniqueNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpSalaryWage();
		// create duplicate salary wage with same name and rate
		CompensationItem salaryCompItem = new CompensationItem();
		salaryCompItem.setCompensationType(CompensationType.Salary);
		salaryCompItem.setRate(new BigDecimal("1200.43"));
		salaryCompItem.setName("Salary 45");
		salaryCompItem.setFixedRate(true);
		compensationItemList.add(salaryCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		int count = 0;
		for (CompensationItem compItem : updatedWageItems) {
			if (CompensationType.Salary.equals(compItem.getCompensationType())) {
				if (compItem.getName().equals("Salary 45")) {
					count++;
				} else if (compItem.getName().equals("Salary")) {
					count++;
				}
			}
		}
		AssertJUnit.assertEquals(2, count);
	}

	/**
	 * <P>
	 * Test updating duplicate Salary Wage Item with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 salary wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update one salary item with different rate</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct salary wage has been updated</LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify updated and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testUpdateDuplicateSalaryWageWithSameNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpSalaryWage();
		// create duplicate salary wage with same name and rate
		CompensationItem salaryCompItem = new CompensationItem();
		salaryCompItem.setCompensationType(CompensationType.Salary);
		salaryCompItem.setRate(new BigDecimal("1200.43"));
		salaryCompItem.setName("Salary");
		salaryCompItem.setFixedRate(true);
		compensationItemList.add(salaryCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		// Update the rate of the first salary wage item returned by the Service
		updatedWageItems[0].setRate(new BigDecimal("3000.01"));
		id = updatedWageItems[0].getObjectHandle().getId();
		compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE,
				updatedWageItems[0]));

		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else {
			AssertJUnit.fail("No Compensation Items to Update");
		}

		// Verify that correct Salary has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Salary.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					BigDecimal value = cItem.getRate();
					if (!value.equals(new BigDecimal("3000.01"))) {
						AssertJUnit.fail("Salary Wage Item not updated");
					}
				}
			} else {
				AssertJUnit.fail("Salary Comp Item not saved");
			}
		}
	}

	/**
	 * <P>
	 * Test updating duplicate Salary Wage Item with unique names to salary wage
	 * items with same names
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 salary wage items with unique names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Update one salary item to the same name as the other</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct salary wage has been updated</LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify updated and expected compensation items are the same</LI>
	 * 
	 */
	@Test
	public void testUpdateDuplicateSalaryWageWithUniqueNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpSalaryWage();
		// create duplicate salary wage with same name and rate
		CompensationItem salaryCompItem = new CompensationItem();
		salaryCompItem.setCompensationType(CompensationType.Salary);
		salaryCompItem.setRate(new BigDecimal("1200.43"));
		salaryCompItem.setName("Salary 2");
		salaryCompItem.setFixedRate(true);
		compensationItemList.add(salaryCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Setup for Updates
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		for (CompensationItem compItem : updatedWageItems) {
			if (compItem.getName().equals("Salary 2")) {
				id = compItem.getObjectHandle().getId();
				compItem.setName("Salary");
				compItemList
						.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.UPDATE, compItem));
			}
		}
		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else {
			AssertJUnit.fail("No Compensation Items to Update");
		}

		// Verify that correct Salary has been updated
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Salary.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					String name = cItem.getName();
					if (!name.equals("Salary")) {
						AssertJUnit.fail("Salary Wage Item not updated");
					}
				}
			} else {
				AssertJUnit.fail("Salary Comp Item not saved");
			}
		}
	}

	/**
	 * <P>
	 * Test deleting Salary Wage Item which has the same name as another salary
	 * wage item
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up 2 salary wage items with same names</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Delete one salary wage item</LI>
	 * <LI>Call WageService's saveCompensationItems command to perform the
	 * actual save work</LI>
	 * <LI>Call WageService's getCompensationItems command to perform the actual
	 * retrieve work</LI>
	 * <LI>Verify the correct salary wage has been delete/LI>
	 * 
	 * </OL>
	 * <H3>
	 * <OL>
	 * <LI>
	 * Verify that expected salary wage has been deleted</LI>
	 * 
	 */
	@Test
	public void testRemoveDuplicateSalaryWageWithSameNames() throws Exception {
		// Setup
		List<CompensationItem> compensationItemList = setUpSalaryWage();
		// create duplicate salary wage with same name and rate
		CompensationItem salaryCompItem = new CompensationItem();
		salaryCompItem.setCompensationType(CompensationType.Salary);
		salaryCompItem.setRate(new BigDecimal("1200.43"));
		salaryCompItem.setName("Salary");
		salaryCompItem.setFixedRate(true);
		compensationItemList.add(salaryCompItem);

		CompensationItem[] wageItems = new CompensationItem[compensationItemList.size()];
		wageItems = compensationItemList.toArray(wageItems);
		// Exercise
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, wageItems));
		// Verify
		CompensationItem[] updatedWageItems = sWageService.getCompensationItems(sWorkerId);
		List<SaveCompensationItemRequest> compItemList = new ArrayList<SaveCompensationItemRequest>();
		Long id = null;
		// Delete first salary wage item returned by the Service
		id = updatedWageItems[0].getObjectHandle().getId();
		compItemList.add(mapToSaveCompensationItemRequestObject(SaveCompensationItemOperation.DELETE,
				updatedWageItems[0]));

		if (compItemList.size() > 0) {
			SaveCompensationItemRequest[] newCompItems = new SaveCompensationItemRequest[compItemList.size()];
			newCompItems = compItemList.toArray(newCompItems);

			sWageService.saveCompensationItems(sWorkerId, newCompItems);
		} else {
			AssertJUnit.fail("No Compensation Items to Update");
		}

		// Verify that correct Salary has been deleted
		CompensationItem[] workerWageItemsNew = sWageService.getCompensationItems(sWorkerId);
		for (CompensationItem cItem : workerWageItemsNew) {
			if (CompensationType.Salary.equals(cItem.getCompensationType())) {
				if (cItem.getObjectHandle().getId() == id) {
					AssertJUnit.fail("Salary Wage Item not deleted");
				}
			} else {
				AssertJUnit.fail("Salary Comp Item not saved");
			}
		}
	}

}
