package com.intuit.sb.payroll.qa.services.employer;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;

/**
 * Tests the addWorkLocations operation of the EmployerService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class AddWorkLocationsTest extends EmployerServiceTestBase {
    
    /**
     * <P>Test calling addWorkLocations with an empty array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's addWorkLocations() with an empty array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testAddWorkLocationsWithEmptyArray() throws Exception {
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.addWorkLocations(new WorkLocation[0]).getWorkLocations();
    }

    /**
     * <P>Test calling addWorkLocations with a null WorkLocation container</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's addWorkLocations() with a null WorkLocation container</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testAddWorkLocationsWithNullContainer() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        // Purposely set it to null
        newWorkLocation[0] = null;
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.addWorkLocations(newWorkLocation).getWorkLocations();
    }

    /**
     * <P>Test calling addWorkLocations with null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's addWorkLocations() with null</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testAddWorkLocationsWithNull() throws Exception {
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.addWorkLocations(null).getWorkLocations();
    }

    /**
     * <P>Test calling addWorkLocations with a non-existent workLocation id</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's addWorkLocations() with a non-existent workLocation id</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testAddWorkLocationsWithNonExistentId() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        newWorkLocation[0] = createWorkLocation("Transient WorkLocation", "800-333-4444");
        // Now purposely set the uniqueId to a non existent value
        newWorkLocation[0].setObjectHandle(new ObjectHandle(new Long("83434176"), new Long("0")));
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.addWorkLocations(newWorkLocation).getWorkLocations();
    }

    /**
     * <P>Test calling addWorkLocations with a null WorkLocation name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's addWorkLocations() with a null WorkLocation name</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testAddWorkLocationsWithNullName() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        newWorkLocation[0] = createWorkLocation("Transient WorkLocation", "800-333-4444");
        // Purposely set it to null
        newWorkLocation[0].setName(null);
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.addWorkLocations(newWorkLocation).getWorkLocations();
    }

    /**
     * <P>Test calling addWorkLocations with a null address</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's addWorkLocations() with a null address</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testAddWorkLocationsWithNullAddress() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        newWorkLocation[0] = createWorkLocation("Transient WorkLocation", "800-333-4444");
        // Purposely set it to null
        newWorkLocation[0].setAddress(null);
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.addWorkLocations(newWorkLocation).getWorkLocations();
    }

    /**
     * <P>Test calling addWorkLocations with a null state value</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's addWorkLocations() with a null state value</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testAddWorkLocationsWithNullState() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        newWorkLocation[0] = createWorkLocation("Transient WorkLocation", "800-333-4444");
        // Purposely set it to null
        newWorkLocation[0].getAddress().setStateOrRegion(null);
        @SuppressWarnings("unused")
        WorkLocation[] workLocations = sEmployerService.addWorkLocations(newWorkLocation).getWorkLocations();
    }

    /**
     * <P>Test adding a workLocation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's addWorkLocations() to add a workLocation</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that workLocation is added successfully
     * </UL>
     */
    @Test
    public void testAddOneWorkLocation() throws Exception {
        WorkLocation workLocation = null;
        WorkLocation addedWorkLocation = null;
        WorkLocation[] addedWorkLocations = null;
        boolean workLocationAdded = false;
        try {
            WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
            workLocation = createWorkLocation("Transient WorkLocation", "800-333-4444");
            WorkLocation[] newWorkLocations = new WorkLocation[1];
            newWorkLocations[0] = workLocation;
            addedWorkLocations = sEmployerService.addWorkLocations(newWorkLocations).getWorkLocations();
            workLocationAdded = true;
            // Locate the workLocation that was added
            for (WorkLocation aWorkLocation : addedWorkLocations) {
                if (aWorkLocation.getName().equals(workLocation.getName())) {
                    addedWorkLocation = aWorkLocation;
                    break;
                }
            }
            // Verify
            AssertJUnit.assertEquals("Number of WorkLocations mismatch!", existingLocations.length+1, addedWorkLocations.length);
            AssertJUnit.assertTrue("WorkLocation info mismatch", verifyWorkLocation(newWorkLocations[0], addedWorkLocation));
        } finally {
            if (workLocationAdded) {
                com.intuit.sb.payroll.entity.model.WorkLocation workLocationFromDB = null;
                workLocationFromDB = entityService.findById(com.intuit.sb.payroll.entity.model.WorkLocation.class, addedWorkLocation.getObjectHandle().getId());
                entityService.makeTransient(workLocationFromDB);
            }
        }
    }

    /**
     * <P>Test adding 2 workLocations</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's addWorkLocations() to add 2 workLocations</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that workLocation are added successfully
     * </UL>
     */
    @Test
    public void testAddTwoWorkLocations() throws Exception {
        final int numOfWorkLocations = 2;
        WorkLocation[] addedWorkLocations = null;
        WorkLocation[] workLocationsAdded = null;
        boolean workLocationAdded = false;
        try {
            // Setup
            WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
            WorkLocation workLocation1 = createWorkLocation("Transient WorkLocation 1", "800-333-4444");
            WorkLocation workLocation2 = createWorkLocation("Transient WorkLocation 2", "800-333-1234");
            WorkLocation[] newWorkLocations = new WorkLocation[numOfWorkLocations];
            newWorkLocations[0] = workLocation1;
            newWorkLocations[1] = workLocation2;
            // Exercise
            addedWorkLocations = sEmployerService.addWorkLocations(newWorkLocations).getWorkLocations();
            workLocationAdded = true;
            workLocationsAdded = new WorkLocation[numOfWorkLocations];
            int i = 0;
            // Locate the workLocations that were added
            for (WorkLocation aWorkLocation : addedWorkLocations) {
                if (aWorkLocation.getName().equals(workLocation1.getName()) ||
                    aWorkLocation.getName().equals(workLocation2.getName())) {
                    workLocationsAdded[i++] = aWorkLocation;
                    if (i >= numOfWorkLocations) {
                        break;
                    }
                }
            }
            // Verify
            AssertJUnit.assertEquals("Number of WorkLocations mismatch!", existingLocations.length+numOfWorkLocations, addedWorkLocations.length);
            AssertJUnit.assertTrue("WorkLocation info mismatch", verifyWorkLocations(newWorkLocations, workLocationsAdded));
            WorkLocation[] allLocations = sEmployerService.getWorkLocations();
            AssertJUnit.assertTrue("WorkLocation info mismatch", verifyWorkLocations(allLocations, addedWorkLocations));
        } finally {
            if (workLocationAdded) {
                com.intuit.sb.payroll.entity.model.WorkLocation workLocationFromDB = null;
                for (int i = 0; i < numOfWorkLocations; i++) {
                    workLocationFromDB = entityService.findById(com.intuit.sb.payroll.entity.model.WorkLocation.class, workLocationsAdded[i].getObjectHandle().getId());
                    entityService.makeTransient(workLocationFromDB);
                }
            }
        }
    }

}
