package com.intuit.sb.payroll.qa.services.employer;

import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.taxes.TaxComponent;
import com.intuit.sb.payroll.entity.model.Employee;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.TaxDtl;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.services.employer.WorkerLocation;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * Tests the assignWorkLocationsToWorkers operation of the EmployerService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class AssignWorkLocationsToWorkersTest extends EmployerServiceTestBase
{    
    private boolean verifyWorkerWorkLocationMap(WorkerLocation[] expected, WorkerLocation[] computed)
    {
        boolean compareResult = expected.equals(computed);
        return compareResult;
    }
    
    /**
     * <P>Test calling assignWorkLocationsToWorkers with null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's assignWorkLocationsToWorkers() with null</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testWithNull() throws Exception
    {
        // Exercise
        sEmployerService.assignWorkLocationsToWorkers(null);
    }

    /**
     * <P>Test calling assignWorkLocationsToWorkers with WorkerLocation array containing a null workerId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's assignWorkLocationsToWorkers() with WorkerLocation array containing a null workerId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testWithNullWorkerId() throws Exception
    {
        // Setup
    	WorkerLocation[] existingWorkerWorkLocationMap = getWorkerWorkLocationAssociations();
        WorkerLocation[] workLocations = new WorkerLocation[1];
        workLocations[0] = new WorkerLocation();
        workLocations[0].setWorkLocationId(existingWorkerWorkLocationMap[0].getWorkLocationId());

        // Exercise
        sEmployerService.assignWorkLocationsToWorkers(workLocations);
    }

    /**
     * <P>Test calling assignWorkLocationsToWorkers with WorkerLocation array containing a non-existent workerId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's assignWorkLocationsToWorkers() with WorkerLocation array containing a non-existent workerId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testWithNonExistentWorkerId() throws Exception
    {
        // Setup
    	WorkerLocation[] existingWorkerWorkLocationMap = getWorkerWorkLocationAssociations();

    	WorkerLocation[] workLocations = new WorkerLocation[1];
        workLocations[0] = new WorkerLocation();
        workLocations[0].setWorkerId(new Long("92134847"));
        workLocations[0].setWorkLocationId(existingWorkerWorkLocationMap[0].getWorkLocationId());
        
        // Exercise
        sEmployerService.assignWorkLocationsToWorkers(workLocations);
    }

    /**
     * <P>Test calling assignWorkLocationsToWorkers with WorkerLocation array containing a non-existent workLocationId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's assignWorkLocationsToWorkers() with WorkerLocation array containing a non-existent workLocationId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ServiceFailureException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testWithNonExistentWorkLocationId() throws Exception
    {
        // Setup
    	WorkerLocation[] existingWorkerWorkLocationMap = getWorkerWorkLocationAssociations();

        WorkerLocation[] workLocations = new WorkerLocation[1];
        workLocations[0] = new WorkerLocation();
        workLocations[0].setWorkerId(existingWorkerWorkLocationMap[0].getWorkerId());
        workLocations[0].setWorkLocationId(new Long("72493844"));

        // Exercise
        sEmployerService.assignWorkLocationsToWorkers(workLocations);
    }

    /**
     * <P>Test calling assignWorkLocationsToWorkers with empty WorkerLocation array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's assignWorkLocationsToWorkers() with empty WorkerLocation array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that no worker's workLocation is changed
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testWithEmptyHashMap() throws Exception
    {
        // Setup
    	WorkerLocation[] existingWorkerWorkLocationMap = getWorkerWorkLocationAssociations();

        WorkerLocation[] workLocations = new WorkerLocation[1];
        workLocations[0] = new WorkerLocation();
        // Exercise
        sEmployerService.assignWorkLocationsToWorkers(workLocations);
        // Verify
        WorkerLocation[] newWorkerWorkLocationMap = getWorkerWorkLocationAssociations();
        AssertJUnit.assertTrue("Unexpected changes in worker workLocation association!",
                          verifyWorkerWorkLocationMap(existingWorkerWorkLocationMap, newWorkerWorkLocationMap));
    }

    /**
     * <P>Test calling assignWorkLocationsToWorkers to change workLocation for 1 worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a new WorkLocation</LI>
     * <LI>Use the first worker retrieved from the DB that's in a different workLocation state than the newly added workLocation</LI>
     * <LI>Apply applicable taxes for this worker</LI>
     * <LI>Setup Map with the worker and the new WorkLocationId</LI>
     * <LI>Call EmployerService's assignWorkLocationsToWorkers() to change workLocation for 1 worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a worker's workLocation is changed, and also that the applicable taxes are re-applied
     * </UL>
     */
    @Test
    public void testChangeWorkLocationForAWorker() throws Exception
    {
        WorkLocation workLocation = null;
        WorkLocation addedWorkLocation = null;
        WorkLocation[] addedWorkLocations = null;
        Long workerId = null;
        Long locationId = null;
        Worker worker = null;
        int numOfTaxDtls = 0;
        boolean workLocationAdded = false;
        try
        {
            // Setup
            workLocation = createWorkLocation("Transient WorkLocation", "800-333-4444");
            WorkLocation[] newWorkLocations = new WorkLocation[1];
            newWorkLocations[0] = workLocation;
            addedWorkLocations = sEmployerService.addWorkLocations(newWorkLocations).getWorkLocations();
            // Locate the workLocation that was added
            for (WorkLocation aWorkLocation : addedWorkLocations)
            {
                if (aWorkLocation.getName().equals(workLocation.getName()))
                {
                    addedWorkLocation = aWorkLocation;
                    break;
                }
            }
            WorkerLocation[] existingWorkerWorkLocationMap = getWorkerWorkLocationAssociations();
            for (WorkerLocation mapItem : existingWorkerWorkLocationMap)
            {
	            worker = entityService.findById(Worker.class, mapItem.getWorkerId());
	            //Use local variables which helps debugging
	            GlobalPostalAddress addressEntity = worker.getWorkLocation().getTaxResidenceAddress();
	            USPostalAddress usAddressEntityProxy = AddressConvertor.getUSFromGlobal(addressEntity, new USPostalAddressImpl());
	            PhysicalAddress mappedAddress = addedWorkLocation.getAddress();
	            
	            if (!usAddressEntityProxy.getRegionOrState().equals(mappedAddress.getStateOrRegion()))
	            {
	                workerId = mapItem.getWorkerId();
	                locationId = mapItem.getWorkLocationId();
	                break;
	            }
            } 
            TaxComponent taxComponent = qaBootstrap.getEjbFromClass(TaxComponent.class);
            worker = taxComponent.applyApplicableTaxes(worker);
            numOfTaxDtls = worker.getTaxDtls().size();
            workLocationAdded = true;
            WorkerLocation[] workLocations = new WorkerLocation[1];
            workLocations[0] = new WorkerLocation();
            workLocations[0].setWorkerId(workerId);
            workLocations[0].setWorkLocationId(addedWorkLocation.getObjectHandle().getId());
            // Exercise
            com.intuit.sb.payroll.services.worker.Worker[] updatedWorkers = sEmployerService.assignWorkLocationsToWorkers(workLocations);
            // Verify
            AssertJUnit.assertEquals("workers returned by assignWorkLocationsToWorkers()", 
        	    1, updatedWorkers.length);
            AssertJUnit.assertEquals("WorkLocation Id does not match expected value!",
                                addedWorkLocation.getObjectHandle().getId(), worker.getWorkLocation().getId());
            AssertJUnit.assertTrue("Applicable taxes not reapplied!", (numOfTaxDtls != worker.getTaxDtls().size()));
        }
        finally
        {
            // Cleanup
            if (workLocationAdded)
            {
                // Restore the worker's original workLocation
                com.intuit.sb.payroll.entity.model.WorkLocation origWorkLocation = entityService.findById(com.intuit.sb.payroll.entity.model.WorkLocation.class, locationId);
                worker.setWorkLocation(origWorkLocation);
                // Delete the workLocation that was added
                com.intuit.sb.payroll.entity.model.WorkLocation workLocationFromDB = null;
                workLocationFromDB = entityService.findById(com.intuit.sb.payroll.entity.model.WorkLocation.class, addedWorkLocation.getObjectHandle().getId());
                entityService.makeTransient(workLocationFromDB);
            }
        }
    }

    /**
     * <P>Test calling assignWorkLocationsToWorkers to remove workLocation for 1 worker with null WorkLocationId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Use the first worker retrieved from the DB</LI>
     * <LI>Setup Map with the worker and null WorkLocationId</LI>
     * <LI>Call EmployerService's assignWorkLocationsToWorkers() to remove workLocation for 1 worker with null WorkLocationId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a worker's workLocation is removed, the applicable taxes are removed, and that the worker is "Not Ready for Pay"
     * </UL>
     */
    @Test
    public void testRemoveAWorkerWorkLocation_WithNullWorkLocationId() throws Exception
    {
    	Long workerId = null;
    	Long locationId = null;
        Worker worker = null;
            // Setup
            WorkerLocation[] existingWorkerWorkLocationMap = getWorkerWorkLocationAssociations();
            for (WorkerLocation mapItem : existingWorkerWorkLocationMap)
            {
	            worker = entityService.findById(Worker.class, mapItem.getWorkerId());
                if (worker instanceof Employee)
	            {
	                workerId = mapItem.getWorkerId();
	                locationId = mapItem.getWorkLocationId();
	                break;
	            }
            } 
            // Apply applicable taxes
            TaxComponent taxComponent = qaBootstrap.getEjbFromClass(TaxComponent.class);
            worker = taxComponent.applyApplicableTaxes(worker);
            WorkerLocation[] workLocations = new WorkerLocation[1];
            workLocations[0] = new WorkerLocation();
            workLocations[0].setWorkerId(workerId);
            // Exercise
            sEmployerService.assignWorkLocationsToWorkers(workLocations);
            // Verify
            AssertJUnit.assertNull("WorkLocation not Removed!", worker.getWorkLocation());
            Set<TaxDtl> taxDtls = worker.getTaxDtls();
            for (TaxDtl taxDtl : taxDtls) {
				AssertJUnit.assertTrue("Expected TaxDtl to be inactive", taxDtl.isDeleted());
			}
            AssertJUnit.assertFalse("Worker should not be ready for pay!", worker.isReadyForPay());

            // Cleanup
            // Restore the worker's original workLocation
            com.intuit.sb.payroll.entity.model.WorkLocation origWorkLocation = entityService.findById(com.intuit.sb.payroll.entity.model.WorkLocation.class, locationId);
            worker.setWorkLocation(origWorkLocation);
    }

    /**
     * <P>Test calling assignWorkLocationsToWorkers to change workLocation for all workers</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a new WorkLocation</LI>
     * <LI>Setup Map of all existing workers with the newly added WorkLocation</LI>
     * <LI>Call EmployerService's assignWorkLocationsToWorkers() to change workLocation for all workers</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that all workers' workLocation is changed
     * </UL>
     */
    @Test
    public void testChangeWorkLocationForAllWorkers() throws Exception
    {
        WorkLocation workLocation = null;
        WorkLocation addedWorkLocation = null;
        WorkLocation[] addedWorkLocations = null;
        WorkerLocation[] existingWorkerWorkLocationMap = null;
        Worker worker = null;
        boolean workLocationAdded = false;
        try
        {
            // Setup
            workLocation = createWorkLocation("Transient WorkLocation", "800-333-4444");
            WorkLocation[] newWorkLocations = new WorkLocation[1];
            newWorkLocations[0] = workLocation;
            addedWorkLocations = sEmployerService.addWorkLocations(newWorkLocations).getWorkLocations();
            // Locate the workLocation that was added
            for (WorkLocation aWorkLocation : addedWorkLocations)
            {
                if (aWorkLocation.getName().equals(workLocation.getName()))
                {
                    addedWorkLocation = aWorkLocation;
                    break;
                }
            }
            existingWorkerWorkLocationMap = getWorkerWorkLocationAssociations();
            WorkerLocation[] workLocations = existingWorkerWorkLocationMap.clone();
            int index = 0;
            for (WorkerLocation mapItem : existingWorkerWorkLocationMap)
            {
            	workLocations[index].setWorkerId(mapItem.getWorkerId());
            	workLocations[index++].setWorkLocationId(addedWorkLocation.getObjectHandle().getId());
           }
            workLocationAdded = true;
            // Exercise
            com.intuit.sb.payroll.services.worker.Worker[] updatedWorkers = sEmployerService.assignWorkLocationsToWorkers(workLocations);
            // Verify
            AssertJUnit.assertEquals("Number of workers returned by assignWorkLocationsToWorkers()", 
        	    workLocations.length, updatedWorkers.length);
            for (WorkerLocation mapItem : existingWorkerWorkLocationMap)
            {
                worker = entityService.findById(Worker.class, mapItem.getWorkerId());
                AssertJUnit.assertEquals("WorkLocation Id does not match expected value!",
                                    addedWorkLocation.getObjectHandle().getId(), worker.getWorkLocation().getId());
            }
        }
        finally
        {
            // Cleanup
            if (workLocationAdded)
            {
                // Restore the worker's original workLocation
            	for (WorkerLocation mapItem : existingWorkerWorkLocationMap)
            	{
                    com.intuit.sb.payroll.entity.model.WorkLocation origWorkLocation = entityService.findById(com.intuit.sb.payroll.entity.model.WorkLocation.class, mapItem.getWorkLocationId());
                    worker = entityService.findById(Worker.class, mapItem.getWorkerId());
                    worker.setWorkLocation(origWorkLocation);
            	}
                // Delete the workLocation that was added
                com.intuit.sb.payroll.entity.model.WorkLocation workLocationFromDB = null;
                workLocationFromDB = entityService.findById(com.intuit.sb.payroll.entity.model.WorkLocation.class, addedWorkLocation.getObjectHandle().getId());
                entityService.makeTransient(workLocationFromDB);
            }
        }
    }
}
