package com.intuit.sb.payroll.qa.services.employer;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;

/**
 * Tests the deleteWorkLocations operation of the EmployerService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class DeleteWorkLocationsTest extends EmployerServiceTestBase {
    
    /**
     * <P>Test calling deleteWorkLocations with an empty array</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's deleteWorkLocations() with an empty array</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteWorkLocationsWithEmptyArray() throws Exception {
        sEmployerService.deleteWorkLocations(new WorkLocation[0]);
    }

    /**
     * <P>Test calling deleteWorkLocations with a null WorkLocation container</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's deleteWorkLocations() with a null WorkLocation container</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteWorkLocationsWithNullContainer() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        // Purposely set it to null
        newWorkLocation[0] = null;
        sEmployerService.deleteWorkLocations(newWorkLocation);
    }

    /**
     * <P>Test calling deleteWorkLocations with null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's deleteWorkLocations() with null</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteWorkLocationsWithNull() throws Exception {
        sEmployerService.deleteWorkLocations(null);
    }

    /**
     * <P>Test calling deleteWorkLocations with a non-existent workLocation id</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's deleteWorkLocations() with a non-existent workLocation id</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testDeleteWorkLocationsWithNonExistentId() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        newWorkLocation[0] = createWorkLocation("Transient WorkLocation", "800-333-4444");
        // Now purposely set the uniqueId to a non existent value
        newWorkLocation[0].setObjectHandle(new ObjectHandle(new Long("83434176"), new Long("0")));
        sEmployerService.deleteWorkLocations(newWorkLocation);
    }

    /**
     * <P>Test calling deleteWorkLocations with a null workLocation id</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call EmployerService's deleteWorkLocations() with a null workLocation id</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteWorkLocationsWithNullId() throws Exception {
        WorkLocation[] newWorkLocation = new WorkLocation[1];
        newWorkLocation[0] = createWorkLocation("Transient WorkLocation", "800-333-4444");
        // Now purposely set the uniqueId to a null value
        newWorkLocation[0].setObjectHandle(null);
        sEmployerService.deleteWorkLocations(newWorkLocation);
    }

    /**
     * <P>Test deleting existing workLocation that has worker assigned to it</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's deleteWorkLocations() to delete existing workLocation that has worker assigned to it</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteExistingWorkLocationWithWorkerAssignedToIt() throws Exception {
        // Setup
        WorkLocation workLocation = null;
        WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
        workLocation = existingLocations[existingLocations.length-1];
        WorkLocation[] workLocationsToDelete = new WorkLocation[1];
        workLocationsToDelete[0] = workLocation;
        // Exercise
        sEmployerService.deleteWorkLocations(workLocationsToDelete);
        // Verify
        WorkLocation[] newExistingLocations = sEmployerService.getWorkLocations();
        AssertJUnit.assertEquals("Number of WorkLocations mismatch!", existingLocations.length-1, newExistingLocations.length);
        // Verify that the deleted workLocation is really deleted
        for (WorkLocation aWorkLocation : newExistingLocations) {
            if (aWorkLocation.getName().equals(workLocation.getName())) {
                AssertJUnit.fail("workLocation not deleted!");
                break;
            }
        }
        // For cleanup, the entire transaction is rolled back in BasicTestBase
    }

    /**
     * <P>Test deleting the last workLocation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's deleteWorkLocations() to delete the last workLocation</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteLastWorkLocation() throws Exception {
        // Setup
        WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
        for (WorkLocation workLocation : existingLocations) {
            removeAllWorkersFromWorkLocation(workLocation.getObjectHandle());
        }
        // Exercise
        sEmployerService.deleteWorkLocations(existingLocations);
        // For cleanup, the entire transaction is rolled back in BasicTestBase
    }

    /**
     * <P>Test deleting a workLocation that has worker assigned to it</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's deleteWorkLocations() to delete a workLocation that has worker assigned to it</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testDeleteNewWorkLocationWithWorkerAssignedToIt() throws Exception {
        WorkLocation[] addedWorkLocations = null;
        WorkLocation addedWorkLocation = null;
        // Setup
        WorkLocation workLocation = createWorkLocation("Transient WorkLocation", "800-333-4444");
        WorkLocation[] newWorkLocations = new WorkLocation[1];
        newWorkLocations[0] = workLocation;
        addedWorkLocations = sEmployerService.addWorkLocations(newWorkLocations).getWorkLocations();
        // Locate the workLocation that was added
        for (WorkLocation aWorkLocation : addedWorkLocations) {
            if (aWorkLocation.getName().equals(workLocation.getName())) {
                addedWorkLocation = aWorkLocation;
                break;
            }
        }
        @SuppressWarnings("unused")
        Long workerId = createWorkerWithWorkLocation(addedWorkLocation);
        WorkLocation[] workLocationsToDelete = new WorkLocation[1];
        workLocationsToDelete[0] = addedWorkLocation;
        // Exercise
        sEmployerService.deleteWorkLocations(workLocationsToDelete);
        // For cleanup, the entire transaction is rolled back in BasicTestBase
    }

    /**
     * <P>Test deleting the same workLocation twice</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's deleteWorkLocations() to delete the same workLocation twice</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown
     * </UL>
     */
    @Test (expectedExceptions=ServiceFailureException.class)
    public void testDeleteSameWorkLocationTwice() throws Exception {
        WorkLocation[] addedWorkLocations = null;
        WorkLocation addedWorkLocation = null;
        // Setup
        WorkLocation workLocation = createWorkLocation("Transient WorkLocation", "800-333-4444");
        WorkLocation[] newWorkLocations = new WorkLocation[1];
        newWorkLocations[0] = workLocation;
        addedWorkLocations = sEmployerService.addWorkLocations(newWorkLocations).getWorkLocations();
        // Locate the workLocation that was added
        for (WorkLocation aWorkLocation : addedWorkLocations) {
            if (aWorkLocation.getName().equals(workLocation.getName())) {
                addedWorkLocation = aWorkLocation;
                break;
            }
        }
        WorkLocation[] workLocationsToDelete = new WorkLocation[1];
        workLocationsToDelete[0] = addedWorkLocation;
        // Exercise
        sEmployerService.deleteWorkLocations(workLocationsToDelete);
        // Try to delete it again
        sEmployerService.deleteWorkLocations(workLocationsToDelete);
        // For cleanup, the entire transaction is rolled back in BasicTestBase
    }

    /**
     * <P>Test deleting a workLocation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's deleteWorkLocations() to delete a workLocation</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that workLocation is deleted successfully
     * </UL>
     */
    @Test
    public void testDeleteOneWorkLocation() throws Exception {
        WorkLocation[] addedWorkLocations = null;
        WorkLocation addedWorkLocation = null;
        // Setup
        WorkLocation[] originalLocations = sEmployerService.getWorkLocations();
        WorkLocation workLocation = createWorkLocation("Transient WorkLocation", "800-333-4444");
        WorkLocation[] newWorkLocations = new WorkLocation[1];
        newWorkLocations[0] = workLocation;
        addedWorkLocations = sEmployerService.addWorkLocations(newWorkLocations).getWorkLocations();
        // Locate the workLocation that was added
        for (WorkLocation aWorkLocation : addedWorkLocations) {
            if (aWorkLocation.getName().equals(workLocation.getName())) {
                addedWorkLocation = aWorkLocation;
                break;
            }
        }
        @SuppressWarnings("unused")
        Long workerId = createWorkerWithWorkLocation(addedWorkLocation);
        WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
        WorkLocation[] workLocationsToDelete = new WorkLocation[1];
        workLocationsToDelete[0] = addedWorkLocation;
        removeAllWorkersFromWorkLocation(addedWorkLocation.getObjectHandle());
        // Exercise
        WorkLocation[] workLocationsAfterDelete = sEmployerService.deleteWorkLocations(workLocationsToDelete);
        // Verify
        WorkLocation[] newExistingLocations = sEmployerService.getWorkLocations();
        AssertJUnit.assertEquals("Number of WorkLocations mismatch!", existingLocations.length-1, newExistingLocations.length);
        // Verify that the deleted workLocation is really deleted
        for (WorkLocation aWorkLocation : newExistingLocations) {
            if (aWorkLocation.getName().equals(addedWorkLocation.getName())) {
                AssertJUnit.fail("workLocation not deleted!");
                break;
            }
        }
        // Verify that remaining workLocations are correct
        AssertJUnit.assertTrue("WorkLocation info mismatch", verifyWorkLocations(originalLocations, workLocationsAfterDelete));
        // For cleanup, the entire transaction is rolled back in BasicTestBase
    }

    /**
     * <P>Test deleting 2 workLocations</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's deleteWorkLocations() to delete 2 workLocations</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that workLocation are deleted successfully
     * </UL>
     */
    @Test
    public void testDeleteTwoWorkLocations() throws Exception {
        final int numOfWorkLocations = 2;
        WorkLocation[] addedWorkLocations = null;
        WorkLocation[] workLocationsAdded = null;
        // Setup
        WorkLocation[] originalLocations = sEmployerService.getWorkLocations();
        WorkLocation workLocation1 = createWorkLocation("Transient WorkLocation 1", "800-333-4444");
        WorkLocation workLocation2 = createWorkLocation("Transient WorkLocation 2", "800-333-3333");
        WorkLocation[] newWorkLocations = new WorkLocation[numOfWorkLocations];
        newWorkLocations[0] = workLocation1;
        newWorkLocations[1] = workLocation2;
        addedWorkLocations = sEmployerService.addWorkLocations(newWorkLocations).getWorkLocations();
        workLocationsAdded = new WorkLocation[numOfWorkLocations];
        int i = 0;
        // Locate the workLocations that were added
        for (WorkLocation aWorkLocation : addedWorkLocations) {
            if (aWorkLocation.getName().equals(workLocation1.getName()) ||
                aWorkLocation.getName().equals(workLocation2.getName())) {
                workLocationsAdded[i++] = aWorkLocation;
                if (i >= numOfWorkLocations) {
                    break;
                }
            }
        }
        @SuppressWarnings("unused")
        Long workerId1 = createWorkerWithWorkLocation(workLocationsAdded[0]);
        @SuppressWarnings("unused")
        Long workerId2 = createWorkerWithWorkLocation(workLocationsAdded[1]);
        WorkLocation[] existingLocations = sEmployerService.getWorkLocations();
        removeAllWorkersFromWorkLocation(workLocationsAdded[0].getObjectHandle());
        removeAllWorkersFromWorkLocation(workLocationsAdded[1].getObjectHandle());
        // Exercise
        WorkLocation[] workLocationsAfterDelete = sEmployerService.deleteWorkLocations(workLocationsAdded);
        // Verify
        WorkLocation[] newExistingLocations = sEmployerService.getWorkLocations();
        AssertJUnit.assertEquals("Number of WorkLocations mismatch!", existingLocations.length-numOfWorkLocations, newExistingLocations.length);
        // Verify that the deleted workLocation is really deleted
        for (WorkLocation aWorkLocation : newExistingLocations) {
            if (aWorkLocation.getName().equals(workLocation1.getName()) ||
                aWorkLocation.getName().equals(workLocation2.getName())) {
                AssertJUnit.fail("workLocation not deleted!");
                break;
            }
        }
        // Verify that remaining workLocations are correct
        AssertJUnit.assertTrue("WorkLocation info mismatch", verifyWorkLocations(originalLocations, workLocationsAfterDelete));
        // For cleanup, the entire transaction is rolled back in BasicTestBase
    }

}
