/**
 * Copyright: 2007, Intuit, Inc. All rights reserved.
 *
 */
package com.intuit.sb.payroll.qa.services.employer;


import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.EmployerInfo;
import com.intuit.sb.payroll.services.employer.EmployerService;
import com.intuit.sb.payroll.services.employer.EmployerServiceTestBase;
import com.intuit.sb.payroll.task.Task;
import com.intuit.sb.payroll.task.TaskService;

/**
 * <P>The <CODE>UpdateAssistedStateTest</CODE> class contains test methods for updateAssistedState() in employer service
 *
 * $Author: kvenkatraman $
 * $Revision: #1 $
 * $DateTime: 2009/03/3 23:35:31 $
 *
 */
public class UpdateAssistedStateTest extends EmployerServiceTestBase{

	protected static EmployerService employerService;
	protected static TaskService taskService;

	@BeforeClass
	public static void initializeUpdateAssistedStateTestClasss()
	{
		employerService = qaBootstrap.getEjbFromClass(EmployerService.class);
		taskService = qaBootstrap.getEjbFromClass(TaskService.class);
	}

	// Not able to delete Current Employer as the txn is initiated in the BasicTestBase Before
	
	/**
	 * <P>
	 * Test updateAssistedState with null parameter
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Pass null parameter to  updateAssistedState
	 * </OL>
	 * <OL>
	 * <LI>
	 * Verify that ValidationException is thrown</LI>
	 * </OL>
	 */
	@Test(expectedExceptions=ValidationException.class)
	public void testUpdateAssistedStateWithNull() throws Exception{
		employerService.updateAssistedState(null);

	}
	/**
	 * <P>
	 * Test updateAssistedState passing Assisted type as InitiatedOnBoadting
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Pass InitiatedOnBoadting type parameter to  updateAssistedState
	 * </OL>
	 * <OL>
	 * <LI>
	 * Verify that Employer assisted state is INITIATE ASSISTED</LI>
	 * </OL>
	 */
	@Test(enabled=false, groups={"broken"})
	public void testUpdateAssistedStateWithAssistedState()throws Exception{
		// Delete the current employer in a txn that eventually will be rolled back
		EmployerComponentTestUtils.deleteCurrentEmployer();
		
		employerService.loadApplication();
		
		// Check the Assisted status for the newly created company
		EmployerInfo employerInfo = employerService.getEmployerInfo();

     	AssertJUnit.assertEquals("Expected Assisted State in not returned ", 
     			AssistedStateType.NOT_ASSISTED,employerInfo.getAssistedStateType());
     	
		
		employerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);
		employerInfo = employerService.getEmployerInfo();

     	AssertJUnit.assertEquals("Expected Assisted State in not returned ", 
     			AssistedStateType.INITIATED_ONBOARDING, employerInfo.getAssistedStateType());
	}
	/**
	 * <P>
	 * Test updateAssistedState passing Assisted type as empty assisted state type
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Pass AssistedStateType() type parameter to  updateAssistedState
	 * </OL>
	 * <OL>
	 * <LI>
	 * Verify that Employer assisted state is NOT_ASSISTED</LI>
	 * </OL>
	 */
	@Test(enabled=false, groups={"broken"})
	public void testUpdateAssistedStateWithNonAssistedState() throws Exception{
		// Delete the current employer in a txn that eventually will be rolled back
		EmployerComponentTestUtils.deleteCurrentEmployer();
		employerService.loadApplication();
		
		employerService.updateAssistedState(AssistedStateType.NOT_ASSISTED);
		EmployerInfo employerInfo = employerService.getEmployerInfo();
		
     	AssertJUnit.assertEquals("Expected Assisted State in not returned ", AssistedStateType.NOT_ASSISTED,
     			employerInfo.getAssistedStateType());
	}

	/**
	 * <P>
	 * Test that no task SETUP_ASSISTED_TASK is returned by the system when the employer assisted state is InitiatedOnBoarding
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set the employer assisted state to InitiatedOnBoarding
	 * <LI>Get the task from the task service
	 * </OL>
	 * <OL>
	 * <LI>
	 * Verify that SETUP_ASSISTED_TASK is not returned</LI>
	 * </OL>
	 */

	@Test(enabled=false, groups={"broken"})
	public void testUpdateAssistedForNoSetupAssistedTask() throws Exception{

		employerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);
		Task[] posttasks = taskService.getCurrentTasks();

		for(int i=0 ; i< posttasks.length;i++){
			AssertJUnit.assertFalse("SETUP_ASSISTED_TASK is not expected",posttasks[i].getName().equals(EmployerPTSWorkflowTasks.SETUP_ASSISTED_TASK));
		}
	}
	
	// TODO : JavaDoc
	@Test (enabled=false,expectedExceptions=ValidationException.class) // Move this test to UpdateAssistedStateTest
	public void testUpdateAssistedStateAfterAddCompanyPTSCall() throws ValidationException , Exception{
		
		
		
		//addCompanyToPTS();
		
		sEmployerService.updateAssistedState(AssistedStateType.NOT_ASSISTED); // this should throw validation Exception
	}


}
