package com.intuit.sb.payroll.qa.services.pto;

import java.math.BigDecimal;
import java.util.Calendar;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.pto.AccrualStartType;
import com.intuit.sb.payroll.services.pto.PTOAccrualType;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.pto.PTOService;
import com.intuit.sb.payroll.services.pto.PTOType;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.PtoBaseRateType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;

public class SavePTOTest extends WorkerServiceTestBase
{
	private static PTOService sPTOService;
	
	private static Worker newWorker;
	private static Worker savedWorker;
	private static long sWorkerId;
	
	@BeforeClass
	public static void initializeSavePTOTestClass()
	{
		sPTOService = qaBootstrap.getEjbFromClass(PTOService.class);
	}
	
	public void setUpWorker() throws Exception
	{
		newWorker = createWorker(); // create a worker with minimum data

		// Test SaveWorker with no compensations
		savedWorker = sWorkerService.saveWorker(newWorker);
		sWorkerId = savedWorker.getObjectHandle().getId();
		
		// save salary compensation item
		CompensationItem[] inputCompensations = new CompensationItem[1];
		inputCompensations[0] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Salary, "salary",true);
		inputCompensations[0].setPtoBaseRateType(PtoBaseRateType.PTO);

        // Test inserting compensations
		sWageService.saveCompensationItems(sWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, inputCompensations));
	}
	
	public void tearDownWorker() throws Exception
	{
		cleanUpEIWorker(savedWorker.getObjectHandle().getId());
	}
	
	private PTOInfo createPTOInfo(PTOType type) throws ValidationException, ServiceFailureException
	{
		PTOInfo ptoInfo = new PTOInfo();
		ptoInfo.setWorkerId(sWorkerId);
		ptoInfo.setName("Floating goliday");
		ptoInfo.setPtoType(type);
		ptoInfo.setAccrualRate(new Double("2.35"));
		ptoInfo.setAllowCarryOver(true);
		ptoInfo.setPtoAccrualType(PTOAccrualType.PER_YEAR_FROM_HIRE_DATE);
		ptoInfo.setLimitAccrual(true);
		ptoInfo.setMaximumAccrual(new Double("200.00"));
		ptoInfo.setAccruedHours(new Double("5.40"));
		ptoInfo.setDisbursedHoursYTD(new Double("0.00"));
		
		Calendar now = DateUtil.getNowCalendar();
		ptoInfo.setAccrualStartDate(now.getTime());
		ptoInfo.setAccrualStartType(AccrualStartType.IMMEDIATELY);
		
		return ptoInfo;
	}
	
	private boolean comparePTOInfo(PTOInfo ptoInfo, PTOInfo resultPTOInfo)
	{
		if(ptoInfo.getAccrualRate().doubleValue() == resultPTOInfo.getAccrualRate().doubleValue() 
				&& ptoInfo.getAccrualStartDate().equals(resultPTOInfo.getAccrualStartDate())
				&& ptoInfo.getAccrualStartType().equals(resultPTOInfo.getAccrualStartType())
				&& ptoInfo.getAccruedHours().doubleValue() == resultPTOInfo.getAccruedHours().doubleValue()
				&& ptoInfo.getDisbursedHoursYTD().doubleValue() == resultPTOInfo.getDisbursedHoursYTD().doubleValue()
				&& ptoInfo.getMaximumAccrual().doubleValue() == resultPTOInfo.getMaximumAccrual().doubleValue()
				&& ptoInfo.getName().equals(resultPTOInfo.getName())
				&& ptoInfo.getPtoAccrualType().equals(resultPTOInfo.getPtoAccrualType())
				&& ptoInfo.getPtoType().equals(resultPTOInfo.getPtoType())
				&& ptoInfo.getWorkerId().longValue() == resultPTOInfo.getWorkerId().longValue())
		{
			return true;
		}
		
		return false;
	}
	
	/**
     * <P>Test savePTO() with  all valid details - Happy Path</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker with compensation items and Save worker</LI>
     * <LI>Create PTOInfo(in this case, VACATION)</LI>
     * <LI>Save the PTOInfo</LI>
     * <LI>make the getPTOByNameAndType() API call with sWorkerId, PTO Name and PTO Type</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PTOInfo is saved on the worker correctly</LI>
     * @throws Exception
     * </UL>
     */
	@Test
	public void testNewSaveVacationPTO() throws Exception
	{
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.VACATION);
		sPTOService.savePTO(expectedPTOInfo);
		
		PTOInfo resultPTOInfo = sPTOService.getPTOByNameAndType(sWorkerId, expectedPTOInfo.getName(), expectedPTOInfo.getPtoType());
		AssertJUnit.assertTrue(comparePTOInfo(expectedPTOInfo, resultPTOInfo));
		tearDownWorker();
	}
	
	/**
     * <P>Test savePTO() with  all valid details - Happy Path</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker with compensation items and Save worker</LI>
     * <LI>Create PTOInfo(in this case, SICK)</LI>
     * <LI>Save the PTOInfo</LI>
     * <LI>make the getPTOByNameAndType() API call with sWorkerId, PTO Name and PTO Type</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PTOInfo is saved on the worker correctly</LI>
     * @throws Exception
     * </UL>
     */
	@Test
	public void testNewSickPTOInfo() throws Exception
	{
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME);
		sPTOService.savePTO(expectedPTOInfo);
		
		PTOInfo resultPTOInfo = sPTOService.getPTOByNameAndType(sWorkerId, expectedPTOInfo.getName(), expectedPTOInfo.getPtoType());
		AssertJUnit.assertTrue(comparePTOInfo(expectedPTOInfo, resultPTOInfo));
		tearDownWorker();
	}
	
	/**
     * <P>Test savePTO() by passing a PTO that already exists on a worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker with compensation items and Save worker</LI>
     * <LI>Create PTOInfo(in this case, SICK)</LI>
     * <LI>Save the PTOInfo</LI>
     * <LI>Create the second PTOInfo(in this case, SICK)</LI>
     * <LI>Save the PTOInfo</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown</LI>
     * @throws Exception
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testSaveDuplicatePTO() throws Exception
	{
		setUpWorker();
		PTOInfo onePTOInfo = createPTOInfo(PTOType.SICK_TIME);
		sPTOService.savePTO(onePTOInfo);
		
		PTOInfo twoPTOInfo = createPTOInfo(PTOType.SICK_TIME);
		twoPTOInfo.setName(onePTOInfo.getName());
		sPTOService.savePTO(twoPTOInfo);
		tearDownWorker();
	}
	
	/**
     * <P>Test savePTO() with  Update</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker with compensation items and Save worker</LI>
     * <LI>Create PTOInfo(in this case, SICK)</LI>
     * <LI>Save the PTOInfo</LI>
     * <LI>make the getPTOByNameAndType() API call with sWorkerId, PTO Name and PTO Type</LI>
     * <LI>Make changes to the PTOInfo</LI>
     * <LI>Save the updated PTOInfo</LI>
     * <LI>make the getPTOByNameAndType() API call with sWorkerId, PTO Name and PTO Type</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PTOInfo is updated on the worker correctly</LI>
     * @throws Exception
     * </UL>
     */
//ASTEC PTO FIXME
	@Test(enabled=false, groups={"broken"})
	public void testUpdateExistingPTO() throws Exception
	{
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME);
		sPTOService.savePTO(expectedPTOInfo);
		
		PTOInfo newPTOInfo = sPTOService.getPTOByNameAndType(expectedPTOInfo.getWorkerId(), expectedPTOInfo.getName(), expectedPTOInfo.getPtoType());
		newPTOInfo.setName("Floating Holiday");
		newPTOInfo.setAccrualRate(new Double("4.00"));
		
		Calendar now = DateUtil.getNowCalendar();
		now.add(Calendar.DAY_OF_MONTH, 10);
		newPTOInfo.setAccrualStartDate(now.getTime());			
		newPTOInfo.setAccrualStartType(AccrualStartType.ON_START_DATE);
		newPTOInfo.setPtoType(PTOType.VACATION);
		newPTOInfo.setPtoAccrualType(PTOAccrualType.PER_YEAR_FROM_HIRE_DATE);
		newPTOInfo.setAllowCarryOver(false);
		newPTOInfo.setLimitAccrual(false);
		newPTOInfo.setMaximumAccrual(new Double("999.00"));
		newPTOInfo.setAccruedHours(new Double("100.00"));
		newPTOInfo.setDisbursedHoursYTD(new Double("0.00"));
		
		sPTOService.savePTO(newPTOInfo);
		
		PTOInfo resultPTOInfo = sPTOService.getPTOByNameAndType(sWorkerId, newPTOInfo.getName(), newPTOInfo.getPtoType());
		AssertJUnit.assertTrue(comparePTOInfo(newPTOInfo, resultPTOInfo));
	}
	
	/**
     * <P>Test savePTO() with  null PTO name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup worker with compensation items</LI>
     * <LI>Create a SICK PTOInfo</LI>
     * <LI>Set the PTO name to null</LI>
     * <LI>Save the PTO</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * @throws Exception
     * </UL>
     */
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testSavePTOInfoWithNullPTOName() throws Exception
	{
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME);
		expectedPTOInfo.setName(null);
		sPTOService.savePTO(expectedPTOInfo);
		tearDownWorker();
	}
	
	/**
     * <P>Test savePTO() with  empty("") PTO name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup worker with compensation items</LI>
     * <LI>Create a SICK PTOInfo</LI>
     * <LI>Set the PTO name to ""</LI>
     * <LI>Save the PTO</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * @throws Exception
     * </UL>
     */
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testSavePTOInfoWithEmptyPTOName() throws Exception
	{
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME);
		expectedPTOInfo.setName("");
		sPTOService.savePTO(expectedPTOInfo);
		
		tearDownWorker();
	}
	
	/**
     * <P>Test savePTO() without compensation items on Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create Worker without compensation items</LI>
     * <LI>Save worker</LI>
     * <LI>Create a SICK PTOInfo</LI>
     * <LI>Set WorkerID on the PTOInfo</LI>
     * <LI>Save the PTO</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a validationException is thrown</LI>
     * @throws Exception
     * </UL>
     */
	@Test (expectedExceptions=ValidationException.class)
	public void testSavePTOInfoWithoutCompensationItemsOnWorker() throws Exception
	{
		Worker worker = createWorker();
		Worker savedWorker = sWorkerService.saveWorker(worker);
			
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME);
		expectedPTOInfo.setWorkerId(savedWorker.getObjectHandle().getId());
		
		sPTOService.savePTO(expectedPTOInfo);
		// No need to cleanup since the entire transaction is rolled back
	}
	
	/**
     * <P>Test savePTO() with accrual start date before current date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Setup Worker with compensation items</LI>
     * <LI>Create a SICK PTOInfo</LI>
     * <LI>Set the accrual start date to some date in history</LI>
     * <LI>Save the PTO</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the ValidationException is thrown</LI>
     * @throws Exception
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testSavePTOWithAccrualStartDateBeforeWorkerHireDate() throws Exception
	{
		setUpWorker();

		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME);
		
		Calendar now = DateUtil.getNowCalendar();
		now.add(Calendar.DAY_OF_YEAR, -30);
		expectedPTOInfo.setAccrualStartDate(now.getTime());
		sPTOService.savePTO(expectedPTOInfo);
		tearDownWorker();
	}
}
