package com.intuit.sb.payroll.qa.services.taxes;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import junit.framework.Assert;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.Agency;
import com.intuit.sb.payroll.entity.model.AgencyEnum;
import com.intuit.sb.payroll.entity.model.Jurisdiction;
import com.intuit.sb.payroll.entity.model.UnemploymentInsDocDtl;
import com.intuit.sb.payroll.entity.taxes.withholding.UnemploymentInsDtlEAO;
import com.intuit.sb.payroll.services.taxes.EmployerTaxSettingsServiceTestBase;
import com.intuit.sb.payroll.services.taxes.EmployerUnemploymentIns;
import com.intuit.sb.payroll.services.taxes.impl.SuiTaxMapper;

// TODO 2) compare the exception messages
// TODO 3) Don't compare exception messages since they will be globalized soon.  ErrorCodes coming soon.  -TH
/**
 * The SUISettingTest defines test which validate the SUI rates for various validation criterias.
 */
public class SUISettingTest extends EmployerTaxSettingsServiceTestBase {

	// EAO for the Unemployment Insurance.
	protected static UnemploymentInsDtlEAO sUnemploymentInsDtlEAO;

	private double ZERO_RATE = 0.0;
	private double NEGATIVE_RATE = -50.2;
	private double RATE_TOO_LARGE = 101;
	private double RATE_WITH_SIX_DECIMAL_PLACES = 10.123456;
	private static String ALPHANUMERIC_RATE = "A&b12%3";

	@BeforeClass
	public static void initializeSUISettingTestClass()
	{
		sUnemploymentInsDtlEAO = qaBootstrap.getEjbFromClass(UnemploymentInsDtlEAO.class);
	}
	


	/**
	 * <P>
	 * Test updateEmployerSUIsettings() passing null as parameter
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call updateEmployerSUIsettings() with null as parameter</LI>
	 * <LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * 
	 * @throws ValidationException </UL>
	 */
	@Test
	public void testUpdateEmployerSUIsettingsWithNullSuisetting() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			mEmployerTaxSettingService.updateEmployerUnemploymentIns(null);
		} catch (ValidationException ex) {
			// Validation Exception is expected.
			validationExceptionThrown = true;
			AssertJUnit.assertEquals("EmployerUnemploymentIns can not be null", ex.getMessage());
		} catch (ServiceFailureException e) {
			AssertJUnit.fail("Service Failure Exception raised when updating the tax settings : " + e.getMessage());
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
	}

	/**
	 * <P>
	 * Test updateEmployerSUIsettings() passing empty SUI setting collection as parameter
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Call updateEmployerSUIsettings() with empty SUI setting collection as parameter</LI>
	 * <LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that ValidationException is thrown</LI>
	 * 
	 * @throws ValidationException </UL>
	 */
	@Test
	public void testUpdateEmployerSUIsettingsWithEmptySuisetting() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			mEmployerTaxSettingService.updateEmployerUnemploymentIns(new EmployerUnemploymentIns[] {});
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
			AssertJUnit.assertEquals("EmployerUnemploymentIns can not be empty", ex.getMessage());
		} catch (ServiceFailureException e) {
			AssertJUnit.fail("Service Failure Exception raised when updating the tax settings : " + e.getMessage());
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
	}

	/**
	 * <P>
	 * Test updateEmployerSUIsettings() with current rate empty
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the SUI setting info from the system</LI>
	 * <LI>Loop through the SUI setting info collection and update the current rate with empty string for each element
	 * in the collection</LI>
	 * <LI>Call updateEmployerSUIsettings() with updated collection as parameter</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that the the ValidationException is thrown</LI>
	 * 
	 * @throws ValidationException </UL>
	 */
	@Test(expectedExceptions = ValidationException.class)
	public void testUpdateEmployerSUIsettingsWithEmptyCurrentRate() throws Exception {
		boolean validationExceptionThrown = false;
		EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
		for (EmployerUnemploymentIns s : suisettings) {
			s.setEffectiveRate("");
		}
		mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisettings);
		AssertJUnit.assertTrue(validationExceptionThrown);
	}


	/**
	 * <P>
	 * Test updateEmployerSUIsettings() with alphanumeric character for current rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the SUI setting info from the system</LI>
	 * <LI>Loop through the SUI setting info collection and update current rate with alphanumeric for each element in
	 * the collection</LI>
	 * <LI>Call updateEmployerSUIsettings() with updated collection as parameter</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that the the ValidationException is thrown</LI>
	 * 
	 * @throws ValidationException </UL>
	 */
	@Test
	public void testUpdateEmployerSUIsettingsWithAlphaNumericCurrentRate() throws Exception {
		boolean validationExceptionThrown = false;
		try {
			EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
			for (EmployerUnemploymentIns s : suisettings) {
				s.setEffectiveRate(ALPHANUMERIC_RATE);
			}
			mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisettings);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		} catch (ServiceFailureException e) {
			AssertJUnit.fail("Service Failure Exception raised when updating the tax settings : " + e.getMessage());
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
	}

	/**
	 * <P>
	 * Test updateEmployerSUIsettings() with negative current rate
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the SUI setting info from the system</LI>
	 * <LI>Loop through the SUI setting info collection and update current rate with negative rate for each element in
	 * the collection</LI>
	 * <LI>Call updateEmployerSUIsettings() with updated collection as parameter</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that the the ValidationException is thrown</LI>
	 * 
	 * @throws ValidationException </UL>
	 */
	@Test
	public void testUpdateEmployerSUIsettingsWithNegativeCurrentRate() {
		boolean validationExceptionThrown = false;
		try {
			EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
			for (EmployerUnemploymentIns s : suisettings) {
				s.setEffectiveRate(new Double(NEGATIVE_RATE).toString());
				this.NEGATIVE_RATE = this.NEGATIVE_RATE + 1;
			}
			mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisettings);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		} catch (ServiceFailureException e) {
			AssertJUnit.fail("Service Failure Exception raised when updating the tax settings : " + e.getMessage());
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
	}

	/**
	 * Validate that a exception is thrown when a null current rate is being set.
	 * 
	 * @throws Exception
	 */
	@Test
	public void testUpdateEmployerSUIsettingsWithNullCurrentRate() {
		boolean validationExceptionThrown = false;
		try {
			EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
			for (EmployerUnemploymentIns s : suisettings) {
				s.setEffectiveRate(null);
			}
			mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisettings);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		} catch (ServiceFailureException e) {
			AssertJUnit.fail("Service Failure Exception raised when updating the tax settings : " + e.getMessage());
		}

		AssertJUnit.assertTrue(validationExceptionThrown);
	}

	/**
	 * <P>
	 * Test updateEmployerSUIsettings() with current rate > 100
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the SUI setting info from the system</LI>
	 * <LI>Loop through the SUI setting info collection and update current rate with value > 100 for each element in the
	 * collection</LI>
	 * <LI>Call updateEmployerSUIsettings() with updated collection as parameter</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that the the ValidationException is thrown</LI>
	 * 
	 * @throws ValidationException </UL>
	 */
	@Test
	public void testUpdateEmployerSUIsettingsWithCurrentRateTooLarge() {
		boolean validationExceptionThrown = false;
		try {
			EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
			for (EmployerUnemploymentIns s : suisettings) {
				s.setEffectiveRate(new Double(RATE_TOO_LARGE).toString());
			}
			mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisettings);
		} catch (ValidationException ex) {
			validationExceptionThrown = true;
		} catch (ServiceFailureException e) {
			AssertJUnit.fail("Service Failure Exception raised when updating the tax settings : " + e.getMessage());
		}
		AssertJUnit.assertTrue(validationExceptionThrown);
	}

	/**
	 * <P>
	 * Test updateEmployerSUIsettings() with current rate with value having more than 5 decimal places
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the SUI setting info from the system</LI>
	 * <LI>Loop through the SUI setting info collection and update current rate with value more than 5 decimal places
	 * for each element in the collection</LI>
	 * <LI>Call updateEmployerSUIsettings() with updated collection as parameter</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that the the ValidationException is thrown</LI>
	 * 
	 * @throws ValidationException </UL>
	 */
	// Test is ignored and bug is logged (28/10/2008)
	@Test(enabled=false,expectedExceptions = ValidationException.class)
	public void testUpdateEmployerSUIsettingsWithCurrentRateMoreThanFiveDecimalPlace() {
		try {
			EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
			for (EmployerUnemploymentIns s : suisettings) {
				s.setEffectiveRate(new Double(RATE_WITH_SIX_DECIMAL_PLACES).toString());
				this.RATE_WITH_SIX_DECIMAL_PLACES = this.RATE_WITH_SIX_DECIMAL_PLACES + 1;
			}
			mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisettings);
		} catch (ValidationException ex) {

		} catch (ServiceFailureException e) {
			AssertJUnit.fail("Service Failure Exception raised when updating the tax settings : " + e.getMessage());
		}
	}

	/**
	 * <P>
	 * Test updateEmployerSUIsettings() with current rate with value 0
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the SUI setting info from the system</LI>
	 * <LI>Loop through the SUI setting info collection and update current rate with value 0 for each element in the
	 * collection</LI>
	 * <LI>Call updateEmployerSUIsettings() with updated collection as parameter</LI>
	 * <LI>Call getEmployerSUIsettings() after updation
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the updated collection is same as expected collection</LI>
	 * 
	 * @throws ValidationException </UL>
	 */
	@Test
	public void testUpdateEmployerSUIsettingsWithCurrentRateWithZeroValue() {

		try {
			EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
			for (EmployerUnemploymentIns s : suisettings) {
				s.setEffectiveRate(new Double(ZERO_RATE).toString());
			}
			mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisettings);
			EmployerUnemploymentIns[] updatedsuisetting = mEmployerTaxSettingService.getEmployerUnemploymentIns();
			for (EmployerUnemploymentIns s : updatedsuisetting) {
				AssertJUnit.assertEquals(new Double(ZERO_RATE).toString(), s.getEffectiveRate());
			}
		} catch (Exception ex) {
			AssertJUnit.fail("Exception raised when setting 0 rate for current rate : " + ex.getMessage());
		}
	}

	
	/**
	 * <P>
	 * Test updateEmployerSUIsettings() with valid values - happy path
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the SUI setting info from the system</LI>
	 * <LI>Loop through the SUI setting info collection and update the Effective rate 
	 * for each element in the collection</LI>
	 * <LI>Call updateEmployerSUIsettings() with updated collection as parameter</LI>
	 * <LI>Get the updated SUI Setting from the database again</LI>
	 * <LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that the SUI setting collection is same as that of updated one</LI>
	 * @throws ServiceFailureException 
	 * 
	 * @throws ValidationException </UL>
	 */
	@Test
	public void testUpdateEmployerSUIsettings() throws ServiceFailureException{
		
		EmployerUnemploymentIns[] empSUIRates = mEmployerTaxSettingService.getEmployerUnemploymentIns();
		validateSUIDataModel(empSUIRates);
		// TODO test more
	}
	

	/**
	 * <P>
	 * Test getEmployerSUISettings() - happypath
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Get the SUI setting info from the system</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that the mandatory data are fetched</LI>
	 * @throws ServiceFailureException 
	 * 
	 * @throws ValidationException </UL>
	 */

	@Test
	public void testGetEmployerSUIsettings() throws ServiceFailureException // + ive case
	{
		EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
		validateSUIDataModel(suisettings);
	}
	
	
	@Test(expectedExceptions=ValidationException.class)//IP000106
	public void testUpdateSuiSettingState() throws Exception
	{
		EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
		for(EmployerUnemploymentIns s : suisettings)
        {
           s.setState("XYZ"); 
        }
		mEmployerTaxSettingService.updateEmployerUnemploymentIns(suisettings);
	}

	/**
	 * <P>
	 * Test getEmployerSUISettings() without worklocation
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>get the worklocation for the employer</LI>
	 * <LI>delete the worklocation for the employer</LI>
	 * <LI>call the getEmployerSuisetting()</LI>
	 * <LI>revert back the worklocation by adding it again</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that the mandatory data are fetched</LI>
	 * 
	 * @throws ValidationException </UL>
	 * @throws PayrollValidationException
	 */
	@Test
	public void testGetEmployerSUIsettingsWithOutCAWorkLocationRemoved() throws Exception {
		removeSomeWorkLocations(sMountainViewLocation);
		EmployerUnemploymentIns[] suisettings = mEmployerTaxSettingService.getEmployerUnemploymentIns();
		validateSUIDataModel(suisettings);
	}


	/**
	 * Validate the SUI Data Model once a transaction has occurred.
	 */
	private void validateSUIDataModel(EmployerUnemploymentIns[] empSUIRates) throws ServiceFailureException {
		UnemploymentInsDocDtl[] unimIns = new UnemploymentInsDocDtl[empSUIRates.length];
		
		for (int i =0; i < empSUIRates.length; i++){
			unimIns[i] = SuiTaxMapper.mapToUnemploymentInsDocDtl(empSUIRates[i]);
		}
		
		// Ensure that we have Unemployment ins for all Jurisdictions
		HashMap<AgencyEnum, List<UnemploymentInsDocDtl>> listOfStateSUI = new HashMap<AgencyEnum, List<UnemploymentInsDocDtl>>();

		List<Jurisdiction> allJurisdictions = sEmployerComponent.determineAllJurisdictions();
		// Create a  Map for all available Agencies based on Jurisdictions
		for(Jurisdiction jurisdiction: allJurisdictions){
			Set<Agency> agencies = jurisdiction.getAgencies();
			for (Agency agency: agencies){
				if (!listOfStateSUI.containsKey(agency.getAgencyKey())){
					listOfStateSUI.put(agency.getAgencyKey(), new ArrayList<UnemploymentInsDocDtl>());
				}
			}
		}
		
		// Map UnEmployment ins to these agencies
		for (UnemploymentInsDocDtl unempInsDocDtl: unimIns){
			if (unempInsDocDtl.getLawId().getValue().equals(QboeLawIdConstants.sCASUILawId)){
				listOfStateSUI.get(AgencyEnum.CA_EDD).add(unempInsDocDtl);
			}else if(unempInsDocDtl.getLawId().getValue().equals(QboeLawIdConstants.sTXSUILawId)){
				listOfStateSUI.get(AgencyEnum.TX_TWC).add(unempInsDocDtl);
			}else{
				Assert.fail("UnEmployment Insurance found for no Agency ");
			}
				
		}
		
		Set<AgencyEnum> agencySet = listOfStateSUI.keySet();
		for (AgencyEnum agency: agencySet){
			if (!agency.equals(AgencyEnum.US_IRS)){
				Assert.assertTrue("No UnEmployment Ins rate found", listOfStateSUI.get(agency).size() > 0);
				validateUnemploymentInsDocDtl(listOfStateSUI.get(agency));
			}
		}
		
	}

	private void validateUnemploymentInsDocDtl(List<UnemploymentInsDocDtl> list) {
		// Sort on start of EffectiveDate
		SortedSet<UnemploymentInsDocDtl> sortedSet = getSortedUnemploymentInsDocDtl(list);
		
		Date startDate = null;
		Date endDate = null;
		for (UnemploymentInsDocDtl unEmpInsDtl:sortedSet){
			// General Validation
			Assert.assertNotNull(unEmpInsDtl.getMaxRate());
			Assert.assertNotNull(unEmpInsDtl.getMinRate());
			Assert.assertNotNull(unEmpInsDtl.getEffectivePeriod());
			Assert.assertNotNull(unEmpInsDtl.getLawId());
			Assert.assertTrue(unEmpInsDtl.getId() > 0L);
			
			
			// Validate continuous dates
			Assert.assertNotNull(unEmpInsDtl.getEffectivePeriod().getDateRange().getStartDate());
			if ((startDate == null) && (endDate == null)){
				// First run
				startDate = unEmpInsDtl.getEffectivePeriod().getDateRange().getStartDate();
				endDate = unEmpInsDtl.getEffectivePeriod().getDateRange().getEndDate();
			}else{
				// Subsequent Run
				Assert.assertEquals(endDate, unEmpInsDtl.getEffectivePeriod().getDateRange().getStartDate());
				startDate = unEmpInsDtl.getEffectivePeriod().getDateRange().getStartDate();
				endDate = unEmpInsDtl.getEffectivePeriod().getDateRange().getEndDate();
			}
		}
		Assert.assertNull("End Date not null", endDate);
	}
	
	
    /**
     * Return sorted UnemploymentInsDocDtl entities based on the Date the status was changed
     * @param List<UnemploymentInsDocDtl>
     * @return SortedSet of UnemploymentInsDocDtl
     */

    public static SortedSet<UnemploymentInsDocDtl> getSortedUnemploymentInsDocDtl(List<UnemploymentInsDocDtl> list){
        SortedSet<UnemploymentInsDocDtl> sortedUnemploymentInsDocDtl =  new TreeSet<UnemploymentInsDocDtl>(new UnInsComparator());
        for (UnemploymentInsDocDtl UnEmpInsDtl : list) {
        	sortedUnemploymentInsDocDtl.add(UnEmpInsDtl);
        }
        return sortedUnemploymentInsDocDtl;
    }

    private static class UnInsComparator implements Comparator<UnemploymentInsDocDtl>{
        public int compare(UnemploymentInsDocDtl o1, UnemploymentInsDocDtl o2) {
            return o2.getEffectivePeriod().getDateRange().getStartDate()
                .compareTo(o1.getEffectivePeriod().getDateRange().getStartDate());
        }
    }
}