package com.intuit.sb.payroll.qa.services.taxes;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.WorkerComplianceDocDtl;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.common.impl.ObjectHandleMapper;
import com.intuit.sb.payroll.services.taxes.FormType;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingInfoRequest;
import com.intuit.sb.payroll.services.taxes.SaveWithholdingResponse;
import com.intuit.sb.payroll.services.taxes.WithholdingInfoOperation;
import com.intuit.sb.payroll.services.taxes.WithholdingInformation;
import com.intuit.sb.payroll.services.taxes.WithholdingStatus;
import com.intuit.sb.payroll.services.taxes.impl.WithholdingInformationEIMapper;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerType;

public class WithholdingsTest extends EntityUtilsTestBase {

	private static Long sUniqueId = null;

	@BeforeClass
	public static void setUpWithholdingsTest() {
		try {
			getUserTransaction().begin();
			// Save a new Worker to be used by all tests
			sUniqueId = createAWorkerWithWorkLocation();
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in setUpWithholdingsTest(): " + t.getMessage());
		}
	}

	@AfterClass
	public static void tearDownWithholdingsTest() {
		try {
			getUserTransaction().begin();
			cleanUpEIWorker(sUniqueId);
			getUserTransaction().commit();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			try {
				getUserTransaction().rollback();
			} catch (Exception e) {
			}
			AssertJUnit.fail("Error in tearDownWithholdingsTest(): " + t.getMessage());
		}
	}

	/**
	 * <P>Test getting W4 withholding information only with withholding status for W4 of Single</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up W4 withholding information</LI>
	 * <LI>Call TaxWithholdingService's API  to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the withholding information returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testGettingW4WithholdingInfoOnly() throws Exception
	{
		try
		{
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
					W4WithholdingStatusEnum.SINGLE, 1, new BigDecimal("0.00"));
			BigDecimal zeroValue = new BigDecimal(0.00);
			zeroValue = zeroValue.setScale(2);
			WithholdingInformation[] expectedWithholdingInfo = new WithholdingInformation[1];
			expectedWithholdingInfo[0] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.W4IRS)),
					FormType.US_W4, WithholdingStatus.SINGLE, 1, 0,
					zeroValue, sNotExemptFromTax);

			WithholdingInformation[] computedWithholdingInfo = sTaxWithholdingsService.getEmployeeWithholdings(sWorkerId);
			AssertJUnit.assertEquals("Withholding info size mismatch!", expectedWithholdingInfo.length,
					computedWithholdingInfo.length);
			// Fix up the id value(s)
			expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
			AssertJUnit.assertTrue("Withholding Info mismatch!", verifyEIWithholdingInfo(expectedWithholdingInfo,
					computedWithholdingInfo));
		}
		finally
		{
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
		}
	}

	/**
	 * <P>Test getting worker by id who has W4 and DE4 withholding information,
	 *    withholding status for W4 of Single and for DE4 of Single</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up W4 withholding information</LI>
	 * <LI>Set up DE4 withholding information</LI>
	 * <LI>Call TaxWithholdingService's API  to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the withholding information returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testGettingW4AndDE4WithholdingInfo_Single() throws Exception
	{
		try
		{
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, W4WithholdingStatusEnum.SINGLE,
					1, new BigDecimal("50.00"));
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax, DE4WithholdingStatusEnum.SINGLE,
					1, 0, new BigDecimal("50.00"));
			BigDecimal fifty = new BigDecimal(50.00);
			fifty = fifty.setScale(2);
			WithholdingInformation[] expectedWithholdingInfo = new WithholdingInformation[2];
			expectedWithholdingInfo[0] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.W4IRS)),
					FormType.US_W4, WithholdingStatus.SINGLE, 1,0,
					fifty, sNotExemptFromTax);
			expectedWithholdingInfo[1] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.DE4)),
					FormType.CA_DE4, WithholdingStatus.SINGLE, 1,0,
					fifty, sNotExemptFromTax);

			WithholdingInformation[] computedWithholdingInfo = sTaxWithholdingsService.getEmployeeWithholdings(sWorkerId);
			AssertJUnit.assertEquals("Withholding info size mismatch!", expectedWithholdingInfo.length,
					computedWithholdingInfo.length);
			// Fix up the id value(s)
			if (expectedWithholdingInfo[0].getFormType().equals(computedWithholdingInfo[0].getFormType())) {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
			} else {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
			}
			AssertJUnit.assertTrue("Withholding Info mismatch!", verifyEIWithholdingInfo(expectedWithholdingInfo,
					computedWithholdingInfo));
		}
		finally
		{
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}
	}

	/**
	 * <P>Test getting worker by id who has W4 and DE4 withholding information,
	 *    withholding status for W4 of Married and for DE4 of MarriedWithOneInc</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up W4 withholding information</LI>
	 * <LI>Set up DE4 withholding information</LI>
	 * <LI>Call TaxWithholdingService's API  to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the withholding information returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testGettingW4AndDE4WithholdingInfo_MarriedOneIncome() throws Exception
	{
		try
		{
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, W4WithholdingStatusEnum.MARRIED,
					1, new BigDecimal("50.00"));
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax, DE4WithholdingStatusEnum.MARRIEDONEINCOME,
					1, 0, new BigDecimal("50.00"));
			BigDecimal fifty = new BigDecimal(50.00);
			fifty = fifty.setScale(2);
			WithholdingInformation[] expectedWithholdingInfo = new WithholdingInformation[2];
			expectedWithholdingInfo[0] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.W4IRS)),
					FormType.US_W4, WithholdingStatus.MARRIED, 1,0,
					fifty, sNotExemptFromTax);
			//FIXME  Hack: Since there isn't a corresponding MARRIEDONEINCOME value in WithholdingStatus, 
			//             MARRIEDONEINCOME is currently mapped to MARRIED_SINGLE_RATE
			expectedWithholdingInfo[1] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.DE4)),
					FormType.CA_DE4, WithholdingStatus.MARRIED_SINGLE_RATE, 1,0,
					fifty, sNotExemptFromTax);

			WithholdingInformation[] computedWithholdingInfo = sTaxWithholdingsService.getEmployeeWithholdings(sWorkerId);
			AssertJUnit.assertEquals("Withholding info size mismatch!", expectedWithholdingInfo.length,
					computedWithholdingInfo.length);
			// Fix up the id value(s)
			if (expectedWithholdingInfo[0].getFormType().equals(computedWithholdingInfo[0].getFormType())) {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
			} else {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
			}
			AssertJUnit.assertTrue("Withholding Info mismatch!", verifyEIWithholdingInfo(expectedWithholdingInfo,
					computedWithholdingInfo));
		}
		finally
		{
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}
	}

	/**
	 * <P>Test getting worker by id who has W4 and DE4 withholding information,
	 *    withholding status for W4 of MarriedAtSingle and for DE4 of HeadofHousehold</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up W4 withholding information</LI>
	 * <LI>Set up DE4 withholding information</LI>
	 * <LI>Call TaxWithholdingService's API  to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the withholding information returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testGettingW4AndDE4WithholdingInfo_HeadOfHousehold() throws Exception
	{
		try
		{
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, W4WithholdingStatusEnum.MARRIEDSINGLERATE,
					1, new BigDecimal("50.00"));
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax, DE4WithholdingStatusEnum.HEADOFHOUSEHOLD,
					1, 0, new BigDecimal("50.00"));
			BigDecimal fifty = new BigDecimal(50.00);
			fifty = fifty.setScale(2);
			WithholdingInformation[] expectedWithholdingInfo = new WithholdingInformation[2];
			expectedWithholdingInfo[0] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.W4IRS)),
					FormType.US_W4, WithholdingStatus.MARRIED_SINGLE_RATE, 1,0,
					fifty, sNotExemptFromTax);
			expectedWithholdingInfo[1] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.DE4)),
					FormType.CA_DE4, WithholdingStatus.MARRIED_HEAD_OF_HOUSEHOLD, 1,0,
					fifty, sNotExemptFromTax);

			WithholdingInformation[] computedWithholdingInfo = sTaxWithholdingsService.getEmployeeWithholdings(sWorkerId);
			AssertJUnit.assertEquals("Withholding info size mismatch!", expectedWithholdingInfo.length,
					computedWithholdingInfo.length);
			// Fix up the id value(s)
			if (expectedWithholdingInfo[0].getFormType().equals(computedWithholdingInfo[0].getFormType())) {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
			} else {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
			}
			AssertJUnit.assertTrue("Withholding Info mismatch!", verifyEIWithholdingInfo(expectedWithholdingInfo,
					computedWithholdingInfo));
		}
		finally
		{
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}
	}

	/**
	 * <P>Test getting worker by id who has W4 and DE4 withholding information,
	 *    withholding status for W4 of MarriedAtSingle and for DE4 of SingleOrMarriedWithMultipleIncomes</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up W4 withholding information</LI>
	 * <LI>Set up DE4 withholding information</LI>
	 * <LI>Call TaxWithholdingService's API  to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL> 
	 * <LI>Verify the withholding information returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testGettingW4AndDE4WithholdingInfo_MarriedTwoIncome() throws Exception
	{
		try
		{
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, W4WithholdingStatusEnum.MARRIEDSINGLERATE,
					1, new BigDecimal("50.00"));
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax, DE4WithholdingStatusEnum.SINGLE,
					1, 0, new BigDecimal("50.00"));
			BigDecimal fifty = new BigDecimal(50.00);
			fifty = fifty.setScale(2);
			WithholdingInformation[] expectedWithholdingInfo = new WithholdingInformation[2];
			expectedWithholdingInfo[0] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.W4IRS)),
					FormType.US_W4, WithholdingStatus.MARRIED_SINGLE_RATE, 1,0,
					fifty, sNotExemptFromTax);
			expectedWithholdingInfo[1] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.DE4)),
					FormType.CA_DE4, WithholdingStatus.SINGLE, 1,0,
					fifty, sNotExemptFromTax);

			WithholdingInformation[] computedWithholdingInfo = sTaxWithholdingsService.getEmployeeWithholdings(sWorkerId);
			AssertJUnit.assertEquals("Withholding info size mismatch!", expectedWithholdingInfo.length,
					computedWithholdingInfo.length);
			// Fix up the id value(s)
			if (expectedWithholdingInfo[0].getFormType().equals(computedWithholdingInfo[0].getFormType())) {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
			} else {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
			}
			AssertJUnit.assertTrue("Withholding Info mismatch!", verifyEIWithholdingInfo(expectedWithholdingInfo,
					computedWithholdingInfo));
		}
		finally
		{
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}
	}

	/**
	 * <P>Test getting worker by id who has W4 and DE4 withholding information,
	 *    withholding status for W4 of DontWithhold and for DE4 of DontWithhold</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up W4 withholding information</LI>
	 * <LI>Set up DE4 withholding information</LI>
	 * <LI>Call TaxWithholdingService's API  to perform the actual work</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the withholding information returned matches what's expected 
	 * </UL>
	 */
	@Test
	public void testGettingW4AndDE4WithholdingInfo_DontWithhold() throws Exception
	{
		try
		{
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sExemptFromTax, W4WithholdingStatusEnum.NOTSET,
					1, new BigDecimal("50.00"));
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sExemptFromTax, DE4WithholdingStatusEnum.NOTSET,
					1, 0, new BigDecimal("50.00"));
			BigDecimal fifty = new BigDecimal(50.00);
			fifty = fifty.setScale(2);
			WithholdingInformation[] expectedWithholdingInfo = new WithholdingInformation[2];
			expectedWithholdingInfo[0] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.W4IRS)),
					FormType.US_W4, WithholdingStatus.NOT_SET, 1,0,
					fifty, sExemptFromTax);
			expectedWithholdingInfo[1] = new WithholdingInformation(
					ObjectHandleMapper.mapToObjectHandle(getComplianceDocDtl(sWorkerId, ComplianceDocIdEnum.DE4)),
					FormType.CA_DE4, WithholdingStatus.NOT_SET, 1,0,
					fifty, sExemptFromTax);

			WithholdingInformation[] computedWithholdingInfo = sTaxWithholdingsService.getEmployeeWithholdings(sWorkerId);
			AssertJUnit.assertEquals("Withholding info size mismatch!", expectedWithholdingInfo.length,
					computedWithholdingInfo.length);
			// Fix up the id value(s)
			if (expectedWithholdingInfo[0].getFormType().equals(computedWithholdingInfo[0].getFormType())) {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
			} else {
				expectedWithholdingInfo[0].setObjectHandle(computedWithholdingInfo[1].getObjectHandle());
				expectedWithholdingInfo[1].setObjectHandle(computedWithholdingInfo[0].getObjectHandle());
			}
			AssertJUnit.assertTrue("Withholding Info mismatch!", verifyEIWithholdingInfo(expectedWithholdingInfo,
					computedWithholdingInfo));
		}

		finally
		{
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}
	}


	/**
	 * <P>
	 * Test to update a saved Worker with WithholdingInfo
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create new SaveWorkerRequest</LI>
	 * <LI>Call getWorkerById to retrieve saved Worker </LI>
	 * <LI>Set WithholdingInfo on SaveWorkerRequest</LI>
	 * <LI>Pass SaveWorkerRequest to WorkerService.SaveWorker()</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that Worker is updated in the db with WithholdingInfo.</LI>
	 * </UL>
	 */
	@Test
	public void testSaveWorkerUpdateWithNewWithholdingInfo() throws Exception {
		com.intuit.sb.payroll.entity.model.Worker prWorker = null;
		List<WorkerComplianceDocDtl> empWHDataToDelete = new ArrayList<WorkerComplianceDocDtl>();
		HashSet<WithholdingInformation> computedEIWHInfoSet = new HashSet<WithholdingInformation>();	
		HashSet<WithholdingInformation> expectedEIWHInfoSet = new HashSet<WithholdingInformation>();
		WithholdingInformation[] expectedWithholdings = null;
		boolean isCA = false;

		try {			
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker eiWorker = sWorkerService.getWorkerById(sUniqueId);
			expectedWithholdings = setUpExpectedWithholdingInfo(eiWorker);
			isCA = expectedWithholdings.length > 1;

			// Exercise
			SaveWithholdingInfoRequest request = new SaveWithholdingInfoRequest();
			
			request.setWithholdingInfo(expectedWithholdings[0]);
			request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
			request.setWorkerId(eiWorker.getObjectHandle().getId());
			SaveWithholdingResponse response = sTaxWithholdingsService.saveWithholdingInformation(request);

			// Fix up the id
			expectedWithholdings[0].setObjectHandle(response.getWithholding().getObjectHandle());
			
			if (isCA) {
				SaveWithholdingInfoRequest de4Request = new SaveWithholdingInfoRequest();
				
				de4Request.setWithholdingInfo(expectedWithholdings[1]);
				de4Request.setWithholdingInfoOperation(WithholdingInfoOperation.ADD);
				de4Request.setWorkerId(eiWorker.getObjectHandle().getId());
				SaveWithholdingResponse de4Response = sTaxWithholdingsService.saveWithholdingInformation(de4Request);
				
				expectedWithholdings[1].setObjectHandle(de4Response.getWithholding().getObjectHandle());
			}

			//Build expected HashSet
			for(WithholdingInformation info : expectedWithholdings) {
				expectedEIWHInfoSet.add(info);
			}

			prWorker = getWorkerWithholdingInfoData(sUniqueId, empWHDataToDelete, computedEIWHInfoSet);
			// Verify			
			//Check if size matches
			AssertJUnit.assertEquals("Expected number of Withholdings are present", expectedEIWHInfoSet.size(), computedEIWHInfoSet.size());

			// Compare list of expected and computed WithholdingInformation
			AssertJUnit.assertTrue(expectedEIWHInfoSet.equals(computedEIWHInfoSet));	
		} finally {
			// Teardown
			// - Additional teardown - Clean up WithholdingInformation
			// - Implicit teardown in tearDown()
			if(prWorker != null) {
				for (WorkerComplianceDocDtl data : empWHDataToDelete) {
					prWorker.removeWorkerComplianceDocDtl(data);
				}
			}
		}
	}

	/**
	 * <P>
	 * Test to update a saved Worker with a WithholdingInformation with a bogus Unique Id
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create new SaveWorkerRequest</LI>
	 * <LI>Call getWorkerById to retrieve saved Worker </LI>
	 * <LI>Set bogus Unique Id on Withholding</LI>
	 * <LI>Set updated Worker on SaveWorkerRequest</LI>
	 * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException was thrown.</LI>
	 * </UL>
	 */
	@Test (expectedExceptions=ServiceFailureException.class)
	public void testSaveWorkerUpdateWithNewWithholdingInfoUnknownId() throws Exception {
		// Setup
		// - Implicit setup in setUp()
		// - Additional setup
		Worker eiWorker = sWorkerService.getWorkerById(sUniqueId);
		WithholdingInformation expectedWithholding = createEIWithholdingInformation(eiWorker);
		expectedWithholding.setObjectHandle(new ObjectHandle(new Long("87343723"), 0l));
		SaveWithholdingInfoRequest request = new SaveWithholdingInfoRequest();
		request.setWithholdingInfo(expectedWithholding);
		request.setWorkerId(sUniqueId);
		request.setWithholdingInfoOperation(WithholdingInfoOperation.UPDATE);
		// Exercise
		sTaxWithholdingsService.saveWithholdingInformation(request);

		// Verify
		// - Implicit JUnit exception handling

		// Teardown
		// - Implicit teardown in tearDown()
	}

	/**
	 * Utility method to retrieve a Worker's WithholdingData & EIWithholdingInfo, and return the payroll component worker.  
	 */
	private com.intuit.sb.payroll.entity.model.Worker getWorkerWithholdingInfoData(Long uniqueId,
			List<WorkerComplianceDocDtl> empWHDataToDelete, HashSet<WithholdingInformation> computedEIWHInfoSet) throws Exception {
		com.intuit.sb.payroll.entity.model.Worker prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, uniqueId);

		Set<WorkerComplianceDocDtl> compData = prWorker.getWorkerComplianceDocDtls();
		Iterator<WorkerComplianceDocDtl> compDataIter = compData.iterator();

		while(compDataIter.hasNext()) {
			WorkerComplianceDocDtl data = (WorkerComplianceDocDtl)compDataIter.next();
			if (data.getDocId() == ComplianceDocIdEnum.W4IRS ||
					data.getDocId() == ComplianceDocIdEnum.DE4) {
				WithholdingInformation apiWHInfo = WithholdingInformationEIMapper.mapFromEntityToPojo(prWorker, data);

				// Build computed HashSet
				computedEIWHInfoSet.add(apiWHInfo);                
				empWHDataToDelete.add(data);
			}
		}
		return prWorker;
	}

	private boolean verifyEIWithholdingInfo(WithholdingInformation[] expected, WithholdingInformation[] computed)
	{
		boolean compareResult = expected.length == computed.length;
		Set<WithholdingInformation> computedSet = new HashSet<WithholdingInformation>();
		Set<WithholdingInformation> expectedSet = new HashSet<WithholdingInformation>();
		if (compareResult)
		{
			// Only compare the WithholdingInformation items if the number of items matches the expected number
			for(int i = 0; i < computed.length; i++)
			{
				computedSet.add(computed[i]);
			} 
			for(int i = 0; i < expected.length; i++)
			{
				expectedSet.add(expected[i]);
			}
			compareResult = computedSet.equals(expectedSet);
		}
		if (false == compareResult)
		{
			sLogger.info("Computed WithholdingInformation:");     
			dumpWithholdings(computedSet, "computed");
			sLogger.info("Expected WithholdingInformation:");            
			dumpWithholdings(expectedSet, "expected");
		}
		return compareResult;
	}

	@SuppressWarnings("unused")
	private void dumpWithholdings(WithholdingInformation[] withholdings, String name) {
		dumpWithholdings(Arrays.asList(withholdings), name);
	}
	
	private void dumpWithholdings(Collection<WithholdingInformation> withholdings, String name) {
		if (name == null) name = "withholding";
		for (WithholdingInformation withholding : withholdings) {
			sLogger.info(name + withholding.toString());
		}            
	}
	/**
	 * Utility method to save a Worker with WorkLocation in the db and return the uniqueId.  
	 * A fully populated Worker is needed to avoid Workflow exceptions in the tests.
	 */

	public static Long createAWorkerWithWorkLocation() throws Exception {
		Worker savedWorker = null;

		Worker worker = createDtoWorker();
		worker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
		worker.setWorkLocation(getAWorkLocation());
		worker.setWorkerType(WorkerType.EMPLOYEE);

	    savedWorker = sWorkerService.saveWorker(worker);

		return savedWorker.getObjectHandle().getId();
	}

	/*
	 * Utility method to find and return a work location
	 */
	public static WorkLocation getAWorkLocation() throws Exception {
		WorkLocation[] workLocations = sEmployerService.getWorkLocations();
		return workLocations[0];
	}

}
