package com.intuit.sb.payroll.qa.services.worker;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.NonEmployee;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;

/**
 * Tests the getWorkerById operation of the WorkerService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetWorkerByIdTest extends WorkerServiceTestBase {
    
    /**
     * <P>Test Exception caused by null Id</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerService's getWorkerById command with a null Id</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since Id is null
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullId() throws Exception {
	    sWorkerService.getWorkerById(null);
    }

    /**
     * <P>Test getting worker by id who does not have wage items or withholding information</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up expected Entity Interface Worker values</LI>
     * <LI>Call WorkerService's getWorkerById command to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify the worker information matches expected values 
     * <LI>Verify the withholding information returned is null 
     * </UL>
     */
    @Test
    public void testNoWithholdingInformation() throws Exception {
        
        Worker worker = initEIWorkerValues(sWorkerId);
		Worker responseWorker = sWorkerService.getWorkerById(sWorkerId);
		AssertJUnit.assertTrue("Worker mismatch", verifyEIWorker(worker, responseWorker));
    }

    /**
     * <P>
     * Test getting worker by id with nonemployee who does not have wage items
     * or withholding information
     * </P>
     * 
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up expected Entity Interface Worker values</LI>
     * <LI>Call WorkerService's getWorkerById command to perform the actual
     * work</LI>
     * </OL>
     * 
     * <UL>
     * <LI>Verify the worker information matches expected values
     * <LI>Verify the withholding information returned
     * is null
     * </UL>
     */
    @Test
    public void testNonEmployeeNoWithholdingInformation() throws Exception {
        List<NonEmployee>  nonEmployeeList = entityService.findAll(NonEmployee.class);
        // use the first one
        NonEmployee nonEmployee = nonEmployeeList.get(0);
            AssertJUnit.assertNotNull("Can't find a non employee!", nonEmployee);
		Long workerId = nonEmployee.getId();
		Worker worker = initEIWorkerValues(workerId);
		Worker responseWorker = sWorkerService.getWorkerById(workerId);
		AssertJUnit.assertTrue("Worker mismatch", verifyEIWorker(worker, responseWorker));
	}

    /**
     * <P>Test getting worker by id who has Land PhoneNumber contact detail</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Add a land TelephoneNumber contact detail</LI>
     * <LI>Set up expected Entity Interface Worker values</LI>
     * <LI>Call WorkerService's getWorkerById command to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify the worker information matches expected values
     * </UL>
     */
    @Test
    public void testNoWithholdingInformation_WithLandPhone() throws Exception {
        com.intuit.sb.payroll.entity.model.Worker workerFromDB = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, sWorkerId);
        String phoneString = "650-944-3002";
        TelephoneNumber homePhone = new TelephoneNumber();
        homePhone.setFreeFormNumber(phoneString);
        workerFromDB.setHomePhone(homePhone);
        Worker worker = sWorkerService.getWorkerById(sWorkerId);
        AssertJUnit.assertEquals(phoneString, worker.getPhoneNumber());
    }

    /**
     * <P>Test getting worker by id who has Email contact detail</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Add a Email contact detail</LI>
     * <LI>Set up expected Entity Interface Worker values</LI>
     * <LI>Call WorkerService's getWorkerById command to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify the worker information matches expected values
     * </UL>
     */
    @Test
    public void testNoWithholdingInformation_WithEmail() throws Exception {
        com.intuit.sb.payroll.entity.model.Worker workerFromDB = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, sWorkerId);
        EmailAddress emailAddress = new EmailAddress();
        String emailString = "worker@SomeIsp.Net";
        emailAddress.setAddress(emailString);
        workerFromDB.setEmailAddress(emailAddress);
        Worker worker = sWorkerService.getWorkerById(sWorkerId);
        AssertJUnit.assertEquals(emailString, worker.getEmailAddress());
    }

    /**
     * <P>Test getting worker by id who only have first and last name information</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a new worker with first and last name information only</LI>
     * <LI>Set up expected Entity Interface Worker values</LI>
     * <LI>Call WorkerService's getWorkerById command to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify the worker information matches expected values
     * </UL>
     */
    @Test
    public void testFirstAndLastNameInformationOnly() throws Exception {
        Long workerId = null;
        try {
            Worker newWorker = new Worker();
            String firstName = "Jane";
            String lastName = "Doe";
	    newWorker.setFirstName(firstName);
	    newWorker.setLastName(lastName);
            Worker savedWorker = sWorkerService.saveWorker(newWorker);
            workerId = new Long(savedWorker.getObjectHandle().getId());
            Worker worker = sWorkerService.getWorkerById(workerId);
            AssertJUnit.assertEquals(firstName, worker.getFirstName());
            AssertJUnit.assertEquals(lastName, worker.getLastName());
        } finally {
        	cleanUpEIWorker(workerId);
        }
    }
    
    @Test
    public void testGetPayrollReadyWorkerById() throws Exception {
    	Long readyToPayWorkerId = null;
    	try {
        	readyToPayWorkerId = createPayrollReadyWorker();
        	Worker worker = sWorkerService.getWorkerById(readyToPayWorkerId);
        	AssertJUnit.assertTrue(worker.isReadyToPay());
    	}  finally {
    		cleanUpEIWorker(readyToPayWorkerId);
    	}
    }
}
