package com.intuit.sb.payroll.qa.services.worker;

import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.GlobalPostalAddress;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.intl.capability.address.AddressConvertor;
import com.intuit.sb.payroll.intl.capability.address.USPostalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;
import com.intuit.sb.payroll.us.capability.address.impl.USPostalAddressImpl;

/**
 * <P>The <CODE>GetWorkersByWorkLocationTest</CODE> class contains test methods for the GetWorkersByWorkLocation() 
 * Entity Interface API. 
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 */
public class GetWorkersByWorkLocationTest extends WorkerServiceTestBase 
{
    private static String newWorkLocationName = "Temp Location";
    
	/**
     * <P>Test Exception caused by null WorkLocation uniqueId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call getWorkersByWorkLocation() with null uniqueId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since WorkLocation uniqueId is null</LI>
     * </UL>
     */
	@Test(expectedExceptions=ValidationException.class)
	public void testNullWorkLocationUniqueId() throws Exception {
		sWorkerService.getWorkersByWorkLocation(null);
	}
	
	/**
     * <P>Test getWorkersByWorkLocation() with valid WorkLocation uniqueId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Get list of all workLocations</LI>
     * <LI>For each workLocation, call getWorkersByWorkLocation() with valid WorkLocation uniqueId</LI>
     * <LI>For each workLocation, compare the list of Workers returned with the expected values</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from getWorkersByWorkLocation() matches the expected</LI>
     * </UL>
     */
    @Test
    public void testGettingWorkersWithValidWorkLocationUniqueId() throws Exception {		
		// Setup
		// Get list of workLocations
		WorkLocation[] workLocations = sEmployerService.getWorkLocations();
        for (WorkLocation workLocation: workLocations) {
            Long workLocationId = workLocation.getObjectHandle().getId();
            com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
        	new com.intuit.sb.payroll.services.worker.Worker[1];
            Long workerId = findWorkerForWorkLocation(workLocationId);
            expectedWorkers[0] = initEIWorkerValues(workerId);
            // Exercise
            com.intuit.sb.payroll.services.worker.Worker[] workers = 
        	sWorkerService.getWorkersByWorkLocation(workLocationId);
			
            // Verify
            AssertJUnit.assertTrue("Worker mismatch!", verifyEIWorkers(expectedWorkers, workers));
        }
    }

    /**
     * <P>Test getWorkersByWorkLocation() with valid WorkLocation uniqueId containing no workers</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a new workLocation</LI>
     * <LI>Call getWorkersByWorkLocation() with valid WorkLocation uniqueId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from getWorkersByWorkLocation() is empty</LI>
     * </UL>
     */
    @Test
    public void testWorkLocationWithNoWorkers() throws ServiceFailureException, ValidationException {       
        // Setup
        com.intuit.sb.payroll.entity.model.WorkLocation workLocation = null;
        try {
            workLocation = createWorkLocationInDatabase();
            Long workLocationId = workLocation.getId();
            // Exercise
            com.intuit.sb.payroll.services.worker.Worker[] workers = 
        	sWorkerService.getWorkersByWorkLocation(workLocationId);
            
            // Verify
            AssertJUnit.assertEquals("Workers array is not empty!", 0, workers.length);
        } finally {
            if (null != workLocation) {
                entityService.makeTransient(workLocation);
            }
        }
    }
    
    private Long findWorkerForWorkLocation(Long workLocationId) {
        Long workerId = null;
        List<Worker>  workers = entityService.findAll(Worker.class);
        for (Worker worker : workers) {
            if (workLocationId.equals(worker.getWorkLocation().getId())) {
                workerId = worker.getId();
                break;
            }
        }
        return workerId;
    }
    
    private com.intuit.sb.payroll.entity.model.WorkLocation createWorkLocationInDatabase() {
        com.intuit.sb.payroll.entity.model.WorkLocation workLocation = new com.intuit.sb.payroll.entity.model.WorkLocation();
        workLocation.setName(newWorkLocationName);
        TelephoneNumber mainPhone = new TelephoneNumber();
        mainPhone.setFreeFormNumber("650-944-9999");
        mainPhone.setExtension("1234");
        workLocation.setMainPhone(mainPhone);
        USPostalAddress anAddress = new USPostalAddressImpl();
        anAddress.setAddressLine1("123 A St.");
        anAddress.setCity("San Carlos");
        anAddress.setRegionOrState("CA");
        anAddress.setZipOrPostalCode("94402");
        //Get the Global Address from USProxy and Set It
        GlobalPostalAddress addressEntity = AddressConvertor.getGlobalFromUS(
        		anAddress, new GlobalPostalAddress());
        workLocation.setMailingAddress(addressEntity);
        workLocation = entityService.makePersistent(workLocation);
        return workLocation;
    }
    
}
