package com.intuit.sb.payroll.qa.services.worker;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.cdm.basetypes.EmailAddress;
import com.intuit.sb.cdm.basetypes.TelephoneNumber;
import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.WorkflowException;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemRequest;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemResponse;
import com.intuit.sb.payroll.services.worker.GetWorkersFilter;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;

/**
 * <P>The <CODE>GetWorkersTest</CODE> class contains test methods for the GetWorkers() 
 * Entity Interface API. 
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetWorkersTest extends WorkerServiceTestBase {
    private static List<Long> sIDsOfEmployeesReadyForPay= null;
    private static int sNumberOfWorkers = 2;
    
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        if (sIDsOfEmployeesReadyForPay == null) {
            // Setup Ready for Pay Employees
            sIDsOfEmployeesReadyForPay = setEmployeesReadyForPay();
        }
    }
    
    @AfterMethod
    public void tearDown() throws Exception {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
            if (null != sIDsOfEmployeesReadyForPay) {
                for (int i = 0; i < sIDsOfEmployeesReadyForPay.size(); i++) {
                    //Clean up all the worker attributes
                    cleanUpEIWorker(sIDsOfEmployeesReadyForPay.get(i));
                }
                sIDsOfEmployeesReadyForPay = null;
            }
        }

        super.tearDown();
    }

    /**
     * <P>Test Exception caused by null GetWorkersFilter</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new WorkerService</LI>
     * <LI>Pass null GetWorkersFilter to WorkerService.getWorkers()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since workerId is null</LI>
     * </UL>
     */
    @Test
    public void testNullGetWorkersFilter() {
        try {
            sWorkerService.getWorkers(null);
            AssertJUnit.fail("ValidationException was not thrown");
        } catch (ValidationException e) {
            AssertJUnit.assertTrue(e.getMessage().contains("GetWorkersFilter is null"));
        } catch (Exception e) {
            AssertJUnit.fail("Expected ValidationExeption was not thrown.");
        }
    }

    /**
     * <P>Test getWorkers() with GetWorkersFilter.ALL</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Pass GetWorkersFilter.ALL to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     */
    @Test
    public void testAllGetWorkersFilter() throws Exception {
        List<Worker>  workerEntityList = entityService.findAll(Worker.class);
        com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
            new com.intuit.sb.payroll.services.worker.Worker[workerEntityList.size()];
        for (int i = 0; i < workerEntityList.size(); i++) {
            expectedWorkers[i] = WorkerMapper.mapFromEntityToPojo(workerEntityList.get(i));
        }
        com.intuit.sb.payroll.services.worker.Worker[] workerList = sWorkerService.getWorkers(GetWorkersFilter.ALL);
        AssertJUnit.assertTrue(verifyEIWorkers(expectedWorkers, workerList));
    }

    /**
     * <P>Test getWorkers() with GetWorkersFilter.Ready_for_Payroll</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Set up Ready for Pay Workers</LI>
     * <LI>Pass GetWorkersFilter.Ready_for_Payroll to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     */
    @Test
    public void testReadyForPayGetWorkersFilter() throws Exception {
            // Build list of Workers Ready for Pay
            com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
            new com.intuit.sb.payroll.services.worker.Worker[sIDsOfEmployeesReadyForPay.size()];
            for (int i = 0; i < sIDsOfEmployeesReadyForPay.size(); i++) {
                expectedWorkers[i] = initEIWorkerValues(sIDsOfEmployeesReadyForPay.get(i));
            }
            
            com.intuit.sb.payroll.services.worker.Worker[] workerList = sWorkerService.getWorkers(GetWorkersFilter.READY_FOR_PAYROLL);
            AssertJUnit.assertTrue("Worker mismatch", verifyEIWorkers(expectedWorkers, workerList));
            
            // Verify isReadyForPay flag
            for(com.intuit.sb.payroll.services.worker.Worker w : workerList) {
                AssertJUnit.assertTrue(w.isReadyToPay());
            }
    }
    
    /**
     * <P>Test getWorkers() with GetWorkersFilter.Ready_for_Payroll</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Set up Ready for Pay Workers</LI>
     * <LI>Pass GetWorkersFilter.Ready_for_Payroll to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     */
    @SuppressWarnings("deprecation")
    @Test(enabled=false, groups={"broken"})
    public void testReadyForPayGetWorkersFilter_WithPhoneAndEmail() throws Exception {       
            // Build list of Workers Ready for Pay
            com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
            new com.intuit.sb.payroll.services.worker.Worker[sIDsOfEmployeesReadyForPay.size()];
            for (int i = 0; i < sIDsOfEmployeesReadyForPay.size(); i++) {
                Long workerId = sIDsOfEmployeesReadyForPay.get(i);
                Worker workerFromDB = entityService.findById(Worker.class, workerId);
                TelephoneNumber homePhone = new TelephoneNumber();
                homePhone.setFreeFormNumber("650-944-300"+i);
                workerFromDB.setHomePhone(homePhone);
                EmailAddress emailAddress = new EmailAddress();
                emailAddress.setAddress("worker@SomeIsp.Net");
                workerFromDB.setEmailAddress(emailAddress);
                expectedWorkers[i] = initEIWorkerValues(workerId);
                // Hack - The version value is still incorrect after calling initEIWorkerValues(), mostly
                //        likely due to the fact the the changed data wasn't committed.  So, have to fix the version
                //        value with the following line of code
                ObjectHandle objectHandle = expectedWorkers[i].getObjectHandle();
                //FIXME This should return the correct objectHandle version.
                if (i==0) {    // dumb hack - not sure why it only works for one of the workers
                    objectHandle.setVersion(objectHandle.getVersion()+1);
                }
            }
            
            com.intuit.sb.payroll.services.worker.Worker[] workerList = sWorkerService.getWorkers(GetWorkersFilter.READY_FOR_PAYROLL);
            AssertJUnit.assertTrue(verifyEIWorkers(expectedWorkers, workerList));
            
            // Verify isReadyForPay flag
            for(com.intuit.sb.payroll.services.worker.Worker w : workerList) {
                AssertJUnit.assertTrue(w.isReadyToPay());
            }
    }
    
    /**
     * <P>Test getWorkers() with GetWorkersFilter.Ready_for_Payroll with no Hire Date on a Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Set up Ready for Pay Workers</LI>
     * <LI>Unset hire date on one worker (i.e. remove Ready for Payroll state)</LI>
     * <LI>Pass GetWorkersFilter.Ready_for_Payroll to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     * @throws Exception
     */
    @Test(enabled=false, groups={"broken"})
    public void testReadyForPayGetWorkersFilterNoHireDate() throws Exception {
        Long workerId = null;
        Date hireDate = null;
        
        try  {
            // Build list of Workers Ready for Pay
            List<Worker> workersReadyForPay = new ArrayList<Worker>();
            Iterator<Long> workerIdsIter = sIDsOfEmployeesReadyForPay.iterator();
            while(workerIdsIter.hasNext()) {
                Long id = workerIdsIter.next();
                workersReadyForPay.add(entityService.findById(Worker.class, id));
            }
            
            //"Unset" HireDate on one worker
            Worker worker = workersReadyForPay.get(0);
            workerId = worker.getId();
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            hireDate = eiWorker.getHireDate();
            eiWorker.setHireDate(null);
            sWorkerService.saveWorker(eiWorker);
            
            // Update list of workers ready for pay
            workersReadyForPay.remove(worker);
            com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
            new com.intuit.sb.payroll.services.worker.Worker[workersReadyForPay.size()];
            for (int i = 0; i < workersReadyForPay.size(); i++) {
                expectedWorkers[i] = initEIWorkerValues(workersReadyForPay.get(i).getId());
            }
            
            com.intuit.sb.payroll.services.worker.Worker[] workerList = 
            sWorkerService.getWorkers(GetWorkersFilter.READY_FOR_PAYROLL);
            
            AssertJUnit.assertTrue(verifyEIWorkers(expectedWorkers, workerList));
            
            // Verify isReadyForPay flag
            for(com.intuit.sb.payroll.services.worker.Worker wName : workerList) {
                AssertJUnit.assertTrue(wName.isReadyToPay());
            }
        } 
        catch (Exception e) {
            //Nothing should have gone wrong.
            AssertJUnit.fail("Failed to get Workers with no hire date: " + e.getMessage());
        } finally {
            // Set hire date back on Worker
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            eiWorker.setHireDate(hireDate);
            sWorkerService.saveWorker(eiWorker);        
        }
    }

    /**
     * <P>Test getWorkers() with GetWorkersFilter.Ready_for_Payroll with no Last Name on a Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Set up Ready for Pay Workers</LI>
     * <LI>Unset last name on one worker (i.e. remove Ready for Payroll state)</LI>
     * <LI>Pass GetWorkersFilter.Ready_for_Payroll to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     * @throws Exception 
     */
    @Test(enabled=false, groups={"broken"})
    public void testReadyForPayGetWorkersFilterNoLastName() throws Exception {
        Long workerId = null;
        String lastName = null;
        
        try {
            // Build list of Workers Ready for Pay
            List<Worker> workersReadyForPay = new ArrayList<Worker>();
            Iterator<Long> workerIdsIter = sIDsOfEmployeesReadyForPay.iterator();
            while(workerIdsIter.hasNext()) {
                Long id = workerIdsIter.next();
                workersReadyForPay.add(entityService.findById(Worker.class, id));
            }
            
            //"Unset" LastName on one worker
            Worker worker = workersReadyForPay.get(0);
            workerId = worker.getId();
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            lastName = eiWorker.getLastName();
            eiWorker.setLastName(null);
            sWorkerService.saveWorker(eiWorker);
            
            // Update list of workers ready for pay
            workersReadyForPay.remove(worker);
            com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
            new com.intuit.sb.payroll.services.worker.Worker[workersReadyForPay.size()];
            for (int i = 0; i < workersReadyForPay.size(); i++) {
                expectedWorkers[i] = initEIWorkerValues(workersReadyForPay.get(i).getId());
            }
            
            com.intuit.sb.payroll.services.worker.Worker[] workerList = sWorkerService.getWorkers(GetWorkersFilter.READY_FOR_PAYROLL);
            
            AssertJUnit.assertTrue(verifyEIWorkers(expectedWorkers, workerList));
            
            // Verify isReadyForPay flag
            for(com.intuit.sb.payroll.services.worker.Worker w : workerList) {
                AssertJUnit.assertTrue(w.isReadyToPay());
            }
        } catch (Exception e) {
            //Nothing should have gone wrong.
            AssertJUnit.fail("Failed to get all workers: " + e.getMessage());
        } finally {
            // Set last name back on Worker
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            eiWorker.setLastName(lastName);
            sWorkerService.saveWorker(eiWorker);
        }
    }

    /**
     * <P>Test getWorkers() with GetWorkersFilter.Ready_for_Payroll with no Home Address on a Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Set up Ready for Pay Workers</LI>
     * <LI>Unset last name on one worker (i.e. remove Ready for Payroll state)</LI>
     * <LI>Pass GetWorkersFilter.Ready_for_Payroll to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     * @throws Exception 
     */
    @Test(enabled=false, groups={"broken"})
    public void testReadyForPayGetWorkersFilterNoHomeAddress() throws Exception {
        Long workerId = null;
        PhysicalAddress taxResidenceAddress = null;
        
        try {
            // Build list of Workers Ready for Pay
            List<Worker> workersReadyForPay = new ArrayList<Worker>();
            Iterator<Long> workerIdsIter = sIDsOfEmployeesReadyForPay.iterator();
            while(workerIdsIter.hasNext()) {
                Long id = workerIdsIter.next();
                workersReadyForPay.add(entityService.findById(Worker.class, id));
            }

            //"Unset" Home Address on one worker
            Worker worker = workersReadyForPay.get(0);
            workerId = worker.getId();
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            taxResidenceAddress = eiWorker.getTaxResidenceAddress();
            eiWorker.setTaxResidenceAddress(null);
            sWorkerService.saveWorker(eiWorker);

            // Update list of workers ready for pay
            workersReadyForPay.remove(worker);
            com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
            new com.intuit.sb.payroll.services.worker.Worker[workersReadyForPay.size()];
            for (int i = 0; i < workersReadyForPay.size(); i++) {
                expectedWorkers[i] = initEIWorkerValues(workersReadyForPay.get(i).getId());
            }

            com.intuit.sb.payroll.services.worker.Worker[] workerList = sWorkerService.getWorkers(GetWorkersFilter.READY_FOR_PAYROLL);

            AssertJUnit.assertTrue(verifyEIWorkers(expectedWorkers, workerList));
            
            // Verify isReadyForPay flag
            for(com.intuit.sb.payroll.services.worker.Worker w : workerList) {
                AssertJUnit.assertTrue(w.isReadyToPay());
            }
        } catch (Exception e) {
            //Nothing should have gone wrong.
            AssertJUnit.fail("Failed to get all workers: " + e.getMessage());
        } finally {
            // Set home address back on Worker
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            eiWorker.setTaxResidenceAddress(taxResidenceAddress);
            sWorkerService.saveWorker(eiWorker);
        }
    }
    
    /**
     * <P>Test getWorkers() with GetWorkersFilter.Ready_for_Payroll with no Work Location on a Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Set up Ready for Pay Workers</LI>
     * <LI>Unset work location on one worker (i.e. remove Ready for Payroll state)</LI>
     * <LI>Pass GetWorkersFilter.Ready_for_Payroll to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     * @throws Exception 
     */
    @Test(enabled=false, groups={"broken"})
    public void testReadyForPayGetWorkersFilterNoWorkLocation() throws Exception {
        Long workerId = null;
        com.intuit.sb.payroll.services.common.WorkLocation eiWorkLocation = null;

        try {
            // Build list of Workers Ready for Pay
            List<Worker> workersReadyForPay = new ArrayList<Worker>();
            Iterator<Long> workerIdsIter = sIDsOfEmployeesReadyForPay.iterator();
            while(workerIdsIter.hasNext()) {
                Long id = workerIdsIter.next();
                workersReadyForPay.add(entityService.findById(Worker.class, id));
            }

            //"Unset" Work Location on one worker
            Worker worker = workersReadyForPay.get(0);
            workerId = worker.getId();
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            eiWorkLocation = eiWorker.getWorkLocation();
            eiWorker.setWorkLocation(null);
            sWorkerService.saveWorker(eiWorker);

            // Update list of workers ready for pay
            workersReadyForPay.remove(worker);
            com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
                new com.intuit.sb.payroll.services.worker.Worker[workersReadyForPay.size()];
            for (int i = 0; i < workersReadyForPay.size(); i++) {
                expectedWorkers[i] = initEIWorkerValues(workersReadyForPay.get(i).getId());
            }

            com.intuit.sb.payroll.services.worker.Worker[] workerList = sWorkerService.getWorkers(GetWorkersFilter.READY_FOR_PAYROLL);

            AssertJUnit.assertTrue(verifyEIWorkers(expectedWorkers, workerList));
            
            // Verify isReadyForPay flag
            for(com.intuit.sb.payroll.services.worker.Worker w : workerList) {
                AssertJUnit.assertTrue(w.isReadyToPay());
            }
        } catch (Exception e) {
            //Nothing should have gone wrong.
            AssertJUnit.fail("Failed to get all workers: " + e.getMessage());
        } finally {
            // Set work location back on Worker
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            eiWorker.setWorkLocation(eiWorkLocation);
            sWorkerService.saveWorker(eiWorker);
        }
    }

    /**
     * <P>Test getWorkers() with GetWorkersFilter.Ready_for_Payroll with no Work Location on a Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Set up Ready for Pay Workers</LI>
     * <LI>Unset Withholding data on one worker (i.e. remove Ready for Payroll state)</LI>
     * <LI>Pass GetWorkersFilter.Ready_for_Payroll to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     * @throws Exception 
     */
    @Test(enabled=false, groups={"broken"})
    public void testReadyForPayGetWorkersFilterNoWithholdingData() throws Exception {
        Long workerId = null;

        try {
            // Build list of Workers Ready for Pay
            List<Worker> workersReadyForPay = new ArrayList<Worker>();
            Iterator<Long> workerIdsIter = sIDsOfEmployeesReadyForPay.iterator();
            while(workerIdsIter.hasNext()) {
                Long id = workerIdsIter.next();
                workersReadyForPay.add(entityService.findById(Worker.class, id));
            }

            //"Unset" Withholding Data on one worker
            Worker worker = workersReadyForPay.get(0);
            workerId = worker.getId();
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            sWorkerService.saveWorker(eiWorker);

            // Update list of workers ready for pay
            workersReadyForPay.remove(worker);
            com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
                new com.intuit.sb.payroll.services.worker.Worker[workersReadyForPay.size()];
            for (int i = 0; i < workersReadyForPay.size(); i++) {
                expectedWorkers[i] = initEIWorkerValues(workersReadyForPay.get(i).getId());
            }

            com.intuit.sb.payroll.services.worker.Worker[] workerList = sWorkerService.getWorkers(GetWorkersFilter.READY_FOR_PAYROLL);

            AssertJUnit.assertTrue(verifyEIWorkers(expectedWorkers, workerList));
            
            // Verify isReadyForPay flag
            for(com.intuit.sb.payroll.services.worker.Worker w : workerList) {
                AssertJUnit.assertTrue(w.isReadyToPay());
            }
        } catch (Exception e) {
            //Nothing should have gone wrong.
            AssertJUnit.fail("Failed to get all workers: " + e.getMessage());
        } finally {
            // Set withholding data back on Worker
            com.intuit.sb.payroll.services.worker.Worker eiWorker = sWorkerService.getWorkerById(workerId);
            sWorkerService.saveWorker(eiWorker);
        }
    }

    /**
     * <P>Test getWorkers() with GetWorkersFilter.Ready_for_Payroll by saving a Worker through the Entity Interface</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Save a new Worker through the Entity Interface</LI>
     * <LI>Set up Ready for Pay Workers</LI>
     * <LI>Pass GetWorkersFilter.Ready_for_Payroll to WorkerService.getWorkers()</LI>
     * <LI>Check the isReadyForPay flag on the returned Worker.</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() contains the new Worker and the isReadyForPay flag is set</LI>
     * </UL>
     */
    @Test
    public void testSaveNewWorkerReadyForPay() throws Exception {
    com.intuit.sb.payroll.services.worker.Worker[] computedWorkerList = null;
        com.intuit.sb.payroll.services.worker.Worker savedWorker = null;
        com.intuit.sb.payroll.services.worker.Worker eiWorker;
        Long uniqueId = null;
        PhysicalAddress taxResidenceAddress = PojoUtils.createUSCAPhysicalAddress();
        
        try {            
            eiWorker = createDtoWorker();
            eiWorker.setTaxResidenceAddress(taxResidenceAddress);

            // Set work location
            eiWorker.setWorkLocation(getNamedWorkLocation(sMountainViewLocation));

			// Save ready for pay worker
			savedWorker = sWorkerService.saveWorker(eiWorker);
			// Set the Withholdings
			savedWorker = addW4WithholdingInfo(savedWorker.getObjectHandle().getId());
			savedWorker = addDe4WithholdingInfo(savedWorker.getObjectHandle().getId());

            // Setup compensation
            CompensationItem[] compensationItems = createCompensationItems();
            SaveCompensationItemRequest[] compensationItemRequests = new SaveCompensationItemRequest[compensationItems.length];
            for (int i = 0; i < compensationItems.length; i++) {
                compensationItemRequests[i] = new SaveCompensationItemRequest(compensationItems[i], SaveCompensationItemOperation.ADD);
            }

    		SaveCompensationItemResponse saveCompensationResponse = sWageService.saveCompensationItems(savedWorker.getObjectHandle().getId(), compensationItemRequests);
    		savedWorker = saveCompensationResponse.getWorker();
			// Save ready for pay worker
			savedWorker = sWorkerService.saveWorker(savedWorker);
            
            computedWorkerList = sWorkerService.getWorkers(GetWorkersFilter.ALL);
            
            // Check that new Worker is in the computed list
            boolean inList = false;
            for (com.intuit.sb.payroll.services.worker.Worker w : computedWorkerList) {
                if (w.getObjectHandle().getId() == savedWorker.getObjectHandle().getId() && w.isReadyToPay()) {
                    inList = true;
                    break;
                }
            }
            if (!inList) {
	            StringBuilder builder = new StringBuilder();
	            String[] missingReadyToPayProperties = savedWorker.getMissingReadyToPayProperties();
	            for (String string : missingReadyToPayProperties) {
					builder.append(string);
				}
					AssertJUnit.assertTrue("Missing attributes: " + builder.toString(), inList);   
	        }
        } catch (ValidationException e) {
            AssertJUnit.fail("ValidationException was thrown\n" + e.getMessage());
        } catch (WorkflowException e) {
            AssertJUnit.fail("WorkflowException was thrown\n" + e.getMessage());
        } catch (ServiceFailureException e) {
            AssertJUnit.fail("ServiceFailureException was thrown\n" + e.getMessage());
        } finally {
			if (savedWorker != null) {
				// Clean up 
				uniqueId = savedWorker.getObjectHandle().getId();
				cleanUpEIWorker(uniqueId);
			}
        }        
    }
    
    /**
     * <P>Test getWorkers() with GetWorkersFilter.ACTIVE</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Query the database for all Workers</LI>
     * <LI>Pass GetWorkersFilter.ACTIVE to WorkerService.getWorkers()</LI>
     * <LI>Compare the list of Workers from the interface call to the Workers in the db</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the list of Workers returned from WorkerService.getWorkers() matches the db</LI>
     * </UL>
     */
    @Test
    public void testGetActiveWorkersFilter() throws Exception {

        List<Worker> workerEntityList = entityService.findAll(Worker.class);
        com.intuit.sb.payroll.services.worker.Worker[] expectedWorkers = 
            new com.intuit.sb.payroll.services.worker.Worker[workerEntityList.size()];
        for (int i = 0; i < workerEntityList.size(); i++) {
            Worker workerEntity = workerEntityList.get(i);
            if (workerEntity.isActive()) {
            	expectedWorkers[i] = WorkerMapper.mapFromEntityToPojo(workerEntity);
            }
        }
        com.intuit.sb.payroll.services.worker.Worker[] workerList = sWorkerService.getWorkers(GetWorkersFilter.ACTIVE);
        AssertJUnit.assertTrue(verifyEIWorkers(expectedWorkers, workerList));
    }
    
    /**
     * This method sets all employees as Ready For Pay, 
     * @return UniqueId of the employees
     */
    private List<Long> setEmployeesReadyForPay() throws Exception {

        List<Long> idList = new ArrayList<Long>();
        for(int numOfWorkers = 0; numOfWorkers < sNumberOfWorkers; numOfWorkers++) {
            Long id = createPayrollReadyWorker();
            idList.add(id);
        }
        return idList;
    }
}
