package com.intuit.sb.payroll.qa.services.worker;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.List;

import javax.transaction.Status;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.services.benefits.BenefitsService;
import com.intuit.sb.payroll.services.benefits.HealthBenefit;
import com.intuit.sb.payroll.services.benefits.HealthBenefitResponse;
import com.intuit.sb.payroll.services.benefits.HealthBenefitType;
import com.intuit.sb.payroll.services.common.ObjectHandle;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.pto.AccrualStartType;
import com.intuit.sb.payroll.services.pto.PTOAccrualType;
import com.intuit.sb.payroll.services.pto.PTOInfo;
import com.intuit.sb.payroll.services.pto.PTOService;
import com.intuit.sb.payroll.services.pto.PTOType;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.PtoBaseRateType;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.wages.WageService;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;
import com.intuit.sb.payroll.services.worker.WorkerType;
import com.intuit.sb.payroll.services.worker.impl.WorkerMapper;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;

/**
 * <P>The <CODE>SaveWorkerTest</CODE> class contains test methods for the SaveWorker() 
 * Payroll Engine Entity Interface API. 
 * 
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class SaveWorkerTest extends WorkerServiceTestBase {
	
	private static Long sUniqueId = null;
	private static Worker sourceWorker;
	private static Long sourceWorkerId;
	
	protected static PTOService sPTOService;
	protected static WageService sWageService;
    protected static BenefitsService sBenefitsService;
    
	@BeforeClass
	public static void setUpSaveWorkerTest() {
		sPTOService = qaBootstrap.getEjbFromClass(PTOService.class);
		sWageService = qaBootstrap.getEjbFromClass(WageService.class);
		sBenefitsService = qaBootstrap.getEjbFromClass(BenefitsService.class);
		
        try {
            getUserTransaction().begin();
            // Save a new Worker to be used by all tests
			sUniqueId = createAWorkerWithWorkLocation();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpSaveWorkerTest(): " + t.getMessage());
        }
	}
	
	@AfterClass
	public static void tearDownSaveWorkerTest() {
        try {
            getUserTransaction().begin();
    		cleanUpEIWorker(sUniqueId);
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownSaveWorkerTest(): " + t.getMessage());
        }
	}
	
	//Utility method to create some random worker to be used as source worker 
	//when trying to create a worker by copying from existing
	public void setUpWorker() throws Exception
	{
		// Test SaveWorker with no compensations
		sourceWorker = sWorkerService.saveWorker(createWorker());
		sourceWorkerId = sourceWorker.getObjectHandle().getId();
		
		// save salary compensation item
		CompensationItem[] inputCompensations = new CompensationItem[1];
		inputCompensations[0] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Salary, "salary",true);
		inputCompensations[0].setPtoBaseRateType(PtoBaseRateType.PTO);
        // Test inserting compensations
		sWageService.saveCompensationItems(sourceWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, inputCompensations));
	}
	
	/**
     * <P>Test Exception caused by null SaveWorkerRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Pass null SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since workerId is null</LI>
     * </UL>
     */
	@Test
	public void testNullSaveWorkerRequest() {
		// Setup
		// - Implicit setup in setUp()
		
		// Exercise
		try {
			sWorkerService.saveWorker(null);
			AssertJUnit.fail("ValidationException was not thrown");
		}
		
		// Verify
		catch (ServiceFailureException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("worker cannot be null"));			
		} catch (Exception e) {
			AssertJUnit.fail("Expected ServiceFailureException was not thrown");
		}	
		
		// Teardown 
		//- Implicit teardown in tearDown()
	}
	
	/**
     * <P>Test to save one new Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker on SaveWorkerRequest with no uniqueID </LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker with uniqueId returned from saveWorker exists in the db.</LI>
     * </UL>
     */
	@Test
	public void testSaveNewWorker() throws Exception {
		Worker worker;
		com.intuit.sb.payroll.entity.model.Worker prWorker = null;
		try {
			worker = sWorkerService.saveWorker(createDtoWorker());
			
			// Verify
			prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, worker.getObjectHandle().getId());
			AssertJUnit.assertTrue(prWorker != null);			
		} finally {
			// Teardown 
			// - Additional teardown
			// - Implicit teardown in tearDown() 
			cleanUpEIWorker(prWorker);
		}
		
	}
	
	@Test
	public void testSaveWorkerForPaycheckDistributionMethod() throws Exception
	{
		Worker worker;
		com.intuit.sb.payroll.entity.model.Worker prWorker = null;
		try {
		    worker = sWorkerService.saveWorker(createDtoWorker());
			
			// Verify
			prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, worker.getObjectHandle().getId());
			AssertJUnit.assertEquals(PayrollDistributionMethodEnum.HANDWRITTENPAYCHECK, prWorker.getPayrollDistributionMethod());			
		} finally {
			// Teardown 
			// - Additional teardown
			// - Implicit teardown in tearDown() 
			cleanUpEIWorker(prWorker);
		}
	}
	
	@Test
	public void testUpdatepaycheckDistributionMethod() throws Exception
	{
		Worker responseWorker = null;
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker worker = new Worker();
	    	worker.setFirstName("James Bond");
	    	worker.setPaycheckDistributionMethod(PaycheckDistributionMethod.HANDWRITTEN);
			
			// Exercise
			Worker savedWorker = sWorkerService.saveWorker(worker);
			responseWorker = sWorkerService.getWorkerById(savedWorker.getObjectHandle().getId());
			
			//Update Avatar
			responseWorker.setPaycheckDistributionMethod(PaycheckDistributionMethod.CASH);
			savedWorker = sWorkerService.saveWorker(responseWorker);
			
			//Verify
			responseWorker = sWorkerService.getWorkerById(savedWorker.getObjectHandle().getId());
			AssertJUnit.assertTrue(responseWorker != null);	
			AssertJUnit.assertEquals(PaycheckDistributionMethod.CASH, responseWorker.getPaycheckDistributionMethod());
			AssertJUnit.assertEquals(worker.getFirstName(), responseWorker.getFirstName());
		} finally {
			// Teardown
			// - Additional teardown
			// - Implicit teardown in tearDown()
			cleanUpEIWorker(responseWorker.getObjectHandle().getId());
		}
	}
	
	/**
     * <P>Test to save one new Worker with no name set</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker on SaveWorkerRequest with no First, Middle, or Last name </LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException was thrown.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerNoName() throws Exception {
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker eiWorker = createDtoWorker();
			eiWorker.setFirstName(null);
			eiWorker.setMiddleName(null);
			eiWorker.setLastName(null);
			
			// Exercise
			sWorkerService.saveWorker(eiWorker);
			AssertJUnit.fail("Exception not thrown");
			
		} catch (ValidationException e) {
            AssertJUnit.assertTrue(true);
		}
		
		// Teardown
		// - Implicit teardown in tearDown()
	}
	
	/**
     * <P>Test to save one new Worker with only a first name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker with only first name on SaveWorkerRequest </LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker is saved</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerFirstNameOnly() throws Exception {
	    com.intuit.sb.payroll.entity.model.Worker prWorker = null;
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker worker = new Worker();
	    	worker.setFirstName("Barry");
			
			// Exercise
			Worker savedWorker = sWorkerService.saveWorker(worker);
			
			// Verify
			prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, savedWorker.getObjectHandle().getId());
			AssertJUnit.assertTrue(prWorker != null);			
		} finally {
			// Teardown
			// - Additional teardown
			// - Implicit teardown in tearDown()
			cleanUpEIWorker(prWorker);
		}
	}	
	
	/**
     * <P>Test to save one new Worker with only basic info including a work location.</P>
    *
    * <H3>Steps:</H3>
    * <OL>
    * <LI>Create new SaveWorkerRequest</LI>
    * <LI>Set Worker with only basic info including work location on SaveWorkerRequest </LI>
    * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
    * </OL>
    *
    * <UL>
    * <LI>Verify that Worker is saved</LI>
    * </UL>
    */
	@Test
	public void testSaveWorkerWithWorkLocation() throws Exception {
	    com.intuit.sb.payroll.entity.model.Worker prWorker = null;
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker worker = new Worker();
	    	worker.setFirstName("Barry");
	    	worker.setLastName("Bonds");
	    	worker.setJobTitle("Slugger");
	    	worker.setHireDate(new Date());
	    	worker.setBirthDate(new Date());
	    	worker.setWorkerType(WorkerType.EMPLOYEE);
		    worker.setWorkLocation(getAWorkLocation());
			
			// Exercise
			Worker savedWorker = sWorkerService.saveWorker(worker);
			
			// Verify
			prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, savedWorker.getObjectHandle().getId());
			AssertJUnit.assertTrue(prWorker != null);			
		} finally {
			// Teardown
			// - Additional teardown
			// - Implicit teardown in tearDown()
			cleanUpEIWorker(prWorker);
		}
	}
	
	/**
     * <P>Test to save one new Worker with no first name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker with no first name on SaveWorkerRequest </LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerNullFirstName() throws Exception {
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker eiWorker = createDtoWorker();
			eiWorker.setFirstName(null);
			
			// Exercise
			sWorkerService.saveWorker(eiWorker);
			AssertJUnit.fail("Exception not thrown");
			
		} catch (ValidationException e) {
            AssertJUnit.assertTrue(true);
		}
		
		// Teardown
		// - Implicit teardown in tearDown()
	}
	/**
     * <P>Test to save one new Worker with empty first name string</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker with no first name on SaveWorkerRequest </LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerEmptyFirstName() throws Exception {
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker eiWorker = createDtoWorker();
			eiWorker.setFirstName("");
			
			// Exercise
			sWorkerService.saveWorker(eiWorker);
			AssertJUnit.fail("Exception not thrown");
			
		} catch (ValidationException e) {
            AssertJUnit.assertTrue(true);
		}
		
		// Teardown
		// - Implicit teardown in tearDown()
	}
	
	/**
     * <P>Test to save one new Worker with no last name</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker with no last name on SaveWorkerRequest </LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker with uniqueId returned from saveWorker exists in the db.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerNoLastName() throws Exception {
		Worker savedWorker;
		com.intuit.sb.payroll.entity.model.Worker prWorker = null;
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker eiWorker = createDtoWorker();
			eiWorker.setLastName(null);
			
			// Exercise
			savedWorker = sWorkerService.saveWorker(eiWorker);

			// Verify
			prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, savedWorker.getObjectHandle().getId());
			AssertJUnit.assertTrue(prWorker != null);			
		} finally {
			// Teardown
			// - Additional teardown
			// - Implicit teardown in tearDown()
			cleanUpEIWorker(prWorker);
		}
	}
	
	/**
     * <P>Test to update a saved Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Call getWorkerById to retrieve saved Worker </LI>
     * <LI>Update last name on worker</LI>
     * <LI>Set updated Worker on SaveWorkerRequest</LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker is updated in the db.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerUpdateLastName() throws Exception {
		// Setup
		// - Implicit setup in setUp()
		// - Additional setup
		Worker eiWorker = sWorkerService.getWorkerById(sUniqueId);
		String origLastName = eiWorker.getLastName();
		eiWorker.setLastName("Sanders");
		
		// Exercise
		Worker savedWorker = sWorkerService.saveWorker(eiWorker);
		AssertJUnit.assertEquals("Last Name didn't change", "Sanders", savedWorker.getLastName());
		
		// Verify
		com.intuit.sb.payroll.entity.model.Worker prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, sUniqueId);
		Worker aWorker = WorkerMapper.mapFromEntityToPojo(prWorker);
		AssertJUnit.assertEquals(eiWorker.getLastName(), aWorker.getLastName());
		
		// Teardown
		// - Additional teardown
		// - Implicit teardown in tearDown()
		eiWorker.setLastName(origLastName);
		eiWorker.setObjectHandle(savedWorker.getObjectHandle());
		sWorkerService.saveWorker(eiWorker);
	}

	/**
	 * <P>Test to update a saved Worker</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create new SaveWorkerRequest</LI>
	 * <LI>Call getWorkerById to retrieve saved Worker </LI>
	 * <LI>Update last name on worker</LI>
	 * <LI>Set updated Worker on SaveWorkerRequest</LI>
	 * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that Worker is updated in the db.</LI>
	 * </UL>
	 */
	@Test(enabled=false, groups={"broken"})
	public void testSaveWorkerUpdateWorkerType() throws Exception {

		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker eiWorker = sWorkerService.getWorkerById(sUniqueId);
			eiWorker.setWorkerType(WorkerType.NON_EMPLOYEE);
			
			// Exercise
			sWorkerService.saveWorker(eiWorker);

			// Verify
			Worker newEiWorker = sWorkerService.getWorkerById(sUniqueId);
			AssertJUnit.assertTrue(newEiWorker.getWorkerType().equals(WorkerType.NON_EMPLOYEE));
			
		} finally {
			// Teardown
			// - Additional teardown - Set WorkerType back on employee
			// - Implicit teardown in tearDown()
			Worker eiWorker = sWorkerService.getWorkerById(sUniqueId);
			eiWorker.setWorkerType(WorkerType.EMPLOYEE);
			sWorkerService.saveWorker(eiWorker);
		}
	}

		/**
	 * <P>
	 * Test to update a saved Worker with a new Unique Id
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create new SaveWorkerRequest</LI>
	 * <LI>Call getWorkerById to retrieve saved Worker </LI>
	 * <LI>Set a random Unique Id on worker</LI>
	 * <LI>Set updated Worker on SaveWorkerRequest</LI>
	 * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException was thrown.</LI>
	 * </UL>
	 */
	@SuppressWarnings("deprecation")
	@Test (expectedExceptions=ServiceFailureException.class)
	public void testSaveWorkerUpdateUnknownId() throws Exception {
		// Setup
		// - Implicit setup in setUp()
		// - Additional setup
		Worker eiWorker = sWorkerService.getWorkerById(sUniqueId);
		eiWorker.getObjectHandle().setId(new Long("87348875"));
		
		// Exercise
		sWorkerService.saveWorker(eiWorker);
		
		// Verify
		// - Implicit JUnit exception handling
		
		// Teardown
		// - Implicit teardown in tearDown()
	}

	/**
	 * <P>
	 * Test to update a saved Worker with a WorkLocation with a bogus Unique Id
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create new SaveWorkerRequest</LI>
	 * <LI>Call getWorkerById to retrieve saved Worker </LI>
	 * <LI>Set bogus Unique Id on WorkLocation</LI>
	 * <LI>Set updated Worker on SaveWorkerRequest</LI>
	 * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify that ServiceFailureException was thrown.</LI>
	 * </UL>
	 */
	@SuppressWarnings("deprecation")
	@Test (expectedExceptions=ServiceFailureException.class)
	public void testSaveWorkerUpdateWithNewWorkLocationUnknownId() throws Exception {
		WorkLocation origWorkLocation = null;
		Worker eiWorker = null;
		
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			eiWorker = sWorkerService.getWorkerById(sUniqueId);
		
			// Remember original work location
			origWorkLocation = eiWorker.getWorkLocation();

			WorkLocation[] workLocations = sEmployerService.getWorkLocations();
			if(workLocations != null) {
				WorkLocation savedLocation = workLocations[0];
				savedLocation.getObjectHandle().setId(new Long("2389394"));
				eiWorker.setWorkLocation(savedLocation);
				
				// Exercise
				sWorkerService.saveWorker(eiWorker);
			} else {
				AssertJUnit.fail("No Work locations in the db");
			}
		} finally {
			// Teardown
			// - Additional teardown - Set original work location back on the Worker
			// - Implicit teardown in tearDown()
            if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
    			if(eiWorker != null && origWorkLocation != null) {
    				eiWorker.setWorkLocation(origWorkLocation);
    				sWorkerService.saveWorker(eiWorker);
    			}
            }
		}
	}
	
	/**
     * <P>Test to update a saved Worker with a WorkLocation</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Call getWorkerById to retrieve saved Worker </LI>
     * <LI>Update WorkLocation on Worker to new location.  Test assumes Worker already has a WorkLocation set</LI>
     * <LI>Set updated Worker on SaveWorkerRequest</LI>
     * <LI>Save updated worker through IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker is updated in the db.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerUpdateWithUpdatedWorkLocation() throws Exception {
		// Setup
		// - Implicit setup in setUp()
		// - Additional setup
		WorkLocation[] workLocations = null;
		Worker eiWorker = sWorkerService.getWorkerById(sUniqueId);
		
		// Remember original work location
		WorkLocation origWorkLocation = eiWorker.getWorkLocation();
		workLocations = sEmployerService.getWorkLocations();
		if(workLocations != null) {
			for(WorkLocation loc : workLocations) {
				// Use different work location
				if(!(loc.getObjectHandle().getId() == origWorkLocation.getObjectHandle().getId())) {
					eiWorker.setWorkLocation(loc);
                    break;
				}
			}
			
			// Exercise
			Worker savedWorker = sWorkerService.saveWorker(eiWorker);

			// Verify
			// - Check db to see if Worker was updated.
			com.intuit.sb.payroll.entity.model.Worker prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, sUniqueId);
			Worker aWorker = WorkerMapper.mapFromEntityToPojo(prWorker);			// Compare UniqueIds
			AssertJUnit.assertTrue(eiWorker.getWorkLocation().getObjectHandle().getId() == aWorker.getWorkLocation().getObjectHandle().getId());
			
			// Teardown
			// - Additional teardown - Set original work location back on the Worker
			// - Implicit teardown in tearDown()
			eiWorker.setWorkLocation(origWorkLocation);
			eiWorker.setObjectHandle(savedWorker.getObjectHandle());
			sWorkerService.saveWorker(eiWorker);
		} else {
			AssertJUnit.fail("No Work locations in the db");
		}	
	}

	
    
	/**
     * <P>Test to save a new Worker with Other PhoneType</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker with Other PhoneType</LI>
     * <LI>Save Worker </LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker with uniqueId returned from saveWorker has Other PhoneType.</LI>
     * </UL>
     */
	// PhoneType.Other has been removed
	/*
	@Test
	public void testSaveNewWorkerWithPhoneTypeOther() throws Exception {
		// Setup
		// - Implicit setup in setUp()
		// - Additional setup
		PhoneType expectedPhoneType = PhoneType.Other;
		
		// Exercise
		// Verify
		verifyPhoneType(expectedPhoneType);
		
		// Teardown
		// - Implicit teardown in tearDown()
	}
	*/
	
    /***
     * <P>Test to save one new Worker and verify the returned Worker.</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker</LI>
     * <LI>Pass Worker to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker with uniqueId returned from saveWorker exists in the db.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerVerifyWorker() throws Exception {
		Worker newWorker;
		Worker savedWorker;
		com.intuit.sb.payroll.entity.model.Worker prWorker = null;
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			newWorker = createDtoWorker();
			newWorker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
			newWorker.setWagePlanCode("S");
			
			// Exercise
			savedWorker = sWorkerService.saveWorker(newWorker);
			
            prWorker = entityService.findById(com.intuit.sb.payroll.entity.model.Worker.class, savedWorker.getObjectHandle().getId());
            AssertJUnit.assertNotNull("Can't find Barry!", prWorker);
            
            // Set objectHandles and missingReadyForPayProperties in newWorker for comparison
            newWorker.setObjectHandle(savedWorker.getObjectHandle());
            newWorker.setTakehomeWorkerObjectHandle(savedWorker.getTakehomeWorkerObjectHandle());
            newWorker.setMissingReadyToPayProperties(savedWorker.getMissingReadyToPayProperties());
            newWorker.setEmploymentStatus(savedWorker.getEmploymentStatus());
            newWorker.setActive(savedWorker.isActive());
			
			AssertJUnit.assertEquals("worker", newWorker, savedWorker);			
		} finally {
			// Teardown
			// - Additional teardown
			// - Implicit teardown in tearDown() 
			cleanUpEIWorker(prWorker);
		}
		
	}
	
	/**
     * <P>Test to save one new Worker with first name and an avatar and verify the avatar returned.</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker on SaveWorkerRequest with no uniqueID </LI>
     * <LI>Set the first name and avatar on the worker</LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker is returned with the correct first name and avatar.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerWithFirstNameAndAvatar() throws Exception
	{
		Worker responseWorker = null;
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker worker = new Worker();
	    	worker.setFirstName("Barry");
	    	worker.setAvatar("default.jpg");
			
			// Exercise
			Worker savedWorker = sWorkerService.saveWorker(worker);
			
			// Verify
			responseWorker = sWorkerService.getWorkerById(savedWorker.getObjectHandle().getId());
			AssertJUnit.assertTrue(responseWorker != null);	
			AssertJUnit.assertEquals(worker.getAvatar(), responseWorker.getAvatar());
			AssertJUnit.assertEquals(worker.getFirstName(), responseWorker.getFirstName());
		} finally {
			// Teardown
			// - Additional teardown
			// - Implicit teardown in tearDown()
			cleanUpEIWorker(responseWorker.getObjectHandle().getId());
		}
	}
	
	/**
     * <P>Test to save one new Worker with just a first name and verify the avatar returned.</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker on SaveWorkerRequest with no uniqueID </LI>
     * <LI>Set the first name on the worker</LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker is returned with the correct first name and the default avatar.</LI>
     * </UL>
     */
	@Test
	public void testSaveWorkerWithOnlyFirstNameNoAvatar() throws Exception
	{
		Worker responseWorker = null;
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker worker = new Worker();
	    	worker.setFirstName("Barry");
			
			// Exercise
			Worker savedWorker = sWorkerService.saveWorker(worker);
			
			// Verify
			responseWorker = sWorkerService.getWorkerById(savedWorker.getObjectHandle().getId());
			AssertJUnit.assertTrue(responseWorker != null);	
			AssertJUnit.assertEquals(null, responseWorker.getAvatar());
			AssertJUnit.assertEquals(worker.getFirstName(), responseWorker.getFirstName());
		} finally {
			// Teardown
			// - Additional teardown
			// - Implicit teardown in tearDown()
			cleanUpEIWorker(responseWorker.getObjectHandle().getId());
		}
	}
	
	/**
     * <P>Test to update Worker's Avatar and verify that it is saved correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new SaveWorkerRequest</LI>
     * <LI>Set Worker on SaveWorkerRequest with no uniqueID </LI>
     * <LI>Set the first name and avatar on the worker</LI>
     * <LI>Pass SaveWorkerRequest to IWorkerService.SaveWorker()</LI>
     * <LI>Retrieve the worker by calling IWorkerService.getWorkerById()</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Worker is returned with the correct first name and updated avatar.</LI>
     * </UL>
     */
	@Test
	public void testUpdateAvatarOnWorker() throws Exception
	{
		Worker responseWorker = null;
		try {
			// Setup
			// - Implicit setup in setUp()
			// - Additional setup
			Worker worker = new Worker();
	    	worker.setFirstName("Barry");
			
			// Exercise
			Worker savedWorker = sWorkerService.saveWorker(worker);
			responseWorker = sWorkerService.getWorkerById(savedWorker.getObjectHandle().getId());
			
			//Update Avatar
			responseWorker.setAvatar("new.jpg");
			savedWorker = sWorkerService.saveWorker(responseWorker);
			
			//Verify
			responseWorker = sWorkerService.getWorkerById(savedWorker.getObjectHandle().getId());
			AssertJUnit.assertTrue(responseWorker != null);	
			AssertJUnit.assertEquals("new.jpg", responseWorker.getAvatar());
			AssertJUnit.assertEquals(worker.getFirstName(), responseWorker.getFirstName());
		} finally {
			// Teardown
			// - Additional teardown
			// - Implicit teardown in tearDown()
			cleanUpEIWorker(responseWorker.getObjectHandle().getId());
		}
	}
	
	/**
     * <P>Test to see if the mimic copy worker functionality works correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker in memory</LI>
     * <LI>Set Compensation Items, HealthBenefits and PTO on newWorker </LI>
     * <LI>Save newWorker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that payroll items on newWorker are correct after the save</LI>
     * </UL>
     * FIXME [th] this is not the proper way to manage payroll items, use the WageService.
     * Brian is checking to see if the app relies on this 
     */
	@Test(enabled=false, groups={"broken"})
	public void testCopyWorkerWithAllPayrollItemTypes() throws Exception
	{
		Worker newWorker = createWorker();

		CompensationItem[] compensationItems = new CompensationItem[2];
		compensationItems[0] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Salary, "salary", true);
		compensationItems[0].setPtoBaseRateType(PtoBaseRateType.PTO);
		compensationItems[1] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Doubletime, "DblTime", true);
		compensationItems[1].setLinkedCompensationItem(compensationItems[0]);
		
		PTOInfo[] ptoInfos = new PTOInfo[1];
		ptoInfos[0] = createPTOInfo(PTOType.SICK_TIME, sourceWorkerId);
    	
        HealthBenefit[] healthBenefits = new HealthBenefit[1];
        healthBenefits[0] = createHealthBenefit(HealthBenefitType.HEALTH, "Pre-tax Health", new BigDecimal("17.95"));
        
    	newWorker.setPtoInfos(ptoInfos);
    	newWorker.setCompensationItems(compensationItems);
    	newWorker.setHealthBenefits(healthBenefits);

    	//Save new worker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	
    	PTOInfo newPTOInfo = sPTOService.getPTOByNameAndType(newWorker.getObjectHandle().getId(), ptoInfos[0].getName(), ptoInfos[0].getPtoType());
    	
    	//Make sure the PTOInfo is not null
    	AssertJUnit.assertNotNull("PTOInfo on new worker is null", newPTOInfo);
    	
    	//Make sure the compensation items on new worker are not null
    	CompensationItem[] newCompensationItems = sWageService.getCompensationItems(newWorker.getObjectHandle().getId());
    	AssertJUnit.assertNotNull("Compensation Items on new worker are null", newCompensationItems);
    	// Extra compensation is a disbursement auto added for the PTO
    	AssertJUnit.assertEquals("Compensation Items on new worker are present", compensationItems.length + 1, newCompensationItems.length);
    	
    	//Now verify PTOs and WageItems
    	AssertJUnit.assertTrue(comparePTOInfo(ptoInfos[0], newPTOInfo));
    	//now make sure compensation items are copied
    	AssertJUnit.assertTrue(compareCompensations(compensationItems, newCompensationItems));
    	CompensationItem dblTime = null;
    	CompensationItem salary = null;
    	for (CompensationItem compensationItem : newCompensationItems) {
    		if (compensationItem.getCompensationType().ordinal() == CompensationType.Doubletime.ordinal()) {
    			dblTime = compensationItem;
    		}
    		else if (compensationItem.getCompensationType().ordinal() == CompensationType.Salary.ordinal()) {
    			salary = compensationItem;
    		}
    	}
    	AssertJUnit.assertNotNull("Salary found", salary);
    	AssertJUnit.assertNotNull("Doubletime found", dblTime);
    	AssertJUnit.assertTrue("Doubletime links to Salary", dblTime.getLinkedCompensationItem() == salary);
    	
    	//clean up worker
    	cleanUpEIWorker(newWorker.getObjectHandle().getId());
	}
	
	/**
     * <P>Test to see if the mimic copy worker health benefits functionality works correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker in memory</LI>
     * <LI>Set HealthBenefits  on newWorker </LI>
     * <LI>Save newWorker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Health Benefit payroll item on newWorker are correct after the save</LI>
     * </UL>
     */
	@Test
	public void testCopyWorkerWithHealthBenefitPayrollItemType() throws Exception
	{
		Worker newWorker = createWorker();
			
		//Create Health benefit
    	
        HealthBenefit[] healthBenefits = new HealthBenefit[1];
        healthBenefits[0] = createHealthBenefit(HealthBenefitType.HEALTH, "Pre-tax Health", new BigDecimal("17.95"));

    	newWorker.setHealthBenefits(healthBenefits);

    	//Save new worker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	
    	if (newWorker!= null ){
    		HealthBenefit[] healthBenefits1 = newWorker.getHealthBenefits();
    		if (healthBenefits1.length < 1) {
    			AssertJUnit.fail("Health Benefit not set on worker");
    		}
    		HealthBenefit healthBenefit = (HealthBenefit)healthBenefits1[0];
    		AssertJUnit.assertEquals(healthBenefit.getType().name(), HealthBenefitType.HEALTH.name());
     		AssertJUnit.assertEquals(healthBenefit.getName(), "Pre-tax Health");
    	}
    	//clean up worker
    	cleanUpEIWorker(newWorker.getObjectHandle().getId());
	   }
	
	/**
     * <P>Test to see if the mimic copy worker Pay functionality works correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker in memory</LI>
     * <LI>Set Pay  on newWorker </LI>
     * <LI>Save newWorker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Pay payroll item on newWorker are correct after the save</LI>
     * </UL>
     */
	@Test
	public void testCopyWorkerWithPayPayrollItemType() throws Exception
	{
		Worker newWorker = createWorker();
			
		//Create Pay
    	
		CompensationItem[] compensationItems = new CompensationItem[1];
		compensationItems[0] = new CompensationItem(null, new BigDecimal(100), 
				0, CompensationType.Commission, "Commission", true);

		newWorker.setCompensationItems(compensationItems);

    	//Save new worker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	
    	if (newWorker!= null ){
    		CompensationItem[] compItems = newWorker.getCompensationItems();
    		if (compItems.length < 1) {
    			AssertJUnit.fail("Pay Commission not set on worker");
    		}
    		CompensationItem compItem = (CompensationItem)compItems[0];
    		AssertJUnit.assertEquals(compItem.getCompensationType().name(),CompensationType.Commission.name());
    		AssertJUnit.assertEquals(compItem.getName(), "Commission");
    	}
    	//clean up worker
    	cleanUpEIWorker(newWorker.getObjectHandle().getId());
	   }
	
	/**
     * <P>Test to see if the mimic copy worker Pay, PTO functionality works correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker in memory</LI>
     * <LI>Set Compensation Items and PTO on newWorker </LI>
     * <LI>Save newWorker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that payroll items on newWorker are correct after the save</LI>
     * </UL>
     */
	@Test
	public void testCopyWorkerWithPayPtoPayrollItemTypes() throws Exception
	{
		Worker newWorker = createWorker();

		CompensationItem[] compensationItems = new CompensationItem[1];
		compensationItems[0] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Salary, "salary", true);
		compensationItems[0].setPtoBaseRateType(PtoBaseRateType.PTO);
		
		PTOInfo[] ptoInfos = new PTOInfo[1];
		ptoInfos[0] = createPTOInfo(PTOType.SICK_TIME, sourceWorkerId);

        
    	newWorker.setPtoInfos(ptoInfos);
    	newWorker.setCompensationItems(compensationItems);
 
    	//Save new worker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	
    	   	
    	//Make sure the PTOInfo is not null
    	AssertJUnit.assertNotNull("PTOInfo on new worker is null", newWorker.getPtoInfos());
    	
    	//Make sure the compensation items on new worker are not null
    	CompensationItem[] newCompensationItems = newWorker.getCompensationItems();
    	AssertJUnit.assertNotNull("Compensation Items on new worker are null", newCompensationItems);
    	// Extra compensation is a disbursement auto added for the PTO
    	AssertJUnit.assertEquals("Compensation Items on new worker are present", compensationItems.length + 1, newCompensationItems.length);
    	
    	//Now verify PTOs and WageItems
    	AssertJUnit.assertTrue(comparePTOInfo(ptoInfos[0], newWorker.getPtoInfos()[0]));
    	//now make sure compensation items are copied
    	AssertJUnit.assertTrue(compareCompensations(compensationItems, newCompensationItems));
    	  	
    	//clean up worker
    	cleanUpEIWorker(newWorker.getObjectHandle().getId());
	}
	
	
	/**
     * <P>Test to see if the mimic copy worker functionality works correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker and call it sourceWorker</LI>
     * <LI>Set Compensation Items and PTO on sourceWorker </LI>
     * <LI>Create another worker called newWorker</LI>
     * <LI>Now, copy compensation items and PTOInfo from sourceWorker to newWorker</LI>
     * <LI>Save newWorker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that compensationItems and PTOInfos of both sourceWorker and newWorker are the same since we copied</LI>
     * </UL>
     */
	@Test
	public void testCopyWorkerWithAllCompensationAndOnePTOInfo() throws Exception
	{
		//Create a  payroll ready worker to be used as source worker 
		//from which we copy settings to new worker
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME, sourceWorkerId);
		sPTOService.savePTO(expectedPTOInfo);
    	
		//refresh worker
		sourceWorker = sWorkerService.getWorkerById(sourceWorkerId);
    	//Now create a new worker and set basic properties
    	Worker newWorker = new Worker();
    	newWorker.setFirstName("Anakin");
    	newWorker.setLastName("Skywalker");
    	newWorker.setHireDate(DateUtil.getNowDate());

    	//Save newWorker
    	newWorker = sWorkerService.saveWorker(newWorker);

    	//Copy existing Compensation Items into new worker
    	CompensationItem[] existingCompensationItems = sWageService.getCompensationItems(sourceWorkerId);
    	List<CompensationItem> newCompensationItemsList = new ArrayList<CompensationItem>();
    	int item = 0;
    	//Don't add SickTaken and VacationTaken
    	for(int count = 0;count < existingCompensationItems.length;count++)
    	{
    		if(!(existingCompensationItems[count].getCompensationType().equals(CompensationType.SickTaken)) &&
    				!(existingCompensationItems[count].getCompensationType().equals(CompensationType.VacationTaken)))
    		{
    			//Make sure you clear the ObjectHandle so that the service thinks this is new compensation item
    			//for the worker.
    			existingCompensationItems[count].setObjectHandle(null);
    			newCompensationItemsList.add(existingCompensationItems[count]);
    			item++;
    		}
    	}
    	CompensationItem[] newCompensationItems = new CompensationItem[item];
    	newCompensationItemsList.toArray(newCompensationItems);
    	newWorker.setCompensationItems(newCompensationItems);
    	
    	//Save newWorker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	
    	//Copy existing PTO into new worker
    	PTOInfo[] sourcePTOInfo = new PTOInfo[1];
    	sourcePTOInfo[0] = sPTOService.getPTOByNameAndType(sourceWorkerId, expectedPTOInfo.getName(), expectedPTOInfo.getPtoType());
    	sourcePTOInfo[0].setObjectHandle(null);
    	newWorker.setPtoInfos(sourcePTOInfo);

    	//Save new worker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	
    	PTOInfo  newPTOInfo = sPTOService.getPTOByNameAndType(newWorker.getObjectHandle().getId(), expectedPTOInfo.getName(), expectedPTOInfo.getPtoType());
    	
    	//Make sure the PTOInfo is not null
    	AssertJUnit.assertNotNull("PTOInfo on new worker is null", newPTOInfo);
    	
    	//Make sure the compensation items on new worker are not null
    	AssertJUnit.assertNotNull("Compensation Items on new worker are null", sWageService.getCompensationItems(newWorker.getObjectHandle().getId()));
    	
    	//Now verify PTOs and WageItems
    	AssertJUnit.assertTrue(comparePTOInfo(sourcePTOInfo[0], newPTOInfo));
    	//now make sure compensation items are copied
    	AssertJUnit.assertTrue(compareCompensations(sWageService.getCompensationItems(sourceWorkerId), sWageService.getCompensationItems(newWorker.getObjectHandle().getId())));
    	
    	//clean up worker
    	cleanUpEIWorker(newWorker.getObjectHandle().getId());
	}
	
	/**
     * <P>Test to see if the mimic copy worker functionality works correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker and call it sourceWorker</LI>
     * <LI>Set Compensation Items and PTO on sourceWorker </LI>
     * <LI>Create another worker called newWorker</LI>
     * <LI>Now, copy only compensation items to newWorker</LI>
     * <LI>Save newWorker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that compensationItems of both sourceWorker and newWorker are the same since we copied</LI>
     * </UL>
     */
	@Test
	public void testCopyOnlyCompensationItemsToNewWorker() throws Exception
	{
		//Create a  payroll ready worker to be used as source worker 
		//from which we copy settings to new worker
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME, sourceWorkerId);
		sPTOService.savePTO(expectedPTOInfo);
    	
		//refresh worker
		sourceWorker = sWorkerService.getWorkerById(sourceWorkerId);
    	//Now create a new worker and set basic properties
    	Worker newWorker = new Worker();
    	newWorker.setFirstName("Anakin");
    	newWorker.setLastName("Skywalker");

    	//Save newWorker
    	newWorker = sWorkerService.saveWorker(newWorker);

    	//Copy existing Compensation Items into new worker
    	CompensationItem[] existingCompensationItems = sWageService.getCompensationItems(sourceWorkerId);
    	List<CompensationItem> newCompensationItemsList = new ArrayList<CompensationItem>();
    	int item = 0;
    	//Don't add SickTaken and VacationTaken
    	for(int count = 0;count < existingCompensationItems.length;count++)
    	{
    		if(!(existingCompensationItems[count].getCompensationType().equals(CompensationType.SickTaken)) &&
    				!(existingCompensationItems[count].getCompensationType().equals(CompensationType.VacationTaken)))
    		{
    			//Make sure you clear the ObjectHandle so that the service thinks this is new compensation item
    			//for the worker.
    			existingCompensationItems[count].setObjectHandle(null);
    			newCompensationItemsList.add(existingCompensationItems[count]);
    			item++;
    		}
    	}
    	CompensationItem[] newCompensationItems = new CompensationItem[item];
    	newCompensationItemsList.toArray(newCompensationItems);
    	newWorker.setCompensationItems(newCompensationItems);
    	
    	//Save newWorker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	
    	//Make sure the compensation items on new worker are not null
    	AssertJUnit.assertNotNull("Compensation Items on new worker are null", sWageService.getCompensationItems(newWorker.getObjectHandle().getId()));
    	
    	AssertJUnit.assertTrue(compareCompensations(sWageService.getCompensationItems(sourceWorkerId), sWageService.getCompensationItems(newWorker.getObjectHandle().getId())));
    	
    	//clean up worker
    	cleanUpEIWorker(newWorker.getObjectHandle().getId());
	}
	
	/**
     * <P>Test to see if the mimic copy worker functionality works correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker and call it sourceWorker</LI>
     * <LI>Set Compensation Items and PTO on sourceWorker </LI>
     * <LI>Create another worker called newWorker</LI>
     * <LI>Now, copy only PTOInfo to newWorker</LI>
     * <LI>Save newWorker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that PTOInfo of both sourceWorker and newWorker are the same since we copied</LI>
     * </UL>
     */
	@Test
	public void testCopyOnlyPTOToNewWorker() throws Exception
	{
		//Create a  payroll ready worker to be used as source worker 
		//from which we copy settings to new worker
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME, sourceWorkerId);
		sPTOService.savePTO(expectedPTOInfo);
    	
		//refresh worker
		sourceWorker = sWorkerService.getWorkerById(sourceWorkerId);
    	//Now create a new worker and set basic properties
    	Worker newWorker = new Worker();
    	newWorker.setFirstName("Anakin");
    	newWorker.setLastName("Skywalker");
    	newWorker.setHireDate(DateUtil.getNowDate());

    	//Save newWorker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	Long newWorkerId = newWorker.getObjectHandle().getId();
    	
		CompensationItem[] inputCompensations = new CompensationItem[1];
		inputCompensations[0] = new CompensationItem(null, new BigDecimal(10000), 
				0, CompensationType.Hourly, "hourly",true);
		inputCompensations[0].setPtoBaseRateType(PtoBaseRateType.PTO);
        // Test inserting compensations
		sWageService.saveCompensationItems(newWorkerId, mapToSaveCompensationItemRequest(
				SaveCompensationItemOperation.ADD, inputCompensations));
		
		//Refresh worker
		newWorker = sWorkerService.getWorkerById(newWorkerId);
    	
    	//Copy existing PTO into new worker
    	PTOInfo[] sourcePTOInfo = new PTOInfo[1];
    	sourcePTOInfo[0] = sPTOService.getPTOByNameAndType(sourceWorkerId, expectedPTOInfo.getName(), expectedPTOInfo.getPtoType());
    	sourcePTOInfo[0].setObjectHandle(null);
    	newWorker.setPtoInfos(sourcePTOInfo);

    	//Save new worker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	
    	PTOInfo  newPTOInfo = sPTOService.getPTOByNameAndType(newWorker.getObjectHandle().getId(), expectedPTOInfo.getName(), expectedPTOInfo.getPtoType());
    	
    	//Make sure the PTOInfo is not null
    	AssertJUnit.assertNotNull("PTOInfo on new worker is null", newPTOInfo);
    	
    	//Make sure the compensation items on new worker are not null
    	AssertJUnit.assertNotNull("Compensation Items on new worker are null", sWageService.getCompensationItems(newWorker.getObjectHandle().getId()));
    	
    	//Now verify PTOs and WageItems
    	AssertJUnit.assertTrue(comparePTOInfo(sourcePTOInfo[0], newPTOInfo));
    	//now make sure compensation items are different
    	AssertJUnit.assertFalse(compareCompensations(sWageService.getCompensationItems(sourceWorkerId), sWageService.getCompensationItems(newWorker.getObjectHandle().getId())));
    	
    	//clean up worker
    	cleanUpEIWorker(newWorker.getObjectHandle().getId());
	}
	
	/**
     * <P>Test to see if the mimic copy worker functionality works correctly</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create new Worker and call it sourceWorker</LI>
     * <LI>Set Compensation Items, PTO and Benefits on sourceWorker </LI>
     * <LI>Create another worker called newWorker</LI>
     * <LI>Now, copy only Benefits to newWorker</LI>
     * <LI>Save newWorker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that Benefits of both sourceWorker and newWorker are the same since we copied</LI>
     * </UL>
     */
	@Test
	public void testCopyBenefitsToWorker() throws Exception
	{
		setUpWorker();
		PTOInfo expectedPTOInfo = createPTOInfo(PTOType.SICK_TIME, sourceWorkerId);
		sPTOService.savePTO(expectedPTOInfo);
    	
		//refresh worker
		sourceWorker = sWorkerService.getWorkerById(sourceWorkerId);
        ObjectHandle workerHandle = sourceWorker.getObjectHandle();
		
        HealthBenefitResponse healthBenefitResponse = null;
        HealthBenefit[] healthBenefits = null;
        BigDecimal benefitAmount = new BigDecimal("17.95");
        HealthBenefit[] healthBenefitsToAdd = new HealthBenefit[1];
        healthBenefitsToAdd[0] = createHealthBenefit(HealthBenefitType.HEALTH, "Pre-tax Health", benefitAmount);
        healthBenefitResponse = sBenefitsService.addHealthBenefits(workerHandle, healthBenefitsToAdd);
        healthBenefits = healthBenefitResponse.getBenefits();
        
    	//Now create a new worker and set basic properties
    	Worker newWorker = new Worker();
    	newWorker.setFirstName("Anakin");
    	newWorker.setLastName("Skywalker");

    	//Save newWorker
    	newWorker = sWorkerService.saveWorker(newWorker);
    	ObjectHandle newWorkerHandle = newWorker.getObjectHandle();
        HealthBenefitResponse newHealthBenefitResponse = null;
        HealthBenefit[] newHealthBenefits = null;
        healthBenefits[0].setObjectHandle(null);
        healthBenefitsToAdd[0] = healthBenefits[0];
        newHealthBenefitResponse = sBenefitsService.addHealthBenefits(newWorkerHandle, healthBenefitsToAdd);
        newHealthBenefits = newHealthBenefitResponse.getBenefits();
        
        AssertJUnit.assertEquals("Only expecting 1 health benefit item!", 1, newHealthBenefits.length);
        
        //This is for verification purposes only
        healthBenefits[0].setObjectHandle(newHealthBenefits[0].getObjectHandle());
        
        AssertJUnit.assertTrue(compareHealthBenefits(healthBenefits, newHealthBenefits));    
        
        //cleanup health benefits
        sBenefitsService.deleteHealthBenefits(newHealthBenefitResponse.getWorker().getObjectHandle(), newHealthBenefits);
    	//clean up worker
    	cleanUpEIWorker(newWorker.getObjectHandle().getId());
	}
	
	/**
	 * Utility method to save a Worker with WorkLocation in the db and return the uniqueId.  
	 * A fully populated Worker is needed to avoid Workflow exceptions in the tests.
	 */
	
	public static Long createAWorkerWithWorkLocation() throws Exception {
		Worker savedWorker = null;
		
    	Worker worker = createDtoWorker();
    	worker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
    	worker.setWorkLocation(getAWorkLocation());
    	worker.setWorkerType(WorkerType.EMPLOYEE);
		
		try {
		    savedWorker = sWorkerService.saveWorker(worker);
		} catch (Exception e) {
			AssertJUnit.fail("Unexpected exception occurred" + e.getMessage());
		}
		
		return savedWorker.getObjectHandle().getId();
	}
	
	/*
	 * Utility method to find and return a work location
	 */
	public static WorkLocation getAWorkLocation() throws Exception {
		WorkLocation[] workLocations = sEmployerService.getWorkLocations();
		return workLocations[0];
	}
	
	// Utility method to createPTOInfo
	private PTOInfo createPTOInfo(PTOType type, Long workerId) throws ValidationException, ServiceFailureException
	{
		PTOInfo ptoInfo = new PTOInfo();
		ptoInfo.setWorkerId(workerId);
		ptoInfo.setName("Floating goliday");
		ptoInfo.setPtoType(type);
		ptoInfo.setAccrualRate(new Double("2.35"));
		ptoInfo.setPtoAccrualType(PTOAccrualType.EACH_PAYCHECK);
		ptoInfo.setAllowCarryOver(true);
		ptoInfo.setLimitAccrual(true);
		ptoInfo.setMaximumAccrual(new Double("200.00"));
		ptoInfo.setAccruedHours(new Double("5.40"));
		ptoInfo.setDisbursedHoursYTD(new Double("0.00"));
		
		Calendar now = DateUtil.getNowCalendar();
		ptoInfo.setAccrualStartDate(now.getTime());
		ptoInfo.setAccrualStartType(AccrualStartType.IMMEDIATELY);
		
		return ptoInfo;
	}
	
    private HealthBenefit createHealthBenefit(HealthBenefitType type, String name, BigDecimal amount) {
        HealthBenefit healthBenefit = new HealthBenefit();
        healthBenefit.setObjectHandle(null);
        healthBenefit.setType(type);
        healthBenefit.setName(name);
        healthBenefit.setRate(amount);
        return healthBenefit;
    }
	
	//Utility method to compare PTOInfos
	private boolean comparePTOInfo(PTOInfo ptoInfo, PTOInfo resultPTOInfo)
	{
		if(ptoInfo.getAccrualRate().doubleValue() == resultPTOInfo.getAccrualRate().doubleValue() 
				&& ptoInfo.getAccrualStartDate().equals(resultPTOInfo.getAccrualStartDate())
				&& ptoInfo.getAccrualStartType().equals(resultPTOInfo.getAccrualStartType())
				&& ptoInfo.getAccruedHours().doubleValue() == resultPTOInfo.getAccruedHours().doubleValue()
				&& ptoInfo.getDisbursedHoursYTD().doubleValue() == resultPTOInfo.getDisbursedHoursYTD().doubleValue()
				&& ptoInfo.getMaximumAccrual().doubleValue() == resultPTOInfo.getMaximumAccrual().doubleValue()
				&& ptoInfo.getName().equals(resultPTOInfo.getName())
				&& ptoInfo.getPtoAccrualType().equals(resultPTOInfo.getPtoAccrualType())
				&& ptoInfo.getPtoType().equals(resultPTOInfo.getPtoType())
				// Source workerId is null in the copy worker with PTOs case
				&& (ptoInfo.getWorkerId() == null || ptoInfo.getWorkerId().longValue() != resultPTOInfo.getWorkerId().longValue()))
		{
			return true;
		}
		
		return false;
	}
	
	//Utility to compare compensation items for the copy worker case
	private boolean compareCompensations(CompensationItem[] expectedArray, CompensationItem[] actualArray)
	{
		boolean isTrue = false;
		for(CompensationItem expectedItem:expectedArray)
		{
			//Check only "Compensation" items. the compensations returned from the service will have
			//disbursement items also in it.
			if(!(expectedItem.getCompensationType().equals(CompensationType.SickTaken)) &&
					!(expectedItem.getCompensationType().equals(CompensationType.VacationTaken)))	
			{
				for(CompensationItem actualItem:actualArray)
				{
					if(expectedItem.getCompensationType().equals(actualItem.getCompensationType()))
					{
						if(expectedItem.getRate().equals(actualItem.getRate()) &&
								expectedItem.getUnits() == actualItem.getUnits() &&
								expectedItem.getName().equalsIgnoreCase(actualItem.getName()))
						{
							isTrue = true;
							continue;
						}
						isTrue = false;
					}
				}
			}
		}
		return isTrue;
	}
	
    protected boolean compareHealthBenefits(HealthBenefit[] expected, HealthBenefit[] computed) {
        if ((null == expected) || (null == computed)) {
            throw new RuntimeException("Null parameter");
        }
        boolean compareResult = (expected.length == computed.length);
        HashSet<HealthBenefit> expectedSet = new HashSet<HealthBenefit>();
        for (HealthBenefit healthBenefit : expected) {
            expectedSet.add(healthBenefit);
        }
        HashSet<HealthBenefit> computedSet = new HashSet<HealthBenefit>();
        for (HealthBenefit healthBenefit : computed) {
            computedSet.add(healthBenefit);
        }
        compareResult = expectedSet.equals(computedSet);
        if (false == compareResult) {
            sLogger.error("Expected HealthBenefit:");            
            for (HealthBenefit healthBenefit : expected) {
                sLogger.error(healthBenefit.toString());            
            }            
            sLogger.error("Computed HealthBenefit:");            
            for (HealthBenefit healthBenefit : computed) {
                sLogger.error(healthBenefit.toString());            
            }            
        }
        return compareResult;
    }

}
