package com.intuit.sb.payroll.qa.services.worker;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.common.EntityUtilsTestBase;
import com.intuit.sb.payroll.services.worker.Worker;


/**
 *  <P>The <CODE>TerminateWorkerExceptionsTest</CODE> class contains tests for testing exceptions that can be thrown
 *     by terminateWorker() in WorkerService</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class TerminateWorkerExceptionsTest extends EntityUtilsTestBase
{
    /**
     * <P>Test calling terminateWorker() with a null terminationDate</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Call IWorkerService.terminateWorker()with a null terminationDate</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testNullTerminationDate() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	Worker worker = sWorkerService.getWorkerById(workerId);
    	// Exercise
    	worker = sWorkerService.terminateWorker(worker, null);
    }

    /**
     * <P>Test calling terminateWorker() with a null Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IWorkerService.terminateWorker()with a null Worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testNullWorker() throws Exception
    {
    	// Exercise
    	@SuppressWarnings("unused")
    	Worker worker = sWorkerService.terminateWorker(null, DateUtil.getNowDate());
    }

    /**
     * <P>Test calling terminateWorker() with a non-persisted Worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IWorkerService.terminateWorker()with a non-persisted Worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testWithNonPersistedWorker() throws Exception
    {
    	// Setup
    	Worker worker = createDtoWorker();
    	// Exercise
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNowDate());
    }

    /**
     * <P>Test calling terminateWorker() with a deleted worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Delete the worker</LI>
     * <LI>Call IWorkerService.terminateWorker()with the deleted worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ServiceFailureException.class)
    public void testDeletedWorker() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	Worker worker = sWorkerService.getWorkerById(workerId);
    	cleanUpEIWorker(workerId);
    	// Exercise
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNowDate());
    }

    /**
     * <P>Test calling terminateWorker() with a terminated worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a payroll ready worker</LI>
     * <LI>Terminate the worker</LI>
     * <LI>Call IWorkerService.terminateWorker()with a terminated worker</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(groups = {"worker.terminateWorker"}, expectedExceptions=ValidationException.class)
    public void testWithTerminatedWorker() throws Exception
    {
    	// Setup
    	Long workerId = createPayrollReadyWorker();
    	Worker worker = sWorkerService.getWorkerById(workerId);
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNowDate());
    	// Exercise
    	worker = sWorkerService.terminateWorker(worker, DateUtil.getNowDate());
    }

}
