package com.intuit.sb.payroll.qa.services.worker;

import org.testng.AssertJUnit;

import org.testng.annotations.Test;

import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.PojoUtils;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.worker.WorkerServiceTestBase;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;

public class WorkerPayrollReadyStateTests extends WorkerServiceTestBase {
	
	/**
	 * <P>Verifies if the worker is ready to pay without withholdings,compensation</P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location but no withholdings,compensation</LI>
	 * <LI>Call Worker Service isWorkerReadyToPay </LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testNonPersistedPayrollReadyServiceWorkerState() {
		Worker worker = null;
		try {
			//Setup
			worker = createWorker();
			worker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
			worker.setWorkLocation(getWorkLocation());

			//Exercise
			boolean isReadyForPay = sWorkerService.isWorkerReadyForPay(worker);

			//Verify
			if (isReadyForPay) {
				AssertJUnit.fail("Worker cannot be payroll ready without withholding and compensation data");
			}
		} catch (Exception e) {
		}
	}

	/**
	 * <P>Verifies if the worker is ready to pay with all the properties set for ready to pay </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Call Worker Service isWorkerReadyToPay </LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */

	@Test
	public void testPersistedPayrollReadyServiceWorkerState() throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();

		//Exercise
		Worker worker = sWorkerService.getWorkerById(workerId);

		//Verify
		if (!worker.isReadyToPay()) {
			AssertJUnit.fail("Worker should be ready for Pay");
		}
	}

	/**
	 * <P>Verifies if the worker is ready to pay without the paycheck distribution method property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers paycheck distribution to Not Set <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the paycheck distribution missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNoPaycheckType()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		worker.setPaycheckDistributionMethod(PaycheckDistributionMethod.NOT_SET);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"paycheckDistributionMethod".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("Paycheck Distribution method should be present in the MissingPropertiesList");
				}
			}
		}
	}
	
	/**
	 * <P>Verifies if the worker is ready to pay with null paycheck distribution method property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers paycheck distribution to null <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the paycheck distribution missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNullPaycheckType()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		worker.setPaycheckDistributionMethod(null);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"paycheckDistributionMethod".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("Paycheck Distribution method should be present in the MissingPropertiesList");
				}
			}
		}
	}
	
	

	/**
	 * <P>Verifies if the worker is ready to pay without the Live State property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers live State to null  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the live state missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNullLiveState()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the live state to null
		PhysicalAddress address = worker.getTaxResidenceAddress();
		address.setStateOrRegion(null);
		worker.setTaxResidenceAddress(address);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"liveState".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("Live State should be present in the MissingPropertiesList");
				}
			}
		}
	}
	
	 /** <P>Verifies if the worker is ready to pay without the live address </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers tax residence address to null  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the live state missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNullLiveAddress()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the address to null
		worker.setTaxResidenceAddress(null);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"liveState".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("Live State should be present in the MissingPropertiesList");
				}
			}
		}
	}
	
	/**
	 * <P>Verifies if the worker is ready to pay without the Live State property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers live State to empty string  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the live state missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithEmptyLiveState()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the live state to null
		PhysicalAddress address = worker.getTaxResidenceAddress();
		address.setStateOrRegion("");
		worker.setTaxResidenceAddress(address);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"liveState".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("Live State should be present in the MissingPropertiesList");
				}
			}
		}
	}

	/**
	 * <P>Verifies if the worker is ready to pay without the Last Name property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers last name to  null  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the Last Name missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNullLastName() throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update last name to null
		worker.setLastName(null);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"lastName".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("Last Name should be present in the MissingPropertiesList");
				}
			}
		}
	}
	
	/**
	 * <P>Verifies if the worker is ready to pay without the Last Name property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers last name to  empty string  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the Last Name missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithEmptyLastName() throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update last name to null
		worker.setLastName("");

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"lastName".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("Last Name should be present in the MissingPropertiesList");
				}
			}
		}
	}

	/**
	 * <P>Verifies if the worker is ready to pay without the Work State property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers work State to null  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the workState missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNullWorkState()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the live state to null
		WorkLocation workLocation = worker.getWorkLocation();
		PhysicalAddress address = workLocation.getAddress();
		address.setStateOrRegion(null);
		workLocation.setAddress(address);
		worker.setWorkLocation(workLocation);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"workState".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("workState should be present in the MissingPropertiesList");
				}
			}
		}
	}
	
	/**
	 * <P>Verifies if the worker is ready to pay with empty work state </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers work state to empty string <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the workState missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithEmptyWorkState()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the live state to null
		WorkLocation workLocation = worker.getWorkLocation();
		PhysicalAddress address = workLocation.getAddress();
		address.setStateOrRegion("");
		workLocation.setAddress(address);
		worker.setWorkLocation(workLocation);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"workState".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("workState should be present in the MissingPropertiesList");
				}
			}
		}
	}
	
	/**
	 * <P>Verifies if the worker is ready to pay with empty work location </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers WorkLocation to null <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the workState missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNullWorklocation()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the work location to null
		worker.setWorkLocation(null);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"workState".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("workState should be present in the MissingPropertiesList");
				}
			}
		}
	}

	/**
	 * <P>Verifies if the worker is ready to pay without the Hire Date property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings,compensation</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers Hire Date to null  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify that the missing properties list includes the Hire Date missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNoHireDate() throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the hire date to null
		worker.setHireDate(null);

		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"hiredate".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("hiredate should be present in the MissingPropertiesList");
				}
			}
		}
	}

	/**
	 * <P>Verifies if the worker is ready to pay without the FirstName property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers FirstName to null  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify the validation exception is thrown while saving worker with the null first name</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test(expectedExceptions=ValidationException.class)
	public void testOfferingPayrollReadyWorkerWithNullFirstName()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the first name to null
		worker.setFirstName(null);
		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"firstName".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("firstName should be present in the MissingPropertiesList");
				}
			}
		}
	}

	/**
	 * <P>Verifies if the worker is ready to pay without the FirstName property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings</LI>
	 * <LI>Save the worker</LI>
	 * <li>Update the workers FirstName to null  <li>
	 * <li>Save the worker </li>
	 * <Li>Verify the validation exception is thrown while saving worker with the null first name</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test(expectedExceptions=ValidationException.class)
	public void testOfferingPayrollReadyWorkerWithEmptyFirstName()
			throws Exception {
		//Setup 
		Long workerId = createPayrollReadyWorker();
		Worker worker = sWorkerService.getWorkerById(workerId);
		//update the first name to null
		worker.setFirstName("");
		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);

		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"firstName".equalsIgnoreCase(missing)) {
					AssertJUnit.fail("firstName should be present in the MissingPropertiesList");
				}
			}
		}
	}

	/**
	 * <P>Verifies if the worker is ready to pay without wage Items </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a worker, setup the tax residence address, set up work location,withholdings but no compensation</LI>
	 * <LI>Save the worker</LI>
	 * <Li>Verify that the missing properties list includes the wage items missing property</li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */
	@Test
	public void testOfferingPayrollReadyWorkerWithNoWageItems() throws Exception {
		//Setup 
		Worker worker = createWorker();
		worker.setTaxResidenceAddress(PojoUtils.createUSCAPhysicalAddress());
		worker.setWorkLocation(getWorkLocation());
		worker.setPaycheckDistributionMethod(PaycheckDistributionMethod.HANDWRITTEN);
		
		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);
	
		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if (!"wageItems".equalsIgnoreCase(missing)) {
					AssertJUnit
							.fail("wageItems should be present in the MissingPropertiesList");
				}
			}
		}
	}
	/**
	 * <P>Verifies if the all the missing payroll ready property </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a basic worker, </LI>
	 * <LI>Save the worker</LI>
	 * <Li>Verify that the missing properties list </li>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify the worker is not ready for pay as he does not have withholdings</LI>
	 * </UL>
	 */	
	@Test
	public void testAllExceptFirstNameMissingReadyToPayProperties() throws Exception {
		//Setup 
		Worker worker = createWorker();
		worker.setTaxResidenceAddress(null);
		worker.setHireDate(null);
		worker.setWorkLocation(null);
		worker.setPaycheckDistributionMethod(PaycheckDistributionMethod.NOT_SET);
		worker.setLastName("");
		//Exercise
		Worker newWorker = sWorkerService.saveWorker(worker);
		
		//Verify the missing property
		String[] missingProperties = newWorker.getMissingReadyToPayProperties();
		int counter = 0;
		if (missingProperties.length > 0) {
			for (String missing : missingProperties) {
				if ("wageItems".equalsIgnoreCase(missing)) {
					counter++;
				}
				if ("lastName".equalsIgnoreCase(missing)) {
					counter++;
				}
				if ("hiredate".equalsIgnoreCase(missing)) {
					counter++;
				}
				if ("paycheckDistributionMethod".equalsIgnoreCase(missing)) {
					counter++;
				}
				if ("liveState".equalsIgnoreCase(missing)) {
					counter++;
				}
				if ("workState".equalsIgnoreCase(missing)) {
					counter++;
				}
		
			}
		}
		if (counter != 6) {
			AssertJUnit.fail("Payroll Ready Property not populated correctly in the missing property list");
		}
	}
	

}
