package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.benefits.HealthBenefit;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.CompensationUtils;
import com.intuit.sb.payroll.services.wages.SaveCompensationItemOperation;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PaycheckStatus;
import com.intuit.sb.payroll.services.workerpayments.PaycheckSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunState;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 * Tests the createPayrollRun operation of the IPayEmployeeService.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class AdhocPayrollRunTest extends WorkerPaymentServiceTestBase
{
	private String mHourlyRate = "10.00";
	private String mOvertimeRate = "1.5";
	private String mBonusAmount = "1000.00";
	private double mHourlyHours = 80.0;
	private double mOvertimeHours = 8.0;
    
    /**
     * <P>Test adhoc PayrollRun using an hourly worker with hourly, overtime, double-time and bonus items</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up an hourly worker</LI>
     * <LI>Add a double-time compensation item</LI>
     * <LI>Set the PayrollRunType to ADHOC</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() to commit the PayrollRun</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithDoubleTimeHourlyWorker() throws Exception {
    	PayrollRunSummary payrollSummary = null;
    	double doubleTimeHours = 2.0;

    	//Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Tim", sMountainViewLocation);
		worker = WorkerTestUtils.addDoubleTime(worker, true);
		Long doubleTimeWorkerId = worker.getId();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = doubleTimeWorkerId;
		// Setup wage items for the first worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(doubleTimeWorkerId);
		CompensationItem[] updatedWageItems = setupWageItemsForAllWageTypes(wageItems, mHourlyRate, mHourlyHours,
		                                           mOvertimeRate, mOvertimeHours, mBonusAmount, doubleTimeHours);
		sWageService.saveCompensationItems(doubleTimeWorkerId,
		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
		                                                                    updatedWageItems));
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);
    	// Set the PayrollRun type to Adhoc
    	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.ADHOC);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	//Exercise
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	// Setup expected values
    	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
    	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
    	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
    	expectedPayrollRunSummary.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
    	paycheckSummaries[0] = createPaycheckSummary(doubleTimeWorkerId,
    	                                             ""+(mHourlyHours+mOvertimeHours+doubleTimeHours),
    	                                             new BigDecimal("1960.00"), new BigDecimal("1416.70"),
    	                                             new BigDecimal("543.30"), new BigDecimal("0.00"),
    	                                             PaycheckDistributionMethod.HANDWRITTEN,
    	                                             PaycheckStatus.COMMITTED, payrollSummary);
    	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
    	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("1416.70"));
    	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("1960.00"));
    	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("543.30"));
    	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("234.22"));
    	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
    	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
    	
    	//Verify
    	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
    	                                                                         payrollSummary));
    	
    	//TearDown
    	//The entire transaction in this test will be rolled back in BasicTestBase
    }

    /**
     * <P>Test adhoc PayrollRun using an hourly worker with hourly, overtime and bonus items.  Worker also has
     * health care deduction</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up an hourly worker</LI>
     * <LI>Add a health care deduction item</LI>
     * <LI>Set the PayrollRunType to ADHOC</LI>
     * <LI>Change the health care deduction to 0.00</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() to commit the PayrollRun</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithHourlyWorkerAndHealthCareDeduction() throws Exception {
    	PayrollRunSummary payrollSummary = null;

    	//Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Tim", sMountainViewLocation);
		worker = WorkerTestUtils.addHealthBenefit(worker);
		Long workerId = worker.getId();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = workerId;
		// Setup wage items for the first worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(workerId);
		CompensationItem[] updatedWageItems = setupWageItemsForAllWageTypes(wageItems, mHourlyRate, mHourlyHours,
		                                           mOvertimeRate, mOvertimeHours, "0.00", 0.0);
		sWageService.saveCompensationItems(workerId,
		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
		                                                                    updatedWageItems));
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);
    	// Set the PayrollRun type to Adhoc
    	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	// Now change the Health insurance deduction amount to 0.00
    	com.intuit.sb.payroll.services.worker.Worker workerObj = sWorkerService.getWorkerById(workerId);
    	HealthBenefit healthBenefit = workerObj.getHealthBenefits()[0];
    	PaycheckRequest paycheckRequest = payrollRunSaveRequest.getPaycheckRequests()[0];
    	addToCalcInputs(paycheckRequest, healthBenefit, new BigDecimal(0.00), 0.0);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	//Exercise
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	// Setup expected values
    	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
    	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
    	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
    	expectedPayrollRunSummary.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
    	paycheckSummaries[0] = createPaycheckSummary(workerId,
    	                                             ""+(mHourlyHours+mOvertimeHours),
    	                                             new BigDecimal("920.00"), new BigDecimal("741.56"),
    	                                             new BigDecimal("178.44"), new BigDecimal("0.00"),
    	                                             PaycheckDistributionMethod.HANDWRITTEN,
    	                                             PaycheckStatus.COMMITTED, payrollSummary);
    	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
    	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("741.56"));
    	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("920.00"));
    	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("178.44"));
    	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("109.94"));
    	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
    	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
    	
    	//Verify
    	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
    	                                                                         payrollSummary));
    	
    	//TearDown
    	//The entire transaction in this test will be rolled back in BasicTestBase
    }

    /**
     * <P>Test adhoc PayrollRun using an hourly worker with hourly, overtime and bonus items.  Worker also has
     * a fixed commission item</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up an hourly worker</LI>
     * <LI>Add a fixed commission item</LI>
     * <LI>Set the PayrollRunType to ADHOC</LI>
     * <LI>Change the commission amount to 55.00</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() to commit the PayrollRun</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithHourlyWorkerAndCommission() throws Exception {
    	PayrollRunSummary payrollSummary = null;

    	//Setup
		Worker worker = WorkerTestUtils.createAPayrollReadyWorker("Tim", sMountainViewLocation);
		worker = WorkerTestUtils.addCommission(worker, true);
		Long workerId = worker.getId();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = workerId;
		// Setup wage items for the first worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(workerId);
		CompensationItem[] updatedWageItems = setupWageItemsForAllWageTypes(wageItems, mHourlyRate, mHourlyHours,
		                                           mOvertimeRate, mOvertimeHours, "0.00", 0.0);
		sWageService.saveCompensationItems(workerId,
		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
		                                                                    updatedWageItems));
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);
    	// Set the PayrollRun type to Adhoc
    	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	// Now change the commission amount to 55.00
    	CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerId);
    	PaycheckRequest paycheckRequest = payrollRunSaveRequest.getPaycheckRequests()[0];
    	fixUpCalcInputs(paycheckRequest, workerCompensationItems, CompensationType.Commission,
    			new BigDecimal(55.00), 0.0);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	//Exercise
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	// Setup expected values
    	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
    	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
    	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
    	expectedPayrollRunSummary.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
    	paycheckSummaries[0] = createPaycheckSummary(workerId,
    	                                             ""+(mHourlyHours+mOvertimeHours),
    	                                             new BigDecimal("975.00"), new BigDecimal("781.71"),
    	                                             new BigDecimal("193.29"), new BigDecimal("0.00"),
    	                                             PaycheckDistributionMethod.HANDWRITTEN,
    	                                             PaycheckStatus.COMMITTED, payrollSummary);
    	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
    	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("781.71"));
    	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("975.00"));
    	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("193.29"));
    	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("116.52"));
    	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
    	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
    	
    	//Verify
    	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
    	                                                                         payrollSummary));
    	
    	//TearDown
    	//The entire transaction in this test will be rolled back in BasicTestBase
    }

    /**
     * <P>Test adhoc PayrollRun using a worker with a fixed bonus item.</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a no wage worker</LI>
     * <LI>Add a fixed bonus item</LI>
     * <LI>Set the PayrollRunType to ADHOC</LI>
     * <LI>Change the bonus amount to 550.00</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() to commit the PayrollRun</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithWorkerAndBonusOnly() throws Exception {
    	PayrollRunSummary payrollSummary = null;

    	//Setup
		Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Tim", sMountainViewLocation);
		worker = WorkerTestUtils.addBonus(worker, true);
		Long workerId = worker.getId();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = workerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);
    	// Set the PayrollRun type to Adhoc
    	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	// Now change the bonus amount to 550.00
    	CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerId);
    	PaycheckRequest paycheckRequest = payrollRunSaveRequest.getPaycheckRequests()[0];
    	fixUpCalcInputs(paycheckRequest, workerCompensationItems, CompensationType.Bonus,
    			new BigDecimal(550.00), 0.0);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	//Exercise
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	// Setup expected values
    	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
    	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
    	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
    	expectedPayrollRunSummary.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
    	paycheckSummaries[0] = createPaycheckSummary(workerId, "Bonus",
    	                                             new BigDecimal("550.00"), new BigDecimal("469.47"),
    	                                             new BigDecimal("80.53"), new BigDecimal("0.00"),
    	                                             PaycheckDistributionMethod.HANDWRITTEN,
    	                                             PaycheckStatus.COMMITTED, payrollSummary);
    	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
    	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("469.47"));
    	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("550.00"));
    	//NOTE: The employee paid tax amount is $0.01 less than the total amount on the paycheck due to penny rounding.
    	//      The total gross amount is $550.00, and the total medicare amount (Employer+Employee) is $15.95 (2.9%).
    	//      On the paycheck the amount for both employee & employer are $7.98 respectively, hence the rounding.
    	//      I guess QBOE/IOP put the penny adjustment to the employee.
    	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("80.52"));
    	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("65.73"));
    	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
    	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
    	
    	//Verify
    	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
    	                                                                         payrollSummary));
    	
    	//TearDown
    	//The entire transaction in this test will be rolled back in BasicTestBase
    }

    /**
     * <P>Test adhoc PayrollRun using a worker with a fixed PaycheckTips item.</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a no wage worker</LI>
     * <LI>Add a fixed PaycheckTips item</LI>
     * <LI>Set the PayrollRunType to ADHOC</LI>
     * <LI>Change the PaycheckTips amount to 550.00</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() to commit the PayrollRun</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithWorkerAndPaycheckTipsOnly() throws Exception {
    	PayrollRunSummary payrollSummary = null;

    	//Setup
		Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Tim", sMountainViewLocation);
		worker = WorkerTestUtils.addPaycheckTips(worker, true);
		Long workerId = worker.getId();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = workerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);
    	// Set the PayrollRun type to Adhoc
    	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	// Now change the PaycheckTips amount to 550.00
    	CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerId);
    	PaycheckRequest paycheckRequest = payrollRunSaveRequest.getPaycheckRequests()[0];
    	fixUpCalcInputs(paycheckRequest, workerCompensationItems, CompensationType.TipsPaycheck,
    			new BigDecimal(550.00), 0.0);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	//Exercise
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	// Setup expected values
    	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
    	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
    	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
    	expectedPayrollRunSummary.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
    	paycheckSummaries[0] = createPaycheckSummary(workerId, "PaycheckTips",
    	                                             new BigDecimal("550.00"), new BigDecimal("469.47"),
    	                                             new BigDecimal("80.53"), new BigDecimal("0.00"),
    	                                             PaycheckDistributionMethod.HANDWRITTEN,
    	                                             PaycheckStatus.COMMITTED, payrollSummary);
    	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
    	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("469.47"));
    	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("550.00"));
    	//NOTE: The employee paid tax amount is $0.01 less than the total amount on the paycheck due to penny rounding.
    	//      The total gross amount is $550.00, and the total medicare amount (Employer+Employee) is $15.95 (2.9%).
    	//      On the paycheck the amount for both employee & employer are $7.98 respectively, hence the rounding.
    	//      I guess QBOE/IOP put the penny adjustment to the employee.
    	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("80.52"));
    	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("65.73"));
    	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
    	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
    	
    	//Verify
    	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
    	                                                                         payrollSummary));
    	
    	//TearDown
    	//The entire transaction in this test will be rolled back in BasicTestBase
    }

    /**
     * <P>Test adhoc PayrollRun using a worker with a fixed CashTips item.</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a no wage worker</LI>
     * <LI>Add an hourly item</LI>
     * <LI>Add a fixed CashTips item</LI>
     * <LI>Set the PayrollRunType to ADHOC</LI>
     * <LI>Change the CashTips amount to 550.00</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() to commit the PayrollRun</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    @Test
    public void testCommitPayrollRunWithWorkerAndCashTips() throws Exception {
    	PayrollRunSummary payrollSummary = null;

    	//Setup
		Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Tim", sMountainViewLocation);
		worker = WorkerTestUtils.addHourlyWage(worker, null, false);
		worker = WorkerTestUtils.addCashTips(worker, true);
		Long workerId = worker.getId();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = workerId;
		// Setup wage items for the first worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(workerId);
		CompensationItem[] updatedWageItems = setupWageItemsForAllWageTypes(wageItems, mHourlyRate, mHourlyHours,
		                                           mOvertimeRate, 0.0, "0.00", 0.0);
		sWageService.saveCompensationItems(workerId,
		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
		                                                                    updatedWageItems));
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);
    	// Set the PayrollRun type to Adhoc
    	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	// Now change the CashTips amount to 550.00
    	CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerId);
    	PaycheckRequest paycheckRequest = payrollRunSaveRequest.getPaycheckRequests()[0];
    	fixUpCalcInputs(paycheckRequest, workerCompensationItems, CompensationType.TipsCash,
    			new BigDecimal(550.00), 0.0);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	//Exercise
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	// Setup expected values
    	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
    	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
    	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
    	expectedPayrollRunSummary.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
    	paycheckSummaries[0] = createPaycheckSummary(workerId, ""+mHourlyHours,
    	                                             new BigDecimal("1350.00"), new BigDecimal("499.37"),
    	                                             new BigDecimal("300.63"), new BigDecimal("0.00"),
    	                                             PaycheckDistributionMethod.HANDWRITTEN,
    	                                             PaycheckStatus.COMMITTED, payrollSummary);
    	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
    	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("499.37"));
    	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("1350.00"));
    	//NOTE: The employee paid tax amount is $0.01 less than the total amount on the paycheck due to penny rounding.
    	//      The total gross amount is $1350.00, and the total medicare amount (Employer+Employee) is $39.15 (2.9%).
    	//      On the paycheck the amount for both employee & employer are $19.58 respectively, hence the rounding.
    	//      I guess QBOE/IOP put the penny adjustment to the employee.
    	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("300.62"));
    	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("161.33"));
    	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
    	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
    	
    	//Verify
    	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
    	                                                                         payrollSummary));
    	
    	//TearDown
    	//The entire transaction in this test will be rolled back in BasicTestBase
    }
    
    /**
     * <P>Test adhoc PayrollRun using a worker with a fixed Piecework item.</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a no wage worker</LI>
     * <LI>Add an hourly item</LI>
     * <LI>Add a fixed Piecework item</LI>
     * <LI>Set the PayrollRunType to ADHOC</LI>
     * <LI>Change the Piecework amount to $0.50/piece</LI>
     * <LI>Call WorkerPaymentService's commitPayrollRun() to commit the PayrollRun</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunSummary returned is correct</LI>
     * </UL>
     */
    //TODO Enable this test when we support overrides for fixed Piecework item
    @Test(enabled=false, groups={"broken"})
    public void testCommitPayrollRunWithWorkerAndPiecework() throws Exception {
    	PayrollRunSummary payrollSummary = null;

    	//Setup
		Worker worker = WorkerTestUtils.createANoWagePayrollReadyWorker("Tim", sMountainViewLocation);
		worker = WorkerTestUtils.addHourlyWage(worker, null, false);
        // Add a piecework item
        worker = WorkerTestUtils.addPiecework(worker, true);
		Long workerId = worker.getId();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = workerId;
		// Setup wage items for the first worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(workerId);
		CompensationItem[] updatedWageItems = setupWageItemsForAllWageTypes(wageItems, mHourlyRate, mHourlyHours,
		                                           mOvertimeRate, 0.0, "0.00", 0.0);
		sWageService.saveCompensationItems(workerId,
		                 CompensationUtils.mapToSaveCompensationItemRequest(SaveCompensationItemOperation.UPDATE,
		                                                                    updatedWageItems));
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, null);
    	// Set the PayrollRun type to Adhoc
    	payrollRunSaveRequest.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	// Now change the Piecework amount to $0.50/piece
    	CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerId);
    	PaycheckRequest paycheckRequest = payrollRunSaveRequest.getPaycheckRequests()[0];
    	fixUpCalcInputs(paycheckRequest, workerCompensationItems, CompensationType.Piecework,
    			new BigDecimal(0.50), 1100.0);

        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	//Exercise
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	// Setup expected values
    	PayrollRunSummary expectedPayrollRunSummary = new PayrollRunSummary();
    	expectedPayrollRunSummary.setPayrollRunId(payrollSummary.getPayrollRunId());
    	expectedPayrollRunSummary.setPayrollPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	expectedPayrollRunSummary.setPayrollRunState(PayrollRunState.COMPLETE);
    	expectedPayrollRunSummary.setPayrollRunType(PayrollRunTypeEnum.ADHOC);
    	PaycheckSummary[] paycheckSummaries = new PaycheckSummary[1];
    	paycheckSummaries[0] = createPaycheckSummary(workerId, ""+mHourlyHours,
    	                                             new BigDecimal("1350.00"), new BigDecimal("1049.37"),
    	                                             new BigDecimal("300.63"), new BigDecimal("0.00"),
    	                                             PaycheckDistributionMethod.HANDWRITTEN,
    	                                             PaycheckStatus.COMMITTED, payrollSummary);
    	expectedPayrollRunSummary.setPaycheckSummaries(paycheckSummaries);
    	expectedPayrollRunSummary.setTotalEmployeeNet(new BigDecimal("1049.37"));
    	expectedPayrollRunSummary.setTotalEmployeeGross(new BigDecimal("1350.00"));
    	//NOTE: The employee paid tax amount is $0.01 less than the total amount on the paycheck due to penny rounding.
    	//      The total gross amount is $1350.00, and the total medicare amount (Employer+Employee) is $39.15 (2.9%).
    	//      On the paycheck the amount for both employee & employer are $19.58 respectively, hence the rounding.
    	//      I guess QBOE/IOP put the penny adjustment to the employee.
    	expectedPayrollRunSummary.setTotalEmployeePaidTax(new BigDecimal("300.62"));
    	expectedPayrollRunSummary.setTotalEmployerPaidTax(new BigDecimal("161.33"));
    	expectedPayrollRunSummary.setTotalEmployeePaidBenefits(new BigDecimal("0.00"));
    	expectedPayrollRunSummary.setTotalEmployerPaidBenefits(new BigDecimal("0.00"));
    	
    	//Verify
    	AssertJUnit.assertTrue("PayrollRunSummary mismatch!", verifyPayrollRunSummary(expectedPayrollRunSummary,
    	                                                                         payrollSummary));
    	
    	//TearDown
    	//The entire transaction in this test will be rolled back in BasicTestBase
    }
    
    //TODO  Add test for Salary

    private void addToCalcInputs(PaycheckRequest paycheckRequest, HealthBenefit healthBenefit, BigDecimal rate,
    		double units) {
    	CalcInputs[] calcInputs = paycheckRequest.getCalcInputs();
		List<CalcInputs> inputList = new ArrayList<CalcInputs>();
		for (CalcInputs calcInput : calcInputs) {
			inputList.add(calcInput);
		}
		CalcInputs calcInput = new CalcInputs();
		calcInput.setUnits(units);
		calcInput.setRate(rate);
		calcInput.setPayItemObjectHandle(healthBenefit.getObjectHandle());
		inputList.add(calcInput);
		paycheckRequest.setCalcInputs(inputList.toArray(new CalcInputs[inputList.size()]));
    }
    
    private void fixUpCalcInputs(PaycheckRequest paycheckRequest, CompensationItem[] workerCompensationItems,
    		CompensationType compensationType, BigDecimal rate, double units) {
    	boolean found = false;
    	CalcInputs[] calcInputs = paycheckRequest.getCalcInputs();
		for (CompensationItem compensationItem : workerCompensationItems) {
			if (compensationType.equals(compensationItem.getCompensationType())) {
				for (CalcInputs calcInput : calcInputs) {
					if (calcInput.getPayItemObjectHandle().equals(compensationItem.getObjectHandle())) {
						calcInput.setUnits(units);
						calcInput.setRate(rate);
						found = true;
						break;
					}
				}
			}
		}
		AssertJUnit.assertTrue("Cannot find compensation type: " + compensationType.toString() + "!", found);
    }
}
