package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;


/**
 *  <P>The <CODE>CalcPaycheckDetailsExceptionsTest</CODE> class contains tests for testing exceptions that are thrown
 *     by IPayEmployeeService.calcPaycheckDetails()</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class CalcPaycheckDetailsExceptionsTest extends WorkerPaymentServiceTestBase {
    private String mHourlyRate = "10.00";
    private String mOvertimeRate = "15.00";
    private String mBonusAmount = "1000.00";
    private double mHourlyHours = 80.0;
    private double mOvertimeHours = 8.0;
    private static CompensationItem[] wageItems = null;

    @BeforeClass
    public static void setUpCalcPaycheckDetailsExceptionsTest() {
        setUpSanMateoAddress();
        // Set up Company LegalStructure
        sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
        sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
        sEmployer.setNonProfit(false);
    }
    
    @BeforeMethod
    public void setUp() throws Exception {
        super.setUp();
        try {
            refreshWorker();
            // Set up wages
            setupWages(sWorkerId);
            // Apply the applicable taxes
            setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
            applyApplicableTaxes();
            // Set up the withholding info
            setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax, 
            		W4WithholdingStatusEnum.MARRIED, 1, new BigDecimal("0.00"));
            setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax, 
                                 DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, new BigDecimal("0.00"));
            
            // Get the wage items for this worker
            wageItems = sWageService.getCompensationItems(sWorkerId);
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            AssertJUnit.fail("Error in setUp() in CalcPaycheckDetailsExceptionsTest(): " + t.getMessage());
        }
    }
    
    @AfterMethod
    public void tearDown() throws Exception {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
            // Remove all withholding information associated with the worker
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
        }
        super.tearDown();
    }
    
    /**
     * <P>Test calling calcPaycheckDetails() with a null CalcRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method with a null CalcRequest</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullCalcRequest() throws Exception {
        @SuppressWarnings("unused")
        PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails((PaycheckRequest)null);
    }

    /**
     * <P>Test calling calcPaycheckDetails() with a null UniqueId in the CalcRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object with a null UniqueId</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullUniqueId() throws Exception {
	    PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
	    PaycheckRequest calcRequest = new PaycheckRequest();
	    calcRequest.setWorkerId(null);
	    CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, 
	    		mOvertimeRate, mOvertimeHours, mBonusAmount);
	    CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
	    calcRequest.setPayrollRunPeriod(payrollRunPeriod);
	    
	    @SuppressWarnings("unused")
	    PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
    }

    /**
     * <P>Test calling calcPaycheckDetails() without setting the UniqueId value in the CalcRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object without setting the UniqueId</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testUniqueIdNotSet() throws Exception {
        PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
        PaycheckRequest calcRequest = new PaycheckRequest();
	    CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, 
	    		mOvertimeRate, mOvertimeHours, mBonusAmount);
	    CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
        calcRequest.setPayrollRunPeriod(payrollRunPeriod);
        
        @SuppressWarnings("unused")
        PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
    }

    /**
     * <P>Test calling calcPaycheckDetails() without setting the PayrollScheduleFrequency value in the CalcRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object without setting the PayrollScheduleFrequency</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test (expectedExceptions=ValidationException.class)
    public void testPayrollScheduleFrequencyNotSet() throws Exception {
        PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
        payrollRunPeriod.setFrequency(null);
        PaycheckRequest calcRequest = new PaycheckRequest();
        calcRequest.setWorkerId(sWorkerId);
	    CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, 
	    		mOvertimeRate, mOvertimeHours, mBonusAmount);
	    CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
        calcRequest.setPayrollRunPeriod(payrollRunPeriod);

        @SuppressWarnings("unused")
        PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
    }

    /**
     * <P>Test calling calcPaycheckDetails() without setting the PayrollRunStartDate value in the CalcRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object without setting the PayrollRunStartDate</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test
    public void testPayrollRunStartDateNotSet() throws Exception {
        try {
            PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
            payrollRunPeriod.setPeriodStartDate(null);
            PaycheckRequest calcRequest = new PaycheckRequest();
            calcRequest.setWorkerId(sWorkerId);
    	    CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, 
    	    		mOvertimeRate, mOvertimeHours, mBonusAmount);
    	    CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
    		calcRequest.setCalcInputs(calcInputs);
            calcRequest.setPayrollRunPeriod(payrollRunPeriod);

            @SuppressWarnings("unused")
            PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
            AssertJUnit.fail("Expected Exception is not thrown!\n");
        } catch (ValidationException ex) {
            AssertJUnit.assertEquals("periodStartDate is null", ex.getMessage());
        }
    }

    /**
     * <P>Test calling calcPaycheckDetails() without setting the PayrollRunEndDate value in the CalcRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object without setting the PayrollRunEndDate</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test
    public void testPayrollRunEndDateNotSet() throws Exception {
        try {
            PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
            payrollRunPeriod.setPeriodEndDate(null);
            PaycheckRequest calcRequest = new PaycheckRequest();
            calcRequest.setWorkerId(sWorkerId);
    	    CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, 
    	    		mOvertimeRate, mOvertimeHours, mBonusAmount);
    	    CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
    		calcRequest.setCalcInputs(calcInputs);
            calcRequest.setPayrollRunPeriod(payrollRunPeriod);

            @SuppressWarnings("unused")
            PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
            AssertJUnit.fail("Expected Exception is not thrown!\n");
        } catch (ValidationException ex) {
            AssertJUnit.assertEquals("periodEndDate is null", ex.getMessage());
        }
    }

    /**
     * <P>Test calling calcPaycheckDetails() without setting the PaycheckDate value in the CalcRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object without setting the PaycheckDate</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown</LI>
     * </UL>
     */
    @Test
    public void testPaycheckDateNotSet() throws Exception {
        try {
            PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
            payrollRunPeriod.setPayDate(null);
            PaycheckRequest calcRequest = new PaycheckRequest();
            calcRequest.setWorkerId(sWorkerId);
    	    CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, 
    	    		mOvertimeRate, mOvertimeHours, mBonusAmount);
    	    CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
    		calcRequest.setCalcInputs(calcInputs);
            calcRequest.setPayrollRunPeriod(payrollRunPeriod);

            @SuppressWarnings("unused")
            PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
            AssertJUnit.fail("Expected Exception is not thrown!\n");
        }
        catch (ValidationException ex)
        {
            AssertJUnit.assertEquals("paydate is null", ex.getMessage());
        }
    }

    /**
     * <P>Test calling calcPaycheckDetails() with paycheck start date after the paycheck end date</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object with the Paycheck StartDate after the EndDate</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testPaycheckStartDateAfterEndDate() throws Exception {
        PaycheckRequest calcRequest = new PaycheckRequest();
        calcRequest.setWorkerId(sWorkerId);
        PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
        // Purposely swap the begin and end dates so that the begin date is after the end date
        payrollRunPeriod.setPeriodStartDate(mPeriodEndDate);
        payrollRunPeriod.setPeriodEndDate(sPeriodStartDate);
	    CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, 
	    		mOvertimeRate, mOvertimeHours, mBonusAmount);
	    CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
        calcRequest.setPayrollRunPeriod(payrollRunPeriod);

        @SuppressWarnings("unused")
        PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
    }

    /**
     * <P>Test calling calcPaycheckDetails() without setting the WageItems value in the CalcRequest</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a CalcRequest object without setting the WageItems</LI>
     * <LI>Call IPayEmployeeService's calcPaycheckDetails method to perform the actual work</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testWageItemsNotSet() throws Exception {
	    PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
	    PaycheckRequest calcRequest = new PaycheckRequest();
	    calcRequest.setWorkerId(sWorkerId);
	    calcRequest.setPayrollRunPeriod(payrollRunPeriod);
	
	    @SuppressWarnings("unused")
	    PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
    }

}
