package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;
import java.util.Date;

import javax.transaction.Status;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.PayrollFrequencyEnum;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDistributionMethod;
import com.intuit.sb.payroll.services.workerpayments.PaycheckLineItem;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollItemCategoryType;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;


/**
 * <P>
 * The <CODE>CalcPaycheckDetailsSalaryYTDTest</CODE> class contains tests for testing
 * the YTD paycheck calculation functionality of WorkerPaymentService.calcPaycheckDetails using a Salaried worker()
 * </P>
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class CalcPaycheckDetailsSalaryYTDTest extends WorkerPaymentServiceTestBase {
	private static BigDecimal sZeroBigDecimal = new BigDecimal("0.00");
	private static CompensationItem notACompensationItem = null;
	private String mBonusAmount = "1000.00";

	@BeforeClass
	public static void setUpCalcPaycheckDetailsSalaryYTDTest() {
		setUpSanMateoAddress();
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);
	}

	@BeforeMethod
	public void setUp() throws Exception {
		super.setUp();
		try {
			refreshWorker();
			// Set up wages
			setupSalaryAndBonusWages(sWorkerId);
			// Apply the applicable taxes
			setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
			applyApplicableTaxes();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in setUp() in CalcPaycheckDetailsSalaryYTDTest(): " + t.getMessage());
		}
	}

	@AfterMethod
	public void tearDown() throws Exception {
		if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
			// Remove all withholding information associated with the worker
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}

		super.tearDown();
	}
	
    private PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Date periodEndDate,
    				PayrollFrequencyEnum frequency, String bonusAmount) throws Exception {
        PayrollRunSaveRequest saveRequest = new PayrollRunSaveRequest();
        PaycheckRequest[] paycheckRequests = new PaycheckRequest[workerIds.length];
        PayrollRunPeriod payrollRunPeriod = createAPayrollRunPeriod(periodEndDate,
                                                                    frequency, sDefaultPayrollDayOffset,
                                                                    sDefaultPayrollRunDayOffset);

        for(int count = 0;count < workerIds.length;count++) {
            paycheckRequests[count] = new PaycheckRequest();
            CompensationItem[] workerCompensationItems = sWageService.getCompensationItems(workerIds[count]);
            for (CompensationItem compensationItem : workerCompensationItems) {
				if (compensationItem.getCompensationType().equals(CompensationType.Bonus)) {
					compensationItem.setRate(new BigDecimal(bonusAmount));
				}
			}
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(workerCompensationItems);
			paycheckRequests[count].setCalcInputs(calcInputs);
            paycheckRequests[count].setWorkerId(workerIds[count]);
            paycheckRequests[count].setDistributionMethod(PaycheckDistributionMethod.PRINT);
            paycheckRequests[count].setPayrollRunPeriod(payrollRunPeriod);
        }
        
        saveRequest.setPayrollRunPeriod(payrollRunPeriod);
        saveRequest.setPaycheckRequests(paycheckRequests);
        saveRequest.setPayrollRunType(PayrollRunTypeEnum.SCHEDULED);
        
        return saveRequest;     
    }
    
//    private PayrollRunSaveRequest createPayrollRunSaveRequest(Long[] workerIds, Date periodEndDate,
//    				PayrollFrequencyEnum frequency) throws Exception {
//    	return createPayrollRunSaveRequest(workerIds, periodEndDate, frequency, mBonusAmount);
//    }
    
	/**
	 * <P>
	 * Test calculating 3 biweekly paychecks for a worker, withholding of Single 1.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first 2 paychecks by committing payroll runs</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to calculate the third paycheck</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweekly_3Paychecks() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
    	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

		// Now calculate the third paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, "100");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2638.46"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1805.75"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("7615.38"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("100.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("300.00"),
		                                     new BigDecimal("100.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("163.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("490.75")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("163.58"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("490.75")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("38.25"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("114.77")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("38.25"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("114.77")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("465.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1395.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("13.78"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("144.78"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("434.34")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("21.10"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("63.32")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("58.58"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.72"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating 4 biweekly paychecks for a worker, withholding of Single 1.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first 3 paychecks by committing payroll runs</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to calculate the third paycheck</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweekly_4Paychecks() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
    	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();

		// Now calculate the third paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
    	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

		// Now calculate the fourth paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, "100");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2638.46"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1805.72"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("10153.84"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("100.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("400.00"),
		                                     new BigDecimal("100.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("163.59"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("654.34")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("163.59"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("654.34")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("38.26"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("153.03")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("38.26"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("153.03")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("465.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1860.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("144.78"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("579.12")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("21.11"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("84.43")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating 7 biweekly paychecks for a worker so the 7th paycheck is in the next quarter, withholding of
	 * Single 1.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first 6 paychecks by committing payroll runs</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to calculate the third paycheck</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweekly_7Paychecks_CrossingQuarter() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
    	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();

		// Now calculate the third paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
    	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();

		// Now calculate the fourth paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
    	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();

		// Now calculate the fifth paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
    	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();

		// Now calculate the sixth paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
    	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
    	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
    	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

		// Now calculate the seventh paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, "100");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2638.46"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1805.75"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("17769.22"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("100.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("700.00"),
		                                     new BigDecimal("100.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("163.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("1145.09")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("163.58"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("1145.09")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("38.25"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("267.80")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("38.25"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("267.80")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("465.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("3255.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("144.78"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("1013.46")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("21.10"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("147.75")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating 25 biweekly paychecks for a worker, withholding of Single 1.  There are only 25 paychecks
	 * because the 26th paycheck would have a paycheck date of 01/01/2009 which is in the next year.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first 24 paychecks by committing payroll runs</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to calculate the 25th paycheck</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweekly_25Paychecks_ForEntireYear() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          "100");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
		
		PayrollRunPeriod newPayrollRunPeriod = null;
		// Now calculate the next 23 paycheck
		for (int i = 0; i < 23; i++) {
    		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
        	payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, newPayrollRunPeriod.getPeriodEndDate(),
        	                                                    PayrollFrequencyEnum.BIWEEKLY, "100");
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
        	payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
		}
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

		// Now calculate the 25th paycheck
		newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, "100");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2638.46"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1805.73"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("63461.50"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("100.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("2500.00"),
		                                     new BigDecimal("100.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("163.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("4089.61")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("163.58"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("4089.61")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("38.26"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("956.44")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("38.26"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("956.44")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("465.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("11625.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("144.78"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("3619.50")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("21.11"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("527.69")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker, withholding of Single 1 containing Ytd values, limit reached
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithYtd_LimitReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.WEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.WEEKLY,
    	                                                                          "120000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("2269.23"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("1585.58"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("1269.23"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("2538.46"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("121000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("32.91"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1791.31")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("32.91"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1791.31")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("490.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("42486.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("160.74"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("12408.19")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit for
	 * FUTA, CASUI and CAETT already reached.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithYtd_FUTA_CASUI_CAETT_LimitAlreadyReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          "5000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("3538.46"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2310.97"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("5076.92"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("6000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("219.39"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("686.77")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("219.39"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("686.77")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("51.31"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("160.62")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("51.31"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("160.62")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("700.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("2570.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("228.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("828.96")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("28.31"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("88.62")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit for
	 * FUTA, CASUI and CAETT just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithYtd_FUTA_CASUI_CAETT_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          "3000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("3538.46"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2310.97"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("5076.92"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("4000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("219.39"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("562.77")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("219.39"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("562.77")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("51.31"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("131.62")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("51.31"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("131.62")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("700.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("1960.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("11.69"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("228.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("642.96")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("28.31"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("72.62")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("49.69"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.46"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values, $102,000 limit
	 * for Social Security just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithYtd_SocialSecurity_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          "98000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("3538.46"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2468.05"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("5076.92"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("99000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("90.62"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("90.62"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("51.31"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1509.12")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("51.31"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1509.12")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("700.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("34992.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("228.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("10097.37")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single 1 containing Ytd values, $86,698 limit
	 * for CASDI just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithYtd_CASDI_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.BIWEEKLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.BIWEEKLY,
    	                                                                          "82000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("3538.46"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2322.01"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("5076.92"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("83000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("219.39"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("5460.77")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("219.39"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("5460.77")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("51.31"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1277.12")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("51.31"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1277.12")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("700.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("29392.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("228.48"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("8449.37")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("17.27"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semimonthly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit
	 * for FUTA, CASUI and CAETT just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithYtd_FUTA_CASUI_CAETT_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.SEMIMONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.SEMIMONTHLY,
    	                                                                          "3000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("3750.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2457.36"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2750.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("5500.00"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("4000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("232.50"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("589.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("232.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("589.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("54.37"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("137.75")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("54.37"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("137.75")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("736.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("2032.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("10.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("239.77"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("665.54")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("30.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("76.00")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("42.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.25"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semimonthly paycheck for a worker, withholding of Single 1 containing Ytd values, $102,000 limit
	 * for Social Security just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithYtd_SocialSecurity_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.SEMIMONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.SEMIMONTHLY,
    	                                                                          "98000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("3750.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2642.36"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2750.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("5500.00"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("99000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("77.50"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("77.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("54.37"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1515.25")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("54.37"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1515.25")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("736.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("35027.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("239.77"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("10089.90")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semimonthly paycheck for a worker, withholding of Single 1 containing Ytd values, $86,698 limit
	 * for CASDI just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithYtd_CASDI_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.SEMIMONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.SEMIMONTHLY,
    	                                                                          "82000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("3750.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("2471.78"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2750.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("5500.00"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("83000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("232.50"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("5487.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("232.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("5487.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("54.37"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1283.25")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("54.37"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1283.25")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("736.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("29427.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("239.77"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("8441.90")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1 containing Ytd values, $7000 limit
	 * for FUTA, CASUI and CAETT just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithYtd_FUTA_CASUI_CAETT_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.MONTHLY,
    	                                                                          "0");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("6500.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("4360.22"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("5500.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("11000.00"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("403.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("744.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("403.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("744.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("94.25"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("174.00")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("94.25"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("174.00")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("1204.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("2158.00")),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("12.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("386.53"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("680.06")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("52.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("96.00")),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("51.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.50"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1 containing Ytd values, $102,000 limit
	 * for Social Security just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithYtd_SocialSecurity_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.MONTHLY,
    	                                                                          "93000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("6500.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("4598.22"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("5500.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("11000.00"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("94000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("217.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("217.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("6324.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("94.25"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1522.50")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("94.25"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1522.50")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("1204.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("33736.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("386.53"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("9477.80")),
		                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1 containing Ytd values, $86,698 limit
	 * for CASDI just reached in current paycheck.
	 * 
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Calculate the first paycheck by committing a payroll run</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithYtd_CASDI_LimitJustReached() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
		                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

		// Set up paycheck related dates
		setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);
		
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
		
		// Calculate the first paycheck
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sWorkerId;
    	PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds, mPeriodEndDate,
    	                                                                          PayrollFrequencyEnum.MONTHLY,
    	                                                                          "80000");
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
		@SuppressWarnings("unused")
    	PayrollRunSummary payrollSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();
		
		// Now calculate the second paycheck
		PayrollRunPeriod newPayrollRunPeriod = calculateNextPayrollRunPeriod(payrollRunSaveRequest.getPayrollRunPeriod());
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, "", 0.0, "", 0.0, "500.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		
		calcRequest.setPayrollRunPeriod(newPayrollRunPeriod);
		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		
		AssertJUnit.assertEquals("Gross total", new BigDecimal("6000.00"), paycheckDetails.getGrossTotal());
		AssertJUnit.assertEquals("Net total", new BigDecimal("4112.39"), paycheckDetails.getNetTotal());
		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("5500.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sSalaryWageDescStr,
		                                                          CompensationType.Salary),
		                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("11000.00"),
		                                     WorkerTestUtils.sSalaryAmount, 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("500.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("80500.00"),
		                                     new BigDecimal("500.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("372.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId, new BigDecimal("5673.00")),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("372.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId, new BigDecimal("5673.00")),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("87.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId, new BigDecimal("1326.75")),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("87.00"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId, new BigDecimal("1326.75")),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("1079.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId, new BigDecimal("29061.00")),
		                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId, new BigDecimal("56.00")),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("340.03"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId, new BigDecimal("8092.30")),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("9.58"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId, new BigDecimal("693.58")),
		                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId, new BigDecimal("238.00")),
		                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId, new BigDecimal("7.00")) };

		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

}
