package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;

import javax.transaction.Status;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.entity.model.ComplianceDocIdEnum;
import com.intuit.sb.payroll.entity.model.DE4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.IndustryTypeEnum;
import com.intuit.sb.payroll.entity.model.LegalStructureEnum;
import com.intuit.sb.payroll.entity.model.RelationshipToOwnerEnum;
import com.intuit.sb.payroll.entity.model.W4WithholdingStatusEnum;
import com.intuit.sb.payroll.entity.model.Worker;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckLineItem;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollItemCategoryType;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 * <P>
 * The <CODE>CalcPaycheckDetailsTest</CODE> class contains tests for testing
 * the paycheck calculation functionality of
 * WorkerPaymentService.calcPaycheckDetails()
 * </P>
 * 
 * $Author$ 
 * $Revision$ 
 * $DateTime$
 * 
 */
public class CalcPaycheckDetailsTest extends WorkerPaymentServiceTestBase {
	private static BigDecimal sZeroBigDecimal = new BigDecimal("0.00");
	private String mHourlyRate = "10.00";
	private String mOvertimeRate = "1.5";
	private String mBonusAmount = "1000.00";
	private double mHourlyHours = 80.0;
	private double mOvertimeHours = 8.0;

	@BeforeClass
	public static void setUpCalcPaycheckDetailsTest() {
		setUpSanMateoAddress();
		// Set up Company LegalStructure
		sEmployer.setIndustryType(IndustryTypeEnum.OTHER);
		sEmployer.setLegalStructure(LegalStructureEnum.CCORP);
		sEmployer.setNonProfit(false);
	}

	@BeforeMethod
	public void setUp() throws Exception {
		super.setUp();
		try {
			refreshWorker();
			// Set up wages
			sWorker = setupWages(sWorker);
			// Apply the applicable taxes
			setUpTestData(sEmployer, sPhysicalAddress, sAge22BirthDate, RelationshipToOwnerEnum.UNRELATED);
			applyApplicableTaxes();
		} catch (Throwable t) {
			sLogger.error(t.getMessage());
			AssertJUnit.fail("Error in setUp() in CalcPaycheckDetailsTest(): " + t.getMessage());
		}
	}

	@AfterMethod
	public void tearDown() throws Exception {
		if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE) {
			// Remove all withholding information associated with the worker
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.W4IRS);
			removeWorkerWithholdingInfoByType(sWorkerId, ComplianceDocIdEnum.DE4);
		}

		super.tearDown();
	}
	
	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker, withholding of Single 1
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithholdSingle1() {
		try {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);
			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.WEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 40.0, mOvertimeRate, 4.0,
			                                                     "500.00");
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);

			calcRequest.setPayrollRunPeriod(payrollRunPeriod);

			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertEquals("Gross total", new BigDecimal("960.00"), paycheckDetails.getGrossTotal());
			AssertJUnit.assertEquals("Net total", new BigDecimal("696.90"), paycheckDetails.getNetTotal());
			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("400.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("400.00"),
			                                     new BigDecimal(mHourlyRate), 40.0),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("60.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("60.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), 4.0),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("500.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("500.00"),
			                                     new BigDecimal("500.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("59.52"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("59.52"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("13.92"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("13.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("143.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("7.68"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("38.98"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("7.68"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("32.64"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.96"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };

			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
		} catch (Exception ex) {
			AssertJUnit.fail("Unexpected type of Exception was thrown!\n" + ex.getMessage());
		}
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdMarried1() {
		try {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                     mOvertimeHours, mBonusAmount);
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);
			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1528.14"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("192.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("37.62"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
		} catch (Exception ex) {
			AssertJUnit.fail("Unexpected type of Exception was thrown!\n" + ex.getMessage());
		}
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker with a fixed commission item, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a fixed commission compensation item to the worker</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithFixedCommissionWithholdMarried1() {
		try {
	        Worker worker = entityService.findById(Worker.class, sWorkerId);
	        // Add a fixed rate commission item
	        worker = WorkerTestUtils.addCommission(worker, true);
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);

			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                     mOvertimeHours, mBonusAmount);
			
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			
			//Remove commission from the inputs or the override will be respected.
			long id = 0;
			for (CompensationItem compensationItem : updatedWageItems) {
				if (compensationItem.getCompensationType().equals(CompensationType.Commission)) {
					id = compensationItem.getObjectHandle().getId();
				}
			}			
			calcInputs = removeInputFromArray(id, calcInputs);

			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);
			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("2420.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1884.49"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                					 new BigDecimal("1000.00"), 0.0),
 			                new PaycheckLineItem(sCommissionWageDescStr, new BigDecimal("500.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sCommissionWageDescStr,
			                                                          CompensationType.Commission),
			                                     QboeLawIdConstants.sCommissionLawId, new BigDecimal("500.00"),
			                                     new BigDecimal("500.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("150.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("150.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("35.09"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("35.09"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("267.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("19.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("64.02"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("19.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("82.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("2.42"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
		} catch (Exception ex) {
			AssertJUnit.fail("Unexpected type of Exception was thrown!\n" + ex.getMessage());
		}
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker with a varies commission item, withholding of Married
	 * 1 for Federal and MarriedWithOneIncome 1 for State
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a varies commission compensation item to the worker</LI>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithVariesCommissionWithholdMarried1() throws Exception {
	        Worker worker = entityService.findById(Worker.class, sWorkerId);
	        // Add a varied rate commission item
	        worker = WorkerTestUtils.addCommission(worker, false);
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.MARRIEDONEINCOME, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                     mOvertimeHours, mBonusAmount);
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);
			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("2920.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("2235.10"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
 			                new PaycheckLineItem(sCommissionWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sCommissionWageDescStr,
			                                                          CompensationType.Commission),
			                                     QboeLawIdConstants.sCommissionLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("181.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("181.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("42.34"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("42.34"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("343.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("23.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("95.16"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("23.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("99.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("2.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Married
	 * 1 for Federal and MarriedWithTwoIncome 1 for State
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdMarried1CAMarriedTwoIncome1() throws Exception {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.MARRIED, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                     mOvertimeHours, mBonusAmount);
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);
			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1487.80"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("192.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Single
	 * 1 for Federal and HeadOfHousehold 1 for State
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdSingle1CAHeadOfHousehold1() throws Exception {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.HEADOFHOUSEHOLD, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                     mOvertimeHours, mBonusAmount);
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);
			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1438.66"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("33.10"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, withholding of Married
	 * using Single 1 for Federal and Single 1 for State
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyWithholdMarriedUsingSingle1CASingle1() throws Exception {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.MARRIEDSINGLERATE, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                     mOvertimeHours, mBonusAmount);
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);

			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1393.80"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("286.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("77.96"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a Weekly paycheck for a worker, withholding exempt for
	 * Federal and exempt for State
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testWeeklyWithholdExempt() throws Exception {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sExemptFromTax,
			                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sExemptFromTax,
			                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.WEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                     mOvertimeHours, mBonusAmount);
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);

			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1757.76"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a semi-monthly paycheck for a worker, withholding of
	 * Single 1
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testSemimonthlyWithholdSingle1() throws Exception {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.SEMIMONTHLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                     mOvertimeHours, mBonusAmount);
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);

			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1416.97"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("270.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("70.79"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a monthly paycheck for a worker, withholding of Single 1
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testMonthlyWithholdSingle1() throws Exception {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

			// Set up paycheck related dates
			setupPaycheckDates(PayrollScheduleFrequency.MONTHLY);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			// Use the CalcRequest constructor
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
			                                                  mOvertimeHours, mBonusAmount);
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);

			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			calcRequest.setCalcInputs(calcInputs);
			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.MONTHLY);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);

			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertEquals("Gross total", new BigDecimal("1920.00"), paycheckDetails.getGrossTotal());
			AssertJUnit.assertEquals("Net total", new BigDecimal("1554.18"), paycheckDetails.getNetTotal());

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
			                                     new BigDecimal(mHourlyRate), mHourlyHours),
			                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
			                                     new BigDecimal("1000.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("179.00"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("24.58"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calling calcPaycheckDetails() with an array of empty WageItems in
	 * the CalcRequest
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Set up a CalcRequest object with an array of empty WageItems</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testEmptyWageItems() throws Exception {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.MONTHLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			calcRequest.setCalcInputs(new CalcInputs[0]);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);

			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), sZeroBigDecimal, "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), sZeroBigDecimal, "Net total"));
			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, sZeroBigDecimal,
			                                     new BigDecimal(mHourlyRate), 0.0),
			                new PaycheckLineItem(sOvertimeWageDescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, sZeroBigDecimal,
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), 0.0),
			                new PaycheckLineItem(sBonusWageDescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, sZeroBigDecimal,
			                                     null, 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a weekly paycheck for a worker with Bonus wage item
	 * only, withholding of Single 1
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBonusOnlyWeeklyWithholdSingle1() throws Exception {
			// Set up the withholding info
			setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
			                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
			setUpDe4WithholdingInfo(sWorkerId, sDE4DocId, ComplianceDocIdEnum.DE4, sNotExemptFromTax,
			                        DE4WithholdingStatusEnum.SINGLE, 1, 0, sZeroBigDecimal);

			// Get the wage items for this worker
			CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

			PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.WEEKLY);
			PaycheckRequest calcRequest = new PaycheckRequest();
			calcRequest.setWorkerId(sWorkerId);
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, 0.0, mOvertimeRate, 0.0,
			                                                     "100.00");
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
			calcRequest.setCalcInputs(calcInputs);
			calcRequest.setPayrollRunPeriod(payrollRunPeriod);

			PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("100.00"), "Gross total"));
			AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("91.55"), "Net total"));

			PaycheckLineItem expectedPaycheckLineItems[] = {
			                new PaycheckLineItem(sHourlyWageDescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sHourlyWageDescStr,
			                                                          CompensationType.Hourly),
			                                     QboeLawIdConstants.sHourlyLawId, sZeroBigDecimal,
			                                     new BigDecimal(mHourlyRate), 0.0),
			                new PaycheckLineItem(sOvertimeWageDescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
			                                                          CompensationType.Overtime),
			                                     QboeLawIdConstants.sOvertimeLawId, sZeroBigDecimal,
			                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), 0.0),
			                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("100.00"),
			                                     PayrollItemCategoryType.COMPENSATION,
			                                     findCompensationItem(wageItems, sBonusWageDescStr,
			                                                          CompensationType.Bonus),
			                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("100.00"),
			                                     new BigDecimal("100.00"), 0.0),
			                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("6.20"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_EELawId),
			                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("6.20"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sSST_ERLawId),
			                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("1.45"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_EELawId),
			                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("1.45"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sMED_ERLawId),
			                new PaycheckLineItem(sFICA_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFICALawId),
			                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("0.80"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sFUTALawId),
			                new PaycheckLineItem(sCAPIT_DescStr, sZeroBigDecimal,
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAPITLawId),
			                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("0.80"),
			                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASDILawId),
			                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("3.40"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCASUILawId),
			                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.10"),
			                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
			                                     QboeLawIdConstants.sCAETTLawId) };
			AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
			                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
			                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}
	
	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, no withholding set on the worker.  Calc engine should use
	 * default withholding of Single 0 for Federal and Single 0 for State
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyNoWithholdingSet() throws Exception {
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		calcRequest.setPayrollRunPeriod(payrollRunPeriod);

		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1357.19"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("319.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("81.57"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, W4 is set to exempt and no DE4.  Calc engine should use
	 * the Federal withholding values for State and hence both Federal and State income taxes are not withheld.
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the W4 withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyW4Exempt() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sExemptFromTax,
		                     W4WithholdingStatusEnum.SINGLE, 1, sZeroBigDecimal);
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
		                                                     0.0, "0.00");
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		calcRequest.setPayrollRunPeriod(payrollRunPeriod);

		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("800.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("732.40"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("0.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("0.00"),
		                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), 0.0),
		                new PaycheckLineItem(sBonusWageDescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, sZeroBigDecimal,
		                                     sZeroBigDecimal, 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("49.60"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("49.60"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("11.60"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("11.60"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId),
		                new PaycheckLineItem(sFICA_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("6.40"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId),
		                new PaycheckLineItem(sCAPIT_DescStr, sZeroBigDecimal,
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("6.40"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("27.20"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.80"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

	/**
	 * <P>
	 * Test calculating a biweekly paycheck for a worker, W4 is set to Married using Single rate, allowance of 1, extra
	 * $15.00 withholding and no DE4.  Calc engine should derive the state withholding value from Federal and the state
	 * withholding will be Single 1 with $15.00 extra withholding.
	 * </P>
	 * 
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Set up the W4 withholding info</LI>
	 * <LI>Set up paycheck related dates</LI>
	 * <LI>Obtain list of wage items</LI>
	 * <LI>Set up a CalcRequest object</LI>
	 * <LI>Call WorkerPaymentService's calcPaycheckDetails method to perform
	 * the actual work</LI>
	 * </OL>
	 * 
	 * <UL>
	 * <LI>Verify the list of paycheck details returned matches what's expected</LI>
	 * </UL>
	 */
	@Test
	public void testBiweeklyW4MarriedSingleRate() throws Exception {
		// Set up the withholding info
		setUpW4WithholdingInfo(sWorkerId, sW4DocId, ComplianceDocIdEnum.W4IRS, sNotExemptFromTax,
		                     W4WithholdingStatusEnum.MARRIEDSINGLERATE, 1, new BigDecimal("15.00"));
		// Get the wage items for this worker
		CompensationItem[] wageItems = sWageService.getCompensationItems(sWorkerId);

		PayrollRunPeriod payrollRunPeriod = createPayrollRunPeriod(PayrollScheduleFrequency.BIWEEKLY);
		PaycheckRequest calcRequest = new PaycheckRequest();
		calcRequest.setWorkerId(sWorkerId);
		CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
															 mOvertimeHours, mBonusAmount);
		CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
		calcRequest.setCalcInputs(calcInputs);
		calcRequest.setPayrollRunPeriod(payrollRunPeriod);

		PaycheckDetails paycheckDetails = sWorkerPayService.calcPaycheckDetails(calcRequest);
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getGrossTotal(), new BigDecimal("1920.00"), "Gross total"));
		AssertJUnit.assertTrue(verifyAmount(paycheckDetails.getNetTotal(), new BigDecimal("1363.80"), "Net total"));

		PaycheckLineItem expectedPaycheckLineItems[] = {
		                new PaycheckLineItem(sHourlyWageDescStr, new BigDecimal("800.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sHourlyWageDescStr,
		                                                          CompensationType.Hourly),
		                                     QboeLawIdConstants.sHourlyLawId, new BigDecimal("800.00"),
		                                     new BigDecimal(mHourlyRate), mHourlyHours),
 		                new PaycheckLineItem(sOvertimeWageDescStr, new BigDecimal("120.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
		                                                          CompensationType.Overtime),
		                                     QboeLawIdConstants.sOvertimeLawId, new BigDecimal("120.00"),
		                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mOvertimeHours),
		                new PaycheckLineItem(sBonusWageDescStr, new BigDecimal("1000.00"),
		                                     PayrollItemCategoryType.COMPENSATION,
		                                     findCompensationItem(wageItems, sBonusWageDescStr,
		                                                          CompensationType.Bonus),
		                                     QboeLawIdConstants.sBonusLawId, new BigDecimal("1000.00"),
		                                     new BigDecimal("1000.00"), 0.0),
		                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_EELawId),
		                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("119.04"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sSST_ERLawId),
		                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_EELawId),
		                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("27.84"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sMED_ERLawId),
		                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("301.00"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFICALawId),
		                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sFUTALawId),
		                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("92.96"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAPITLawId),
		                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("15.36"),
		                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASDILawId),
		                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("65.28"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCASUILawId),
		                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.92"),
		                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
		                                     QboeLawIdConstants.sCAETTLawId) };
		AssertJUnit.assertTrue("Paycheck LineItems mismatch!",
		                  verifyPaycheckLineItems(paycheckDetails.getPaycheckLineItemsList(),
		                                          expectedPaycheckLineItems, "Paycheck Line Items"));
	}

}
