package com.intuit.sb.payroll.qa.services.workerpayments;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSaveRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunSummary;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class CancelPayrollRunByIdTest extends WorkerPaymentServiceTestBase {
    private static Long sReadyForPayWorkerId = null;
	private static int numberOfPayrollReadyWorkers = 1;
	
	@BeforeClass
	public static void setUpCancelPayrollRunByIdTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createPayrollReadyWorker();
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCancelPayrollRunByIdTest(): " + t.getMessage());
        }
	}
	
	@AfterClass
	public static void tearDownCancelPayrollRunByIdTest() {
        try {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCancelPayrollRunByIdTest(): " + t.getMessage());
        }
	}
	
    /**
     * <P>Test whether ServiceFailureException is thrown with invalid payrollRunId</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's cancelPayrollRunById() with an invalid payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the payrollRunId is invalid</LI>
     * </UL>
     */	
	@Test(expectedExceptions=ServiceFailureException.class)
	public void testInvalidCancelPayrollRun() throws Exception {
		//Setup
		//Exercise
			//pass sWorkerId which is invalid
		sWorkerPayService.cancelPayrollRunById(sWorkerId);
		AssertJUnit.fail("ServiceFailureException not thrown");		
		//tear down		
	}
	
    /**
     * <P>Test whether the saved PayrollRun is deleted</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's cancelPayrollRunById() with a saved payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the payrollRun is deleted</LI>
     * </UL>
     */	
	@Test(enabled=false, groups={"broken"})
	public void testCancelSavedPayrollRun() throws Exception {
		PayrollRunSummary payrollRunSummary = null;
		PayrollRunResponse payrollRunResponse = null;
		try {
			//Setup
	    	Long[] workerIds = new Long[numberOfPayrollReadyWorkers];
	    	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
			payrollRunSummary = sWorkerPayService.saveDraftPayrollRun(payrollRunSaveRequest);
			
			//Exercise
			sWorkerPayService.cancelPayrollRunById(payrollRunSummary.getPayrollRunId());
			
			//Verify
			payrollRunResponse = sWorkerPayService.getPayrollRunById(payrollRunSummary.getPayrollRunId());
			AssertJUnit.fail("ServiceFailureException not thrown");
		} catch(ServiceFailureException e) {
			AssertJUnit.assertNull(payrollRunResponse);
		}
		//Tear down - Implicit tear down in tearDownClass()
	}
	
    /**
     * <P>Test whether ValidationException is thrown since the PayrollRun is already committed</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's cancelPayrollRunById() with a committed payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ValidationException is thrown since the PayrollRun is committed and it cannot be deleted</LI>
     * </UL>
     */	
	@Test
	public void testCancelCommittedPayrollRun() throws Exception {
		getUserTransaction().commit();
		PayrollRunSummary payrollRunSummary = null;
		try {
			//Setup
	    	Long[] workerIds = new Long[numberOfPayrollReadyWorkers];
	    	workerIds[0] = sReadyForPayWorkerId;
			PayrollRunSaveRequest payrollRunSaveRequest = createPayrollRunSaveRequest(workerIds,null);
            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunSaveRequest.getPayrollRunPeriod().getPayrollRunDate());
			payrollRunSummary = sWorkerPayService.commitPayrollRun(payrollRunSaveRequest).getPayrollRunSummary();
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();
			
			//Exercise
			sWorkerPayService.cancelPayrollRunById(payrollRunSummary.getPayrollRunId());
			AssertJUnit.fail("ValidationException not thrown");
		} catch(ValidationException e) {
			AssertJUnit.assertTrue(e.getMessage().contains("PayrollRunId:" + payrollRunSummary.getPayrollRunId() + "is already committed"));
		} finally {
			getUserTransaction().begin();
            //Tear down
            if(payrollRunSummary != null) {
                cleanUpPayrollRunItems(new Long(payrollRunSummary.getPayrollRunId()));
            }
    		getUserTransaction().commit();
        }
	}
	
    /**
     * <P>Test whether ValidationException is thrown since the PayrollRunId is null</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's cancelPayrollRunById() with a null payrollRunId</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a {@link ValidationException} is thrown since the PayrollRunId is null</LI>
     * </UL>
     */		
	@Test(expectedExceptions=ValidationException.class)
	public void testNullCancelPayrollRunById() throws Exception {
		//Exercise
		sWorkerPayService.cancelPayrollRunById(null);
	}
}
