package com.intuit.sb.payroll.qa.services.workerpayments;

import org.testng.AssertJUnit;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 * <P>
 * The <CODE>CreatePayrollRunExceptionsTest</CODE> class contains tests for testing the
 * exceptions thrown by createPayrollRun() in WorkerPaymentService
 * </P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class CreatePayrollRunExceptionsTest extends WorkerPaymentServiceTestBase {
    private static Long sReadyForPayWorkerId = null;
	private static int sNumberOfPayrollReadyWorkers = 1;

    @BeforeClass
    public static void setUpCreatePayrollRunExceptionsTest() {
        try {
            getUserTransaction().begin();
            sReadyForPayWorkerId = createMVPayrollReadyWorker("Paul");
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in setUpCreatePayrollRunExceptionsTest(): " + t.getMessage());
        }
    }

    @AfterClass
    public static void tearDownCreatePayrollRunExceptionsTest() {
        try {
            getUserTransaction().begin();
            if (null != sReadyForPayWorkerId) {
                cleanUpEIWorker(sReadyForPayWorkerId);
                sReadyForPayWorkerId = null;
            }
            getUserTransaction().commit();
        } catch (Throwable t) {
            sLogger.error(t.getMessage());
            try {
                getUserTransaction().rollback();
            } catch (Exception e) {
            }
            AssertJUnit.fail("Error in tearDownCreatePayrollRunExceptionsTest(): " + t.getMessage());
        }
    }

    /**
     * <P>Test ValidationException caused by null PayrollRunCreateRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's createPayrollRun() with a null PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that a ServiceFailureException is thrown since the PayrollRunCreateRequest object is null</LI>
     * </UL>
     */
    @Test(expectedExceptions=ServiceFailureException.class)
    public void testNullPayrollRunSaveRequest() throws Exception {
        sWorkerPayService.createPayrollRun(null);
    }

    /**
     * <P>Test createPayrollRun() API with periodStartDate set to null in PayrollRunCreateRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's createPayrollRun() with periodStartDate set to null in PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the periodStartDate is null in PayrollRunCreateRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullPeriodStartDate() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
    	PayrollRunCreateRequest createRequest = createPayrollRunCreateRequest(workerIds);
		PayrollRunPeriod period = createRequest.getPeriod();

		//Set start date to null
		period.setPeriodStartDate(null);
		createRequest.setPeriod(period);

		//Exercise
		sWorkerPayService.createPayrollRun(createRequest);
   }

    /**
     * <P>Test createPayrollRun() API with periodEndDate set to null in PayrollRunCreateRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's createPayrollRun() with periodEndDate set to null in PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the periodEndDate is null in PayrollRunCreateRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullPeriodEndDate() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
    	PayrollRunCreateRequest createRequest = createPayrollRunCreateRequest(workerIds);
		PayrollRunPeriod period = createRequest.getPeriod();

		//Set start date to null
		period.setPeriodEndDate(null);
		createRequest.setPeriod(period);

		//Exercise
		sWorkerPayService.createPayrollRun(createRequest);
   }

    /**
     * <P>Test createPayrollRun() API with payrollRunDate set to null in PayrollRunCreateRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's createPayrollRun() with payrollRunDate set to null in PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the payrollRunDate is null in PayrollRunCreateRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullPayrollRunDate() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
    	PayrollRunCreateRequest createRequest = createPayrollRunCreateRequest(workerIds);
		PayrollRunPeriod period = createRequest.getPeriod();

		//Set start date to null
		period.setPayrollRunDate(null);
		createRequest.setPeriod(period);

		//Exercise
		sWorkerPayService.createPayrollRun(createRequest);
   }

    /**
     * <P>Test createPayrollRun() API with payDate set to null in PayrollRunCreateRequest object</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's createPayrollRun() with payDate set to null in PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that ValidationException is thrown since the payDate is null in PayrollRunCreateRequest</LI>
     * </UL>
     */
    @Test(expectedExceptions=ValidationException.class)
    public void testNullPayDate() throws Exception {
		//Setup
	    Long[] workerIds = new Long[sNumberOfPayrollReadyWorkers];
    	workerIds[0] = sReadyForPayWorkerId;
    	PayrollRunCreateRequest createRequest = createPayrollRunCreateRequest(workerIds);
		PayrollRunPeriod period = createRequest.getPeriod();

		//Set start date to null
		period.setPayDate(null);
		createRequest.setPeriod(period);

		//Exercise
		sWorkerPayService.createPayrollRun(createRequest);
   }

}
