package com.intuit.sb.payroll.qa.services.workerpayments;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.impl.calcengine.QboeLawIdConstants;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollRunTypeEnum;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckLineItem;
import com.intuit.sb.payroll.services.workerpayments.PaycheckStatus;
import com.intuit.sb.payroll.services.workerpayments.PayrollItemCategoryType;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunCreateRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunResponse;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunState;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;

/**
 *  <P>The <CODE>CreatePayrollRunTest</CODE> class contains tests for testing createPayrollRun() in WorkerPaymentService</P>
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class CreatePayrollRunTest extends WorkerPaymentServiceTestBase {
    private static Long sReadyForPayWorkerId = null;
    private static Long sReadyForPayWorkerId2 = null;
	private String mHourlyRate = "40.00";
	private String mOvertimeRate = "1.5";
	private String mTipsAmount = "0.00";
	private double mZeroHours = 0.0;

    /**
     * <P>Test createPayrollRun() using an hourly worker with a fixed rate tips compensation item</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up an hourly worker</LI>
     * <LI>Call WorkerPaymentService's createPayrollRun() with a valid PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify expected results</LI>
     * </UL>
     */
    @Test(groups = {"worker.createPayrollRun"})
    public void testCreatePayrollRunWith1HourlyWorker() throws Exception {
    	//Setup
        sReadyForPayWorkerId = createMVPayrollReadyWorker("Paul");
		PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sReadyForPayWorkerId;
		//create a pay period
		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(DateUtil.createDate(2008, Calendar.DECEMBER, 14));
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(period.getPayrollRunDate());
		createRequest.setWorkerIds(workerIds);
		createRequest.setPeriod(period);
		createRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		
    	//Exercise
		PayrollRunResponse payResponse = sWorkerPayService.createPayrollRun(createRequest);
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	//Verify
		AssertJUnit.assertEquals("PayPeriod mismatch!", period, payResponse.getPayrollRunPeriod());
		AssertJUnit.assertEquals("PayrollRunType mismatch!", createRequest.getPayrollRunType(), payResponse.getPayrollRunType());
		AssertJUnit.assertEquals("PayrollRunState mismatch!", PayrollRunState.DRAFT, payResponse.getPayrollRunState());
		AssertJUnit.assertNotNull(payResponse.getPayrollRunId());
		BigDecimal zeroAmount = new BigDecimal("0.00");
		CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId);
		
		com.intuit.sb.payroll.entity.model.Worker entityWorker = sWorkerEAO.findById(sReadyForPayWorkerId);
		
		PaycheckLineItem expectedPaycheckLineItems[] = {
                new PaycheckLineItem(sHourlyWageDescStr, zeroAmount,
                                     PayrollItemCategoryType.COMPENSATION,
                                     findCompensationItem(wageItems, sHourlyWageDescStr,
                                                          CompensationType.Hourly),
                                     QboeLawIdConstants.sHourlyLawId, zeroAmount,
                                     new BigDecimal(mHourlyRate), mZeroHours),
                new PaycheckLineItem(sOvertimeWageDescStr, zeroAmount,
                                     PayrollItemCategoryType.COMPENSATION,
                                     findCompensationItem(wageItems, sOvertimeWageDescStr,
                                                          CompensationType.Overtime),
                                     QboeLawIdConstants.sOvertimeLawId, zeroAmount,
                                     getCalculatedOvertimeRate(mHourlyRate, mOvertimeRate), mZeroHours),
                new PaycheckLineItem(sPaycheckTipsWageDescStr, zeroAmount,
                                     PayrollItemCategoryType.COMPENSATION,
                                     findCompensationItem(wageItems, sPaycheckTipsWageDescStr,
                                                          CompensationType.TipsPaycheck),
                                     QboeLawIdConstants.sPaycheckTipsLawId, zeroAmount,
                                     new BigDecimal(mTipsAmount), 0.0),
                new PaycheckLineItem(sSST_EE_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sSST_EELawId),
                new PaycheckLineItem(sSST_ER_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sSST_ERLawId),
                new PaycheckLineItem(sMEDT_EE_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sMED_EELawId),
                new PaycheckLineItem(sMEDT_ER_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sMED_ERLawId),
                new PaycheckLineItem(sFICA_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sFICALawId),
                new PaycheckLineItem(sFUTA_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sFUTALawId),
                new PaycheckLineItem(sCAPIT_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCAPITLawId),
                new PaycheckLineItem(sCASDI_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCASDILawId),
                new PaycheckLineItem(sCASUI_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCASUILawId),
                new PaycheckLineItem(sCAETT_DescStr, zeroAmount,
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCAETTLawId) };
		
		fixupSourcePayrollItemDetailId(expectedPaycheckLineItems, entityWorker);
		
		PaycheckDetails expectedPaycheckDetails = new PaycheckDetails(sReadyForPayWorkerId, zeroAmount, zeroAmount,
		                                                              expectedPaycheckLineItems);
		// Fix up the expected paycheckId, CheckNumber, paycheckStatus and PaycheckDistributionMethod
		PaycheckDetails paycheckDetails = payResponse.getPaycheckDetails()[0];
		expectedPaycheckDetails.setPaycheckId(paycheckDetails.getPaycheckId());
		expectedPaycheckDetails.setCheckNumber(null);
		expectedPaycheckDetails.setPaycheckStatus(PaycheckStatus.NOT_SET);
		
		Worker worker = sWorkerService.getWorkerById(sReadyForPayWorkerId);
		expectedPaycheckDetails.setDistributionMethod(worker.getPaycheckDistributionMethod());
		AssertJUnit.assertTrue("Expected PaycheckDetails mismatch!", verifyPaycheckDetails(expectedPaycheckDetails,
		                                                                      paycheckDetails));
    }

    /**
     * <P>Test createPayrollRun() using a salary worker</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a salary worker</LI>
     * <LI>Call WorkerPaymentService's createPayrollRun() with a valid PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify expected results</LI>
     * </UL>
     */
    @Test(groups = {"worker.createPayrollRun"})
    public void testCreatePayrollRunWith1SalaryWorker() throws Exception {
    	//Setup
        sReadyForPayWorkerId2 = WorkerTestUtils.createSalaryPayrollReadyWorker("Tim", sMountainViewLocation);
		PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sReadyForPayWorkerId2;
		//create a pay period
		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(DateUtil.createDate(2008, Calendar.DECEMBER, 14));
        // Set the Payroll system date to the payroll run date
        setPayrollSystemDate(period.getPayrollRunDate());
		createRequest.setWorkerIds(workerIds);
		createRequest.setPeriod(period);
		createRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		
    	//Exercise
		PayrollRunResponse payResponse = sWorkerPayService.createPayrollRun(createRequest);
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	//Verify
		AssertJUnit.assertEquals("PayPeriod mismatch!", period, payResponse.getPayrollRunPeriod());
		AssertJUnit.assertEquals("PayrollRunType mismatch!", createRequest.getPayrollRunType(), payResponse.getPayrollRunType());
		AssertJUnit.assertEquals("PayrollRunState mismatch!", PayrollRunState.DRAFT, payResponse.getPayrollRunState());
		AssertJUnit.assertNotNull(payResponse.getPayrollRunId());
		CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId2);
		
		com.intuit.sb.payroll.entity.model.Worker entityWorker = sWorkerEAO.findById(sReadyForPayWorkerId2);
		
		PaycheckLineItem expectedPaycheckLineItems[] = {
                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("2538.46"),
                                     PayrollItemCategoryType.COMPENSATION,
                                     findCompensationItem(wageItems, sSalaryWageDescStr,
                                                          CompensationType.Salary),
                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("2538.46"),
                                     WorkerTestUtils.sSalaryAmount, mZeroHours),
                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("157.38"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sSST_EELawId),
                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("157.38"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sSST_ERLawId),
                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("36.81"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sMED_EELawId),
                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("36.81"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sMED_ERLawId),
                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("440.00"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sFICALawId),
                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("20.31"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sFUTALawId),
                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("135.48"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCAPITLawId),
                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("20.31"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCASDILawId),
                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("86.31"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCASUILawId),
                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("2.54"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCAETTLawId) };
		
		fixupSourcePayrollItemDetailId(expectedPaycheckLineItems, entityWorker);
		
		PaycheckDetails expectedPaycheckDetails = new PaycheckDetails(sReadyForPayWorkerId2, new BigDecimal("2538.46"),
				new BigDecimal("1748.48"), expectedPaycheckLineItems);
		// Fix up the expected paycheckId, CheckNumber, paycheckStatus and PaycheckDistributionMethod
		PaycheckDetails paycheckDetails = payResponse.getPaycheckDetails()[0];
		expectedPaycheckDetails.setPaycheckId(paycheckDetails.getPaycheckId());
		expectedPaycheckDetails.setCheckNumber(null);
		expectedPaycheckDetails.setPaycheckStatus(PaycheckStatus.NOT_SET);
		Worker worker = sWorkerService.getWorkerById(sReadyForPayWorkerId2);
		expectedPaycheckDetails.setDistributionMethod(worker.getPaycheckDistributionMethod());
		AssertJUnit.assertTrue("Expected PaycheckDetails mismatch!", verifyPaycheckDetails(expectedPaycheckDetails,
		                                                                      paycheckDetails));
    }

    /**
     * <P>Test createPayrollRun() using a salary worker whom only worked for 5 days</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a salary worker</LI>
     * <LI>Set up a partial pay period with 5 work days</LI>
     * <LI>Call WorkerPaymentService's createPayrollRun() with a valid PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify expected results</LI>
     * </UL>
     */
    @Test(groups = {"worker.createPayrollRun"})
    public void testCreatePayrollRunWith1SalaryWorker_Worked5Days() throws Exception {
    	//Setup
        sReadyForPayWorkerId2 = WorkerTestUtils.createSalaryPayrollReadyWorker("Temp", sMountainViewLocation);
        Date terminationDate = DateUtil.createDate(2008, Calendar.DECEMBER, 5);
		PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sReadyForPayWorkerId2;
		//create a pay period
		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(DateUtil.createDate(2008, Calendar.DECEMBER, 14));
		// Change the pay period end date, payroll run date and pay date to the termination date
		period.setPayrollRunDate(terminationDate);
		period.setPeriodEndDate(terminationDate);
		period.setPayDate(terminationDate);
		period.setPartialPeriod(true);
        // Set the Payroll system date to the termination date
        setPayrollSystemDate(terminationDate);
		createRequest.setWorkerIds(workerIds);
		createRequest.setPeriod(period);
		createRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		
    	//Exercise
		PayrollRunResponse payResponse = sWorkerPayService.createPayrollRun(createRequest);
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	//Verify
		AssertJUnit.assertEquals("PayPeriod mismatch!", period, payResponse.getPayrollRunPeriod());
		AssertJUnit.assertEquals("PayrollRunType mismatch!", createRequest.getPayrollRunType(), payResponse.getPayrollRunType());
		AssertJUnit.assertEquals("PayrollRunState mismatch!", PayrollRunState.DRAFT, payResponse.getPayrollRunState());
		AssertJUnit.assertNotNull(payResponse.getPayrollRunId());
		CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId2);
		
		com.intuit.sb.payroll.entity.model.Worker entityWorker = sWorkerEAO.findById(sReadyForPayWorkerId2);
		
		PaycheckLineItem expectedPaycheckLineItems[] = {
                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("1269.23"),
                                     PayrollItemCategoryType.COMPENSATION,
                                     findCompensationItem(wageItems, sSalaryWageDescStr,
                                                          CompensationType.Salary),
                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("1269.23"),
                                     WorkerTestUtils.sSalaryAmount, mZeroHours),
                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("78.69"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sSST_EELawId),
                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("78.69"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sSST_ERLawId),
                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("18.40"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sMED_EELawId),
                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("18.40"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sMED_ERLawId),
                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("140.00"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sFICALawId),
                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("10.15"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sFUTALawId),
                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("29.70"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCAPITLawId),
                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("10.15"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCASDILawId),
                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("43.15"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCASUILawId),
                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("1.27"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCAETTLawId) };
		
		fixupSourcePayrollItemDetailId(expectedPaycheckLineItems, entityWorker);
		
		PaycheckDetails expectedPaycheckDetails = new PaycheckDetails(sReadyForPayWorkerId2, new BigDecimal("1269.23"),
				new BigDecimal("992.29"), expectedPaycheckLineItems);
		// Fix up the expected paycheckId, CheckNumber, paycheckStatus and PaycheckDistributionMethod
		PaycheckDetails paycheckDetails = payResponse.getPaycheckDetails()[0];
		expectedPaycheckDetails.setPaycheckId(paycheckDetails.getPaycheckId());
		expectedPaycheckDetails.setCheckNumber(null);
		expectedPaycheckDetails.setPaycheckStatus(PaycheckStatus.NOT_SET);
		Worker worker = sWorkerService.getWorkerById(sReadyForPayWorkerId2);
		expectedPaycheckDetails.setDistributionMethod(worker.getPaycheckDistributionMethod());
		AssertJUnit.assertTrue("Expected PaycheckDetails mismatch!", verifyPaycheckDetails(expectedPaycheckDetails,
		                                                                      paycheckDetails));
    }

    /**
     * <P>Test createPayrollRun() using a salary worker whom only worked for 1 day</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Set up a salary worker</LI>
     * <LI>Set up a partial pay period with 1 work day</LI>
     * <LI>Call WorkerPaymentService's createPayrollRun() with a valid PayrollRunCreateRequest object</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify expected results</LI>
     * </UL>
     */
    @Test(groups = {"worker.createPayrollRun"})
    public void testCreatePayrollRunWith1SalaryWorker_Worked1Day() throws Exception {
    	//Setup
        sReadyForPayWorkerId2 = WorkerTestUtils.createSalaryPayrollReadyWorker("Temp", sMountainViewLocation);
        Date terminationDate = DateUtil.createDate(2008, Calendar.DECEMBER, 1);
		PayrollRunCreateRequest createRequest = new PayrollRunCreateRequest();
	    Long[] workerIds = new Long[1];
    	workerIds[0] = sReadyForPayWorkerId2;
		//create a pay period
		PayrollRunPeriod period = createBiweeklyPayrollRunPeriod(DateUtil.createDate(2008, Calendar.DECEMBER, 14));
		// Change the pay period end date, payroll run date and pay date to the termination date
		period.setPayrollRunDate(terminationDate);
		period.setPeriodEndDate(terminationDate);
		period.setPayDate(terminationDate);
		period.setPartialPeriod(true);
        // Set the Payroll system date to the termination date
        setPayrollSystemDate(terminationDate);
		createRequest.setWorkerIds(workerIds);
		createRequest.setPeriod(period);
		createRequest.setPayrollRunType(PayrollRunTypeEnum.TERMINATION);
		
    	//Exercise
		PayrollRunResponse payResponse = sWorkerPayService.createPayrollRun(createRequest);
     	// Restore the Payroll system date to the current date
     	resetPayrollSystemDate();

    	//Verify
		AssertJUnit.assertEquals("PayPeriod mismatch!", period, payResponse.getPayrollRunPeriod());
		AssertJUnit.assertEquals("PayrollRunType mismatch!", createRequest.getPayrollRunType(), payResponse.getPayrollRunType());
		AssertJUnit.assertEquals("PayrollRunState mismatch!", PayrollRunState.DRAFT, payResponse.getPayrollRunState());
		AssertJUnit.assertNotNull(payResponse.getPayrollRunId());
		
		com.intuit.sb.payroll.entity.model.Worker entityWorker = sWorkerEAO.findById(sReadyForPayWorkerId2);
		
		CompensationItem[] wageItems = sWageService.getCompensationItems(sReadyForPayWorkerId2);
		PaycheckLineItem expectedPaycheckLineItems[] = {
                new PaycheckLineItem(sSalaryWageDescStr, new BigDecimal("253.85"),
                                     PayrollItemCategoryType.COMPENSATION,
                                     findCompensationItem(wageItems, sSalaryWageDescStr,
                                                          CompensationType.Salary),
                                     QboeLawIdConstants.sSalaryLawId, new BigDecimal("253.85"),
                                     WorkerTestUtils.sSalaryAmount, mZeroHours),
                new PaycheckLineItem(sSST_EE_DescStr, new BigDecimal("15.74"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sSST_EELawId),
                new PaycheckLineItem(sSST_ER_DescStr, new BigDecimal("15.74"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sSST_ERLawId),
                new PaycheckLineItem(sMEDT_EE_DescStr, new BigDecimal("3.68"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sMED_EELawId),
                new PaycheckLineItem(sMEDT_ER_DescStr, new BigDecimal("3.68"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sMED_ERLawId),
                new PaycheckLineItem(sFICA_DescStr, new BigDecimal("2.00"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sFICALawId),
                new PaycheckLineItem(sFUTA_DescStr, new BigDecimal("2.03"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sFUTALawId),
                new PaycheckLineItem(sCAPIT_DescStr, new BigDecimal("0.00"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCAPITLawId),
                new PaycheckLineItem(sCASDI_DescStr, new BigDecimal("2.03"),
                                     PayrollItemCategoryType.EMPLOYEE_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCASDILawId),
                new PaycheckLineItem(sCASUI_DescStr, new BigDecimal("8.63"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCASUILawId),
                new PaycheckLineItem(sCAETT_DescStr, new BigDecimal("0.25"),
                                     PayrollItemCategoryType.EMPLOYER_PAID_TAX, notACompensationItem,
                                     QboeLawIdConstants.sCAETTLawId) };
		
		fixupSourcePayrollItemDetailId(expectedPaycheckLineItems, entityWorker);
		
		PaycheckDetails expectedPaycheckDetails = new PaycheckDetails(sReadyForPayWorkerId2, new BigDecimal("253.85"),
				new BigDecimal("230.40"), expectedPaycheckLineItems);
		// Fix up the expected paycheckId, CheckNumber and PaycheckDistributionMethod
		PaycheckDetails paycheckDetails = payResponse.getPaycheckDetails()[0];
		expectedPaycheckDetails.setPaycheckId(paycheckDetails.getPaycheckId());
		expectedPaycheckDetails.setCheckNumber(null);
		expectedPaycheckDetails.setPaycheckStatus(PaycheckStatus.NOT_SET);
		Worker worker = sWorkerService.getWorkerById(sReadyForPayWorkerId2);
		expectedPaycheckDetails.setDistributionMethod(worker.getPaycheckDistributionMethod());
		AssertJUnit.assertTrue("Expected PaycheckDetails mismatch!", verifyPaycheckDetails(expectedPaycheckDetails,
		                                                                      paycheckDetails));
    }

}
