package com.intuit.sb.payroll.qa.services.workerpayments;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleComponent;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.entity.model.PayrollSchedule;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunState;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;
import com.intuit.sb.payroll.services.workerpayments.impl.PayrollFrequencyMapper;

/**
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */

public class GetNextScheduledPayrollRunInfoTest extends WorkerPaymentServiceTestBase {
	PayrollScheduleComponent pScheduleComponent;

	@BeforeMethod
	public void initializeGetNextScheduledPayrollRunInfoTest()
	{
		pScheduleComponent = qaBootstrap.getEjbFromClass(PayrollScheduleComponent.class);
	}
	
    /**
     * <P>Test getNextScheduledPayrollRunInfo when there is no default Payroll Schedule</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call WorkerPaymentService's getNextScheduledPayrollRunInfo() when no default Payroll Schedule created</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that null is returned</LI>
     * </UL>
     */
    @Test
    public void testNoDefaultPayrollSchedule() throws Exception {
    	// Close the existing transaction
    	getUserTransaction().commit();
    	PayrollSchedule payrollSchedule = null;
    	boolean scheduleCreated = false;
    	// Setup
    	try {
    		payrollSchedule = pScheduleComponent.getDefaultPayrollSchedule();
    		// If a default PayrollSchedule exists, then change it so that it's not set as the default
    		payrollSchedule.setDefaultt(false);
    		payrollSchedule = pScheduleComponent.updatePayrollSchedule(payrollSchedule);
    	} catch (ServiceFailureException ex) {
    		// No default schedule exist, create a non-default schedule
			getUserTransaction().begin();
			PayrollScheduleTestUtils.createDefaultSchedule();
    		payrollSchedule = pScheduleComponent.getDefaultPayrollSchedule();
    		payrollSchedule.setDefaultt(false);
			payrollSchedule = pScheduleComponent.updatePayrollSchedule(payrollSchedule);
			getUserTransaction().commit();
			scheduleCreated = true;
    	}
    	try {
        	// Exercise
            PayrollRunInfo payrollRunInfo = sWorkerPayService.getNextScheduledPayrollRunInfo();
            AssertJUnit.assertNull("PayrollRunInfo is not null!", payrollRunInfo);
    	} finally {
    		if (scheduleCreated) {
        		// Delete the schedule if it was created by this test
    			pScheduleComponent.deletePayrollSchedule(payrollSchedule);
    		} else {
    			// Set the schedule back to be the default
        		payrollSchedule.setDefaultt(true);
        		payrollSchedule = pScheduleComponent.updatePayrollSchedule(payrollSchedule);
    		}
    	}
    }

    /**
     * <P>Test getting the next scheduled PayrollRunInfo</P>
     *
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a default PayrollSchedule if needed</LI>
     * <LI>Call WorkerPaymentService's getNextScheduledPayrollRunInfo() when with a default Payroll Schedule created</LI>
     * </OL>
     *
     * <UL>
     * <LI>Verify that the PayrollRunInfo returned is correct</LI>
     * </UL>
     */
    @Test
    public void testGetNextScheduledPayrollRunInfo() throws Exception {
    	// Close the existing transaction
    	getUserTransaction().commit();
    	PayrollSchedule payrollSchedule = null;
    	boolean scheduleCreated = false;
    	boolean defaultScheduleExists = true;
    	// Setup
    	try {
    		payrollSchedule = pScheduleComponent.getDefaultPayrollSchedule();
    		if (null == payrollSchedule) {
    			defaultScheduleExists = false;
    		}
    	} catch (ServiceFailureException ex) {
    		// No default schedule exist, we need to create one
			getUserTransaction().begin();
			PayrollScheduleTestUtils.createDefaultSchedule();
			getUserTransaction().commit();
    		payrollSchedule = pScheduleComponent.getDefaultPayrollSchedule();
			scheduleCreated = true;
    	}
    	try {
        	// Exercise
            PayrollRunInfo computed = sWorkerPayService.getNextScheduledPayrollRunInfo();
            // Verify
            if (defaultScheduleExists) {
                com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo payrollRunInfo =
                	pScheduleComponent.getNextScheduledPayrollRunInfo(payrollSchedule);
                PayrollRunInfo expected = new PayrollRunInfo();
                expected.setPayrollRunId(null);
                PayrollRunPeriod period = new PayrollRunPeriod();
                period.setPeriodStartDate(payrollRunInfo.getPayPeriod().getStartDate());
                period.setPeriodEndDate(payrollRunInfo.getPayPeriod().getEndDate());
                period.setPayDate(payrollRunInfo.getPayDate());
                period.setPayrollRunDate(payrollRunInfo.getPayrollRunDate());
                period.setFrequency(PayrollFrequencyMapper.mapFromDEntityToEI(payrollRunInfo.getFrequency()));
                expected.setPayrollRunPeriod(period);
                expected.setPayrollRunState(PayrollRunState.NOT_SET);
                AssertJUnit.assertTrue("PayrollRunInfo mismatch!", verifyPayrollRunInfo(expected, computed));
            } else {
                AssertJUnit.assertNull("PayrollRunInfo is not null!", computed);
            }
    	} finally {
    		// Delete the schedule if it was created by this test
    		if (scheduleCreated) {
    			pScheduleComponent.deletePayrollSchedule(payrollSchedule);
    		}
    	}
    }

}
