package com.intuit.sb.payroll.qa.services.workerpayments;

import java.util.Calendar;

import javax.transaction.Status;

import org.testng.AssertJUnit;

import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.capability.common.BasicTestBase;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentServiceTestBase;


/**
 * <P>The <CODE>GetPayrollScheduleTest</CODE> class is the test class for testing the getPayrollSchedule API.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetPayrollScheduleTest extends WorkerPaymentServiceTestBase
{
    @AfterMethod
    public void tearDown() throws Exception
    {
        if (getUserTransaction().getStatus() == Status.STATUS_ACTIVE)
        {
        	BasicTestBase.deleteAllPayrollSchedule();
        }
        super.tearDown();
    }
    
    @BeforeMethod
    public void testInit() throws Exception {
    	PayrollScheduleTestUtils.createDefaultSchedule();
    }
    
    /**
     * <P>
     * Test to verify the Default PayDay from Payroll Schedule for a company. 
     * </P>
     * <OL>
     * <LI>Setting the default static variable so that other test cases won't affect it</LI>
     * <LI>Call IEmployerService's getPayrollSchedule() to perform the actual work</LI>
     * <LI>Create the Calendar current date and assign the values we get from the PeriodEndDate</LI>
     * <LI>We will be adding the PayrollRunDayOffset and PayDateOffset in this date and getting the PayDay.</LI>
     * </OL>
     * <UL>
     * <LI>Verify the PayDay on Default PayrollSchedule it should return which is matches with the Friday
     * </UL>
     */
    @Test
    public void getDefaultpayrollSchedule() throws Exception
    {
        // Setup
        sDefaultPayrollDayOffset = 3;
        sDefaultPayrollRunDayOffset = 1;
        sDefaultPayPeriodEndDayOfMonth1 = 0;
        sDefaultPayPeriodEndDayOfMonth2 = 0;
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate = DateUtil.getNowCalendar();
        pDate.setTime(pSch.getPeriodEndDate());
        Calendar pDate1 = DateUtil.getNowCalendar();
        //Exercise
        pDate1.add(Calendar.DATE, 1);
        pDate1 = DateUtil.getNextWeekDayOfWeek(pDate1, Calendar.FRIDAY);   
        pDate.add(Calendar.DATE, pSch.getPayrollPayDateOffset());
        AssertJUnit.assertEquals("Pay day is not an upcoming Friday", pDate1.get(Calendar.DAY_OF_WEEK),pDate.get(Calendar.DAY_OF_WEEK));
    }

    /**
     * <P>
     * Test to verify the Default PayrollDateOffSet & getFrequency from Payroll Schedule. These should written integer value 2.
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to perform the actual work</LI>
     * </OL>
     * <UL>
     * <LI>Verify the PayrollDateOffSet & getFrequency on Default PayrollSchedule it should return 2.
     * </UL>
     */
    @Test
    public void getDefaultPayrollScheduleValues() throws Exception
    {
        //Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        //Verify
        AssertJUnit.assertEquals("Expected payrollDateOffset mismatch", sDefaultPayrollDayOffset, pSch.getPayrollPayDateOffset());
        AssertJUnit.assertEquals("Expected payrollDateOffset mismatch", sDefaultPayrollRunDayOffset, pSch.getPayrollRunDateOffset());
        AssertJUnit.assertEquals("Expected pay frequency mismatch",PayrollScheduleFrequency.BIWEEKLY,pSch.getFrequency());
        AssertJUnit.assertEquals("Expected pay frequency mismatch",sDefaultPayPeriodEndDayOfMonth1,pSch.getPayPeriodEndDayofMonth1());
        AssertJUnit.assertEquals("Expected pay frequency mismatch",sDefaultPayPeriodEndDayOfMonth2,pSch.getPayPeriodEndDayofMonth2());
    }

   
    /**
     * <P>
     * Test to verify the Default PeriodEndDate from Payroll Schedule for a company.
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IEmployerService's getPayrollSchedule() to perform the actual work</LI>
     * <LI>Create the Calendar current date, Which give us the Tuesday which occurs before next Friday
     *  </LI>
     * </OL>
     * <UL>
     * <LI>Verify the Date with Default PayrollSchedule EndDate</L1>
     * </UL>
     */
    @Test
    public void getDefaultEndDate() throws Exception
    {
        //Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate1 = DateUtil.getNowCalendar();
        //Exercise
        pDate1.add(Calendar.DATE, sDefaultPayrollDayOffset - sDefaultPayrollRunDayOffset);
        pDate1 = DateUtil.getNextWeekDayOfWeek(pDate1, Calendar.FRIDAY);
        pDate1.add(Calendar.DATE, -sDefaultPayrollDayOffset);        
        Calendar pDate = DateUtil.getNowCalendar();
        pDate.setTime(pSch.getPeriodEndDate());
        //Verify
        AssertJUnit.assertTrue("Expected Payroll End Date",DateUtil.isSameDay(pDate1,pDate));
    }

    /**
     * <P>
     * Test to verify the Default PeriodStartDate from Payroll Schedule for a company.
     * </P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Call IWorkerPaymentsService's getPayrollSchedule() to perform the actual work</LI>
     * <L1>Calling the function getBacktoBackWeekDayOfWeek to get the right date from current. </L1>
     * </OL>
     * <UL>
     * <LI>Verify the Date with Default PayrollSchedule Start Date</L1>
     * </UL>
     */
    
    @Test(enabled=false, groups={"broken"})
    public void getDefaultStartDate() throws Exception
    {
        //Setup
        pSch = sWorkerPayService.getPayrollSchedule();
        Calendar pDate1 = DateUtil.getNowCalendar();
        //Exercise
        pDate1 = getBacktoBackWeekDayOfWeek(pDate1, Calendar.WEDNESDAY, sDefaultWeekCounter);
        Calendar pDate = DateUtil.getNowCalendar();
        pDate.setTime(pSch.getPeriodStartDate());
        //Verify     
        AssertJUnit.assertTrue("Expected Payroll Start Date",DateUtil.isSameDay(pDate1,pDate));
    }

    private static Calendar getBacktoBackWeekDayOfWeek(Calendar calendar, int dayofWeek, int defaultCount) 
    { 
        if (calendar == null)
        {
            return null;
        }
      
        Integer Count = 0; 
        Calendar tmpCalendar = DateUtil.getNowCalendar(); 
        while (!((tmpCalendar.get(Calendar.DAY_OF_WEEK) == dayofWeek) && (defaultCount == Count))) 
        { 
            tmpCalendar.add(Calendar.DAY_OF_YEAR, -1); 
            if (tmpCalendar.get(Calendar.DAY_OF_WEEK) == dayofWeek) 
            { 
                Count++; 
            } 
        } 
        return tmpCalendar; 
    } 
}
