/**
 * Copyright: 2007, Intuit, Inc. All rights reserved.
 *
 */
package com.intuit.sb.payroll.qa.task;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.testng.AssertJUnit;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.ServiceFailureException;
import com.intuit.sb.payroll.ValidationException;
import com.intuit.sb.payroll.capability.employer.EmployerComponentTestUtils;
import com.intuit.sb.payroll.capability.employer.impl.EmployerComponentTasks;
import com.intuit.sb.payroll.capability.employer.impl.EmployerPTSWorkflowTasks;
import com.intuit.sb.payroll.capability.worker.WorkerTestUtils;
import com.intuit.sb.payroll.capability.worker.impl.WorkerComponentTasks;
import com.intuit.sb.payroll.capability.workerpayments.PaycheckCalculationRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunCommitRequest;
import com.intuit.sb.payroll.capability.workerpayments.PayrollRunInfo;
import com.intuit.sb.payroll.capability.workerpayments.PayrollScheduleTestUtils;
import com.intuit.sb.payroll.capability.workerpayments.WorkerPaymentsComponentTestBase;
import com.intuit.sb.payroll.capability.workerpayments.impl.WorkerPaymentsComponentTasks;
import com.intuit.sb.payroll.common.DateUtil;
import com.intuit.sb.payroll.entity.model.PayrollDistributionMethodEnum;
import com.intuit.sb.payroll.entity.model.PayrollRun;
import com.intuit.sb.payroll.entity.model.PayrollRunWorkflowStatusEnum;
import com.intuit.sb.payroll.entity.taxes.TaxEAO;
import com.intuit.sb.payroll.entity.wages.CompensationEAO;
import com.intuit.sb.payroll.services.employer.AssistedStateType;
import com.intuit.sb.payroll.services.employer.EmployerService;
import com.intuit.sb.payroll.services.workerpayments.PayrollSchedule;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService;
import com.intuit.sb.payroll.task.Task;
import com.intuit.sb.payroll.task.TaskService;
import com.intuit.sb.payroll.task.TaskType;

/**
 * <P>The <CODE>GetCurrentTasksTest</CODE> class contains test methods for the getCurrentTasks()
 * Payroll Engine TaskService API.
 *
 * $Author$
 * $Revision$
 * $DateTime$
 *
 */
public class GetCurrentTasksTest extends WorkerPaymentsComponentTestBase
{
	protected static TaskService taskService;
	protected static CompensationEAO compensationEAO;
	protected static TaxEAO taxEAO;
	protected static WorkerPaymentService workerPayment;
	protected static EmployerService employerService;

	private final BigDecimal HOURLY_RATE = new BigDecimal("10.0");
	private final BigDecimal BONUS_AMOUNT = new BigDecimal("200.0");

	private static Long sReadyForPayWorkerId = 0L;
	private static boolean overDue = false;
	private static PayrollSchedule ps = null;
	HashMap<String,Object> taskTypeMap = new HashMap<String,Object>();

	@BeforeClass
	public static void initializeGetCurrentTasksTestClass()
	{
		taskService = qaBootstrap.getEjbFromClass(TaskService.class);
		compensationEAO = qaBootstrap.getEjbFromClass(CompensationEAO.class);
		taxEAO = qaBootstrap.getEjbFromClass(TaxEAO.class);
		workerPayment = qaBootstrap.getEjbFromClass(WorkerPaymentService.class);
		employerService = qaBootstrap.getEjbFromClass(EmployerService.class);
	}
	
	@BeforeMethod
	public void initializeGetCurrentTasksTest(){
		taskTypeMap = getExpectedTaskTypeMap();
	}
	//Not checking the status of the tasks in the tests for time being. Should be hooked up later.

	/**
     * <P>Test company setup Tasks having work location</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Add a work location</LI>
     * <LI>Call TaskService.getCurrentTasks()</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK and EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK Task is received</LI>
     * </UL>
     */
    @Test
    public void testCompanySetupTasksWithWorklocation() throws Exception {

    	// Setup
		String [] expectedTaskNames = {WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK ,EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK};
    	HashMap<String , Integer> taskPriorities = null ;

		//Delete the current Employer and create a new Employer with out workers and payroll schedule
    	//but with a work location
    	EmployerComponentTestUtils.deleteCurrentEmployer();
    	EmployerComponentTestUtils.createAndSaveNewEmployer();
    	EmployerComponentTestUtils.createWorkLocation();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();
		taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);

    	//TO DO list priority check
    	AssertJUnit.assertFalse("Task Priority violated",taskPriorities.get(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK) < taskPriorities.get(WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK));
    }

    /**
     * <P>Test delete work location from Initial company Tasks</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Add a work location</LI>
     * <LI>Call TaskService.getCurrentTasks()</LI>
     * <LI>Delete the work location and check the task is again reopened</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK, EmployerComponentTasks.WORKLOCATION_SETUP_TASK Tasks are received</LI>
     * </UL>
     */
    @Test(enabled=false, groups={"broken"})
    public void testReOpenWorklocationTaskAfterDeleteWorklocation() throws Exception {

    	// Setup
		String [] expectedTaskNames = {WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK ,EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK};
    	HashMap<String , Integer> taskPriorities = null ;

		//Delete the current Employer and create a new Employer with out workers and payroll schedule
    	//but with a work location
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();
		taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);

    	//TO DO list priority check
    	AssertJUnit.assertFalse("Task Priority violated",taskPriorities.get(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK) < taskPriorities.get(WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK));

    	//Delete Worklocation
    	sEmployer.removeWorkLocation(EmployerComponentTestUtils.workLocation);
    	sEmployerComponent.saveEmployer(sEmployer);

    	// Setup
		String [] expectedNewTaskNames = {WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK ,EmployerComponentTasks.WORKLOCATION_SETUP_TASK};

    	// Exercise
    	checkTasks = taskService.getCurrentTasks();

		taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedNewTaskNames);

		//TO DO list priority check
    	AssertJUnit.assertFalse("Task Priority violated",taskPriorities.get(EmployerComponentTasks.WORKLOCATION_SETUP_TASK) < taskPriorities.get(WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK));
    }

    /**
     * <P>Test Newly registered company Tasks</P>
     * <H3>Steps:</H3>
     * <OL>
     * <LI>Create a New Employer with out work location</LI>
     * <LI>Call TaskService.getCurrentTasks()</LI>
     * </OL>
     * <UL>
     * <LI>Verify that the WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK, EmployerComponentTasks.WORKLOCATION_SETUP_TASK Task is received</LI>
     * </UL>
     */
    @Test
    public void testNewlyRegisteredCompanyTasks() throws Exception{

    	// Setup
		String [] expectedTaskNames = {WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK, EmployerComponentTasks.COMPANY_SETUP_TASK};
    	HashMap<String , Integer> taskPriorities = null ;

		//Delete the current Employer and create a new Employer with out workers and payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
    	EmployerComponentTestUtils.createAndSaveNewEmployer();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);

    	//TO DO list priority check
		AssertJUnit.assertTrue(
						"Task Priority violated",
						taskPriorities.get(WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK) < 
						taskPriorities.get(EmployerComponentTasks.COMPANY_SETUP_TASK));
    }

    /**
	 * <P>
	 * Test partial company setup tasks with Payroll-ready workers
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a work location</LI>
	 * <LI>Add payroll-ready worker</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK is received </LI>
	 * </UL>
     * @throws Exception
	 */
    @Test
    public void testPartialCompanySetupTasksWithPayrollReadyWorker() throws Exception
    {
    	// Setup
		String [] expectedTaskNames = {EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK };

		//Delete the current Employer and create a new Employer with out workers and payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
		EmployerComponentTestUtils.createPayrollReadyWorker();
    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);
    }

    /**
	 * <P>
	 * Test partial company setup tasks with non Payroll-ready workers
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a work location</LI>
	 * <LI>Add a worker with only name and no other info</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that WorkerComponentTasks.WORKER_PAYROLL_READY_TASK and EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK tasks are returned</LI>
	 * </UL>
	 */
    @Test
    public void testPartialCompanySetupTasksWithNonPayrollReadyWorker() throws Exception
    {
    	// Setup
		String [] expectedTaskNames = {EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK ,WorkerComponentTasks.WORKER_PAYROLL_READY_TASK, WorkerComponentTasks.WORKER_WITHHOLDING_TASK};
    	HashMap<String , Integer> taskPriorities = null ;

    	//Delete the current Employer and create a new Employer with NonPayroll ready workers with no payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
		EmployerComponentTestUtils.createNonPayrollReadyWorker();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);

    	//TO DO list priority check
    	AssertJUnit.assertFalse("Task Priority violated",(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK))||
    			(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK))||
    			(taskPriorities.get(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK) < taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK)));

    }

    /**
	 * <P>
	 * Test tasks of a Company ready to run payroll
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a work location</LI>
	 * <LI>Add ready for pay worker</LI>
	 * <LI>Add new payroll schedule</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK task is returned</LI>
	 * </UL>
     * @throws Exception
	 */
    @Test
    public void testPayRunReadyCompanyTasks() throws Exception
    {
    	// Setup
		String [] expectedTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK};

		//Delete the current Employer and create a new Employer with NonPayroll ready workers with no payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
    	createPayrollSchedule();
    	EmployerComponentTestUtils.createPayrollReadyWorker();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);
    }

    /**
	 * <P>
	 * Test Company With Out Workers tasks
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a work location</LI>
	 * <LI>Add new payroll schedule</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK task returned</LI>
	 * </UL>
	 */
    @Test
    public void testCompanyWithOutWorkerTasks() throws Exception
    {
    	// Setup
		String [] expectedTaskNames = {WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK};

		//Delete the current Employer and create a new Employer with out workers but with Payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
    	createPayrollSchedule();
    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);
    }

    /**
	 * <P>
	 * Test tasks of a company with work location, payroll schedule, a non payroll ready worker
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a new work location</LI>
	 * <LI>Add a new payroll schedule</LI>
	 * <LI>Add a non payroll ready worker</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 * FIXME [th] enable this
	 * <UL>
	 * <LI>Verify that WorkerComponentTasks.WORKER_PAYROLL_READY_TASK, WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK and WorkerComponentTasks.WORKER_WITHHOLDING_TASK tasks are returned</LI>
	 * </UL>
	 */
    @Test(enabled=false, groups={"broken"})
    public void testWorkerPayrollReadyTask() throws Exception
    {
    	// Setup
		String [] expectedTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK ,WorkerComponentTasks.WORKER_PAYROLL_READY_TASK, WorkerComponentTasks.WORKER_WITHHOLDING_TASK};
    	HashMap<String , Integer> taskPriorities = null ;

    	//Delete the current Employer and create a new Employer with NonPayroll ready workers with no payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
    	EmployerComponentTestUtils.createAndSaveNewEmployer();
    	EmployerComponentTestUtils.createWorkLocation();
    	EmployerComponentTestUtils.createNonPayrollReadyWorker();
    	createPayrollSchedule();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();
		taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);

		//TO DO list priority check
		if(!overDue)
	    	AssertJUnit.assertFalse("Task Priority violated",(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK))||
	    			(taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK) < taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK))||
	    			(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK)));
		else
			AssertJUnit.assertFalse("Task Priority violated",(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK))||
	     		(taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK) < taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK))||
	     		(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK)));
	}

    /**
	 * <P>
	 * Test tasks of a company ready for Payroll run, worker with no withholding
	 * </P>
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a new work location</LI>
	 * <LI>Add Payroll Ready Worker with No WithHoldings</LI>
	 * <LI>Add new payroll schedule</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 * <UL>
	 * <LI>Verify that WorkerComponentTasks.WORKER_WITHHOLDING_TASK and WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK tasks returned</LI>
	 * </UL>
	 */
    @Test
    public void testWorkerWithholdingTask()throws Exception{
    	// Setup
		String [] expectedTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK , WorkerComponentTasks.WORKER_WITHHOLDING_TASK};
    	HashMap<String , Integer> taskPriorities = null ;

    	//Delete the current Employer and create a new Employer with Payroll schedule and with a worker who is
    	//payroll Ready not but with no WithHolding info
    	EmployerComponentTestUtils.deleteCurrentEmployer();
    	EmployerComponentTestUtils.createAndSaveNewEmployer();
    	EmployerComponentTestUtils.createWorkLocation();
    	createPayrollSchedule();
    	EmployerComponentTestUtils.createPayrollReadyWorkerWithNoWithHolding();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();
		taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);

    	//TO DO list priority check
    	AssertJUnit.assertFalse("Task Priority violated",taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK));
    }

    /**
	 * <P>
	 * Test tasks after deleting PayItems of a worker
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a New Work location</LI>
	 * <LI>Add a Payroll Ready Worker with No WithHoldings</LI>
	 * <LI>Add new payroll schedule</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * <LI>Delete Payroll Items</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that WorkerComponentTasks.WORKER_PAYROLL_READY_TASK,WorkerComponentTasks.WORKER_WITHHOLDING_TASK and WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK tasks returned</LI>
	 * </UL>
	 */
    @Test(enabled=false, groups={"broken"})
    public void testWorkerPayrollReadyTaskAfterPayItemsDeleted()throws Exception{
    	// Setup
		String [] expectedTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK, WorkerComponentTasks.WORKER_WITHHOLDING_TASK};
    	HashMap<String , Integer> taskPriorities = null ;

    	//Delete the current Employer and create a new Employer with Payroll schedule and with a worker who is
    	//payroll Ready not but with no WithHolding info
    	EmployerComponentTestUtils.deleteCurrentEmployer();
    	EmployerComponentTestUtils.createAndSaveNewEmployer();
    	EmployerComponentTestUtils.createWorkLocation();
    	createPayrollSchedule();
    	EmployerComponentTestUtils.createPayrollReadyWorkerWithNoWithHolding();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();
		taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);

    	//TO DO list priority check
    	AssertJUnit.assertFalse("Task Priority violated",taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK));

    	//Delete Payroll items
    	EmployerComponentTestUtils.deleteWageItems();

    	// Setup
    	String [] expectedNewTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK, WorkerComponentTasks.WORKER_PAYROLL_READY_TASK, WorkerComponentTasks.WORKER_WITHHOLDING_TASK};

    	// Exercise
    	checkTasks = taskService.getCurrentTasks();
    	taskPriorities = getTaskPriorities(checkTasks);

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedNewTaskNames);

		//TO DO list priority check
		if(!overDue)
	    	AssertJUnit.assertFalse("Task Priority violated",(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK))||
	    			(taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK) < taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK))||
	    			(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK)));
		else
			AssertJUnit.assertFalse("Task Priority violated",(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK))||
	     		(taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK) < taskPriorities.get(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK))||
	     		(taskPriorities.get(WorkerComponentTasks.WORKER_WITHHOLDING_TASK) < taskPriorities.get(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK)));
	}


    /**
	 * <P>
	 * Test tasks after first Payroll Run executed
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a New work location</LI>
	 * <LI>Add a ready for pay worker</LI>
	 * <LI>Add new payroll schedule</LI>
	 * <LI>Execute First Payroll Run</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that "Run Payroll" task is returned</LI>
	 * </UL>
	 */
    @Test
    public void testRunPayrollTaskAfterFirstPayrollRun() throws Exception{
    	// Setup
		String [] expectedTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK, 
													WorkerPaymentsComponentTasks.FINISH_PAYCHECK_DISTRIBUTION_TASK,
													EmployerPTSWorkflowTasks.SETUP_ASSISTED_TASK};

		//Delete the current Employer and create a new Employer with Payroll ready workers and Payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
    	createPayrollSchedule();
    	EmployerComponentTestUtils.createPayrollReadyWorker();

       	executeFirstPayrollRun();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);
    }


	/**
	 * <P>
	 * Test tasks after draft Payroll run
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a New work location</LI>
	 * <LI>Add ready for pay worker</LI>
	 * <LI>Add new payroll schedule</LI>
	 * <LI>Draft First Payroll Run</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that "Run Payroll" tasks are returned</LI>
	 * </UL>
	 */
    @Test
    public void testRunPayrollTaskAfterDraftPayrollRun() throws Exception{
    	// Setup
		String [] expectedTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK};

		//Delete the current Employer and create a new Employer with Payroll ready workers and Payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
    	createPayrollSchedule();
    	EmployerComponentTestUtils.createPayrollReadyWorker();

    	//Draft Payroll run
    	try{
    		draftFirstPayrollRun();
        	}
        	catch(Exception e){
        		AssertJUnit.fail("Could not execute PayrollRun");
        	}

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);
    }

    /**
	 * <P>
	 * Test  tasks after first Payroll Run executed
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Create a New work location</LI>
	 * <LI>Add ready for pay workers</LI>
	 * <LI>Add new payroll schedule</LI>
	 * <LI>Execute First Payroll Run for only one payroll ready worker</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that "Run Payroll" task for current pay period is marked complete</LI>
	 * </UL>
	 */
    @Test
    public void testRunPayrollTaskCompleteAfterSelectivePayrollRun() throws Exception{
    	// Setup
		String [] expectedTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK,
													WorkerPaymentsComponentTasks.FINISH_PAYCHECK_DISTRIBUTION_TASK,
													EmployerPTSWorkflowTasks.SETUP_ASSISTED_TASK};

		//Delete the current Employer and create a new Employer with Payroll ready workers and Payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
    	createPayrollSchedule();
    	createPayrollReadyWorkers();

    	//Execute Payroll run
    	executeFirstPayrollRun();

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);
    }

    /**
	 * <P>
	 * Test prepare quarterly taxes task closed after Employer sign up for assisted service
	 * </P>
	 *
	 * <H3>Steps:</H3>
	 * <OL>
	 * <LI>Add a New work location</LI>
	 * <LI>Add a ready for pay worker</LI>
	 * <LI>Add new payroll schedule</LI>
	 * <LI>Execute First Payroll Run</LI>
	 * <LI>Sign up Employer for Assisted Service</LI>
	 * <LI>Call TaskService.getCurrentTasks()</LI>
	 * </OL>
	 *
	 * <UL>
	 * <LI>Verify that "RUN_PAYROLLRUN_TASK", "FINISH_PAYCHECK_DISTRIBUTION_TASK" tasks are returned</LI>
	 * </UL>
	 */
    @Test
    public void testPrepareQuarterlyTaskClosedAfterAssistedSignUp() throws Exception{
    	// Setup
		String [] expectedTaskNames = {WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK,
												WorkerPaymentsComponentTasks.FINISH_PAYCHECK_DISTRIBUTION_TASK};

		//Delete the current Employer and create a new Employer with Payroll ready workers and Payroll schedule
    	EmployerComponentTestUtils.deleteCurrentEmployer();
		EmployerComponentTestUtils.createAndSaveNewEmployer();
		EmployerComponentTestUtils.createWorkLocation();
    	createPayrollSchedule();
    	EmployerComponentTestUtils.createPayrollReadyWorker();

    	//Execute Payroll run
       	PayrollRun payrollRun = executeFirstPayrollRun();
       	employerService.updateAssistedState(AssistedStateType.INITIATED_ONBOARDING);

    	// Exercise
    	Task[] checkTasks = taskService.getCurrentTasks();

		// verify task name and type
		verifyTaskNameAndType(checkTasks,expectedTaskNames);
		if (payrollRun != null) {
        	Long payrollRunId = payrollRun.getId();
        	cleanUpPayrollRun(payrollRunId);
    	}
    }

    /**
	 * Creates a Payroll run Request and commits the Payroll Run
	 *
	 * @throws Exception
	 */
    private PayrollRun executeFirstPayrollRun() throws Exception{
    	PayrollRun payrollRun = null;
    	sReadyForPayWorkerId = sWorker.getId();
    		//Setup
        	aSchedule = createBiweeklySchedule(ps.getPeriodEndDate(), ps.getPayrollRunDateOffset(),
        	                                   ps.getPayrollPayDateOffset());
    		PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    		List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();

    		PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, HOURLY_RATE, new Double(80), new Double(10), BONUS_AMOUNT,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    		paycheckCalculationRequests.add(calcRequest);
    		PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    		payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
    		payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);
    		//getUserTransaction().commit();

            // Set the Payroll system date to the payroll run date
            setPayrollSystemDate(payrollRunInfo.getPayrollRunDate());
    		//Exercise
    		payrollRun = sWorkerPaymentsComponent.commitPayrollRun(payrollRunRequest);
         	// Restore the Payroll system date to the current date
         	resetPayrollSystemDate();

    		//Verify
    		AssertJUnit.assertTrue(validatePayrollRun(payrollRun, payrollRunRequest));
    		AssertJUnit.assertEquals("Number of paycheck(s) mismatch!", 1, payrollRun.getPaychecks().size());
    		AssertJUnit.assertEquals("workflowStatus mismatch!", PayrollRunWorkflowStatusEnum.COMMITTED,
    					payrollRun.getWorkflowStatus());
    		return payrollRun;
    }

    /**
	 * Creates a Payroll run Request but not commits the Payroll Run
	 *
	 * @throws Exception
	 */
    private void draftFirstPayrollRun() throws Exception{
    	PayrollRun payrollRun = null;
    	sReadyForPayWorkerId = sWorker.getId();
    	try {
    		//Setup
        	aSchedule = createBiweeklySchedule(ps.getPeriodEndDate(),ps.getPayrollRunDateOffset(),
                    									ps.getPayrollPayDateOffset());
    		PayrollRunInfo payrollRunInfo = createAPayrollRunInfo(aSchedule);
    		List<PaycheckCalculationRequest> paycheckCalculationRequests = new ArrayList<PaycheckCalculationRequest>();

    		PaycheckCalculationRequest calcRequest = createPaycheckCalculationRequest(sReadyForPayWorkerId,
    					payrollRunInfo, HOURLY_RATE, new Double(80), new Double(10), BONUS_AMOUNT,
    					PayrollDistributionMethodEnum.PRINTEDPAYCHECK);
    		paycheckCalculationRequests.add(calcRequest);
    		PayrollRunCommitRequest payrollRunRequest = new PayrollRunCommitRequest();
    		payrollRunRequest.setPayrollRunInfo(payrollRunInfo);
    		payrollRunRequest.setCalculationRequests(paycheckCalculationRequests);

    	} catch (Exception e) {
    		AssertJUnit.fail("Unexpected exception: " + e.getMessage());
    	}
    	finally {
    		//Tear down
    		if (payrollRun != null) {
        	Long payrollRunId = payrollRun.getId();
        	cleanUpPayrollRun(payrollRunId);
    		}
    	}
    }

    /**
	 * Creates a Payroll Schedule
	 *
	 * @throws ServiceFailureException and ValidationException
	 */
    private void createPayrollSchedule() throws ServiceFailureException, ValidationException{
    	// For Payroll Schedule to get updated
    	PayrollScheduleTestUtils.createDefaultSchedule();
    	ps = workerPayment.getPayrollSchedule();
    	Date periodEndDate = DateUtil.createDate(2009, Calendar.JANUARY, 25);
    	ps.setPeriodEndDate(periodEndDate);
    	ps.setPayrollScheduleMemo("Changed");
    	ps = workerPayment.updatePayrollSchedule(ps);

	}


	private HashMap<String, Integer> getTaskPriorities(Task[] tasks){

		HashMap<String, Integer> taskPriorityMap = new HashMap<String, Integer>();

		for(int i=0 ; i< tasks.length ; i++){

			if(tasks[i].getName().equals(WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK)){
				taskPriorityMap.put(WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK,tasks[i].getPriority());
			}
			else if(tasks[i].getName().equals(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK)){
				taskPriorityMap.put(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK,tasks[i].getPriority());
			}
			else if(tasks[i].getName().equals(EmployerComponentTasks.COMPANY_SETUP_TASK)){
				taskPriorityMap.put(EmployerComponentTasks.COMPANY_SETUP_TASK,tasks[i].getPriority());
			}
			else if(tasks[i].getName().equals(EmployerComponentTasks.WORKLOCATION_SETUP_TASK)){
				taskPriorityMap.put(EmployerComponentTasks.WORKLOCATION_SETUP_TASK,tasks[i].getPriority());
			}
			else if(tasks[i].getName().equals(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK)){
				taskPriorityMap.put(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK,tasks[i].getPriority());
			}
			else if(tasks[i].getName().equals(WorkerComponentTasks.WORKER_WITHHOLDING_TASK)){
				taskPriorityMap.put(WorkerComponentTasks.WORKER_WITHHOLDING_TASK,tasks[i].getPriority());
			}
			else if(tasks[i].getName().equals(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK)){
				taskPriorityMap.put(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK,tasks[i].getPriority());
			}
		}
		return taskPriorityMap;
	}


	private HashMap<String,Object> getExpectedTaskTypeMap(){

		HashMap<String,Object> typeMap = new HashMap<String,Object>();

		typeMap.put(WorkerComponentTasks.SETUP_FIRST_EMPLOYEE_TASK,TaskType.SETUP);
		typeMap.put(EmployerComponentTasks.PAYROLLSCHEDULE_SETUP_TASK,TaskType.SETUP);
		typeMap.put(EmployerComponentTasks.COMPANY_SETUP_TASK,TaskType.SETUP);
		typeMap.put(EmployerComponentTasks.WORKLOCATION_SETUP_TASK,TaskType.SETUP);
		typeMap.put(WorkerComponentTasks.WORKER_PAYROLL_READY_TASK,TaskType.SETUP);
		typeMap.put(WorkerComponentTasks.WORKER_WITHHOLDING_TASK,TaskType.SETUP);
		typeMap.put(WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK,TaskType.SCHEDULED);
		typeMap.put(EmployerPTSWorkflowTasks.SETUP_ASSISTED_TASK, TaskType.SETUP);
		typeMap.put(WorkerPaymentsComponentTasks.FINISH_PAYCHECK_DISTRIBUTION_TASK, TaskType.NON_SCHEDULED);

		return typeMap;
	}

	private void verifyTaskNameAndType(Task[] tasks , String[] expectedTaskNames){
		List<String> expectedTaskNamesList = Arrays.asList(expectedTaskNames);
		Date today = new Date();

		AssertJUnit.assertEquals("The expected number of tasks not found",expectedTaskNames.length, tasks.length);

		for(int i = 0 ; i < expectedTaskNames.length ; i++){
			// assert task name
			String taskName = tasks[i].getName();
			AssertJUnit.assertTrue("Unexpected task name found: " + taskName, expectedTaskNamesList.contains(taskName));
			// assert task type
			AssertJUnit.assertEquals("The expected task type not found for task name " + taskName, tasks[i].getType(),taskTypeMap.get(taskName));

			if(tasks[i].getName()==WorkerPaymentsComponentTasks.RUN_PAYROLLRUN_TASK)
			overDue = tasks[i].getDueDate().before(today); //this is a temporary fix
		}
	}

	private void createPayrollReadyWorkers() throws Exception {
    	//Save payroll-ready workers
     	String firstName = "Jim";
    	String workLocation = sSanFranciscoLocation;
    	sWorker = WorkerTestUtils.createAPayrollReadyWorker(firstName, workLocation);

    	//Save payroll-ready workers
     	firstName = "Tim";
    	sWorker = WorkerTestUtils.createAPayrollReadyWorker(firstName, workLocation);
    }

}
