package com.intuit.sb.payroll.performance;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import com.intuit.sb.payroll.common.ServiceLocator;
import com.intuit.sb.payroll.services.wages.CompensationItem;
import com.intuit.sb.payroll.services.wages.CompensationType;
import com.intuit.sb.payroll.services.wages.WageService;
import com.intuit.sb.payroll.services.worker.Worker;
import com.intuit.sb.payroll.services.workerpayments.CalcInputs;
import com.intuit.sb.payroll.services.workerpayments.PaycheckDetails;
import com.intuit.sb.payroll.services.workerpayments.PaycheckRequest;
import com.intuit.sb.payroll.services.workerpayments.PayrollRunPeriod;
import com.intuit.sb.payroll.services.workerpayments.PayrollScheduleFrequency;
import com.intuit.sb.payroll.services.workerpayments.WorkerPaymentService;

public class CalculatePayCheckServlet extends PerformanceTestServlet {

	private static final long serialVersionUID = 5468445475569126498L;
	
	private WageService wageService = ServiceLocator.getInstance().getService(WageService.JNDI_NAME);
	private WorkerPaymentService workerPayService = ServiceLocator.getInstance().getService(WorkerPaymentService.JNDI_NAME);
	private String mHourlyRate = "10.00";
	private String mOvertimeRate = "1.5";
	private String mBonusAmount = "1000.00";
	private double mHourlyHours = 80.0;
	private double mOvertimeHours = 8.0;
	
    /**
     * set up Biweekly payroll run period.
     * @return
     */
    private PayrollRunPeriod createBiWeeklyPayrollRunPeriod()
    {
        // Set up paycheck related dates
        PayrollRunPeriod payrollRunPeriod = new PayrollRunPeriod();
        payrollRunPeriod.setFrequency(PayrollScheduleFrequency.Biweekly);
        GregorianCalendar cal = new GregorianCalendar();
        payrollRunPeriod.setPayDate(cal.getTime());
        payrollRunPeriod.setPayrollRunDate(cal.getTime());
        payrollRunPeriod.setPeriodEndDate(cal.getTime());
        cal.add(Calendar.DAY_OF_YEAR, -14);
        payrollRunPeriod.setPeriodStartDate(cal.getTime());
        
        return payrollRunPeriod;
    }
    
    /**
     * helper method to set up wage items for PaycheckRequest
     * @param compensationItems
     * @param hourlyRate
     * @param hourlyHours
     * @param overtimeRate
     * @param overtimeHours
     * @param tipAmount
     * @return
     */    
    public CompensationItem[] setupWageItems(CompensationItem[] compensationItems, String hourlyRate, double hourlyHours,
            String overtimeRate, double overtimeHours, String tipAmount)
	{
		CompensationItem[] setupCompensationItems = new CompensationItem[compensationItems.length];
		CompensationItem hourlyCompensationItem = null;
		for (int i = 0; i < compensationItems.length; i++)
		{
		setupCompensationItems[i] = new CompensationItem();
		CompensationType wageType = compensationItems[i].getCompensationType();
		setupCompensationItems[i].setName(compensationItems[i].getName());
		setupCompensationItems[i].setObjectHandle(compensationItems[i].getObjectHandle());
		setupCompensationItems[i].setCompensationType(wageType);
		setupCompensationItems[i].setFixedRate(compensationItems[i].isFixedRate());
		//set rate and unit to the same as in compensationItems[i]. Rate and unit will be overwritten by parameters passed in
		setupCompensationItems[i].setRate(compensationItems[i].getRate());
		setupCompensationItems[i].setUnits(compensationItems[i].getUnits());
		
		if (CompensationType.Hourly.equals(wageType)) {
		setupCompensationItems[i].setRate(new BigDecimal(hourlyRate));
		setupCompensationItems[i].setUnits(hourlyHours);
		// Remember the hourly item
		hourlyCompensationItem = setupCompensationItems[i];
		}
		else if (CompensationType.Overtime.equals(wageType)) {
		setupCompensationItems[i].setRate(new BigDecimal(overtimeRate));
		setupCompensationItems[i].setUnits(overtimeHours);
		// hourly item must have been set up already before overtime can be set up
		setupCompensationItems[i].setLinkedCompensationItem(hourlyCompensationItem);
		}
		else if (CompensationType.TipsPaycheck.equals(wageType) ||
		CompensationType.TipsCash.equals(wageType) ||
		CompensationType.TipsAllocated.equals(wageType)) {
		setupCompensationItems[i].setRate(new BigDecimal(tipAmount));
		setupCompensationItems[i].setUnits(0.0);
		}
		else if (CompensationType.Bonus.equals(wageType)) {
		setupCompensationItems[i].setRate(new BigDecimal(tipAmount));
		setupCompensationItems[i].setUnits(0.0);
		}
		else if (CompensationType.Commission.equals(wageType)) {
		setupCompensationItems[i].setRate(new BigDecimal(tipAmount));
		setupCompensationItems[i].setUnits(0.0);
		}
		else if (CompensationType.Piecework.equals(wageType)) {
		setupCompensationItems[i].setUnits(1000.0);
		}
		else {
			//do nothing 
		}
		}
		return setupCompensationItems;
	}
	/**
	 * Test API com.intuit.sb.payroll.composite.entityinterface.payemployees.IPayEmployeeService.calcPaycheckDetails()
	 * @param workerUniqueId is the unique ID which identifies a worker
	 */
	public String calculatePaycheckForWorker(long workerUniqueId)
	{		
		StringBuffer sb = new StringBuffer("");
		        
        // Get the wage items for this worker
		try {
	        PaycheckRequest paycheckRequest = new PaycheckRequest();
	        paycheckRequest.setWorkerId(workerUniqueId);
	        
			CompensationItem wageItems[] = wageService.getCompensationItems(workerUniqueId);
			
			CompensationItem[] updatedWageItems = setupWageItems(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
                    mOvertimeHours, mBonusAmount);		

//			CompensationItem[] updatedWageItems = WorkerPaymentServiceTestBase.setupWageItemsForAllWageTypes(wageItems, mHourlyRate, mHourlyHours, mOvertimeRate,
//                    mOvertimeHours, mBonusAmount, 100);	
			CalcInputs[] calcInputs = createCalcInputsFromCompensationItems(updatedWageItems);
	        paycheckRequest.setCalcInputs(calcInputs);
	        
	        PayrollRunPeriod payrollRunPeriod = createBiWeeklyPayrollRunPeriod();
	        paycheckRequest.setPayrollRunPeriod(payrollRunPeriod);
	        
		    PaycheckDetails paycheckDetails = workerPayService.calcPaycheckDetails(paycheckRequest);    
		    sb.append("Gross Total " + paycheckDetails.getGrossTotal() + "<br>");
		    sb.append("Net Total " + paycheckDetails.getNetTotal() + "<br>");      
		}
        catch (Exception ex)
        {
        	throw new RuntimeException("ValidationException in calculating paycheck for worker", ex);
        }

        return sb.toString();
	}
	
	protected static CalcInputs[] createCalcInputsFromCompensationItems(CompensationItem[] workerCompensationItems) {
		List<CalcInputs> inputList = new ArrayList<CalcInputs>();
		for (CompensationItem compensationItem : workerCompensationItems) {
			CalcInputs calcInputs = new CalcInputs();
			calcInputs.setUnits(compensationItem.getUnits());
			calcInputs.setRate(compensationItem.getRate());
			calcInputs.setPayItemObjectHandle(compensationItem.getObjectHandle());
			inputList.add(calcInputs);
		}
		return inputList.toArray(new CalcInputs[inputList.size()]);
	}
	
	/**
	 * test calculate paycheck in ith's Realm
	 * @param realmId
	 */
	public String performTest(String username)
	{
		StringBuffer payResult = new StringBuffer("Employee paycheck details: <p>");
				
		try
		{
			//Get WorkerNames in the realm
			Worker [] workers = PayrollDataContainer.getReadyToPayWorkers(username);
			//the second work is salary worker
			if ( workers != null && workers.length >= 1)
			{
				Worker worker = workers[1];
				long workerUniqueId = worker.getObjectHandle().getId();
				//calculate paycheck for the first employee
				payResult.append(worker.getFirstName() + " " + worker.getLastName() + ":<br>");
				payResult.append(calculatePaycheckForWorker(workerUniqueId));
				payResult.append("<br>");	
			}
		}
		catch (Exception ex)
		{
			throw new RuntimeException(ex);
		}
		
		return payResult.toString();
	}		
}
