package com.intuit.sb.payroll.employer;

import org.testng.Assert;
import org.testng.annotations.Test;

import com.intuit.sb.payroll.services.TestInitializer;
import com.intuit.sb.payroll.services.common.PhysicalAddress;
import com.intuit.sb.payroll.services.common.WorkLocation;
import com.intuit.sb.payroll.ValidationException;

@Test(groups = "serviceittests")
public class EmployerWorkLocationsTest extends TestInitializer {
       
    /**
     * Adds a new WorkLocation and retrieves it back to see it is created properly
     * @throws Exception 
     */
    @Test
    public void addWorkLocationTest() throws Exception
    {

    	String workLocationName = "addWorkLocationTest";
    	String phoneNumber = "650-555-1212";
		WorkLocation newWorkLocation = createWorkLocation(workLocationName, phoneNumber);
    	WorkLocation[] locations = new WorkLocation[1];
    	locations[0] = newWorkLocation;
    	
    	WorkLocation[] locationsBeforeAdd = getEmployerService().getWorkLocations();
    	WorkLocation[] locationsAfterAdd = getEmployerService().addWorkLocations(locations);
    	//Check that we have one more work location.    	
    	Assert.assertTrue(locationsBeforeAdd.length + 1 == locationsAfterAdd.length, "One more work location is added");
    	//Find the new guy
    	boolean newWorkLocationFound = false;
    	for (WorkLocation workLocation : locationsAfterAdd) {
			if (workLocation.getName().equals(workLocationName)) {
				newWorkLocationFound = true;
				newWorkLocation = workLocation;
				break;
			}
		}
    	Assert.assertTrue(newWorkLocationFound, "Did not find the WorkLocation named addWorkLocatonTest");
    	
    	// Clean up
    	WorkLocation[] locationsToDelete = new WorkLocation[1];
    	locationsToDelete[0] = newWorkLocation;
    	getEmployerService().deleteWorkLocations(locationsToDelete);
    	
    	WorkLocation[] locationsAfterDelete = getEmployerService().getWorkLocations();
    	Assert.assertTrue(locationsBeforeAdd.length == locationsAfterDelete.length, "One more work location is added");
    }
    
    
    /**
     * Deletes a WorkLocation and checks it is deleted
     */
    @Test
    public void deleteWorkLocationTest() throws Exception
    {
    	// Same setup as the add, but now we assert the dude is not in the DB
    	String workLocationName = "deleteWorkLocationTest";
		WorkLocation newWL = createWorkLocation(workLocationName, "650-555-6666");
    	WorkLocation[] locations = new WorkLocation[1];
    	locations[0] = newWL;
    	
    	WorkLocation [] locationsAfterAdd = getEmployerService().addWorkLocations(locations);
    	WorkLocation newWorkLocation = null;
    	
    	for (WorkLocation workLocation : locationsAfterAdd) {
			if (workLocation.getName().equals(workLocationName)) {
				newWorkLocation = workLocation;
				break;
			}
		}
    	// Clean up
    	WorkLocation[] locationsToDelete = new WorkLocation[1];
    	locationsToDelete[0] = newWorkLocation;
    	getEmployerService().deleteWorkLocations(locationsToDelete);
    	
    	//Check that we have the same number of worklocations.
    	WorkLocation[] locationsAfterDelete = getEmployerService().getWorkLocations();
    	
    	Assert.assertEquals(locationsAfterAdd.length, locationsAfterDelete.length +1, 
    			"The WorkLocation named deleteWorkLocatonTest was not deleted");
    	
    }
    
    /**
     * Updates a WorkLocation and verifies the update succeeded.  
     */
    @Test
    public void updateWorkLocationTest() throws Exception
    {
    	String workLocationName = "updateWorkLocationTest";
		WorkLocation newWorkLocation = createWorkLocation(workLocationName, "650-555-9999");
    	WorkLocation[] locations = new WorkLocation[1];
    	locations[0] = newWorkLocation;
    	getEmployerService().addWorkLocations(locations);

    	//Update the Phone Number
    	String updatedPhoneNumber = "415-555-0000";
		newWorkLocation.setPhoneNumber(updatedPhoneNumber);

    	WorkLocation[] locationsToUpdate = new WorkLocation[1];
    	locationsToUpdate[0] = newWorkLocation;
    	WorkLocation[] updatedLocations = getEmployerService().updateWorkLocations(locationsToUpdate);
    	
    	//Find the updated guy
    	boolean updated = false;
    	for (WorkLocation workLocation : updatedLocations) {
			if (workLocation.getName().equals(workLocationName) && workLocation.getPhoneNumber().equals(updatedPhoneNumber)) {
				newWorkLocation = workLocation;
				updated = true;
				break;
			}
		}
    	Assert.assertTrue(updated, "The WorkLocation named updateWorkLocatonTest was not updated");
    	
    	// Clean up
    	WorkLocation[] locationsToDelete = new WorkLocation[1];
    	locationsToDelete[0] = newWorkLocation;
    	getEmployerService().deleteWorkLocations(locationsToDelete);
    }
    
    //**********************************************************
    // Validation Tests - these only validate the basic validations at the composite layer.
    // Business logic validations and their tests are at the Capability layer.
    //***********************************************************
    
    
    @Test(expectedExceptions  = ValidationException.class)
    public void nullWorkLocationArrayAddTest() throws Exception {
    	getEmployerService().addWorkLocations(null);
    }

    @Test(expectedExceptions  = ValidationException.class)
    public void nullWorkLocationArrayUpdateTest() throws Exception {
    	getEmployerService().updateWorkLocations(null);
    }

    @Test(expectedExceptions  = ValidationException.class)
    public void nullWorkLocationArrayDeleteTest() throws Exception {
    	getEmployerService().deleteWorkLocations(null);
    }

    @Test(expectedExceptions  = ValidationException.class)
    public void emptyWorkLocationArrayAddTest() throws Exception {
    	getEmployerService().addWorkLocations(new WorkLocation[0]);
    }

    @Test(expectedExceptions  = ValidationException.class)
    public void emptyWorkLocationArrayUpdateTest() throws Exception {
    	getEmployerService().updateWorkLocations(new WorkLocation[0]);
    }

    @Test(expectedExceptions  = ValidationException.class)
    public void emptyWorkLocationArrayDeleteTest() throws Exception {
    	getEmployerService().deleteWorkLocations(new WorkLocation[0]);
    }

	private WorkLocation createWorkLocation(String name, String phoneNumber) {
    	WorkLocation workLocation = new WorkLocation();
    	
    	workLocation.setName(name);
    	workLocation.setPhoneNumber(phoneNumber);
    	workLocation.setExtn("5440");
    	//set the Physical Address
    	workLocation.setAddress(createPhysicalAddress());
    	
    	return workLocation;
    }
	
	/**
	 * Utility method to create a PhysicalAddress
	 */
	private PhysicalAddress createPhysicalAddress() {
		PhysicalAddress physicalAddress = new PhysicalAddress();
		physicalAddress.setAddressLine1("123 Main St.");
		physicalAddress.setCity("Mountain View");
		physicalAddress.setStateOrRegion("CA");
		physicalAddress.setZipOrPostalCode("94043");
		physicalAddress.setCountry("USA");
		
		return physicalAddress;
	}
}
