/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.ASTUtils.java
 * 
 * Created: Jul 12, 2011 1:20:47 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import com.intuit.sb.services.query.parser.Node;
import com.intuit.sb.services.query.parser.SimpleNode;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

/**
 *
 * @author gkesler
 */
public final class ASTUtils {
    private static final boolean trace = false;
    private static final DatatypeFactory dataTypeFactory;
    static {
        try {
            dataTypeFactory = DatatypeFactory.newInstance();
        } catch (DatatypeConfigurationException e) {
            throw new ExceptionInInitializerError(e);
        }
    }
    
    // disable instantiation
    private ASTUtils () {
    }

    /**
     * 
     * @param msg
     * @param node
     * @param prefix 
     */
    public static void dump (String msg, Node node, String prefix) {
        if (trace) {
            System.out.println(msg);
            ((SimpleNode)node).dump(prefix);
        }
    }
    
    public static void trace (String msg) {
        if (trace) {
            System.out.println(msg);
        }
    }
    
    /**
     * Obtains index of s specified node in its parent list of children
     * 
     * @param node  a Node instance to retrieve index
     * @return index of the specified Node
     */
    public static int indexOf (Node node) {
        final Node parent = node.jjtGetParent();
        int index = parent.jjtGetNumChildren();
        while (index-- > 0) {
            if (parent.jjtGetChild(index) == node) {
                return index;
            }
        }

        throw new AssertionError("could not find child " + node + " in its parent collection");
    }

    /**
     * Makes the specified node a child of the new parent
     * 
     * @param node          a Node instance to insert into the newParent's children list
     * @param newParent     a new parent for the specified node
     * @param atIndex       insertion point. If atIndex < newParent.jjtGetNumChildren(), the node
     *                      replaces existing node in the newParent children list. Otherwise, the
     *                      node is added at the end of newParent list of children
     */
    public static void setChildAt (Node node, Node newParent, int atIndex) {
        newParent.jjtAddChild(node, atIndex);
        node.jjtSetParent(newParent);
    }

    public interface ReplaceStrategy<T> {

        /**
         * Finds Node to be replaced
         * 
         * @param node  subtree root to start traversal from
         * @param data  additional arguments to look for replacement node
         * @return node to be replaced
         */
        SimpleNode getReplaceeNode (Node node, T data);

        /**
         * Finds replacement Node
         * 
         * @param node  subtree root to start traversal from
         * @param data  additional arguments to look for replacement node
         * @return  replacement node
         */
        SimpleNode replacementNode (Node node, T data);
    }

    /**
     * Transforms the tree in such a way that it replaces a node with some subtree
     * calculated with the provided ReplaceStrategy starting traversal from the specified node
     * 
     * @param node      subtree root to start traversal from
     * @param data      additional arguments to look for replacement node
     * @param strategy  strategy to find ends for this operation 
     */
    public static <T> void replace (Node node, T data, ReplaceStrategy<T> strategy) {
        final Node replaceeNode = strategy.getReplaceeNode(node, data);
        if (replaceeNode == null) {
            throw new AssertionError("could not perform replace: replacee=null");
        }
        dump("replaceeNode:", replaceeNode, "---");

        final Node replaceeParent = replaceeNode.jjtGetParent();
        if (replaceeParent == null) {
            throw new AssertionError("could not perform replace: replaceeParent=null" + ", replacee=" + replaceeNode);
        }
        final int replaceeIndex = indexOf(replaceeNode);
        dump("replaceeIndex=" + replaceeIndex + ", replaceeParent:", replaceeParent, "---");
        
        final Node replacementNode = strategy.replacementNode(node, data);
        if (replacementNode == null) {
            throw new AssertionError("could not perform replace: replaceeNode=" + replaceeNode + ", replacement=" + replacementNode);
        }
        dump("replacementNode:", replacementNode, "---");

        setChildAt(replacementNode, replaceeParent, replaceeIndex);
    }
    
    /**
     * 
     * @param dateTime
     * @return 
     */
    public static XMLGregorianCalendar newXMLGregorianCalendar (String dateTime) {
        return dataTypeFactory.newXMLGregorianCalendar(dateTime);
    }
}
