/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.SimpleQueryContext.java
 * 
 * Created: Nov 28, 2011 11:59:33 AM
 * Author: gkesler
 */
package com.intuit.sb.services.query;

import com.intuit.sb.services.query.validator.context.AbstractRootContextFactory;
import com.intuit.sb.services.query.validator.context.BeanInfoProvider;
import com.intuit.sb.services.query.validator.context.Context;
import com.intuit.sb.services.query.validator.context.ContextFactory;
import com.intuit.sb.services.query.validator.context.MetadataProvider;
import com.intuit.sb.services.query.validator.context.MetadataProviderFactory;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import org.apache.log4j.Logger;

/**
 *
 * @author gkesler
 */
public class SimpleQueryContext {
    private static final Logger logger = Logger.getLogger(SimpleQueryContext.class);

    public SimpleQueryContext (Map<String, Object> properties, ContextFactory contextFactory) {
        this.properties = properties;
        this.contextFactory = contextFactory;
    }
    
    public SimpleQueryContext (Map<String, Object> properties) {
        this(properties, createContextFactory(properties));
    }
    
    public static interface PropertiesAccess {
        String getProperty (String name);
    }
    
    public SimpleQueryContext (Map<String, Object> properties, PropertiesAccess contextProperties) {
        this(convertProperties(collectProperties(properties, contextProperties)));
    }
    
    public SimpleQueryContext () {
        this(new HashMap<String, Object>(), systemProperties);
    }

    /**
     * Constructs a new SimpleQuery object ready to parse a provided stream with the
     * properties and context factory for the further validation
     * Note! This is an opportunity to specify custom defined MetadataProvider
     * 
     * @see ContextFactory
     * 
     * @param stream            stream containing SimpleQuery statement
     */
    public SimpleQuery newSimpleQuery (Reader reader) {
        // FIXME: if SimpleQuery is pooled, return from the
        // pool instead of creating a new one
        return new SimpleQuery(reader, this);
    }
    
    /**
     * Constructs a new SimpleQuery object ready to parse a provided stream with the
     * properties and context factory for the further validation
     * Note! This is an opportunity to specify custom defined MetadataProvider
     * 
     * @see ContextFactory
     * 
     * @param stream            stream containing SimpleQuery statement
     */
    public SimpleQuery newSimpleQuery (InputStream stream) {
        return newSimpleQuery(new InputStreamReader(stream));
    }
    
    /**
     * Constructs a new SimpleQuery object ready to parse a provided stream with the
     * properties and context factory for the further validation
     * Note! This is an opportunity to specify custom defined MetadataProvider
     * 
     * @see ContextFactory
     * 
     * @param statement            string containing SimpleQuery statement
     */
    public SimpleQuery newSimpleQuery (String statement) {
        return newSimpleQuery(new StringReader(statement));
    }
    
    ContextFactory getContextFactory() {
        return contextFactory;
    }

    Map<String, Object> getProperties () {
        return properties;
    }
    
    private static abstract class ValueConvertor<T> {
        boolean canConvert (Object test) {
            return test instanceof String;
        }
        abstract T fromValue (String value);
    }
    
    private final Map<String, Object> properties;
    private final ContextFactory contextFactory;        
    private static final ConcurrentHashMap<String, ValueConvertor> convertors = 
            new ConcurrentHashMap<String, ValueConvertor>() {
        {
            ValueConvertor stringConvertor = new ValueConvertor<String>() {
                public String fromValue(String value) {
                    return value;
                }
            };
            put(ContextFactory.CONTEXT_APP_ID, stringConvertor);
            put(ContextFactory.CONTEXT_NAME, stringConvertor);
            put(ContextFactory.CONTEXT_PREFIX, stringConvertor);
            put(ContextFactory.CONTEXT_USER_ID, stringConvertor);
            
            ValueConvertor floatConvertor = new ValueConvertor<Float>() {
                public Float fromValue(String value) {
                    return Float.valueOf(value);
                }
            };
            put(ContextFactory.DISTANCE_MAX_LIMIT, floatConvertor);
            put(ContextFactory.DISTANCE_MIN_LIMIT, floatConvertor);
            
            ValueConvertor integerConvertor = new ValueConvertor<Integer>() {
                public Integer fromValue(String value) {
                    return Integer.valueOf(value);
                }
            };
            put(ContextFactory.PAGE_MAX_LIMIT, integerConvertor);
            put(ContextFactory.PAGE_MIN_LIMIT, integerConvertor);
            
            ValueConvertor contextFactoryConvertor = new ValueConvertor<MetadataProviderFactory>() {
                public MetadataProviderFactory fromValue(String value) {
                    try {
                        logger.info("using MetadataProviderFactory=" + value);
                        return (MetadataProviderFactory)Class.forName(value).newInstance();
                    } catch (ClassNotFoundException e) {    
                        throw new IllegalArgumentException("could not load class:" + value, e);
                    } catch (InstantiationException e) {
                        throw new IllegalArgumentException("could not instantiate:" + value, e);
                    } catch (IllegalAccessException e) {
                        throw new IllegalArgumentException("could not instantiate:" + value, e);
                    }
                }
            };
            put(ContextFactory.METADATA_PROVIDER_FACTORY, contextFactoryConvertor);
        }
    };
    
    private static final MetadataProviderFactory defaultMetadataFactory = new MetadataProviderFactory() {
        public MetadataProvider createMetadataProvider(Context context, Class contextType) {
            return new BeanInfoProvider(this, context, contextType);
        }
    };
    
    public static Map<String, Object> convertProperties (Map<String, Object> properties) {
        for (Map.Entry<String, ValueConvertor> entry: convertors.entrySet()) {
            final String propertyName = entry.getKey();
            final ValueConvertor propertyConvertor = entry.getValue();
            
            Object propertyValue = properties.get(propertyName);
            if (propertyConvertor.canConvert(propertyValue)) {
                properties.put(propertyName, propertyConvertor.fromValue((String)propertyValue));
            }
        }
        
        return properties;
    }
    
    public static Map<String, Object> collectProperties (Map<String, Object> properties, PropertiesAccess propertiesAccess) {
        for (Map.Entry<String, ValueConvertor> entry: convertors.entrySet()) {
            String propertyKey = entry.getKey();
            String propertyValue = propertiesAccess.getProperty(propertyKey);
            if (propertyValue != null) {
                properties.put(propertyKey, propertyValue);
            }
        }
        
        return properties;
    }
    
    private static final PropertiesAccess systemProperties = new PropertiesAccess() {
        public String getProperty(String name) {
            return System.getProperty(name);
        }
    };
    
    private static ContextFactory createContextFactory (final Map<String, Object> properties) {   
        return new AbstractRootContextFactory() {
            MetadataProviderFactory metadataFactory;
            {
                metadataFactory = (MetadataProviderFactory)properties.get(ContextFactory.METADATA_PROVIDER_FACTORY);
                if (metadataFactory == null) {
                    metadataFactory = defaultMetadataFactory;
                }
            }
            
            protected MetadataProviderFactory getMetadataProviderFactory() {
                return metadataFactory;
            }
        };
    }

    @Override
    public String toString() {
        return "SimpleQueryContext{" + "properties=" + properties + ", contextFactory=" + contextFactory + '}';
    }
}
