/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.NotOptimizer.java
 * 
 * Created: Jul 12, 2011 1:53:24 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.optimizer;

import com.intuit.sb.services.query.ASTUtils;
import com.intuit.sb.services.query.VisitorStub;
import com.intuit.sb.services.query.parser.*;

/**
 *
 * @author gkesler
 */
public class NotOptimizer implements Optimizer<ASTNot>, SimpleQueryParserTreeConstants {
    private static final NotOptimizer instance = new NotOptimizer();

    private NotOptimizer () {
    }
    
    public static final NotOptimizer getInstance () {
        return instance;
    }
    
    public void optimize(ASTNot node, Object data) {
        node.childrenAccept(optimizer, node);
    }
    
    private final SimpleQueryParserVisitor optimizer = new VisitorStub() {
        @Override
        public Object visit(ASTNot node, Object data) {
            // replace: <grandparent> NOT NOT expression
            // with: <grandparent> expression
            ASTUtils.replace(node, (ASTNot)data, firstChild);            
            return null;
        }

        @Override
        public Object visit(ASTEq node, Object data) {
            // replace: <grandparent> NOT EQ expression
            // with: <grandparent> NEQ expression
            ASTUtils.replace(node, (ASTNot)data, newNEqNode);
            return null;
        }

        @Override
        public Object visit(ASTNEq node, Object data) {
            // replace: <grandparent> NOT NEQ expression
            // with: <grandparent> EQ expression
            ASTUtils.replace(node, (ASTNot)data, newEqNode);
            return null;
        }

        @Override
        public Object visit(ASTLt node, Object data) {
            // replace: <grandparent> NOT LT <expression>
            // with: <grandparent> GTE <expression>
            ASTUtils.replace(node, (ASTNot)data, newGteNode);
            return null;
        }

        @Override
        public Object visit(ASTLte node, Object data) {
            // replace: <grandparent> NOT LTE <expression>
            // with: <grandparent> GT <expression>
            ASTUtils.replace(node, (ASTNot)data, newGtNode);
            return null;
        }

        @Override
        public Object visit(ASTGt node, Object data) {
            // replace: <grandparent> NOT GT <expression>
            // with: <grandparent> LTE <expression>
            ASTUtils.replace(node, (ASTNot)data, newLteNode);
            return super.visit(node, data);
        }

        @Override
        public Object visit(ASTGte node, Object data) {
            // replace: <grandparent> NOT GT <expression>
            // with: <grandparent> LT <expression>
            ASTUtils.replace(node, (ASTNot)data, newLtNode);
            return null;
        }
    };
    
    private static abstract class ReplacementStrategy 
            implements ASTUtils.ReplaceStrategy<ASTNot> {
        public SimpleNode getReplaceeNode(Node node, ASTNot data) {
            return data;
        }
    }
    
    private static abstract class NewNodeReplacementStrategy extends ReplacementStrategy {
        abstract SimpleNode newNode (Node node);

        public SimpleNode replacementNode(Node node, ASTNot data) {
            final SimpleNode subRoot = newNode(node);
            
            subRoot.jjtOpen();

            subRoot.jjtSetFirstToken(data.jjtGetFirstToken());
            subRoot.jjtSetLastToken(((SimpleNode)node).jjtGetLastToken());

            // 2. copy this node's children to the newly created one
            for (int i = node.jjtGetNumChildren() - 1; i >= 0; i--) {
                subRoot.jjtAddChild(node.jjtGetChild(i), i);
            }

            subRoot.jjtClose();
            
            return subRoot;
        }
    }
    
    private static final ASTUtils.ReplaceStrategy<ASTNot> firstChild =
            new ReplacementStrategy() {
        public SimpleNode replacementNode(Node node, ASTNot data) {
            return (SimpleNode)node.jjtGetChild(0);
        }
    };
    private static final ASTUtils.ReplaceStrategy<ASTNot> newNEqNode =
            new NewNodeReplacementStrategy() {
        SimpleNode newNode(Node node) {
            return new ASTNEq(JJTNEQ);
        }
    };
    private static final ASTUtils.ReplaceStrategy<ASTNot> newEqNode =
            new NewNodeReplacementStrategy() {
        SimpleNode newNode(Node node) {
            return new ASTEq(JJTEQ);
        }
    };
    private static final ASTUtils.ReplaceStrategy<ASTNot> newGtNode =
            new NewNodeReplacementStrategy() {
        SimpleNode newNode(Node node) {
            return new ASTGt(JJTGT);
        }
    };
    private static final ASTUtils.ReplaceStrategy<ASTNot> newGteNode =
            new NewNodeReplacementStrategy() {
        SimpleNode newNode(Node node) {
            return new ASTGte(JJTGTE);
        }
    };
    private static final ASTUtils.ReplaceStrategy<ASTNot> newLtNode =
            new NewNodeReplacementStrategy() {
        SimpleNode newNode(Node node) {
            return new ASTLt(JJTLT);
        }
    };
    private static final ASTUtils.ReplaceStrategy<ASTNot> newLteNode =
            new NewNodeReplacementStrategy() {
        SimpleNode newNode(Node node) {
            return new ASTLte(JJTLTE);
        }
    };
}
