/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.validator.ValidatorHelper.java
 * 
 * Created: Aug 12, 2011 12:17:42 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Pattern;

import javax.xml.datatype.XMLGregorianCalendar;

import com.intuit.sb.services.query.ASTUtils;

/**
 * 
 * @author gkesler
 */
public class ValidatorHelper {
	private static final ConcurrentHashMap<Class, ValueValidator> valueValidatorsMap = new ConcurrentHashMap<Class, ValueValidator>() {
		{
			ValueValidator objectValidator = new ValueValidator() {
				public boolean validate(Object value) {
					return true;
				}
			};
			put(Object.class, objectValidator);

			ValueValidator charValidator = new ValueValidator() {
				public boolean validate(Object value) {
					// verify that string contains ONLY 1 character
					return value.toString().length() == 1;
				}
			};
			put(char.class, charValidator);
			put(Character.class, charValidator);

			ValueValidator byteValidator = new NumberValidator<Byte>() {
				protected Byte decode(String value)
						throws NumberFormatException {
					return Byte.decode(value);
				}
			};
			put(byte.class, byteValidator);
			put(Byte.class, byteValidator);

			ValueValidator shortValidator = new NumberValidator<Short>() {
				protected Short decode(String value)
						throws NumberFormatException {
					return Short.decode(value);
				}
			};
			put(short.class, shortValidator);
			put(Short.class, shortValidator);

			ValueValidator intValidator = new NumberValidator<Integer>() {
				protected Integer decode(String value)
						throws NumberFormatException {
					return Integer.decode(value);
				}
			};
			put(int.class, intValidator);
			//put(short.class, shortValidator);
			// This is the fix to ensure that java.lang.Integer type can be supported.
			// This change is suggested by Greg Kesler - BY Chandra - 03/15/2012
			put(Integer.class,intValidator);
			ValueValidator longValidator = new NumberValidator<Long>() {
				protected Long decode(String value)
						throws NumberFormatException {
					return Long.decode(value);
				}
			};
			put(long.class, longValidator);
			put(Long.class, longValidator);

			ValueValidator floatValidator = new NumberValidator<Float>() {
				protected Float decode(String value)
						throws NumberFormatException {
					return Float.valueOf(value);
				}
			};
			put(float.class, floatValidator);
			put(Float.class, floatValidator);

			ValueValidator doubleValidator = new NumberValidator<Double>() {
				protected Double decode(String value)
						throws NumberFormatException {
					return Double.valueOf(value);
				}
			};
			put(double.class, doubleValidator);
			put(Double.class, doubleValidator);

			ValueValidator booleanValidator = new ValueValidator() {
				final Pattern booleanPattern = Pattern.compile("true|false",
						Pattern.CASE_INSENSITIVE);

				public boolean validate(Object value) {
					return booleanPattern.matcher(value.toString()).matches();
				}
			};
			put(boolean.class, booleanValidator);
			put(Boolean.class, booleanValidator);

			ValueValidator stringValidator = new ValueValidator() {
				public boolean validate(Object value) {
					return value.toString().length() >= 0;
				}
			};
			put(String.class, stringValidator);

			ValueValidator dateValidator = new ValueValidator() {
				public boolean validate(Object value) {
					try {
						ASTUtils.newXMLGregorianCalendar(value.toString());
						return true;
					} catch (IllegalArgumentException e) {
						return false;
					}
				}
			};
			put(Date.class, dateValidator);

			ValueValidator timestampValidator = new ValueValidator() {
				public boolean validate(Object value) {
					try {					
						Calendar calendar = Calendar.getInstance();
						calendar.setTime((Date)value);
						if (calendar.get(Calendar.HOUR) > -1
								&& calendar.get(Calendar.MINUTE) > -1
								&& calendar.get(Calendar.SECOND) > -1
								&& calendar.get(Calendar.YEAR) > -1
								&& calendar.get(Calendar.MONTH) > -1
								&& calendar.get(Calendar.DATE) > -1)							
							return true;
						else
							return false;
					} catch (IllegalArgumentException e) {
						return false;
					}
				}
			};
			put(Timestamp.class, timestampValidator);

			ValueValidator bigIntegerValidator = new NumberValidator<BigInteger>() {
				protected BigInteger decode(String value)
						throws NumberFormatException {
					return new BigInteger(value);
				}
			};
			put(BigInteger.class, bigIntegerValidator);

			ValueValidator bigDecimalValidator = new NumberValidator<BigDecimal>() {
				protected BigDecimal decode(String value)
						throws NumberFormatException {
					return new BigDecimal(value);
				}
			};
			put(BigDecimal.class, bigDecimalValidator);

			ValueValidator idValidator = new ValueValidator() {
				final Pattern idPattern = Pattern.compile("\\w+:.+");

				public boolean validate(Object value) {
					return idPattern.matcher(value.toString()).matches();
				}
			};
		}
	};

	private ValidatorHelper() {
	}

	public static boolean isPrimitiveOrWrapper(Class cls) {
		return valueValidatorsMap.containsKey(cls);
	}

	public static boolean isEnum(Class cls) {
		return cls.isEnum();
	}

	public static boolean isArray(Class cls) {
		return cls.isArray();
	}

	public static boolean isCollection(Class cls) {
		return Collection.class.isAssignableFrom(cls);
	}

	public static ValueValidator getValueValidator(Class cls) {
		ValueValidator valueValidator = valueValidatorsMap.get(cls);
		if (valueValidator == null) {
			// cache ValueValidator for Enum class
			if (isEnum(cls)) {
				valueValidatorsMap.putIfAbsent(cls,
						valueValidator = createEnumValidator(cls));
			}
		}

		return valueValidator;
	}

	private static final EnumValidator createEnumValidator(final Class cls) {
		// check if the provided Enum class is a JAXB (CDM) enum
		try {
			final Method fromValue = cls.getMethod("fromValue", String.class);
			// OK, fromValue method is found, cache JAXB validator
			return new EnumValidator() {
				protected Enum valueOf(String name)
						throws IllegalAccessException,
						InvocationTargetException {
					return (Enum) fromValue.invoke(null, name);
				}
			};
		} catch (NoSuchMethodException e) {
			// regular Java enum
			return new EnumValidator() {
				protected Enum valueOf(String name)
						throws IllegalAccessException,
						InvocationTargetException {
					return Enum.valueOf(cls, name);
				}
			};
		}
	}
}
