/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.validator.ValidatorMemento.java
 * 
 * Created: Nov 8, 2011 2:53:38 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import com.intuit.sb.services.query.Holder;
import com.intuit.sb.services.query.parser.ASTPathExpression;
import com.intuit.sb.services.query.parser.Node;
import com.intuit.sb.services.query.parser.SimpleQueryParser;
import com.intuit.sb.services.query.validator.context.Context;
import com.intuit.sb.services.query.validator.context.ContextFactory;
import com.intuit.sb.services.query.validator.context.FirstClassContext;
import com.intuit.sb.services.query.validator.context.MetadataProvider;
import com.intuit.sb.services.query.validator.context.PathBuilder;
import com.intuit.sb.services.query.validator.context.Property;
import com.intuit.sb.services.query.validator.context.RootContext;

/**
 *
 * @author gkesler
 */
final class ValidatorMemento extends Holder<Property> implements ContextDecl, RootContext {
    static final String DEFAULT_CONTEXT_ALIAS = SimpleQueryParser.DEFAULT_CONTEXT_ALIAS;
    
    final ContextFactory contextFactory;
    final Map<String, Object> properties;
    final Map<String, RootContext> contextMap = new HashMap<String, RootContext>();
    final ArrayList<Node> forwardDecls = new ArrayList<Node>();
    
    String contextName;
    String contextAlias;

    ValidatorMemento (ContextFactory contextFactory, Map<String, Object> properties) {
        this.contextFactory = contextFactory;
        this.properties = properties;
        
        String defContextName = (String)properties.get(ContextFactory.CONTEXT_NAME);
        setDefaultContext(defContextName != null ? createRootContext(defContextName) : this);
    }

    RootContext addAliasedContext (String alias, String name) {
        return addRootContext(alias, createRootContext(name));
    }

    private RootContext createRootContext (String contextName) {
        return contextFactory.createContext(contextName, properties);
    }
    
    private RootContext addRootContext (String alias, RootContext context) {
        RootContext oldContext = contextMap.put(alias, context);
        if (oldContext != null && oldContext != this) {
            // duplicate context found
            throw new ValidatorException("duplicate alias name:" + alias);
        }
        
        return context;
    }

    RootContext getAliasedContext (String alias) {
        return contextMap.get(alias);
    }

    private void setDefaultContext (RootContext context) {
        addRootContext(DEFAULT_CONTEXT_ALIAS, context);
    }

    RootContext getDefaultContext () {
    	/**
    	 * jmc 6/5/12 - this code used to assume that the default context was only to be found
    	 * by looking at "$", but if there is only one from element, and it happens to be aliased,
    	 * that is still a default context.
    	 */
    	if (contextAlias != null) {
    		return getAliasedContext(contextAlias);
    	} else {
    		return getAliasedContext(DEFAULT_CONTEXT_ALIAS);
    	}
    }

    ValidatorMemento visitPendedNodes (SimpleQueryValidator outer) {
    	/**
    	 * jmc 6/5/12 - note that the forwarddecl's list is getting accidentally
    	 * added to when processing the forwarddecls during normal visit processing. 
    	 * For now, let's just only agree to process the nodes that are in the list
    	 * when we start and ignore the extra additions.   Otherwise a 
    	 * ConcurrentModificationException results in the iterator.
    	 */
    	int originalListSize = forwardDecls.size();
        // finish processing deferred constructs
        for (int i = 0; i < originalListSize; i++) {
        	Node node = forwardDecls.get(i);
            node.jjtAccept(outer, this);
        }
        return this;
    }
    
    PathBuilder createPathBuilderDecorator () {
        return new PathBuilder() {
            PathBuilder delegate = null;

            public void addPathElement(String name) {
                if (checkPathElement(name)) {
                    // if checkPathElement returns true, we need to treat
                    // the name as property, not alias name
                    delegate.addPathElement(name);
                }
            }

            public void addAnyPathElement() {
                // if not created, create default path builder
                checkPathElement(null);
                delegate.addAnyPathElement();
            }

            public void addIdPathElement(String idValue) {
                // if not created, create default path builder
                checkPathElement(null);
                delegate.addIdPathElement(idValue);
            }

            public void addNamePathElement(String nameValue) {
                // if not created, create default path builder
                checkPathElement(null);
                delegate.addNamePathElement(nameValue);
            }

            public Property evaluate(ASTPathExpression path) {
                // delegate evaluation to the real PathBuilder
                return delegate.evaluate(path);
            }
            
            private boolean checkPathElement (String name) {
                boolean isPathElement = true;
                if (delegate == null) {
                    RootContext context = null;
                    if (name != null) {
                        // lookup context aliased with the name
                        context = getAliasedContext(name);
                    }
                    
                    isPathElement = context == null;
                    if (isPathElement) {
                        // if no aliased context found, use default one
                        context = getDefaultContext();
                    }
                    
                    // create PathBuilder for the selected context
                    delegate = context.createPathBuilder();
                }
                
                return isPathElement;
            }
        };
    }

    // RootContext methods
    public Object getContextProperty(String propName) {
        return properties.get(propName);
    }

    public Class getContextClass(String contextName) {
        return FirstClassContext.getContextClass(contextName, properties);
    }

    public MetadataProvider getMetadataProvider() {
        throw new AssertionError("invalid root context");
    }

    public String getName() {
        return DEFAULT_CONTEXT_ALIAS;
    }

    public Context[] getFullPath() {
        final Context[] path = {this};
        
        return path;
    }

    public RootContext getRootContext() {
        return this;
    }

    public Context getParentContext() {
        return null;
    }
    
    public PathBuilder createPathBuilder() {
        return new PathBuilder() {
            public void addPathElement(String name) {
            }

            public void addAnyPathElement() {
            }

            public void addIdPathElement(String idValue) {
            }

            public void addNamePathElement(String nameValue) {
            }

            public Property evaluate(ASTPathExpression path) {
                forwardDecls.add(path);
                return createProperty(path);
            }
        };
    }

    public Property lookupProperty(String name) {
        throw new AssertionError("invalid root context");
    }

    public Property lookupIdProperty(String idValue) {
        throw new AssertionError("invalid root context");
    }

    public Property lookupNameProperty(String nameValue) {
        throw new AssertionError("invalid root context");
    }

    public Collection<Property> getAllProperties() {
        throw new AssertionError("invalid root context");
    }

    public void setPropertyFilter(PropertyFilter filter) {
        throw new AssertionError("invalid root context");
    }

    public Collection<Property> getProperties() {
        throw new AssertionError("invalid root context");
    }

    private static ValidatorException generateException (String name) {
        return new ValidatorException("no context for the expression:" + name);
    }
    
    private static Property createProperty (final ASTPathExpression path) {
        return (Property)Proxy.newProxyInstance(ValidatorMemento.class.getClassLoader(), proxyInterfaces, new InvocationHandler() {
            public Object invoke(Object o, Method method, Object[] os) throws Throwable {
                throw generateException(path.toString()).setProblemNode(path);
            }
        });
    }

    final static Class[] proxyInterfaces = {Property.class};

    // ContextDecl methods
    public String getContextName() {
        return contextName;
    }

    public void setContextName(String contextName) {
        this.contextName = contextName;
    }

    public String getContextAlias() {
        return contextAlias;
    }

    public void setContextAlias(String contextAlias) {
        this.contextAlias = contextAlias;
    }
}
