/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.services.query.AbstractContext.java
 * 
 * Created: Jul 14, 2011 2:17:13 PM
 * Author: gkesler
 */
package com.intuit.sb.services.query.validator.context;

import com.intuit.sb.services.query.ASTUtils;
import com.intuit.sb.services.query.validator.ValidatorException;
import com.intuit.sb.services.query.validator.ValidatorHelper;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Deque;
import java.util.LinkedList;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 *
 * @author gkesler
 */
public abstract class AbstractContext implements Context {
    protected final Context parent;
    protected final MetadataProvider metadataProvider;
    protected PropertyFilter propertyFilter = allProperties;

    protected AbstractContext (Context parent, Class contextType, MetadataProviderFactory metadataProviderFactory) {
        this.parent = parent;
        this.metadataProvider = metadataProviderFactory.createMetadataProvider(this, contextType);
        
        metadataProvider.init();
    }
    
    public String getName() {
        return metadataProvider.getName();
    }

    public Context[] getFullPath() {
        final Deque<Context> path = new LinkedList<Context>();
        Context context = this;
        while (context != null) {
            path.addFirst(context);
            context = context.getParentContext();
        }
        
        return path.toArray(new Context[path.size()]);
    }

    public RootContext getRootContext() {
        return parent.getRootContext();
    }

    public Context getParentContext() {
        return parent;
    }

    public PathBuilder createPathBuilder() {
        return metadataProvider.createPathBuilder();
    }
    
    public Property lookupProperty(String name) {
        Property property = metadataProvider.lookupProperty(name);
        if (property == null) {
            throw generateException(name);
        }
        return property;
    }

    public Collection<Property> getAllProperties() {
        return metadataProvider.getAllProperties().values();
    }

    public Collection<Property> getProperties() {
        final Collection<Property> properties = getAllProperties();
        ArrayList<Property> filteredProps = new ArrayList<Property>(properties.size());
        for (Property prop: properties) {
            if (propertyFilter.accept(prop)) {
                filteredProps.add(prop);
            }
        }
        
        return filteredProps;
    }

    public void setPropertyFilter(PropertyFilter filter) {
        this.propertyFilter = filter != null ? filter : allProperties;
    }

    protected ValidatorException generateException (String name) {
        return new ValidatorException("property '" + name + "' is not valid for the context '" + getName() + "'");
    }
    
    public static Property createProperty(Context parent, String propertyName, Class propertyType, Method accessor, boolean queryable, boolean sortable) {
        Property property = null;

        if (ValidatorHelper.isEnum(propertyType)) {
            // create single value enum property
            ASTUtils.trace("single value enum property:" + propertyName + "[" + propertyType + "]");
            property = createSingleProperty(parent, propertyName, propertyType, queryable, sortable);
        } else if (ValidatorHelper.isArray(propertyType)) {
            // create collection valued property
            throw new AssertionError("arrays are not supported yet:" + propertyName);
        } else if (ValidatorHelper.isPrimitiveOrWrapper(propertyType)) {
            // create single value property
            ASTUtils.trace("single value property:" + propertyName + "[" + propertyType + "]");
            property = createSingleProperty(parent, propertyName, propertyType, queryable, sortable);
        } else if (ValidatorHelper.isCollection(propertyType)) {
            // create collection valued property
            ASTUtils.trace("collection value property:" + propertyName);
            property = createCollectionProperty(parent, propertyName, (ParameterizedType)accessor.getGenericReturnType(), queryable, sortable);
        } else {
            // create multi-valued property
            ASTUtils.trace("persistent property:" + propertyName + " [" + propertyType + "]");
            property = createPersistentProperty(parent, propertyName, propertyType, queryable, sortable);
        }
        
        return property;
    }
    
    public static Property createPersistentProperty (Context parent, String propertyName, Class propertyType, boolean queryable, boolean sortable) {
        Class contextType = propertyType;
        while (contextType != Object.class) {
            CustomPropertyCtor ctor = customPropertiesMap.get(contextType);
            if (ctor != null) {
                // newProperty composite property
                return ctor.create(parent, propertyName, propertyType, queryable, sortable);
            }

            contextType = contextType.getSuperclass();
        }

        return new PersistentProperty(parent, propertyName, propertyType, queryable, sortable);
    }
    
    public static Property createCompositeProperty (Context parent, String propertyName, 
            Property keyProperty, Property valueProperty, boolean queryable, boolean sortable) {
        return new CompositeProperty(parent, propertyName, keyProperty, valueProperty, queryable,  sortable);
    }
    
    public static Property createCollectionProperty (Context parent, String propertyName, ParameterizedType genericType, boolean queryable, boolean sortable) {
        return new CollectionValueProperty(parent, propertyName, genericType, queryable, sortable);
    }
    
    public static Property createSingleProperty (Context parent, String propertyName, Class propertyType, boolean queryable, boolean sortable) {
        return new SingleValueProperty(parent, propertyName, propertyType, queryable, sortable);
    }
    
    public static Property createWildcardProperty (Context parent, String contextName) {
        return new WildcardProperty(parent, contextName);
    }
    
    public interface CustomPropertyCtor {
        Property create (Context parent, String propertyName, Class propertyType, boolean queryable, boolean sortable); 
    }
    
    private static final Map<Class, CustomPropertyCtor> customPropertiesMap = 
            new ConcurrentHashMap<Class, CustomPropertyCtor>() {
        {
            CustomPropertyCtor customFieldCtor = new CustomPropertyCtor() {
                public Property create(Context parent, String propertyName, Class propertyType, boolean queryable, boolean sortable) {
                    ASTUtils.trace("customField property:" + propertyName + " [" + propertyType + "]");
                    return new CustomFieldProperty(parent, propertyName, propertyType, queryable, sortable);
                }
            };
            
            put(com.intuit.schema.finance.v3.CustomField.class, customFieldCtor);
            
        }
    };
    
    private static final PropertyFilter allProperties = new PropertyFilter() {
        public boolean accept(Property property) {
            return true;
        }
    };

    public MetadataProvider getMetadataProvider() {
        return metadataProvider;
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + "{" + 
                "name=" + getName() + 
                ", parent=" + (parent != null ? parent.getName() : null) + 
               '}';
    }
}
