/*
 * Copyright 2007 Intuit Inc. All rights reserved. Unauthorized reproduction
 * is a violation of applicable law. This material contains certain
 * confidential or proprietary information and trade secrets of Intuit Inc.
 * 
 * com.intuit.sb.datasync.query.rest.Class.java
 * 
 * Created: Nov 28, 2011 6:00:21 PM
 * Author: gkesler
 */
package com.intuit.sb.datasync.query.ws.rs;

import com.intuit.sb.datasync.query.ws.rs.utils.ParseException;
import com.intuit.sb.datasync.query.ws.rs.utils.UriTemplate;
import com.intuit.sb.services.query.SimpleQueryContext;
import com.intuit.sb.services.query.validator.context.FirstClassContext;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.servlet.ServletContext;
import javax.ws.rs.core.UriInfo;
import javax.xml.bind.annotation.XmlType;
import org.apache.log4j.Logger;
import org.jboss.virtual.VFS;
import org.jboss.virtual.VFSUtils;
import org.jboss.virtual.VirtualFile;
import org.scannotation.AnnotationDB;
import org.scannotation.ClasspathUrlFinder;

/**
 *
 * @author gkesler
 */
public class ContextMap implements SimpleQueryConstants {
    private static final Logger logger = Logger.getLogger(ContextMap.class);
    
    private static final String VFS_PROTOCOL_ZIP = "vfszip";
    private static final String VFS_PROTOCOL_FILE = "vfsfile";
    
    private static final Pattern CLASS_BASES_SPLIT_PATTERN = Pattern.compile(
            "([^:]+):([A-Za-z_][A-Za-z0-9_$.]*)");
    private static int CLASS_BASES_VERSION = 1;
    private static int CLASS_BASES_CLASS = 2;
    
    private static final String DEFAULT_CLASS_BASES =
            "v2:" + FirstClassContext.CDM_V2_PACKAGE + ".CdmBase";
    private static final String DEFAULT_URI_TEMPLATE =
            "/{version: v.*}/{realmId}/{entity}|" +
            "/{version: v.*}/{realmId}|" +
            "/{entity}/{version: v.*}/{realmId}";
        
    static class ContextDesc {
        final String prefix;
        final String name;
        
        ContextDesc (String contextClass) {
            int lastDot = contextClass.lastIndexOf('.');
            this.prefix = contextClass.substring(0, lastDot);
            this.name = lastDot < contextClass.length() - 1
                    ? contextClass.substring(lastDot + 1) : null;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final ContextDesc other = (ContextDesc) obj;
            if ((this.prefix == null) ? (other.prefix != null) : !this.prefix.equals(other.prefix)) {
                return false;
            }
            if ((this.name == null) ? (other.name != null) : !this.name.equals(other.name)) {
                return false;
            }
            return true;
        }

        @Override
        public int hashCode() {
            int hash = 7;
            hash = 37 * hash + (this.prefix != null ? this.prefix.hashCode() : 0);
            hash = 37 * hash + (this.name != null ? this.name.hashCode() : 0);
            return hash;
        }

        @Override
        public String toString() {
            return "ContextDesc{" + "prefix=" + prefix + ", name=" + name + '}';
        }
    }
    
    private final Map<String, Object> contextProperties;
    private final Map<ContextMapKey, ContextDesc> contextMap;
    private final UriTemplate uriTemplate;
    
    ContextMap (ServletContext servletContext) {
        this.contextMap = createContextMap(servletContext);
        this.uriTemplate = createUriTemplate(servletContext);
        this.contextProperties = createContextProperties(servletContext);
    }
    
    public ContextDesc getContextDesc (UriInfo uriInfo) {
        return getContextDesc(uriInfo.getPath());
    }
    
    public ContextDesc getContextDesc (final String path) {
        logger.debug("get context class for path:" + path);
        final UriTemplate.Match match = uriTemplate.matches(path);
        if (match == null) {
            throw new AssertionError("Could not parse request path:" + path + " with uri template:" + uriTemplate);
        }
        
        return contextMap.get(new ContextMapKey(match.group(VERSION), match.group(ENTITY)));
    }
    
    public Map<String, Object> getContextProps () {
        return (Map<String, Object>)((HashMap<String, Object>)contextProperties).clone();
    }
    
    static class ContextMapKey {
        final String version;
        final String entity;
        
        ContextMapKey (String version, String entity) {
            this.version = toLowerCase(version);
            this.entity = toLowerCase(entity);
        }

        private static final String toLowerCase (String s) {
            return s != null ? s.toLowerCase() : null;
        }
        
        @Override
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final ContextMapKey other = (ContextMapKey) obj;
            if ((this.version == null) ? (other.version != null) : !this.version.equals(other.version)) {
                return false;
            }
            if ((this.entity == null) ? (other.entity != null) : !this.entity.equals(other.entity)) {
                return false;
            }
            return true;
        }

        @Override
        public int hashCode() {
            int hash = 3;
            hash = 37 * hash + (this.version != null ? this.version.hashCode() : 0);
            hash = 37 * hash + (this.entity != null ? this.entity.hashCode() : 0);
            return hash;
        }

        @Override
        public String toString() {
            return "ContextMapKey{" + "version=" + version + ", entity=" + entity + '}';
        }
    }
    
    private static final Map<ContextMapKey, ContextDesc> createContextMap (ServletContext servletContext) {
        final String classBases = getProperty(servletContext, CLASS_BASES, DEFAULT_CLASS_BASES);
        logger.info("Using classBases:" + classBases);        
        if (classBases == null) {
            throw new AssertionError("classBases is not specified");
        }
        
        try {
            Map<ContextMapKey, ContextDesc> contextMap = new ConcurrentHashMap<ContextMapKey, ContextDesc>();
            
            Matcher matcher = CLASS_BASES_SPLIT_PATTERN.matcher(classBases);
            while (matcher.find()) {
                final String version = matcher.group(CLASS_BASES_VERSION);
                final String anchorClassName = matcher.group(CLASS_BASES_CLASS);
                
                Class anchorClass = Class.forName(anchorClassName);
                URL classPath = ClasspathUrlFinder.findClassBase(anchorClass);   
                
                URL vfsUrl = null;
                if (classPath != null)
                {
                    	//Check if the url is a VFSZIP url. If yes then decode the real url from it
                        String urlProtocol = classPath.getProtocol();
                        if(urlProtocol.equals(VFS_PROTOCOL_ZIP) || urlProtocol.equals(VFS_PROTOCOL_FILE))
                        {
                            //Gets virual file
                            VirtualFile virtualFile = VFS.getRoot(classPath);
                            
                            //Gets real url for Scannotation running
                            try {
								vfsUrl = VFSUtils.getRealURL(virtualFile);
								logger.info("Found vfs url:" + vfsUrl);
							} catch (URISyntaxException e) {
								e.printStackTrace();
								return null;
							}
                        }
						else
						{
							vfsUrl = classPath;
						}
                }

                
                logger.info("Found classPath:" + classPath + " for class:" + anchorClass);
                
                AnnotationDB annotationDb = new AnnotationDB();
                annotationDb.setScanFieldAnnotations(false);
                annotationDb.setScanMethodAnnotations(false);
                annotationDb.setScanParameterAnnotations(false);                
                annotationDb.scanArchives(vfsUrl);
                
                Map<String, Set<String>> classIndex = annotationDb.getClassIndex();
                for (Map.Entry<String, Set<String>> indexEntry: classIndex.entrySet()) {
                    if (indexEntry.getValue().contains(XmlType.class.getName())) {
                        String contextClass = indexEntry.getKey();
                        int lastDot = contextClass.lastIndexOf('.');
                        contextMap.put(new ContextMapKey(version, contextClass.substring(++lastDot)), new ContextDesc(contextClass));
                    }
                }
                
                // put defaults for the case when entity is not specified
                int lastDot = anchorClassName.lastIndexOf('.');
                contextMap.put(new ContextMapKey(version, null), new ContextDesc(anchorClassName.substring(0, ++lastDot)));
            }
            
            return contextMap;
        } catch (ClassNotFoundException e) {
            throw new ExceptionInInitializerError(e);
        } catch (IOException e) {
            throw new ExceptionInInitializerError(e);
        }
    }
    
    private static final UriTemplate createUriTemplate (ServletContext servletContext) {
        final String pattern = getProperty(servletContext, URI_TEMPLATE, DEFAULT_URI_TEMPLATE);
        logger.info("using uri pattern:" + pattern);
        if (pattern == null) {
            throw new AssertionError("invalid uri pattern: pattern");
        }
        try {
            return UriTemplate.compile(pattern);
        } catch (ParseException e) {
            throw new ExceptionInInitializerError(e);
        }
    }
    
    private static final Map<String, Object> createContextProperties (final ServletContext servletContext) {
        Map<String, Object> properties = SimpleQueryContext.collectProperties(new HashMap<String, Object>(), 
                new SimpleQueryContext.PropertiesAccess() {
            public String getProperty(String name) {
                return ContextMap.getProperty(servletContext, name, null);
            }
        });
        
        return SimpleQueryContext.convertProperties(properties);
    }
    
    private final static String getProperty (ServletContext context, String name, String def) {
        String property = context.getInitParameter(name);
        if (property == null) {
            property = System.getProperty(name, def);
        }
        
        return property;
    }
        
    Map<String, Object> getProperties () {
        return contextProperties;
    }
    
    Map<ContextMapKey, ContextDesc> getMap () {
        return contextMap;
    }

    UriTemplate getUriTemplate() {
        return uriTemplate;
    }
}
